// addPages/gateway/index.js
import ability from '../../utils/classDef/ability'
import groups from '../../static/js/jsConfig/groups'
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    wifiList: [],
    currentWifi: {},
    index: 0,
    ssid: '',
    password: '',
    
    wifiAvalible: true, // 默认蓝牙能力ok
    showActionsheet: false, // 是否显示actionsheet
    groups: groups,

    isIos: false, // 是否ios
    allWifis: {}, // 所有WiFi信息

    needRemember: true, // 默认需要记住密码
    needShow: false, // 默认不显示密码
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    this.isIhg = options?.from == 'ihg'; // 是否从ihg进入
    // 打开蓝牙
    let _isIos = ((app.globalData.systemInfo || {}).system || '').indexOf('iOS') >= 0;
    this.setData({
      isIos: _isIos
    })
    console.log("onLoad isIos = ", _isIos);
    // 取得WiFi信息
    this.data.allWifis = app.getStorageByKey("allWifis") || {};
    // 开启wifi
    this.startWifi();
  },
  startWifi (reOpenWifi) {
    ability.startWifi().then((res) => {
      console.log("startWifi res, reOpenWifi = ", res, reOpenWifi);
      // 使能
      this.data.wifiAvalible = true;

      if (reOpenWifi == undefined && !this.data.isIos) {
        // 添加WiFi的监听
        this.addWifiConnected();
      } else {
        this.getWifis();
      }
    }).catch(e => {
      console.log("startWifi catch e = ", e);
      this.data.wifiAvalible = false;
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // check network
    // this.checkNetworkStatue();
  },
  // // 检查网络状态
  // checkNetworkStatue () {
  //   let _networkType = app.globalData.networkType;
  //   if (_networkType != "wifi") {
  //     ability.showToast({
  //       title: "请先连接WiFi"
  //     })
  //   }
  // },
  addWifiConnected () {
    // 这个方法在小米11pro上有个问题：连上WiFi之后也显示不出ssid等信息，但是能拿到事件
    ability.onWifiConnected((res) => {
      console.log("addWifiConnected res = ", res);
      // 判断是否有打开蓝牙
      if (this.data.wifiAvalible) {
        // 开启蓝牙
        this.getWifis();
      } else {
        // 未开WiFi，重新打开时
        this.startWifi();
      }
    })
  },
  getWifis () {
    if (!this.data.wifiAvalible) {
      console.error("getWifis wifiAvalible false");
      return;
    }
    var onGetWifiList = ability.onGetWifiList();
    var getWifiList = ability.getWifiList();
    var getConnectedWifi = ability.getConnectedWifi();

    let _allPromise = [];
    if (this.data.isIos) {
      _allPromise = [getWifiList, getConnectedWifi];
    } else {
      _allPromise = [onGetWifiList, getWifiList, getConnectedWifi];
    }

    console.log("getWifis onGetWifiList getWifiList getConnectedWifi");

    // ios需要特殊处理
    Promise.all(_allPromise).then((response) => {
      console.log("getWifis success response = ", response);
      let _len = response?.length;
      if (!response || _len < 1) {
        return;
      }
      // 如果ios获取到的只有两个数据返回，此时为了不改后面的，需要增加一个空数组
      let res = response;
      if (_len == 2) {
        res.unshift([]);
      }
      // WiFi列表
      let _list = res[0]?.wifiList || [];
      let _array = [];
      for (let item of _list) {
        let {SSID} = item;
        if (_array.indexOf(SSID) >= 0 || !SSID) {
          continue;
        }
        _array.push(SSID)
      }
      // 数组去重
      this.data.wifiList = _array;
      // 当前WiFi
      this.data.currentWifi = res[2]?.wifi || {};
      // ssid
      let _ssid = this.data.currentWifi.SSID;
      this.data.ssid = _ssid;
      // 如果之前有配置过
      this.data.password = this.data.allWifis[_ssid] || '';
      // 设置选中的值
      let _index = this.data.wifiList.indexOf(_ssid);
      this.data.index = _index;
      this.setData({
        index: this.data.index,
        ssid: _ssid,
        // currentWifi: this.data.currentWifi,
        wifiList: this.data.wifiList,
        password: this.data.password
      })
    }).catch(e => {
      console.log("getWifis fail e = ", e);
      // 关闭WiFi
      this.stopWifiInfo();
      // 使能位
      this.data.wifiAvalible = false;
      // 提示
      ability.showToast({
        title: "请确认是否连接WiFi"
      })
    })
    // // 取得WiFi列表回调
    // ability.onGetWifiList((res) => {
    //   console.log("getWifiList success res = ", res);
    //   this.data.wofo
    // });
    // // 取得WiFi列表
    // ability.getWifiList(); // .then((res) => {});
    // // 取得当前连接的WiFi
    // ability.getConnectedWifi().then((res) => {
    //   console.log("getConnectedWifi success res = ", res);
    //   this.data.currentWifi = res.wifi;
    // });
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    this.stopWifiInfo();
  },
  stopWifiInfo () {
    ability.offGetWifiList();
    ability.stopWifi();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  // 下一步
  nextStep () {
    // this.goSetting();
    // 检查ssid和密码是否满足长度
    let _ssidLen = this.data.ssid.length;
    let _passwordLen = this.data.password.length;
    if (_ssidLen == 0 || _ssidLen > 32) {
      ability.showToast({
        title: "请输入1-32长度的WiFi名称"
      })
      return;
    }
    if (_passwordLen > 0 && _passwordLen < 8) {
      ability.showToast({
        title: "WiFi密码为空或者至少8位"
      })
      return;
    } else if (_passwordLen > 64) {
      ability.showToast({
        title: "WiFi密码最长为64"
      })
      return;
    }

    // 如果家庭没有集群信息
    let groupIndex = app.globalData.familyInfo?.groupIndex;
    if (groupIndex == undefined) {
      this.data.showActionsheet = true;
      // 显示actionsheet
      this.setData({
        showActionsheet: this.data.showActionsheet
      })
    } else {
      // 选择的数据
      this.actionNext(groupIndex);
    }
  },

  // goSetting () {
  //   // 初始化ssid
  //   wx.openSetting({
  //     success (res) {
  //       console.log("success = ", res.authSetting)
  //       // res.authSetting = {
  //       //   "scope.userInfo": true,
  //       //   "scope.userLocation": true
  //       // }
  //     }, 
  //     fail (res) {
  //       console.log("fail = ", res)
  //     }
  //   })
  // },
  close: function () {
    this.setData({
      showActionsheet: false
    })
  },
  bindPickerChange (e) {
    let _value = e.detail.value;
    console.log('picker发送选择改变，携带值为', _value);
    this.data.index = _value;
    let _ssid = this.data.wifiList[_value];
    this.data.currentWifi = {
      SSID: _ssid
    };
    this.data.ssid = _ssid;
    // 密码
    this.data.password = this.data.allWifis[_ssid] || '';
    this.setData({
      index: _value,
      ssid: _ssid,
      password: this.data.password
      // currentWifi: this.data.currentWifi
    })
  },
  actionBtnClick (e) {
    console.log("actionBtnClick e = ", e);
    let {value, index} = e.detail || {};
    // 设置选择的集群
    this.actionNext(index, value);
  },
  actionNext (index, value) {
    // 判断是否一致，如果一致的话，则不用保存
    if (index != app.globalData.familyInfo?.groupIndex) {
      app.globalData.familyInfo.groupIndex = index;
    }
    // 更新集群信息
    let _v = value || groups[index]?.value || groups[0].value;
    app.globalData.blsdk.updateCtrlHost(_v);
    // 选择的数据详细
    let _item = {
      password: this.data.password,
      ssid: this.data.ssid,
      host: _v,
      // hostType: '06' / '0b'
    };
    console.log("actionBtnClick _item = ", _item, index);
    // 保存wifi信息
    app.globalData.wifiInfo = _item;

    // 是否需要保存WiFi信息
    let _needRemember = this.data.needRemember;
    if (_needRemember) {
      // ssid和密码
      this.data.allWifis[_item.ssid] = _item.password;
      app.setBizStorage("allWifis", this.data.allWifis);
    }

    // 关闭actionsheet
    this.close();

    if (this.isIhg) {
      return ability.redirectTo("../ihg/config/index")
    }
    // 跳转到下一个配网页面
    ability.navigateTo("../bleIndex/index?gateway=1")
  },
  ssidChange (e) {
    this.data.ssid = e.detail.value
    this.setData({
      ssid: this.data.ssid
    })
  },
  blurName () {
    // 密码
    let _ssid = this.data.ssid;
    this.data.password = this.data.allWifis[_ssid] || '';
    this.setData({
      password: this.data.password
    })
  },
  passwordChange (e) {
    this.data.password = e.detail.value
    this.setData({
      password: this.data.password
    })
  },
  // 是否显示可见的密码
  setNeedShow () {
    this.data.needShow = !this.data.needShow;
    this.setData({
      needShow: this.data.needShow
    })
  },
  // 是否需要记住密码
  setNeedRemember () {
    this.data.needRemember = !this.data.needRemember;
    this.setData({
      needRemember: this.data.needRemember
    })
  }
})