// addPages/ihg/index.js
import bizUtil from "../../../utils/bizUtil"
import ability from "../../../utils/classDef/ability"
import GROUPS from '../../../static/js/jsConfig/groups';
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    list: [], // 显示的状态
    ssid: "", // ssid名称

    addrs: [],
    findDs: [], // 设备信息

    status: 1, // 当前配网状态 默认第一步

    bleUnavailable: false, // 默认蓝牙可用
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    // 保存页面来源，有可能从ihg属性进入
    this.fromStr = options?.from;
    // 个数
    this.count = options?.count || 0;
    console.log("onLoad this.count = ", this.count);
    // 设备信息
    let _devInfo = app.globalData.devInfo || {};
    console.log("onLoad _devInfo = ", _devInfo);
    // ihg类型
    let _type = this.getIhgTypeByDevInfo(_devInfo);
    console.log("onLoad _type = ", _type);
    this.type = _type;
    // ssid信息
    this.data.ssid = _type == 1 ? '' : ("“" + (app.globalData.wifiInfo?.ssid || "") + "”");
    // 取得列表
    let _list = this.getListByStatus(this.data.status, _type);
    this.setData({
      list: _list,
      ssid: this.data.ssid
    });
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // // 初始化定时器 90s 暂时不需要定时
    // this.initIhgTimer();
    // 开始发送IHG的配网数据
    this.startIhgConfig();
  },

  initIhgTimer () {
    // 初始化之前先销毁
    this.destoryTimer(this.setTimeout);
    // 初始化定时器 90s
    this.setTimeout = setTimeout(this.checkFinish, 90000);
  },
  destoryTimer (timer) {
    let _timer = timer || this.setTimeout;
    if (!_timer) {
      return;
    }
    clearTimeout(_timer);
    _timer = null;
  },
  checkFinish (msg) {
    // 是否有设备配网成功
    let _status = this.status;
    if (_status >= 3) {
      // 有配置成功，不弹窗
      return;
    }
    // 停止配网
    this.destorySDKConfig();

    // 如果没有配置到网关
    let _content = msg || "设备配网超时，请重新配置";
    let _title = "";

    // 提示
    wx.showModal({
      title: _title,
      content: _content,
      showCancel: true,
      confirmText: "去配置",
      confirmColor: "#F78D3F",
      cancelText: "取消",
      cancelColor: '#000000'
    }).then(result => {
      let _confirm = result?.confirm;
      if (_confirm) {
        // 清空设备信息
        app.globalData.devInfo = undefined;
        // 去配置，进入输入WiFi密码页面
        this.goBack(1)
      } else {
        // 进入首页
        this.goBack(3)
      }
    })
  },
  goBack (num) {
    // 返回
    ability.navigateBack(num || 1);
  },
  startIhgConfig () {
    let _extension = Object.assign({}, {
      isIHG: this.type,
      devInfo: app.globalData.devInfo,
      configType: "ihgconfig",
      nocallback: true
      // allowDuplicatesKey: false // 是否允许重复上报同一设备
    }, app.globalData.wifiInfo);

    // 这里需要根据集群配网
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    _extension.host = _groupStr;

    // 先停止之前的
    // app.globalData.blsdk.stopBleReceiveService();
    app.globalData.blsdk.startConfigByFilter([10058], (res) => {
      console.error("startIhgConfig res = ", res);
      if (!res) {
        // 销毁定时器
        this.destoryTimer();
        // 主动调用配网失败
        this.checkFinish();
        return;
      }
      // 返回的数据
      let {addrs} = res;
      if (!addrs) {
        return;
      }
      // 是否配网成功
      let _isSuccess = false;
      let _devInfo = app.globalData.devInfo || {};
      let {a, d} = _devInfo;
      // 是否需要更新
      for (let addr in addrs) {
        let _item = addrs[addr] || {};
        // 判断短地址是否相同
        if (a != addr) {
          continue;
        }
        // let _devInfo = app.globalData.devInfo; // bizUtil.getDevInfoByDInJson(app.globalData.devs, d);
        if (Object.keys(_devInfo).length == 0) {
          continue;
        }
        _devInfo.wd = _item.wd;
        _devInfo.id = _item.id || 0;
        _devInfo.k = _item.k || "";
        _devInfo.online = true; // 多网关时不在线的网关要放后面，防止第一次远程不了
        // // 保存设备信息
        // app.globalData.devInfo = _devInfo;
        _isSuccess = true; // 配网成功
      }
      // 更新数据
      if (!_isSuccess) {
        return;
      }
      // 停止配网
      this.destoryAll();
      let _status = 4; // 完成
      // 加入显示
      this.setData({
        status: _status,
        list: this.getListByStatus(_status, this.type)
      });
      // 加入数据
      let _roomId = _devInfo.rid || _devInfo.id || 0;

      // 加入之前先判断是否有重复的设备在
      // 取得所有设备的map
      let {devJson} = bizUtil.getDidMapByConfig(app.globalData.devs, ['devs']);
      if (!devJson || !devJson[d]) {
        (app.globalData.devs[_roomId]?.devs) && (app.globalData.devs[_roomId].devs.push(app.globalData.devInfo));

        // 清除udp数据
        this.clearGatewayUdpData(_devInfo);
      }
      // 保存设备信息
      app.saveAllDevsMap();
    }, _extension);
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    this.destoryAll();
  },
  destoryAll () {
    // 销毁定时器
    this.destoryTimer();
    // 停止配网
    this.destorySDKConfig();
  },
  destorySDKConfig () {
    // 停止配网
    !!app.globalData.blsdk && (app.globalData.blsdk.destory());
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  // 判断是有线ihg还是无线ihg
  getIhgTypeByDevInfo (devInfo) {
    let {v, t} = devInfo || {};
    let _devTypeStr = bizUtil.getDevTypeStr(t);
    let _isIhg = bizUtil.isIhg(t);
    let {byte0} = bizUtil.getJsonByAblitys(v, _isIhg ? 'ihg' : _devTypeStr, {t});
    console.log("getIhgTypeByDevInfo byte0 = ", byte0);
    if (byte0 == 0x51) {
      // 有线
      return 1;
    } else if (byte0 == 0x52) {
      // 无线
      return 0;
    } else if (byte0 == 0x53) {
      // 有线+无线 认为是无线的
      return 0;
    }
    return -1;
  },

  // 根据ihg状态取得步骤
  getListByStatus (status, type) {
    // 所有显示数据
    let _allList = this.getAllList();
    // // 如果是有线的，则不需要向设备发送WiFi以及密码
    // if (type == 1) {
    //   _allList = _allList.slice(1);
    // }
    // 当前状态
    _allList.map(item => {
      if (status >= item.id) {
        item.isLoading = false;
      }
      return item;
    });
    console.log("getListByStatus _allList = ", _allList)
    return _allList;
  },
  getAllList () {
    // let {ssid} = app.globalData.wifiInfo || {};
    return [{
      id: 1,
      text: "正在向设备发送配对信息", // Wi-Fi名称和密码
      isLoading: true // 默认都是loading
    }, {
      id: 2,
      text: "设备正在连接" + this.data.ssid,
      isLoading: true // 默认都是loading
    }, {
      id: 3,
      text: "设备已成功配对",
      isLoading: true // 默认都是loading
    }]
  },
  jumpAfterComplete () {
    ability.navigateBack();
    // // 返回上一页
    // if (this.fromStr == "property") {
    //   return ability.navigateBack();
    // }

    // // 跳转到第三方语音页面
    // let _devInfo = app.globalData.devInfo;
    // console.log("jumpAfterComplete _devInfo, devs = ", _devInfo, app.globalData.devs);
    // // 跳转到第三方语音页面
    // return ability.redirectTo("/device/gateway/thirdparty/index?from=ihgconfig&title=" + encodeURIComponent(_devInfo?.n || ""))
  },
  // 清除网关udp私有数据
  clearGatewayUdpData (devInfo) {
    if (!devInfo) {
      return;
    }
    let _devInfo = devInfo;
    // 上报的集群域名
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    // 清空数据
    app.globalData.blsdk.manageUdpPrivateData({
      type: "add",
      d: _devInfo?.wd || _devInfo?.d,
      t: _devInfo?.t,
      k: _devInfo?.k,
      data: ""
    }, {
      host: _groupStr
    });
  }
})