// app.js
import GROUPS from './static/js/jsConfig/groups';
import { Utils } from './utils/util';
import ability from './utils/classDef/ability'
import bizUtil from "./utils/bizUtil"
import blsdk from './utils/classDef/blsdk'
// 无网关时上报私有数据
import noGatewayMethod from './static/js/behavior/noGatewayFamilyId'

const COLORS = require('./static/js/jsConfig/colors.js');
App({
  onLaunch(options) {
    // 取得配置信息
    this.addListener();
    var _systemInfo = this.getSystemInfo();
    this.globalData.systemInfo = _systemInfo;
    (!!_systemInfo.brand) && (this.globalData.userInfo.brandname = _systemInfo.brand);
    // 蓝牙服务
    this.initBleServer(_systemInfo);
    // 页面尺寸
    this.initPageSize(_systemInfo);
    // 默认样式
    this.initColors();
    // 家庭列表
    this.initFamilys();
    // 用户信息
    this.initUserInfo();

    // 扫码处理
    var _shareStr = this.initShareData(options);
    this.globalData.shareStr = _shareStr;
    console.log("onLaunch _shareStr, options = ", _shareStr, options);
    // 初始化数据
    this.initDataByStorage();
  },
  globalData: {
    systemInfo: {},
    userInfo: {
      brandname: "", // 手机名称
      nickname: "游客",
      avatarUrl: ""
    },
    roleNum: 0,
    devs: {}, // 所有设备信息
    devInfo: {},
    devStatus: {},
    companyInfo: bizUtil.getNameSpace('bl'), // 'bl' 'bh' // 'qm' // 'uart' // 'aux'
    shareStr: '',
    roomInfo: {}, // 房间信息
    groupIds: [], // 批量控制用
    customPageSize: {
        width: 320, // 单位是px
        statusHeight: 0,
        navHeight: 88,
        tabHeight: 98,
        contentHeight: 0, // 
        multip: 2
    },
    colors: COLORS, // 常用颜色
    bleModes: {}, // 
    allStatus: {}, // 所有设备上次状态
    sceneInfo: {}, // 单个场景编辑
    panelInfo: {}, // 单个面板选择
    timerInfo: {}, // 定时选择
    timerList: [], // 定时列表
    wifiInfo: {}, // 网关配置的WiFi信息
    wifiInfos: {}, // 所有WiFi信息
    familyInfo: {}, // 家庭信息
    familys: [], // 家庭列表
    tmpData: {
      qrTime: 2591999, // 二维码分享最大30天 单位s
      avalible: 900, // 15分钟 单位s
      maxQrTime: 2592000, // 单位s
      devInfo: {}, // 临时保存设备替换用
    }, // 临时数据
    allFamilysMap: {}, // 所有家庭数据 之所以与家庭信息区分是为了防止后续变动，不要耦合性太强 // 12-13 算了 变得太快 耦合不动了，就这样吧
    tgroupId: undefined, // 临时group
    groupInfo: {}, // 固定群组
    networkType: "none", // 默认没有网络

    websocket: null, // 远程调试的websocket
    // websocketInterval: null, // 远程调试的websocket定时器发送心跳
  },
  onShow () {
    console.log("onShow _userInfo = ", this.globalData.userInfo);
    // 获取网络状态
    this.getNetworkType();
    // 如果是安卓的话，需要获取地理位置
    var isAndroid = bizUtil.isAndroid(this.globalData.systemInfo?.system);
    console.log("isAndroid = ", isAndroid);
    if (!isAndroid) {
        return;
    }
    // 取得定位
    this.getLocation();
  },
  onHide () {
    console.log("app.js onHide");
    // 销毁 关闭蓝牙监听器
    !!this.globalData.blsdk && (this.globalData.blsdk.destory());
    // 保存数据
    this.saveWhenHide();
  },
  saveWhenHide () {
    // 只保存数据 这个接口很多地方用
    this.saveAllDevsMap();
    // 保存所有家庭信息
    this.setBizStorage("familys", this.globalData.familys);
    // 保存当前选择的家庭信息 这里应该保存家庭index的，但是刚开始没有多家庭，为了兼容，没有动，可能导致数据不同步的问题
    this.setBizStorage("family", this.globalData.familyInfo);
  },
  // 只保存设备相关信息
  saveAllDevsMap () {
    // 保存家庭与设备映射关系
    this.setBizStorage('allFamilysMap', this.globalData.allFamilysMap)
  },
  getNetworkType () {
    // ability.getNetworkType({
    //     success: (res) => {
    //         let {networkType} = res || {};
    //         console.log("getNetworkType networkType = ", networkType);
    //         this.globalData.networkType = networkType || "none";
    //         // wifi 2-5g none unknown(Android 下不常见的网络类型)
    //     },
    //     fail: () => {
    //         this.globalData.networkType = "none";
    //     }
    // });
  },
  getLocation () {
    ability.getLocation({
      success: (res) => {
        console.log("getLocation success res = ", res)
      },
      fail: (e) => {
        console.error("getLocation e = ", e);
        ability.showToast({
          title: "为了更好使用，请开启定位服务"
        })
      }
    })
  },
  // 初始化所有家庭的数据 这里不是当初的设计 还要兼容之前的数据
  initFamilys () {
    let _familys = this.getStorageByKey("familys") || [];
    let _family = this.getStorageByKey("family");
    if (_familys.length == 0) {
      if (_family) {
        _familys = [_family];
      } else {
        let _sdk = this.globalData.blsdk || {};
        let _bleCode = !!_sdk.getBleCode && (_sdk.getBleCode());
        let _seq = !!_sdk.getNextSeq && (_sdk.getNextSeq());
        let _familyInfo = bizUtil.getOriginFamilyInfo(_bleCode || [], _seq);
        _familys = [_familyInfo];
      }
    }
    console.log("initFamilys _familys, _family = ", _familys, _family);
    this.globalData.familys = _familys;
    this.globalData.familyInfo = _family || _familys[0];
    
    // // for test 测试集群
    // this.globalData.familyInfo.groupIndex = 3;
  },
  // 初始化用户信息
  initUserInfo () {
    let _userInfo = this.getStorageByKey("userInfo");
    // console.log("initUserInfo _userInfo = ", _userInfo);
    if (!_userInfo || Object.keys(_userInfo).length == 0) {
      return;
    }
    console.log("initUserInfo _userInfo = ", _userInfo);
    this.globalData.userInfo = _userInfo;
  },
  initDataByStorage () {
      // 取得所有家庭信息
      let _allFamilysMap = this.getStorageByKey("allFamilysMap") || {};
      console.log("initDataByStorage _allFamilysMap = ", _allFamilysMap, this.globalData.familyInfo);
      // 家庭信息
      let {id, floor} = this.globalData.familyInfo;
      console.log("initDataByStorage id = ", id);
      // 取得网关数据
      let _gatewayInfo = undefined;
      // 所有设备列表
      var _dev = _allFamilysMap[id];
      if (!_dev || Object.keys(_dev).length == 0) {
        _dev = {};
      }
      let _originRoom = bizUtil.getOriginRooms();
      if (!_dev || Object.keys(_dev).length == 0) {
        _dev = bizUtil.initDevByRooms({}, _originRoom, true);
      } else {
        console.log("initDataByStorage _dev = ", _dev);
        // 是否存在多楼层
        let _isFloor = floor;
        // 遍历数据
        for (var index in _dev) {
            let item = _dev[index] || {};
            let _id = item.id || 0;
            let _scenes = item.scenes || [];
            item.devs = item.devs || []; // 设备信息
            // 兼容以前的数据 如果没有场景，增加默认场景
            (_scenes.length == 0) && (item.scenes = bizUtil.getDefaultScenes(_id));
            // item.scenes = item.scenes || ; // 情景信息
            !item.n && (item.n = _originRoom[_id]?.n || '');

            // 存在多楼层
            if (_id > 0 && (_id % 32 == 0) && _isFloor < 0) {
                _isFloor = bizUtil.getFloorByRoomId(_id);
            }

            // bizUtil.setArrayItem(item.devs, ['id'], [_id]);
            // 遍历设备数组
            for (let devInfo of item.devs) {
                // 如果没有房间，则设置房间id
                if (devInfo.id == undefined) {
                    devInfo.id = _id;
                }
                // 取第一个网关设备
                if (!!_gatewayInfo) {
                    continue;
                }
                // 是否网关设备
                let _isGatewayDev = bizUtil.isGatewayDev(devInfo.t);
                // 不是网关设备则继续
                if (!_isGatewayDev) {
                    continue;
                }
                _gatewayInfo = devInfo;
            }
        }
        // 多楼层数据
        if (_isFloor == undefined || _isFloor < 0) {
            this.globalData.familyInfo.floor = _isFloor;
        }
        console.log("updateGatewayInfo _gatewayInfo = ", _gatewayInfo);
      }
      
      // 保存数据
      !_allFamilysMap[id] && (_allFamilysMap[id] = _dev);
      this.globalData.allFamilysMap = _allFamilysMap;
      console.log("initDataByStorage _dev = ", _dev);

      // 增加默认数据
      bizUtil.addDefaultGroupByDevs(_dev);
      // 设备信息
      this.globalData.devs = _dev;

      // 更新设备状态
      this.globalData.allStatus = this.getStorageByKey("devStatus") || {};
      // 如果有网关设备，则设置
      !!(_gatewayInfo) && (this.globalData.blsdk.updateGatewayInfo(_gatewayInfo));
      // 更新信息
      this.updateSDKByFamilyConfig({
        familyInfo: this.globalData.familyInfo
      });
  },
  updateSDKByFamilyConfig (config) {
    if (!config) {
      return;
    }
    // , devStatus , gatewayInfo
    let {familyInfo} = config;
    if (!familyInfo) {
      return;
    }

    let {addr1, addr2, bleCode, groupIndex, roleNum, devTimestamp} = familyInfo;
    this.globalData.familyInfo = familyInfo;
    // 默认关闭远程调试
    familyInfo.isRemote = false;
    // 保存uuid
    familyInfo.uuid = familyInfo.uuid || noGatewayMethod.createWxUuid();
    // 更新角色信息
    this.globalData.roleNum = roleNum || 0;
    // 更新当前集群信息
    console.log("updateSDKByFamilyConfig familyInfo = ", familyInfo);
    this.globalData.blsdk.updateCtrlHost(GROUPS[groupIndex || 0]?.value || GROUPS[0].value);
    // 如果有控制密钥，则更新
    !!bleCode && (this.globalData.blsdk.updateBleCode(bleCode));
    // 如果有短地址，则更新
    this.globalData.blsdk.updateAddr(addr1 || 1);
    this.globalData.blsdk.updateAddr2(addr2 || 257);
    // 如果是分享权限，则取得设备有效时长
    if (this.globalData.roleNum == 2 && devTimestamp > 0) {
      // 如果是分享者权限，且有有效时长，则没有
      this.globalData.blsdk.updateCtrlTimestamp(devTimestamp);
    } else {
      this.globalData.blsdk.updateCtrlTimestamp(0);
    }
  },
  initPageSize (systemInfo) {
      if (!systemInfo) {
          return;
      }
      console.log("initPageSize systemInfo = ", systemInfo);
      var safeArea = systemInfo.safeArea, windowHeight = systemInfo.windowHeight, statusBarHeight = systemInfo.statusBarHeight, width = systemInfo.windowWidth;
      var _a = safeArea || {}, height = _a.height, bottom = _a.bottom;
      var _multip = bizUtil.initMultip(systemInfo);
      var _navHeight = 88;
      var _height = height;
      var _contentHeight = 0;
      var _tabHeight = 0; // 98;
      if (windowHeight > 0 && windowHeight < _height) {
          _contentHeight = windowHeight;
      }
      else if (windowHeight == _height && bottom == _height && windowHeight > 0) {
        _contentHeight = windowHeight - statusBarHeight - (_navHeight / _multip);
      }
      else {
          _contentHeight = _height - (_navHeight / _multip);
      }
      this.globalData.customPageSize = {
          width: width, // 长度的单位是px
          statusHeight: statusBarHeight,
          navHeight: _navHeight,
          tabHeight: _tabHeight,
          contentHeight: _contentHeight,
          multip: _multip,
          devicePixelRatio: systemInfo?.devicePixelRatio || 1
      };
  },
  initShareData (options) {
      var query = options.query || {};
      return this.initShareStr(decodeURIComponent(query.q || ""));
  },
  // 初始化在线二维码
  initOnlineQrcode (jsonParams) {
    let {cloud, did, token} = jsonParams || {};
    // 判断是否为空
    if (!cloud || !did || !token) {
        console.error("initOnlineQrcode cloud, did, token = ", cloud, did, token);
        setTimeout(() => {
            ability.showToast({
                title: '无效二维码',
                icon: "none"
            });
        }, 0);
        return true;
    }
    // 为了兼容 去除app-service-
    jsonParams.cloud = jsonParams.cloud.replace("app-service-", "");
    return this.globalData.blsdk.getInfoByData(jsonParams);
  },
  initShareStr (str) {
    console.log("initShareStr str = ", str);
      if (!str || (typeof str != "string")) {
          return '';
      }
      let _data = Utils.urlStrToJson(str);
      console.log("initShareStr data = ", _data);
      if (typeof _data == 'string') {
          // 清除缓存的二维码数据
          this.setBizStorage("shareData", '');
          return _data;
      } else {
          let {data, total, index, random, cloud, did, token} = _data;

          // 判断是否云端二维码
          if (!!cloud && !!did && !!token) {
            return this.initOnlineQrcode(_data);
          }

          // 本地二维码
          if (total > 1 && index >= 0) {
            // 有分页的二维码
            let _shareData = this.getStorageByKey("shareData") || {};
            console.log("_shareData.data = ", _shareData.data, index);
            if (_shareData.random != random) {
                // 清空之前的数据
                let _data = {total, random};
                _data.data = {[index]: data};
                console.log("_data = ", JSON.stringify(_data));
                this.setBizStorage("shareData", _data);
                console.error("showToast new = ", parseInt(index) + 1);
                
                setTimeout(() => {
                    ability.showToast({
                        title: `第${parseInt(index) + 1}/${total}页`,
                        icon: "none"
                    });
                }, 0);

                return true;
            }
            // 如果是当前的
            _shareData.data = _shareData.data || {};

            // 该二维码是否扫描过
            let _isExist = !!_shareData.data[index];

            // 保存扫码数据的下标
            _shareData.data[index] = data;
            // 判断是否完整的
            let _count = Object.keys(_shareData.data);
            console.log("_count, index = ", _count, index);
            if (_count.length == total) {
                // 清空数据
                this.setBizStorage("shareData", '');
                // 扫码完毕
                return Object.values(_shareData.data).join("");
            } else {
                
                // 判断是否扫码过
                if (_isExist) {
                    setTimeout(() => {
                        ability.showToast({
                            title: '该二维码已经被扫描过，请继续滑动',
                            icon: "none"
                        });
                    }, 0);
                    return true;
                }

                setTimeout(() => {
                    ability.showToast({
                        title: `第${parseInt(index) + 1}/${total}页`,
                        icon: "none"
                    });
                }, 0);
                
                this.setBizStorage("shareData", _shareData);
                return true;
            }
          } else {
            // 清除缓存的二维码数据
            ability.removeStorage({
                key: 'shareData'
            });
          }
          // 清除
          return data;
      }
  },
  getStorageByKey (key) {
      let _nameSpace = this.globalData.companyInfo?.namespace;
      return ability.getStorage({
          key: _nameSpace + key
      }, true);
  },
  addListener () {
    ability.globalOnError((e) => {
      console.error("catch wx error e = ", e);
    });
  },
  getSystemInfo () {
    return ability.getSystemInfo() || {};
  },
  initBleServer (systemInfo) {
    var isIos = ((systemInfo || {}).system || '').indexOf('iOS') >= 0;
    console.log("initBleServer isIos = ", isIos);
    this.globalData.blsdk = new blsdk({
      nameSpace: this.globalData.companyInfo.namespace,
      isIos: isIos,
      // bleCode: xxx,
      bleStatusChange (status) {
        console.log("bleStatusChange status = ", status);
        // 蓝牙状态改变事件
        var _status = status || {};
        if (_status.available == false) {
          ability.showBleFail();
        } else {
          ability.hideBleFail();
        }
      }
    });
    // console.log("getSDKVersion, isIos = ", this.globalData.blsdk.getSDKVersion(), isIos);
    // this.globalData.blsdk.startDebug(false);
  },
  initColors () {
      let _nameSpace = this.globalData.companyInfo.namespace;
      let _this = this;
      console.log("this.globalData.colors = ", this.globalData.colors);
      ability.getStorage({
          key: _nameSpace + 'colors',
          succCall: (res) => {
            console.log("initColors res = ", res);
            var _array = res || [];
            if (_array.length == 0) {
                _array = COLORS;
            }
            _this.globalData.colors = _array || [];
          }
      });
  },
  getAvalibleTime () {
      var _a = this.globalData;
      var devTimestamp = _a.familyInfo?.devTimestamp || 0;
      var roleNum = _a.familyInfo?.roleNum;
      
      if (roleNum != 2) {
          // 管理权限
          return "admin";
      } else if (devTimestamp == 0) {
          // 永久权限
          return "always"
      }
      console.log("getLastContent devTimestamp = ", devTimestamp);
      return (devTimestamp || 0) - (Date.now() / 1000);
  },
  setBizStorage (key, data) {
    if (!key) {
        return;
    }
    // 保存设备信息
    let _nameSpace = this.globalData.companyInfo.namespace;
    ability.setStorage({
      key: _nameSpace + key,
      data: data // this.globalData.devs
    });
  },
  uploadPrivateData (callback, allDevs, familyInfo, dataJson, familyProtoStr) {
    // 家庭信息
    let _familyInfo = familyInfo || this.globalData.familyInfo || {};
    // 如果当前没有权限，则不上传
    let _roleNum = _familyInfo.roleNum;
    if (_roleNum == 2) {
        return;
    }
    // 所有设备集合
    let _allDevJson = allDevs || this.globalData.devs || {};
    // 数据获取
    let _data = dataJson || bizUtil.getPrivateData(_allDevJson);
    if (!_data) {
        console.error("uploadPrivateData _data not exist");
        return;
    };
    console.log("uploadPrivateData _data = ", _data);
    
    // 上报的集群域名
    let _groupStr = GROUPS[Number(_familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    console.log("uploadPrivateData _groupStr = ", _groupStr);

    // 家庭信息
    let {sid, skey} = _familyInfo;

    // 兼容
    let _config = {
      did: sid || "",
      token: skey || "",
      familyInfo: _familyInfo, // 上报家庭信息用
      bleCode: familyInfo?.bleCode, // 控制密钥，用于在A家庭下管理上报B家庭数据，可以为空
      host: _groupStr, // 上报的集群域名
      familyProtoStr: familyProtoStr // 家庭私有数据，永久二维码用
    };
    // 如果没有网关且不存在sid和skey则不上报数据 使用无网关的数据上报
    if (!sid && !skey && _data?.gateways?.length == 0) {
      let _uuid = _familyInfo.uuid || noGatewayMethod.createWxUuid();
      // 创建token
      let _uuidToken = noGatewayMethod.createUuidtokenByUuid(_uuid);
      console.log("uploadPrivateData _uuid, _uuidToken = ", _uuid, _uuidToken);
      _config.uuid = _uuid;
      _config.uuidToken = _uuidToken;
    }
    // 上传接口
    let _result = this.globalData.blsdk.updatePrivateData(_data, _config);
    console.log("uploadPrivateData _result = ", _result);
    if (!_result || !_result.then) {
        return;
    } else if ((sid && skey) && !familyProtoStr) {
      // 回调
      !!callback && (callback(sid, skey));
      return;
    }

    _result.then(res => {
      console.error("uploadPrivateData success res = ", res);
      let {status, data} = res || {};
      if (status != 0 && data?.length == 0) {
        !!callback && (callback());
        return;
      }
      let {did, token} = data[0] || {};
      if (_familyInfo && did && token) {
        _familyInfo.sid = did; // 虚拟d
        _familyInfo.skey = token;
        // 回调
        !!callback && (callback(did, token, res));
      }
    }).catch(e => {
      console.error("uploadPrivateData e = ", e);
      // 回调
      !!callback && (callback());
    })
  },
  // 原先是删除网关私有数据，现在接口变更，暂时保留这个方法，方便后续区分
  deletePrivateData (devInfo) {
    // 更新数据
    this.uploadPrivateData();
  },
  resetUserInfo () {
    this.globalData.userInfo = {
      nickname: "游客",
      avatarUrl: "",
      brandname: this.globalData.companyInfo?.brand || ""
    };
    this.setBizStorage("userInfo", this.globalData.userInfo);
  }
})
