// components/customSlider/index.js
// import ability from "../../../../utils/classDef/ability";
import bizUtil from '../../utils/bizUtil'
var app = getApp();
Component({
  options: {
    addGlobalClass: true, // 全局样式
  },
  /**
   * 组件的属性列表
   */
  properties: {
    config: {
      type: Object
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    x: 0, // 移动的距离
    y: 0, // 移动的距离
    tempX: -1,
    percent: -1, // 设置为-1为了保证第一次进入不相等时去刷新数据
    value: 0,
    thumbWidth: 26, // rpx
    multip: app.globalData.customPageSize.multip,
    // 默认的slider
    bgStr: "#F6F8FA",

    thumbOffset: {}, // 容器的页面位置
  },

  observers: {
    'config.percent': function (percent) {
      console.log("observers percent = ", percent);
      if (this.data.value == this.data.config?.value) {
        return;
      }
      // 根据百分比初始化数据
      this.initPositionByPercent(percent / 100);
    },
    'config.bg': function (bg) {
      let _percent = this.data.config?.percent;
      console.log("observers bg = ", bg, _percent);
      if (!bg || _percent == undefined) {
        return;
      }
      // 设置背景颜色
      this.setBgByPercent(_percent / 100);
      // if (this.data.value == this.data.config?.bg) {
      //   return;
      // }
      // // 根据百分比初始化数据
      // this.initPositionByPercent(percent / 100);
    }
  },

  lifetimes: {
    attached () {
      // 更新rpx与px的比
      this.data.multip = app.globalData.customPageSize.multip;
      // 如果有百分比，则初始化位置
      this.initPositionByPercent((this.data.config.percent || 0) / 100);
    }
  },

  ready () {
    console.log("customSlider ready");
    // 取得容器距离页面的距离
    this.getThumbContainerSize();
  },

  pageLifetimes: {
    show () {}
  },

  /**
   * 组件的方法列表
   */
  methods: {
    // 取得容器的边距
    getThumbContainerSize () {
      const query = this.createSelectorQuery()
      query.select('#thumbContainer').boundingClientRect();
      query.exec((res) => {
        // console.log("getThumbContainerSize res = ", res);
        this.data.thumbOffset = res[0] || {};
        // res[0].top       // #the-id节点的上边界坐标
        // res[1].scrollTop // 显示区域的竖直滚动位置
      })
    },
    // 根据滑动的百分比取得真实的百分比
    getRealMutiple (percent) {
      let _minPercent = this.data.config.minPercent || 0;
      return parseInt(percent * (1 - _minPercent / 100)) + _minPercent;
    },
    // 根据真实百分比取得缩小后的百分比
    getPercentMultip (percent) {
      // 根据换算
      let _minPercent = this.data.config.minPercent || 0;
      let _realPercent = (percent - _minPercent) / (100 - _minPercent) * 100;
      return parseInt(_realPercent);
    },
    // 取得最小的百分比值
    getMinPercent () {
      let _min = this.data.config.minPercent;
      let _minPercent = (_min != undefined) ? (_min || 0) : 1;
      return _minPercent;
    },
    getDefaultBg (percent) {
      if (!percent) {
        return "#F6F8FA"
      }
      let _percent = parseInt(percent * 100);
      let _defaultBg = this.data.config?.bg || "#56ABFF";
      if (_percent == 100) {
        return _defaultBg;
      }
      return "linear-gradient(to right, " + _defaultBg + " " + (_percent) + "%, #F6F8FA " + (_percent + 0.01) + "%, #F6F8FA 100%)"
    },

    // 设置背景颜色
    setBgByPercent (percent) {
      this.data.bgStr = this.getDefaultBg(percent);
      this.setData({
        bgStr: this.data.bgStr
      })
    },

    initPositionByPercent (percentVal) {
      if (percentVal == undefined) {
        return;
      }
      let {percent, width} = this.data.config;
      let _percent = percentVal;
      let _position = width * _percent;
      this.data.x = _position - width / 2;
      // console.log("this.data.x, width = ", this.data.x, width, _percent);
      this.setData({
        x: this.data.x,
        
      });
      // 设置背景颜色
      this.setBgByPercent(_percent);
    },
    getPositionByE (e) {
      // console.log("getPositionByE e = ", e);
      let _position = e.changedTouches || e.touches || [];
      return _position[0];
    },
    getPercentByPosition (x) {
      if (x == undefined) {
        return;
      }
      let {width} = this.data.config;
      // let _mul = Math.abs(x);
      if (x > (width / 2)) {
        return 1;
      } else if (x < -(width / 2)) {
        return 0;
      }
      return (x + width / 2) / width;
    },
    dragStart (e) {
      // 小米11手机 clientX = 0
      let {clientX, pageX} = this.getPositionByE(e);
      this.data.tempX = pageX || clientX;
      this.data.originX = this.data.x; // rpx
      console.log("dragStart clientX, this.data.originX = ", clientX, this.data.originX);
    },
    getTimeStamp () {
      return Date.parse(new Date()) / 1000;
    },
    dragMove (e) {
      let {clientX, pageX} = this.getPositionByE(e);
      // let {width} = this.data.config;
      let _multip = this.data.multip;
      // 现在的位置
      let _position = pageX || clientX;
      // console.log("dragMove _position = ", _position);
      // 滑动的距离
      let _moveX = _position - this.data.tempX;
      let _absVal = Math.abs(_moveX);
      // 如果移动比较小，则不显示
      if (_absVal < 1) {
        return;
      }
      // 当前位置
      let _positionX = _moveX * _multip + this.data.originX;
      // 判断滑动是否越界
      let _percent = this.getPercentByPosition(_positionX);
      if (_percent == undefined || _percent < 0 || _percent > 1) {
        return;
      }
      // 最大最小值的处理
      if (_percent == 0) {
        _positionX = -this.data.config.width / 2;
      } else if (_percent == 1) {
        _positionX = this.data.config.width / 2;
      }
      // console.log("dragMove = ",);
      this.data.x = _positionX;
      // 取得滑块的真实百分比 0 - 100
      let _realPercent = this.getRealMutiple(this.getRealPercent(_percent * 100));
      // 根据最大最小值需要除以对应的倍数
      // let _minPercent = this.getMinPercent();
      this.data.percent = _realPercent; // (_realPercent < _minPercent) ? _minPercent : _realPercent;
      // 考虑到最大最小值的百分比
      this.setData({
        x: _positionX,
      });
      // 判断是否默认配置
      this.setBgByPercent(_percent);
    },
    dragEnd (e, val) {
      this.dealTouchEnd();
    },
    dealTouchEnd () {
      // 上传数据
      this.upEvent(this.data.percent / 100);
    },
    dealByPoint (percent) {
      // 数据处理
      let _points = this.data.config.points;
      if (!_points) {
        return percent;
      }
      let _per = 1 / _points.length;
      // 支持节点上报
      let _isHalf = (percent % _per) >= (_per / 2);
      let _realP = parseInt(percent / _per) * _per + (_isHalf ? _per : 0);
      console.log("dealTouchEnd _realP = ", _realP, _isHalf);
      // 设置thumb的位置
      this.initPositionByPercent(_realP);
      return _realP;
    },
    // 取得百分比
    getRealPercent (percent) {
      let _percent = percent || 0;
      if (percent > 99.3) {
        _percent = 100;
      } else if (percent < 0.7) {
        _percent = 0;
      }
      return parseInt(_percent);
    },
    getRealValByPercent (percent) {
      return bizUtil.getRealValByPercent(percent, this.data.config);
    },
    upEvent (percent, isMoving) {
      // 这里有个问题 0.57传进来会被当做 0.56999999 转出来之后变成了56
      let _percentNum = (percent * 100);
      if (_percentNum - parseInt(_percentNum) > 0.99) {
        _percentNum = parseInt(_percentNum) + 1;
      } else {
        _percentNum = parseInt(_percentNum);
      }
      console.log("upEvent percent, _percentNum = ", percent, _percentNum);
      // 取得百分比
      let _percent = this.getRealPercent(_percentNum);
      // 取得真实值
      let _realVal = this.getRealValByPercent(parseInt(_percent));
      let _realValInt = parseInt(_realVal);
      console.log("upEvent _percent, _realVal = ", _percent, _realVal, _realValInt);
      // 保存当前的温度百分比
      this.data.value = _realValInt;
      // 自定义事件 event.detail.value
      let _eventName = 'sliderchange';
      this.triggerEvent(_eventName, {
        percent: _percent,
        originValue: _realVal,
        value: _realValInt,
        isMoving: isMoving
      })
    },
    getSliderBar () {
      const query = wx.createSelectorQuery()
      query.select('#sliderBar').boundingClientRect()
      query.exec((res) => {
        // let _sizeObj = this.getAllHeightFromArray(res);
        console.log("getRealContentHeight res, _sizeObj, this.data.scrollHeight = ", res); // , _sizeObj
      });
    },
    clickSlider (e) {
      // 总长度
      let {multip} = app.globalData.customPageSize;
      let _sliderWidth = this.data.config.width;
      // let _multip = this.data.multip;
      // 容器的座标
      let {left} = this.data.thumbOffset;
      // 点击位置的座标
      let _left = (e.detail.x - (left || 0)) * multip;
      let _marginL = 0; // (width * multip - _sliderWidth) / 2; // rpx
      console.log("clickSlider _left, multip, _sliderWidth, e = ", _left, multip, _sliderWidth, e);
      // 放大缩小的比例
      // let _scales = this.data.config.scales || 1; // 
      // console.log("clickSlider e, _multip = ", _left, _marginL, _sliderWidth / 2, width);
      // let _percent = _left / (this.data.config.width || 1);
      this.data.x = (_left - _marginL - _sliderWidth / 2);
      // console.log("clickSlider this.data.x = ", this.data.x);
      this.setData({
        x: this.data.x
      });
      // 百分比
      let _percent = (this.data.x + _sliderWidth / 2) / _sliderWidth;
      // 取得百分比
      let _percent100 = this.getRealPercent(_percent * 100);
      // 百分比处理
      let _minPercent = this.getMinPercent();
      // 真实的百分比
      let _realPercent = (_percent100 < _minPercent) ? _minPercent : _percent100;

      // 百分比除以100
      let _realPercentBelow1 = _realPercent / 100;
      // 判断是否支持节点
      _realPercentBelow1 = this.dealByPoint(_realPercentBelow1);

      this.setData({
        percent: parseInt(_realPercentBelow1 * 100)
      });
      // 显示滑块
      this.setBgByPercent(_realPercentBelow1);
      // 上报
      this.upEvent(_realPercentBelow1);
    },
  }
})
