// components/customDrag/index.js
Component({
  options: {
    addGlobalClass: true, // 全局样式
  },
  /**
   * 组件的属性列表
   */
  properties: {
    list: {
      type: Object
    },
    config: {
      type: Object
    },
    roomjson: {
      type: Object,
      value: undefined
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    height: 69, // 高度px
    seperator: 10, // 间距px

    dragIdx: -1, // 移动元素的下标
    dragId: -1,

    dataList: [], // 房间列表
    dataLen: 0, // 房间列表长度
    // dataListBak: [],
    moveY: 0, // 移动的距离
    // top: 64, // 默认头部的高度

    y: 0,
    originY: 0,
    allForbidden: true, // 为了防止滑动与滚动冲突 房间id

    roomJson: undefined
  },

  observers: {
    'list': function (params) {
      if (params != this.data.dataList) {
        // console.log("observers list = ", params);
        this.initData(params);
      }
    },
    // 多楼层和房间名称时传递数据，否则为空
    "roomjson": function (json) {
      this.initRoomJson(json);
    }
  },

  lifetimes: {
    // 生命周期函数，可以为函数，或一个在methods段中定义的方法名
    attached: function () {
      console.log("customDrag list = ", this.data.list);
    },
  },
  pageLifetimes: {
    show () {
      // 初始化配置
      this.initConfig();
      // 初始化设备信息
      this.initData();
      // console.log("customDrag pageLifetimes show this.data.list = ", this.data.list);
      // // 计算moveContainer的位置
      //  var query = this.createSelectorQuery().select('#moveContainer').boundingClientRect();
      // query.exec((res) => {
      //   if (!res || res.length == 0) {
      //     return;
      //   }
      //   let _dom = res[0];
      //   this.data.top = _dom.top;
      //   this.data.percent = _dom.width / 750; // 比例
      //   // console.log("res = ", res);
      // });
    }
  },

  /**
   * 组件的方法列表
   */
  methods: {
    initRoomJson (roomjson) {
      let _json = roomjson || {};
      if (Object.keys(_json).length == 0) {
        return;
      }
      console.log("initRoomJson roomjson = ", roomjson);
      // 初始化高度
      this.data.roomJson = _json;
      // 初始化家庭数据
      this.setData({
        roomJson: this.data.roomJson
      });
    },

    initConfig () {
      let _config = this.data.config || {};
      !!_config.height && (this.data.height = _config.height);
      !!_config.seperator && (this.data.seperator = _config.seperator);
      this.setData({
        height: this.data.height,
        seperator: this.data.seperator
      })
    },

    getIndexByY (y) {
      let _seperator = this.data.seperator;
      let _perHeight = _seperator + this.data.height;
      return parseInt(y / _perHeight);
    },
    getYByIndex (index) {
      let _seperator = this.data.seperator;
      let _perHeight = _seperator + this.data.height;
      return index * _perHeight;
    },

    // 初始化数据
    initData (list, exceptIdx) {
      let _list = list || this.data.list || [];
      for (let index in _list) {
        let _item = _list[index];
        if (!_item) {
          continue;
        }
        _item.index = index;
        
        if (index == exceptIdx) {
          continue;
        }
        _item.y = index * (this.data.seperator + this.data.height);
      }
      // console.log("customDrag _list, this.data.list = ", _list);
      this.data.dataList = _list;
      // this.data.dataListBak = _list;
      this.setData({
        dataList: this.data.dataList
      })
    },
    touchMoveV (e) {
      console.log("touchMoveV");
    },
    moveChange (e) {
      let _dragIdx = this.data.dragIdx;
      if (_dragIdx == undefined || _dragIdx < 0) {
        return;
      }
      // 第一次移动
      let {x, y, source} = e.detail;
      if (source != "touch") {
        // console.error("moveChange source y = ", source, y);
        return;
      }
      let _moveY = (y - this.data.originY); // 移动距离
      let _seperator = this.data.seperator;
      let _perHeight = _seperator + this.data.height;
      let _count = parseInt(_moveY / _perHeight);
      // let _px = parseInt(_moveY % _perHeight);
      // let _absPx = Math.abs(_px);
      let _dataList = this.data.dataList;
      // console.log("moveChange y, _count, this.data.dragCount = ", y, _count, this.data.dragCount, _moveY, y / _perHeight);

      // 取得当前元素下标
      let _index = this.getIndexByY(y);
      console.log("moveChange _index = ", _index);
      // 不能移动到第一个
      if (_index == 0 && (_dataList[0] || {}).id == 0) {
        return;
      }

      if (_moveY > 0) {
        // 向下
        _index += 1;
      }
      // 数据超出
      if (_index >= _dataList.length) {
        return;
      }
      if (_dragIdx != _index) {
        console.error("moveChange _index = ", _index, _dragIdx);
        let _item = _dataList[_dragIdx];
        // 保存id
        this.data.dragId = _item.id;
        _item.y = y;
        _dataList.splice(_dragIdx, 1); // 移除
        // console.log("1 _dataList = ", _dataList.concat([]), y);
        _dataList.splice(_index, 0, _item); // 补充
        _dataList[_index].y = y;
        console.log("2 _index = ", _index, _dataList.length);

        // 保存
        this.data.dragIdx = _index;

        let _min = Math.min(_index, _dragIdx);
        let _max = Math.max(_index, _dragIdx);
        for (let i = _min; i <= _max; i++) {
          let _item = _dataList[i];
          if (!_item) {
            continue;
          }
          if (i == _index) {
            _item.y = y;
          } else {
            _item.y = this.getYByIndex(i);
          }
        }

        console.log("4 _dataList = ", _dataList, _min, _max);
        this.setData({
          dataList: _dataList,
          dragId: this.data.dragId
        })
      }

      // 保存当前移动距离为上一次的距离，供下次调用
      this.data.dragCount = _count;
      this.data.lastY = y;
      this.data.lastIdx = _index;
    },
    itemClick (e) {
      // 是否需要上报
      if (this.data.config && this.data.config.disUpEvent) {
        console.error("itemClick = ", this.data.config, this.data.config.disUpEvent);
        return;
      }
      // console.log("itemClick e = ", e);
      // 自定义事件
      this.triggerEvent('itemclick', {
        index: e.currentTarget.dataset.index,
      })
    },
    longItemClick (e) {
      // 是否需要上报
      if (this.data.config && this.data.config.disUpEvent) {
        console.error("itemClick = ", this.data.config, this.data.config.disUpEvent);
        return;
      }
      // console.log("longItemClick e = ", e);
      // 自定义事件
      this.triggerEvent('itemlongclick', {
        index: e.currentTarget.dataset.index,
      })
    },
    touchStart (e) {
      // 是否点击过，如果点击过不可移动的地方，则忽略下面的步骤
      let _positions = this.data.position;
      // console.log("touchStart e, _positions = ", e, _positions);
      if (_positions && _positions.pageX) {
        console.error("touchStart _positions = ", _positions);
        return;
      }

      let _changedTouches = this.getTouchInfo(e);
      // let _changedTouches = e.changedTouches || e.touches;
      if (!_changedTouches) {
        console.error("touchStart _changedTouches = ", _changedTouches);
        return;
      }
      // 如果是从不能滑动的地方点击，则不进入处理
      // if (this.data.allForbidden) {
      //   return;
      // }
      let _touchDom = _changedTouches[0];
      console.error("touchStart _changedTouches, _touchDom = ", _changedTouches, _touchDom);
      // 记住当前的位置
      this.data.position = {
        x: _touchDom.pageX,
        y: _touchDom.pageY
      }
      let _index = e.currentTarget.dataset.index;
      let _item = e.currentTarget.dataset.item || {};
      this.data.originY = _item.y; // 移动的横坐标
      this.data.dragIdx = _index;
      this.data.dragId = _item.id;
      this.data.dragCount = -1;
      this.data.allForbidden = false;
      this.setData({
        dragId: this.data.dragId,
        allForbidden: this.data.allForbidden
      })
    },
    touchMove (e) {
      console.log("touchMove e = ", e);
    },
    // touchEnds (e) {
    //   console.error("touchEnds e = ", e);
    // },
    forbidenDrag (enable) {
      // 取消禁止滑动的事件
      this.data.allForbidden = enable || false;
      this.setData({
        allForbidden: this.data.allForbidden
      })
    },
    touchEnd (e) {
      // 正常处理
      let _changedTouches = this.getTouchInfo(e);
      // console.log("touchEnd e, this.data.allForbidden = ", e, _changedTouches, this.data.allForbidden);
      
      // 取消允许滑动事件
      this.forbidenDrag(true);

      // 是否点击过，如果点击过不可移动的地方，则进入详细
      let _b = this.data.position;
      if (_b && _b.pageX) {
        console.log("itemClick _changedTouches, _b 1 = ", _changedTouches, _b);
        if (!_changedTouches || !_b) {
          console.error("itemClick _changedTouches, _b 2 = ", _changedTouches, _b);
          this.data.position = undefined;
          return;
        }
        // 记住当前的位置
        let _cur = _changedTouches[0];
        console.error("itemClick _cur = ", _cur);
        if (_b.pageX == _cur.pageX && _b.pageY == _cur.pageY) {
          this.itemClick(e);
        }
        this.data.position = undefined;
        return;
      }

      // let _changedTouches = e.changedTouches;
      if (!_changedTouches || !_b) {
        return;
      }
      // 拖动的元素重置
      this.data.dragIdx = -1;
      this.data.dragId = -1;
      this.data.dragCount = -1;
      this.data.position = undefined;
      this.data.originY = 0;
      // 根据位置排序后取得数据 并初始化
      console.log("touchEnd dataList = ", this.data.dataList);
      this.data.dataList = this.data.dataList.sort((a, b) => (a.y - b.y));
      this.initData();
      this.setData({
        dragId: this.data.dragId
      })
      // 拖动结束自定义事件
      this.triggerEvent('dragends', {
        data: this.data.dataList
      })
    },

    getTouchInfo (e) {
      let _result = e.changedTouches || e.touches;
      if (!_result || _result.length != 1) {
        return false;
      }
      return _result;
    },

    catchNotMove (e) {
      this.data.dragIdx = -1;
      this.data.dragId = -1;
      // 不能移动的id
      let _changedTouches = this.getTouchInfo(e);
      console.log("catchNotMove _changedTouches = ", _changedTouches);
      if (!_changedTouches) {
        return
      }
      // 记住当前的位置
      this.data.position = _changedTouches[0];
    },

    // 图片点击
    imgClick (e) {
      console.log("imgClick");
      // 自定义事件
      this.triggerEvent('dragimgclick', {
        index: e.currentTarget.dataset.index,
      })
    }
  }
})

