// components/customPrompt/index.js
Component({
  options: {
    addGlobalClass: true, // 全局样式
  },
  /**
   * 组件的属性列表
   */
  properties: {
    // 配置文件
    promptconfig: {
      type: Object
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    devName: "",
    shows: false
  },

  observers: {
    "promptconfig.devName" (devName) {
      this.setDataDevName(devName);
    },
    "promptconfig.showPrompt" (shows) {
      this.setShows(shows);
    }
  },

  /**
   * 组件的方法列表
   */
  methods: {
    setDataDevName (name) {
      this.data.devName = name || "";
      this.setData({
        devName: this.data.devName
      })
    },
    setShows (show) {
      this.data.shows = show || false;
      this.setData({
        shows: this.data.shows
      })
    },
    nameInput (e) {
      let _name = e.detail.value;
  
      // 只能输入汉字 字母和数字
      _name = _name.replace(/[^\w\u4E00-\u9FA5]/g, '');
      
      // 判断长度
      let _nameLen = _name.length;
      if (_nameLen > 16) {
        return this.showToast("设备名称长度不能大于16").then(() => {
          _name = _name.substr(0, 16);
          // 上报数据
          this.setDataDevName(_name);
        });
      }
      // 上报数据
      this.setDataDevName(_name);
    },
    showToast (str) {
      return wx.showModal({
        showCancel: false,
        content: str || ""
      });
    },
    trigName (name) {
      this.triggerEvent('devname', {
        name: name
      })
    },
    cancel () {
      // 隐藏
      this.setShows(false);
      // 上报
      this.trigName();
    },
    save () {
      let _nameLen = this.data.devName?.length;
      if (_nameLen == 0) {
        return this.showToast("设备名称不能为空");
      }
      // // 隐藏
      // this.setShows(false);
      // 上报
      this.trigName(this.data.devName);
    }
  }
})
