// components/customCall/index.js
const app = getApp();

Component({
  options: {
    addGlobalClass: true, // 全局样式
  },
  /**
   * 组件的属性列表
   */
  properties: {
    config: {
      type: Object
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    content: {}
  },

  observers: {
    'config.devTypeStr': function (devTypeStr) {
      if (!devTypeStr) {
        return;
      }
      // 初始化数据
      this.initContentByConfig(devTypeStr);
    }
  },

  lifetimes: {
    // 生命周期函数，可以为函数，或一个在methods段中定义的方法名
    attached: function () {
      // 根据配置初始化内容
      this.initContentByConfig(this.data.config?.devTypeStr);
    },
  },
  pageLifetimes: {
    show () {
    }
  },

  /**
   * 组件的方法列表
   */
  methods: {
    // 根据设备类型名称取得对应的文案以及图片
    getConfigByDevType (devTypeStr) {
      let _result = {};
      let _devTypeStr = devTypeStr || "panel";
      _result.imgKey = _devTypeStr || '';
      console.log("getConfigByDevType _devTypeStr = ", _devTypeStr);
      if (_devTypeStr == "panel") {
        // 面板文案
        _result.title = "面板唤醒配置步骤：";
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、组合按键#1&#4或者#2&#3，长按4s，进入到设备配置模式，所有指示灯常亮"
        }];
      } else if (_devTypeStr == "uart") {
        // 面板图片
        _result.imgKey = 'panel';
        // 超级面板文案与面板文案一致
        _result.title = "超级面板唤醒配置步骤：";
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、组合按键#1&#4或者#2&#3，长按4s，进入到设备配置模式，所有指示灯常亮"
        }];
      } else if (_devTypeStr == "doorsensor") {
        // 门磁文案
        _result.title = "门磁传感器唤醒配置步骤："
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、请长按传感器复位键5秒，直到红灯闪烁"
        }];
      } else if (_devTypeStr == "bodysensor") {
        // 使用门磁的图标
        _result.imgKey = 'doorsensor';
        // 人体传感器文案
        _result.title = "人体传感器唤醒配置步骤："
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、请长按传感器复位键5秒，直到红灯闪烁"
        }];
      } else if (_devTypeStr == "watersensor") {
        // 水浸文案
        _result.title = "水浸传感器唤醒配置步骤："
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、请长按传感器复位键5秒，直到红灯闪烁"
        }];
      } else if (_devTypeStr == "gateway") {
        // 网关文案
        _result.title = "网关唤醒配置步骤："
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、请长按网关复位键5秒，直到蓝灯闪烁"
        }];
      } else if (_devTypeStr == "temphumidity") {
        // 温湿度传感器文案
        _result.title = "温湿度传感器唤醒配置步骤："
        _result.data = [{
          "text": "1、请距离设备5米以内"
        }, {
          "text": "2、电池重新上电即进入配网状态，状态持续2分钟"
        }];
      }
      return _result;
    },
    // 初始化文案
    initContentByConfig (devTypeStr) {
      let _result = this.getConfigByDevType(devTypeStr);
      console.log("initContentByConfig devTypeStr = ", devTypeStr);
      this.data.content = _result;
      this.setData({
        content: this.data.content
      })
    }
  }
})
