// components/customSlider/index.js
import ability from "../../utils/classDef/ability";
import bizUtil from '../../utils/bizUtil'
var app = getApp();
Component({
  options: {
    addGlobalClass: true, // 全局样式
  },
  /**
   * 组件的属性列表
   */
  properties: {
    config: {
      type: Object
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    x: 0, // 移动的距离
    y: 0, // 移动的距离
    tempX: -1,
    percent: 0,
    value: 0,
    timeStamp: 0, // 时间戳
    thumbWidth: 64, // rpx
    multip: app.globalData.customPageSize.multip,
    // sliderWidth: 0
    // 默认的slider
    isDefault: true,
    bgStr: "#E1E1E1",
    timer: null, // 长按定时器
    isLongTap: false, // 是否长按标志
    isDel: false, // 默认减
    timeStamp: 0 // 点击开始时间戳
  },

  observers: {
    'config.percent': function (percent) {
      // 根据换算
      // let _realPercent = this.getPercentMultip(percent);
      console.log("observers percent = ", percent);
      let _percent = percent;
      let _minPercent = this.data.config.minPercent;
      if (_percent < _minPercent) {
        _percent = _minPercent;
      }
      // 根据百分比初始化数据
      this.initPositionByPercent(_percent / 100);
      // 更新数据
      this.setPercent(_percent);
    }
  },

  lifetimes: {
    attached () {
      // console.log("attached config = ", this.data.config);
      // 更新rpx与px的比
      this.data.multip = app.globalData.customPageSize.multip;
      // 是否默认配置
      // console.log("this.data.isDefault = ", this.data.config);
      this.data.isDefault = !(this.data.config || {}).bgImg;
      // console.log("this.data.isDefault = ", this.data.isDefault);
      // 如果有百分比，则初始化位置
      this.initPositionByPercent((this.data.config.percent || 0) / 100);
      // // 设置
      // this.setData({
      //   isDefault: this.data.isDefault
      // })
    }
  },

  pageLifetimes: {
    show () {}
  },

  /**
   * 组件的方法列表
   */
  methods: {
    // 根据滑动的百分比取得真实的百分比
    getRealMutiple (percent) {
      let _minPercent = this.data.config.minPercent || 0;
      return parseInt(percent * (1 - _minPercent / 100)) + _minPercent;
    },
    // 根据真实百分比取得缩小后的百分比
    getPercentMultip (percent) {
      // 根据换算
      let _minPercent = this.data.config.minPercent || 0;
      let _realPercent = (percent - _minPercent) / (100 - _minPercent) * 100;
      return parseInt(_realPercent);
    },
    // 取得最小的百分比值
    getMinPercent () {
      let _min = this.data.config.minPercent;
      let _minPercent = (_min != undefined) ? (_min || 0) : 1;
      return _minPercent;
    },
    // 设置显示的百分比
    setPercent (percent) {
      if (percent == undefined) {
        return;
      }
      let _percent = parseInt(percent);
      console.log("_percent = ", _percent);
      if (_percent < 0) {
        _percent = 0;
      } else if (_percent > 100) {
        _percent = 100;
      }
      this.data.percent = _percent || this.getMinPercent();
      console.log("this.data.percent = ", _percent, this.data.percent);
      this.setData({
        percent: this.data.percent,
      });
      // 更新数据
      this.setValueByPercent(this.data.percent);
    },
    setValueByPercent (percent) {
      // 更新值
      if (!this.data.config?.unit) {
        return;
      }
      let _value = Math.round(this.getRealValByPercent(percent));
      console.log("setValueByPercent _value, percent = ", _value, percent)
      this.setData({
        value: _value
      });
    },
    getDefaultBg (percent) {
      if (!percent) {
        return "#E1E1E1"
      }
      let _percent = parseInt(percent * 100);
      if (_percent == 100) {
        return "#F78D3F"
      }
      return "linear-gradient(to right, #F78D3F " + (_percent) + "%, #E1E1E1 " + (_percent + 0.01) + "%, #E1E1E1 100%)"
    },

    dealByDefault (percent) {
      // console.log("dealByDefault percent, this.data.isDefault = ", percent, this.data.isDefault);
      if (this.data.isDefault) {
        this.data.bgStr = this.getDefaultBg(percent);
        this.setData({
          bgStr: this.data.bgStr
        })
      }
    },

    initPositionByPercent (percentVal) {
      if (percentVal == undefined) {
        return;
      }
      let {percent, width} = this.data.config;
      let _percent = percentVal;
      let _position = width * _percent;
      this.data.x = _position - width / 2;
      // console.log("this.data.x, width = ", this.data.x, width, _percent);
      this.setData({
        x: this.data.x
      });
      // 判断是否默认配置
      this.dealByDefault(_percent);
    },
    getPositionByE (e) {
      // console.log("getPositionByE e = ", e);
      let _position = e.changedTouches || e.touches || [];
      return _position[0];
    },
    getPercentByPosition (x) {
      if (x == undefined) {
        return;
      }
      let {width} = this.data.config;
      // let _mul = Math.abs(x);
      if (x > (width / 2)) {
        return 1;
      } else if (x < -(width / 2)) {
        return 0;
      }
      return (x + width / 2) / width;
    },
    dragStart (e) {
      // 小米11手机 clientX = 0
      let {clientX, pageX} = this.getPositionByE(e);
      this.data.tempX = pageX || clientX;
      this.data.originX = this.data.x; // rpx
      // this.data.timeStamp = this.getTimeStamp();
      console.log("dragStart clientX, this.data.originX = ", clientX, this.data.originX);
    },
    getTimeStamp () {
      return Date.parse(new Date()) / 1000;
    },
    dragMove (e) {
      let {clientX, pageX} = this.getPositionByE(e);
      // let {width} = this.data.config;
      let _multip = this.data.multip;
      // 现在的位置
      let _position = pageX || clientX;
      // console.log("dragMove _position = ", _position);
      // 滑动的距离
      let _moveX = _position - this.data.tempX;
      let _absVal = Math.abs(_moveX);
      // 如果移动比较小，则不显示
      if (_absVal < 1) {
        return;
      }
      // 当前位置
      let _positionX = _moveX * _multip + this.data.originX;
      // 判断滑动是否越界
      let _percent = this.getPercentByPosition(_positionX);
      if (_percent == undefined || _percent < 0 || _percent > 1) {
        return;
      }
      // 最大最小值的处理
      if (_percent == 0) {
        _positionX = -this.data.config.width / 2;
      } else if (_percent == 1) {
        _positionX = this.data.config.width / 2;
      }
      // console.log("dragMove = ",);
      this.data.x = _positionX;
      // 取得滑块的真实百分比 0 - 100
      let _realPercent = this.getRealMutiple(this.getRealPercent(_percent * 100));
      // 根据最大最小值需要除以对应的倍数
      // let _minPercent = this.getMinPercent();
      this.data.percent = _realPercent; // (_realPercent < _minPercent) ? _minPercent : _realPercent;
      // console.log("dragMove _percent = ", _realPercent, _minPercent, this.data.percent);
      // 考虑到最大最小值的百分比
      this.setData({
        x: _positionX,
        percent: this.data.percent
      });
      // 判断是否默认配置
      this.dealByDefault(_percent);
      
      // 更新数据
      this.setValueByPercent(this.data.percent);

      // 如果不支持渐变，则退出
      if (this.data.config?.upEventWhileMove) {
        this.upEvent(_percent, true);
      }

      // // 如果不支持渐变，则退出
      // if (!this.data.config.gradient) {
      //   return;
      // }
      // // 如果超过1s,则上报
      // let _timestamp = this.getTimeStamp();
      // if (_timestamp - this.data.timeStamp < 1) {
      //   return;
      // }
      // // 更新
      // this.data.timeStamp = _timestamp;
      // // 上报
      // this.upEvent(_percent, true);
    },
    dragEnd (e, val) {
      this.dealTouchEnd();
    },
    dealTouchEnd () {
      // let _originX = this.data.x;
      // let _allWidth = this.data.config.width;
      // // 百分比数据
      // let _percent = (_originX + _allWidth / 2) / _allWidth;
      // console.log("dealTouchEnd _percent = ", _percent);
      
      // // 判断是否支持节点
      // _percent = this.dealByPoint(_percent);
      // 上传数据
      this.upEvent(this.data.percent / 100);
    },
    dealByPoint (percent) {
      // 数据处理
      let _points = this.data.config.points;
      if (!_points) {
        return percent;
      }
      let _per = 1 / _points.length;
      // 支持节点上报
      let _isHalf = (percent % _per) >= (_per / 2);
      let _realP = parseInt(percent / _per) * _per + (_isHalf ? _per : 0);
      console.log("dealTouchEnd _realP = ", _realP, _isHalf);
      // 设置thumb的位置
      this.initPositionByPercent(_realP);
      return _realP;
    },
    // 取得百分比
    getRealPercent (percent) {
      let _percent = percent || 0;
      if (percent > 99.3) {
        _percent = 100;
      } else if (percent < 0.7) {
        _percent = 0;
      }
      return parseInt(_percent);
    },
    getRealValByPercent (percent) {
      return bizUtil.getRealValByPercent(percent, this.data.config);
    },
    leftClick () {
      this.setLeftRightClick(0);
    },
    rightClick () {
      this.setLeftRightClick(100);
    },
    setLeftRightClick (value) {
      // 暂时只支持窗帘
      if (this.data.config.devType != "curtain") {
        return;
      }
      let _realPercent = value || 0;
      let _percent = _realPercent / 100;
      // 设置位置
      this.initPositionByPercent(_percent);
      // 显示滑块
      this.dealByDefault(_percent);
      // 更新百分比数据
      this.setValueByPercent(_realPercent);
      // 更新数据
      this.setPercent(_realPercent);
      // 最右侧文字点击
      this.upEvent(_realPercent, false);
    },
    upEvent (percent, isMoving, realValue) {

      // 默认
      let _percent = 0;
      let _realVal = 0;
      let _realValInt = 0;

      // 是否上报真实值
      if (realValue != undefined) {
        _percent = bizUtil.getPercentByVal(realValue, this.data.config);
        _realVal = realValue;
        _realValInt = parseInt(_realVal);
      } else {
        // 这里有个问题 0.57传进来会被当做 0.56999999 转出来之后变成了56
        let _percentNum = (percent * 100);
        if (_percentNum - parseInt(_percentNum) > 0.99) {
          _percentNum = parseInt(_percentNum) + 1;
        } else {
          _percentNum = parseInt(_percentNum);
        }
        console.log("upEvent percent, _percentNum = ", percent, _percentNum);
        // 取得百分比
        _percent = this.getRealPercent(_percentNum);
        // 取得真实值
        _realVal = this.getRealValByPercent(parseInt(_percent));
        _realValInt = parseInt(_realVal);
      }

      console.log("upEvent _percent, _realVal = ", _percent, _realVal, _realValInt);
      // 自定义事件 event.detail.value
      let _eventName = ((this.data.config || {}).bgImg) ? 'percentchange' : 'brightchange';
      this.triggerEvent(_eventName, {
        percent: _percent,
        originValue: _realVal,
        value: _realValInt,
        isMoving: isMoving
      })
    },
    getSliderBar () {
      const query = wx.createSelectorQuery()
      query.select('#sliderBar').boundingClientRect()
      query.exec((res) => {
        // let _sizeObj = this.getAllHeightFromArray(res);
        console.log("getRealContentHeight res, _sizeObj, this.data.scrollHeight = ", res); // , _sizeObj
      });
    },
    clickSlider (e) {
      // 总长度
      let {width, multip} = app.globalData.customPageSize;
      let _sliderWidth = this.data.config.width;
      // let _multip = this.data.multip;
      //  - e.target.offsetLeft
      let _left = (e.detail.x) * multip;
      let _marginL = (width * multip - _sliderWidth) / 2; // rpx
      // 放大缩小的比例
      // let _scales = this.data.config.scales || 1; // 
      // console.log("clickSlider e, _multip = ", _left, _marginL, _sliderWidth / 2, width);
      // let _percent = _left / (this.data.config.width || 1);
      this.data.x = (_left - _marginL - _sliderWidth / 2);
      // console.log("clickSlider this.data.x = ", this.data.x);
      this.setData({
        x: this.data.x
      });
      // 百分比
      let _percent = (this.data.x + _sliderWidth / 2) / _sliderWidth;
      // 取得百分比
      let _percent100 = this.getRealPercent(_percent * 100);
      // 百分比处理
      let _minPercent = this.getMinPercent();
      // 真实的百分比
      let _realPercent = (_percent100 < _minPercent) ? _minPercent : _percent100;

      // 百分比除以100
      let _realPercentBelow1 = _realPercent / 100;
      // 判断是否支持节点
      _realPercentBelow1 = this.dealByPoint(_realPercentBelow1);

      this.setData({
        percent: parseInt(_realPercentBelow1 * 100)
      });
      // 显示滑块
      this.dealByDefault(_realPercentBelow1);
      // 更新百分比数据
      this.setValueByPercent(_realPercent); 
      // 上报
      this.upEvent(_realPercentBelow1);
    },
    // 加减按钮点击的事件
    plus () {
      this.data.isLongTap = false;// 是否长按标志
      console.log("plus");
      this.dealImgClick(false);
    },
    minus () {
      this.data.isLongTap = false;// 是否长按标志
      console.log("minus");
      this.dealImgClick(true);
    },
    dealImgClick (isDel, notUpload) {
      // 当前百分比
      let _percent100 = (this.data.percent || 0);
      // 百分比处理
      let _minPercent = this.getMinPercent();
      console.log("dealImgClick _percent100, _minPercent = ", _percent100, _minPercent);
      let _maxPercent = 100;
      // 判断是否最大最小值
      if ((_percent100 <= _minPercent) && isDel) {
        ability.showToast({
          title: "已经是最小值" // 提示信息
        });
        // 清理定时器
        this.clearTimer(this.data.timer);
        return false;
      } else if ((_maxPercent <= _percent100) && !isDel) {
        // 最大值再加
        ability.showToast({
          title: "已经是最大值" // 提示信息
        });
        // 清理定时器
        this.clearTimer(this.data.timer);
        return false;
      }

      // 真实的值
      let _realValue = 0;
      // 判断是值的增加还是百分比的增加 changeValue
      let {changeValue, value} = this.data.config || {};
      if (changeValue) {
        console.log("dealImgClick changeValue, value = ", changeValue, value);
        // 是否修改值，默认为修改百分比
        let _value = value != undefined ? (Number(value) || 0) : Math.round(this.getRealValByPercent(_percent100));
        console.log("dealImgClick _value = ", _value);
        _realValue = _value + (isDel ? -1 : 1);
        // 真实的值
        this.data.config.value = _realValue;
        _percent100 = bizUtil.getPercentByVal(_realValue, this.data.config);
      } else {
        // 百分比调节
        _percent100 = _percent100 + (isDel ? -1 : 1);
      }

      // 真实的百分比
      let _realPercent = (_percent100 < _minPercent) ? _minPercent : (_percent100 > 100 ? 100 : _percent100);
      console.log("_realPercent = ", _realPercent);
      this.setData({
        percent: _realPercent
      });
      let _percent = _realPercent / 100;
      // 设置位置
      this.initPositionByPercent(_percent);
      // 显示滑块
      this.dealByDefault(_percent);
      // 更新百分比数据
      this.setValueByPercent(_realPercent);
      
      // 上传数据
      if (!changeValue) {
        !notUpload && (this.upEvent(_percent));
      } else {
        !notUpload && (this.upEvent(_percent, undefined, _realValue));
      }
      return true;
    },
    longMinus (e) {
      console.log("longMinus e = ", e);
      this.dealLongTap(true);
    },
    longPlus (e) {
      console.log("longPlus");
      this.dealLongTap(false);
    },

    minusStart () {
      this.data.isDel = true;
      this.dealTouchStart();
    },
    plusStart () {
      this.data.isDel = false;
      this.dealTouchStart();
    },
    dealTouchStart () {
      // 默认长按事件
      this.data.isLongTap = true;
      // 记住按下的时间
      this.data.timeStamp = Date.now();
      setTimeout(() => {
        // 是否长按标志
        if (!this.data.isLongTap) {
          return;
        } else {
          // 初始化定时器
          this.dealLongTap(this.data.isDel);
        }
      }, 800);
    },
    minusEnd () {
      this.dealImgTouchEnd(true);
    },
    plusEnd () {
      this.dealImgTouchEnd(false);
    },
    dealImgTouchEnd (isDel) {
      // 清理定时器
      this.clearTimer(this.data.timer);
      // 单位毫秒
      let _now = Date.now();
      if ((_now - this.data.timeStamp) < 700) {
        // 是否长按标志
        this.data.isLongTap = false;
        // 短按事件
        !!isDel ? (this.minus()) : (this.plus());
      } else {
        // 是否长按标志
        this.data.isLongTap = false;
        // 长按结束上报数据
        this.upEvent(this.data.percent / 100)
      }
    },
    // 长按事件
    dealLongTap (isDel) {
      // 清理定时器
      this.clearTimer(this.data.timer);
      // 初始化定时器
      this.data.timer = setInterval(() => {
        if (!this.data.timer) {
          return;
        }
        console.log("dealLongTap");
        this.dealImgClick(isDel);
      }, 300);
    },
    clearTimer (timer) {
      bizUtil.clearTimer(timer, "interval");
    }
  }
})
