// device/aircondition/control/index.js
import ability from "../../../utils/classDef/ability";
import bizUtil from '../../../utils/bizUtil';
import AirConfig from "../../../static/js/jsConfig/aircondition"
import {Utils} from "../../../utils/util"
const app = getApp();

// 模式和风速的枚举值
const MODE = AirConfig.MODE;
var SPEED = [];
if (app.globalData.companyInfo.wxName == "Aux") {
  SPEED = [
    {
      text: "低风",
      value: 1,
      key: "low"
    }, {
      text: "中风",
      value: 2,
      key: "mid"
    }, {
      text: "高风",
      value: 3,
      key: "high"
    }, {
      text: "自动",
      value: 0,
      key: "auto"
    }, {
      text: "静音",
      value: 4,
      key: "mute"
    }, {
      text: "中低风",
      value: 5,
      key: "midlow"
    }, {
      text: "中高风",
      value: 6,
      key: "midhigh"
    }, {
      text: "强力",
      value: 7,
      key: "strong"
    }
  ];
} else {
  SPEED = AirConfig.SPEED;
}

Page({
  data: {
    // 显示列表详情
    showDeail: false,
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: app.globalData.devInfo?.n || ''
      }
    },
    controlList: [], // 底部按钮
    bleUnavailable: false,
    scales: "none", // 适配小屏幕手机，暂时方案为 整体缩小

    devStatus: {}, // 默认的状态，用于继电器面板

    showErrorCode: true, // 是否显示设备故障码
    errorMsg: "", // 故障码内容
    // tmpTempVal: 24, // 滑块滑动时不能更新devStatus，因为失败时温度要更新回去，所以额外增加变量

    sheetList: [], // 展示的actionsheet配置
    showActionSheet: false, // 是否显示

    tempSlider: {
      width: 445, // rpx
      percent: 100, // 百分比
      labelTxt: "亮度",
      minVal: AirConfig?.TEMPS?.min,
      maxVal: AirConfig?.TEMPS?.max,
      minPercent: 0, // 最小百分比
      gradient: false, // 默认不支持滑动时调节
      labelClass: 'color7F828A',
      bg: "#F78D3F"
    },

    showDetail: false, // 是否展示详情列表
    detailList: [], // 详情列表

    // devTypeStr: "" // 设备类型
  },
  onLoad () {
    // 初始化设备类型
    this.devTypeStr = this.initDevTypeStr();
    
    // 判断是否用于空调demo调试用
    this.isUartDemo = this.isDemoTest();
    if (this.isUartDemo) {
      this.data.devStatus = this.getDemoDefaultStatus() || {};
      return;
    }

    // 取得设备状态
    this.data.devStatus = this.getDevStatus() || {};

    // 取得详细
    let _detailList = this.getDetailListByV(this.devTypeStr, this.data.devStatus);
    this.setData({
      detailList: _detailList
    })
  },
  initDevTypeStr () {
    let {t} = app.globalData.devInfo || {};
    return bizUtil.getDevTypeStr(t);
  },
  getDetailListByV (devTypeStr, devStatus) {
    let _list = this.getAllDetailList();
    if (devTypeStr == "aircondition") {
      return _list.slice(0, 1);
    }
   
    let {hot, wind, airhotwind} = devStatus || {};
    console.log("getDetailListByV hot, wind, airhotwind = ", hot, wind, airhotwind);
    // 有空调和地暖的时候
    if ((!!hot && (airhotwind == 4 || airhotwind == 7)) || (!!wind && (airhotwind == 5 || airhotwind == 7))) {
      return _list;
    }
    return _list.slice(0, 1);
  },
  getAllDetailList () {
    return [{
      title: "设备详情",
      bindTap: "jumpDetail",
      key: "detail"
    }, {
      title: "功能设置",
      bindTap: "jumpFuncSet",
      key: "funcset"
    }]
  },
  onReady () {
  },
  onShow () {

    // 隐藏对话框
    this.showDetailDialog();

    // 设置标题
    this.setNavTitle(app.globalData.devInfo?.n);
    // 初始化List
    this.initLists();

    // 如果是透传uart用则不查询设备快照
    if (this.isUartDemo) {
      return;
    }

    // 查询设备快照
    this.batchqueryDevSnap(app.globalData.devInfo);
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(title || "");
  },
  getPercentByValue (value, tempSlider) {
    return bizUtil.getPercentByVal(value, tempSlider)
    // let {minVal, maxVal} = this.getTempMinAndMaxVal();
    // return (value - minVal) / (maxVal - minVal) * 100;
  },
  // 取得设备信息
  getDevStatusByDevInfo (devInfo) {
    if (!devInfo) {
      return {};
    }
    // 取得所有设备保存的状态数据
    let _allDevStatus = app.globalData.allStatus || {};
    // 根据设备did+短地址获取设备状态
    let {a, d} = devInfo;
    // 设备状态
    return _allDevStatus[d + "_" + a] || {};
  },
  getDevStatus () {
    // 根据设备did+短地址获取设备状态
    let _devInfo = app.globalData.devInfo || {};
    // 设备状态
    let _oldDevStatus = this.getDevStatusByDevInfo(_devInfo);
    console.log("getDevStatus _oldDevStatus = ", _oldDevStatus);
    // 默认状态
    let _defaultObj = {
      pwr: 1,
      mode: 2, // 制热 1 => 默认制冷
      speed: 1,  // 低速
      temp: 26,
      unit: "℃",
    };
    if (this.devTypeStr == "thermostat") {
      // 三合一面板
      let {airhotwind} = bizUtil.getThermostatAbility(_devInfo?.v, this.devTypeStr);

      _defaultObj.airhotwind = airhotwind;
      console.log("getDevStatus airhotwind = ", airhotwind);
      if (airhotwind == 2) {
        // 只有地暖 没有风速
        // delete _defaultObj.speed;
        // delete _defaultObj.mode;
        // 单地暖
        _defaultObj.hot = "hot";
        _defaultObj.mode = 1;
      } else if (airhotwind == 3) {
        // // 只有新风
        // delete _defaultObj.temp;
        // delete _defaultObj.mode;
        _defaultObj.mode = 4; // 默认新风
        // 单新风
        _defaultObj.wind = "wind";
      } else if (airhotwind == 6) {
        // // 地暖+新风
        // delete _defaultObj.mode;
        // 单新风
        _defaultObj.hot = "hot";
        _defaultObj.wind = "wind";
        _defaultObj.mode = 1; // 默认制热模式
      }
      // 有空调和地暖的时候
      if (airhotwind == 4 || airhotwind == 7) {
        _defaultObj.hot = "air,hot"; // 默认只有空调
      }
      // 有空调和新风
      if (airhotwind == 5 || airhotwind == 7) {
        _defaultObj.wind = "air,wind"; // 默认只有空调
      }
    }
    console.log("getDevStatus _defaultObj = ", _defaultObj);
    return _defaultObj; // _oldDevStatus
  },
  onUnload () {
    // 页面销毁 如果设置过百分比和最佳位置则保存设备状态
    this.saveDevStatus();
    // 重置
    this.resetSendConfig();
  },
  saveDevStatus (onlyCache) {
    // 保存的设备状态
    let _allDevStatus = {}; // app.globalData.allStatus || {};
    let _dev = app.globalData.devInfo;
    if (!_dev?.d) {
      return;
    }
    // 设备状态
    _allDevStatus[_dev.d + "_" + _dev.a] = this.data.devStatus;
    app.globalData.allStatus = _allDevStatus;

    // 如果仅保存信息
    if (onlyCache) {
      return;
    }
    console.log("saveDevStatus _allDevStatus = ", _allDevStatus);
    // 保存到缓存中
    app.setBizStorage("devStatus", _allDevStatus);
  },
  // 区分群控页面
  initLists () {
    let _devStatus = this.data.devStatus;
    // 因为温控器可以修改制热和制冷设备值
    if (this.devTypeStr == "thermostat") {
      let {hot, wind} = this.getDevStatusByDevInfo(app.globalData.devInfo);
      _devStatus.hot = hot || "";
      _devStatus.wind = wind || "";
    }
    console.log("initLists _devStatus = ", _devStatus);
    // 初始化控制列表
    let {list} = this.getControlList(_devStatus) || [];
    // 根据设备状态处理数据
    let _showList = this.setDataForList(list, _devStatus);
    
    // 如果是模式，则更新模式名称用于头部显示
    if (_devStatus.mode != undefined) {
      let {text, key} = this.getConfigByValueInArray(MODE, _devStatus.mode);
      _devStatus.modeName = text;
      // 更新模式对应的背景图片
      _devStatus.bgKey = (key == "hot" || key == "dehumid") ? ("-" + key) : "-cold";
    } else {
      _devStatus.bgKey = "-cold";
    }

    // 是否制热模式
    let _isHot = _devStatus.bgKey == "-hot";
    // 是否初始模式
    let _isDehumid = _devStatus.bgKey == "-dehumid"
    // 设置温度值
    let _temp = _devStatus?.temp
    console.log("initLists _temp = ", _temp);
    if (_temp != undefined) {
      let _tempValue = _temp || 24;
      this.data.tempSlider.value = _tempValue;
      this.data.tempSlider.bg = _isHot ? '#F78D3F' : (_isDehumid ? '#23D2B2' : ''); // 设置默认的背景

      let {airhotwind} = _devStatus;
      console.log("initLists airhotwind = ", airhotwind);
      if (airhotwind == 2 || airhotwind == 6 || airhotwind == 7) {
        // 单地暖或者地暖+新风
        this.data.tempSlider.minVal = AirConfig?.TEMPS?.hotMin;
        this.data.tempSlider.maxVal = AirConfig?.TEMPS?.hotMax;
      }
      this.data.tempSlider.percent = this.getPercentByValue(_tempValue, this.data.tempSlider);
    }
    // console.log("_tempValue, this.data.tempSlider.minVal = ", _tempValue, this.data.tempSlider.minVal)
    // if (_tempValue < 19) {
    //   this.data.tempSlider.labelClass = " "; // 防止使用默认的白色
    // } else {
    //   this.data.tempSlider.labelClass = ""; // 使用默认的白色
    // }
    // this.setProperValue("tempSlider", this.data.tempSlider);

    this.setData({
      tempSlider: this.data.tempSlider,
      devStatus: _devStatus,
      controlList: _showList
    })
  },

  // // 根据模式的值设置互斥
  // setHuchiByStatus (status, list) {
  //   if (!status || !list) {
  //     return;
  //   }
  //   let _mode = status.mode;
  //   let _item = list.filter(item => item.key == "temp");
  //   if (_item.length == 0) {
  //     return;
  //   }
  //   _item[0].hidden = (_mode != 1 && _mode != 2); // 只有制冷制热才能设置温度
  // },

  // 根据数据区分
  getControlList (devStatus) {
    // 控制列表
    let list = [{
      title: '开关',
      key: "pwr",
      type: "none"
    }, {
      title: '温度',
      key: "temp",
      type: "slider"
    }, {
      title: '模式',
      key: "mode",
      type: "enum"
    }, {
      title: '风速',
      key: "speed",
      type: "enum"
    }];

    // 判断是否透传测试用
    if (this.isUartDemo) {
      let _extentList = this.getDemoTestExtendList() || [];
      list = list.concat(_extentList);
      // // 设置默认的值
      // let {key} = list[0] || {};
      // if (key == "pwr") {
      //   list[0].values = ["000101", "000100"];
      // }
    }

    // 根据设备状态显示对应的状态
    let _filters = list.filter(item => devStatus[item.key] != undefined);

    return {
      list: _filters
    };
  },

  /***************** 设备控制相关 ***************/
  showLoading (str) {
    ability.showLoading({
      title: str || ""
    })
  },
  hideLoading () {
    ability.hideLoading();
  },
  jumpDetailClick () {
    // 如果是透传测试，则跳转透传页面
    if (this.isUartDemo) {
      ability.navigateTo("../../uart/control/index")
      return;
    }
    // 判断是否大于2个选项
    if (this.data.detailList?.length > 1) {
      // 显示
      this.showDetailDialog(!this.data.showDetail);
      return;
    }
    // 跳转详情
    this.jumpDetail();
  },
  // 跳转设备详情页面
  jumpDetail () {
    // 空调网关还是空调内机
    let {sub, t, v} = app.globalData.devInfo || {};
    let _devTypeStr = this.devTypeStr;

    if (_devTypeStr == "thermostat") {
      // 三合一面板
      // let {air, hot, wind} = bizUtil.getThermostatAbility(v, t);
      let {hot, wind, airhotwind} = this.data.devStatus;
      let _extendStr = "&airhotwind=" + airhotwind;
      // _extendStr = "&hot=air%2Chot&wind=air%2Cwind"; // for test
      // // 有空调和地暖的时候
      if (hot) {
        // let _str1 = this.getActionDeviceByStatus("hot", this.data.devStatus);
        _extendStr += ("&hot=" + encodeURIComponent(hot))
        console.log("hot = ", hot);
      }
      // 有空调和新风
      if (wind) {
        // let _str2 = this.getActionDeviceByStatus("wind", this.data.devStatus);
        console.log("wind = ", wind);
        _extendStr += ("&wind=" + encodeURIComponent(wind))
      }
      console.log("jumpDetail _extendStr = ", _extendStr);
      // 仅保存设备状态
      this.saveDevStatus(true);
      // 跳转页面
      ability.navigateTo("../../light/devProperty/index?remote=0" + _extendStr);
    } else {
      // 空调网关
      if (sub != undefined) {
        return ability.navigateTo('/device/aircondition/manage/index');
      }
      // 跳转页面
      ability.navigateTo("../../light/devProperty/index?remote=0");
    }
  },
  getActionDeviceByStatus (key, devStatus) {
    let {air, hot, wind} = devStatus || {};
    console.log("getActionDeviceByStatus air, hot, wind = ", air, hot, wind);
    let _result = [];
    air && (_result.push("air")); // 是否存在空调
    if (key == "hot") {
      hot && (_result.push("hot")); // 是否存在地暖
    } else if (key == "wind") {
      wind && (_result.push("wind")); // 是否存在新风
    }
    return _result.join(",")
  },
  funcClick (e) {
    let {index} = e.currentTarget.dataset;
    let {key} = this.data.controlList[index];
    console.log("funcClick index, key = ", index, key);

    if (key == "pwr") {
      // todo list 第一个字节为panelId, 第二个字节为开关
      this.actDev([1 << index, 1 << index]);
    } else if (key == "temp") {
      // todo list
      this.actDev([1 << index, 0]);
    } else if (key == "mode" || key == "speed") {
      console.log("funcClick devStatus = ", this.data.devStatus);
      // 当前选中的数据
      let _dealActionList = this.getActionShowList(key, this.data.devStatus);
      if (_dealActionList.length <= 1) {
        // 如果只有一条数据，则不弹出提示框
        return;
      }
      // 显示actionsheet和内容
      this.actionSheetShow();
      console.log("funcClick _dealActionList = ", _dealActionList);
      this.setProperValue("sheetList", _dealActionList);
    }
  },
  actDev (cmdKey, callback, config) {

    // // test
    // !!callback && (callback(0));
    // return;

    // 判断是否透传测试用
    if (this.isUartDemo) {
      this.uartSendCmd(cmdKey, callback);
      return;
    }

    // 业务逻辑处理
    let _cmdData = this.dealCmdByBiz(cmdKey, config);

    let _devInfo = app.globalData.devInfo;
    console.log("actDev _devInfo, cmdKey, _cmdData = ", _devInfo, cmdKey, _cmdData);
    let _sendResult = app.globalData.blsdk.controlWithDevice(_devInfo, _cmdData);
    console.log("actDev _sendResult = ", _sendResult);
    if (!callback) {
      return;
    }
    if (!(_sendResult?.then)) {
      return callback(-1);
    }
    
    callback && (callback(0));
    // _sendResult.then(() => {
    //   callback && (callback(0));
    // })
  },

  // 根据业务逻辑发送的指令特殊处理（温控器）
  dealCmdByBiz (cmdKey, config) {
    // 如果是温控器且cmdKey是json个数，则取判断
    if (this.devTypeStr == "thermostat" && typeof cmdKey != "string" && !(cmdKey instanceof Array)) {
      let {sendKey, key} = config || {};
      // 如果是模式控制，且是温控器，需要判断是哪些响应
      if (sendKey == "mode") {
        // 模式控制
        // 制热模式可配置使用空调制热、地暖制热或者空调、地暖同时制热。
        // 通风模式可配置使用空调通风、新风通风或者空调、新风同时通风
        if ((['hot', 'fan'].indexOf(key) >= 0)) {
          // 取得设置
        }
        // 制冷模式和除湿模式仅开启空调
        // 时钟鑫那款温控器不支持自动，但是还是会显示，按照制冷和除湿逻辑设置 只设置空调
        else {}
      } else if (cmdKey.pwr >= 0) {
        // 开关控制
        // 开机时根据当前默认模式配置项，开启对应设备
        // 关机时会同时关闭空调、新风和地暖    
      }
      else if (cmdKey.speed >= 0) {
        // 风速控制
        // 制冷模式下仅调节空调风速
        // 制热和通风模式下，根据当前默认模式配置，控制对应设备调节风速。
        // 除湿模式下，不支持调节风速。 只调节空调
      }
    }
    return cmdKey;
  },

  // 取得展示的列表
  getActionShowList (key, devStatus) {
    let defalutValue = devStatus[key]; // 默认默认的值
    let {airhotwind} = devStatus;
    console.log("getActionShowList airhotwind, key = ", airhotwind, key);
    // 数据赋值
    let _list = [];
    let _filter = [];
    if (key == "mode") {
      _list = MODE;
      // 只有新风
      if (airhotwind == 3) {
        // 只有送风模式
        _filter = ["fan"];
      } else if (airhotwind == 2) {
        // 只有制热模式
        _filter = ["hot"];
      } else if (airhotwind == 6) {
        // 制热和送风模式
        _filter = ["fan", "hot"];
      } else {
        // 正常
        // _list = MODE;
      }
    } else {
      _list = SPEED;
    }
    console.log("getActionShowList _filter, _list = ", _filter, _list);
    // 过滤
    _list = _list.filter(item => {
      if (_filter.length > 0) {
        return _filter.indexOf(item.key) >= 0
      } else {
        return true;
      }
    });
    console.log("getActionShowList _list = ", _list, _list[0]);

    // 增加img字段 是否选中 .concat([])
    return _list.map(item => {
      (item.selected = defalutValue == item.value);

      if (this.isUartDemo) {
        item.img = "";
      } else {
        (!item.img && (item.img = "/static/img/common/mode/" + item.key + ".png"));
      }
      
      // 增加一个参数用于actionsheet点击后更新devStatus的主键
      item.sendKey = key;
      return item;
    });
  },

  // 设置数据
  setDataForList (list, devStatus) {
    let _devStatus = devStatus || {};
    // 是否制冷制热模式，如果不是，则隐藏温度
    let _mode = _devStatus['mode'];
    let _tempHide = false; // 默认显示温度
    if (_mode != undefined) {
      _tempHide = _mode != 1 && _mode != 2
    };

    // 如果是空调透传调试
    if (this.isUartDemo) {
      _tempHide = false;
    }

    console.log("setDataForList _devStatus = ", _devStatus);
    return list.map(item => {
      let {key, type} = item;
      // 赋值
      item.value = _devStatus[key];

      // 按照测试要求，不论什么模式都显示温度
      // // 温度的互斥
      // if (key == 'temp') {
      //   item.hidden = _tempHide;
      // }

      // 如果是枚举类型，则取得枚举的text
      if (type == "enum") {
        let {key: modeKey, text} = this.getConfigByValueInArray(key == "mode" ? MODE : SPEED, item.value);
        item.text = text;

        if (this.isUartDemo) {
          item.img = "";
        } else {
          item.img = "/static/img/common/mode/" + modeKey + ".png";
        }
      }
      return item;
    })
  },

  // 从数组中取得value值的text
  getConfigByValueInArray (array, value) {
    // console.log("getConfigByValueInArray array, value = ", array, value);
    if (!array || array.length == 0) {
      return {};
    }
    let _results = array.filter(item => item.value == value);
    if (_results.length == 0) {
      return {};
    }
    return _results[0];
  },

  // 显示操作栏
  actionSheetShow () {
    this.setProperValue("showActionSheet", true);
  },
  actionSheetHide () {
    this.setProperValue("showActionSheet", false);
  },

  // 同步数据到页面
  setProperValue (key, val) {
    if (this.data == undefined || val == undefined) {
      return;
    }
    this.data[key] = val;
    this.setData({
      [key]: val
    });
  },

  closeErrorPop () {
    console.log("closeErrorPop");
    // // 消息为空
    // this.setData({
    //   errorMsg: ""
    // });
    this.setProperValue("errorMsg", "");
  },
  actionSheetClick (e) {
    let {index} = e.detail;
    if (index < 0) {
      return this.actionSheetHide();
    }
    let {key, value, sendKey} = this.data.sheetList[index];
    console.log("actionSheetClick key, value, sendKey = ", key, value, sendKey);

    // 设置选中的数据，并下发
    this.actDev({
      [sendKey]: value
    }, (errCode) => {
      this.dealAfterSend(errCode, {value, sendKey});
    }, {
      sendKey: sendKey,
      key: key
    });
  },
  dealAfterSend (code, {value, sendKey}) {
    if (code < 0 || code == undefined) {
      console.log("dealAfterSend code = ", code);
      ability.showToast({
        title: '下发失败',
      })
      return;
    }
    console.log("dealAfterSend sendKey, value = ", sendKey, value);
    // 更新数据
    this.data.devStatus[sendKey] = value;
    // 更新列表
    this.initLists();
    // 关闭对话框
    this.actionSheetHide();
  },
  // 滑动完成之后的事件
  tempChange (e) {
    let _val = e.detail.value;
    console.log("tempChange _val = ", _val);
    let _key = "temp";
    // 设置选中的数据，并下发
    this.actDev({
      [_key]: _val
    }, (errCode) => {
      console.log("tempChange errCode = ", errCode);
      // 报错的提示
      if (errCode != 0) {
        this.data.tempSlider.value = _val;
        this.setData({
          tempSlider: this.data.tempSlider
        });
        return;
      }
      this.dealAfterSend(errCode, {value: _val, sendKey: _key});
    });
  },
  getItemByE (e) {
    let {index} = e.currentTarget.dataset;
    // console.log("openClick index, this.data.controlList = ", index, this.data.controlList);
    return this.data.controlList[index] || {};
  },
  openClick (e) {
    let {key, values} = this.getItemByE(e);
    console.log("openClick values, key = ", values, key);
    if (key == "pwr") {
      if (!!values) {
        this.sendPwrClick(values[0]);
      } else {
        this.sendPwrClick(1);
      }
    } else {
      this.uartControl(values[0]);
    }
  },
  closeClick (e) {
    let {key, values} = this.getItemByE(e);
    console.log("closeClick values, e = ", values, e);
    if (key == "pwr") {
      if (!!values) {
        this.sendPwrClick(values[1] || 0);
      } else {
        this.sendPwrClick(0);
      }
    } else {
      this.uartControl(values[1]);
    }
  },
  sendPwrClick (pwr) {
    let sendKey = "pwr"
    let value = pwr || 0;
    console.log("sendPwrClick value = ", value);
    this.actDev({
      pwr: value
    }, (errCode) => {
      this.dealAfterSend(errCode, {value, sendKey});
    })
  },

  // 设备快照获取
  batchqueryDevSnap (devInfo) {
    // 当前家庭信息
    let {sid, skey} = app.globalData.familyInfo;
    let _devInfo = devInfo || {};
    console.log("batchqueryDevSnap sid, skey = ", sid, skey);
    if (!sid && !skey) {
      this.setData({
        errorMsg: ''
      });
      return;
    }
    app.globalData.blsdk.batchqueryDevSnap({
      // remoteHost: "chn-ee08f451.ibroadlink.com", // 中国测试集群
      did: sid,
      token: skey,
      devInfo: _devInfo
    }).then(res => {
      console.log("batchqueryDevSnap res = ", res);
      let {lowbattery} = this.dealResponse(res);
      this.setData({
        errorMsg: lowbattery ? '低电量' : ''
      });
    });
  },
  dealResponse (res) {
    let {code, data} = res || {};
    if (code != 0) {
      return {};
    }
    let {stateDna} = data[0] || {};
    let {doorsensor_status, lowbattery, online, tamperevident} = stateDna || {};
    // if (lowbattery) {
    // 是否低电量
    return {
      lowbattery: Boolean(lowbattery) || 0
    }
    // }
  },

  // 详情按钮
  showDetailDialog(value) {
    this.setData({
      showDetail: value || false
    })
  },
  jumpFuncSet () {

    if (this.devTypeStr == "thermostat") {
      // 仅保存设备状态
      this.saveDevStatus(true);
    }

    // 是否支持遥控器
    let _remote = (this.data.abilitys || {}).remote || 0;
    let _itemArray = this.data.detailList.filter(item => item.key == "funcset");
    let _title = _itemArray[0]?.title || "";
    // 跳转详情配置页面
    ability.navigateTo("/device/light/funcSet/index?remote=" + _remote + "&title=" + encodeURIComponent(_title))
  },

  /*** uart aircondition demo test */
  getDemoTestExtendList () {
    return [{
      title: '上下摆风',
      key: "updownswing",
      type: "none",
      values: ["040101", "040100"]
    }, {
      title: '左右摆风',
      key: "leftrightswing",
      type: "none",
      values: ["050101", "050100"]
    }, {
      title: '屏显',
      key: "screen",
      type: "none",
      values: ["060101", "060100"]
    }, {
      title: '睡眠',
      key: "sleep",
      type: "none",
      values: ["070101", "070100"]
    }, {
      title: '电辅热',
      key: "heat",
      type: "none",
      values: ["080101", "080100"]
    }, {
      title: '健康',
      key: "health",
      type: "none",
      values: ["090101", "090100"]
    }/*, {
      title: '童锁',
      key: "child",
      type: "none",
      values: ["0a0101", "0a0100"]
    }, {
      title: 'ECO',
      key: "eco",
      type: "none",
      values: ["0b0101", "0b0100"]
    }, {
      title: '清洁',
      key: "clean",
      type: "none",
      values: ["0c0101", "0c0100"]
    }, {
      title: '防霉',
      key: "proof",
      type: "none",
      values: ["0d0101", "0d0100"]
    }, {
      title: '零风感',
      key: "zerowind",
      type: "none",
      values: ["0e0101", "0e0100"]
    }, {
      title: '母婴风',
      key: "motherwind",
      type: "none",
      values: ["0f0101", "0f0100"]
    }, {
      title: 'AI森吧',
      key: "aisenba",
      type: "none",
      values: ["100101", "100100"]
    }, {
      title: '新风',
      key: "freshair",
      type: "none",
      values: ["110101", "110100"]
    }, {
      title: '净化',
      key: "purify",
      type: "none",
      values: ["120101", "120100"]
    }, {
      title: '除菌',
      key: "degerming",
      type: "none",
      values: ["130101", "130100"]
    }, {
      title: '舒风',
      key: "shufeng",
      type: "none",
      values: ["140101", "140100"]
    }*/];
  },
  isDemoTest () {
    // 如果是透传工具，则进入透传页面
    let {namespace} = app.globalData.companyInfo || {};
    return namespace.indexOf('aux') >= 0;
  },
  uartSendCmd (cmdKey, callback) {
    console.log("uartSendCmd cmdKey = ", cmdKey)
    let _cmdKeyJson = cmdKey || {};
    let _objLength = Object.keys(_cmdKeyJson).length;
    if (_objLength != 1) {
      !!callback && (callback(-1));
      return;
    }
    let _cmdHex = "";
    if (cmdKey['pwr'] >= 0) {
      _cmdHex = cmdKey['pwr'] ? "000101" : "000100"; // 空调开、关
    } else if (cmdKey['mode'] >= 0) {
      // 自动：020100 0
      // 制冷：020101 2
      // 制热：020102 1
      // 送风：020103 4
      // 除湿：020104 8
      let _modeValue = cmdKey['mode'];
      if (_modeValue == 0) {
        _cmdHex = "020100";
      } else if (_modeValue == 2) {
        _cmdHex = "020101";
      } else if (_modeValue == 1) {
        _cmdHex = "020102";
      } else if (_modeValue == 4) {
        _cmdHex = "020103";
      } else if (_modeValue == 8) {
        _cmdHex = "020104";
      }
    } else if (cmdKey['temp'] >= 0) {
      let _temp = cmdKey['temp'] || 16;
      _cmdHex = "0101" + _temp.toString(16).padStart(2, 0)
    } else if (cmdKey['speed'] >= 0) {
      // 自动：020100 0
      // 制冷：020101 2
      // 制热：020102 1
      // 送风：020103 4
      // 除湿：020104 8
      let _speed = cmdKey['speed'] || 0;
      if (_speed == 0) {
        _cmdHex = "030100";
      } else if (_speed == 1) {
        _cmdHex = "030102";
      } else if (_speed == 2) {
        _cmdHex = "030104";
      } else if (_speed == 3) {
        _cmdHex = "030106";
      } else if (_speed == 4) {
        _cmdHex = "030101";
      } else if (_speed == 5) {
        _cmdHex = "030103";
      } else if (_speed == 6) {
        _cmdHex = "030105";
      } else if (_speed == 7) {
        _cmdHex = "030107";
      }
    }
    this.uartControl(_cmdHex, callback);
  },
  uartControl (hexCmd, callback) {
    console.log("uartControl hexCmd = ", hexCmd);
    let _devInfo = app.globalData.devInfo || {};
    let _hexCmdArray = Utils.hexStrToArray(hexCmd || "");

    // aux空调tlv头前面加
    _hexCmdArray = [0x01].concat(_hexCmdArray);

    console.log("uartControl _hexCmdArray, _devInfo = ", _hexCmdArray, _devInfo);
    ability.showLoading({});
    // 有返回的控制命令
    this.sendUartByCount({
      a: _devInfo?.a,
      hexCmdArray: _hexCmdArray,
      magicArray: app.globalData?.companyInfo?.magicArray,
      count: 3
    }, (res) => {
      ability.hideLoading();
      !!callback && (callback(res || 0));
    })
    // app.globalData.blsdk.uartCtrlNoRes(_devInfo?.a, _hexCmdArray, {
    //   magicArray: app.globalData?.companyInfo?.magicArray
    // });
  },
  resetSendConfig () {
    this.uartCount = 0;
    bizUtil.clearTimer(this.intervalSendTimer, "interval");
  },
  sendUartByCount (params, callback) {
    let {a, hexCmdArray, magicArray, count} = params || {};
    let _count = Number(count) || 1;
    let _sendMethod = (callback) => {
      // 有返回的控制命令
      app.globalData.blsdk.uartCtrlNoRes(a, hexCmdArray, {
        magicArray: magicArray,
        callback: callback
      });
    };
    if (_count <= 1) {
      _sendMethod(callback);
      return;
    }
    // 发送次数大于1
    this.resetSendConfig();
    this.intervalSendTimer = setInterval(() => {
      if (this.uartCount >= _count) {
        this.resetSendConfig();
        !!callback && (callback(0));
        return;
      } else {
        this.uartCount++;
        _sendMethod();
      }
    }, 350);
    
  },
  getDemoDefaultStatus () {
    return {
      pwr: 1,
      temp: 24,
      unit: "℃",
      mode: 1, // 制热
      speed: 1,  // 低速
      updownswing: 0, // 上下摆风
      leftrightswing: 1, // 左右摆风
      screen: 1, // 屏显
      sleep: 1, // 睡眠
      heat: 1, // 电辅热
      health: 1, // 健康
      // child: 1, // 童锁
      // eco: 1, // ECO
      // clean: 1, // 清洁
      // proof: 1, // 防霉
      // zerowind: 1, // 零风感
      // motherwind: 1, // 母婴风
      // aisenba: 1, // AI森吧
      // freshair: 1, // 新风
      // purify: 1, // 净化
      // degerming: 1, // 除菌
      // shufeng: 1, // 舒风
    };
  },
  /*** end ***/
})
