// groups/manage/index.js
import bizUtil from '../../../utils/bizUtil'
import ability from "../../../utils/classDef/ability"

const app = getApp();
var deviceBehavior = require("../../../static/js/behavior/device")
Page({

  behaviors: [deviceBehavior],

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: ""
      }
    },

    // 编辑模式
    isEdit: false,
    // 空调内机列表
    devList: [],
    bleUnavailable: false,
    roleNum: 0, // 用户权限
    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined, // 家庭下所有设备信息

    roomJson: {}, // 房间key
    isFloor: false, // 是否多楼层
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 初始化家庭和家庭下的设备数据
    this.initAllDataByFamily(options.familyId);

    // 取得当前家庭的数据
    let {roleNum, floor} = this.data.familyInfo || {};

    // 所有房间数据
    let {roomJson} = bizUtil.getRoomsByDevs(this.data.allDevs);
    this.data.roomJson = roomJson;
    this.data.isFloor = floor >= 0;

    // 权限
    this.data.roleNum = roleNum == undefined ? app.globalData.roleNum : roleNum;
    // 传递的房间数据
    this.setData({
      roleNum: this.data.roleNum,
      roomJson: this.data.roomJson,
    });
  },
  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },
  initTitle (title) {
    let _title = title || "";
    ability.setNavigationBarTitle(_title);
  },
  setProperValue (key, val) {
    if (!key) {
      return;
    }
    this.data[key] = val || '';
    this.setData({
      [key]: val || ""
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 为了保持空调网关进入和设备进入
    app.globalData.devInfo = app.globalData.groupInfo;
    // 更新标题
    this.initTitle(app.globalData.groupInfo?.n || "");
    // 更新设备信息
    this.initSubDevs();
  },

  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    })
  },
  hideLoading () {
    ability.hideLoading();
  },

  // 初始化房间信息
  initSubDevs () {
    // console.log("initSubDevs this.data.allDevs = ", this.data.allDevs);
    // 取得房间下所有的d对应的详细信息
    let {devs} = bizUtil.getAllDevsBySort(this.data.allDevs, this.data.allDevs);
    let {aircondition} = bizUtil.getAllPidByArray(devs);
    let {subDMap} = bizUtil.getAirconditionMap(aircondition);
    // 网关d
    let _d = app.globalData.groupInfo?.d;
    // 内机列表
    let _array = bizUtil.sortArrayByKey(subDMap[_d] || [], 'a');
    console.log("initSubDevs _array, subDMap, _d = ", _array, subDMap, _d);
    this.setData({
      devList: _array
    })
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
    console.log("roomDetail onHide");
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {
  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {
  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {
  },

  longItemClick (e) {
    console.log("longItemClick this.data.roleNum = ", this.data.roleNum)
    // 如果是分享权限，则无法长按删除
    if (this.data.roleNum == 2) {
      return;
    }

    let {index} = e.currentTarget.dataset;
    let {n, d} = this.data.devList[index] || {};
    // 提示
    wx.showModal({
      title: "",
      content: "是否确认删除" + (n || "") + "？",
      showCancel: true,
      confirmText: "是",
      cancelText: "否",
      cancelColor: '#000000'
    }).then(result => {
      if (!(result?.confirm)) {
        return;
      }
      // 删除方法
      return this.delDevInGroupByIndex(index);
    })
  },

  // 删除空调内机
  delDevInGroupByIndex (index) {
    let _list = this.data.devList || [];
    let _dataLen = _list.length || 0;
    if (index >= _dataLen || index < 0) {
      console.error("delDevInGroupByIndex index, _dataLen = ", index, _dataLen);
      return;
    }

    // 删除设备
    this.delDeviceByInfo(_list[index], () => {
      this.showDelSuccessToast();
    });
  },
  // 删除成功的提示
  showDelSuccessToast () {
    // 提示删除结果
    ability.showToast({
      title: "删除成功",
      icon: "success",
      mask: true,
      complete: () => {
        // 返回首页，重新加载
        setTimeout(() => {
          // 重新加载页面
          this.initSubDevs();
        }, 1200);
      }
    })
  },

  // 跳转详情
  jumpDetail () {
    // 空调网关
    app.globalData.devInfo = app.globalData.groupInfo;
    ability.navigateTo("/device/light/devProperty/index?remote=0")
  },

  getGroupRoomId () {
    return app.globalData.groupInfo?.rid || 0;
  },

  // 编辑按钮点击
  manageDevs () {
    // 群组房间
    let _roomId = this.getGroupRoomId();
    // // 房间名称
    // let _rn = this.data.allDevs[_roomId]?.n || '';
    // console.log("manageDevs _rn = ", _rn);
    // 来源页面
    let _origin = "groupdel";

    // 跳转
    this.jumpGroupManage({
      from: _origin,
      title: "编辑群组设备",
      rid: _roomId,
      familyId: this.data.familyId
    })
  },
  jsonToUrlParamsStr (json) {
    let _result = [];
    if (!json) {
      return "";
    }
    let _keyArray = Object.keys(json);
    let _valueArray = Object.values(json);
    for (let index in _keyArray) {
      let _key = _keyArray[index];
      let _value = _valueArray[index];
      if (_key == undefined || _value == undefined) {
        continue;
      }
      _result.push(_key + "=" + encodeURIComponent(_value));
    }
    return "?" + _result.join("&");
  },
  jumpGroupManage (params) {
    ability.navigateTo("/pages/batch/index" + this.jsonToUrlParamsStr(params));
  },
  itemClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.devList[index];
    app.globalData.devInfo = _item;
    ability.navigateTo("/device/aircondition/control/index");
  }
})