// device/coathanger/control/index.js
const bizUtil = require("../../../utils/bizUtil");
const ability = require("../../../utils/classDef/ability");
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    lineHeight: 0, // 晾衣架线长 默认为0，最大为452
    transitionAct: "", // 动画效果

    midList: [], // 中部按钮列表
    bottomList: [] // 底部按钮列表
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
  },
  setNavTitle (name) {
    ability.setNavigationBarTitle(name || "")
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {
    this.queryObj = this.createQueryObj();
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 设备信息
    let _devInfo = app.globalData.devInfo || {};
    // 设置标题
    this.setNavTitle(_devInfo?.n);
    
    let _midList = this.initMidList();
    let _bottomList = this.initBottomList();
    this.setData({
      midList: _midList,
      bottomList: _bottomList
    })
  },
  initMidList () {
    return [{
      imgStr: "../../img/coathanger/lamp_1.png",
      text: "照明",
      key: "lamp",
      list: [{
        text: "开",
        value: 1
      }, {
        text: "关",
        value: 0
      }]
    }, {
      imgStr: "../../img/coathanger/fan_1.png",
      text: "风干",
      key: "fan",
      list: [{
        text: "开",
        value: 1
      }, {
        text: "关",
        value: 0
      }]
    }, {
      imgStr: "../../img/coathanger/dry_1.png",
      text: "烘干",
      key: "dry",
      list: [{
        text: "开",
        value: 1
      }, {
        text: "关",
        value: 0
      }]
    }, {
      imgStr: "../../img/coathanger/disinfect_1.png",
      text: "消毒",
      key: "disinfect",
      list: [{
        text: "开",
        value: 1
      }, {
        text: "关",
        value: 0
      }]
    }]
  },
  initBottomList () {
    return [{
      key: "down",
      index: 0,
      value: 1,
      text: "下降"
    }, {
      key: "pause",
      index: 0,
      value: 2,
      text: "停止"
    }, {
      key: "up",
      index: 0,
      value: 0,
      text: "上升"
    }]
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 关闭蓝牙监听器
    !!app.globalData.blsdk && (app.globalData.blsdk.destory());
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  itemClick (e) {
    let {index, idx} = e.currentTarget.dataset;
    let _item = this.data.midList[index] || {};
    let _child = (_item.list || [])[idx] || {};
    // 发送命令
    this.actDev({
      [_item.key]: _child.value
    })
  },

  bottomClick (e) {
    let {index} = e.currentTarget.dataset;
    let {key, value} = this.data.bottomList[index];
    if (key == "up") {
      this.setLineWidth(0);
    } else if (key == "pause") {
      this.setLineWidth(50);
    } else if (key == "down") {
      this.setLineWidth(100);
    }
    // 底部修改需要切换一下图标在切换回来
    this.updateBottomByIndex(index, 1);
    setTimeout(() => {
      this.updateBottomByIndex(index, 0);
    }, 300);

    this.actDev({
      updown: value
    });
  },
  updateBottomByIndex (index, value) {
    this.setData({
      [`bottomList[${index}].index`]: value || 0
    });
  },
  setLineWidth (height) {
    let _intVal = Number(height) || 0;
    if (_intVal == 0 || _intVal == 100) {
      this.setData({
        lineHeight: _intVal + "%",
        transitionAct: "transitionAct"
      })
    } else {
      this.queryMultipleNodes("coathangerId", (res) => {
        this.setData({
          lineHeight: (res?.height || 0) + "px",
          transitionAct: ""
        })
      });
    }
  },
  createQueryObj () {
    return wx.createSelectorQuery().in(this)
  },
  // 动态获取当前高度
  queryMultipleNodes (id, callback) {
    const query = this.queryObj || this.createQueryObj();
    query.select('#' + id).boundingClientRect((res) => {
      console.log("queryMultipleNodes res = ", res);
      !!callback && (callback(res));
    }).exec();
  },
  actDev (cmdJson) {
    console.log("actDev cmdJson = ", cmdJson);
    let _devInfo = app.globalData.devInfo;
    app.globalData.blsdk.controlWithDevice(_devInfo, cmdJson, {
      noStop: false, // 晾衣架需要主动停止发包
      timestamp: 1000
    });
  },

  // 跳转设备详情页面
  jumpDetail () {
    // 是否支持遥控器
    let _remote = (this.data.abilitys || {}).remote || 0;

    // // 如果是群组控制，则进入群组管理页面
    // if (this.isGroupCtrl()) {
    //   console.log("groupCtrl app.globalData.groupInfo = ", app.globalData.groupInfo);
    //   // 跳转页面
    //   return ability.navigateTo("/groups/manage/index?remote=" + _remote);
    // }
    console.log("jumpDetail _remote = ", _remote);
    // 跳转页面
    ability.navigateTo("/device/light/devProperty/index?remote=" + _remote);
  },
})