// device/components/calendar/index.js
Component({
  options: {
    addGlobalClass: true, // 全局样式
  },
  /**
   * 组件的属性列表
   */
  properties: {
    config: {
      type: Object,
      value: {
        showBottom: false,
        datas: []
      }
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    weeks: ["日", "一", "二", "三", "四", "五", "六"], // 星期
    days: [], // 当月的列表
    currentDate: [], // 年月日
    afterDayEnable: false, // 后一天默认可以点击
    afterMonthEnable: false, // 下一个月是否可以点击
    showBottom: false, // 是否展示底部按钮
  },

  lifetimes: {
    created () {
    }
  },
  pageLifetimes: {
    show () {
    }
  },

  observers: {
    'config.datas': function (dates) {
      if (!!dates && dates.length > 0) {
        console.log("dates = ", dates)
        this.data.currentDate = dates;
        // 初始化数据
        this.initDateByDateArray(dates);
      }
    }
  },

  /**
   * 组件的方法列表
   */
  methods: {
    // 初始化
    initDateByDateArray (dataArray) {
      // 取得日期
      let _date = this.getDate(dataArray);
      // 取得年月日时分秒的数组和天数和第一天的星期
      let {array, count, firstDay} = this.getCurrentDate(_date);
      // 当前日期
      let {array: currentArray} = this.getCurrentDate();
      // 比较数组的月大小
      let _monthEnable = this.compareArray(array, currentArray, 2);
      // console.log("initDateByDateArray _monthEnable, currentArray = ", _monthEnable, currentArray);
      // 如果是动态时间，则判断是否大于当前时间
      if (!!dataArray) {
        // 如果选择的日期大于等于当前月份，则下一个月不可点击
        this.setProperValue("afterMonthEnable", _monthEnable === false);
        // 如果选择的日期大于等于当前日，则下一天不可点击
        let _dayEnable = this.compareArray(array, currentArray, 3);
        // console.log("initDateByDateArray _dayEnable = ", _dayEnable);
        this.setProperValue("afterDayEnable", _dayEnable === false);
        // 如果选择的日期大于当前日期，则不初始化数据
        if (_dayEnable == true) {
          return;
        }
      }

      // 保存
      this.data.currentDate = array;
      // console.log("initDateByDateArray array, count, firstDay = ", array, count, firstDay);
      // 取得该月一共有几天
      let _dayArray = this.getDayArrayByCount(count);
      // 判断是否大于当前日期，如果是，则把超出的设置为false
      // if (_monthEnable === 0 || _monthEnable === true) {
        let _needSetDay = (_monthEnable !== true) ? array[2] : 0;
        _dayArray.map(item => {
          item.disable = _monthEnable === false ? false : (item?.value > currentArray[2]); // 是否可选
          if (item.disable) {
            item.selected = false;
          } else {
            item.selected = item.value == _needSetDay;
          }
          return item;
        })
      // } else {
      //   let _needSetDay = array[2];
      //   _dayArray.map(item => {
      //     item.disable = false; // 是否可选
      //     item.selected = item.value == _needSetDay;
      //     return item;
      //   })
      // }
      // 把数组的天数往前增加第一天星期的天数
      this.data.days = this.newArrayByCount(firstDay).concat(_dayArray);
      // console.log("initDateByDateArray this.data.days = ", this.data.days);
      console.log();
      // 设置数据
      this.setData({
        days: this.data.days,
        currentDate: this.data.currentDate
      })
    },
    // 比较两个数组的长度值是否相等 0：相等  true: array1大  false: array2大
    compareArray (array1, array2, length) {
      if (!array1 || !array2) {
        return 0;
      }
      let _len = length || array1.length;
      let i = 0;
      do {
        let _v1 = array1[i];
        let _v2 = array2[i];
        if (_v1 != _v2) {
          return _v1 > _v2;
        }
        i ++;
      } while(length > i);
      return 0;
    },
    getDate (dataArray) {
      if (!dataArray?.length) {
        return new Date();
      }
      let _year = dataArray[0] || 0;
      let _month = (dataArray[1] - 1) || 0; // 这里月份显示3月时，实际是3月，但是换算日期是2月
      let _day = dataArray[2] || 0;
      let _hour = dataArray[3] || 0;
      let _minite = dataArray[4] || 0;
      let _second = dataArray[5] || 0;
      return new Date(_year, _month, _day, _hour, _minite, _second);
    },
    // 取得当前的年月
    getCurrentDate (date) {
      let _date = date || (new Date());

      // // 这里时分秒都设置为0
      // _date.setHours(0);
      // _date.setMinutes(0);
      // _date.setSeconds(0);

      let year = _date.getFullYear();
      let month = _date.getMonth() + 1;
      let day = _date.getDate();
      // let hour = _date.getHours();
      // let minute = _date.getMinutes();
      // let second = _date.getSeconds();
      return {
        array: [year, month, day], // , hour, minute, second
        count: this.getDateCountByYearAndMonth(year, month),
        firstDay: this.getWeekByDate(year, month - 1),
        // timestampYMD: parseInt(Date.parse(_date) / 1000), // 单位s
        // timestampYM: parseInt(Date.parse(_date) / 1000) // 单位s
      };
    },
    // 根据月份天数取得月份列表
    initDays (count) {
      this.data.days = this.getDayArrayByCount(count);
      this.setData({
        days: this.data.days
      }) 
    },
    // 根据年月取得月份一共有多少天
    getDateCountByYearAndMonth (year, month) {
      return new Date(year, month, 0).getDate();
    },
    newArrayByCount (count) {
      let _array = new Array();
      for (let i = 0; i < count; i++) {
        _array.push({});
      }
      return _array;
      // return new Array(count).fill({});
    },
    getDayArrayByCount (count) {
      let _array = this.newArrayByCount(count);
      return _array.map((item, index) => {
        item.value = (Number(index) + 1);
        return item;
      });
    },
    // 取得当前是周几
    getWeekByDate (year, month) {
      return new Date(year, month, 0).getDay();
      // return (date || new Date()).getDay();
    },
    beforeDay () {
      // 上一天
      this.dealDateArray({
        day: -1
      });
    },
    afterDay () {
      // 如果不可点击，则返回
      if (!this.data.afterDayEnable) {
        return;
      }
      // 下一天
      this.dealDateArray({
        day: 1
      });
    },
    beforeMonth () {
      // 上一个月
      this.dealDateArray({
        month: -1,
        dayValue: 1
      });
    },
    afterMonth () {
      // 如果不可点击，则返回
      if (!this.data.afterMonthEnable) {
        return;
      }
      // 下一个月
      this.dealDateArray({
        month: 1,
        dayValue: 1
      });
    },
    dealDateArray (params) {
      let _date = this.data.currentDate;
      if (_date.length < 3) {
        return;
      }
      // console.log("dealDateArray params = ", params);
      let {month, day, year, dayValue, monthValue, yearValue} = params || {};
      if (yearValue) {
        _date[0] = Number(yearValue);
      } else {
        _date[0] += (year || 0);
      }
      if (monthValue) {
        _date[1] = Number(monthValue);
      } else {
        _date[1] += (month || 0);
      }
      // 天数可以前后，也可以选中具体某一天
      if (dayValue) {
        _date[2] = Number(dayValue);
      } else {
        _date[2] += (day || 0);
      }
      // console.log("dealDateArray params, _date = ", params, _date);

      // 更新日期
      this.initDateByDateArray(_date);
      // 上报数据
      this.upEvent({
        showContent: true,
        values: this.data.currentDate
      });
    },
    showBottom () {
      let _isShow = this.data.showBottom;
      this.setProperValue("showBottom", !_isShow);
      this.upEvent({
        showContent: _isShow
      });
    },
    dayClick (e) {
      let {item} = e.currentTarget.dataset;
      if (!item) {
        return;
      }
      console.log("dayClick item = ", item);
      // 选中某一天
      this.dealDateArray({
        dayValue: Number(item.value),
        monthValue: Number(this.data.currentDate[1]),
        yearValue: Number(this.data.currentDate[0])
      });
      // 隐藏bottom
      this.hideBottom();
      // 上报数据
      this.upEvent({
        showContent: true,
        values: this.data.currentDate
      });
    },
    hideBottom () {
      this.setProperValue("showBottom", false);
    },
    // 同步数据到页面
    setProperValue (key, val) {
      if (this.data == undefined || val == undefined) {
        return;
      }
      // this.data[key] = val;
      this.setData({
        [key]: val
      });
    },
    upEvent (params) {
      console.log("upEvent params = ", params);
      this.triggerEvent('showevent', params || {})
    }
  }
})
