// device/fan/control/index.js
import bizUtil from '../../../utils/bizUtil'
import ability from '../../../utils/classDef/ability'
var app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    // 标题
    navObj: {
      left: {
        img: "/static/img/back_white.png",
        type: "img"
      },
      center: {
        style: 'white',
        text: ""
      },
    },
    // 延时时间
    delayInfo: {
      h: 0,
      m: 0
    },
    // 当前设备状态
    devStatus: {
      timestamp: 0,
      fan: {
        pwr: 0,
        speed: 0,
      },
      light: {
        pwr: 0,
        brightness: 0,
      }
    },
    currentIndex: 0, // 默认是风扇调节
    showTimerDialog: true, // 是否显示定时的对话框.默认隐藏
    // 亮度配置
    brightSliders: {
      leftImg: "/static/img/common/bright_down.png",
      rightImg: "/static/img/common/bright_up.png",
      leftClass: "bDown",
      rightClass: "bUp",
      width: 516, // rpx
      percent: 100, // 百分比
      labelTxt: "",
      minVal: 2,
      maxVal: 127,
      minPercent: 1, // 最小百分比
      gradient: false // 默认不支持滑动时调节
    },
    // 风速配置
    fanSliders: {
      leftImg: "/device/img/fan/min.png",
      rightImg: "/device/img/fan/max.png",
      leftClass: "fanSpeedImg",
      rightClass: "fanSpeedImg",
      width: 516, // rpx
      percent: 100, // 百分比
      labelTxt: "",
      minVal: 26,
      maxVal: 127,
      minPercent: 20, // 最小百分比
      gradient: false, // 默认不支持滑动时调节
      // points: [1, 2, 3, 4]
    },
    tabs: [],
    delayTimerArray: [], // 定时支持的时间

    timer: null, // 定时器用于定时的倒计时
    pageSet: {
      pageX: 0,
      pageY: 0,
      // height: 360, // rpx 圆环的高度
      // width: 465, // rpx 圆环的长度
      centerX: (app.globalData.customPageSize.width - 60 / app.globalData.customPageSize.multip) / 2, // 中心点px
      centerY: 465 / 2 / app.globalData.customPageSize.multip,
      r: (465 - 25) / 2 / app.globalData.customPageSize.multip,
      multip: app.globalData.customPageSize.multip,
      minAngle: 215,
      maxAngle: 325,
      defaultAngle: 215
    },
    thumb: {
      x: 0,
      y: 0
    },

    brightness: 100, // 亮度
    speed: 0, // 档位置
    // directionNum: 1, // 0是左，1是右
    actionStr: 'rotateRight1', // 转动样式 rotateLeft rotateRight
    isAuto: false, // 是否自然风
    // lightSel: 

    isFan: false, // 是否风扇 默认风扇灯
    fanConfigList: [], // 底部显示的条目配置

    showDetail: false, // 是否展示详情列表
    detailList: [], // 详情列表
    topHeight: 0
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 设备信息
    let _devInfo = app.globalData.devInfo || {};
    // 判断进入页面的类型 默认是风扇灯（0） 风扇（1）
    this.data.isFan = this.checkFanTypeByDevInfo(_devInfo);
    // 初始化定时时间
    let _delayTimerArray = this.getDelays();
    // 初始化功能
    this.initDevFunctions(_devInfo);
    // 初始化tabs数据
    let _tabs = this.getTabsByDevInfo(this.data.isFan);
    // 取得设置的数据
    let _list = this.getListByInfo(this.data.isFan);

    // 取得详细
    let _detailList = this.getDetailListByV();
    // 由于是自定义的头，所以需要计算
    let {multip, statusHeight, navHeight} = app.globalData.customPageSize || {};
    let _topHeight = ((statusHeight * multip) || 0) + 100 + navHeight;

    this.setData({
      isFan: this.data.isFan,
      delayTimerArray: _delayTimerArray,
      tabs: _tabs,
      fanConfigList: _list,
      detailList: _detailList,
      topHeight: _topHeight
    })
  },

  getDetailListByV (backlight, devTypeStr) {
    let _list = this.getAllDetailList();
    if (!this.data.isFan) {
      _list = _list.slice(0, 1);
    }
    return _list;
  },
  getAllDetailList () {
    return [{
      title: "设备详情",
      bindTap: "jumpDetail",
      key: "detail"
    }, {
      title: "功能设置",
      bindTap: "jumpFuncSet",
      key: "funcset"
    }]
  },

  getTabsByDevInfo (isFan) {
    if (isFan) {
      return [{
        text: '风扇'
      }]
    } else {
      return [{
        text: '风扇'
      }, {
        text: '灯光'
      }]
    };
  },
  // 取得控制列表
  getListByInfo (isFan) {
    // 通用定义
    let _directionObj = {
      text: '风扇',
      key: "direction",
      type: "imgBtn",
      // value: -1, // 默认未选中
      list: [{
        imgStr: "../../img/fan/right.png",
        key: "forward",
        imgClass: 'fanCommonImg',
        text: "正转",
        value: 1,
        isSelected: false
      }, {
        imgStr: "../../img/fan/left.png",
        imgClass: 'fanCommonImg',
        key: "opposite",
        text: "反转",
        value: 2,
        isSelected: false
      }]
    };
    // 返回的数据
    let _result = [{
      text: '风扇',
      key: "pwr",
      type: "switch",
      list: [{
        text: "开",
        value: 1
      }, {
        text: "关",
        value: 0
      }]
    }];
    if (isFan) {
      _directionObj.list[0].value = 3;
      _directionObj.list[1].value = 4;
      _result.push({
        text: '空气净化',
        key: "air",
        type: "switch",
        list: [{
          text: "开",
          value: 1
        }, {
          text: "关",
          value: 0
        }]
      }, _directionObj);
    } else {
      // 增加自然风
      _directionObj.list.push({
        imgStr: "../../img/fan/auto.png",
        imgClass: 'fanCommonImg',
        text: "自然风",
        key: "auto",
        value: 3,
        isSelected: false
      });
      _result.push(_directionObj)
    };
    _result.push({
      text: '风速',
      key: "speedTitle",
      type: "label",
      value: "" // {{(speed < 0)?'关':(speed + '档')}}
    });
    _result.push({
      type: "slider",
      key: "speed",
    });
    return _result;
  },

  checkFanTypeByDevInfo (devInfo) {
    let _devTypeStr = bizUtil.getDevTypeStr(devInfo?.t);
    if (_devTypeStr == "fan") {
      return 1;
    }
    return 0;
  },

  // 根据点击的按钮设置滑块的配置
  initSliderConfigByType (val) {
    if (this.data.isFan) {
      return {
        minVal: 1,
        maxVal: 127
      };
    }
    // 设置风速的滑块
    let _allPointsJson = this.getPointsConfig();
    return _allPointsJson[val || 0] || _allPointsJson[0];
  },
  // 初始化正反转
  initActionRoll (val, speedLevel) {
    // 取得前面的标识
    let _str = '';
    if (val == 1) {
      _str = 'rotateLeft';
    } else if (val == 0 || val == undefined) {
      _str = 'rotateRight';
    }
    // 取得风速百分比
    let _speed = speedLevel || this.initSpeed(this.data.fanSliders.percent) || 1;
    _str += _speed;
    console.log("initActionRoll _str = ", _str);
    // 设置数据
    this.data.actionStr = _str;
    let _directionNum = _str.indexOf("Left") >= 0;
    let _directionObj = !_directionNum ? {
      forward: 1, opposite: 0 // 正转
    } : {
      forward: 0, opposite: 1 // 反转
    }
    // 设置正反转
    this.setChildListByKey(this.data.fanConfigList, "direction", _directionObj);
    this.setData({
      actionStr: this.data.actionStr,
      fanConfigList: this.data.fanConfigList
      // directionNum: this.data.directionNum
    })
  },
  getItemByKeyInList (configList, key) {
    let _configList = configList || [];
    let _filter = _configList.filter(item => item.key == key);
    if (_filter.length == 0) {
      return {};
    }
    return _filter[0];
  },
  setChildListByKey (configList, key, config) {
    if (!config) {
      return;
    }
    let _item = this.getItemByKeyInList(configList, key);
    if (!_item.list) {
      return;
    }
    // let {forward, opposite, auto} = config;
    for (let item of _item.list) {
      if (config[item.key] == undefined) {
        continue;
      }
      item.isSelected = config[item.key] || false;
    }
  },
  setConfigListByKey (configList, key, value) {
    let _item = this.getItemByKeyInList(configList, key);
    _item.value = value || 0;
  },

  // 初始化正反转
  getPointsConfig () {
    return {
      1: {
        points: [1, 2],
        minVal: 26,
        maxVal: 76
      },
      0: {
        points: [1, 2, 3, 4],
        minVal: 26,
        maxVal: 127
      }
    }
  },
  // 页面默认的状态
  getDefaultStatus (isFan) {
    if (isFan) {
      return {
        pwr: 0, 
        speed: 127, 
        mode: 0, 
        direction: 0, 
        buzzer: 0, 
        wheel: 0, 
        air: 0,
        timestamp: 0
      }
    } else {
      return {
        timestamp: 0,
        fan: {
          pwr: 0,
          speed: 0,
          // 默认正转 0
        },
        light: {
          pwr: 0,
          brightness: 127,
          oriBrightness: 127, // 用于关闭的时候保存原来的亮度值
          c: 127,
          w: 127
        }
      };
    }
  },
  initDevFunctions (devInfo) {
    // 是否风扇
    let _isFan = this.data.isFan;
    // 初始化设备通用功能
    let _devStatus = this.getDefaultStatus(_isFan);
    // 风扇灯目前只支持单个控制
    if (!devInfo) {
      return;
    }

    // 判断设备单个还是批量或者群组
    let _funcInfo = this.getFuncList(!!devInfo ? [devInfo] : undefined);
    console.log("_funcInfo = ", _funcInfo);
    let {maxStatus, versions} = (_funcInfo || {});
    // 赋值给devStatus
    Object.assign(_devStatus, maxStatus);
    console.log("last devStatus, maxStatus = ", _devStatus, maxStatus);
    // 取得上次保存的值
    let _valStatus = this.getDevStatus(true);
    console.log("initDevFunctions _valStatus = ", _valStatus);
    for (let key in _devStatus) {
      let _val = _valStatus[key];
      (_val != undefined) && (_devStatus[key] = _val);
    }
    // 保存设备状态
    app.globalData.devStatus = this.data.devStatus = _devStatus;

    // 判断是否风扇
    if (!_isFan) {
      // 亮度
      let _ob = _valStatus?.light?.originBPercent;
      let _b = _valStatus?.light?.brightness;
      if (_ob != undefined) {
        this.data.brightSliders.percent = _ob;
        this.initBrightness(_ob);
      } else if (_b != undefined) {
        let _p = this.getPercentByRealValue(_b, this.data.brightSliders);
        this.data.brightSliders.percent = _p;
        this.initBrightness(_p);
      }
      // 色温
      let _c = _valStatus?.light?.c;
      (_c != undefined) && (this.data.pageSet.defaultAngle = this.getAngleByValue(_c));
      // 设置风扇的风速
      let _s = _valStatus?.fan?.speed;
      // 风速
      if (_s != undefined) {
        let _percent = this.getPercentByVal(_s, this.data.fanSliders);
        console.log("initDevFunctions _percent, _s = ", _percent, _s);
        // 初始化挡位
        this.initSpeed(_percent);
        // 根据挡位取得百分比
        this.data.fanSliders.percent = this.dealByPoint(_percent / 100) * 100; // _percent;
      } else {
        // 初始化挡位
        this.initSpeed(this.data.fanSliders.percent);
      }
      // 是正转还是反转
      let _forward = _valStatus?.fan?.forward;
      // 根据正反转设置滑块的配置
      let _json = this.initSliderConfigByType(_forward);
      console.log("fanDirection fanSliders _json = ", _json);
      Object.assign(this.data.fanSliders, _json);
      // 初始化正反转
      this.initActionRoll(_forward, this.data.speed);
      // 设置自然风
      this.initAuto(_valStatus?.fan?.unnatrWind);
      // 保存能力集合
      this.data.abilitys = bizUtil.getMaxAbilityByVs(versions) || {};
      console.log("this.data.abilitys = ", this.data.abilitys);
      // 能力集合
      let {gradient, minBrightness} = this.data.abilitys;
      // 是否支持渐变
      // let _gradient = this.data.abilitys.gradient;
      // 更新滑块状态
      this.data.brightSliders.gradient = gradient;
      // this.data.cwSliders.gradient = _gradient;
      console.error("minBrightness = ", minBrightness, this.data.abilitys, this.data.brightSliders);
      // 更新亮度
      !!minBrightness && (this.data.brightSliders.minVal = minBrightness.value, this.data.brightSliders.minPercent = minBrightness.percent);
    } else {
      this.data.fanSliders.minVal = 1; // 设置最小值
      this.data.fanSliders.percent = bizUtil.getPercentByVal(_valStatus?.speed || 0, this.data.fanSliders);

      // 初始化正反转
      let _directionObj = _devStatus.direction == "forward" ? {
        forward: 1, opposite: 0 // 正转
      } : {
        forward: 0, opposite: 1 // 反转
      }
      // 设置正反转
      this.setChildListByKey(this.data.fanConfigList, "direction", _directionObj);
    }
    
    // console.log("initDevFunctions _s = ", _s, this.data.speed, this.data.fanSliders.percent);

    this.setData({
      devStatus: this.data.devStatus,
      brightSliders: this.data.brightSliders,
      pageSet: this.data.pageSet,
      fanSliders: this.data.fanSliders
    })
  },
  getPercentByVal (value, config) {
    let {minVal, maxVal} = config || {};
    let _minVal = minVal || 0;
    let _maxVal = maxVal || 100;
    if (value >= _maxVal) {
      return 100;
    } else if (value <= _minVal) {
      return 0;
    }
    return Math.round((value - _minVal) / (_maxVal - _minVal) * 100);
  },
  // 根据point处理百分比
  dealByPoint (percent) {
    // 数据处理
    let _points = this.data.fanSliders.points;
    if (!_points) {
      return percent;
    }
    let _per = 1 / _points.length;
    // 支持节点上报
    let _isHalf = (percent % _per) >= (_per / 2);
    let _realP = parseInt(percent / _per) * _per + (_isHalf ? _per : 0);
    console.log("dealTouchEnd _realP = ", percent, _per, _realP, _isHalf);
    return _realP;
  },
  // 取得滑块值的百分比
  getPercentByRealValue (realValue, obj) {
    console.log("getPercentByRealValue realValue, obj = ", realValue);
    if (!obj) {
      return 0;
    }
    let {minVal, maxVal} = obj;
    let _minVal = minVal || 0;
    let _maxVal = maxVal || 100;
    if (realValue < _minVal) {
      return _minVal;
    } else if (realValue > _maxVal) {
      return _maxVal;
    }
    // 这里 (73.25 - 2) / (127 - 2) * 100 = 56.999999999 计算器算出来是57
    let _percent = (realValue - _minVal) / (_maxVal - _minVal);
    let _pointInt = _percent * 100 - parseInt(_percent * 100);
    let _result = parseInt(_percent * 100) + ((_pointInt > 0.99) ? 1 : 0);
    console.log("_pointInt, _percent, _result = ", _pointInt, _percent, _result);
    return _result;
  },
  // 根据品类取得最小功能集合
  getFuncList (devs) {
    // 取得全屋设备 (默认所有设备都在这里)
    let _roomId = (app.globalData.roomInfo || {}).id;
    let _devs = devs || (bizUtil.getAllDevsFilter(app.globalData.devs, _roomId > 0 ? _roomId : undefined)) || [];
    // 是否批量控制
    let groupIds = app.globalData.groupIds || [];
    let _isAll = groupIds.length == 0;
    // console.log("checkOnlyRgb groupIds, _devs, devs = ", groupIds, _devs, devs, app.globalData.devs)
    // 最终集合
    let pids = [], versions = [], maxStatus = {};
    // 遍历数据取得集合
    for (let index in _devs) {
      let _item = _devs[index];
      // 添加版本分支集合
      versions.push(_item.v);
      // 防止重复添加
      if (pids.indexOf(_item.t) >= 0) {
        continue;
      }
      if (_isAll || (!_isAll && groupIds.indexOf(_item.a) >= 0)) {
        pids.push(_item.t);
      }
    }
    return {
      pids: pids,
      versions: versions, // 版本能力集合
      maxStatus: maxStatus
    };
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
    // 风扇不需要取得容器的位置
    if (this.data.isFan) {
      return;
    }
    // 取得高度
    this.getContainerOffset();
    // 初始化thumb位置
    this.initThumbPosition(this.data.pageSet.defaultAngle);
  },
  
  initThumbPosition (angle) {
    console.log("initThumbPosition angle = ", angle);
    // 取得角度的坐标位置
    let {x, y} = this.getPositionByAngle(angle);
    console.log("initThumbPosition x, y = ", x, y);
    // 取得角度对应的颜色
    // this.initThumbColorByAngle(x, y);
    this.data.thumb.x = x;
    this.data.thumb.y = y;
    console.log("initThumbPosition x, y, this.data.thumb.angle = ", x, y, this.data.thumb.angle);
    this.setData({
      thumb: this.data.thumb
    })
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 隐藏对话框
    this.showDetailDialog();

    // 设备信息
    let _devInfo = app.globalData.devInfo || {};
    // 更新页面标题
    this.initTitleAndBg(_devInfo?.n, this.data.isFan);
    // 更新自定义时间
    let _timestamp = app.globalData.timerInfo?.timestamp || 0;
    (_timestamp > 0) && (this.data.devStatus.timestamp = _timestamp);
    // 更新时间
    this.initTimerByCustom();
    // 设置是否需要开启定时器
    this.timerAct();
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    // 页面销毁 如果设置过百分比和最佳位置则保存设备状态
    this.saveDevStatus();
    // 清除定时器
    this.clearIntervalTimer(this.data.timer);
  },
  getDevStatus (isFirst) {
    // 取得当前设备的状态
    let _devStatus = app.globalData.devStatus;
    // 第一次
    if (isFirst) {
      // 取得所有设备保存的状态数据
      let _allDevStatus = app.globalData.allStatus || {};
      this.data.allDevStatus = _allDevStatus;
      console.log("getDevStatus storage _allDevStatus = ", _allDevStatus);
      // 根据设备did+短地址获取设备状态
      let _dev = app.globalData.devInfo || {};
      // 设备状态
      _devStatus = _allDevStatus[_dev.d + "_" + _dev.a] || {};
    }
    // 取得当前设备的状态
    return _devStatus || {};
  },
  saveDevStatus () {
    // 保存的设备状态
    let _allDevStatus = this.data.allDevStatus || {};
    let {a, d} = app.globalData.devInfo;

    let _keystr = d + "_" + a;
    
    // 静音数据不覆盖
    !!this.data.devStatus && (this.data.devStatus.buzzer = _allDevStatus[_keystr]?.buzzer || 0);

    // 设备状态
    _allDevStatus[_keystr] = this.data.devStatus;

    app.globalData.allStatus = _allDevStatus;
    // 保存到缓存中
    app.setBizStorage('devStatus', _allDevStatus);
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  // 根据是否风扇还是风扇灯初始化页面
  initTitleAndBg (name, isFan) {
    // ability.setNavigationBarTitle(_title);
    this.data.navObj.center.text = name || '';
    if (!!isFan) {
      // 风扇
      this.data.navObj.bgClass = "bgF9";
      this.data.navObj.center.style = "";
      this.data.navObj.left.type = "img";
      this.data.navObj.left.img = "";
    } else {
      this.data.navObj.center.style = "white";
      this.data.navObj.left.type = "img";
      this.data.navObj.left.img = "/static/img/back_white.png";
    }
    this.setData({
      navObj: this.data.navObj
    })
  },
  getDelays () {
    let _result = [{
      text: '1分钟',
      value: 1
    }, {
      text: '3分钟',
      value: 3
    }, {
      text: '5分钟',
      value: 5
    }, {
      text: '10分钟',
      value: 10
    }, {
      text: '15分钟',
      value: 15
    }, {
      text: '30分钟',
      value: 30
    }, {
      text: '1小时',
      value: 60
    }, {
      text: '2小时',
      value: 120
    }, {
      text: '自定义',
      value: -1
    }];
    // for (let i = 0; i < 9; i++) {
    //   _result.push({
    //     title: ,
    //     value: 
    //   })
    // }
    return _result
  },
  selectIndex (e) {
    console.log("selectIndex e = ", e);
    let {index} = e.currentTarget.dataset;
    if (index == this.data.currentIndex) {
      return;
    }
    console.log("selectIndex index = ", index);
    this.data.currentIndex = index;
    this.setData({
      currentIndex: index
    })
  },

  openTimerDialog () {
    // 打开定时设置
    this.setDialogFlg(false);
  },
  hideDialog () {
    this.setDialogFlg(true);
  },
  setDialogFlg (value) {
    // 打开定时设置
    this.data.showTimerDialog = value || false;
    this.setData({
      showTimerDialog: this.data.showTimerDialog
    })
  },
  timerSet (e) {
    let {index} = e.currentTarget.dataset;
    let {value} = this.data.delayTimerArray[index];
    let {brightness, oriBrightness} = this.data.devStatus.light || {};
    let _brightness = oriBrightness || brightness;
    if (value < 0) {
      // 关闭对话框
      this.hideDialog();
      // 自定义则跳转页面
      return ability.navigateTo('/device/fan/timer/index?brightness=' + _brightness + "&")
    }
    // 当前时间戳
    let _timestamp = Date.now() / 1000;
    // 延迟秒数
    let _delay = parseInt(value * 60);
    console.log("timerSet _delay = ", _delay);
    // 关闭定时
    this.actDevDelaySet({
      delay: _delay,        // 延迟时间，单位秒 (最大2个字节)
      start: _brightness,        // 起始亮度 (最大1个字节)
      interval: 0,   // 执行间隔，单位秒(最大1个字节)
      step: -127,        // 步长(正负) (最大1个字节，大于0表示变亮，小与0表示变暗)
      count: 1      // 执行次数(最大1个字节)
    }, () => {
      // 设置时间戳
      this.data.devStatus.timestamp = _timestamp + _delay;
      // 关闭dialog
      this.hideDialog();
      // 更新定时数据
      this.initTimerByCustom();
      // 设置是否需要开启定时器
      this.timerAct();
    })
  },
  initTimerByCustom () {
    // 取得时间戳
    let timestamp = this.data.devStatus.timestamp || 0;
    // 取得当前时间戳
    let _timestamp = Date.now() / 1000;
    console.log("initTimerByCustom _timestamp = ", _timestamp);
    // 差值
    let _seconds = Math.round(timestamp - _timestamp);
    console.log("_seconds = ", _seconds, !!timestamp);
    // 判断大小
    if (!timestamp || _seconds < 0) {
      // 当前时间戳大于设置的定时时间戳
      this.data.delayInfo.h = '--';
      this.data.delayInfo.m = '--';
      // 清除定时器
      this.clearIntervalTimer(this.data.timer);
    } else {
      this.data.delayInfo.h = parseInt(_seconds / 3600);
      this.data.delayInfo.m = parseInt((_seconds % 3600) / 60);
    }
    this.setData({
      delayInfo: this.data.delayInfo
    })
  },
  clearIntervalTimer (timer) {
    // 清除定时器
    bizUtil.clearTimer(timer, "interval");
  },
  timerAct () {
    // 清除定时器
    this.clearIntervalTimer(this.data.timer);
    // 判断如果有定时关闭的话，则启动定时，否则不处理
    let {h, m} = this.data.delayInfo;
    let _str = h + m;
    if ((h + m) <= 0 || _str == '----') {
      return;
    }
    console.log("now = ", Date.now() / 1000);
    this.data.timer = setInterval(this.initTimerByCustom, 30000);
  },

  // 打开风扇
  fanOpen () {
    console.log("fanOpen");
    let _devStatus = this.data.devStatus;
    this.actDev('fan', {pwr: 1, speed: _devStatus.fan.speed || 26});
  },
  fanClose () {
    console.log("fanClose");
    this.actDev('fan', {pwr: 0});
  },
  lightOpen () {
    console.log("lightOpen");
    let _devStatus = this.data.devStatus;
    let {oriBrightness, brightness} = _devStatus.light;
    this.actDev('light', {brightness: oriBrightness || brightness}); // pwr: 1, 
  },
  lightClose () {
    console.log("lightClose");
    this.actDev('light', {brightness: 0}); // pwr: 0, 
  },
  allOpen () {
    // 风扇控制
    if (this.data.isFan) {
      // 风扇控制
      return this.actFanDev({
        pwr: 1,
        buzzer: 1,
        wheel: 1,
        air: 1
      });
    }

    return app.globalData.blsdk.controlWithDevice(app.globalData.devInfo, [0x8C, 0, 0, 0, 0, 0, 0x01, 0x80]);

    let _devStatus = this.data.devStatus;
    let {oriBrightness, brightness} = _devStatus.light;
    console.log("allOpen oriBrightness, brightness = ", oriBrightness, brightness);
    this.actDev(['light', 'fan'], [{brightness: oriBrightness || brightness}, {pwr: 1, speed: _devStatus.fan.speed || 26}]); // pwr: 1, 
  },
  allClose () {
    // 风扇控制
    if (this.data.isFan) {
      // 风扇控制
      return this.actFanDev({
        pwr: 0,
        buzzer: 0,
        wheel: 0,
        air: 0
      });
    }

    return app.globalData.blsdk.controlWithDevice(app.globalData.devInfo, [0x8D, 0, 0, 0, 0, 0, 0x01, 0]);

    this.actDev(['light', 'fan'], [{brightness: 0}, {pwr: 0}]);// pwr: 0, 
    // // 设置亮度为关
    // let _p = this.getPercentByRealValue(_ob || _b, this.data.brightSliders);
    // this.data.brightSliders.percent = _p;
    // this.initBrightness(_p);
  },
  // 根据风速和选中的正反转设置值
  getValueBySpeedAndIndex (index, speed) {
    // // 如果切换的值前后相等
    // if (index == )
    // 取得配置
    let _json = this.getPointsConfig();
    let _tarJson = {}, _oriJson = {};
    // 旧的风速
    // let _speed = speed;
    // 计算出老的间隔
    if (index == 1) {
      // 正转 5裆
      _tarJson = _json[0];
      _oriJson = _json[1];
    } else if (index == 2) {
      // 反转 3裆
      _tarJson = _json[1];
      _oriJson = _json[0];
    }
    // 计算出原始数据占了几个间隔
    let _seperate = (speed - _oriJson.minVal) * (_oriJson.points.length) / (_oriJson.maxVal - _oriJson.minVal);
    // 四舍五入
    _seperate = Math.round(_seperate);
    console.log("getValueBySpeedAndIndex _seperate, speed = ", _seperate, speed);
    // 计算出现在的几个格子百分比
    let _percent = _seperate * 100 / (_tarJson.points.length);
    console.log("getValueBySpeedAndIndex _seperate, speed, _percent = ", _seperate, speed, _percent);
    if (_percent > 100) {
      _percent = 100;
    } else if (_percent < 0) {
      _percent =  0;
    }
    let _newSpeed = (_percent / 100) * (_tarJson.maxVal - _tarJson.minVal) + _tarJson.minVal;
    console.log("getValueBySpeedAndIndex _seperate, _newSpeed = ", _seperate, _newSpeed);
    return {
      percent: _percent,
      speed: _newSpeed,
      seperator: _seperate
    }
  },
  fanDirection (e) {
    let {index} = e.currentTarget.dataset;
    console.log("fanDirection index = ", index, e);
    // 风速
    let _speed = this.data.devStatus.fan.speed || 26;
    if (index == 1) {

      // 取得正反转的配置
      let _config = this.initSliderConfigByType(0);
      console.log("fanDirection fanSliders _config = ", _config);
      Object.assign(this.data.fanSliders, _config);

      // 取得新数据
      let {speed, percent, seperator} = this.getValueBySpeedAndIndex(index, _speed);
      this.data.devStatus.fan.speed = speed;
      this.data.fanSliders.percent = percent;
      this.setData({
        fanSliders: this.data.fanSliders
      })

      // 正转 20 - 100 %
      this.actDev('fan', {pwr: 1, forward: 0, speed: speed});
      this.initActionRoll(0, this.initSpeed(percent));
      // 初始化百分比
    } else if (index == 2) {
      // 反转 20 - 60 %

      // 取得正反转的配置
      let _config = this.initSliderConfigByType(1);
      console.log("fanDirection fanSliders _config = ", _config);
      Object.assign(this.data.fanSliders, _config);

      // 取得新数据
      let {speed, percent, seperator} = this.getValueBySpeedAndIndex(index, _speed);
      this.data.devStatus.fan.speed = speed;
      this.data.fanSliders.percent = percent;
      this.setData({
        fanSliders: this.data.fanSliders
      })

      // 计算出
      // let _isMax = _speed > _max;
      this.actDev('fan', {pwr: 1, forward: 1, speed: speed});
      this.initActionRoll(1, this.initSpeed(percent));
    } else if (index == 3) {
      // 取得自然风的状态
      let _unnatrWind = this.data.devStatus.fan.unnatrWind || 0;
      let _value = Number(!_unnatrWind);
      console.log("_unnatrWind = ", _value);
      // 设置数据
      this.initAuto(_value);
      // 自然风
      this.actDev('fan', {pwr: 1, unnatrWind: _value, speed: _speed})
    }
  },
  lightDirection (e) {
    let {index} = e.currentTarget.dataset;
    // 亮度值
    let {brightness, oriBrightness} = this.data.devStatus.light;
    if (index == 1) {
      // 正白
      this.actDev('light', {pwr: 1, c: 255, w: 0, brightness: brightness || oriBrightness})
      // 滑动色温条
      this.initThumbPosition(this.data.pageSet.minAngle);
    } else if (index == 2) {
      // 中性
      this.actDev('light', {pwr: 1, c: 127, w: 127, brightness: brightness || oriBrightness})
      // 滑动色温条
      let {minAngle, maxAngle} = this.data.pageSet;
      let _half = (360 - maxAngle + minAngle) / 2;
      this.initThumbPosition(minAngle - _half);
    } else if (index == 3) {
      // 暖光
      this.actDev('light', {pwr: 1, c: 0, w: 255, brightness: brightness || oriBrightness})
      // 滑动色温条
      this.initThumbPosition(this.data.pageSet.maxAngle);
    }
  },
  closeTimerSet () {
    // 关闭定时
    this.actDevDelaySet({
      count: 0      // 执行次数(最大1个字节)
    }, () => {
      // 设置时间戳
      this.data.devStatus.timestamp = 0;
      // 关闭dialog
      this.hideDialog();
      // 更新定时数据
      this.initTimerByCustom();
      // 设置是否需要开启定时器
      this.timerAct();
    })
  },
  actDevDelaySet (cmd, callback) {
    let _cmd = cmd || {};
    if (this.data.isFan) {
      // 风扇是以分钟为单位的 这里传毫秒，sdk会转成分钟
      this.actFanDev({
        timestamp: _cmd.delay || 0
      })
      return;
    }
    // 关闭定时
    return app.globalData.blsdk.delayCtrl(app.globalData.devInfo, _cmd, callback)
  },
  actDev (key, dataJson) {
    let _devStatus = this.data.devStatus;
    let _json = dataJson || {};
    let _oriBrightness = _devStatus?.light?.brightness; // 是否关灯
    console.log("actDev _oriBrightness = ", _oriBrightness);
    let _isClose = false;
    // 判断是否单个控制还是全部控制，如果单个控制，需要设置一个flag
    if (typeof key == "string") {
      // string类型
      // 如果是pwr=0,则不保存数据
      if (_json.brightness != 0) {
        // Object.assign(_devStatus[key], dataJson || {});
      } else {
        _isClose = true;
      }
      Object.assign(_devStatus[key], dataJson || {});

      // 判断是否单个控制
      if (key == "fan") {
        _devStatus["notLight"] = 1;
        _devStatus["notFan"] = 0;
      } else if (key == "light") {
        _devStatus["notLight"] = 0;
        _devStatus["notFan"] = 1;
      }
    } else {
      // array
      for (let index in key) {
        if (_json[index].brightness == 0) {
          _isClose = true;
          // continue;
        }
        Object.assign(_devStatus[key[index]], _json[index] || {});
      }
      // 两个都控制
      _devStatus["notLight"] = 0;
      _devStatus["notFan"] = 0;
    }
    console.log("_isClose = ", _isClose, _devStatus);
    // 这里有个逻辑：关闭时亮度需要设置为0，其他操作需要用关闭之前的亮度
    // 发送数据
    if (_isClose) {
      // 保留原始亮度
      _devStatus.light.oriBrightness = _devStatus.light.brightness || _oriBrightness || 127;
      _devStatus.light.brightness = 0;
      _devStatus.light.pwr = 0;
    }
    app.globalData.blsdk.controlWithDevice(app.globalData.devInfo, _devStatus);
  },
  jumpDetail () {
    console.log("jumpDetail");
    ability.navigateTo("/device/light/devProperty/index?remote=0")
  },
  // 滑块百分比
  sliderChange (event) {
    let {percent, value, originValue} = event.detail || {};
    // let _value = event.detail.value;
    // let _isMoving = event.detail.isMoving;
    // let _originValue = event.detail.originValue;
    // 区间为 5 - 127 默认为0 - 100
    console.log("sliderChange value, percent = ", value, percent, originValue);
    // 保存亮度
    this.initBrightness(percent);
    // this.setData({
    //   brightness: percent
    // })
    // let _status = {
    //   brightness: value
    // };
    // 增加处理 保存原始的值（精确到小数点 为了防止下次进入亮度不一致）
    this.data.devStatus.light.originBPercent = percent;
    // 执行
    this.actDev('light', {pwr: 1, brightness: value})
  },
  // 滑块百分比
  fanSliderChange (event) {
    let {percent, value} = event.detail || {};
    // let _percent = event.detail.percent;
    // let _value = event.detail.value;
    // let _isMoving = event.detail.isMoving;
    // let _originValue = event.detail.originValue;
    // 区间为 5 - 127 默认为0 - 100
    // _val = 5 + parseInt(_val * 1.22);
    console.log("sliderChange value, percent = ", value, percent);

    // 风扇还是风扇灯
    if (this.data.isFan) {
      this.actFanDev({
        speed: value
      });
      return;
    }

    // 初始化风速显示
    this.initSpeed(percent);
    // 初始化滑动速度
    // 是正转还是反转
    let _forward = this.data.devStatus?.fan?.forward;
    // 初始化正反转
    this.initActionRoll(_forward, this.data.speed);
    // let _status = {
    //   brightness: _value
    // };
    // 增加处理 保存原始的值（精确到小数点 为了防止下次进入亮度不一致）
    // this.data.devStatus.originBrightness = _originValue;
    // 执行
    this.actDev('fan', {pwr: 1, speed: value, unnatrWind: 0});
    // 退出自然风
    this.initAuto(0);
  },
  initSpeed (percent) {
    // 是否风扇
    if (this.data.isFan) {
      return;
    }
    // 根据正反转判断显示的挡位
    let _points = (this.data.fanSliders.points || []).length;
    let _per = 100 / _points;
    let _val = Math.round(percent / _per) + 1;
    // if (percent < 100) {
    //   _val += 1;
    // }
    console.log("initSpeed percent, _points, _val = ", percent, _points, _val);
    this.data.speed = _val;
    this.setData({
      speed: this.data.speed
    })
  },
  getSpeedPWithVal (value) {
    console.log("getSpeedPWithVal this.data.fanSliders = ", this.data.fanSliders);
    return bizUtil.getPercentByVal(value, this.data.fanSliders); 
  },
  initBrightness (percent) {
    // 保存亮度
    this.setData({
      brightness: (percent || 0)
    })
  },
  initAuto (value) {
    this.data.isAuto = value || false;
    // 保存亮度
    this.setData({
      isAuto: this.data.isAuto
    })
  },
  // initBrightnessByVal (val) {
  //   return val / 1.27;
  // },








  // 取得canvas的位置
  getContainerOffset () {
    // 根据尺寸取得实际长度
    const query = wx.createSelectorQuery()
    query.select('#fanId').boundingClientRect()
    query.exec((res) => {
      console.log("onReady res = ", res);
      if (!res) {
        return;
      }
      // 取得尺寸
      let _size = res[0] || {};
      this.data.pageSet.pageX = _size.left;
      this.data.pageSet.pageY = _size.top;
      this.data.pageSet.width = _size.width;
      this.data.pageSet.height = _size.height;
    })
  },
  getValueByAngle (angle) {
    let _angle = angle || 0;
    if (_angle > 270) {
      _angle = _angle - 360;
    }
    let {minAngle, maxAngle} = this.data.pageSet;
    console.log("getValueByAngle _angle = ", _angle);
    // 0 - 255
    if (_angle == minAngle) {
      return 255;
    } else if (_angle == (360 - maxAngle)) {
      return 0;
    }
    let _mul = - (360 - (maxAngle - minAngle)) / 255;
    console.log("getValueByAngle _mul = ", _mul);
    // let _tmpAngle = minAngle + _val * _mul;
    let _val = Math.abs(_angle - minAngle) / _mul;
    return _val;
  },
  getAngleByValue (value) {
    let _val = value || 0;
    let {minAngle, maxAngle} = this.data.pageSet;
    // 0 - 255
    if (_val <= 0) {
      return minAngle;
    } else if (_val >= 255) {
      return maxAngle;
    }
    let _mul = -(360 - (maxAngle - minAngle)) / 255;
    let _tmpAngle = minAngle + _val * _mul;
    if (_tmpAngle < 0) {
      _tmpAngle = 360 + _tmpAngle;
    }
    return _tmpAngle;
  },
  // 根据x,y坐标取得角度
  getAngleByPosition (x, y) {
    // 半径
    let {centerX, centerY} = this.data.pageSet;
    let _isRight = x >= centerX;
    let _isDown = y >= centerY;
    // 需要减去中心点坐标
    let _realX = Math.abs(x - centerX); // 
    let _realY = Math.abs(y - centerY); // 
    console.log("getAngleByPosition _realX, _realY = ", _realX, _realY, _isRight, _isDown, centerX, centerY, x, y);
    // _angle大于0
    let _angle = 0;
    if (_realX == 0) {
      _angle = _isDown ? 270 : 90;
    } else {
      let _originAngle = Math.atan(_realY / _realX) * 180 / Math.PI;
      _angle = Math.abs(_originAngle);
      console.log("getAngleByPosition _angle, _originAngle = ", _angle, _originAngle);
      if (_isRight) {
        // 第四和一象限
        _angle = _isDown ? (360 - _angle) : (_angle);
      } else {
        // 第三和二象限
        _angle = _isDown ? (180 + _angle) : (180 - _angle); // 180 + _angle
      }
    }
    return _angle;
  },
  // 根据x，y轴坐标取得x轴对应的y坐标
  getPositionByX (position) {
    let {x, y} = position;
    // 半径
    let {r, centerX, centerY, minAngle, maxAngle} = this.data.pageSet;
    let _isRight = x >= centerX;
    let _isDown = y >= centerY;
    // console.log("getPositionByX _tanDegree = ", );
    // 取得角度
    let _tanDegree = Math.round(this.getAngleByPosition(x, y));
    // if (_tanDegree > minAngle && _tanDegree < 270) {
    //   _tanDegree = minAngle;
    // } else if (_tanDegree > 270 && _tanDegree < (maxAngle + 15)) {
    //   _tanDegree = maxAngle;
    // } else 
    if (_tanDegree > minAngle && _tanDegree < maxAngle) {
      console.log("getPositionByX _tanDegree = ", _tanDegree, minAngle, maxAngle);
      return null;
    }

    console.log("getPositionByX _tanDegree = ", _tanDegree, _isRight, _isDown);
    // 根据半径和角度取得实际的x和y
    let _x = r * Math.cos(_tanDegree * Math.PI / 180);
    let _y = r * Math.sin(_tanDegree * Math.PI / 180);
    return {
      x: Math.abs(_x) * (_isRight ? 1 : -1), // centerX + 
      y: Math.abs(_y) * (_isDown ? 1 : -1), // centerY + 
      angle: _tanDegree
    }
  },
  // 根据角度取得位置
  getPositionByAngle (degree) {
    let {r} = this.data.pageSet;
    console.log("getPositionByAngle r, degree = ", r, degree);
    let x = Math.cos(degree / 180 * Math.PI) * (r);
    let y = Math.sin(degree / 180 * Math.PI) * (r);
    let _isRight = !(degree >= 90 && degree < 270);
    let _isDown = !(degree >= 0 && degree < 180);
    // 绘制canvas的时候需要有半个宽度的补偿值 76是thumb的宽度 先这样写吧
    let _buchang = 0; // circleWidth / 2 - (76 / 2 / this.data.canvas.multip);
    return {
      x: _buchang + Math.abs(x) * (_isRight ? 1 : -1), // centerX + 
      y: _buchang + Math.abs(y) * (_isDown ? 1 : -1), // centerY + 
    };
    // // 半径
    // let {r, centerX, centerY} = this.data.pageSet;
    // // 根据半径和角度取得实际的x和y
    // let _x = r * Math.cos(_tanDegree * Math.PI / 180);
    // let _y = r * Math.sin(_tanDegree * Math.PI / 180);
    // return {
    //   x: centerX + Math.abs(_x) * (_isRight ? 1 : -1),
    //   y: centerY + Math.abs(_y) * (_isDown ? 1 : -1),
    //   angle: angle
    // }
  },
  getPosition (e) {
    let {changedTouches, touches} = e;
    let {pageX, pageY} = this.data.pageSet;
    let _item = (changedTouches || touches)[0];
    // console.log("getPosition _item = ", _item, pageX, pageY);
    return {
      x: ((_item.pageX - (pageX || 0)) || _item.clientX),
      y: ((_item.pageY - (pageY || 0)) || _item.clientY) 
    }
  },
  touchStart (e) {
    // 保存之前的角度，如果幅度小于1，则不获取
    let _position = this.getPosition(e);
    let {angle} = this.getPositionByX(_position) || {};
    if (angle == undefined) {
      return;
    }
    this.data.lastAngle = angle;
    console.log("touchStart angle, this.data.canvas = ", angle, this.data.pageSet);
  },
  touchMove (e) {
    this.dealTouchRgb(e);
  },
  dealTouchRgb (e, needSet) {
    let _position = this.getPosition(e);
    let {x, y, angle} = this.getPositionByX(_position) || {};
    if (angle == undefined) {
      console.error("dealTouchRgb angle = ", angle);
      return;
    }
    
    // 是否需要发送，一般是在touchEnd中
    if (needSet) {
      return angle;
    }

    // 如果角度相差小于1，则不移动
    if (Math.abs(angle - this.data.lastAngle) < 1) {
      return;
    }
    // 保存上一次的角度值
    this.data.lastAngle = angle;
    this.data.thumb.x = x;
    this.data.thumb.y = y;
    this.setData({
      thumb: this.data.thumb
    })
  },
  touchEnd (e) {
    console.log("touchEnd");
    let _angle = this.dealTouchRgb(e, true) || this.data.lastAngle;
    if (_angle == undefined) {
      console.log("touchEnd _angle = ", _angle, this.data.lastAngle);
      return;
    }
    // 清空
    this.data.lastAngle = undefined;
    // 发送数据
    let _c = parseInt(this.getValueByAngle(_angle));
    console.log("touchEnd _c1 = ", _c);
    // if (_c == -255) {
    //   _c = 255;
    // } else 
    if (_c < 0) {
      _c = 255 + _c;
    }
    _c = _c < 0 ? 0 : (_c > 255 ? 255 : _c);
    console.log("touchEnd _c2 = ", _c);

    // _c = 255 - _c;
    let _w = 255 - _c;
    // 亮度值
    let {brightness, oriBrightness} = this.data.devStatus.light;
    // 发送暖光命令
    this.actDev('light', {pwr: 1, c: _c, w: _w, brightness: brightness || oriBrightness})
  },
  clickCW (e) {
    let _position = this.getPosition(e);
    console.log("clickCW _position = ", _position);
  },

  // 空气净化开
  fanAirOpen () {
    let _air
  },
  // 空气净化关
  fanAirClose () {

  },

  // 按钮点击事件
  switchClick (e) {
    let {index, idx} = e.currentTarget.dataset;
    let _parentObj = this.data.fanConfigList[index];
    let _childObj = (_parentObj?.list || [])[idx];
    // key
    let _keyStr = _parentObj.key;
    // value
    let _value = _childObj.value;
    // 是否风扇控制
    let _isFan = this.data.isFan;
    console.log("switchClick _isFan = ", _isFan);
    // 判断点击的类型
    if (_keyStr == "pwr") {
      if (_isFan) {
        this.actFanDev({
          pwr: _value
        });
      } else {
        return app.globalData.blsdk.controlWithDevice(app.globalData.devInfo, [0x8E, 0, 0, 0, 0, 0, 0x01, _value ? 0x80 : 0]);
        
        // 风扇灯
        if (_value) {
          this.fanOpen();
        } else {
          this.fanClose();
        }
      }
    } else if (_keyStr == "air") {
      // 空气净化只有风扇才有
      this.actFanDev({
        air: _value
      });
    } else if (_keyStr == "direction") {
      // 风扇和风扇灯都有
      if (_isFan) {
        this.actFanDev({
          direction: _childObj.key
        });
      } else {
        // 风扇灯
        this.fanDirection({
          currentTarget: {
            dataset: {
              index: _value
            }
          }
        });
      }
    }
  },
  actFanDev (cmdJson) {
    console.log("actFanDev cmdJson = ", cmdJson);
    // 设备信息
    let _devInfo = app.globalData.devInfo;
    // 根据能力取得支持的功能集合
    let _realCmdJson = this.getJsonByDevStatus(cmdJson, this.data.devStatus);
    console.log("actFanDev cmdJson = ", cmdJson);
    // 单个设备控制
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, _realCmdJson);
    if (!_result?.then) {
      return;
    }
    _result.then(res => {
      // 保存数据 如果是定时则需要加上当前时间
      if (_realCmdJson.timestamp) {
        _realCmdJson.timestamp += Math.round(Date.now() / 1000);
      }
      Object.assign(this.data.devStatus, _realCmdJson);

      // 是否需要更新风扇
      this.updateImgBtnByCmd(_realCmdJson);

      // 是否有定时数据更新 
      // 如果是定时则需要加上当前时间
      this.updateFanTimerByCmd(_realCmdJson);
    })
  },
  // 更新风扇正反转数据
  updateImgBtnByCmd (json) {
    console.log("updateImgBtnByCmd json = ", json);
    if (!json) {
      return;
    }
    let _key = "direction";
    let _value = json[_key];
    if (_value == undefined) {
      return;
    }
    // 取得条目
    let _index = this.data.fanConfigList.findIndex(item => item.key == _key);
    console.log("updateImgBtnByCmd _index = ", _index);
    if (_index < 0) {
      return;
    }
    // let _filterArray = this.data.fanConfigList[_index]; // this.data.fanConfigList.filter(item => item.key == _key);
    let _filterObj = this.data.fanConfigList[_index]; // _filterArray[0];
    let _childList = _filterObj?.list || [];
    console.log("updateImgBtnByCmd _childList = ", _childList);
    if (_childList.length == 0) {
      return;
    }
    // // 设置为全部未选中关
    // bizUtil.setArrayItem(_childList, ['isSelected', false]);
    _childList.map(item => {
      item.isSelected = item.key == _value;
      return item;
    })
    console.log("updateImgBtnByCmd _filterObj = ", _filterObj);
    this.setData({
      [`fanConfigList[${_index}]`]: _filterObj
    })
  },
  // 更新风扇定时数据
  updateFanTimerByCmd (json) {
    console.log("updateFanTimerByCmd json = ", json);
    if (!json) {
      return;
    }
    if (json.timestamp == undefined) {
      return;
    }
    // 关闭对话框
    this.hideDialog();
    // 更新定时数据
    this.initTimerByCustom();
    // 设置是否需要开启定时器
    this.timerAct();
  },
  // 根据设备状态能力取得最终的命令json
  getJsonByDevStatus (cmdJson, devStatus) {
    if (!cmdJson) {
      return {};
    }
    if (!devStatus) {
      return cmdJson;
    }
    for (let key in cmdJson) {
      if (devStatus[key] == undefined) {
        delete cmdJson[key];
      }
    }
    return cmdJson;
  },


  jumpDetailClick () {
    // 判断是否大于2个选项
    if (this.data.detailList?.length > 1) {
      // 显示
      this.showDetailDialog(!this.data.showDetail);
      return;
    }
    this.jumpDetail();
  },
  // 详情按钮
  showDetailDialog(value) {
    this.setData({
      showDetail: value || false
    })
  },
  jumpFuncSet () {
    // 是否支持遥控器
    let _remote = (this.data.abilitys || {}).remote || 0;
    let _itemArray = this.data.detailList.filter(item => item.key == "funcset");
    let _title = _itemArray[0]?.title || "";
    // 跳转详情配置页面
    ability.navigateTo("/device/light/funcSet/index?remote=" + _remote + "&title=" + encodeURIComponent(_title))
  }
})