// import { Utils } from "../../utils/util";
import AirConfig from "../../../../static/js/jsConfig/aircondition"
const ability = require("../../../../utils/classDef/ability")
const app = getApp();
const bizUtil = require("../../../../utils/bizUtil")

// 模式和风速的枚举值
const MODE = AirConfig.MODE;
const SPEED = AirConfig.SPEED;

const timeBehavior = require("../../behavior/time")
// scene/pwrAvalible/index.js
Page({
  behaviors: [timeBehavior],
  /**
   * 页面的初始数据
   */
  data: {
    // navObj: {
    //   left: {
    //     type: "img"
    //   },
    //   center: {
    //     text: (app.globalData.panelInfo || {}).n || '按键选择'
    //   }
    // },

    isPwrAvalible: true, // 默认是继电器面板
    devTypeStr: "panel", // 默认是面板 

    // 三个状态 当前 设备状态 例外
    types: [],
    currentType: 0,

    name: '', // 设备名称
    controlList: [], // 操作条目
    actions: [], // 底部按钮列表
    devStatus: {
      pwr: 1,
      mode: 1,
      speed: 1,
      temp: 24,
      unit: "℃", 
    }, // 设备状态
    showActionSheet: false, // 操作栏是否显示
    sheetList: [], // 展示的actionsheet配置

    tempSlider: {
      width: 445, // rpx
      percent: 100, // 百分比
      labelTxt: "亮度",
      minVal: AirConfig?.TEMPS?.min,
      maxVal: AirConfig?.TEMPS?.max,
      minPercent: 0, // 最小百分比
      gradient: false, // 默认不支持滑动时调节
    }
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 根据pid取得品类名称
    let _devTypeStr = bizUtil.getDevTypeStr(app.globalData.devInfo?.t);
    this.data.devTypeStr = _devTypeStr;
    // 组网使能
    this.meshFlag = options?.meshFlag;
    // 是否编辑页面
    let _index = options.index;
    this.isEditIndex = _index == undefined ? -1 : Number(_index);
    // 是否编辑
    this.isEdit = options?.edit || false;
    // 如果是继电器面板设备
    if (_devTypeStr == 'aircondition') {
      // 初始化默认数据
      if (_index >= 0) {
        let _cmdArray = (app.globalData.sceneInfo?.devs || [])[_index].cmd;
        console.log("onLoad _cmdArray = ", _cmdArray);
        // 初始化默认值
        (_cmdArray?.length > 0) && (this.data.devStatus = this.initDefaultVal(_cmdArray));
      }
    }
    // 初始化按钮列表
    this.initActionsByEdit(this.isEditIndex);
    
    // 设置名称和标题
    this.initTitle(app.globalData.devInfo?.n || '');
    // 初始化列表
    this.initLists();
    // 设备状态
    this.setProperValue("devStatus", this.data.devStatus);
    console.log("onLoad this.data.devStatus = ", this.data.devStatus);
    // 根据设备状态初始化滑动条
    this.initTempSlider();
  },

  initTitle (name) {
    let _name = name || "";
    // 初始化名称
    this.setProperValue("name", _name);
    // 设置标题
    ability.setNavigationBarTitle(_name);
  },

  // 区分群控页面
  initLists () {
    let _defaultStatus = this.data.devStatus || {};
    // 初始化控制列表
    let {list} = this.getControlList() || [];

    console.log("initLists _defaultStatus = ", _defaultStatus);

    // 如果是关闭，则忽略后面的数据
    if (!_defaultStatus.pwr) {
      list = list.slice(0, 1);
    }

    // 根据设备状态处理数据
    this.setProperValue("controlList", this.setDataForList(list, _defaultStatus));
  },
  initTempSlider () {
    // 如果是模式，则更新模式名称用于头部显示
    let {key} = this.getConfigByValueInArray(MODE, this.data.devStatus?.mode);
    // 设置亮度值
    let _tempValue = this.data.devStatus?.temp || 24;
    this.data.tempSlider.value = _tempValue;
    if (this.airhotwind == 2 || this.airhotwind == 6 || this.airhotwind == 7) {
      // 单地暖或者地暖+新风
      this.data.tempSlider.minVal = AirConfig?.TEMPS?.hotMin;
      this.data.tempSlider.maxVal = AirConfig?.TEMPS?.hotMax;
    }
    this.data.tempSlider.percent = this.getPercentByValue(_tempValue);
    this.data.tempSlider.bg = (key == "hot") ? '#F78D3F' : (key == "dehumid" ? '#23D2B2' : '#56ABFF'); // 设置默认的背景
    if (_tempValue < 19) {
      this.data.tempSlider.labelClass = " "; // 防止使用默认的白色
    } else {
      this.data.tempSlider.labelClass = ""; // 使用默认的白色
    }
    this.setProperValue("tempSlider", this.data.tempSlider);
  },
  getValueByPercent (percent) {
    let {minVal, maxVal} = this.getTempMinAndMaxVal();
    return minVal + parseInt((maxVal - minVal) * (percent || 0));
  },
  getPercentByValue (value) {
    let {minVal, maxVal} = this.getTempMinAndMaxVal();
    return (value - minVal) / (maxVal - minVal) * 100;
  },
  getTempMinAndMaxVal () {
    let {minVal, maxVal} = this.data.tempSlider || {};
    return {
      minVal,
      maxVal
    }
  },
  // 设置数据
  setDataForList (list, devStatus) {
    let _devStatus = devStatus || {};
    return list.map(item => {
      let {key, type} = item;
      // 赋值
      item.value = _devStatus[key];
      // 如果是枚举类型，则取得枚举的text
      if (type == "enum") {
        let {key: modeKey, text} = this.getConfigByValueInArray(key == "mode" ? MODE : SPEED, item.value);
        item.text = text;
        item.img = "/static/img/common/mode/" + modeKey + ".png";
      }
      return item;
    })
  },
  // 同步数据到页面
  setProperValue (keys, vals) {
    if (this.data == undefined || vals == undefined) {
      return;
    }
    let _obj = {};
    if (keys instanceof(Array)) {
      for (let index in keys) {
        _obj[keys[index]] = vals[index];
      }
    } else {
      _obj[keys] = vals;
    }
    this.setData(_obj);
  },
  // 根据数据区分
  getControlList () {
    // 控制列表
    let list = [{
      title: '开关',
      key: "pwr",
      type: "none"
    }, {
      title: '温度',
      key: "temp",
      type: "slider"
    }, {
      title: '模式',
      key: "mode",
      type: "enum"
    }, {
      title: '风速',
      key: "speed",
      type: "enum"
    }];
    return {
      list
    };
  },

  initDefaultVal (cmdArray) {
    let _config = app.globalData.blsdk.getAirconditionConfig();
    // 初始化默认的按键
    return bizUtil.airconditionCmdToJson(cmdArray, _config);
  },

  getAllActions () {
    return [{
      n: '删除',
      classStr: 'delBtn',
      handle: "unbindScene"
    }, {
      n: '保存',
      classStr: 'saveBtn marginL49',
      handle: "save"
    }]
  },

  initActionsByEdit (panelId) {
    let _list = this.getAllActions();
    // 判断是否编辑
    if (panelId >= 0) {
      // 编辑
    } else {
      // 新增
      _list = _list.slice(1);
      _list[0].classStr = _list[0].classStr.replace("marginL49", "");
    }
    this.setProperValue("actions", _list);
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    // app.saveAllDevsMap();
    // app.setBizStorage("devs", app.globalData.devs);
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },
  setPanelSceneInfo (roomId, did, sceneInfo, key) {
    let _panels = app.globalData.devs[roomId].panels || [];
    let _index = this.getPanelByD(_panels, did);
    if (_index < 0) {
      return false;
    }
    _panels[_index].ds = _panels[_index].ds || [];
    _panels[_index].push({
      sid: sceneInfo.sid,
      rid: sceneInfo.rid,
      t: sceneInfo.t,
      key: key
    })
  },
  getPanelByD (array, did) {
    let _index = -1;
    if (!array || array.length == 0) {
      return _index;
    }
    for (let index of array) {
      let _item = array[index];
      if (_item.d == did) {
        return index;
      }
    }
    return _index;
  },

  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    });
  },
  hideLoading () {
    ability.hideLoading();
  },

  getCmdArray () {
    let {pwr, temp, speed, mode} = this.data.devStatus || {};
    let _cmdArray = [];
    if (pwr == 0) {
      _cmdArray.push(0);
    } else {
      _cmdArray.push(pwr);
      _cmdArray.push(mode);
      _cmdArray.push(temp);
      _cmdArray.push(speed);
    }
    return _cmdArray; // Utils.ab2hex(_cmdArray);
  },

  save () {
    // 场景信息
    let _sceneInfo = app.globalData.sceneInfo;
    let _devInfo = app.globalData.devInfo || {}; // 选择的设备

    // 命令串
    let _cmdArray = this.getCmdArray();
    console.log("save _cmdHexStr = ", _cmdArray);
    
    // 提示等待中
    this.showLoading('设置中…');

    let _sid = _sceneInfo.sid, _rid = _sceneInfo.rid || 0;
    app.globalData.blsdk.roomSceneAddWithCommand(_devInfo, (_rid << 16) + _sid, _cmdArray, (result) => {
      console.error("save result = ", result);
      this.hideLoading();
      if (result < 0 || result === false || result == undefined || result.errCode) {
        return ability.showToast({
          title: "设置失败",
          icon: "error"
        });
      }
      // this.setShowCustomToast(true, 'success', '设置成功', true);
      ability.showToast({
        title: "设置成功",
        icon: "success"
      });
      // 当前状态status = {}
      setTimeout(() => {
        this.dealAfterSaveScene(_cmdArray);
      }, 1500);
    })
  },

  dealAfterSaveScene (cmdArray) {
    let _devInfo = app.globalData.devInfo || {}; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    // 把json的数据转string
    let _tip = bizUtil.getTipByStatus(cmdArray, _devInfo.t);
    console.log("dealAfterSend _sceneInfo, cmdArray, this.isEditIndex = ", _tip, _sceneInfo, cmdArray, this.isEditIndex);
    // 防止不存在数据
    _sceneInfo.devs = _sceneInfo.devs || [];
    if (this.isEditIndex < 0) {
      _sceneInfo.devs.push({
        a: _devInfo.a,
        d: _devInfo.d,
        n: _devInfo.n,
        t: _devInfo.t,
        roomId: _devInfo.id,
        cmd: cmdArray,
        tip: _tip
      })
    } else {
      let _index = this.isEditIndex;
      if (_index >= 0) {
        (_sceneInfo.devs[_index] || {}).a = _devInfo.a;
        (_sceneInfo.devs[_index] || {}).d = _devInfo.d;
        (_sceneInfo.devs[_index] || {}).n = _devInfo.n;
        (_sceneInfo.devs[_index] || {}).t = _devInfo.t;
        (_sceneInfo.devs[_index] || {}).roomId = _devInfo.id;
        (_sceneInfo.devs[_index] || {}).tip = _tip;
        (_sceneInfo.devs[_index] || {}).cmd = cmdArray;
      }
    }
    // 返回上一页
    if (this.isEditIndex >= 0) {
      if (this.isEdit) {
        this.back(2);
      } else {
        this.back(1);
      }
    } else {
      this.back(2);
    }
  },

  // 删除场景下绑定的执行设备
  unbindScene () {
    // 很奇怪的问题  使用ability调用showModal不提示了
    wx.showModal({
      title: "",
      content: "是否删除该场景下的按键"
    }).then((res) => {
      if (!res || !res.confirm) { return; }
      setTimeout(() => {
        ability.showToast({
          title: "删除中",
          icon: "loading",
          duration: 19000
        })
      }, 0);
      
      // 删除设备
      this.roomSceneDeleteWithDeviceAddr((result) => {
        console.error("delDev result = ", result);
        let _isSuccess = result >= 0;
        // 成功
        setTimeout(() => {
          ability.showToast({
            title: _isSuccess ? '删除成功' : "删除失败请重试",
            icon: _isSuccess ? 'success' : 'none'
          });
        }, 0);
        // 如果失败，则不处理后续
        if (!_isSuccess) {
          return;
        }
        // 取得所有设备
        let _devs = app.globalData.sceneInfo?.devs || [];
        // 取得下标
        let _index = this.isEditIndex;
        // 删除设备
        _devs.splice(_index, 1);
        // 返回按钮
        setTimeout(this.back, 1500);
      });
    })
  },
  // 删除场景下已经绑定的设备
  roomSceneDeleteWithDeviceAddr (callback) {
    let _devInfo = app.globalData.devInfo;
    let {rid, sid} = app.globalData.sceneInfo;
    // console.log("app.globalData.sceneInfo status = ", app.globalData.sceneInfo, _devInfo)
    //  resetPannelBindCommandWithShortAddr
    app.globalData.blsdk.roomSceneDeleteWithDeviceAddr(_devInfo.a, (rid << 16) + sid, callback)
  },

  getAllIndex (allStr, str) {
    let _result = [];
    if (!allStr) {
      return _result;
    }
    let _allStr = allStr;
    let _isFinish = false;
    while (!_isFinish) {
      let _index = _allStr.indexOf(str);
      if (_index < 0) {
        _isFinish = true;
        return _result;
      } else {
        _result.push(_index);
        _allStr = _allStr.slice(_index + 1);
      }
    }
    return _result;
  },

  // 发送控制命令
  sendCtrl (index, keyStr) {
    console.log("funcClick index = ", index);
    // 判断发送的数据
    if (keyStr == "open") {
      // todo list 第一个字节为panelId, 第二个字节为开关
      this.actDev([1 << index, 1 << index]);
    } else if (keyStr == "close") {
      // todo list
      this.actDev([1 << index, 0]);
    }
  },
  actDev (cmdKey, callback) {

    let _devInfo = app.globalData.devInfo;
    app.globalData.blsdk.controlWithDevice(_devInfo, cmdKey);

    // 检查蓝牙是否可用
    let _isAvalible = app.globalData.blsdk.isBleAvailable();
    if (!_isAvalible) {
      return;
    }
    !!callback && (callback(0));
  },

  back (num) {
    // 返回上一页
    ability.navigateBack(num || 1);
  },
  // 设备执行类型设置
  typeItemClick (e) {
    // 设置选中的值
    let {index} = e.currentTarget.dataset;
    this.data.currentType = index || 0;
    this.setData({
      currentType: this.data.currentType
    });
  },
  currentStatus () {
    // 保存当前状态
    let _devInfo = app.globalData.devInfo || {}; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息

    // return this.dealAfterSend();

    // 提示等待中
    this.setShowCustomToast(true, 'loading', '设置中…', 20000);
 
    let _sid = _sceneInfo.sid, _rid = _sceneInfo.rid || 0;
    // let _devStatus = this.data.devStatus;
    console.log("currentStatus _sid, _rid = ", _sid, _rid);
    // app.globalData.blsdk.roomSceneAddWithCommand(_devInfo, (_rid << 16) + _sid, _devStatus, (result) => {
    app.globalData.blsdk.roomSceneAddWithDeviceAddr(_devInfo.a, (_rid << 16) + _sid, (result) => {
      console.error("save result = ", result);
      if (result < 0 || result === false || result == undefined || result.errCode) {
        this.setShowCustomToast(true, 'error', '设置失败');
      } else {
        this.setShowCustomToast(true, 'success', '设置成功');
        
        setTimeout(this.dealAfterSend, 1500);
      }
    })
  },
  // 不动作
  customNoAction () {
    let _devInfo = app.globalData.devInfo || {}; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息

    // return this.dealAfterSend();

    // 提示等待中
    this.setShowCustomToast(true, 'loading', '设置中…', 20000);

    let _sid = _sceneInfo.sid, _rid = _sceneInfo.rid || 0;
    app.globalData.blsdk.customSceneAction(_devInfo.a, (_rid << 16) + _sid, (result) => {
      console.error("save result = ", result);
      if (result < 0 || result === false || result == undefined || result.errCode) {
        // this.setShowCustomToast(true, 'error', '设置失败', true);
        this.setShowCustomToast(true, 'error', '设置失败');
      } else {
        // this.setShowCustomToast(true, 'success', '设置成功', true);
        this.setShowCustomToast(true, 'success', '设置成功');

        setTimeout(this.dealAfterSend, 1500);
        // setTimeout(this.back, 1500);
      }
    })
  },
  // 设置是否显示自定义对话框
  setShowCustomToast (value, type, str, duration) {
    console.log("setShowCustomToast ability = ", ability);
    if (!value) {
      ability.hideToast()
      // ability.hideLoading()
    } else {
      let _config = {
        title: str || '',
        icon: type || '',
        duration: duration || 1500
      };
      ability.showToast(_config);
    }
  },
  pwrClick (e) {
    console.log("pwrClick e = ", e);
    let {value} = e.detail;
    this.pwrClickDetail(Number(value));
  },
  openClick () {
    this.pwrClickDetail(1);
  },
  closeClick () {
    this.pwrClickDetail(0);
  },
  pwrClickDetail (value) {
    let sendKey = "pwr";
    this.actDev({
      [sendKey]: value || 0
    }, (errCode) => {
      console.log("pwrClickDetail value, errCode = ", value, errCode);
      this.dealAfterSend(errCode, {value, sendKey});
    });
  },
  tempChange (e) {
    let value = parseInt(e?.detail?.value || 0);
    let sendKey = "temp";
    // 设置选中的数据，并下发
    this.actDev({
      [sendKey]: value
    }, (errCode) => {
      this.dealAfterSend(errCode, {value, sendKey});
    });
  },
  funcClick (e) {
    let {index} = e.currentTarget.dataset;
    let {key} = this.data.controlList[index];
    console.log("funcClick index, key = ", index, key);

    if (key == "pwr") {
      // todo list 第一个字节为panelId, 第二个字节为开关
      this.actDev([1 << index, 1 << index]);
    } else if (key == "temp") {
      // todo list
      this.actDev([1 << index, 0]);
    } else if (key == "mode" || key == "speed") {
      // 显示actionsheet和内容
      this.actionSheetShow();
      // 当前选中的数据
      let _dealActionData = this.getActionShowList(key, this.data.devStatus[key]);
      this.setProperValue("sheetList", _dealActionData);
    }
  },
  // 显示操作栏
  actionSheetShow () {
    this.setProperValue("showActionSheet", true);
  },
  actionSheetHide () {
    this.setProperValue("showActionSheet", false);
  },
  // 取得展示的列表
  getActionShowList (key, defalutValue) {
    let _list = key == "mode" ? MODE : SPEED;
    // 增加img字段 是否选中
    return _list.concat([]).map(item => {
      (item.selected = defalutValue == item.value);
      (!item.img && (item.img = "/static/img/common/mode/" + item.key + ".png"));
      // 增加一个参数用于actionsheet点击后更新devStatus的主键
      item.sendKey = key;
      return item;
    });
  },
  actionSheetClick (e) {
    let {index} = e.detail;
    if (index < 0) {
      return this.actionSheetHide();
    }
    let {key, value, sendKey} = this.data.sheetList[index];
    console.log("actionSheetClick key, value, sendKey = ", key, value, sendKey);
    // 设置选中的数据，并下发
    this.actDev({
      [sendKey]: value
    }, (errCode) => {
      this.dealAfterSend(errCode, {value, sendKey});
    });
  },
  dealAfterSend (code, {value, sendKey}) {
    if (code < 0 || code == undefined) {
      ability.showToast({
        title: '下发失败',
      })
      return;
    }
    console.log("dealAfterSend sendKey, value = ", sendKey, value);
    // 更新数据
    this.data.devStatus[sendKey] = value;
    // 更新列表
    this.initLists();
    // 更新温度
    this.initTempSlider();
    // 关闭对话框
    this.actionSheetHide();
  },
  // 从数组中取得value值的text
  getConfigByValueInArray (array, value) {
    // console.log("getConfigByValueInArray array, value = ", array, value);
    if (!array || array.length == 0) {
      return {};
    }
    let _results = array.filter(item => item.value == value);
    if (_results.length == 0) {
      return {};
    }
    return _results[0];
  },
})