// device/gateway/timer/detail/index.js
import bizUtil from "../../../../utils/bizUtil"
import GROUPS from "../../../../static/js/jsConfig/groups"
import ability from "../../../../utils/classDef/ability"
const timeBehavior = require("../../behavior/time")
const app = getApp();
Page({
  behaviors: [timeBehavior],
  
  /**
   * 页面的初始数据
   */
  data: {
    topItems: [],
    items: [], // 头部的显示内容
    bottomBtns: [], // 底部按钮
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad (options) {
    let {title, index} = options || {};
    // 初始化标题
    this.setNavTitle(title);
    // 是否编辑场景
    this.index = index == undefined ? -1 : index;
    // 初始化底部按钮
    this.initBottomBtnsByIndex(this.index);
    // 保存设备信息
    this.devInfo = app.globalData.devInfo;
  },
  initBottomBtnsByIndex (index) {
    let _list = this.getAllBottomBtns();
    if (index < 0) {
      // 新增
      _list = _list.slice(1, 2);
    }
    this.setData({
      bottomBtns: _list
    })
  },
  getAllBottomBtns () {
    return [{
      text: "删除",
      key: "del",
      className: "delBtn",
      handle: "delClick"
    }, {
      text: "保存",
      key: "save",
      className: "saveBtn",
      handle: "saveClick"
    }]
  },

  setNavTitle (title) {
    ability.setNavigationBarTitle(decodeURIComponent(title || ""))
  },

  initDataByTimer (timerInfo) {
    if (!timerInfo) {
      return;
    }
    // let {repeat, time, actions} = timerInfo;
    // console.log("initDataByTimer repeat, time = ", repeat, time);
    this.data.topItems = this.getTopItems(timerInfo?.name);
    // 顶部条目
    this.data.items = this.getItems(timerInfo);
    // 渲染数据
    this.setData({
      topItems: this.data.topItems,
      items: this.data.items
    })
  },
  getTopItems (name) {
    return [{
      title: "定时名称",
      value: name || "",
      edit: true
    }]
  },
  getItems ({time, cmd}) {
    // 解析执行时间和重复次数
    let {repeatTip, repeat, hour, minute} = this.getRepeatAndTimeByFormat(time);
    let _timeStr = this.getHHMMStr(hour, minute);
    return [{
      title: "定时",
      timeStr: "定时 " + _timeStr,
      repeatStr: repeatTip,
      key: "time"
    }, {
      title: "执行",
      list: this.getActionByCmd(cmd),
      key: "action"
    }];
  },
  getActionByCmd (cmd) {
    let _result = [];
    if (!cmd || cmd.length == 0) {
      return _result;
    }
    // 是否场景触发
    // let _cmdType = 0; // 0 - 单控，1 - 群控，2 - 场景触发， 3 - 循环情景
    // 解析第一层cmd数据 取得_cmdType和命令
    console.log("getActionByCmd cmd = ", cmd);
    let {array, hex, type} = this.getBase64CmdAndType(cmd);
    console.log("getActionByCmd type = ", type, array);

    // 单控
    if (type == 0) {
      // 取得命令的json数据 cmdData json或者数组
      let {meshFlag, data} = this.getStatusJsonByCmdArray(array, type);
      let {device, cmdData} = data;
      console.log("getActionByCmd cmdData, type = ", cmdData, type);
      if (device) {
        // 临时保存执行设备的信息
        this.actionDevs = this.actionDevs || [];
        this.actionDevs.push(Object.assign({}, device, {
          cmd: cmdData
        }));

        _result.push({
          icon: "/static/img/common/" + bizUtil.getDevImgByT(device) + ".png",
          iconClass: "size80",
          n: device?.n || "",
          tip: bizUtil.getTipByStatus(cmdData, device?.t) || "",
          tabIndex: type, // 用于添加页面显示用
          devIndex: this.actionDevs.length - 1
        })
      }
    } else if (type == 2) {
      // 取得命令的json数据 cmdData json或者数组
      let {meshFlag, data: _sceneInfo} = this.getStatusJsonByCmdArray(array, type);
      // let _sceneInfo = 
      _result.push({
        icon: '../../../../static/img/common/scene/' + _sceneInfo.i + '.png',
        iconContainerClass: "size68 bgF78D3F",
        iconClass: "size50",
        n: _sceneInfo.n,
        tip: "",
        tabIndex: type, // 用于addDevice页面显示用
        sid: _sceneInfo?.sid || 0
      })
    } else if (type == 1) {
      // 群组控制
      // 取得命令的json数据 cmdData json或者数组
      let {meshFlag, data} = this.getStatusJsonByCmdArray(array, type);
      let {device, cmdData} = data;
      console.log("getActionByCmd cmdData = ", cmdData, device);
      if (device) {
        // 临时保存执行设备的信息
        this.actionDevs = this.actionDevs || [];
        this.actionDevs.push(Object.assign({}, device, {
          cmd: cmdData
        }));

        _result.push({
          icon: "/static/img/common/group.png",
          iconClass: "devIcon",
          n: device?.n || "",
          tip: bizUtil.getTipByStatus(cmdData, device?.t) || "",
          tabIndex: type, // 用于添加页面显示用
          devIndex: this.actionDevs.length - 1
        })
      }
    }
    return _result;
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    this.reloadDataByInfo();
  },

  reloadDataByInfo() {
    // 保存设备信息
    app.globalData.devInfo = this.devInfo;
    // 清空执行设备
    this.actionDevs = [];
    // 根据定时数据初始化渲染数据
    this.initDataByTimer(app.globalData.timerInfo);
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  delClick () {
    // let _devInfo = app.globalData.devInfo || {};
    let _timeInfo = app.globalData.timerInfo || {};
    console.log("delClick _timeInfo = ", _timeInfo);
    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo || {});
    _devInfo.d = _devInfo.wd || _devInfo.d;
    // 域名
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    

    app.globalData.blsdk.gatewayTimer(_devInfo, {
      host: _groupStr, // 域名

      did: _devInfo?.d,
      act: 1,
      timerlist: [{
        type: _timeInfo?.type,
        id: _timeInfo?.id
      }]
    }, (res) => {
      console.log("delClick res = ", res);
      if (res?.status != 0) {
        return this.showToastMsg("删除失败");
      }
      this.showToastMsg("删除成功");
      setTimeout(() => {
        this.back();
      }, 1000);
    })
  },
  back () {
    ability.navigateBack();
  },
  showToastMsg (msg) {
    ability.showToast({
      title: msg
    });
  },
  saveClick () {
    let _timeInfo = app.globalData.timerInfo || {};

    if (!_timeInfo.cmd) {
      return ability.showToast({
        title: "请选择执行设备"
      });
    }
    console.log("saveClick _timeInfo = ", _timeInfo);

    let _type = 0;
    let _tip = "";
    if (this.index >= 0) {
      // 编辑
      _type = 2;
      _tip = "编辑";
    } else {
      // 新增
      _type = 0;
      _tip = "添加";
      // _timeInfo.id = 0; // 新增为0或者为空
    }
    // 编辑或者新增，使能都为1
    _timeInfo.en = 1; // 定时使能 默认打开
    // 如果仅执行一次，需要把定时时间往后挪一天
    let _onceTimeStr = this.getOnceTimeByTimer(_timeInfo);
    console.log("saveClick _onceTimeStr = ", _onceTimeStr);
    if (_onceTimeStr) {
      // 0_11_21_13_10_*_2022 仅一次才需要处理
      // 转成标准时间 2018/10/07 11:00:00
      let _formatDataStr = this.getOnceFormatStrByTimer(_onceTimeStr);
      console.log("enableTimer _formatDataStr = ", _formatDataStr);
      // 根据时区偏移
      let _dataStr = this.getSetTimeByZone(_formatDataStr);
      console.log("enableTimer _dataStr = ", _dataStr);
      _timeInfo.time = _dataStr;
    } else {
      // 时间设置
      let _tmpStr = this.getWeekZoneTimeByStr(_timeInfo.time);
      _timeInfo.time = _tmpStr;
      console.log("_tmpStr, _timeInfo.time = ", _tmpStr, _timeInfo.time);
    }

    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo || {});
    _devInfo.d = _devInfo.wd || _devInfo.d;
    // 域名
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    
    console.log("saveClick _timeInfo = ", _timeInfo, _devInfo);

    // return;

    app.globalData.blsdk.gatewayTimer(_devInfo, {
      host: _groupStr, // 域名

      did: _devInfo?.d,
      act: _type,
      timerlist: [_timeInfo]
    }, (res) => {
      console.log("saveClick res = ", res);
      let {status, idlist} = res || {};
      if (status == 0 && idlist.length > 0) {
        this.showToastMsg(_tip + "成功");
        setTimeout(() => {
          this.back();
        }, 1000);
      } else {
        this.showToastMsg(_tip + "失败")
      }
    })
  },
  // 下一步
  save () {
    if (this.index >= 0) {
      app.globalData.timerList[this.index] = app.globalData.timerInfo;
    } else {
      app.globalData.timerList.push(app.globalData.timerInfo);
    }
    console.log("save app.globalData.timerInfo = ", app.globalData.timerInfo);
    // 返回
    ability.navigateBack()
  },

  // 长按
  longClick (e) {
    console.log("longClick e = ", e);
    let {index} = e.currentTarget.dataset;
    let {key, list} = this.data.items[index];
    console.log("longClick key, list = ", key, list);
    if (key == "time") {
      return;
    } else if (key == "action") {
      // 只有执行设备才能长按
      if (!list || list.length == 0) {
        return;
      }
      // 删除
      this.showDelModal(() => {
        // 取得所有设备
        delete app.globalData.timerInfo.cmd;
        // 刷新数据
        this.reloadDataByInfo();
      });
    }
  },
  showDelModal (callback) {
    ability.showModal({
      title: "",
      content: "是否删除该执行设备",
      showCancel: true,
      // confirmColor: param.confirmColor || "#576B95",
      // confirmText: param.confirmText || "确定",
      // cancelText: param.cancelText || "取消",
      // cancelColor: param.cancelColor || '#000000',
    }).then(res => {
      if (!res) {
        return;
      }
      !!callback && (callback());
    })
  },

  itemClick (e) {
    let {index} = e.currentTarget.dataset;
    let {key, list} = this.data.items[index];
    if (key == "time") {
      ability.navigateTo("../setTime/index?title=" + encodeURIComponent("定时设置") + "&index=" + this.index);
    } else if (key == "action") {
      console.log("itemClick action list = ", list, this.actionDevs, app.globalData.timerInfo);
      if (!!list && list.length > 0) {

        // 这里需要用到list,因为场景的时候actionDevs=0
        let {tabIndex, sid, devIndex} = list[0];
        if (!this.actionDevs || this.actionDevs.length == 0) {
          ability.navigateTo("../addDevice/index?typeIndex=" + tabIndex + "&did=" + sid + "&index=" + 0);
          return;
        }
        // 把执行设备和状态设置
        let _devInfo = this.actionDevs[devIndex];
        app.globalData.devInfo = _devInfo; // 这里只有一条执行数据，所以取1
        console.log("itemClick this.actionDevs, index = ", _devInfo, index);
        // if (!app.globalData.devInfo) {
        //   return ability.navigateTo("../addDevice/index")
        // }
        // 判断设备品类
        let _devTypeKey = bizUtil.getDevTypeStr(_devInfo?.t);
        console.log("itemClick _devTypeKey = ", _devTypeKey);
        if (_devTypeKey == "panel" || _devTypeKey == "powerstrip") {
          let _extendStr = "&devType=" + _devTypeKey;
          return ability.navigateTo("../pwrAvalible/index?index=" + index + _extendStr);
        }
        return ability.navigateTo("../status/index?index=" + index);
      }
      ability.navigateTo("../addDevice/index")
    }
  },
  topItemClick (e) {
    let {index} = e?.detail || {};
    let {title, value} = this.data.topItems[index];
    // name, from, index, title
    ability.navigateTo("../../../light/deviceName/index?from=gatewaytime&title=" + encodeURIComponent(title) + "&name=" + encodeURIComponent(value));
  }
})