// device/gateway/timer/list/index.js
import ability from '../../../../utils/classDef/ability'
import bizUtil from '../../../../utils/bizUtil'
import {Utils} from "../../../../utils/util"
import GROUPS from "../../../../static/js/jsConfig/groups"
const app = getApp()
const timeBehavior = require("../../behavior/time")
Page({
  behaviors: [timeBehavior],
  /**
   * 页面的初始数据
   */
  data: {
    list: [], // 定时列表
    bottomBtns: [], // 底部按钮

    typeStr: "", // 定时还是循环情景
    roleNum: 0,
    status: -2, // 用于区分是查询中或者查询失败、还是查询结束

    from: "", // 页面来源 默认网关定时、循环进入，另一个是场景添加网关循环场景进入
    selectIdx: -1, // 用于场景绑定循环场景用
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    console.log("onLoad options = ", options);
    // idx, 
    let {title, type, from, index} = options || {};
    // 初始化标题
    this.initNavTitle(title);
    // 初始化类型 是定时还是循环定时
    this.type = type;
    let _isCycle = type == "cycle";
    // 判断来源页面
    this.data.from = from || "";
    // 是否场景中添加循环场景
    this.isFromScene = this.data.from == "scene";
    // 编辑的下标
    this.editIndex = index == undefined ? -1 : (Number(index) || 0);
    if (this.editIndex >= 0) {
      // 选中的场景idx
      let _selectObj = (app.globalData.sceneInfo?.devs || [])[this.editIndex] || {};
      let _cmdArray = _selectObj.cmd || [];
      // 这里有个bug 如果本手机绑定了某个循环情景，却在另一个手机删除，此时进入编辑不应该能直接绑定
      if (_cmdArray[1] != undefined) {
        this.data.selectIdx = _cmdArray[1] || 0;
      }
    }
    // 取得提示文字
    let _tip = this.getNameByType(_isCycle);
    // 底部按钮
    let _bottomBtns = this.getBottomBtns(_isCycle, _tip, this.data.from);
    this.setData({
      isCycle: _isCycle,
      typeStr: _tip,
      roleNum: app.globalData.roleNum, // 用户权限
      bottomBtns: _bottomBtns,
      from: this.data.from,
      selectIdx: this.data.selectIdx
    });
    // 清空定时数据
    app.globalData.timerList = [];
  },

  initNavTitle (title) {
    ability.setNavigationBarTitle(decodeURIComponent(title || ""))
  },

  getNameByType (isCycle) {
    return isCycle ? '循环情景' : '定时'
  },

  getBottomBtns (isCycle, _tip, fromStr) {
    if (fromStr == "scene") {
      let _array = [{
        text: "保存",
        className: "white bgF78D3F",
        handle: "bindSceneCycle"
      }];
      // 是否编辑
      if (this.editIndex >= 0) {
        _array.unshift({
          text: "解绑",
          className: "delBtn",
          handle: "unbindCycle"
        });
      }
      return _array;
    } else if (isCycle) {
      return [{
        text: "停止执行",
        className: "delBtn",
        handle: "stopCycle"
      }, {
        text: "添加" + _tip,
        className: "white bgF78D3F",
        handle: "addTimer"
      }]
    } else {
      return [{
        text: "添加" + _tip,
        className: "white bgF78D3F",
        handle: "addTimer"
      }]
    }
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 查询中
    this.setStatus(-2);
    // 临时缓存的定时数据
    this.timeList = [];
    // 显示等待条
    this.showLoading();
    // 查询网管定时数据
    this.queryList()
  },

  setStatus (value) {
    // 设置
    this.setData({
      status: value || 0 // 用于区分是查询中或者查询失败、还是查询结束 
    })
  },

  getTimeCycleList (array) {
    if (!array || array.length == 0) {
      return [];
    }
    // 是否场景中添加循环场景
    let _isFromScene = this.isFromScene;
    // 取得要过滤的设备
    let _filters = [];
    if (_isFromScene) {
      let _sceneDevs = app.globalData.sceneInfo?.devs || [];
      for (let sceneInfo of _sceneDevs) {
        let {cmd} = sceneInfo || {};
        if ((cmd instanceof Array) && cmd[0] == 0 && cmd.length == 2 && cmd[1] >= 0) {
          _filters.push(Number(cmd[1]) || 0);
        }
      }
    }
    console.log("getTimeCycleList _filters = ", _filters);
    // // 所有数据
    // let _devJson = app.globalData.devs || {};
    // // 取得所有设备对应的d: item
    // let _dMap = bizUtil.getDidMapInJson(_devJson);
    let _array = [];
    let _isCycle = this.isCycle();
    console.log("getTimeCycleList array, _isCycle = ", array, _isCycle);
    for (let item of array) {
      // 是否循环定时
      if (_isCycle) {
        let {idx, name, cnt} = item;

        // 如果是从场景中进入则过滤场景中的idx
        if (_filters.indexOf(Number(idx)) >= 0 && (this.editIndex < 0 || (this.editIndex >= 0 && (idx != this.data.selectIdx)))) {
          continue;
        }

        _array.push({
          name: name || "",
          time: "",
          repeatStr: this.getRepeatStrByCnt(cnt),
          idx: idx,
          // isCycle: true
        });
      } else {
        let {cmd, en, id, name, time, type} = item || {};
        // 重复次数的tip
        let {repeatTip, repeat, hour, minute} = this.getRepeatAndTimeByFormat(time);
        let _timeStr = this.getHHMMStr(hour, minute);
        _array.push({
          name,
          en,
          time: _timeStr, // 为了和定时数据适配，这里名称有歧义
          repeatStr: repeatTip
        });
      }
    }
    return _array;
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  // 使能点击
  actionClick (e) {

  },

  // 是否循环情景
  isCycle () {
    return this.type == "cycle"
  },

  addTimer () {
    this.jumpDetail(-1);
  },
  jumpDetail (index) {
    // 编辑还是新增
    let _str = (index >= 0) ? ("&index=" + index) : "";
    // 取得当前时间
    let {hour, minute} = Utils.getDateObj();
    // 判断当前是定时还是循环情景
    if (this.isCycle()) {
      let _len = app.globalData.timerList?.length || 0;
      // 判断最大16 => 15跳定时数据，有一条全部停止
      if (_len >= 15) {
        return this.toastMsg("循环情景最多15条");
      }
      // 取得当前的循环idx
      let _defaultIdx = bizUtil.getIdByArray(this.data.list, "idx");
      console.log("jumpDetail _defaultIdx = " , _defaultIdx);
      // 新增的定时数据
      app.globalData.timerInfo = {
        headObj: {
          scene_idx: _defaultIdx, // 情景索引
          cycle_cnt: 1, // 循环次数
          cmd_cnt: 0, // 命令个数
          name: "循环情景" + _defaultIdx
        },
        bodyList: [],
        // time: this.getTimerByRepeatAndTime({
        //   hour, minute
        // }),
        // // repeat: "", // 重复次数
        // n: "循环情景" + (_len + 1), // 执行时间 炫彩
        // en: 1, // 定时使能 默认打开
        // actions: [], // 执行设备或者场景
        // delay: 100 // 默认100ms
      };
      ability.navigateTo("../../cyclescene/detail/index?title=" + encodeURIComponent("创建循环情景") + _str)
      return;
    }

    // 编辑
    if (index < 0) {

      // 判断增加的最大条数
      if (this.maxCount <= this.data.list.length) {
        return this.toastMsg("循环定时最多" + this.maxCount + "条");
      }

      // 新增的定时数据
      app.globalData.timerInfo = {
        type: "period",
        // repeat: "", // 重复次数
        time: this.getTimerByRepeatAndTime({
          hour, minute
        }), // 执行时间
        en: 1, // 定时使能 默认打开
        name: "定时" + (this.data.list.length + 1),
        // actions: [], // 执行设备或者场景
      };
      console.log("jumpDetail app.globalData.timerInfo = ", app.globalData.timerInfo);
      // ability.navigateTo("../setTime/index?title=" + encodeURIComponent("添加定时") + _str);
      
      ability.navigateTo("../detail/index?title=" + encodeURIComponent("添加定时") + _str)
    } else {
      // 编辑的定时数据
      app.globalData.timerInfo = app.globalData.timerList[index];
      console.log("jumpDetail index = ", index);
      ability.navigateTo("../detail/index?title=" + encodeURIComponent("编辑定时") + _str)
    }
  },
  jumpTimerPage () {

  },

  itemSwitchChange (e) {
    // 当前值
    let {value} = e.detail;
    // 编辑的下标
    let {index} = e.currentTarget.dataset;
    console.log("itemSwitchChange _value, e = ", value, e);

    // 判断页面来源
    if (this.isFromScene) {
      // 场景绑定循环场景
      this.setData({
        selectIdx: this.data.list[index]?.idx
      })
      return;
    }

    this.enableTimer(Number(value), index);
  },
  // 修改定时使能
  enableTimer (enable, index) {
    let item = app.globalData.timerList[index];

    // 需要更新的内容
    let _obj = {
      type: item.type,
      id: item.id,
      en: enable
    };

    // 如果仅执行一次，需要把定时时间往后挪一天
    console.log("enableTimer enable = ", enable);
    if (enable) {
      let _onceTimeStr = this.getOnceTimeByTimer(item);
      console.log("enableTimer _onceTimeStr, item = ", _onceTimeStr, item);
      if (_onceTimeStr) {
        // 0_11_21_13_10_*_2022 仅一次才需要处理
        // 转成标准时间 2018/10/07 11:00:00
        let _formatDataStr = this.getOnceFormatStrByTimer(_onceTimeStr);
        console.log("enableTimer _formatDataStr = ", _formatDataStr);
        // 根据时区偏移
        let _dataStr = this.getSetTimeByZone(_formatDataStr);
        console.log("enableTimer _dataStr = ", _dataStr);
        _obj.time = _dataStr;
      }
    }

    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo || {});
    _devInfo.d = _devInfo.wd || _devInfo.d;
    // 域名
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    
    app.globalData.blsdk.gatewayTimer(_devInfo, {
      host: _groupStr, // 域名

      did: _devInfo.d,
      act: 2,
      timerlist: [_obj]
    }, (res) => {
      console.log("enableTimer res = ", res);
      if (!res || res.status != 0 || res.idlist[0] != item.id) {
        // 提示
        this.toastMsg("定时设置失败");
        // 失败时不更新数据
        let _str = `list[${index}]`;
        this.setData({
          [_str]: Number(!enable)
        });
      }
    });
  },
  itemClick (e) {

    // 如果是分享者
    if (this.data.roleNum == 2) {
      return;
    }
    // 选中的下标
    let {index} = e.currentTarget.dataset;

    // 如果是场景进入，则不进入
    if (this.isFromScene) {
      // 场景绑定循环场景
      this.setData({
        selectIdx: this.data.list[index]?.idx
      })
      return;
    }


    // 是否场景
    if (this.isCycle()) {
      let _item = this.data.list[index] || {};
      return this.getCycleDetail(_item.idx);
    }
    // 跳转定时详细
    this.jumpDetail(index);
  },
  // 执行循环场景
  actionCycle (e) {
    // todo 执行
    let {index} = e.currentTarget.dataset;
    let {idx} = this.data.list[index] || {};
    console.log("actionCycle idx = ", idx);
    // 执行
    this.actCycle(idx);
  },
  queryList (index = 0, count = 8) {
    let _isCycle = this.isCycle();
    console.log("queryTimer _isCycle = ", _isCycle);
    // 判断是循环场景还是定时
    if (_isCycle) {
      // 循环场景
      this.queryCycle(index, count);
    } else {
      // 定时
      this.queryTimer(index, count);
    }
  },
  // 场景是否在列表中
  checkCycleInArray (idx, list) {
    if (!list || idx == undefined) {
      return false;
    }
    let _filterArray = list.filter(item => item.idx == idx);
    return _filterArray.length;
  },
  // 场景解绑
  unbindCycle () {
    let _idx = this.editIndex;
    // 是否编辑
    if (_idx < 0) {
      return;
    }
    // 判断列表中是否有绑定的数据
    let _isExistCycle = this.checkCycleInArray(_idx, this.data.list);
    if (!_isExistCycle) {
      // 保存成功的处理
      this.deleteSceneCycle(_idx);
      // 成功的提示
      this.unbindSuccess();
      return;
    }

    let _sceneInfo = app.globalData.sceneInfo || {};
    let _gatewayInfo = app.globalData.devInfo || {};
    if (_gatewayInfo.a < 0 || _gatewayInfo.a == undefined) {
      ability.showToast({
        title: "该网关没有短地址"
      });
      return;
    }
    let _sidSDK = bizUtil.getSceneId(_sceneInfo.sid, _sceneInfo.rid);
    ability.showLoading({});
    app.globalData.blsdk.roomSceneDeleteWithDeviceAddr(_gatewayInfo.a, _sidSDK, (res) => {
      console.log("unbindCycle res = ", res);
      ability.hideLoading();
      if (res != 0) {
        ability.showToast({
          title: "解绑失败"
        })
        return;
      }
      // 保存成功的处理
      this.deleteSceneCycle(_idx);
      // 成功的提示
      this.unbindSuccess();
    });
  },
  // 解绑成功
  unbindSuccess () {
    ability.showToast({
      title: "解绑成功"
    });
    setTimeout(this.back, 1000);
  },
  back () {
    ability.navigateBack();
  },
  // 解绑
  deleteSceneCycle (index) {
    let _devInfo = app.globalData.devInfo || {};
    if (!_devInfo) {
      return;
    }
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    let _selectCycles = (_sceneInfo?.devs || [])[index || 0];
    if (!_selectCycles) {
      return;
    }
    _sceneInfo.devs.splice(index, 1);
  },
  // 场景绑定循环场景
  bindSceneCycle () {
    let _idx = this.data.selectIdx;
    if (_idx < 0) {
      ability.showToast({
        title: "请选择循环场景"
      });
      return;
    }
    let _sceneInfo = app.globalData.sceneInfo || {};
    let _gatewayInfo = app.globalData.devInfo || {};
    if (_gatewayInfo.a < 0 || _gatewayInfo.a == undefined) {
      ability.showToast({
        title: "该网关没有短地址"
      });
      return;
    }
    let _sidSDK = bizUtil.getSceneId(_sceneInfo.sid, _sceneInfo.rid);
    console.log("bindSceneCycle _sidSDK, _gatewayInfo, _idx = ", _sidSDK, _gatewayInfo, _idx);
    // 等待条
    ability.showLoading({});
    // 命令
    let _cmdArray = [0, _idx % 256];

    // ihg
    let _devInfo = Object.assign({}, _gatewayInfo);
    _devInfo.d = _devInfo.wd || _devInfo.d;

    app.globalData.blsdk.roomSceneAddWithCommand(_devInfo, _sidSDK, _cmdArray, (res) => {
      console.log("bindSceneCycle res = ", res);
      ability.hideLoading();
      if (res != 0) {
        ability.showToast({
          title: "保存失败"
        })
        return;
      }
      // 保存成功的处理
      this.saveSceneCycle(_idx, _cmdArray);
      ability.showToast({
        title: "保存成功"
      });
      setTimeout(this.back, 1000);
    });
  },
  // 保存成功的处理
  saveSceneCycle (idx, cmdArray) {
    let _devInfo = app.globalData.devInfo || {};
    if (!_devInfo) {
      return;
    }
    let _selectCycles = this.data.list.filter(item => item.idx == idx);
    if (_selectCycles.length == 0) {
      return;
    }
    // 选中的场景数据
    let _selectCycle = _selectCycles[0];

    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    // 把json的数据转string
    let _statusArray = cmdArray || [];
    let _tip = _selectCycle?.name || "";
    console.log("dealAfterSend = ", _statusArray, _tip, _sceneInfo, _devInfo);
    // 防止不存在数据
    _sceneInfo.devs = _sceneInfo.devs || [];

    // 判断数组中是否有包含设备d的
    let _oldIndex = this.indexInArrayByKey(_sceneInfo.devs, "d", _devInfo?.d, idx);
    console.log("dealAfterSend _oldIndex = ", _oldIndex);

    // 编辑的数据
    let _editIndex = this.editIndex;
    if (_editIndex >= 0) {
      // 编辑的场合
      if (_oldIndex < 0) {
        // 不存在
        (_sceneInfo.devs[_editIndex] || {}).tip = _tip;
        (_sceneInfo.devs[_editIndex] || {}).cmd = _statusArray;
        !!_devInfo.v && ((_sceneInfo.devs[_editIndex] || {}).v = _devInfo.v);
        !!_devInfo.t && ((_sceneInfo.devs[_editIndex] || {}).t = _devInfo.t);
      } else {
        // 有重复
        console.error("repeat edit");
      }
    } else {
      // 新增的场合
      if (_oldIndex < 0) {
        // 不存在
        _sceneInfo.devs.push({
          v: _devInfo.v,
          d: _devInfo.d,
          t: _devInfo.t,
          cmd: _statusArray,
          tip: _tip,
        });
      } else {
        // 有重复
        console.error("repeat add");
      }
    }
  },
  indexInArrayByKey (array, key, value, idx) {
    if (!array || array.length == 0) {
      return -1;
    } else if (!key) {
      return -1;
    }
    return array.findIndex(item => (item[key] == value) && (idx == (item?.cmd || [])[1]));
  },
  stopCycle () {
    this.actCycle("0"); // 停止执行
  },
  // 循环场景详细
  getCycleDetail (sceneIndex) {
    console.log("getCycleDetail");
    ability.showLoading({});

    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo);
    _devInfo.d = _devInfo.wd || _devInfo.d;

    app.globalData.blsdk.gatewayCycle(_devInfo, {
      "type": "detail",
      "cscene_idx": sceneIndex || "0"
    }, (res) => {
      ability.hideLoading();
      console.log("getCycleDetail res = ", res);
      let {headObj, bodyList} = res || {};
      if (!headObj && !bodyList) {
        ability.showToast({
          title: "查询失败" + (status || "")
        })
        return;
      }
      console.log("../../cyclescene/detail/index?index=" + headObj?.scene_idx);
      app.globalData.timerInfo = res;
      ability.navigateTo("../../cyclescene/detail/index?index=" + headObj?.scene_idx);
    });
  },
  // 停止循环场景
  actCycle (sceneIndex) {
    console.log("stopCycle");

    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo);
    _devInfo.d = _devInfo.wd || _devInfo.d;

    app.globalData.blsdk.gatewayCycle(_devInfo, {
      "type": "act",
      "cscene_idx": sceneIndex || "0"
    }, (res) => {
      let {status} = res || {};
      let _msgObj = status != 0 ? {
        title: "失败",
        icon: "error"
      } : {
        title: "成功",
        icon: "success"
      };
      this.toastMsg(_msgObj.title, _msgObj.icon);
    });
  },
  // 查询循环场景
  queryCycle (index, count) {
    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo);
    _devInfo.d = _devInfo.wd || _devInfo.d;
    console.log("queryCycle _devInfo = ", _devInfo);

    app.globalData.blsdk.gatewayCycle(_devInfo, {
      type: "query"
    }, (res) => {
      console.log("queryCycle res = ", res);
      let {list, status} = res || {};
      if (!list) {
        // 查询完毕
        this.queryComplete();

        let _status = status || -1;
        // 查询结束
        this.setStatus(_status == -5 ? -5 : -1);
        return;
        //  ability.showToast({
        //   title: "查询失败"
        // })
      }
      console.log("queryTimer list = ", list);
      this.dealQueryCycleData(list);
    });
  },
  getRepeatStrByCnt (cnt) {
    let _cnt = cnt || 0;
    if (_cnt == 0) {
      return "无限循环"
    } else if (_cnt == 1) {
      return "仅执行一次"
    } else {
      return "重复" + _cnt + "次";
    }
  },
  getCycleArrayByArray (array) {
    let _result = [];
    if (array.length == 0) {
      return _result;
    }
    for (let item of array) {
      let {idx, name, cnt} = item;
      _result.push({
        name: name || "",
        time: "",
        repeatStr: this.getRepeatStrByCnt(cnt),
        idx: idx
      })
    }
    return _result;
  },
  dealQueryCycleData (array) {
    // 查询完毕
    this.queryComplete();
    // 取得数据
    let _array = array || [];
    // let _dealArray = this.getCycleArrayByArray(array);
    // 根据时分排序
    let _sortList = _array.sort((a, b) => a.idx - b.idx);
    console.log("setListData _sortList = ", _sortList);
    // 数据
    app.globalData.timerList = _array;
    // 根据定时数据处理
    this.setListData(_sortList);
  },
  // 查询定时列表
  queryTimer (index, count) {
    // ihg
    let _devInfo = Object.assign({}, app.globalData.devInfo);
    _devInfo.d = _devInfo.wd || _devInfo.d;
    // 域名
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;
    
    app.globalData.blsdk.gatewayTimer(_devInfo, {
      host: _groupStr, // 域名

      did: _devInfo.d,					//设备did
      act: 3,
      type: "period",					//定时类型，比如period
      count: count, 					//查询数量
      index: index, 
    }, (res) => {
      console.log("queryTimer res = ", res);
      if (!res || res.status != 0) {
        // 查询完毕
        this.queryComplete();
        // 查询结束
        this.setStatus(-1);
        return;
      }
      this.dealQueryTimerData(res);
    });
  },
  // 返回的数据处理
  dealQueryTimerData (jsonData) {
    if (!jsonData) {
      // 数据不正确
      this.queryFail();
      return;
    }
    // 设备列表
    let _list = this.getListByZone(jsonData?.timerlist || []);
    this.timeList = this.timeList.concat(_list);
    // 条数限制 默认最大16条定时数据
    this.maxExistCount = jsonData?.total || 0;
    this.maxCount = (jsonData?.period_limits || [])[1] || 16;
    // 当前条数
    this.count = this.timeList.length || 0;
    // 当前查询下标
    this.startIndex = (jsonData?.index || 0) + _list.length;
    // 定时版本
    this.ver = jsonData?.ver || 0;
    // 是否继续查询
    if (jsonData?.total == 0 || _list.length == 0) {
      // 查询结束
      this.queryTimerSuccess();
    } else if (this.maxExistCount > this.count) {
      // 再次查询
      return this.queryList(this.startIndex);
    } else {
      // 查询结束
      this.queryTimerSuccess();
    }
  },
  // 根据时分排序
  sortByHHMM (array) {
    if (!array || array.length == 0) {
      return [];
    }
    return array.sort((a, b) => {
      let _aTime = (a?.time || "").split("_");
      let _bTime = (b?.time || "").split("_");
      if (_aTime.length > 2 && _bTime.length > 2) {
        return (_aTime[2] * 60 + Number(_aTime[1])) - (_bTime[2] * 60 + Number(_bTime[1]));
      }
      return false;
    })
  },
  // 设置列表
  setListData (list) {
    // 设置显示的定时数据
    let _list = this.getTimeCycleList(list || []);

    // 判断是否是场景绑定循环场景，如果是，则额外增加一个停止执行的命令
    if (this.isFromScene) {
      // 判断是否包含停止场景
      let _isStopExist = _list.filter(item => item.idx == 0);
      if (_isStopExist.length == 0) {
        _list.push({
          cnt: 0,
          idx: "0",
          name: "停止执行"
        })
      }
    }

    // 设置
    this.setData({
      list: _list, // this.data.list
      status: 0 // 有数据
    });
  },
  queryTimerSuccess () {
    // 查询完毕
    this.queryComplete();
    // 查询结束
    this.setStatus(0);
    // 根据时分排序
    let _sortList = this.sortByHHMM(this.timeList);
    console.log("setListData _sortList = ", _sortList);
    // 数据
    app.globalData.timerList = _sortList;
    // 根据定时数据处理
    this.setListData(_sortList);
  },
  queryFail () {
    // 查询完毕
    this.queryComplete();
    // 提示
    this.toastMsg("查询失败");
    // 查询结束
    this.setStatus(0);
  },
  // 提示错误信息
  toastMsg (str, icon) {
    if (!str) {
      return;
    }
    ability.showToast({
      title: str,
      icon: icon || "none"
    });
  },
  // 查询完毕之后的操作
  queryComplete () {
    // 隐藏等待条
    this.hideLoading();
  },
  showLoading (str) {
    ability.showLoading({
      title: str || ""
    })
  },
  hideLoading () {
    ability.hideLoading();
  },

  // onPullDownRefresh () {
  //   this.onRefresh();
  //   wx.stopPullDownRefresh()
  // },
  onRefresh () {

    // return this.addTimer();

    // 清空定时数据
    app.globalData.timerList = [];
    // 显示等待条
    this.showLoading();
    // 临时缓存的定时数据
    this.timeList = [];
    // 查询网管定时数据
    this.queryList()
  }
})