// device/gateway/timer/setTime/index.js
const timeBehavior = require("../../behavior/time")
const ability = require("../../../../utils/classDef/ability");
const app = getApp();
Page({
  behaviors: [timeBehavior],
  /**
   * 页面的初始数据
   */
  data: {
    // 头部的显示内容
    items: [],
    hours: [],
    minutes: [],
    current: [23, 59]
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title, index} = options || {};
    // 初始化天数，小时和分钟数据
    this.initTimeArray();
    // 设置标题
    this.setNavTitle(title);
    // 保存编辑的下标
    this.index = index == undefined ? -1 : index;
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(decodeURIComponent(title || ""))
  },
  initDataByTimer (timerInfo) {
    if (!timerInfo) {
      return;
    }
    let {time} = timerInfo;
    let {repeat, hour, minute} = this.getRepeatAndTimeByFormat(time);
    this.repeat = repeat;
    let _hmTime = hour + ":" + minute;
    console.log("initDataByTimer repeat, time = ", repeat, time);
    // 顶部条目
    this.data.items = this.getItems(repeat);
    // 渲染picker
    this.data.current = this.getDefaultTime(_hmTime);
    console.log("initDataByTimer this.data.current = ", this.data.current);
    // 渲染数据
    this.setData({
      items: this.data.items,
      current: this.data.current
    })
  },
  getItems (repeat) {
    return [{
      title: "重复",
      value: "",
      tip: this.getTipByRepeat(repeat)
    }];
  },
  getDefaultTime (timeStr) {
    if (!timeStr) {
      return [0, 0];
    }
    let _timeArray = timeStr.split(":");
    return _timeArray.map(item => Number(item));
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 根据定时数据初始化渲染数据
    this.initDataByTimer(app.globalData.timerInfo);
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  initTimeArray () {
    // 初始化小时
    this.data.hours = this.getArray(0, 24);
    // 初始化分钟
    this.data.minutes = this.getArray(0, 60);
    this.setData({
      hours: this.data.hours,
      minutes: this.data.minutes
    })
  },
  getArray (min = 0, max = 60) {
    let _array = [];
    for (let i = min; i < max; i++) {
      _array.push(i);
    }
    return _array
  },

  bindChange (e) {
    let _vals = e.detail.value;
    console.log("bindChange _vals = ", _vals);
    let _valueArray = _vals || [];
    // 如果永久选中，则取消永久的选中
    this.data.current = _valueArray;
    // app.globalData.timerInfo.time = _valueArray.join(":");
  },

  // 下一步
  save () {
    // 取得设置的小时和分钟
    let _current = this.data.current;
    let _hour = _current[0];
    let _minute = _current[1];
    app.globalData.timerInfo.time = this.getTimerByRepeatAndTime ({
      hour: _hour,
      minute: _minute,
      week: this.repeat
    });
    console.log("save app.globalData.timerInfo, this.repeat = ", app.globalData.timerInfo, this.repeat);
    // 跳转页面
    this.back();
    // ability.redirectTo("../detail/index?title=" + encodeURIComponent("创建定时"))
  },
  back () {
    ability.navigateBack();
  },

  itemClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.items[index];
    if (index == 0) {
      ability.navigateTo("../repeat/index")
    }
  }
})