// miniprogram/device/light/control/index.js
// import { textSpanIsEmpty } from "typescript";
import ability from "../../../utils/classDef/ability";
import bizUtil from '../../../utils/bizUtil';
const app = getApp();

Page({
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: ""
      }
    },
    // 设备状态
    devStatus: {
      enumIdx: -1, // 颜色下标
    },
    // 是否群控
    isGroup: false,
    // 固定颜色
    commonColors: [],
    // 常用颜色
    fixColors: [{
      color: '#ffffff',
      text: '', // W
      className: '' // whiteEnum semibold center
    }, {
      // color: '#F7A36A'
      color: '#FF0000'
    }, {
      // color: '#F0846A'
      color: '#00FF00'
    }, {
      // color: '#CD7EFE'
      color: '#0000FF'
    }, {
      color: '#FFFF00'
    }],
    // canvas尺寸
    canvas: {
      pageX: 0, // canvas的左侧顶点位置
      pageY: 0, // canvas的左侧顶点位置
      widthRpx: 466, // rpx,
      heightRpx: 466, // rpx
      padL: 38, // 由于圆需要放到外面，所以需要增大thumb大小
      padR: 38, // 由于圆需要放到外面，所以需要增大
      multip: 2, // rpx与px的比例，默认为2
      centerX: 0, // 中心点横坐标
      centerY: 0, // 中心点纵坐标
      r: 0, // 半径
      circleWidth: 40 // 圆环宽度 rpx
    },
    thumb: {
      bgColor: '#0069FF',
      x: 100,
      y: 100,
      // angle: 235 // 默认在第三象限，角度
    },
    groupIds: [], // 批量控制
    OnlyCW: false, // 默认不是cw
    isOnlyRgb: false, // 默认不是只有rgb
    // isOnlyRgbw: false, // 是否只有rgbw,默认没有

    selectImg: {}, // 选中的按钮（需要变颜色）
    controlList: [], // 底部按钮

    // 色温配置
    cwSliders: {
      leftImg: "../../static/img/common/cDel.png",
      rightImg: "../../static/img/common/cAdd.png",
      bgImg: "../../static/img/common/temp.png",
      leftClass: "cDown",
      rightClass: "cUp",
      bgClass: "tempImg",
      width: 524, // rpx
      percent: 50, // 百分比
      minPercent: 0,
      // minVal: 0,
      // maxVal: 255,
      gradient: false, // 默认不支持滑动时调节
      labelTxt: "色温",
      unit: "K",
      minVal: 2700,
      maxVal: 6500,
    },
    // 亮度配置
    brightSliders: {
      leftImg: "../../static/img/common/bright_down.png",
      rightImg: "../../static/img/common/bright_up.png",
      leftClass: "bDown",
      rightClass: "bUp",
      width: 516, // rpx
      percent: 100, // 百分比
      labelTxt: "亮度",
      minVal: 2,
      maxVal: 127,
      minPercent: 1, // 最小百分比
      gradient: false // 默认不支持滑动时调节
    },
    onActive: "",
    offActive: "",
    rgbUp: "",
    rgbDown: "",
    bleUnavailable: false,
    scales: "none", // 适配小屏幕手机，暂时方案为 整体缩小
    abilitys: {}, // 设备控制的能力
    timer: null, // 定时器

    codewarmList: [], // 调光
    warmcodeIndex: -1, // 默认不选中

    // 显示列表详情
    showDetail: false, 
    detailList: [{
      title: "设备详情",
      bindTap: "jumpDetail",
      key: "detail"
    }, {
      title: "功能设置",
      bindTap: "jumpFuncSet",
      key: "funcset"
    }]
  },
  onLoad(options) {
    // // 标题
    // this.title = options?.title;
    // 批量控制的addr列表
    this.data.groupIds = app.globalData.groupIds || [];
    // 初始化功能
    this.initDevFunctions();
    // 初始化常用颜色
    this.data.commonColors = this.initColors();

    this.setData({
      isAllBatch: this.isAllBatchCtrl(),
      commonColors: this.data.commonColors
    })

    // // 判断页面是否群组，如果是群组，设备列表是否为空
    // let {gid, dids} = app.globalData.groupInfo || {};
    // if (this.isGroupCtrl() && gid >= 0 && dids?.length == 0) {
    //   // 跳转页面
    //   return ability.navigateTo("/groups/manage/index?remote=" + 0);
    // }
  },
  // 常用颜色
  initColors () {
    // 颜色数据
    let _colorArray = [];
    // 家庭id
    let _familyId = app.globalData.familyInfo?.id || 0;
    // 通用颜色数据
    let _colors = app.globalData.colors;
    if (_colors instanceof Array) {
      _colorArray = _colors.concat([]);
    } else {
      _colorArray = _colors[_familyId] || ((_colors?.common || []).concat([])) || [];
    }
    console.log("initColors _colorArray, _colors, _familyId = ", _colorArray, _colors, _familyId)
    // this.data.commonColors = app.globalData.colors || [];
    return _colorArray
    // this.setData({
    //   commonColors: _colorArray
    // })
  },
  onReady () {
    // 初始化比例
    this.initMultip();
    // 取得canvas的位置
    this.getCanvasOffset();
    // 绘制图案
    this.drawProgressbg();
    // 取得真实高度
    this.getRealContentHeight();

    // 角度
    let _lastAngle = (this.data.devStatus || {}).angle;
    let _angle = _lastAngle == undefined ? 235 : _lastAngle;
    let _color = this.data.devStatus.angleColor || '#0069FF';
    console.log("_angle = ", _angle)
    // 初始化thumb坐标信息
    this.initThumbPosition(_angle, _color);
  },
  // 取得真实的高度
  getRealContentHeight () {
    // 根据尺寸取得实际长度
    const query = wx.createSelectorQuery()
    query.select('.circleContainer').boundingClientRect()
    query.select('#cwSlider').boundingClientRect()
    query.select('.commonContainer').boundingClientRect()
    query.select('.fixedContainer').boundingClientRect()
    query.select('#brightSlider').boundingClientRect()
    query.select('.bottom-buttons').boundingClientRect()
    query.exec((res) => {
      let _sizeObj = this.getAllHeightFromArray(res);

      // 页面内容高度
      let _contentHeight = app.globalData.customPageSize?.contentHeight || 0;
      console.log("getRealContentHeight res, _sizeObj, _contentHeight = ", res, _sizeObj, _contentHeight);

      // 如果大于页面高度 底部需要有20像素的留白
      let _realHeight = (_sizeObj.height + _sizeObj.seperator) + 30;
      if (_realHeight > _contentHeight) {
        let _percent = _contentHeight / (_realHeight + _contentHeight * 0.1);
        this.setData({
          scales: 'scale(' + _percent + ')'
        })
      }
    });
  },
  getAllHeightFromArray (array) {
    if (!array) {
      return 0;
    }
    // 总高度
    let _height = 0;
    let _bottom = 0;
    let _seperator = 0;
    // 遍历数据
    let _length = array.length;
    let _itemPositionY = 0;
    for (let index in array) {
      let item = array[index];
      if (!item) {
        continue;
      }
      _height += item.height;
      _bottom = Math.max(item.bottom, _bottom);
      if (index > 0 && _length > 1 && index < (_length - 1)) {
        let _y = item.top - _itemPositionY;
        if (_y > 0) {
          _seperator += _y;
        }
      };
      // 位置
      _itemPositionY = (item.top + item.height);
    }
    return {
      height: _height,
      bottom: _bottom,
      seperator: _seperator
    };
  },
  // 取得canvas的位置
  getCanvasOffset () {
    // 根据尺寸取得实际长度
    const query = wx.createSelectorQuery()
    query.select('#lightBg').boundingClientRect()
    // query.select('#lightCanvas').fields({ node: true, size: true })
    // query.selectViewport().scrollOffset()
    query.exec((res) => {
      console.log("onReady res = ", res);
      let _size = (res || [])[0];
      if (!_size) {
        return;
      }
      this.data.canvas.pageX = _size.left;
      this.data.canvas.pageY = _size.top;
      this.data.canvas.width = _size.width;
      this.data.canvas.height = _size.height;
    })
  },
  initMultip () {
    this.data.canvas.multip = app.globalData.customPageSize.multip;
  },
  // 取得当前页面控制设备列表数组
  getAllCtrlDevsArray () {
    // 是否群组设备
    let {dids} = app.globalData.groupInfo || {};
    if (this.isGroupCtrl()) {
      // 固定群组
      // 取得房间下所有的d对应的详细信息
      let _didMaps = bizUtil.getDidMapInJson(app.globalData.devs);
      // 群组设备列表
      let _groupDevs = dids || [];
      // 群组设备列表详细
      return _groupDevs.map(did => _didMaps[did]);
    } else if (app.globalData.tgroupId && (app.globalData.tmpData?.tdevs)) {
      // 临时群组
      return app.globalData.tmpData?.tdevs || [];
    } else {
      let _devInfo = app.globalData.devInfo;
      if (_devInfo) {
        // 单个设备控制
        return [_devInfo];
      } else if (this.isBatchCtrl()) {
        // 5个一组的批量控制
        return app.globalData?.tmpData?.tdevs || [];
      } else {
        // 房间控制
        return this.getDevArrayByRoomIdInJson(app.globalData.devs, (app.globalData.roomInfo || {}).id)
      }
    }
  },
  initDevFunctions () {
    // 初始化设备通用功能
    let _devStatus = {
      angleColor: '#0069FF',
      angle: 235, // 默认角度为235
      enumIdx: -1, // 颜色下标
      pwr: 1,
      brightness: (this.data.brightSliders?.percent || 100) * 1.27
    };
    // 存在的设备列表
    let _devExistArray = this.getAllCtrlDevsArray();
    // 是否只有rgbw设备
    this.isOnlyRgbw = _devExistArray.filter(item => bizUtil.isRGBW(item?.t))?.length == _devExistArray.length;
    console.log("initDevFunctions _devExistArray, isOnlyRgbw = ", _devExistArray, this.isOnlyRgbw);
    // 判断设备单个还是批量或者群组
    let {maxStatus, versions} = this.getFuncList(_devExistArray) || {};
    // 赋值给devStatus
    _devStatus = Object.assign({}, maxStatus, _devStatus);
    console.log("last devStatus, maxStatus = ", _devStatus, maxStatus);
    // 是否rgb设备
    this.data.isOnlyRgb = this.checkOnlyRgb(_devStatus); // _devInfo ? this.isRgbByPid(app.globalData.devInfo.t) : this.checkOnlyRgb();
    this.data.OnlyCW = this.checkOnlyCW(_devStatus);
    // 判断是否支持调光调色灯
    if (_devStatus.supportCodewarm) {
      this.initCodewarmList();
    }
    console.log("this.data.isOnlyRgb = ", this.data.isOnlyRgb);
    // 取得上次保存的值
    let _valStatus = this.getDevStatus(true);
    console.log("initDevFunctions _valStatus = ", _valStatus);
    for (let key in _devStatus) {
      let _val = _valStatus[key];
      (_val != undefined) && (_devStatus[key] = _val);
    }
    // 多保存一个设备状态用于亮度不改变
    (_valStatus.originBrightness != undefined) && (_devStatus.originBrightness = _valStatus.originBrightness);

    // 保存设备状态
    app.globalData.devStatus = this.data.devStatus = _devStatus;
    // 判断上一次的状态 并更新数据
    (_valStatus.brightness != undefined) && (this.data.brightSliders.percent = this.getPercentByRealValue(_valStatus.originBrightness || _valStatus.brightness, this.data.brightSliders));

    // 保存色温百分比
    let _cwP = _valStatus.cwP;
    if (_cwP != undefined) {
      // let _value = Math.round(_cwP / 100 * (6500 - 2700) + 2700);
      this.data.cwSliders.percent = _cwP;
    } else {
      (_valStatus.c != undefined) && (this.data.cwSliders.percent = this.getPercentByRealValue(((255 - _valStatus.c) || 0) / 255 * (this.data.cwSliders.maxVal - this.data.cwSliders.minVal) + this.data.cwSliders.minVal, this.data.cwSliders));
    }

    console.log("this.data.cwSliders.percent = ", this.data.cwSliders.percent, _valStatus.c, _cwP);
    // 保存能力集合
    this.data.abilitys = bizUtil.getMaxAbilityByVs(versions) || {};
    console.log("this.data.abilitys, versions = ", this.data.abilitys, versions);
    // 能力集合
    let {gradient: _gradient, minBrightness, remote, alexaSet, configLock, noDisturb, lingdong, slowUpDown} = this.data.abilitys;

    // 如果没有设置的状态则不显示功能设置按钮
    if (!remote && !alexaSet && !configLock && !noDisturb && !lingdong && !slowUpDown) {
      console.log("all zero");
      // 没有功能设置页面
      this.data.detailList = this.data.detailList.slice(0, 1);
    } else if (this.isAllCtrl() || this.isGroupCtrl()) {
      this.data.detailList[0].title = "群组详情";
      if (!configLock && !noDisturb && !lingdong && !slowUpDown) {
        console.log("no set");
        // 没有功能设置页面
        this.data.detailList = this.data.detailList.slice(0, 1);
      }
    }

    // 更新滑块状态
    this.data.brightSliders.gradient = _gradient;
    this.data.cwSliders.gradient = _gradient;

    console.error("minBrightness = ", minBrightness, this.data.abilitys, this.data.brightSliders);
    // 更新亮度
    !!minBrightness && (this.data.brightSliders.minVal = minBrightness.value, this.data.brightSliders.minPercent = minBrightness.percent);
    this.setData({
      isOnlyRgb: this.data.isOnlyRgb,
      OnlyCW: this.data.OnlyCW,
      devStatus: this.data.devStatus,
      brightSliders: this.data.brightSliders,
      cwSliders: this.data.cwSliders,
      detailList: this.data.detailList
    })
  },
  // 取得滑块值的百分比
  getPercentByRealValue (realValue, obj) {
    console.log("getPercentByRealValue realValue, obj = ", realValue);
    if (!obj) {
      return 0;
    }
    let {minVal, maxVal} = obj;
    let _minVal = minVal || 0;
    let _maxVal = maxVal || 100;
    if (realValue < _minVal) {
      return _minVal;
    } else if (realValue > _maxVal) {
      return _maxVal;
    }
    // 这里 (73.25 - 2) / (127 - 2) * 100 = 56.999999999 计算器算出来是57
    let _percent = (realValue - _minVal) / (_maxVal - _minVal);
    let _pointInt = _percent * 100 - parseInt(_percent * 100);
    let _result = parseInt(_percent * 100) + ((_pointInt > 0.99) ? 1 : 0);
    console.log("_pointInt, _percent, _result = ", _pointInt, _percent, _result);
    return _result;
  },
  onShow () {
    // 如果是群组，因为设计到群组设备管理，所以这里也需要初始化功能
    if (this.isGroupCtrl()) {
      this.initDevFunctions();
    }
    
    // 重置选中的
    this.data.selectImg = {};
    // 批量控制的addr列表
    this.data.groupIds = app.globalData.groupIds || [];
    // 初始化设备名称
    // this.initDevName();
    // 获取设备状态
    this.setDevStatus(app.globalData.devStatus);
    // this.data.devStatus = app.globalData.devStatus;
    // this.getDevStatus(false);
    // 初始化controlList
    this.initPageByGroup();
    // 未选中
    this.showDetailDialog(false);
  },
  setDevStatus (status) {
    this.data.devStatus = status || app.globalData.devStatus;
    console.log("setDevStatus status = ", status);
    this.setData({
      devStatus: this.data.devStatus
    })
  },
  initThumbPosition (angle, color) {
    // 取得角度的坐标位置
    let {x, y} = this.getPositionByAngle(angle);
    // 取得角度对应的颜色
    // this.initThumbColorByAngle(x, y);
    this.data.thumb.x = x;
    this.data.thumb.y = y;
    this.data.thumb.bgColor = color;
    console.log("initThumbPosition x, y, this.data.thumb.angle = ", x, y, this.data.thumb.angle);
    this.setData({
      thumb: this.data.thumb
    })
  },
  onUnload () {
    // 页面销毁 如果设置过百分比和最佳位置则保存设备状态
    this.saveDevStatus();
    // 保存常用颜色
    this.saveColorsByFamily();
    // 清除定时器
    this.clearTimer(this.data.timer);
    // 清除房间控制的缓存数据
    !!app.globalData?.tmpData?.roomInfo && (delete app.globalData.tmpData.roomInfo);
  },
  saveColorsByFamily () {
    // 家庭id
    let _familyId = app.globalData.familyInfo?.id || 1;
    // 颜色数据
    let _obj = app.globalData.colors;
    let _result = {};
    if (_obj instanceof Array) {
      _result.common = _obj.concat([]);
      _result[_familyId] = this.data.commonColors || [];
    } else {
      _result = _obj;
      _result[_familyId] = this.data.commonColors || [];
    }
    // let _array = this.data.commonColors || [];
    app.globalData.colors = _result;
    console.log("saveColorsByFamily _result = ", _result);
    // 保存到缓存中
    app.setBizStorage("colors", _result);
  },
  // 区分群控页面
  initPageByGroup () {
    let _isGroup = (app.globalData.devInfo == undefined);
    this.data.isGroup = _isGroup;
    // 初始化控制列表
    let _bottomBtns = this.getControlList(_isGroup);
    // 这里做个处理 去除详情
    this.data.controlList = _bottomBtns.filter(item => item.tap != "jumpDetail");
    
    // 初始化标题
    let _title = this.getDevName(_isGroup);
    console.log("initPageByGroup _title = ", _title);
    // this.data.navObj.center.text = _title;
    ability.setNavigationBarTitle(_title);

    // 设置数据
    this.setData({
      controlList: this.data.controlList,
      navObj: this.data.navObj,
      isGroup: _isGroup
    })
  },
  // 设置选中的key
  setKeySelect (key) {
    if (!key) {
      return;
    }
    this.data.selectImg = {};
    this.data.selectImg[key] = true;
  },
  // 根据key确定是否选中
  isSelectedByKey (key) {
    if (!key) {
      return false;
    }
    return this.data.selectImg[key];
  },
  // 根据数据区分
  getControlList (isGroup) {
    // 是否选中
    let _patternSel = this.isSelectedByKey("pattern");
    let _detailSel = this.isSelectedByKey("detail");
    // 控制列表
    let _controlList = [
      {
        imgPath: "../../img/light/pattern" + (_patternSel ? '_sel' : '') + ".png",
        text: "模式",
        tap: "jumpPattern",
        className: _patternSel ? "bgF78D3F white" : "bgWhite color666",
        imgClassName: 'patternImg'
      }, {
        imgPath: "../../img/light/detail" + (_detailSel ? '_sel' : '') + ".png",
        text: "详情", // 设备
        tap: "jumpDetail",
        className: _detailSel ? "bgF78D3F white" : "bgWhite color666",
        imgClassName: 'detailImg'
      }
    ];

    // 群组控制
    if (this.isGroupCtrl()) {
      // _controlList[1].text = "详情";
      return _controlList;
    }

    // 设备状态
    let _devStatus = this.data.devStatus;
    // 设备能力
    let {sleep, timer, cAndcwWhite, xuancai, mic} = this.data.abilitys || {};
    // 批量控制
    let _isBatchCtrl = this.isBatchCtrl();
    console.log("getControlList sleep, _devStatus, isGroup, _isBatchCtrl = ", sleep, _devStatus, isGroup, _isBatchCtrl);
    if (isGroup) {
      if (_isBatchCtrl) {
        // 老的批量不需要有模式
        // return _devStatus.r != undefined ? [_controlList[0]] : [];
        return [];
      } else {
        // 房间控制、临时群组、固定群组 改成显示详情页面
        console.log("getControlList groupInfo, tgroupId = ", app.globalData.groupInfo, app.globalData.tgroupId)
        return (cAndcwWhite || sleep) ? (app.globalData.tgroupId > 0 ? _controlList.slice(0, 1) : _controlList) : [_controlList[1]];
      }
    } else {
      // 只有单个设备才有定时
      let _array = (_devStatus.r == undefined && _devStatus.c == undefined) ? ((sleep || cAndcwWhite) ? _controlList : [_controlList[1]]) : _controlList;

      // 如果ability支持定时 且 需要非分享者权限，则加入数组的第二个
      // let _roleNum = app.globalData?.roleNum || 0;
      // && _roleNum != 2
      if (timer) {
        let _timerItem = {
          imgPath: "../../img/light/timer.png",
          text: "定时",
          tap: "jumpTimer",
          className: _detailSel ? "bgF78D3F white" : "bgWhite color666",
          imgClassName: 'timerImg'
        };
        // 把定时插入到模式和详情的中间
        if (_array.length == 1) {
          // 只有详情
          _array.unshift(_timerItem);
        } else {
          _array.splice(1, 0, _timerItem);
        }
      }

      // 是否支持拾音
      if (mic) {
        let _timerItem = {
          imgPath: "../../img/light/mic.png",
          text: "识音",
          tap: "jumpMic",
          className: _detailSel ? "bgF78D3F white" : "bgWhite coloraaa",
          imgClassName: 'micImg'
        };
        _array.push(_timerItem);
      }

      // 炫彩
      if (xuancai) {
        let _index = _array.findIndex(item => item.tap == "jumpPattern");
        if (_index >= 0) {
          // _array[_index].text = "炫彩";
          _array[_index].tap = "jumpXuancai";
        }
        // _array.push({
        //   imgPath: "../../img/light/pattern.png",
        //   text: "炫彩",
        //   tap: "jumpXuancai",
        //   className: _patternSel ? "bgF78D3F white" : "bgWhite coloraaa",
        //   imgClassName: 'patternImg'
        // })
      }
      
      return _array;
    }
  },
  getDevStatus (isFirst) {
    // 取得当前设备的状态
    let _devStatus = app.globalData.devStatus;
    // 第一次
    if (isFirst) {
      // 取得所有设备保存的状态数据
      let _allDevStatus = app.globalData.allStatus || {};
      this.data.allDevStatus = _allDevStatus;
      // console.log("getDevStatus storage _allDevStatus = ", _allDevStatus, app.globalData.devInfo, app.globalData.groupIds, this.data.groupIds);
      // 根据设备did+短地址获取设备状态
      let _dev = app.globalData.devInfo || {};
      // 如果是批量
      if (this.isBatchCtrl()) {
        console.log("batchcontrol");
        // 如果是批量
        _devStatus = _allDevStatus['batch'] || {};
      } else if (this.isGroupCtrl()) {
        // 这里按照目前的逻辑 需要放在this.isAllCtrl判断之前
        let {gid} = app.globalData.groupInfo || {};
        // 如果是全屋
        _devStatus = _allDevStatus[gid] || {}; 
      } else if (this.isAllCtrl()) {
        console.log("allcontrol");
        // 如果是全屋
        _devStatus = _allDevStatus['all'] || {};
      } else {
        console.log("singlecontrol");
        // 设备状态
        _devStatus = _allDevStatus[_dev.d + "_" + _dev.a] || {};
      }
    }
    // 取得当前设备的状态
    return _devStatus || {};
  },
  saveDevStatus () {
    // 保存的设备状态
    let _allDevStatus = this.data.allDevStatus || {};
    // 如果是批量
    if (this.isBatchCtrl()) {
      console.log("saveDevStatus batchctrl")
      // 如果是批量
      _allDevStatus["batch"] = this.data.devStatus;
    } else if (this.isGroupCtrl()) {
      // 这里按照目前的逻辑 需要放在this.isAllCtrl判断之前
      console.log("saveDevStatus groupCtrl")
      let {gid} = app.globalData.groupInfo || {};
      // 如果是全屋
      _allDevStatus[gid] = this.data.devStatus;
      console.log("openLightDev gid, this.data.devStatus = ", gid, this.data.devStatus);
    } else if (this.isAllCtrl()) {
      console.log("saveDevStatus allCtrl")
      // 如果是全屋
      _allDevStatus["all"] = this.data.devStatus;
    } else {
      console.log("saveDevStatus singleCtrl")
      let _dev = app.globalData.devInfo;
      // 设备状态
      _allDevStatus[_dev.d + "_" + _dev.a] = this.data.devStatus;
    }
    app.globalData.allStatus = _allDevStatus;
    // 保存到缓存中
    app.setBizStorage("devStatus", _allDevStatus);
  },
  // 是否群组控制  固定群组
  isGroupCtrl () {
    return !!app.globalData.groupInfo && !!app.globalData.tgroupId;
  },
  getDevName (isGroup) {
    console.log("getDevName isGroup = ", isGroup);
    if (app.globalData?.tmpData?.roomInfo != undefined) {
      return app.globalData?.tmpData?.roomInfo?.n || "";
    }
    // 补丁+1
    if (this.isGroupCtrl()) {
      // 固定群组
      return app.globalData.groupInfo?.n || "";
    }
    // 补丁
    if (isGroup && app.globalData.tgroupId) {
      // 临时group存在
      return "批量控制"
    }
    return isGroup ? (this.data.groupIds.length > 0 ? "批量控制" : ((app.globalData.devInfo?.n || app.globalData.roomInfo.n || '房间控制'))) : (app.globalData.devInfo || {}).n;
  },

  /***************** 设备控制相关 ***************/
  showLoading (str) {
    ability.showLoading({
      title: str || ""
    })
  },
  hideLoading () {
    ability.hideLoading();
  },

  addAngle () {
    this.dealAngle(1);
  },
  setDataByKey (key, value) {
    this.data[key] = value;
    this.setData({
      [key]: value
    })
  },
  delAngle () {
    this.dealAngle(-1);
  },
  dealAngle (value, hideAction, notSend) {
    // 设置按钮
    if (!hideAction) {
      console.error("dealAngle hideAction = ", hideAction);
      this.setDataByKey(value > 0 ? "rgbUp" : "rgbDown", true);
      setTimeout(() => {
        this.setDataByKey(value > 0 ? "rgbUp" : "rgbDown", false);
      }, 100);
    }

    // 取得0-360之间的角度
    let _angle = (360 + this.data.devStatus.angle + value) % 360;
    console.log("dealAngle _angle = ", _angle);
    // 根据角度取得坐标
    let _position = this.getPositionByAngle(_angle);
    console.log("dealAngle _position = ", _position);
    this.data.thumb.x = _position.x;
    this.data.thumb.y = _position.y;
    // this.data.thumb.angle = _angle;
    // 保存角度
    this.data.devStatus.angle = _angle;
    // 取得像素
    this.getImageData(_angle, _position.x, _position.y, (rgbData, sendData) => {
      let data = this.dealRgbPanel(rgbData);
      if (!data) {
        return;
      }
      let _colorStr = '#' + data[0].toString(16).padStart(2, '0') + data[1].toString(16).padStart(2, '0') + data[2].toString(16).padStart(2, '0');
      console.log("_colorStr = ", _colorStr);
      let _enumIdx = this.data.devStatus.enumIdx;
      let _commonColors = this.data.commonColors || [];
      // 保存颜色
      if (_enumIdx >= 0 && _commonColors[_enumIdx]) {
        (_commonColors[_enumIdx] || {}).color = _colorStr;
        this.setData({
          commonColors: this.data.commonColors
        });
      }

      this.data.thumb.bgColor = 'rgb('+data[0]+','+data[1]+','+data[2] + ')';
      // 保存颜色
      this.data.devStatus.angleColor = this.data.thumb.bgColor;
      this.setData({
        thumb: this.data.thumb
      })

      // 不需要发送
      if (notSend) {
        return;
      }

      // 数据处理
      let _sendRGB = sendData || this.dealRgbSend(data);
      // 发送指令
      this.actDev({
        pwr: 1,
        // brightness: this.data.devStatus.brightness || 127,
        r: _sendRGB[0],
        g: _sendRGB[1],
        b: _sendRGB[2]
      });
    });
  },

  // 由于色盘不准，所以处理一下获取到的颜色
  dealRgbPanel (data) {
    if (!data || data.length < 3) {
      return false;
    }
    let _r = data[0], _g = data[1], _b = data[2];
    if (_r == 0 && _g == 0 && _b == 0) {
      return false;
    }
    // console.error("dealRgbPanel _r, _g, _b = ", _r, _g, _b);
    // if ((255 - _g) <= 0 && _r <= 10 && _b <= 21) {
    //   console.error("green");
    //   return [0, 255, 0];
    // } else if ((255 - _b) < 22 && _r <= 5 && _g <= 17) {
    //   console.error("blue");
    //   return [0, 0, 255];
    // }
    return data;
  },
  // 发送rgb的处理
  dealRgbSend (data) {
    if (!data || data.length < 3) {
      return false;
    }
    let _r = data[0], _g = data[1], _b = data[2];
    // 增加逻辑，realR = r * 255 / (R+G+B)
    let _p = 255 / (_r + _g + _b);
    let _realR = parseInt(_r * _p);
    let _realG = parseInt(_g * _p);
    let _realB = parseInt(_b * _p);

    return [_realR, _realG, _realB];
  },

  clickOpen() {
    this.setColorIndex(-1);
    // console.log("clickOpen this.data.devStatus = ", this.data.devStatus, this.data.devStatus.brightness);
    this.onlyPwrCtrl(this.data.devStatus.brightness || 127);
  },
  clickClose(){
    this.setColorIndex(-1);
    this.onlyPwrCtrl(0);
  },
  // 只有控制出发
  onlyPwrCtrl (brightness) {
    console.log("onlyPwrCtrl brightness = ", brightness);
    let _params = {
      brightness: brightness
    };
    if (brightness > 0) {
      _params.pwr = 1;
    }
    // 动画效果
    this.setClickAction(brightness);
    setTimeout(() => {
      // 动画效果
      this.setClickAction(brightness, true);
    }, 100);
    // 判断是否批量控制
    let _isBatchCtrl = this.isBatchCtrl();
    if (_isBatchCtrl && brightness > 0) {
      _params.r = _params.g = _params.b = _params.c = _params.w = 0;
    }
    this.actDev(_params);
  },
  setClickAction (isOpen, isReset) {
    // 动画效果
    let _obj = {};
    let _val = isReset ? '' : 'pwrActive';
    isOpen?(_obj.onActive=_val,_obj.offActive=''):(_obj.offActive=_val,_obj.onActive='');
    console.log("setClickAction _obj = ", _obj);
    this.setData(_obj);
  },

  // 根据房间id取得json中设备集合
  getDevArrayByRoomIdInJson (json, roomId) {
    let _devs = [];
    if (!json) {
      return _devs;
    }
    if (roomId == 0) {
      // 全屋设备
      _devs = bizUtil.getAllDevsFilter(json) || [];
    } else if (roomId % 32 == 0) {
      // 取得楼层数据
      let _devJson = bizUtil.getDevsArrayByFloor(json, bizUtil.getFloorByRoomId(roomId)) || {};
      _devs = bizUtil.getAllDevsFilter(_devJson);
    } else {
      // 全屋设备
      _devs = bizUtil.getAllDevsFilter(json, roomId) || [];
    }
    // 取得照明的设备
    let {light} = bizUtil.getAllPidByArray(_devs);
    console.log("getFuncList _devs = ", _devs);
    return light || [];
  },

  // 根据品类取得功能集合
  getFuncList (devs) {
    // 被控设备集合
    let _devs = devs || [];
    
    // 最终集合
    let versions = [], minStatus= {}, maxStatus = {};
    // let _minCount = 20, _maxCount = -1;
    // 遍历数据取得集合
    for (let index in _devs) {
      let {t, v} = _devs[index] || {};
      // 如果版本号v不存在，则continue
      if (v == undefined) {
        continue;
      }

      // 如果版本号一致，则continue
      if (versions.indexOf(v) >= 0) {
        continue;
      }
      // 添加版本分支集合
      versions.push(v);
      // 取得能力集
      let _devStatus = bizUtil.getFuncByPid({}, t);
      // 添加最大能力集
      Object.assign(maxStatus, _devStatus);
    }
    console.log("getFuncList maxStatus = ", maxStatus);
    return {
      versions: versions, // 版本能力集合
      maxStatus: maxStatus
    };
  },
  // 是否只包含rgb设备
  checkOnlyRgb (status) {
    if (!status) {
      return false;
    }
    return status.c == undefined && status.w == undefined && status.r != undefined && status.g != undefined && status.b != undefined;
  },
  checkOnlyCW (status) {
    if (!status) {
      return false;
    }
    return status.c != undefined && status.w != undefined && status.r == undefined && status.g == undefined && status.b == undefined;
  },
  // 判断是否rgb
  isRgbByPid (pid) {
    return pid == 43168;
  },
  setColorIndex (index) {
    this.data.devStatus.enumIdx = index;
    this.setData({
      devStatus: this.data.devStatus
    })
  },
  // 颜色点击
  colorEnumClick (e) {
    let _index = e.currentTarget.dataset.index;
    let _isCancel = _index == this.data.devStatus.enumIdx;
    // 更新数据
    this.setColorIndex((_index >= 10 || _isCancel) ? -1 : _index);
    if (_isCancel) {
      return;
    }

    let _item = null;
    if (_index >= 10) {
      _item = this.data.fixColors[_index - 10];
    } else {
      _item = this.data.commonColors[_index];
    }
    let _colorStr = _item.color || "";
    let _rgbStr = "";
    // 如果是rgb 以逗号分开
    if (_colorStr.indexOf("#") >= 0) {
      _rgbStr = _colorStr.split("#")[1] || "";
    } else {
      return;
      // _rgbStr = _colorStr.split(",");
    }

    let _params = {
      pwr: 1,
      brightness: this.data.devStatus.brightness || 127,
    };
    // 特殊处理(https://www.tapd.cn/41682094/bugtrace/bugs/view/1141682094001004307)
    if (_item.color.toLowerCase() == '#ffffff') {
      // 白光调节
      let {r, g, b, c, w} = this.data.devStatus;
      let _devInfo = app.globalData.devInfo;
      console.log("colorEnumClick _devInfo = ", _devInfo);
      // 是否只有rgbw灯
      if (this.isOnlyRgbw) {
        _params.r = _params.g = _params.b = 0;
        _params.c = _params.w = 127;
      }
      // 是否只有rgb灯
      else if (!!_devInfo) {
        console.log("colorEnumClick this.data.devStatus = ", this.data.devStatus);
        let _isRgbw = bizUtil.isRGBW(_devInfo?.t);
        console.log("colorEnumClick _isRgbw = ", _isRgbw);
        // 单个设备
        if ((c == undefined && w == undefined) && !_isRgbw) {
          // 只有rgb
          _params.r = _params.g = _params.b = 255;
          _params.c = _params.w = 0;
        } else {
          _params.r = _params.g = _params.b = 0;
          _params.c = _params.w = 127;
        }
      } else {
        console.log("colorEnumClick isOnlyRgb = ", this.data.isOnlyRgb);
        // 批量和全屋控制，如果全是rgb，则发送255，如果有一个不是，则发送127
        if (this.data.isOnlyRgb) {
          // 只有rgb
          _params.r = _params.g = _params.b = 255;
          _params.c = _params.w = 0;
        } else {
          _params.r = _params.g = _params.b = 0;
          _params.c = _params.w = 127;
        }
      }
    } else {
      _params.r = parseInt("0x" + _rgbStr.substr(0, 2));
      _params.g = parseInt("0x" + _rgbStr.substr(2, 2));
      _params.b = parseInt("0x" + _rgbStr.substr(4, 2));
    }
    // 发送广播
    this.actDev(_params);
  },
  isW () {
    let _devStatus = this.data.devStatus;
    return _devStatus.r == undefined && _devStatus.c == undefined;
  },
  // 滑块百分比
  sliderChange (event) {

    // 初始化选择常用颜色
    this.setColorIndex(-1);

    // let _percent = event.detail.percent;
    let _value = event.detail.value;
    let _isMoving = event.detail.isMoving;
    let _originValue = event.detail.originValue;
    // 区间为 5 - 127 默认为0 - 100
    console.log("sliderChange _value, _originValue = ", _value, _originValue);
    let _status = {
      brightness: _value,
      // pwr: 1
    };
    // 增加处理 保存原始的值（精确到小数点 为了防止下次进入亮度不一致）
    this.data.devStatus.originBrightness = _originValue;
    // 执行 , _isMoving ? 1 : undefined
    this.actDev(_status);
  },
  // 滑块百分比
  cwSliderChange (event) {
    let {value, isMoving, percent} =  event.detail;
    // let _value = event.detail.value;
    // let _isMoving = event.detail.isMoving;
    // let _val = event.detail.value;
    // 区间为 1 - 255 默认为0 - 100
    // _val = 1 + parseInt(_val * 2.54);
    console.log("sliderChange value = ", value, percent, isMoving);

    // 保存色温百分比
    this.data.devStatus.cwP = percent;

    // 色温值 0-255
    let _value = Math.round(percent * 2.55);
    // // 取得色温的最大最小值
    // let {maxVal, minVal} = this.data.cwSliders;
    // if (maxVal != undefined && minVal != undefined) {
    //   let _percent = ((value - minVal) / (maxVal - minVal));
    //   console.log("cwSliderChange percent, _percent = ", percent, _percent);
    //   if (_percent < 0) { _percent = 0; }
    //   else if (_percent > 1) { _percent = 1; }
    //   _value = Math.round(_percent * 255);
    // }
    console.log("cwSliderChange _value = ", _value);
    // 执行 , isMoving ? 1 : undefined
    this.actDev({
      pwr: 1,
      // brightness: this.data.devStatus.brightness || 127,
      c: (255 - _value),
      w: _value
    });
  },
  // 这里是所有批量 老批量是五个短地址的，新批量是走群组控制，只是群组id是特殊的
  isAllBatchCtrl () {
    let _isOldBatch = this.isBatchCtrl();
    // 这里253的是支持群组控制方式的临时群组，所以支持模式
    let _tgroupId = app.globalData.tgroupId;
    console.log("isAllBatchCtrl _isOldBatch, app.globalData.tgroupId = ", _isOldBatch, _tgroupId);
    return _isOldBatch || (_tgroupId == 253);
  },
  // 获取是否批量控制
  isBatchCtrl () {
    return (this.data.groupIds.length > 0) && (app.globalData.devInfo == undefined || Object.keys(app.globalData.devInfo).length == 0);
  },
  // 这里是全屋控制和临时、固定群组控制
  isAllCtrl () {
    let _groupIds = app.globalData.groupIds;
    return (!_groupIds || _groupIds.length == 0) && app.globalData.devInfo == undefined
  },
  actDev (key, configJson) {
    // 保存设备信息
    if (!!key && (typeof key != "string")) {
      // 如果只有亮度调节成0，则不保存亮度的值
      let _keyArray = Object.keys(key) || [];
      if (_keyArray.length == 1 && key['brightness'] == 0) {
        // 不保存
      } else {
        // 不能assign,会有cw等保存进去
        for (let _k in key) {
          if (this.data.devStatus[_k] != undefined) {
            this.data.devStatus[_k] = key[_k];
          }
        }
      }
    }

    // 隐藏对话框
    this.showDetailDialog(false);

    // 群组id
    let _groupIds = this.data.groupIds;
    let _devInfo = app.globalData.devInfo;
    let _isBatchCtrl = this.isBatchCtrl();
    console.log("actDev _isBatchCtrl, _devInfo = ", _isBatchCtrl, !!_devInfo);
    // 判断控制方式
    // 如果有临时group 补丁
    if (app.globalData.tgroupId) {
      console.log("app.globalData.tgroupId = ", app.globalData.tgroupId);
      // 全屋或者房间控制
      app.globalData.blsdk.groupControlWithType(false, app.globalData.tgroupId, key, configJson);
    } else if (_isBatchCtrl) {
      console.log("_isBatchCtrl, _devInfo = ", _groupIds, key);
      // 批量控制
      app.globalData.blsdk.bathControlWithDeviceList(_groupIds, key, configJson);
    } else if (app.globalData.tmpData?.roomInfo?.gid >= 0) {
      let _gid = Number(app.globalData.tmpData?.roomInfo?.gid) || 0;
      // 全屋或者房间控制
      app.globalData.blsdk.groupControlWithType(false, _gid, key, configJson);
    } else if (!_isBatchCtrl && !_devInfo) {
      // 全屋或者房间控制
      app.globalData.blsdk.groupControlWithType(false, (app.globalData.roomInfo || {}).id || 0, key, configJson);
    } else {
      app.globalData.blsdk.controlWithDevice(_devInfo, key, configJson);
    }
  },
  /************** end ***************/
  // 判断简单不重复数据是否在数组中
  isAllInArray (values, array) {
    if (!array) {
      return false;
    }
    let _typeStr = typeof values;
    if (_typeStr == "string" || _typeStr == "number") {
      return array.indexOf(values) >= 0;
    }
    return array.sort().join('').indexOf(values.sort().join('')) >= 0;
  },
  hideDialog () {
    this.showDetailDialog(false);
  },
  // 详情按钮
  showDetailDialog(value) {
    this.setData({
      showDetail: value || false
    })
  },
  // 跳转炫彩编辑
  jumpXuancai (e) {
    let {index} = e.currentTarget.dataset;
    let {text} = this.data.controlList[index] || {};
    console.log("jumpXuancai index = ", index);

    let _isRgb = Number(this.data.devStatus?.r != undefined);
    ability.navigateTo("../xuancai/index?isRbg=" + _isRgb + "&title=" + encodeURIComponent(text || ""));
  },
  // 跳转定时页面
  jumpTimer () {
    // 定时权限
    let _roleNum = app.globalData?.roleNum || 0;
    // 定时数据
    app.globalData.tmpData.timerJson = undefined;
    ability.navigateTo("../../../timer/list/index?roleNum=" + _roleNum)
  },
  // 跳转模式页面
  jumpPattern () {
    console.log("this.data.abilitys, this.data.devStatus = ", this.data.abilitys, this.data.devStatus);
    // 是否支持睡眠模式
    let {sleep, minBrightness, "1024": huidu, cwGradient, cAndcwWhite} = this.data.abilitys || {};
    let _sleep = sleep || 0;
    // 最小值
    let _value = minBrightness?.value || 0;
    // 1024灰度 这里偷懒了，之前这个字段是cw的，后面改了协议变成了1024灰度字段，这里只改了值，传的参数名称没改
    let _cw = huidu || 0;
    // cw灯的渐变跳变能力
    let _cwGradient = cwGradient || 0;
    // 传递当前亮度值
    let _brightnessV = this.data.devStatus.brightness || 127;
    // 跳转页面
    this.setColorIndex(-1);
    // 传递参数
    ability.navigateTo("../mode/index?sleep=" + _sleep + "&mode=" + Number((this.data.devStatus || {}).r != undefined) + (_value != undefined ? ("&min=" + _value) : '') + "&cw=" + _cw + "&gradient=" + _cwGradient + "&brightness=" + _brightnessV + "&cAndcwWhite=" + cAndcwWhite);
  },
  // 跳转识音
  jumpMic () {
    let _array = this.data.controlList.filter(item => item.tap == "jumpMic");
    if (_array.length == 0) {
      return;
    }
    ability.navigateTo("../funcSet/mic/index?title=" + encodeURIComponent(_array[0]?.text || ''));
  },
  jumpDetailClick () {
    // 判断是否大于2个选项
    if (this.data.detailList?.length > 1) {
      // 显示
      this.showDetailDialog(!this.data.showDetail);
      return;
    }
    this.jumpDetail();
  },
  // 跳转设备详情页面
  jumpDetail () {
    // 初始化
    this.setColorIndex(-1);
    // 保存设备状态
    app.globalData.devStatus = this.data.devStatus || {};
    // 是否支持遥控器
    let _remote = (this.data.abilitys || {}).remote || 0;

    // 如果是群组控制，则进入群组管理页面
    if (this.isGroupCtrl()) {
      console.log("groupCtrl app.globalData.groupInfo = ", app.globalData.groupInfo);
      // 跳转页面
      // return ability.navigateTo("/groups/manage/index?remote=" + _remote);
    }
    console.log("devProperty");
    // 跳转页面
    ability.navigateTo("../devProperty/index?type=light&remote=" + _remote);
  },
  // 初始化canvas的尺寸
  initCanvasSize () {

    // let _circleWidth = this.data.canvas.circleWidth;
    // 圆环宽度
    // ctx.setLineWidth(_circleWidth);// 设置圆环的宽度
    // canvas的长和宽
    let {widthRpx, heightRpx, multip, padL, padR, circleWidth} = this.data.canvas || {};
    // 取得px的宽和高
    let _realWidth = parseInt((widthRpx + padL) / multip);
    let _realHeight = parseInt((heightRpx + padL) / multip);
    // let _realPadL = parseInt(padL / multip);
    // 赋最小值 保证是个圆
    _realWidth = _realHeight = Math.min(_realWidth, _realHeight);
    let _r = (widthRpx - circleWidth) / 2 / multip;
    let _centerX = _realWidth / 2, _centerY = _realHeight / 2;
    this.data.canvas.r = _r; // 保存半径
    this.data.canvas.centerX = _centerX; // 保存中心点
    this.data.canvas.centerY = _centerY; // 保存中心点
    this.data.canvas.padLReal = padL / 2 / multip; 
  },
  initCanvas (canvas, context) {
    
    // 等待条
    // this.showLoading();

    // 创建一个图片
    const image = canvas.createImage();
    image.src = "../../img/light/bg.png" // 要加载的图片 url
    image.onload = () => {

      // ability.hideLoading();

      // canvas的长和宽
      let {widthRpx, heightRpx, multip, padL} = this.data.canvas || {};
      let _width = (widthRpx) / multip, _height = heightRpx / multip;
      // 把图片画到离屏 canvas 上
      // context.clearRect(0, 0, _width + padLReal * 2, _height + padLReal * 2);
      // context.drawImage(image, padLReal, padLReal, _width, _height);
      context.clearRect(0, 0, _width, _height);
      context.drawImage(image, 0, 0, _width, _height);
      // 保存绘图实例
      this.data.context = context;
    };
  },
  drawNewDetail (obj) {
    // canvas的长和宽
    let {widthRpx, heightRpx, multip, padL} = obj || {};
    let _width = (widthRpx) / multip, _height = heightRpx / multip;

    // if (this.data.devStatus.r == undefined) {
    //   return;
    // }

    // 创建离屏 2D canvas 实例
    var canvas = wx.createOffscreenCanvas({type: '2d', width: _width, height: _height})
    // let _element = wx.createSelectorQuery();
    // let canvas = _element.select("lightCanvas");
    // // var canvas = wx.createOffscreenCanvas({type: '2d', width: _width, height: _height})
    // // 获取 context。注意这里必须要与创建时的 type 一致
    var context = canvas.getContext('2d');
    this.initCanvas(canvas, context);

    // wx.createSelectorQuery()
    //   .select('#lightCanvas')
    //   .fields({
    //     node: true,
    //     size: true,
    //   })
    //   .exec(() => {
    //   const canvas = res[0].node
    //   const context = canvas.getContext('2d');
    //   this.initCanvas.bind(this)
    // })
  },
  drawProgressbg () {
    // 初始化尺寸
    this.initCanvasSize();
    // 绘图
    this.drawNewDetail(this.data.canvas);
  },
  touchNone () {
    return;
  },
  touchStart (e) {
    // let _position = this.getPosition(e);
    console.log("touchStart");
    // 保存之前的角度，如果幅度小于1，则不获取
    let _position = this.getPosition(e);
    let {x, y, angle} = this.getPositionByX(_position);
    this.data.lastAngle = angle;
    console.log("touchStart angle, this.data.canvas = ", angle, this.data.canvas);
  },
  touchMove (e) {
    this.dealTouchRgb(e);
  },
  dealTouchRgb (e) {
    console.log("dealTouchRgb");
    let _position = this.getPosition(e);
    let {x, y, angle} = this.getPositionByX(_position);

    // 如果角度相差小于1，则不移动
    if (Math.abs(angle - this.data.lastAngle) < 1) {
      return;
    }
    // 保存上一次的角度值
    this.data.lastAngle = angle;

    // 取得canvas的rgb
    this.getImageData(angle, x, y, (rgbData, sendData) => {
      let data = this.dealRgbPanel(rgbData);
      console.log("getImageData data = ", data);
      if (!data) {
        return;
      }
      this.data.thumb.bgColor = 'rgb('+data[0]+','+data[1]+','+data[2] + ')';
      // this.data.thumb.bgColor = 'rgba('+data[0]+','+data[1]+','+data[2]+','+data[3]+')';
      this.data.thumb.x = x;
      this.data.thumb.y = y;
      // this.data.thumb.angle = angle;

      // 更新数据
      let _updateObj = {};
      _updateObj.thumb = this.data.thumb;

      let _enumIdx = this.data.devStatus.enumIdx;
      let _commonColors = this.data.commonColors || [];
      console.log("_enumIdx = ", _enumIdx);
      // 保存颜色
      if (_enumIdx >= 0 && _commonColors[_enumIdx]) {
        let _colorStr = '#' + data[0].toString(16).padStart(2, '0') + data[1].toString(16).padStart(2, '0') + data[2].toString(16).padStart(2, '0');
        (_commonColors[_enumIdx] || {}).color = _colorStr;

        _updateObj.commonColors = _commonColors;
      }

      // 更新
      this.setData(_updateObj)

      if (!this.data.abilitys.gradient) {
        return;
      }
      console.log("touchMove e, new = ", x, y, angle); 
      // 每隔1s发送一次
      let _time = this.data.sendTime;
      let _newTime = Date.now();
      let _sendRGB = sendData || this.dealRgbSend(data);
      // 判断批量个数
      let _lengthBatch = (this.data.groupIds || []).length;
      let _timeCount = _lengthBatch > 5 ? (Math.ceil(_lengthBatch / 5) * (200 + 30)) : 250;
      if (!_time || (_newTime - _time) > _timeCount) {
        // , 1
        this.actDev({
          pwr: 1,
          // brightness: this.data.devStatus.brightness || 127,
          r: _sendRGB[0],
          g: _sendRGB[1],
          b: _sendRGB[2]
        }, {
          // // 远程调试测试用
          // noRemote: true,
        })
        this.data.sendTime = _newTime;
      }
    });
  },
  touchEnd (e) {
    console.log("touchEnd");
    let _position = this.getPosition(e);
    let {x, y, angle} = this.getPositionByX(_position);
    // 保存角度
    this.data.devStatus.angle = angle;
    // 清空
    this.data.lastAngle = undefined;
    // 取得canvas的rgb
    this.getImageData(angle, x, y, (rgbData, sendData) => {
      console.log("touchEnd getImageData rgbData = ", rgbData);
      let data = this.dealRgbPanel(rgbData);
      if (!data) {
        return;
      }
      let _colorStr = '#' + data[0].toString(16).padStart(2, '0') + data[1].toString(16).padStart(2, '0') + data[2].toString(16).padStart(2, '0');
      console.log("_colorStr = ", _colorStr);

      this.data.thumb.bgColor = 'rgb('+data[0]+','+data[1]+','+data[2] + ')';
      this.data.thumb.x = x;
      this.data.thumb.y = y;
      this.setData({
        thumb: this.data.thumb
      })

      // 保存颜色
      this.data.devStatus.angleColor = _colorStr;

      let _enumIdx = this.data.devStatus.enumIdx;
      let _commonColors = this.data.commonColors || [];
      // 保存颜色
      if (_enumIdx >= 0 && _commonColors[_enumIdx]) {
        (_commonColors[_enumIdx] || {}).color = _colorStr;
        this.setData({
          commonColors: this.data.commonColors
        });
      }
      
      let _sendRGB = sendData || this.dealRgbSend(data);
      // 发送命令
      this.actDev({
        pwr: 1,
        // brightness: this.data.devStatus.brightness || 127,
        r: _sendRGB[0],
        g: _sendRGB[1],
        b: _sendRGB[2]
      })
    });
  },
  getRgbByAngle (angle) {
    return bizUtil.getRgbByAngle(angle);
  },
  // 取得canvas的色素
  getImageData (angle, x, y, callBack) {
    if (!this.data.context) {
      return !!callBack && (callBack([]));
    }
    // 次数需要减去圆环到父层的距离
    let _seperator = this.data.canvas.padLReal;
    // console.log("getImageData x - _seperator, _seperator, x = ", x - _seperator, _seperator, x, this.data.canvas);
    // 图片数据
    let {data} = this.data.context.getImageData((x - _seperator), y, 1, 1) || {};
    // console.log("getImageData data, x, y = ", Array.prototype.join.call(data, ','), x, y);
    if (!data || data.length < 4) {
      return !!callBack && (callBack([]));
    }
    // 根据角度取得色值
    let _sendData = this.getRgbByAngle(angle);
    // 返回数据
    !!callBack && (callBack(data, _sendData));
  },
  getPosition (e) {
    let {changedTouches, touches} = e;
    let {pageX, pageY, padLReal, circleWidth} = this.data.canvas;
    let _item = (changedTouches || touches)[0];
    // let _realSepreate = padLReal || 0;
    // console.log("getPosition _item.pageX, _item.pageY = ", _item.pageX, _item.pageY, _realSepreate, _item.pageX - (pageX || 0) - padLReal, _item.pageY - (pageY || 0) - padLReal);
    return {
      x: ((_item.pageX - (pageX || 0)) || _item.clientX),
      y: ((_item.pageY - (pageY || 0)) || _item.clientY) 
    }
  },
  getPositionByAngle (degree) {
    let {centerX, centerY, r, circleWidth} = this.data.canvas;
    console.log("getPositionByAngle r = ", r);
    // let {thumb} = this.data.thumb
    let x = Math.cos(degree / 180 * Math.PI) * (r);
    let y = Math.sin(degree / 180 * Math.PI) * (r);
    let _isRight = !(degree >= 90 && degree < 270);
    let _isDown = !(degree >= 0 && degree < 180);
    // 绘制canvas的时候需要有半个宽度的补偿值 76是thumb的宽度 先这样写吧
    let _buchang = 0; // circleWidth / 2 - (76 / 2 / this.data.canvas.multip);
    return {
      x: centerX + _buchang + Math.abs(x) * (_isRight ? 1 : -1),
      y: centerY + _buchang + Math.abs(y) * (_isDown ? 1 : -1),
    };
  },
  // 根据x,y坐标取得角度
  getAngleByPosition (x, y) {
    // 半径
    let {centerX, centerY} = this.data.canvas;
    let _isRight = x >= centerX;
    let _isDown = y >= centerY;
    // 需要减去中心点坐标
    let _realX = Math.abs(x - centerX);
    let _realY = Math.abs(y - centerY);
    // _angle大于0
    let _angle = 0;
    if (_realX == 0) {
      _angle = _isDown ? 270 : 90;
    } else {
      let _originAngle = Math.atan(_realY / _realX) * 180 / Math.PI;
      _angle = Math.abs(_originAngle);
      // tan(90+a)=tan(90-a) 忽略正负
      // tan(180+a)=tan(a)
      // tan(27+a)=tan(90-a) 忽略正负
      if (_isRight) {
        // 第四和一象限
        _angle = _isDown ? (360 - _angle) : (_angle);
      } else {
        // 第三和二象限
        _angle = _isDown ? (180 + _angle) : (180 - _angle);
      }
      // console.log("_originAngle, _angle = ", _originAngle, _angle);
    }
    return _angle;
  },
  // 根据x，y轴坐标取得x轴对应的y坐标
  getPositionByX (position) {
    let {x, y} = position;
    // 半径
    let {r, centerX, centerY} = this.data.canvas;
    let _isRight = x >= centerX;
    let _isDown = y >= centerY;
    // 取得角度
    let _tanDegree = Math.round(this.getAngleByPosition(x, y));
    // 根据半径和角度取得实际的x和y
    let _x = r * Math.cos(_tanDegree * Math.PI / 180);
    let _y = r * Math.sin(_tanDegree * Math.PI / 180);
    return {
      x: centerX + Math.abs(_x) * (_isRight ? 1 : -1),
      y: centerY + Math.abs(_y) * (_isDown ? 1 : -1),
      angle: _tanDegree
    }
  },
  forbidenMove () {
    console.log("forbidenMove");
  },

  plusStart () {
    this.dealLongTapStart(false);
  },
  dealLongTapStart (isDel) {
    // 防止两个手指重复按下
    if (this.data.rgbDown || this.data.rgbUp) {
      return;
    }
    // 默认长按
    this.data.isLongTap = true;
    // 记住按下时间
    this.data.timeStamp = Date.now();
    // 按下的按钮标识
    let _updateObj = isDel ? {rgbDown: true, rgbUp: false} : {rgbUp: true, rgbDown: false};
    this.data.rgbDown = _updateObj.rgbDown;
    this.data.rgbUp = _updateObj.rgbUp;
    this.setData(_updateObj);
    // 定时器检查是长按还是短按
    setTimeout(() => {
      // 是否长按标志
      if (!this.data.isLongTap) {
        return;
      } else {
        // 初始化定时器
        this.dealLongTap();
      }
    }, 800);
  },
  // 长按事件
  dealLongTap () {
    // 清理定时器
    this.clearTimer(this.data.timer);
    // 初始化定时器
    this.data.timer = setInterval(() => {
      if (!this.data.timer) {
        return;
      }
      console.log("dealLongTap");
      this.dealAngle(!this.data.rgbUp ? -1 : 1, true);
    }, 300);
  },
  plusEnd () {
    console.log("plusEnd");
    this.dealTouchEnd(false);
  },
  dealTouchEnd (isDel) {
    // 清除定时器
    this.clearTimer(this.data.timer);
    // 恢复按钮
    let _updateObj = {rgbDown: false, rgbUp: false};
    this.setData(_updateObj);
    // 判断短按还是长按
    let _now = Date.now();
    console.log("_now - this.data.plusTime = ", _now - this.data.timeStamp);
    if ((_now - this.data.timeStamp) < 700) {
      // 是否长按标志
      this.data.isLongTap = false;
      // 认为是点击事件
      this.dealAngle(!isDel ? 1 : -1, true);
      // !!this.data.rgbUp ? (this.addAngle()) : (this.delAngle());
    } else {
      // 是否长按标志
      this.data.isLongTap = false;
    }
  },
  minusStart () {
    this.dealLongTapStart(true);
  },
  minusEnd () {
    console.log("minusEnd");
    this.dealTouchEnd(true);
  },
  clearTimer (timer) {
    bizUtil.clearTimer(timer, "interval");
  },

  // 初始化冷暖光
  initCodewarmList () {
    this.data.codewarmList = [{
      text: "冷光",
      data: { w: 255, c: 0}
    }, {
      text: "冷/暖光",
      data: { w: 255, c: 255}
    }, {
      text: "暖光",
      data: { w: 0, c: 255}
    }];
    this.setData({
      codewarmList: this.data.codewarmList
    })
  },
  codewarmClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.codewarmList[index];
    this.data.warmcodeIndex = index;
    this.setData({
      warmcodeIndex: index
    });
    // 控制参数
    let _controlParams = Object.assign({
      pwr: 1,
      brightness: this.data.devStatus.brightness || 127
    }, _item?.data || {});
    // 发送控制命令
    this.actDev(_controlParams)
  },
  jumpFuncSet () {
    // 是否支持遥控器
    let _remote = (this.data.abilitys || {}).remote || 0;
    let _itemArray = this.data.detailList.filter(item => item.key == "funcset");
    let _title = _itemArray[0]?.title || "";
    // 需要保存当前状态，用于识音
    this.saveDevStatus();
    // 跳转详情配置页面
    ability.navigateTo("../funcSet/index?devType=light&remote=" + _remote + "&title=" + encodeURIComponent(_title))
  }
})
