// device/light/devProperty/main/index.js
import bizUtil from '../../../../utils/bizUtil'
import ability from "../../../../utils/classDef/ability"
// import groupBehavior from "../../../../static/js/behavior/group"

const app = getApp();
Page({

  // behaviors: [groupBehavior],

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "房间详情"
      }
    },

    // 群组内设备
    groupDevList: [],

    bleUnavailable: false,

    roleNum: 0, // 用户权限

    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined, // 家庭下所有设备信息

    roomJson: {}, // 房间key
    isFloor: false, // 是否多楼层

    // 设备品类
    type: app.globalData.groupInfo?.t,

    // 选中的设备d
    mainDevInfo: {},
    oldD: ""
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    let {familyId, title} = options || {};

    // 设置标题
    this.setNavTitle(title);

    // 初始化家庭和家庭下的设备数据
    this.initAllDataByFamily(familyId);

    console.log("onLoad this.data.familyInfo = ", this.data.familyInfo, app.globalData.groupInfo);
    // 取得当前家庭的数据
    let {roleNum, floor} = this.data.familyInfo || {};

    // 所有房间数据
    let {roomJson} = bizUtil.getRoomsByDevs(this.data.allDevs);
    this.data.roomJson = roomJson;
    this.data.isFloor = floor >= 0;

    // 权限
    this.data.roleNum = roleNum == undefined ? app.globalData.roleNum : roleNum;

    // 传递的房间数据
    this.setData({
      roleNum: this.data.roleNum,
      roomJson: this.data.roomJson,
    });
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(decodeURIComponent(title || ""));
  },
  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },
  initTitle () {
    let _title = app.globalData.groupInfo?.n || "群组设备";
    ability.setNavigationBarTitle(_title);
  },

  // getAllActions () {
  //   return [{
  //     n: "取消主设备",
  //     classStr: "",
  //     handle: ""
  //   }, {
  //     n: "保存",
  //     classStr: "",
  //     handle: ""
  //   }]
  // },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 更新标题
    this.initTitle();
    // 更新设备信息
    this.initGroupDevs();
  },

  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    })
  },
  hideLoading () {
    ability.hideLoading();
  },
  // 取得当前群组中的主设备
  getGroupMainDByDevArray (devArray, gid) {
    let _mainJson = this.getMainDeviceJsonInArray(devArray);
    return _mainJson[gid] || {};
  },
  // 取得设备中当前群组的主设备
  getMainDeviceJsonInArray (devArray) {
    let _result = {}; // groupId: devInfo
    if (!devArray || devArray.length == 0) {
      return _result;
    }
    // 遍历
    for (let devInfo of devArray) {
      let {e, d} = devInfo || {};
      if (!e) {
        continue;
      }
      let {group} = typeof e == "string" ? JSON.parse(e) : e;
      if (group?.main == undefined) {
        continue;
      }
      _result[group.main] = devInfo;
      // _result.push(group.main);
    }
    return _result;
  },

  // 初始化房间信息
  initGroupDevs () {
    console.log("initGroupDevs this.data.allDevs = ", this.data.allDevs);
    // 取得房间下所有的d对应的详细信息
    let _didMaps = bizUtil.getDidMapInJson(this.data.allDevs);
    // 群组设备列表
    let {dids: _groupDevs, gid} = app.globalData.groupInfo || {};
    // 群组设备列表详细 并过滤出群组内支持主设备设置的数据
    let _groupDevList = (_groupDevs || []).map(did => _didMaps[did]).filter(item => bizUtil.getJsonByAblitys(item.v).mainEnable);
    console.log("initGroupDevs _groupDevList = ", _groupDevList);
    // 房间内的数据按照名称排序
    this.data.groupDevList = bizUtil.sortArrayByChar(_groupDevList, "n");
    console.log("this.data.groupDevList = ", this.data.groupDevList);
    // 取得所有的主设备
    let _mainDevInfo = this.getGroupMainDByDevArray(_groupDevList, gid);
    
    // 保存上一次的主设备短地址
    this.lastDevInfo = _mainDevInfo || {};

    this.setData({
      groupDevList: this.data.groupDevList,
      mainDevInfo: _mainDevInfo,
      oldD: _mainDevInfo.d
    })
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
    console.log("roomDetail onHide");
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
    if (!app.globalData.blsdk) {
      return;
    }
    app.globalData.blsdk.stopReceiveAndAdvertising();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {
  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {
  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {
  },

  getGroupId () {
    let {gid, rid} = app.globalData.groupInfo || {};
    return (gid == undefined ? rid : gid) || 0;
  },

  actDev (cmd, deviceInfo) {
    // 控制命令
    app.globalData.blsdk.controlWithDevice(deviceInfo, cmd);
  },

  // 设置群组主设备
  setGroupMain (e) {
    let {index} = e.currentTarget.dataset;
    let _devInfo = this.data.groupDevList[index] || {};
    if (!_devInfo) {
      return;
    }
    let {a, d} = this.data.mainDevInfo;
    let _curDevInfo = d == _devInfo?.d ? {
      a: a,
      d: ''
    } : _devInfo;
    this.setData({
      mainDevInfo: _curDevInfo
    });
  },
  // 取消主设备
  cancalMainOnce () {
  },
  // 设置主设备
  setMain () {
    let {d, a} = this.data.mainDevInfo || {};

    // 判断是取消主设备还是设置主设备
    if (a == undefined) {
      return ability.showToast({
        title: "请选择主设备"
      });
    }
    // 群组id
    let _gid = this.getGroupId();
    // 使能
    let _paramJson = {
      enable: !!d,
      a: a || 0,
      gid: _gid
    };
    // 如果没有选择的主设备，则判断是否之前是否有主设备绑定，如果没有，则提示选择主设备
    if (!d) {
      if (this.lastDevInfo && this.lastDevInfo.a) {
        _paramJson.a = this.lastDevInfo.a;
      } else {
        return ability.showToast({
          title: "请选择主设备"
        });
      }
    }
    ability.showLoading({});
    app.globalData.blsdk.mainGroupDevSet(_paramJson, (res) => {
      ability.hideLoading();
      console.log("mainGroupDevSet res = ", res);
      let _isFailed = ((!res && res != 0) || res < 0);
      ability.showToast({
        title: _isFailed ? "设置失败" : "设置成功"
      });
      if (_isFailed) {
        return;
      }
      // 是否有上一次的主设备，如果有，则下发
      if (!!this.lastDevInfo && this.lastDevInfo.d != d) {
        let _tmpJson = {
          enable: false, 
          a: this.lastDevInfo.a,  
          gid: _gid
        };
        app.globalData.blsdk.mainGroupDevSet(_tmpJson);
        // 主设备赋值
        this.setMainDeviceProper(this.lastDevInfo, _tmpJson);
      }
      // 主设备赋值
      this.setMainDeviceProper(this.data.mainDevInfo, _paramJson);
      // 返回上一页
      setTimeout(() => {
        ability.navigateBack();
      }, 1000);
    })
  },
  setMainDeviceProper (mainDevInfo, {enable, gid}) {
    if (!mainDevInfo) {
      return;
    }
    mainDevInfo.e = mainDevInfo.e || {};
    mainDevInfo.e.group = mainDevInfo.e.group || {};
    mainDevInfo.e.group.main = !!enable ? gid : undefined;
  }
})