// device/light/devProperty/replace/common/index.js
import bizUtil from "../../../../../utils/bizUtil"
import ability from "../../../../../utils/classDef/ability"
import GroupMethod from "../../../../../static/js/behavior/group"
// import { Utils } from "../../../../../utils/util";
const app = getApp();
Page({
  behaviors: [GroupMethod],
  /**
   * 页面的初始数据
   */
  data: {
    bleUnavailable: false,
    devItems: [], // 显示的条目
    stepIndex: 0, // 正在配置第一个
    actions: [{
      n: "下一步",
      key: "next",
      classStr: "white bgF78D3F"
    }]
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad (options) {
    // 要替换设备信息
    let _replaceDevInfo = this.getNewDevInfo();
    console.log("onLoad _replaceDevInfo = ", _replaceDevInfo);
    // 设置标题
    this.setNavTitle(_replaceDevInfo.n);
    // 设置数据
    this.initItemArray();
    // 取得设备加入的所有自定义场景信息
    this.customScenes = this.getAllCustomScenesByDevInfo(app.globalData.devInfo);
    console.log("onLoad this.customScenes = ", this.customScenes)
    // 开始替换
    this.startReplaceDev();
  },
  /**
   * 生命周期函数--监听页面显示
   */
  onShow () {
  },
  getNewDevInfo () {
    return app.globalData.tmpData.devInfo || {};
  },
  setNavTitle (name) {
    ability.setNavigationBarTitle(name || "");
  },
  getAllConfigByDevInfo (devInfo, allGroups) {
    let _values = []; // 主设备和炫彩灯带和拾音
    let _tips = [];
    // let _lightDazzles = []; // 炫彩灯带
    let {e} = devInfo || {};
    let {group, lightDazzle, light} = e || {};
    // 是否主设备,如果是,则设置 即使设备不在群组中,下发群组主设备命令也能成功并正常
    let _groupMain = Number(group?.main);
    if (_groupMain) {
      // 主设备
      _tips.push("主设备配置");
      _values.push({
        key: "mic",
        value: _groupMain
      });
    }
    // 是否存在灯带配置和收音配置,如果是,则设置
    if (lightDazzle) {
      if (lightDazzle.ic_type != undefined) {
        _tips.push("炫彩灯带配置");
        _values.push({
          key: "getXuancaiParams",
          value: {
            "ic_type": lightDazzle.ic_type, // IC 类型
            "beads_sort": lightDazzle.beads_sort, // 灯珠排序
            "beads_number": lightDazzle.beads_number, // 灯珠总数
            "beads_type": lightDazzle.beads_type, // 炫彩模式类型
            "beads_speed": lightDazzle.beads_speed
          }
        });
      }
      if (lightDazzle.mic_enable) {
        _tips.push("拾音配置");
        _values.push({
          key: "mic",
          value: {
            "mic_enable": lightDazzle.mic_enable, // Mic 是否使能
            "mic_type": lightDazzle.mic_type, // Mic 类型
            "mic_speed": lightDazzle.mic_speed, // Mic 变化
          }
        });
      }
    }
    // 是否有配网锁定、上电勿扰、缓起缓灭
    if (light) {
      // 配网锁定
      if (light.config_lock >= 0) {
        _tips.push("配网锁定");
        _values.push({
          key: "getConfigLock",
          value: light.config_lock
        });
      }
      // 上电勿扰
      if (light.pwr_cut >= 0) {
        _tips.push("上电勿扰");
        _values.push({
          key: "getNoDisturb",
          value: light.pwr_cut
        });
      }
      // 缓起缓灭
      if (light.slow_rise >= 0) {
        _tips.push("缓起缓灭");
        _values.push({
          key: "getSlowUpDown",
          value: light.slow_rise
        });
      }
    }
    return {
      tips: _tips,
      values: _values
    };
  },
  getAllFixGroupsByDevInfo (devInfo, allGroups) {
    // 取得设备所在的固定群组列表
    let _groups = allGroups || bizUtil.getAllDevsFilter(app.globalData.devs, undefined, "groups");
    let _existGroups = _groups.filter(item => !!item.dids && item.dids.indexOf(devInfo.d) >= 0 && !bizUtil.isAllRoomCtrl(item));
    let _nameList = _existGroups.map(item => item.n || "");
    return {
      values: _existGroups,
      tips: _nameList
    }
  },
  getAllCustomScenesByDevInfo (devInfo) {
    // 取得设备所有场景
    let _scenes = bizUtil.getAllDevsFilter(app.globalData.devs, undefined, "scenes");
    // 先过滤一层自定义的
    return this.getSceneInfoByD(_scenes, devInfo.d);
  },
  getSceneInfoByD (scenes, d) {
    // let _tips = [];
    // let _sceneIds = [];
    console.log("getSceneInfoByD scenes, d = ", scenes, d);
    let _cmds = [];
    if (!!scenes && scenes.length > 0) {
      for (let item of scenes) {
        if (!item.devs) {
          continue
        }
        let _devCmdInfos = item.devs.filter(item => item.d == d);
        if (_devCmdInfos.length == 0) {
          continue;
        }
        // _tips.push(item.n || "");
        // _sceneIds.push(bizUtil.getSceneId(item.sid || 0, item.rid || 0));
        _cmds.push(item);
      }
    }
    return _cmds
    // {
    //   // tips: _tips,
    //   // scenes: _sceneIds,
    //   cmds: _cmds
    // }
  },

  initItemArray () {
    // 当前设备信息
    let _devInfo = app.globalData.devInfo || {};
    let {t, d, a, id} = _devInfo;
    // 设备房间名称
    let _rn = app.globalData.devs[id]?.n || "";

    // 所有群组信息
    let _allGroups = bizUtil.getAllDevsFilter(app.globalData.devs, undefined, "groups");
    // 取得设备加入的群组信息
    let _fixGroups = this.getAllFixGroupsByDevInfo(_devInfo, _allGroups);
    this.fixGroups = _fixGroups.values || [];

    // 取得设备的配置参数 由于这里要取得群组主设备的名称,所以额外传了群组列表
    let _paramConfig = this.getAllConfigByDevInfo(_devInfo, _allGroups);
    this.configParamArray = _paramConfig.values || [];

    // 取得所有的参数信息
    let _itemArray = this.getAllItems({
      a: a,
      rn: _rn,
      configTip: _paramConfig.tips.join(","),
      groupTip: _fixGroups.tips
    });
    this.setData({
      devItems: _itemArray
    })
  },

  getAllItems ({a, rn, configTip, groupTip}) {
    return [{
      title: "地址",
      tip: a,
      result: 0 // 0未设置 1成功 2失败
    }, {
      title: "房间",
      tip: rn || "",
      result: 0
    }, {
      title: "参数设置",
      tip: configTip || "",
      result: 0
    }, {
      title: "群组",
      tip: groupTip || "",
      result: 0
    }]
  },

  getTimeStamp () {
    return Math.round(Date.now());
  },

  // 开始设置
  startReplaceDev () {
    let _oldDevInfo = app.globalData.devInfo;
    let _currentDevInfo = this.getNewDevInfo();
    // 保存之前设备的房间id
    this.oldId = _oldDevInfo?.id || 0;
    this.oldD = _oldDevInfo?.d || "";
    console.log("startReplaceDev _currentDevInfo = ", _currentDevInfo, _oldDevInfo.a, this.getTimeStamp());
    // 先修改短地址
    this.changeAddrByDevInfo(_currentDevInfo, _oldDevInfo.a).then(res => {
      console.log("changeAddrByDevInfo res = ", res, this.getTimeStamp());
      this.updateItemIndex(1);
      // 删除设备信息
      // 把老设备信息加入到新设备中
      this.afterReplaceAddr(_oldDevInfo, _currentDevInfo);

      this.modefyDevice(_currentDevInfo, _oldDevInfo.id).then(res => {
        console.log("modefyDevice res = ", res, this.getTimeStamp());
        this.updateItemIndex(2);
        
        if (this.oldId != _currentDevInfo.id) {
          // 把房间信息更新
          this.afterReplaceRoom(_currentDevInfo, this.oldId);
        }

        console.log("setParamCommand this.configParamArray = ", this.configParamArray, this.getTimeStamp());
        // 再修改参数
        this.setParamCommand(_currentDevInfo, this.configParamArray).then(res => {
          console.log("setParamCommand res = ", res, this.getTimeStamp());
          this.updateItemIndex(3);

          console.log("sendGroupByDevInfo this.fixGroups = ", this.fixGroups, this.getTimeStamp());
          // 再设置群组
          this.sendGroupByDevInfo(_currentDevInfo, this.fixGroups).then(res => {
            console.log("sendGroupByDevInfo res = ", res, this.getTimeStamp());
            this.updateItemIndex(4);

            this.afterGroupSet(this.fixGroups, _currentDevInfo?.d, this.oldD);
          });
        });
      });
    }).catch(e => {
      console.error("startReplaceDev e = ", e);
    })
  },
  // 短地址被替换后 删除设备、删除固定群组中的dids、删除场景中的
  afterReplaceAddr (oldDev, newDev) {
    if (!oldDev || !newDev) {
      return;
    }
    let {id: oldId, d: oldD, a: oldA, e} = oldDev;
    let _extendJson = Object.assign({}, e || {});
    let _roomDevs = (app.globalData.devs[this.oldId]?.devs || []);
    let _devIndex = _roomDevs.findIndex(item => item.d == oldD);
    console.error("afterReplaceAddr _devIndex = ", _devIndex);
    if (_devIndex >= 0) {
      _roomDevs.splice(_devIndex, 1);
    }
    // 新设备的短地址修改
    newDev.a = oldA;
    newDev.e = _extendJson;
    // 当前设备信息
    app.globalData.devInfo = newDev;
  },
  afterReplaceRoom (newDev, oldId) {
    if (!newDev || newDev.id == oldId) {
      return;
    }
    let {d} = newDev;
    let _roomDevs = (app.globalData.devs[newDev.id]?.devs || []);
    let _index = _roomDevs?.findIndex(item => item.d == d);
    if (_index >= 0) {
      _roomDevs.splice(_index, 1);
    }
    newDev.id = oldId;
    ((app.globalData.devs[oldId] || {}).devs || []).push(newDev);
  },
  afterGroupSet (fixGroups, newD, oldD) {
    if (!fixGroups || fixGroups.length == 0) {
      return;
    }
    for (let fixGroup of fixGroups) {
      let _dids = fixGroup?.dids || [];
      if (_dids.length == 0) {
        continue;
      }
      let _index = _dids.indexOf(oldD);
      if (_index < 0) {
        continue;
      }
      _dids.splice(_index, 1, newD);
    }
  },

  updateItemIndex (index) {
    this.setData({
      stepIndex: index
    })
  },

  // 修改设备短地址 devInfo是要替换的设备，addr是被替换设备的短地址
  changeAddrByDevInfo (devInfo, newAddr) {
    return new Promise((resolve, reject) => {
      app.globalData.blsdk.changeAddrByDevInfo(devInfo, newAddr, (res) => {
        console.error("changeAddrByDevInfo res = ", res);
        res >= 0 ? resolve(res) : reject(res);
      })
    })
  },
  // 修改设备房间
  modefyDevice (devInfo, newRoomId) {
    return new Promise((resolve, reject) => {
      if (newRoomId == devInfo.id) {
        resolve(newRoomId);
        return
      }
      app.globalData.blsdk.modefyDevice(devInfo, newRoomId, (res) => {
        console.error("modefyDevice res = ", res);
        res >= 0 ? resolve(res) : reject(res);
      })
    })
  },

  // 下发参数
  async setParamCommand (devInfo, cmdArray) {
    // return new Promise((resolve, reject) => {
      if (!!cmdArray?.length) {
        // 如果没有修改参数，则直接修改群组
        for (let item of cmdArray) {
          let {key, value} = item;
          // 下发炫彩灯带命令 下发缓期缓灭、配网、锁定、来电勿扰等扩展信息
          if (['getXuancaiParams', 'getConfigLock', 'getNoDisturb', 'getSlowUpDown'].indexOf(key) >= 0) {
            // 设置缓起缓灭
            let {params, config} = this[key](devInfo, value);
            await this.deviceControl(devInfo, params, config);
          } else if ("mic" == key) {
            // 下发主设备命令
            await this.setMainDevice(devInfo, value);
          }
        }
      }
    //   resolve();
    // });
  },
  // setParamCommandDetail () {},
  // 下发炫彩灯带命令
  deviceControl (devInfo, cmd, config) {
    return new Promise((resolve, reject) => {
      let _result = app.globalData.blsdk.controlWithDevice(devInfo, cmd, config);
      if (!_result || !_result.then) {
        reject(_result);
      }
      setTimeout(() => {
        resolve(0);
      }, 300);
    });
  },
  getXuancaiParams (devInfo) {
    if (!devInfo) {
      return [];
    }
    let {beads_number, beads_sort, ic_type} = devInfo?.e?.lightDazzle || {};
    // Data字段描述
    // Data[0] = 0xE1
    // Data[1] = IC类型
    // Data[2] = RGB线序
    // Data[3] = 预留
    // Data[4] = 预留
    // Data[5] = 预留
    // Data[6] = effect_type
    // Data[7] = speed
    // Data[8] = 灯珠数量高字节
    // Data[9] = 灯珠数量低字节
    let _beadsNumber = Number(beads_number) || 0;
    let _highCount = (_beadsNumber >> 8) & 0xff;
    let _lowCount = (_beadsNumber) & 0xff;

    // 这里effect_type有个bug,不能按照current下发 是下标
    let _effectType = Number(this.data.current[0]) || 0;
    _effectType = Number((this.data.modeArray[_effectType]?.split(" ") || [])[0]) || 0;
    // 变化速度
    let _speed = Number(this.data.speedSliders.percent) || 0;

    let _cmdArray = [0xE1, Number(ic_type) || 0, Number(beads_sort) || 0, 0, 0, 0, _effectType, _speed, _highCount, _lowCount];
    return {
      params: _cmdArray,
      config: undefined
    };
  },
  // 下发主设备命令
  setMainDevice (devInfo, groupId) {
    let {d, a} = devInfo || {};
    // 使能
    let _paramJson = {
      enable: !!d,
      a: a || 0,
      gid: groupId
    };
    return new Promise((resolve, reject) => {
      app.globalData.blsdk.mainGroupDevSet(_paramJson, (res) => {
        console.log("setMainDevice res = ", res);
        res > 0 ? resolve(res) : reject(res);
      })
    })
  },
  // 下发配网锁定
  getConfigLock (devInfo, value) {
    let _byte0 = 0
    let _byte1 = devInfo.a || 0;
    return {
      params: {
        tlv: 0,
        subType: (1 << 6) + 10,
        cmds: [_byte0, _byte1, value]
      },
      config: {
        isNewUart: true, // 新协议
      }
    };
  },
  // 下发来电勿扰
  getNoDisturb (devInfo, value) {
    let _byte0 = 0
    let _byte1 = (devInfo.a || 0);
    return {
      params: {
        tlv: 0, 
        subType: (1 << 6) + 11,
        // 0-基于短地址，1-基于组 短地址或组地址
        cmds: [_byte0, _byte1, Number(value) || 0]
      },
      config: {
        isNewUart: true, // 新协议
      }
    }
  },
  // 下发缓期缓灭
  getSlowUpDown (devInfo, value) {
    let _subType = 6;
    let _cmd = [2, Number(devInfo.a) || 0, Number(value) || 0];
    return {
      params: {
        tlv: 0, 
        // 6 - 基于短地址的控制 7 - 基于组地址的控制
        subType: (1 << 6) + _subType,
        cmds: _cmd
      },
      config: {
        isNewUart: true, // 新协议
      }
    }
  },

  // 设置群组
  async sendGroupByDevInfo (devInfo, groups) {
    console.log("sendGroupByDevInfo groups = ", groups, groups.length, this.getTimeStamp());
    // return new Promise((resolve, reject) => {
      if (groups.length) {
        let {devArray, gidArray} = this.getDevArrayInGroup(groups, devInfo);
        console.log("sendGroupByDevInfo devArray, gidArray = ", devArray, gidArray, this.getTimeStamp());
        for (let index in devArray) {
          let array = devArray[index];
          if (!array || array.length == 0) {
            continue;
          }
          console.log("sendGroupByDevInfo gidArray[index] = ", gidArray[index], array, this.getTimeStamp());
          // 下发主设备命令
          await this.sendGroupByDevInfoDetail(gidArray[index], array);
          console.log("sendGroupByDevInfo getTimeStamp = ", this.getTimeStamp());
        }
      }
      // // 如果没有修改参数，则直接修改群组
      // resolve();
    // })
  },
  sendGroupByDevInfoDetail (gid, devs) {
    console.log("sendGroupByDevInfoDetail gid, devs, getTimeStamp = ", gid, devs, this.getTimeStamp());
    return new Promise((resolve, reject) => {
      app.globalData.blsdk.fixedGroupCtrl(gid, devs, {}, () => {
        console.log("sendGroupByDevInfoDetail before getTimeStamp = ", this.getTimeStamp());
        setTimeout(() => {
          console.log("sendGroupByDevInfoDetail after getTimeStamp = ", this.getTimeStamp());
          resolve(0);
        }, 300);
      });
    });
  },
  // 取得设备信息
  getDevArrayInGroup (groupInfos, addDevInfo) {
    let _devArray = [];
    let _groupIdArray = [];
    if (!!groupInfos && groupInfos.length > 0) {
      let _dJson = bizUtil.getDidMapInJson(app.globalData.devs);
      console.log("getDevArrayInGroup _dJson = ", _dJson);
      for (let group of groupInfos) {
        let {dids, gid} = group || {};
        console.log("getDevArrayInGroup dids, gid = ", dids, gid);
        if (!dids || dids.length == 0) {
          continue;
        }
        // 是否包含新设备，如果包含则不需要发
        let _containNewDev = dids.indexOf(addDevInfo.d) >= 0;
        if (_containNewDev) {
          continue;
        }
        let _tmpArray = [addDevInfo];
        for (let d in dids) {
          let _t = _dJson[d];
          if (!_t) {
            continue;
          }
          _tmpArray.push(_t);
        }
        console.log("getDevArrayInGroup _tmpArray, gid = ", _tmpArray, gid);
        _devArray.push(_tmpArray);
        _groupIdArray.push(gid);
      }
    }
    return {
      devArray: _devArray,
      gidArray: _groupIdArray
    };
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },


  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 判断是否成功
    if (this.data.stepIndex == 0) {
      return;
    }
    // 保存数据
    app.saveAllDevsMap();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  actBtnClick (e) {

    // 如果没有完成，则返回
    if (this.data.stepIndex < this.data.devItems.length) {
      return;
    }

    let {index} = e.currentTarget.dataset;
    let {key} = this.data.actions[index];
    if (key == "next") {
      // 判断是否有自定义的场景
      if (this.customScenes.length == 0) {
        ability.redirectTo("/minePages/toastPage/index?type=2&imgKey=success&count=5&extend=" + JSON.stringify({
          content: "替换成功",
          contents: []
        }))
      } else {
        // 当前设备状态
        app.globalData.devInfo = this.getNewDevInfo();
        // 临时缓存自定义场景
        app.globalData.tmpData.tmpCustomScenes = this.customScenes;
        // 跳转页面
        ability.redirectTo("../custom/index?from=replaceDev&oldD=" + this.oldD)
      }
    }
  }
})