// device/light/devProperty/replace/list/index.js
import bizUtil from '../../../../../utils/bizUtil'
import ability from '../../../../../utils/classDef/ability'
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "面板选择"
      }
    },
    devList: [],
    selectIndex: -1,
    roomChanged: false, // 是否弹出选择房间
    rooms: [],
    roomInfo: {
      id: 0, // 默认全屋
      n: '全屋'
    },

    roomJson: {}, // 所有房间json
    isFloor: false, // 是否多楼层

    devTypeStr: "", // 是否需要某一品类的 暂时没用到
    from: "", // 是否从某页面进入  除了场景，还有双控设备
    actions: [], // 底部按钮
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    console.log("onLoad options = ", options);
    // 页面来源
    let {from} = options || {};
    this.from = from;

    // 保存设备信息
    this.devInfo = app.globalData.devInfo;

    // 初始化所有房间的roomJSON
    let _roomJson = this.getRoomJson();
    // 初始化多楼层信息
    let _isFloor = app.globalData.familyInfo?.floor >= 0;
    // 初始化房间信息
    let _roomId = (app.globalData.roomInfo || {}).id;
    if (_roomId != this.data.roomInfo.id && _roomId != undefined) {
      let _roomInfo = app.globalData.devs[_roomId] || app.globalData.devs[0];
      this.data.roomInfo.id = _roomInfo.id;
      this.data.roomInfo.n = _roomInfo.n;
    }
    // 取得当前楼层下的房间列表
    let _rooms = this.getRoomsByFloor();
    
    // 如果是双控，则取得每个did对应的双控个数
    if (this.isFromPwrAvalable()) {
      // 所有设备d对应的双控个数，最大4个
      this.allDMap = this.getAllDMap();
    }
    
    // let _devList = this.getListByRoomId(this.data.roomInfo.id);

    this.setData({
      isFloor: _isFloor,
      roomJson: _roomJson,
      roomInfo: this.data.roomInfo,
      rooms: _rooms,
      from: from,
      // devList: _devList,
      actions: this.getAllActions()
    });
  },

  getAllActions () {
    return [{
      n: "新设备",
      key: "addNewDev",
      classStr: "saveBtn"
    }, {
      n: "下一步",
      key: "next",
      classStr: "saveBtn"
    }]
  },

  // 取得所有设备集合
  getAllDMap () {
    let _result = {};
    // 所有设备列表
    let _allDevArray = bizUtil.getAllDevsFilter(app.globalData.devs);
    // 筛选出双控设备
    let {virtualPanel} = bizUtil.getAllPidByArray(_allDevArray);
    if (!virtualPanel || virtualPanel.length == 0) {
      return _result;
    }
    for (let item of virtualPanel) {
      let {bindDevices} = item?.e || {};
      if (!bindDevices || bindDevices.length == 0) {
        continue;
      }
      for (let child of bindDevices) {
        let {d, key} = child || {};
        _result[d] = _result[d] || 0;
        _result[d] += 1; // 个数+1
      }
    }
    return _result;
  },

  getRoomJson () {
    // 取得多楼层的json数据
    let {roomJson} = bizUtil.getRoomsByDevs(app.globalData.devs);
    return roomJson;
  },

  // 初始化房间
  getRoomsByFloor () {
    // 所有设备
    let _allDevs = app.globalData.devs || {};
    // 当前楼层
    let _floor = app.globalData.familyInfo?.floor || 0;
    // 根据多楼层取得数据
    let _devs = bizUtil.getDevsByFloor(_allDevs, _floor);
    let {rooms} = bizUtil.getRoomsByDevs(_devs);
    return rooms;
  },

  // 是否从双控进入
  isFromPwrAvalable () {
    return this.from == "pwrAvalible"
  },

  // 取得设备品类
  getDevVersion (devInfo) {
    if (!devInfo) {
      return 0;
    }
    let _array = devInfo.split(".") || [];
    return _array[_array.length - 1] || 0;
  },

  getListByRoomId (roomId) {
    let _roomId = roomId || 0;
    console.log("getListByRoomId roomId = ", roomId);

    let {t, v, d} = app.globalData.devInfo || {};
    let _version = this.getDevVersion(v); // 当前设备固件版本号
    // 如果页面来源是双控
    let _allDevList = []; // bizUtil.getDevsOrGroupsByRoomId(_roomId, app.globalData.devs);
    // if (_roomId == 0) {
    //   // 如果是全屋,则显示没有分配房间的设备
    //   _allDevList = app.globalData.devs[_roomId]?.devs || []; // _allDevList.filter(item => !item.id)
    // } else {
      _allDevList = bizUtil.getDevsOrGroupsByRoomId(_roomId, app.globalData.devs);
    // }

    let _devList = _allDevList.filter(item => d != item.d && item.t == t && this.getDevVersion(item.v) >= _version);
    
    // 先把_array排序好 后面的就不用排序了
    return bizUtil.sortArrayByChar(_devList, "n");
  },
  // 根据设备的e: bindDevices中数据去重
  dealFilterByArray (array) {
    if (!array || array.length == 0) {
      return [];
    }
    let {bindDevices} = app.globalData.devInfo?.e || {};
    console.log("dealFilterByArray array, bindDevices = ", array, bindDevices);
    // 根据d去重
    let _dArray = (bindDevices || []).map(item => item.d);
    console.log("dealFilterByArray _dArray = ", _dArray);
    // 去重
    let _result = array.filter(item => _dArray.indexOf(item.d) < 0);
    console.log("dealFilterByArray _result = ", _result);
    return _result;
  },
  // 每个设备是否支持双控，如果支持，默认最大4个
  checkByAbility (array) {
    if (!array || array.length == 0) {
      return [];
    }
    return array.filter(item => {
      let {t, v, d} = item;
      let {virtualPanel} = bizUtil.getJsonByAblitys(v, bizUtil.getDevTypeStr(t), {t});
      console.log("checkByAbility d, virtualPanel = ", d, virtualPanel);
      if (!virtualPanel) {
        // 不支持能力
        return false;
      }
      if (!this.allDMap) {
        return true;
      }
      // 支持但是4个限制
      let _count = this.allDMap[d];
      if (_count >= 4) {
        return false;
      }
      return true;
    })
  },
  setTipByDMap (array, dMap) {
    if (!array) {
      return [];
    }
    if (!dMap) {
      bizUtil.setArrayItem(array, ['tip'], ['未添加']);
      return array;
    }
    for (let item of array) {
      let {d} = item || {};
      item.tip = ""; // (dMap[d]) ? (dMap[d] + '个按键同步') : '未添加';
    }
    return array;
  },


  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

    // 防止配置新设备时改变了当前设备信息
    if (!!this.devInfo) {
      app.globalData.devInfo = this.devInfo;
    }

    // 取得设备
    let _devList = this.getListByRoomId(this.data.roomInfo.id);
    this.setData({
      devList: _devList
    });

    // // 防止配置新设备时改变了当前设备信息
    // if (!this.devInfo) {
    //   return;
    // }
    // app.globalData.devInfo = this.devInfo;
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  selectDev (e) {
    let {index} = e.currentTarget.dataset;
    // let {selected, rt} = !this.data.devList[index] || {};
    // this.data.devList[index].selected = selected;
    // let _text = 'devList['+ index +'].selected';

    // this.data.selectIndex = index;
    this.setData({
      // [_text]: selected,
      selectIndex: index
    })
  },

  chooseRoom (e) {
    let {rid} = e.currentTarget.dataset;
    let _roomInfo = app.globalData.devs[rid || 0];
    this.data.roomInfo.n = _roomInfo.n;
    this.data.roomInfo.id = _roomInfo.id;
    
    // 隐藏
    this.hidePop();
    // 更新设备列表数据
    let _devList = this.getListByRoomId(this.data.roomInfo.id);
    this.setData({
      roomInfo: this.data.roomInfo,
      devList: _devList
    });
  },
  showPop () {
    this.hidePop(true);
  },
  hidePop (isOpen) {
    this.data.roomChanged = isOpen || false;
    this.setData({
      roomChanged: this.data.roomChanged
    })
  },
  hidePops () {
    this.hidePop();
  },

  actBtnClick (e) {
    let {index} = e.currentTarget.dataset;
    let {key} = this.data.actions[index] || {};
    if (key == "next") {
      this.next();
    } else if (key == "addNewDev") {
      this.jumpConfig();
    }
  },

  next () {
    let _selectIndex = this.data.selectIndex;
    console.log("next this.data._selectIndex = ", _selectIndex);
    if (_selectIndex < 0) {
      return ability.showToast({
        title: "请选择设备"
      });
    }
    // 保存选择的设备信息
    let _devInfo = this.data.devList[_selectIndex] || {};
    app.globalData.tmpData.devInfo = _devInfo;
    console.log("next _devInfo = ", _devInfo, app.globalData.tmpData.devInfo);
    // 通用下发页面
    ability.redirectTo("../common/index");
  },
  jumpConfig () {
    ability.navigateTo("/addPages/bleIndex/index?from=replace")
  },
  getExtendParamsByFrom () {
    let _result = "";
    return _result;
  }
})