// device/light/funcSet/index.js

const ability = require("../../../utils/classDef/ability")
const bizUtil = require("../../../utils/bizUtil");
const CONFIG = require("./jsConfig/config")
const historyCommon = require("../../jsConfig/history")
const app = getApp();

// 初始化picker 在setData中设置不起作用 22-09-20
var countList = [];
for (let i = 10; i <= 2400; i++) {
  countList.push(i + "");
}

Page({

  /**
   * 页面的初始数据
   */
  data: {
    bleUnavailable: false,
    topList: [],
    bottomList: [],
    extendList: [], // 灯珠总数等幻彩灯设置

    // 底部actionsheet
    actionObj: {
      show: false,
      list: [],
      title: ""
    },
    pickObj: {
      key: "ledCount",
      show: false,
      list: countList,
      value: 0
    },
    // 灯珠总数输入框
    inputDialogShow: false,
    inputConfig: {
      title: "灯珠总数",
      placeholder: "数量范围10-2400",
      min: 10,
      max: 2400,
      value: 10, // 默认值
      buttons: [{
        text: "取消",
        className: "color333",
        key: "cancel"
      }, {
        text: "确定",
        className: "colorF78D3F",
        key: "confirm"
      }]
    }
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title, remote, devType} = options || {};
    // 是否支持清除遥控器指令
    this.supportRemote = remote != 0;
    // 设备品类
    this.devTypeStr = this.getDevTypeStr();
    // 设备信息
    if (app.globalData?.tmpData?.roomInfo) {
      this.isRoom = true;
      // 房间控制
      this.curInfo = app.globalData?.tmpData?.roomInfo;
    } else if (app.globalData.groupInfo) {
      this.isGroup = true;
      // 固定群组
      this.curInfo = app.globalData.groupInfo
    } else {
      this.curInfo = app.globalData.devInfo;
    }
    // 设置标题
    this.setNavTitle(decodeURIComponent(title || ""));
  },
  setNavTitle (str) {
    ability.setNavigationBarTitle(str || "")
  },
  getDevTypeStr () {
    let {t, v} = app.globalData.devInfo || {};
    return bizUtil.getDevTypeStr(t);
  },
  getTopAndBottomList (supportRemote, devTypeStr) {
    let _list = this.getAllTopList();
    let {t, v} = this.curInfo || {};
    // 能力集合
    let _abilityJson = bizUtil.getJsonByAblitys(v, devTypeStr, {t});
    console.log("getTopAndBottomList _abilityJson, devTypeStr = ", _abilityJson, devTypeStr);
    let _topList = [];
    if (!!devTypeStr) {
      // 是否支持背光灯
      let _supportBacklight = this.supportBackLight(devTypeStr, _abilityJson);
      // 默认都不支持背光灯
      let _filterArray = _supportBacklight ? [] : ["backlight"];
      // 不支持接触遥控器绑定
      if (!supportRemote) {
        _filterArray.push("remote");
      }
      // 是否支持alexa设置
      if (!_abilityJson.alexaSet) {
        _filterArray.push("alexa");
      }
      // 是否支持mic收音设置 for test 默认都支持识音
      if (!_abilityJson.mic) {
        _filterArray.push("mic");
      }
      _topList = _list.filter(item => _filterArray.indexOf(item.key) < 0);
    }
    // 底部按钮列表
    let _bottomList = this.getBottomListByType(devTypeStr, _abilityJson);
    // 炫彩灯额外列表
    let _extendList = this.getExtendListByType(devTypeStr, _abilityJson, this.curInfo);
    
    return {
      topList: _topList,
      bottomList: _bottomList,
      extendList: _extendList,
      // pickObj: this.data.pickObj
    }
  },
  supportBackLight (devTypeStr, abilityJson) {
    if ("panel" == devTypeStr) {
      return abilityJson?.backlight || false;
    }
    return false;
  },
  getAllTopList () {
    // 配网锁定、来电勿扰 凌动开关支持群组和单个设备
    return [
      // {
      //   title: "识音",
      //   value: "",
      //   bindFunc: "topClick",
      //   key: "mic"
      // }, 
      {
        title: "Alexa设置",
        value: "",
        bindFunc: "topClick",
        key: "alexa"
      }, {
        title: "背光灯",
        value: "",
        bindFunc: "topClick",
        key: "backlight"
      }, 
      // {
      //   title: "网关授权",
      //   value: "",
      //   bindFunc: "topClick"
      // }, 
      {
        title: "清除遥控器配对",
        value: "",
        bindFunc: "topClick",
        key: "remote"
      }
    ]
  },
  getBottomListByType (devTypeStr, abilityJson) {
    console.log("getBottomListByType devTypeStr = ", devTypeStr)

    // 如果是插卡取电则有参数设置的条目
    if (devTypeStr == "powerPlug") {
      return this.getPowerPlugBottomList();
    } else if (devTypeStr == "temphumidity") {
      return this.getTemphumidityBottomList();
    } else if (devTypeStr == "fan") {
      return this.getFanBottomList();
    } else if (devTypeStr == "thermostat") {
      return this.getThermostatBottomList();
    }

    let _list = this.getAllBottomList();
    // 如果是群组控制则允许来电勿扰和配网锁定
    if (!devTypeStr && Object.keys(abilityJson).length == 0) {
      // 默认配置
      let _json = {"configLock": false, "noDisturb": false};
      // 判断是否群组
      if (this.isGroup || this.isRoom) {
        // 取得群组能力最大集
        // 存在的设备列表
        let _devExistArray = this.getAllCtrlDevsArray();
        // 判断设备单个还是批量或者群组
        let {maxStatus, versions} = this.getFuncList(_devExistArray) || {};
        // 保存能力集合
        let _maxAbilityJson = bizUtil.getMaxAbilityByVs(versions) || {};

        _json.configLock = _maxAbilityJson?.configLock || false;
        _json.noDisturb = _maxAbilityJson?.noDisturb || false;
        _json.slowUpDown = _maxAbilityJson?.slowUpDown || false;
      }
      return _list.filter(item => _json[item.key]);
    }
    if (devTypeStr != "light" || !abilityJson) {
      return [];
    }
    return _list.filter(item => abilityJson[item.key]);
  },



  // 取得当前页面控制设备列表数组
  getAllCtrlDevsArray () {
    if (this.isGroup) {
      // 固定群组
      // 是否群组设备
      let {dids} = app.globalData.groupInfo || {};
      // 取得房间下所有的d对应的详细信息
      let _didMaps = bizUtil.getDidMapInJson(app.globalData.devs);
      // 群组设备列表
      let _groupDevs = dids || [];
      // 群组设备列表详细
      return _groupDevs.map(did => _didMaps[did]);
    } else if (app.globalData.tgroupId && (app.globalData.tmpData?.tdevs)) {
      // 临时群组
      return app.globalData.tmpData?.tdevs || [];
    } else {
      let _devInfo = app.globalData.devInfo;
      if (_devInfo) {
        // 单个设备控制
        return [_devInfo];
      } else if (this.isBatchCtrl()) {
        // 5个一组的批量控制
        return app.globalData?.tmpData?.tdevs || [];
      } else if (this.isRoom) {
        // 房间控制
        return this.getDevArrayByRoomIdInJson(app.globalData.devs, (app.globalData.roomInfo || {}).id)
      }
    }
  },
  // 获取是否批量控制
  isBatchCtrl () {
    return (this.data.groupIds.length > 0) && (app.globalData.devInfo == undefined || Object.keys(app.globalData.devInfo).length == 0);
  },
  // 根据房间id取得json中设备集合
  getDevArrayByRoomIdInJson (json, roomId) {
    let _devs = [];
    if (!json) {
      return _devs;
    }
    if (roomId == 0) {
      // 全屋设备
      _devs = bizUtil.getAllDevsFilter(json) || [];
    } else if (roomId % 32 == 0) {
      // 取得楼层数据
      let _devJson = bizUtil.getDevsArrayByFloor(json, bizUtil.getFloorByRoomId(roomId)) || {};
      _devs = bizUtil.getAllDevsFilter(_devJson);
    } else {
      // 全屋设备
      _devs = bizUtil.getAllDevsFilter(json, roomId) || [];
    }
    let {t} = this.curInfo || {};
    // 取得照明的设备
    let _json = bizUtil.getAllPidByArray(_devs);
    console.log("getDevArrayByRoomIdInJson _json, t = ", _json, t);
    return _json[t] || [];
  },

  // 根据品类取得功能集合
  getFuncList (devs) {
    // 被控设备集合
    let _devs = devs || [];
    
    // 最终集合
    let versions = [], minStatus= {}, maxStatus = {};
    // let _minCount = 20, _maxCount = -1;
    // 遍历数据取得集合
    for (let index in _devs) {
      let {t, v} = _devs[index] || {};
      // 如果版本号v不存在，则continue
      if (v == undefined) {
        continue;
      }

      // 如果版本号一致，则continue
      if (versions.indexOf(v) >= 0) {
        continue;
      }
      // 添加版本分支集合
      versions.push(v);
      // 取得能力集
      let _devStatus = bizUtil.getFuncByPid({}, t);
      // 添加最大能力集
      Object.assign(maxStatus, _devStatus);
    }
    console.log("getFuncList maxStatus = ", maxStatus);
    return {
      versions: versions, // 版本能力集合
      maxStatus: maxStatus
    };
  },

  getThermostatBottomList () {
    let _list = [];
    // 温控器数据处理
    let {hot, wind, airhotwind} = this.getStatus(app.globalData.devInfo);
    console.log("hot, wind = ", hot, wind);
    // 有空调和地暖的时候
    if (!!hot && (airhotwind == 4 || airhotwind == 7)) {
      // 是否支持制热设备和通风设备
      let _hotValue = decodeURIComponent(hot);
      console.log("_hotValue = ", _hotValue);
      _list.push({
        title: '制热设备',
        key: 'hot',
        bindFunc: 'bottomClick', // 'jumpThermostat',
        value: this.getTipByValue(_hotValue),
        valInt: _hotValue
      })
    }
    // 有空调和新风
    if (!!wind && (airhotwind == 5 || airhotwind == 7)) {
      let _windValue = decodeURIComponent(wind);
      console.log("_windValue = ", _windValue);
      _list.push({
        title: '通风设备',
        key: 'wind',
        bindFunc: 'bottomClick', // 'jumpThermostat',
        value: this.getTipByValue(_windValue),
        valInt: _windValue
      })
    }
    return _list;
  },
  // 取得温控器通风和制热设备的名称显示
  getTipByValue (valueStr) {
    let _tip = {};
    if (!valueStr) {
      return _tip;
    }
    let _vals = valueStr.split(",");
    let _configs = {
      air: "空调",
      wind: "新风",
      hot: "地暖"
    };
    let _tipArray = _vals.map(item => _configs[item]);
    let _tipLength = _tipArray.length;
    if (_tipLength == 1) {
      return _tipArray[0]
    } else if (_tipLength == 0) {
      return "";
    }
    return _tipArray.join(",")
  },
  // 取得温控器的数据
  getStatus (devInfo) {
    // 保存的设备状态
    let _allDevStatus = app.globalData.allStatus || {};
    let {d, a} = devInfo || {};
    // 设备状态
    return _allDevStatus[d + "_" + a] || {};
  },
  getFanBottomList () {
    let {buzzer} = this.getStatus(app.globalData.devInfo) || {};
    console.log("getFanBottomList buzzer = ", buzzer);
    return [{
      title: "提示音",
      key: "buzzer",
      bindFunc: "bottomClick",
      value: this.getBuzzerTitleByValue(buzzer),
      valInt: buzzer || 0
    }];
  },
  getBuzzerTitleByValue (buzzer) {
    return buzzer ? "打开" : "关闭";
  },
  getTemphumidityBottomList () {
    let {unit} = this.getStatus(app.globalData.devInfo) || {};
    return [{
      title: "温度单位",
      key: "unit",
      bindFunc: "bottomClick",
      value: bizUtil.getUnitByValue(unit),
      valInt: unit || 0
    }];
  },
  getPowerPlugBottomList () {
    let {delayOff} = this.getStatus(app.globalData.devInfo) || {};
    let _valInt = 0;
    if (delayOff == undefined) {
      _valInt = -1;
    } else {
      _valInt = Number(delayOff) || 0;
    }
    return [{
      title: "参数设置",
      key: "paramSet",
      value: _valInt < 0 ? "" : (_valInt ? "延时断电" : "不断电"),
      valInt: _valInt,
      bindFunc: "bottomClick"
    }];
  },
  getAllBottomList () {
    // 配网锁定、来电勿扰 凌动开关支持群组和单个设备
    return [{
      title: "配网锁定",
      key: "configLock",
      value: "",
      bindFunc: "bottomClick"
    }, {
      title: "上电勿扰",
      value: "",
      key: "noDisturb",
      bindFunc: "bottomClick"
    }, 
    // {
    //   title: "凌动TM开关",
    //   value: "",
    //   bindFunc: "bottomClick"
    // }, 
    {
      title: "缓起缓灭",
      value: "",
      key: "slowUpDown",
      bindFunc: "bottomClick"
    }];
  },
  // 这里必须炫彩灯珠的数据，因为有标题
  getExtendListByType (devTypeStr, abilityJson, devInfo) {
    console.log("getExtendListByType devTypeStr, abilityJson = ", devTypeStr, abilityJson, devTypeStr, abilityJson);
    if (devTypeStr != "light" || !abilityJson?.xuancai) {
      return [];
    }
    let _list = this.getAllExtendList();
    let {beads_number, beads_sort, ic_type} = devInfo?.e?.lightDazzle || {};
    console.log("getExtendListByType beads_number, beads_sort, ic_type = ", beads_number, beads_sort, ic_type);
    _list[0].valueInt = Number(beads_number) || 10;
    _list[0].value = _list[0].valueInt;

    _list[1].valueInt = Number(ic_type) || CONFIG.icType[0].value;
    let _obj = this.getInArray(CONFIG.icType, "value", _list[1].valueInt);
    _list[1].value = _obj.text || "";

    _list[2].valueInt = Number(beads_sort) || CONFIG.lightSort[0].value;
    let _obj2 = this.getInArray(CONFIG.lightSort, "value", _list[2].valueInt);
    _list[2].value = _obj2.text || "";
    return _list;
  },
  getInArray (array, key, value) {
    if (!array || array.length == 0) {
      return {};
    }
    let _array = array.filter(item => item[key] == value);
    return _array[0] || {};
  },
  getAllExtendList () {
    return [{
      title: "灯珠总数",
      key: "ledCount",
      value: "",
      valueInt: 0,
      bindFunc: "extendClick",
      type: ""
    }, {
      title: "IC类型",
      value: "",
      key: "icType",
      valueInt: 0,
      bindFunc: "extendClick"
    }, {
      title: "灯条排序",
      value: "",
      key: "lightSort",
      valueInt: 0,
      bindFunc: "extendClick"
    }];
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 初始化功能数据
    let {topList, bottomList, extendList} = this.getTopAndBottomList(this.supportRemote, this.devTypeStr);
    // 如果有炫彩灯额外列表
    if (extendList.length > 0) {
      // 灯珠总数
      let _value = Number(extendList[0]?.valueInt) || 0;
      this.data.pickObj.value = _value - Number(countList[0]);
      console.log("getTopAndBottomList this.data.pickObj.value = ", this.data.pickObj.value, extendList[0].valueInt, countList[0]);
      // this.data.pickObj = this.showPickerConfig("ledCount", true);
      // 灯珠总数输入框
      this.data.inputConfig.value = _value;
    }

    this.setData({
      topList: topList,
      bottomList: bottomList,
      extendList: extendList,
      pickObj: this.data.pickObj,
      inputConfig: this.data.inputConfig
    })
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 保存设备信息
    this.data.extendList?.length > 0 && (app.saveAllDevsMap());
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  topClick (e) {
    this.detailItemClick(e, "topList")
  },
  bottomClick (e) {
    this.detailItemClick(e, "bottomList")
  },
  extendClick (e) {
    this.detailItemClick(e, "extendList")
  },
  detailItemClick (e, keyStr) {
    let {index} = e.currentTarget.dataset;
    let _list = this.data[keyStr];
    if (!_list || _list.length == 0) {
      return;
    }
    let {key, valInt, title} = _list[index];
    console.log("detailItemClick key = ", key, valInt);
    if (key == "backlight") {
      // 背光灯设置
      ability.navigateTo("/device/panel/backlight/index")
    } else if (key == "alexa") {
      // alexa设置
      this.showActionConfig(key);
    } else if (key == "configLock") {
      // 配网锁定
      this.showActionConfig(key);
    } else if (key == "noDisturb") {
      // 上电勿扰
      this.showActionConfig(key);
    } else if (key == "slowUpDown") {
      // 缓起缓灭
      this.showActionConfig(key);
    } else if (key == "remote") {
      // 清除遥控器配对
      this.unbindRemoteCtrl();
    } else if (key == "ledCount") {
      // 灯珠总数
      this.showInputDialog(key);
      // this.showPickerConfig(key);
    } else if (key == "icType") {
      // IC类型
      this.showActionConfig(key);
    } else if (key == "lightSort") {
      // 灯条排序
      this.showActionConfig(key);
    } else if (key == "paramSet") {
      this.setPowerPlugParams(valInt);
    } else if (key == "unit") {
      this.showActionConfig(key);
      // this.setPowerPlugParams(valInt);
    } else if (key == "buzzer") {
      this.showActionConfig(key);
    } else if (key == "hot") {
      this.jumpThermostat(key, valInt, title);
    } else if (key == "wind") {
      this.jumpThermostat(key, valInt, title);
    } else if (key == "mic") {
      // 照明设备收音功能 跳转收音功能页面
      this.jumpMic(key, title);
      // this.showActionConfig(key);
    }
  },
  jumpMic (key, title) {
    ability.navigateTo("./mic/index?title=" + encodeURIComponent(title || "") + "&key=" + (key || ""));
  },
  jumpThermostat (key, realValueStr, title) {
    ability.navigateTo("../../thermostat/setConfig/index?title=" + encodeURIComponent(title) + "&key=" + key + "&valueStr=" + encodeURIComponent(realValueStr));
  },
  // 插卡取电参数设置
  setPowerPlugParams (valInt) {
    ability.navigateTo("/device/panel/backlight/index?delayOff=" + (valInt || 0));
  },
  // 打开灯珠输入框
  showInputDialog (value) {
    this.setData({
      inputDialogShow: !!value || false
    })
  },
  // 显示actionsheet
  showPickerConfig (keyStr, noSet) {
    let {list} = this.getActionListByKey(keyStr);
    console.log("showPickerConfig list = ", list);
    this.data.pickObj.show = true;
    this.data.pickObj.key = keyStr;
    this.data.pickObj.value = this.data.pickObj.value || 0;
    this.data.pickObj.list = list;
    console.log("showPickerConfig pickObj = ", this.data.pickObj);
    if (noSet) {
      return this.data.pickObj;
    }
    this.setData({
      pickObj: this.data.pickObj
    });
  },
  showActionConfig (keyStr) {
    let {title, list} = this.getActionListByKey(keyStr);
    this.data.actionObj.title = title;
    this.data.actionObj.show = true;
    this.data.actionObj.key = keyStr;
    this.data.actionObj.list = list;
    console.log("showActionConfig actionObj = ", this.data.actionObj);
    this.setData({
      actionObj: this.data.actionObj
    });
  },
  micDetail (value, callback) {
    console.log("micDetail value = ", value);
    // 收音功能
    let _devInfo = app.globalData.devInfo;
    let _cmd = this.getMicParamsByValue(value);
    console.log("micDetail _cmd = ", _cmd);
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, _cmd);
    if (!callback) {
      return;
    }
    if (!_result?.then) {
      !!callback && (callback(_result));
      return;
    }
    _result.then((res) => {
      console.log("actDev res = ", res);
      !!callback && (callback(0));
    })
  },
  getMicParamsByValue (value) {
    // effect_type取值定义:
    // 0：关闭MIC功能
    // 1：打开MIC功能
    let _enable = Number(value) || 0;
    // 中间空出一个brgcw五个字节
    // 律动类型取值定义：
    // 0x00：七彩渐变律动
    // 0x01：七彩呼吸律动
    // 0x02：七彩跳变律动
    // 0x03：七彩频闪律动 
    let _type = _enable > 0 ? 0 : 0;
    // MIC灵敏度定义范围：
    // 1-100
    let _micValue = _enable > 0 ? 1 : 0;
    return [0xE2, 0, 0, 0, 0, 0, _enable, _type, _micValue];
  },
  alexaSetDetail (value) {
    app.globalData.blsdk.controlWithDevice(this.curInfo, {
      tlv: 0,
      subType: (1 << 6) + 2,
      cmds: [1, value || 0]
    }, {
      isNewUart: true // 新协议
    });
  },
  setStoreAfterControl (key, value, parentKey) {
    // 如果开启了配网
    this.updateStore (key, value, parentKey);
  },
  configLock (value) {
    let _isRoom = this.isRoom;
    let _isGroup = this.isGroup;
    let _byte0 = (_isRoom || _isGroup) ? 1 : 0
    let _byte1 = _isRoom ? (this.curInfo.id || 0) : (_isGroup ? (this.curInfo.gid || 0) : (this.curInfo?.a || 0))
    let _result = app.globalData.blsdk.controlWithDevice(this.curInfo, {
      tlv: 0,
      subType: (1 << 6) + 10,
      cmds: [_byte0, _byte1, value]
    }, {
      isNewUart: true, // 新协议
    });

    if (!_result || _result.then) {
      return;
    }
    // 设置缓存
    this.setStoreAfterControl("config_lock", value, "light");
  },
  noDisturb (value) {
    let _isRoom = this.isRoom;
    let _isGroup = this.isGroup;
    let _byte0 = (_isRoom || _isGroup) ? 1 : 0
    let _byte1 = _isRoom ? (this.curInfo.id || 0) : (_isGroup ? (this.curInfo.gid || 0) : (this.curInfo?.a || 0))
    let _result = app.globalData.blsdk.controlWithDevice(this.curInfo, {
      tlv: 0, 
      subType: (1 << 6) + 11,
      // 0-基于短地址，1-基于组 短地址或组地址
      cmds: [_byte0, _byte1, Number(value) || 0]
    }, {
      isNewUart: true, // 新协议
    });
    if (!_result || _result.then) {
      return;
    }
    // 设置缓存
    this.setStoreAfterControl("pwr_cut", value, "light");
  },
  slowUpDown (value) {
    let _isRoom = this.isRoom;
    let _isGroup = this.isGroup;
    let _subType = 6;
    let _cmd = [];
    if ((_isRoom || _isGroup)) {
      _subType = 7;
      let _sendPid = bizUtil.getSendPidByType(this.curInfo?.t);
      // 转小端
      let _byte0 = _sendPid & 0xff;
      let _byte1 = (_sendPid >> 8) & 0xff;
      _cmd = [4, _byte0, _byte1, Number(this.curInfo?.gid) || 0, Number(value) || 0];
    } else {
      // _subType = 6;
      _cmd = [2, Number(this.curInfo?.a) || 0, Number(value) || 0];
    }
    let _result = app.globalData.blsdk.controlWithDevice(this.curInfo, {
      tlv: 0, 
      // 6 - 基于短地址的控制 7 - 基于组地址的控制
      subType: (1 << 6) + _subType,
      cmds: _cmd
    }, {
      isNewUart: true, // 新协议
    });
    if (!_result || _result.then) {
      return;
    }
    // 设置缓存
    this.setStoreAfterControl("slow_rise", value, "light");
  },
  getActionListByKey (keyStr) {
    let _list = [];
    let _title = "";
    if (keyStr == "alexa" || keyStr == "noDisturb" || keyStr == "configLock" || keyStr == "buzzer" || keyStr == "mic") {
      // alexa设置
      // 配网锁定
      // 来电勿扰
      _list = [{
        text: "开启",
        value: keyStr == "configLock" ? 8 : 1
      }, {
        text: "关闭",
        value: 0
      }]
    } else if (keyStr == "slowUpDown") {
      let _texts = [0, 1, 2, 3, 6];
      let _bit7 = (1 << 7);
      let _values = [0, _bit7 + 1, _bit7 + 2, _bit7 + 3, _bit7 + 4];
      for (let index in _texts) {
        _list.push({
          text: _texts[index] || 0,
          value: _values[index] || 0
        });
      }
    } else if (keyStr == "icType") {
      _list = CONFIG.icType;
    } else if (keyStr == "lightSort") {
      _list = CONFIG.lightSort;
    } else if (keyStr == "ledCount") {
      for (let i = 10; i <= 2400; i++) {
        _list.push(i + "");
      }
    } else if (keyStr == "unit") {
      _list = [{
        text: "°F",
        value: 1
      }, {
        text: "℃",
        value: 0
      }];
    }
    return {
      title: _title,
      list: _list
    };
  },
  // actionsheet点击事件
  actionBtnClick (e) {
    let {key, list} = this.data.actionObj;
    let {value, index} = e.detail || {};
    console.log("actionBtnClick value, index = ", value, index, key, list)
    if (key == "mic") {
      this.micDetail(value);
    } else if (key == "alexa") {
      this.alexaSetDetail(value);
    } else if (key == "configLock") {
      this.configLock(value);
    } else if (key == "noDisturb") {
      this.noDisturb(value);
    } else if (key == "slowUpDown") {
      this.slowUpDown(value);
    } else if (key == "ledCount") {
      // 灯珠总数
    } else if (key == "icType") {
      // IC类型
      this.icType(value);
    } else if (key == "lightSort") {
      // 灯条排序
      this.lightSort(value);
    } else if (key == "unit") {
      let {value, text} = list[index] || {};
      // 保存信息
      this.saveDevStatus(text, key);
      // 刷新数据
      this.updateBottomItemsByKey(key, text);
    } else if (key == "buzzer") {
      let {value, text} = list[index];
      // 保存信息
      this.saveDevStatus(value, key);
      // 刷新数据
      this.updateBottomItemsByKey(key, text);
    }

    // 关闭actionsheet
    this.data.actionObj.show = false;
    this.setData({
      actionObj: this.data.actionObj
    })
  },
  updateBottomItemsByKey (key, value) {
    if (!key) {
      return;
    }
    let _bottomList = this.data.bottomList || [];
    let _index = _bottomList.findIndex(item => item.key == key);
    if (_index < 0) {
      return;
    }
    let _item = _bottomList[_index];
    _item.value = value;
    this.setData({
      [`bottomList[${_index}]`]: _item
    })
  },
  saveDevStatus (str, keyStr) {
    if (!keyStr) {
      return;
    }
    // 保存的设备状态
    let {d, a, t} = app.globalData.devInfo || {};
    let _devTypeStr = bizUtil.getDevTypeStr(t);
    console.log("saveTemphumidityStatus _devTypeStr, str = ", _devTypeStr, str);
    
    let _keyStr = d + "_" + a;
    // 秒秒测单位
    app.globalData.allStatus[_keyStr] = app.globalData.allStatus[_keyStr] || {};
    
    if (_devTypeStr == "temphumidity") {
      let {ht, lt, unit} = app.globalData.allStatus[_keyStr];
      let _unitStr = bizUtil.getUnitByValue(unit);
      console.log("saveTemphumidityStatus _unitStr = ", _unitStr);
      if (_unitStr != str) {
        // 单位的值
        if (ht >= 0) {
          let {temp} = historyCommon.getTempByHistoryJson ({
            temperature: ht * 10, 
            tempunit: unit
          }, str);
          console.log("saveTemphumidityStatus ht temp = ", temp);
          app.globalData.allStatus[_keyStr].ht = temp;
        }
        if (lt >= 0) {
          let {temp} = historyCommon.getTempByHistoryJson ({
            temperature: lt * 10, 
            tempunit: unit
          }, str);
          console.log("saveTemphumidityStatus lt temp = ", temp);
          app.globalData.allStatus[_keyStr].lt = temp;
        }
      }
      // 现在的单位
      app.globalData.allStatus[_keyStr].unit = str || ""; 
    } else if (_devTypeStr == "fan") {
      app.globalData.allStatus[_keyStr][keyStr] = str || ""; 
    }
    // 保存到缓存中
    app.setBizStorage("devStatus", app.globalData.allStatus);
  },
  icType (value) {
    // 更新页面
    this.updateExtendByKeyAndValue("icType", this.getInArray(CONFIG.icType, "value", value)?.text || "");
    // 更新数据缓存
    this.updateStore("ic_type", value);
    // 发送命令
    this.actDev();

    // app.globalData.devInfo.e = app.globalData.devInfo.e || {};
    // app.globalData.devInfo.e.lightDazzle = app.globalData.devInfo.e.lightDazzle || {};
    // app.globalData.devInfo.e.lightDazzle.ic_type = value;
    // this.setData({
    //   [`extendList[1].value`]: this.getInArray(CONFIG.icType, "value", value)?.text || ""
    // });
    // // 发送命令
    // this.actDev();
  },
  lightSort (value) {

    // 更新页面
    this.updateExtendByKeyAndValue("lightSort", this.getInArray(CONFIG.lightSort, "value", value)?.text || "");
    // 更新数据缓存
    this.updateStore("beads_sort", value);
    // 发送命令
    this.actDev();

    // app.globalData.devInfo.e = app.globalData.devInfo.e || {};
    // app.globalData.devInfo.e.lightDazzle = app.globalData.devInfo.e.lightDazzle || {};
    // app.globalData.devInfo.e.lightDazzle.beads_sort = value;
    // this.setData({
    //   [`extendList[2].value`]: this.getInArray(CONFIG.lightSort, "value", value)?.text || ""
    // })
  },

  // 清除遥控器数据
  unbindRemoteCtrl () {
    if (app.globalData.roleNum == 2) {
      return;
    }
    // 提示是否需要清除
    wx.showModal({
      content: "你是否要清除这个设备的遥控器数据？"
    }).then((result) => {
      if (!result?.confirm) {
        return;
      }
      // 解绑
      this.unbindDetail();
    }).catch(e => {
      console.log("unbindRemoteCtrl error");
    });
  },
  unbindDetail () {
    let _addr = this.curInfo?.a;
    if (!_addr) {
      console.log("unbindRemoteCtrl _addr = ", _addr);
      return;
    }
    app.globalData.blsdk.unbindRemoteCtrl(_addr);
  },

  bindPickerChange (e) {
    let _value = e.detail.value;
    console.log('bindPickerChange _value = ', _value, this.data.pickObj.key);
    if (this.data.pickObj.key == "ledCount") {
      let _realInt = Number(_value) + Number(countList[0]);
      console.log('bindPickerChange _realInt = ', _realInt);

      // 更新数据缓存
      this.updateStore("beads_number", _realInt);

      this.setData({
        [`extendList[0].value`]: _realInt
      })
    }
    this.hidePicker();
  },

  hidePicker () {
    this.data.pickObj.show = false;
    this.setData({
      pickObj: this.data.pickObj
    })
  },
  ledCountChange (e) {
    console.log("ledCountChange e = ", e);
    let {key, value} = e.detail;
    if (key == "confirm") {
      // 更新页面
      this.updateExtendByKeyAndValue("ledCount", value);
      // 更新数据缓存
      this.updateStore("beads_number", value);
      // 发送命令
      this.actDev();
    }
    this.showInputDialog(false);
  },
  // 更新数据
  updateExtendByKeyAndValue (key, value) {
    let _list = this.data.extendList || [];
    if (_list.length == 0) {
      return;
    }
    let _index = _list.findIndex(item => item?.key == key);
    if (_index < 0) {
      return;
    }
    this.setData({
      [`extendList[${_index}].value`]: value
    })
  },
  // 更新缓存
  updateStore (key, value, parentKey) {
    if (!key) {
      return;
    }
    let _parentKey = parentKey || "lightDazzle";
    app.globalData.devInfo.e = app.globalData.devInfo.e || {};
    app.globalData.devInfo.e[_parentKey] = app.globalData.devInfo.e[_parentKey] || {};
    app.globalData.devInfo.e[_parentKey][key] = value;

    // 保存数据
    app.saveAllDevsMap();
  },

  // 炫彩灯
  getXuancaiParamsByDevInfo (devInfo) {
    if (!devInfo) {
      return [];
    }
    let {beads_number, beads_sort, ic_type, beads_type, beads_speed} = devInfo?.e?.lightDazzle || {};
    // Data字段描述
    // Data[0] = 0xE1
    // Data[1] = IC类型
    // Data[2] = RGB线序
    // Data[3] = 预留
    // Data[4] = 预留
    // Data[5] = 预留
    // Data[6] = effect_type
    // Data[7] = speed
    // Data[8] = 灯珠数量高字节
    // Data[9] = 灯珠数量低字节
    let _beadsNumber = Number(beads_number) || 0;
    let _highCount = (_beadsNumber >> 8) & 0xff;
    let _lowCount = (_beadsNumber) & 0xff;

    // 这里effect_type有个bug,不能按照current下发 是下标
    let _effectType = Number(beads_type) || 0;
    // 变化速度
    let _speed = beads_speed == undefined ? 100 : (Number(beads_speed) || 0);
    // 拼接命令
    let _cmdArray = [0xE1, Number(ic_type) || 0, Number(beads_sort) || 0, 0, 0, 0, _effectType, _speed, _highCount, _lowCount];
    return _cmdArray;
  },
  actDev (cmd, callback) {
    let _devInfo = app.globalData.devInfo;
    let _cmd = cmd || this.getXuancaiParamsByDevInfo(_devInfo);
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, _cmd);
    if (!callback) {
      return;
    }
    if (!_result?.then) {
      !!callback && (callback(_result));
      return;
    }
    _result.then((res) => {
      console.log("actDev res = ", res);
      !!callback && (callback(0));
    })
  }
})