// device/light/funcSet/mic/index.js
const ability = require("../../../../utils/classDef/ability")
const bizUtil = require("../../../../utils/bizUtil")
const app = getApp()

Page({

  /**
   * 页面的初始数据
   */
  data: {
    bleUnavailable: false,
    topList: [],
    bottomList: [],
    extendList: [], // 灯珠总数等幻彩灯设置

    dataObj: {
      // "mic_enable": 1, // 默认收音是否开启
      // "mic_type": 0, // 默认收音模式
      // "mic_speed": 50, // 默认收音灵敏度
    },

    // 滑块配置
    sliders: {
      leftImg: "/device/img/light/mode/del.png",
      rightImg: "/device/img/light/mode/add.png",
      leftClass: "sDown padding30",
      rightClass: "sUp padding30",
      bgImg: "",
      bgClass: "tempImg",
      width: 590, // rpx
      percent: 50, // 百分比
      minPercent: 0,
      bgHeight: "height50",
      // gradient: true, // 默认不支持滑动时调节
      upEventWhileMove: true,
      minVal: 1, // 单位摄氏度
      maxVal: 100, // 单位摄氏度
    },
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title, devType} = options || {};
    // 设备品类
    this.devTypeStr = this.getDevTypeStr();
    // 设备信息
    if (app.globalData?.tmpData?.roomInfo) {
      this.isRoom = true;
      // 房间控制
      this.curInfo = app.globalData?.tmpData?.roomInfo;
    } else if (app.globalData.groupInfo) {
      this.isGroup = true;
      // 固定群组
      this.curInfo = app.globalData.groupInfo
    } else {
      this.curInfo = app.globalData.devInfo;
    }
    // 设置标题
    this.setNavTitle(decodeURIComponent(title || ""));
    // 初始化功能数据
    let {topList, bottomList} = this.getTopAndBottomList(this.devTypeStr);
    // 根据缓存数据初始化数据
    let _dataObj = this.initDataByDevInfo(this.curInfo);

    // 设置百分比
    this.data.sliders.percent = bizUtil.getPercentByVal(_dataObj.mic_speed, this.data.sliders);

    this.setData({
      topList: topList,
      bottomList: bottomList,
      dataObj: _dataObj,
      sliders: this.data.sliders
    })
  },
  // 根据缓存数据初始化数据
  initDataByDevInfo (devInfo) {
    let _result = {};
    if (!devInfo) {
      return _result;
    }
    let {mic_enable, mic_speed, mic_type} = devInfo?.e?.lightDazzle || {};
    _result.mic_enable = mic_enable || 0;
    _result.mic_speed = mic_speed || 50; // 1 - 100 默认50
    _result.mic_type = mic_type || 0;
    return _result;
  },
  setNavTitle (str) {
    ability.setNavigationBarTitle(str || "")
  },
  getDevTypeStr () {
    let {t, v} = app.globalData.devInfo || {};
    return bizUtil.getDevTypeStr(t);
  },
  getTopAndBottomList (devTypeStr) {
    // let {t, v} = this.curInfo || {};
    // // 能力集合
    // let _abilityJson = bizUtil.getJsonByAblitys(v, devTypeStr);
    // console.log("getTopAndBottomList _abilityJson, devTypeStr = ", _abilityJson, devTypeStr);
    // 顶部条目
    let _topList = this.getAllTopList();
    // 底部按钮列表
    let _bottomList = this.getBottomListByType();
    
    return {
      topList: _topList,
      bottomList: _bottomList
    }
  },
  supportBackLight (devTypeStr, abilityJson) {
    if ("panel" == devTypeStr) {
      return abilityJson?.backlight || false;
    }
    return false;
  },
  getAllTopList () {
    // 配网锁定、来电勿扰 凌动开关支持群组和单个设备
    return [{
      title: "MIC功能",
      value: "",
      // bindFunc: "topClick",
      key: "mic"
    }]
  },
  getBottomListByType () {
    return this.getAllBottomList();
  },



  // 取得当前页面控制设备列表数组
  getAllCtrlDevsArray () {
    if (this.isGroup) {
      // 固定群组
      // 是否群组设备
      let {dids} = app.globalData.groupInfo || {};
      // 取得房间下所有的d对应的详细信息
      let _didMaps = bizUtil.getDidMapInJson(app.globalData.devs);
      // 群组设备列表
      let _groupDevs = dids || [];
      // 群组设备列表详细
      return _groupDevs.map(did => _didMaps[did]);
    } else if (app.globalData.tgroupId && (app.globalData.tmpData?.tdevs)) {
      // 临时群组
      return app.globalData.tmpData?.tdevs || [];
    } else {
      let _devInfo = app.globalData.devInfo;
      if (_devInfo) {
        // 单个设备控制
        return [_devInfo];
      } else if (this.isBatchCtrl()) {
        // 5个一组的批量控制
        return app.globalData?.tmpData?.tdevs || [];
      } else if (this.isRoom) {
        // 房间控制
        return this.getDevArrayByRoomIdInJson(app.globalData.devs, (app.globalData.roomInfo || {}).id)
      }
    }
  },
  // 获取是否批量控制
  isBatchCtrl () {
    return (this.data.groupIds.length > 0) && (app.globalData.devInfo == undefined || Object.keys(app.globalData.devInfo).length == 0);
  },
  // 根据房间id取得json中设备集合
  getDevArrayByRoomIdInJson (json, roomId) {
    let _devs = bizUtil.getDevsOrGroupsByRoomId(roomId, json);
    let {t} = this.curInfo || {};
    // 取得照明的设备
    let _json = bizUtil.getAllPidByArray(_devs);
    console.log("getDevArrayByRoomIdInJson _json, t = ", _json, t);
    return _json[t] || [];
  },

  // 根据品类取得功能集合
  getFuncList (devs) {
    // 被控设备集合
    let _devs = devs || [];
    
    // 最终集合
    let versions = [], minStatus= {}, maxStatus = {};
    // let _minCount = 20, _maxCount = -1;
    // 遍历数据取得集合
    for (let index in _devs) {
      let {t, v} = _devs[index] || {};
      // 如果版本号v不存在，则continue
      if (v == undefined) {
        continue;
      }

      // 如果版本号一致，则continue
      if (versions.indexOf(v) >= 0) {
        continue;
      }
      // 添加版本分支集合
      versions.push(v);
      // 取得能力集
      let _devStatus = bizUtil.getFuncByPid({}, t);
      // 添加最大能力集
      Object.assign(maxStatus, _devStatus);
    }
    console.log("getFuncList maxStatus = ", maxStatus);
    return {
      versions: versions, // 版本能力集合
      maxStatus: maxStatus
    };
  },
  
  getAllBottomList () {
    // 配网锁定、来电勿扰 凌动开关支持群组和单个设备
    return [{
      title: "七彩渐变律动",
      key: "jianbian",
      value: 0,
      bindFunc: "bottomClick"
    }, {
      title: "七彩呼吸律动",
      key: "huxi",
      value: 1,
      bindFunc: "bottomClick"
    }, {
      title: "七彩跳变律动",
      value: 2,
      key: "tiaobian",
      bindFunc: "bottomClick"
    }, {
      title: "七彩频闪律动",
      value: 3,
      key: "pinshan",
      bindFunc: "bottomClick"
    }];
  },
  getInArray (array, key, value) {
    if (!array || array.length == 0) {
      return {};
    }
    let _array = array.filter(item => item[key] == value);
    return _array[0] || {};
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 保存设备信息
    this.updateStore();
  },
  
  // 更新缓存
  updateStore () {
    app.globalData.devInfo.e = app.globalData.devInfo.e || {};
    // app.globalData.devInfo.e.lightDazzle = app.globalData.devInfo.e.lightDazzle || {};
    app.globalData.devInfo.e.lightDazzle = Object.assign(app.globalData.devInfo.e.lightDazzle || {}, this.data.dataObj);
    // 保存数据
    app.saveAllDevsMap();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  // topClick (e) {
  //   this.detailItemClick(e, "topList")
  // },
  bottomClick (e) {
    this.detailItemClick(e, "bottomList")
  },
  detailItemClick (e, keyStr) {
    let {index} = e.currentTarget.dataset;
    let _list = this.data[keyStr];
    if (!_list || _list.length == 0) {
      return;
    }
    let {key, value, title} = _list[index];
    console.log("detailItemClick key = ", key, value);
    // 更新标题
    this.updateDataObjKey("mic_type", value);

    // 发送命令
    this.actDev();
  },
  actDev (callback) {
    // 收音功能
    let _devInfo = app.globalData.devInfo;
    let _cmd = this.getMicParamsByValue();
    console.log("actDev _cmd = ", _cmd);
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, _cmd);
    if (!callback) {
      return;
    }
    if (!_result?.then) {
      !!callback && (callback(_result));
      return;
    }
    _result.then((res) => {
      console.log("actDev res = ", res);
      !!callback && (callback(0));
    })
  },
  // 取得缓存的设备状态
  getDevStatus () {
    let _dev = this.curInfo || {};
    // 取得所有设备保存的状态数据
    let _allDevStatus = app.globalData.allStatus || {};
    // 设备状态
    let _devStatus = _allDevStatus[_dev.d + "_" + _dev.a] || {};
    // 取得当前设备的状态
    return _devStatus || {};
  },
  // 获取开关缓存状态
  getPwrRGBCW () {
    // let {pwr, brightness, r, g, b, c, w} = this.getDevStatus();
    // 识音必须开设备 (1 << 7) + (Number(brightness) || 1), 
    return [0, 0, 0, 0, 0];
    // return [b || 0, r || 0, g || 0, c || 0, w || 0];
  },
  getMicParamsByValue () {
    let {mic_enable, mic_type, mic_speed} = this.data.dataObj || {};
    // effect_type取值定义:
    // 0：关闭MIC功能
    // 1：打开MIC功能
    let _enable = Number(mic_enable) || 0;
    // 中间空出一个brgcw五个字节
    // 律动类型取值定义：
    // 0x00：七彩渐变律动
    // 0x01：七彩呼吸律动
    // 0x02：七彩跳变律动
    // 0x03：七彩频闪律动 
    let _type = Number(mic_type) || 0;
    // MIC灵敏度定义范围：
    // 1-100
    let _micValue = Number(mic_speed) || 0;
    
    // rgbcw数据
    let _rgbcw = this.getPwrRGBCW();
    console.log("getMicParamsByValue _rgbcw = ", _rgbcw);

    return [0xE2].concat(_rgbcw).concat([_enable, _type, _micValue]) // [0xE2, 0, 0, 0, 0, 0, _enable, _type, _micValue];
  },
  sliderChange (e) {
    console.log("sliderChange e = ", e);
    let {isMoving, originValue, percent, value} = e.detail;
    // 更新标题
    this.updateDataObjKey("mic_speed", value);
    // 移动完毕才发送命令
    if (isMoving) {
      return;
    }
    // 发送命令
    this.actDev();
  },

  switchChange (e) {
    let {value} = e?.detail || {};
    this.updateDataObjKey("mic_enable", value);

    // 发送命令
    this.actDev();
  },
  updateDataObjKey (key, value) {
    if (!key || value == undefined) {
      return;
    }
    this.data.dataObj[key] = value;
    this.setData({
      dataObj: this.data.dataObj
    })
  }
})