
import bizUtil from "../../../utils/bizUtil"
import ability from "../../../utils/classDef/ability"
const app = getApp();
Page({
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: ""
      }
    },
    // 亮度配置
    brightSliders: {
      leftImg: "/device/img/light/mode/del.png",
      rightImg: "/device/img/light/mode/add.png",
      leftClass: "sDown",
      rightClass: "sUp",
      width: 524, // rpx
      percent: 100, // 百分比
      labelTxt: "速度",
      labelClass: 'colorF78D3F',
      minVal: 0,
      maxVal: 100,
      minPercent: 1
    },
    bleUnavailable: false,

    isCW: false, // 是否cw模式
    brightness: 127,
    huidu: 0 // 是否1024灰度
  },
  onLoad (options) {
    console.log("onLoad options = ", options);
    let _options = options || {};
    // key
    this.data.keyStr = _options.keystr;
    this.data.mode = JSON.parse(decodeURIComponent(_options.mode || ''))
    // 1024灰度
    this.data.huidu = Number(_options.huidu) || 0;
    // 标题和百分比
    let _defaultVal = Number(_options.val) || 0;
    let _percent = this.getPercentByVal(_defaultVal);
    this.data.brightSliders.percent = _percent;

    // 是否cw调节
    this.data.isCW = Boolean(_options.isCW) || false;
    // 亮度值
    this.data.brightness = ((options.brightness != undefined) ? Number(options.brightness) : 127);

    // 修改名称
    let _title = decodeURIComponent(_options.name) || "模式";
    this.data.navObj.center.text = _title;
    ability.setNavigationBarTitle(_title);

    console.log("onLoad _percent = ", _percent);
    this.setData({
      navObj: this.data.navObj,
      brightSliders: this.data.brightSliders
    })
  },
  onUnload () {
  },
  // 保存数据
  saveModeStorage () {
    app.setBizStorage("modes", app.globalData.bleModes || {});
  },
  // 根据百分比取得树
  getValByPercent (percent) {
    let {maxVal, minVal} = this.data.brightSliders || {};
    let _percent = percent || 0;
    if (percent > 1) {
      _percent = percent / 100;
    }
    return _percent * (maxVal - minVal) + minVal;
  },
  // 取得百分比
  getPercentByVal (value) {
    let {maxVal, minVal} = this.data.brightSliders || {};
    if (maxVal <= minVal || value > maxVal) {
      return 100;
    } else if (value < minVal) {
      return 0;
    }
    let _percent = (value - minVal) / ((maxVal - minVal) || 1);
    return _percent * 100;
  },
  // 滑块百分比 0 - 100
  sliderChange (event) {
    // 是否移动中
    let _isMoving = event.detail.isMoving;
    if (_isMoving) {
      return;
    }
    // 百分比
    let _value = event.detail.value;
    if (!app.globalData.bleModes) {
      app.globalData.bleModes = {};
    }
    let _realVal = _value; // this.getValByPercent(_percent);
    console.log("sliderChange _realVal = ", _realVal);
    // 发送值
    this.sendControl(_realVal);
    // 保存值
    app.globalData.bleModes[this.data.keyStr] = _realVal;
    this.saveModeStorage();
  },
  // 发送控制命令
  sendControl (realVal) {
    // 设备信息
    let _devInfo = app.globalData.devInfo;
    let _roomInfo = app.globalData.roomInfo;
    // 设置默认的值为100
    let _default = this.data.brightSliders.maxVal || 100;

    // 发送的数据
    let item = {
      mode: this.data.mode,
      defaultTime: _default,
      mult: (realVal || 0) / _default,
      huidu1024: Number(this.data.huidu) || 0, // 设置是否支持1024灰度
      key: this.data.keyStr
    };

    // // 增加逻辑，如果是临时组控制
    // if (app.globalData.tgroupId) {
    //   app.globalData.blsdk.sceneControlWithDevice(_devInfo, app.globalData.tgroupId, item);
    //   return;
    // }
    
    // 增加逻辑：如果是房间的色彩控制，需要先发送一个全关的命令
    let _isRoomControl = !_devInfo;
    if (_isRoomControl) {
      let _roomId = _roomInfo.id || 0;

      // 如果有临时群组
      if (app.globalData.tgroupId) {
        _roomId = app.globalData.tgroupId;
      }
      // 更新发送的命令
      let _jsonData = bizUtil.getModeParamsBeforeSet(item, this.data.brightness);
      
      // 先发送全关的命令
      let _result = app.globalData.blsdk.groupControlWithType(undefined, _roomId, _jsonData, {
        noRemote: true, // 不发远程控制命令
        timestamp: 500 // 500ms
      });
      // 如果发送结果是promise的，则继续调用
      if (_result && _result.then) {
        _result.then((res) => {
          app.globalData.blsdk.sceneControlWithDevice(_devInfo, !!_devInfo ? undefined : _roomId, item);
        })
      } else {
        app.globalData.blsdk.sceneControlWithDevice(_devInfo, !!_devInfo ? undefined : _roomId, item);
      }
    } else {
      app.globalData.blsdk.sceneControlWithDevice(_devInfo, !!_devInfo ? undefined : _roomInfo.id, item);
    }
  }
})
