// device/light/xuancai/index.js
const ability = require("../../../utils/classDef/ability");
const CONFIG = require("../funcSet/jsConfig/config")
const app = getApp()
Page({

  /**
   * 页面的初始数据
   */
  data: {
    // 速度配置
    speedSliders: {
      leftImg: "/device/img/light/mode/del.png",
      rightImg: "/device/img/light/mode/add.png",
      leftClass: "sDown padding30",
      rightClass: "sUp padding30",
      width: 600, // rpx
      percent: 100, // 百分比
      labelTxt: "速度",
      labelClass: 'color666',
      bgHeight: "height20",
      marginSlider: "marginB40",
      minVal: 0,
      maxVal: 100,
      minPercent: 1
    },
    bleUnavailable: false,

    // 炫彩模式
    modeJson: {
      selRoomId: 0,
      data: []
    },
    modeArray: [],
    isRGB: true, // 默认是rgb
    current: [0]
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {isRbg, title} = options;
    console.log("onLoad options = ", options);

    // 设置标题
    this.setNavTitle(decodeURIComponent(title || ""));
    
    // 取得配置内容 固定
    this.data.modeJson.data = this.getScrollList();
    // 根据上次缓存数据初始化值
    this.initDataByVGroup(app.globalData.devInfo);

    this.setData({
      modeJson: this.data.modeJson,
      modeArray: this.data.modeArray,
      current: this.data.current,
      speedSliders: this.data.speedSliders
      // isRGB: this.data.isRGB
    })
  },
  initDataByVGroup (devInfo) {
    if (!devInfo) {
      return;
    }
    // "beads_number" // 灯珠总数
    // "beads_sort" // 灯珠排序
    // "ic_type" // IC 类型
    // "beads_type" // 炫彩模式类型
    // "beads_speed" // 炫彩模式变化速度
    let {beads_number, beads_sort, ic_type, beads_type, beads_speed} = devInfo?.e?.lightDazzle || {};

    // 百分比
    this.data.speedSliders.percent = beads_speed == undefined ? 100 : (Number(beads_speed) || 0);

    // 默认选择的顶部tab下标
    let _tabIndex = 0;
    let _pickIndex = 0;
    // 根据上次设置的值初始化本次的数据
    if (beads_type >= 0) {
      let {index, idx} = this.getIndexByValueInList(CONFIG.xuancai, beads_type);
      _tabIndex = index;
      _pickIndex = idx;
    }

    // 默认选择第一项tab
    this.data.modeJson.selRoomId = _tabIndex;
    // picker默认选择第0个
    this.data.current = [_pickIndex];
    // picker list
    this.data.modeArray = this.getModeArrayByIndex(_tabIndex);
  },
  getIndexByValueInList (list, value) {
    // xuancai: [{
    //   title: "基本",
    //   key: "basic",
    //   array: [{
    //     text: "自动循环",
    //     value: 1
    //   }, {
    // 默认值
    let _result = {
      index: 0,
      idx: 0
    };
    if (!list) {
      return _result;
    }
    for (let index in list) {

      let _array = list[index]?.array || [];
      if (_array.length == 0) {
        continue;
      }
      for (let idx in _array) {
        if (_array[idx]?.value == value) {
          _result.index = index;
          _result.idx = idx;
          return _result;
        }
      }
    }
    return _result;
  },
  setNavTitle (str) {
    ability.setNavigationBarTitle(str || "");
  },
  getScrollList () {
    let _list = CONFIG.xuancai.map((item, index) => {
      return {
        id: index,
        n: item.title
      }
    })
    return _list;
  },
  getModeArrayByIndex (index) {
    let {array} = CONFIG.xuancai[index] || {};
    let _length = array?.length;
    if (!_length) {
      return [];
    }
    let _result = [];
    for (let item of array) {
      _result.push((item.value || "") + "  " + (item.text || ""));
    }
    return _result;
  },
  scrollChange (e) {
    console.log("scrollChange e = ", e);
    let {index, id} = e.detail || {};
    this.data.modeJson.selRoomId = index;
    // 根据选择的index取得数据
    this.data.modeArray = this.getModeArrayByIndex(index);
    this.data.current = [0];
    this.setData({
      modeJson: this.data.modeJson,
      modeArray: this.data.modeArray,
      current: this.data.current
    });
    this.actDev();
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 保存
    app.saveAllDevsMap();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  bindChange (e) {
    console.log("bindChange e = ", e);
    let {value} = e?.detail || {};
    if (!(value instanceof Array)) {
      return;
    }
    this.data.current = value;

    this.actDev()
  },
  sliderChange (e) {
    console.log("sliderChange e = ", e);
    let {isMoving, originValue, percent, value} = e?.detail || {};
    this.data.speedSliders.percent = percent;
    this.actDev()
  },
  // 更新缓存
  updateStore (keys, values) {
    if (!keys || keys.length == 0) {
      return;
    }
    app.globalData.devInfo.e = app.globalData.devInfo.e || {};
    app.globalData.devInfo.e.lightDazzle = app.globalData.devInfo.e.lightDazzle || {};
    if (keys instanceof Array) {
      // 数组则更新多个
      for (let index in keys) {
        let _tmpKey = keys[index] || "";
        let _tmpVal = values[index] || "";
        app.globalData.devInfo.e.lightDazzle[_tmpKey] = _tmpVal;
      }
    } else {
      app.globalData.devInfo.e.lightDazzle[keys] = values;
    }
  },
  getXuancaiParamsByDevInfo (devInfo) {
    if (!devInfo) {
      return [];
    }
    let {beads_number, beads_sort, ic_type} = devInfo?.e?.lightDazzle || {};
    // Data字段描述
    // Data[0] = 0xE1
    // Data[1] = IC类型
    // Data[2] = RGB线序
    // Data[3] = 预留
    // Data[4] = 预留
    // Data[5] = 预留
    // Data[6] = effect_type
    // Data[7] = speed
    // Data[8] = 灯珠数量高字节
    // Data[9] = 灯珠数量低字节
    let _beadsNumber = Number(beads_number) || 0;
    let _highCount = (_beadsNumber >> 8) & 0xff;
    let _lowCount = (_beadsNumber) & 0xff;

    // 这里effect_type有个bug,不能按照current下发 是下标
    let _effectType = Number(this.data.current[0]) || 0;
    _effectType = Number((this.data.modeArray[_effectType]?.split(" ") || [])[0]) || 0;
    // 变化速度
    let _speed = Number(this.data.speedSliders.percent) || 0;

    // 增加保存的数据
    // "beads_number": 10, // 灯珠总数
    // "beads_sort": 0, // 灯珠排序
    // "ic_type": 0, // IC 类型
    // "beads_type": 1, // 炫彩模式类型
    // "beads_speed": 50 // 炫彩模式变化速度
    // "beads_number", "beads_sort", "ic_type", 
    this.updateStore(["beads_type", "beads_speed"], [_effectType, _speed]);

    let _cmdArray = [0xE1, Number(ic_type) || 0, Number(beads_sort) || 0, 0, 0, 0, _effectType, _speed, _highCount, _lowCount];
    return _cmdArray;
  },
  actDev (cmd, callback) {
    let _devInfo = app.globalData.devInfo;
    let _cmd = cmd || this.getXuancaiParamsByDevInfo(_devInfo);
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, _cmd);
    if (!callback) {
      return;
    }
    if (!_result?.then) {
      !!callback && (callback(_result));
      return;
    }
    _result.then((res) => {
      console.log("actDev res = ", res);
      !!callback && (callback(0));
    })
  }
})