// device/panel/backlight/index.js
const ability = require("../../../utils/classDef/ability")
const bizUtil = require("../../../utils/bizUtil");
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "设备状态"
      }
    },
    pwrList: [],
    statusList: [],
    // 亮度配置
    brightSliders: {
      leftImg: "../../static/img/common/bright_down.png",
      rightImg: "../../static/img/common/bright_up.png",
      leftClass: "bDown",
      rightClass: "bUp",
      width: 516, // rpx
      percent: 100, // 百分比
      labelTxt: "亮度",
      minVal: 2,
      maxVal: 127,
      minPercent: 1, // 最小百分比
      gradient: false // 默认不支持滑动时调节
    },
    status: {
      pwr: 0,
      brightness: 100
    },
    // 按钮 根据编辑状态动态显示
    actions: [],
    devType: 'light', // 默认灯泡设备

    defalutValue: -1, // 选中的数据
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 插卡取电用
    let {delayOff} = options;

    // 取得设备状态
    let _devStatus = this.getDevStatus();
    // 初始化设备品类
    let _devTypeStr = this.getDevType();

    // 显示的列表
    let _pwrList = [];
    // tip标题
    let _tip = '';
    let _actions = [];
    if (_devTypeStr == "powerPlug") {
      _tip = '选择“拔卡”后电源状态（单选）';

      // 默认值
      if (_devStatus?.delayOff >= 0) {
        delayOff = _devStatus?.delayOff || 0
      }

      _pwrList = this.getPowerPlugList();
      _actions = this.getAllActions();
    } else {
      _tip = '选择“背光灯”设备状态';
      // 初始化开关列表
      _pwrList = this.getPwrList();
      // 初始化数据
      let _jsonStatus = this.getDataByTypeAndStatus(_devTypeStr, _devStatus);
      Object.assign(this.data.brightSliders, _jsonStatus);
    }

    this.setData({
      tip: _tip,
      pwrList: _pwrList,
      status: _devStatus,
      brightSliders: this.data.brightSliders,
      devTypeStr: _devTypeStr,
      actions: _actions,
      defalutValue: Number(delayOff) || 0
    });
  },
  // 初始化设备品类
  getDevType () {
    // 取得设备品类
    return bizUtil.getDevTypeStr(app.globalData.devInfo?.t);
  },
  getPowerPlugList () {
    // 取得设备品类
    return [{
      name: '不断电',
      value: 0
    }, {
      name: '延时断电',
      value: 1
    }];
  },
  getPwrList () {
    // 取得设备品类
    return [{
      name: '开',
      value: 1
    }, {
      name: '关',
      value: 0
    }];
  },
  getAllActions () {
    return [{
      n: '保存',
      classStr: 'saveBtn',
      handle: 'saveInfo'
    }]
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
    // this.initCurrentType();
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
    // 保存状态
    this.saveDevStatus();
  },
  getStorageKey () {
    let _dev = app.globalData.devInfo || {};
    return _dev.d + "_" + _dev.a;
  },
  saveDevStatus () {
    let _key = this.getStorageKey();
    console.log("saveDevStatus _key = ", _key);
    // 设备状态
    app.globalData.allStatus[_key] = this.data.status || {};
    // 保存到缓存中
    app.setBizStorage("devStatus", app.globalData.allStatus);
  },
  getDevStatus () {
    let _key = this.getStorageKey();
    let _json = app.globalData.allStatus[_key];
    if (!_json) {
      return {};
    }
    console.log("getDevStatus _json = ", _json);
    return Object.assign({}, _json);
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  getDataByTypeAndStatus (devTypeStr, status) {
    let _status = status || {};
    let _result = {};

    let {v, t} = app.globalData.devInfo || {};
    // 最大最小值
    let json = bizUtil.getJsonByAblitys(v, devTypeStr, {t});
    let {percent, value} = json?.minBrightness || {};
    _result.minVal = value || 1;
    _result.minPercent = percent || 1;
    // 默认亮度
    let _percent = _status.brightnessP != undefined ? _status.brightnessP : bizUtil.getPercentByVal(_status.brightness, this.data.brightSliders);
    // 默认灯泡
    _result.percent = _percent;
    console.log("_result = ", _result);
    
    return _result;
  },

  saveInfo () {
    let _devInfo = app.globalData.devInfo;
    let _valueInt = Number(this.data.defalutValue) || 0;
    if (_valueInt < 0) {
      return ability.showToast({
        title: "请选择数据"
      })
    }

    // 保存设备信息
    if (this.data.devTypeStr == "powerPlug") {
      // 用于保存拔卡状态
      this.data.status.delayOff = _valueInt;
    }

    // 发送数据
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, {
      delayOff: _valueInt
    });
    if (!_result?.then) {
      return;
    }
    _result.then(res => {
      this.back();
    })
  },

  back () {
    // 返回上一页
    ability.navigateBack();
  },
  
  // 滑块百分比
  sliderChange (event) {
    let _percent = event.detail.percent;
    let _value = event.detail.value;

    console.log("sliderChange _value, _percent = ", _value, _percent);
    // let _originValue = event.detail.originValue;
    // 增加处理 保存原始的值（精确到小数点 为了防止下次进入亮度不一致）
    this.data.status.brightness = _value;

    // 判断百分比是不是比最大的小
    let _minPercent = this.data.brightSliders?.minPercent || 0;
    if (_percent < _minPercent) {
      _percent = _minPercent;
    }
    this.data.status.brightnessP = _percent;
    // 执行
    let _status = {
      brightness: _value
    };
    this.actDev(_status);
  },

  pwrClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.pwrList[index];
    let _valueInt = _item.value || 0;

    // 如果是插卡取电
    if (this.data.devTypeStr == "powerPlug") {
      this.setData({
        defalutValue: _valueInt
      })
      return 
    }

    // 发送数据
    this.pwrClickDetail(_valueInt);
  },
  pwrClickDetail (value) {
    let _value = value;

    this.data.status.pwr = _value;
    let _updateObj = {
      status: this.data.status
    };
    if (_value > 0 && this.data.status.brightnessP) {
      // 更新亮度
      this.data.brightSliders.percent = this.data.status.brightnessP;
      _updateObj.brightSliders = this.data.brightSliders;
    }
    console.log("this.data.status.pwr = ", _value);
    this.setData(_updateObj)
    let brightness = _value ? (1 << 7) : 0;
    // 设备控制
    let _params = {
      brightness: brightness // 开灯
    };
    this.actDev(_params);
  },

  actDev (key) {
    console.log("actDev key = ", key);
    let _devInfo = app.globalData.devInfo;
    let _addr = _devInfo.a % 256;
    // 新协议
    app.globalData.blsdk.controlWithDevice(_devInfo, {
      tlv: 0,
      subType: 32 + (1 << 6),
      cmds: (key?.brightness) ? [_addr, key.brightness] : [_addr, 0]
    }, {
      isNewUart: true,
      noRemote: true
    });
  },
  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    });
  },
  hideLoading () {
    ability.hideLoading();
  },
})