// device/panel/control/index.js
import ability from "../../../utils/classDef/ability";
import bizUtil from '../../../utils/bizUtil';
const app = getApp();

Page({
  data: {
    // sizes
    sizes: {
      widthRpx: 462, // rpx,
      heightRpx: 492, // rpx
      topMarginT: 76, // 4键
      btmMarginT: 239
    },
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: app.globalData.devInfo?.n || ''
      }
    },
    controlList: [], // 底部按钮
    bleUnavailable: false,
    keyHalfArray: [], // 按键半个数

    pwrAvalible: false, // 默认不是继电器设备
    // devStatus: {}, // 继电器设备状态 后续可能还有其他状态

    // backlight: false, // 默认不支持背光调节

    isPowerPlug: false, // 默认不是插卡取电设备
    powerPlugExtendList: [], // 插卡去掉设备额外的电源列表

    showDetail: false, // 是否展示详情
    detailList: [], // 详情列表
  },
  onLoad () {
    let _curDevInfo = app.globalData.devInfo || {};
    // 设备信息
    let {v, t} = _curDevInfo;

    // 设备品类key
    let _devTypeStr = bizUtil.getDevTypeStr(t);

    // 判断是否继电器设备
    let _pwrAvalible = bizUtil.isPwrAvalibleDev(t);
    this.data.pwrAvalible = _pwrAvalible;
    
    // 取得设备的能力集合 是否支持背光能力
    let _vJson = this.getAbilityByV(v, t);
    let _detailList = this.getDetailListByV(_vJson, _devTypeStr);

    // 判断是否开关还是面板还是插卡取电设备
    let _isPowerPlug = _devTypeStr == "powerPlug"; // 是否插卡取电设备
    // 如果是插卡取电设备，则初始化扩展的电源
    let _powerPlugExtendList = this.getExtendListByFlag(_isPowerPlug);

    this.setData({
      // backlight: _backlight,
      isPowerPlug: _isPowerPlug,
      powerPlugExtendList: _powerPlugExtendList,
      detailList: _detailList
    });

    // 如果是开关面板，则获取设备状态
    if (!this.data.pwrAvalible) {
      return;
    }

    // 初始化上次缓存的状态
    this.status = this.getInitStatus();
    // 这里controlList在onshow后更新
    // this.initPwrStatusByArray(_curDevInfo, {}, this.status)

    // 获取本地设备状态
    this.getLocalDevStatusByDevInfo(_curDevInfo);
  },
  // 本地获取开关面板的状态
  getLocalDevStatusByDevInfo (devInfo) {
    // 如果没有设备信息或者蓝牙状态不可用
    if (!devInfo) {
      return;
    }
    app.globalData.blsdk.queryDeviceStatusWithAddrs([devInfo.a], {delayMax: 7}, (statusJson) => {
      // app.globalData.blsdk.stopBleReceiveService();
      console.error("getLocalDevStatus statusJson = ", statusJson);
      if (!statusJson || statusJson < 0) {
        return;
      }
      // 根据设备状态初始化数据
      this.initPwrStatusByArray(devInfo, statusJson);
    });
  },
  // 初始化
  initPwrStatusByArray (devInfo, {a, id, data}, statusJson) {
    // let _allDevJson = app.globalData.devs || {};
    // 设备状态
    let _status = statusJson;
    console.log("initPwrStatusByArray statusJson = ", statusJson);
    if (!_status) {
      _status = bizUtil.getDevStatusByTAndArray(devInfo?.t, data);
      this.status = _status;
    }
    console.log("initPwrStatusByArray _status = ", _status);
    // 更新数据
    let _list = this.updateControlListByData(_status, this.data.controlList);
    console.log("initPwrStatusByArray _list = ", _list);
    this.setData({
      controlList: _list
    })
  },
  // 更新controlList数据
  updateControlListByData (status, list) {
    if (!status || !list?.length) {
      return list || [];
    }
    for (let i in list) {
      if (status[i] == undefined) {
        continue;
      }
      list[i].valueInt = (Number(status[i]) || 0); // >> i;
    }
    return list;
  },
  // 取得能力
  getDetailListByV ({backlight, metapadEnable}, devTypeStr) {
    console.log("getDetailListByV backlight, metapadEnable = ", backlight, metapadEnable);
    let _list = this.getAllDetailList();
    if (devTypeStr == "powerPlug") {
      return _list;
    }
    // 如果没有设置的状态则不显示功能设置按钮
    if (!backlight) {
      // 没有功能设置页面
      _list = _list.slice(0, 1);
    }
    // 根据ability判断是否有按键绑定能力
    if (metapadEnable) {
      _list.push({
        title: "按键绑定",
        bindTap: "jumpPwrBind",
        key: "metapadEnable"
      })
    }

    return _list;
  },
  getAllDetailList () {
    return [{
      title: "设备详情",
      bindTap: "jumpDetail",
      key: "detail"
    }, {
      title: "功能设置",
      bindTap: "jumpFuncSet",
      key: "funcset"
    }]
  },
  // 取得额外扩展的电源列表
  getExtendListByFlag (isPowerPlug) {
    if (!isPowerPlug) {
      return [];
    }
    return [{
      title: "电源",
      pwrEnable: true,
      key: "pwr"
    }]
  },
  onShow () {
    // 设备信息
    let {n} = app.globalData.devInfo || {};
    // 设置标题
    this.setNavTitle(n);
    // 初始化List
    this.initLists();
    // 隐藏对话框
    this.showDetailDialog(false);
  },
  getAbilityByV (v, t) {
    // 是否支持背光
    let _json = bizUtil.getJsonByAblitys(v, "panel", {t});
    return _json || {};
  },
  setNavTitle (n) {
    ability.setNavigationBarTitle(n || '');
  },
  getMapInArrayByKey (array, key) {
    let _result = {};
    if (!array || !key) {
      return _result
    }
    for (let index in array) {
      let _item = array[index] || {};
      _result[_item[key]] = index;
    }
    return _result;
  },
  initKeyStatusByScene (array) {
    console.log("initKeyStatusByScene array = ", JSON.stringify(array));
    let {d} = app.globalData.devInfo || {};
    let {panelScenes, sids} = bizUtil.getAllPanels(app.globalData.devs);

    console.log("initKeyStatusByScene panelScenes, sids = ", panelScenes, sids, app.globalData.devs);

    // 是否包含多楼层
    let _isFloor = false;
    if (app.globalData.familyInfo?.floor >= 0) {
      _isFloor = true;
    }

    // 根据id设置index
    let _keyMap = this.getMapInArrayByKey(array, "id");
    console.log("initKeyStatusByScene _keyMap = ", array, _keyMap);

    let _allKeys = panelScenes[d];
    console.log("initKeyStatusByScene _keys = ", _allKeys);
    if (!!_allKeys) {
      let _keys = Object.keys(_allKeys) || [];
      for (let key of _keys) {
        let _sceneInfo = sids[_allKeys[key]];
        
        // 这里是默认的按照序号 如果是插卡取电面板则不能根据key来取值index
        let _index = (_keyMap[key] == undefined ? Number(key) : Number(_keyMap[key]));
        // 如果不存在
        if (!array[_index]) {
          continue;
        }

        // 房间id
        let _roomId = Number(_sceneInfo.rid) || 0;

        // 是否多楼层
        let _name = "";
        if (_isFloor) {
          if (_roomId % 32 == 0) {
            // 房间名称
            _name = app.globalData.devs[_roomId]?.n || "";
          } else {
            // 楼层名称
            _name = app.globalData.devs[_roomId + (32 - _roomId % 32)]?.n || "";
            // 房间名称
            _name += app.globalData.devs[_roomId]?.n || "";
          }
          // 增加一个空格
          _name += " ";
        }

        array[_index].value = _name + (_sceneInfo.n || '');
        array[_index].sid = _sceneInfo.sid;
        array[_index].rid = _roomId;
      }
    }
    return array;
  },
  onUnload () {
    // 页面销毁 如果设置过百分比和最佳位置则保存设备状态
    this.saveDevStatus();
  },
  getInitStatus () {
    // 如果不是开关面板则返回空
    if (!this.data.pwrAvalible) {
      return {};
    }
    let {a, d} = app.globalData.devInfo || {};
    // 设备状态
    return app.globalData.allStatus[d + "_" + a];
  },
  saveDevStatus () {
    if (this.data.pwrAvalible || !this.status) {
      return;
    }
    let {a, d} = app.globalData.devInfo || {};
    // 设备状态
    app.globalData.allStatus[d + "_" + a] = this.status;
    // 保存到缓存中
    app.setBizStorage("devStatus", app.globalData.allStatus);
  },
  // 区分群控页面
  initLists () {
    // 是否继电器设备
    let _pwrAvalible = this.data.pwrAvalible;

    // 是否插卡取电设备
    let _isPowerPlug = this.data.isPowerPlug;
    if (_isPowerPlug) {
      let _list = this.getPowerPlugList();
      let _controlList = this.initKeyStatusByScene(_list);
      this.setData({
        controlList: _controlList
      })
    } else {
      // 初始化控制列表
      let {keys, list} = this.getControlList(_pwrAvalible) || [];
      let _controlList = _pwrAvalible ? list : this.initKeyStatusByScene(list);
      this.data.controlList = _controlList; // 这一句不能少
      console.log("initLists _controlList, _pwrAvalible = ", _controlList, _pwrAvalible);
      
      // 开关面板不需要更新
      if (_pwrAvalible && this.status && Object.keys(this.status).length > 0) {
        // 根据status更新列表
        this.initPwrStatusByArray(app.globalData.devInfo, {}, this.status);
      }

      // 设置数据
      this.setData({
        controlList: _controlList,
        keyHalfArray: keys // 开关的按键数 / 2
      })
    }
  },
  // 插卡取电的列表
  getPowerPlugList () {
    let _notEdit = app.globalData.roleNum == 2;
    return [{
      title: '有卡',
      id: 1,
      value: '默认配置',
      edit: !_notEdit
    }, {
      title: '无卡',
      id: 0,
      value: '默认配置',
      edit: !_notEdit
    }]
  },
  // 根据数据区分
  getControlList (pwrAvalible) {
    // 是否继电器设备
    let _devInfo = app.globalData.devInfo || {};
    let _count = bizUtil.getPanelKeysByPid(_devInfo.t);
    // 控制列表
    let _controlList = [];
    let _keys = [];
    let _notEdit = app.globalData.roleNum == 2;
    // 取得默认的长度
    let _defaultNames = [];
    if (pwrAvalible) {
      _defaultNames = app.globalData.devInfo?.e?.vGroups || [];
      console.log("getControlList app.globalData.devInfo?.e = ", app.globalData.devInfo?.e, _defaultNames, _count);
      // 如果为空
      if (_defaultNames.length == 0) {
        _defaultNames = bizUtil.getPanelVGroups(_count);
        app.globalData.devInfo.e = {
          vGroups: _defaultNames
        }
      }
    }
    // 遍历数据
    for (let i = 1; i <= _count; i++) {
      let _obj = {};
      if (pwrAvalible) {
        _obj = {
          title: _defaultNames[i - 1]?.name || ('按键' + i),
          id: i - 1,
          pwrAvalible: pwrAvalible
        }
      } else {
        _obj = {
          title: '按键' + i,
          id: i - 1,
          value: '默认配置',
          edit: !_notEdit
        }
      }
      _controlList.push(_obj);

      // 如果item大于3，则数组增加1
      // (i <= Math.round(_count / 2))
      if (_count >= 4 && (i % 2 == 1)) {
        _keys.push(i);
        // _keys.push(i);
      } else if (_count < 4) {
        _keys.push(i);
      }
    }
    console.log("getControlList _keys = ", _keys, _count);
    return {
      list: _controlList,
      keys: _keys
    };
  },

  /***************** 设备控制相关 ***************/
  showLoading (str) {
    ability.showLoading({
      title: str || ""
    })
  },
  hideLoading () {
    ability.hideLoading();
  },
  // setDataByKey (key, value) {
  //   this.data[key] = value;
  //   this.setData({
  //     [key]: value
  //   })
  // },
  // 功能设置按钮
  toFuncSetting() {
    console.log('toFuncSetting')
    wx.navigateTo({
      url: '../funcSetting/index'
    })
  },
  // 设备详情
  toDevDetail() {
    console.log('toDevDetail')
    wx.navigateTo({
      url: '../deviceDetail/index'
    })
  },
  // // 跳转模式页面
  // jumpConfig () {
  //   ability.navigateTo("../config/index");
  // },
  
  jumpDetailClick () {
    // 判断是否大于2个选项
    if (this.data.detailList?.length > 1) {
      // 显示
      this.showDetailDialog(!this.data.showDetail);
      return;
    }
    this.jumpDetail();
  },
  // 详情按钮
  showDetailDialog(value) {
    this.setData({
      showDetail: value || false
    })
  },
  // 跳转设备详情页面
  jumpDetail () {
    // 跳转页面
    ability.navigateTo("../../light/devProperty/index?remote=0");
  },
  itemClick (e) {

    // 如果是继电器设备，则不执行
    if (this.data.pwrAvalible) {
      return;
    } 
    // 如果是编辑权限，则不进入
    if (app.globalData.roleNum == 2) {
      return;
    }

    let {index} = e.detail;
    let _item = this.data.controlList[index];
    console.log("itemClick _item, index = ", _item, index);
    let _sid = _item.sid || -1;
    let _rid = _item.rid || 0; // 默认全开全关没有rid
    let _title = _item.title || "";

    // 插卡取电 有卡为1 无卡为0
    let _realKey = this.data.isPowerPlug ? _item.id : index;

    ability.navigateTo("/device/panel/configDetail/index?key=" + (_realKey) + "&sid=" + _sid + "&rid=" + _rid + "&title=" + encodeURIComponent(_title));
  },
  funcClick (e) {
    let {index, keyStr} = e.detail;
    console.log("funcClick index = ", index);

    if (keyStr == "open") {
      // todo list 第一个字节为panelId, 第二个字节为开关
      this.actDev([1 << index, 1 << index], index);
    } else if (keyStr == "close") {
      // todo list
      this.actDev([1 << index, 0], index);
    } else if (keyStr == "edit") {
      // todo list
      let _item = this.data.controlList[index];
      ability.navigateTo("/device/light/deviceName/index?from=panel&index=" + index + "&name=" + decodeURIComponent(_item.title));
    }
  },
  checkBeforeAct (value, isNewUart, index) {
    // 当前只有开关面板且获取到设备状态之后才停止
    if (!this.data.pwrAvalible || !this.status) {
      return;
    }
    // 先停止发送再发送数据
    app.globalData.blsdk.stopReceiveAndAdvertising();

    console.log("checkBeforeAct value, isNewUart, index, this.status = ", value, isNewUart, index, this.status);

    if (value.pwr >= 0) {
      // 全开全关
      this.setMapValue(this.status, value.pwr);
    } else {
      // 数据处理
      if (isNewUart) {
        this.setMapValue(this.status, value ? 1 : 0);
      } else {
        this.setMapValue(this.status, (Number(value[1]) || 0) >> index, index);
      }
    }
    console.log("checkBeforeAct this.status = ", this.status);
    // 根据status更新列表
    this.initPwrStatusByArray(app.globalData.devInfo, {}, this.status)
  },
  setMapValue (object, value, keyId) {
    if (!object) {
      console.error("setMapValue object empty ");
      return;
    }

    // 这里有个bug，如果未获取到设备状态时，object为空
    else if (object[keyId] == undefined) {
      object[keyId] = value;
      return;
    }

    for (let key in object) {
      if (keyId != undefined) {
        if (keyId == key) {
          object[key] = value;
          return;
        } else {
          continue;
        }
      }
      object[key] = value;
    }
  },
  // 发送命令
  actDev (cmdArray, index) {
    // 当前只有开关面板且获取到设备状态之后才停止
    this.checkBeforeAct(cmdArray, false, index);
    // 发送控制命令
    let _devInfo = app.globalData.devInfo;
    app.globalData.blsdk.controlWithDevice(_devInfo, cmdArray);
  },

  allOpen () {
    this.newActDev(128);
  },
  allClose () {
    this.newActDev(0);
  },
  newActDev (value) {
    // 当前只有开关面板且获取到设备状态之后才停止
    this.checkBeforeAct(value, true);

    let _devInfo = app.globalData.devInfo;
    // 新协议
    app.globalData.blsdk.controlWithDevice(_devInfo, {
      tlv: 0,
      subType: 32,
      cmds: [value]
    }, {
      isNewUart: true
    });
  },
  
  // 插卡取电的开关点击
  funcExtendClick (e) {
    let {index, keyStr} = e.detail;
    console.log("funcExtendClick index, keyStr = ", index, keyStr);
    // open开 close关
    this.actDev({
      pwr: keyStr == "open" ? 1 : 0
    });
  },
  jumpFuncSet () {
    // 是否支持遥控器
    let _remote = 0; // (this.data.abilitys || {}).remote || 0;
    let _itemArray = this.data.detailList.filter(item => item.key == "funcset");
    let _title = _itemArray[0]?.title || "";
    // 跳转详情配置页面
    ability.navigateTo("/device/light/funcSet/index?remote=" + _remote + "&title=" + encodeURIComponent(_title))
  },
  jumpPwrBind () {
    let _itemArray = this.data.detailList.filter(item => item.key == "metapadEnable");
    let _title = _itemArray[0]?.title || "";
    // 跳转详情配置页面
    ability.navigateTo("../pwrMetaBind/index?title=" + encodeURIComponent(_title) + "&from=panel")
  }
})
