// device/powerstrip/index/index.js
import ability from "../../../utils/classDef/ability";
import bizUtil from '../../../utils/bizUtil';
const app = getApp();

Page({
  data: {
    // 显示列表详情
    showDeail: false,
    // sizes
    sizes: {
      widthRpx: 466, // rpx,
      heightRpx: 466, // rpx
      topMarginT: 76, // 4键
      btmMarginT: 239
    },
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: app.globalData.devInfo?.n || ''
      }
    },
    controlList: [], // 底部按钮
    bleUnavailable: false,

    pwrAvalible: false, // 默认不是继电器设备

    backlight: false, // 默认不支持背光调节

    imgKey: "", // 主图片的key
    bottomBtns: [{
      title: "总开",
      imgStr: "../../img/powerstrip/icon_on.png",
      value: 1
    }, {
      title: "总关",
      imgStr: "../../img/powerstrip/icon_off.png",
      value: 0
    }]
  },
  onLoad () {
    // 设备信息
    let {t} = app.globalData.devInfo || {};
    // // 设置标题
    // this.setNavTitle(n);

    // 判断路数
    let _count = bizUtil.getPanelKeysByPid(t, bizUtil.getDevTypeStr(t));
    // 是否单个插座
    this.isSingleSocket = _count == 1; // bizUtil.isOneSocket(t);
    // 插座或者插排的图标
    let _imgKey = 'socket-' + _count; // this.isSingleSocket ? 'socket-1' : "socket-n";
    
    // 如果是插座
    if (this.isSingleSocket) {
      this.data.bottomBtns[0].title = "开";
      this.data.bottomBtns[1].title = "关";
    }

    this.setData({
      imgKey: _imgKey,
      bottomBtns: this.data.bottomBtns
    });
  },
  // 取得额外扩展的电源列表
  getExtendListByFlag (isPowerPlug) {
    if (!isPowerPlug) {
      return [];
    }
    return [{
      title: "电源",
      pwrEnable: true,
      key: "pwr"
    }]
  },
  onShow () {
    // 设备信息
    let {n} = app.globalData.devInfo || {};
    // 设置标题
    this.setNavTitle(n);
    // 初始化List
    this.initLists();
  },
  getBackLightByAbility (v) {
    // 是否支持背光
    let {backlight} = bizUtil.getJsonByAblitys(v, "panel");
    return backlight || false;
  },
  setNavTitle (n) {
    ability.setNavigationBarTitle(n || '');
  },
  onUnload () {
    // 页面销毁 如果设置过百分比和最佳位置则保存设备状态
    this.saveDevStatus();
  },
  saveDevStatus () {
  },
  // 区分群控页面
  initLists () {
    // 是否单个插座还是插排
    let _isSingleSocket = this.isSingleSocket;
    // 初始化控制列表
    let {list} = this.getControlList(_isSingleSocket) || [];
    console.log("initLists list = ", list);
    // 设置数据
    this.setData({
      controlList: list
    })
  },
  // 根据数据区分
  getControlList (isSingleSocket) {
    if (isSingleSocket) {
      return [];
    }
    // 是否继电器设备
    let _devInfo = app.globalData.devInfo || {};
    let _count = bizUtil.getPanelKeysByPid(_devInfo.t, bizUtil.getDevTypeStr(_devInfo.t));
    // 控制列表
    let _controlList = [];
    // 取得默认的长度
    let _defaultNames = [];
    // if (isSingleSocket) {
    _defaultNames = app.globalData.devInfo?.e?.vGroups || [];
    console.log("getControlList app.globalData.devInfo?.e = ", app.globalData.devInfo?.e, _defaultNames, _count);
    // 如果为空
    if (_defaultNames.length == 0 || (typeof _defaultNames == "object" && Object.keys(_defaultNames) == 0)) {
      _defaultNames = bizUtil.getPanelVGroups(_count, {
        defaultFormat: "线路"
      });
      app.globalData.devInfo.e = {
        vGroups: _defaultNames
      }
    }
    // }
    // 遍历数据
    for (let i = 1; i <= _count; i++) {
      let _obj = {
        title: _defaultNames[i - 1]?.name || "",
        id: i - 1,
        pwrAvalible: true
      }
      _controlList.push(_obj);
    }
    console.log("getControlList _count = ", _count);
    return {
      list: _controlList
    };
  },

  /***************** 设备控制相关 ***************/
  showLoading (str) {
    ability.showLoading({
      title: str || ""
    })
  },
  hideLoading () {
    ability.hideLoading();
  },
  // 功能设置按钮
  toFuncSetting() {
    this.setData({
      showDeail: false
    })
    console.log('toFuncSetting')
    wx.navigateTo({
      url: '../funcSetting/index'
    })
  },
  // 设备详情
  toDevDetail() {
    this.setData({
      showDeail: false
    })
    console.log('toDevDetail')
    wx.navigateTo({
      url: '../deviceDetail/index'
    })
  },
  // 跳转设备详情页面
  jumpDetail () {
    // 跳转页面
    ability.navigateTo("../../light/devProperty/index?remote=0");
  },
  funcClick (e) {
    let {index, keyStr} = e.detail;
    console.log("funcClick index = ", index);

    if (keyStr == "open") {
      // todo list 第一个字节为panelId, 第二个字节为开关
      this.actDev([1 << index, 1 << index]);
    } else if (keyStr == "close") {
      // todo list
      this.actDev([1 << index, 0]);
    } else if (keyStr == "edit") {
      // todo list
      let _item = this.data.controlList[index];
      ability.navigateTo("/device/light/deviceName/index?from=powerstrip&index=" + index + "&name=" + decodeURIComponent(_item.title));
    }
  },
  actDev (cmdArray) {
    let _devInfo = app.globalData.devInfo;
    app.globalData.blsdk.controlWithDevice(_devInfo, cmdArray);
  },
  // 总开关点击
  btmBtnClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.bottomBtns[index];
    console.log("funcExtendClick index = ", index);
    // open开 close关
    this.actDev([0xff, _item.value ? 0xff : 0]);
  }
})
