const ability = require("../../../utils/classDef/ability");
const app = getApp();
const bizUtil = require("../../../utils/bizUtil")
// device/panel/configDetail/index.js
Page({

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "面板选择"
      }
    },
    sceneList: [],
    sceneIndex: -1,
    roomChanged: false, // 是否弹出选择房间
    rooms: [],
    roomInfo: {
      id: 0, // 默认全屋
      n: '全屋'
    },
    keyIndex: 0, // 按键

    isBindSid: -1, // 是否绑定
    isBindRid: -1, // 绑定的场景房间

    // 发送时唤醒文案配置
    showSaveDialog: false,
    sendingConfig: {
      devTypeStr: 'bodysensor'
    }
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    let {key, sid, rid, title, devTypeStr, notLow} = options;
    console.log("onLoad options = ", options);
    this.data.keyIndex = Number(key) || 0;
    this.data.isBindSid = Number(sid) || -1;
    this.data.isBindRid = rid == undefined ? -1 : (Number(rid) || 0);
    this.data.sendingConfig.devTypeStr = devTypeStr || "";
    // 是否低功耗配置 默认需要唤醒
    this.notLow = notLow || (devTypeStr == "panelsensor" || devTypeStr == "sossensor" || devTypeStr == "radarsensor");
    // 设置页面标题
    this.setNavBarTitle(decodeURIComponent(title || ""));
    // 初始化房间
    this.initRooms();
    // 根据传入的数据显示默认的房间信息
    this.initRoomInfoByData(this.data.isBindRid);
    this.initListByRoom(this.data.isBindSid);
    // 初始化按钮
    let _sceneId = this.data.isBindSid < 0 ? 0 : bizUtil.getSceneId(this.data.isBindSid, this.data.isBindRid);
    this.initActions(_sceneId);
    this.setData({
      isBindSid: this.data.isBindSid,
      sendingConfig: this.data.sendingConfig
    })
  },
  initRoomInfoByData (roomId) {
    let _roomInfo = undefined;
    if (roomId != undefined) {
      let _roomDetail = app.globalData.devs[roomId];
      if (!!_roomDetail) {

        _roomInfo = {
          id: _roomDetail.id,
          n: this.getFloorName(_roomDetail.id) + _roomDetail.n
        };

      }
    } 
    // 默认数据
    _roomInfo = _roomInfo || {
      id: 0, // 默认全屋
      n: '全屋'
    };
    this.data.roomInfo = _roomInfo;
    this.setData({
      roomInfo: this.data.roomInfo
    })
  },

  initActions (sceneId) {
    let _list = this.getAllActions();
    // 如果sceneId大于0
    if (sceneId <= 0) {
      _list = _list.slice(1);
      _list[0].classStr = _list[0].classStr.replace("marginL49", "");
    }
    this.data.actions = _list;
    this.setData({
      actions: _list
    })
  },
  getAllActions () {
    return [{
      name: "解绑",
      classStr: 'delBtn',
      handles: "unbindPanel"
    }, {
      name: "绑定",
      classStr: 'saveBtn marginL49',
      handles: "bindPanel"
    }]
  },
  setNavBarTitle (title) {
    // 设置标题
    ability.setNavigationBarTitle(title != undefined ? (title || "") : '选择智能场景');
  },

  // 初始化房间
  initRooms () {
    // 所有设备
    let _allDevs = app.globalData.devs || {};
    // 取得房间信息
    let {rooms} = bizUtil.getRoomsByDevs(_allDevs);
    // 按照楼层+房间排序
    this.data.rooms = rooms.sort((a, b)=>{
      let _floorA, _floorB = 0;
      if (a.id % 32 == 0) {
        _floorA = Math.round(a.id / 32) - 1;
      } else {
        _floorA = parseInt(a.id / 32);
      }
      if (b.id % 32 == 0) {
        _floorB = Math.round(b.id / 32) - 1;
      } else {
        _floorB = parseInt(b.id / 32);
      }
      return (_floorA * 50 + a.id % 32)-(_floorB * 50 + b.id%32);
    });
    this.setData({
      rooms: this.data.rooms
    })
  },
  initListByRoom (sid) {
    let _roomInfo = this.data.roomInfo;
    let _roomId = _roomInfo.id || 0;
    // 暂时屏蔽显示所有房间的场景
    // if (_roomId == 0) {
    //   // 如果是全屋 取得所有设备
    //   let _devs = app.globalData.devs || {};
    //   let {rooms} = bizUtil.getRoomsByDevs(_devs);
    //   let {scenes} = bizUtil.getAllDevsBySort(rooms, _devs, 'scenes');
    //   // 取得所有设备信息
    //   this.data.sceneList = scenes;
    // } else {
    this.data.sceneList = (app.globalData.devs[_roomId] || {}).scenes || [];
    // }
    // 取得选中场景
    for (let index in this.data.sceneList) {
      let _item = this.data.sceneList[index];
      if (_item.sid == sid) {
        this.data.sceneIndex = index;
      }
    }
    this.setData({
      sceneList: this.data.sceneList,
      sceneIndex: this.data.sceneIndex
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  selectScene (e) {
    let {index} = e.currentTarget.dataset;
    // // 判断之前是否选中
    // if (index == this.data.sceneIndex) {
    //   this.data.sceneIndex = -1;
    // } else {
    //   this.data.sceneIndex = index;
    // }
    // index == this.data.sceneIndex ? -1 : index
    this.setSceneIndex(index)
  },

  setSceneIndex (value) {
    this.data.sceneIndex = value;
    this.setData({
      "sceneIndex": this.data.sceneIndex
    })
  },

  // 根据房间id取得楼层名称
  getFloorName (_roomId) {
    let _name = "";
    // 是否多楼层
    if ((_roomId % 32 != 0) && app.globalData.familyInfo?.floor >= 0) {
      // 多楼层
      let _floorName = app.globalData.devs[_roomId + 32 - _roomId % 32]?.n;
      _floorName && (_name = _floorName + " | ");
    }
    return _name;
  },

  chooseRoom (e) {
    let {index} = e.currentTarget.dataset;
    let _roomInfo = this.data.rooms[index || 0] || {};

    // 同一个房间
    if (this.data.roomInfo.id == _roomInfo.id) {
      return;
    }
    // 默认选择第一个
    this.data.sceneIndex = 0;

    let _roomId = _roomInfo.id;
    this.data.roomInfo.n = this.getFloorName(_roomId) + _roomInfo.n;
    this.data.roomInfo.id = _roomId;

    // // 是否多楼层
    // if ((_roomId % 32 != 0) && app.globalData.familyInfo?.floor >= 0) {
    //   // 多楼层
    //   let _floorName = app.globalData.devs[_roomId + 32 - _roomId % 32]?.n;
    //   _floorName && (this.data.roomInfo.n = _floorName + " | " + this.data.roomInfo.n);
    // }

    this.setData({
      sceneIndex: this.data.sceneIndex,
      roomInfo: this.data.roomInfo
    });
    // 隐藏
    this.hidePop();
    // 显示对应的房间信息
    this.initListByRoom(this.data.isBindSid);
  },
  showPop () {
    this.hidePop(true);
  },
  closePop () {
    this.hidePop();
  },
  hidePop (isOpen) {
    this.data.roomChanged = isOpen || false;
    this.setData({
      roomChanged: this.data.roomChanged
    })
  },

  unbindPanel () {
    // this.managePanel(-1, "解绑中…");
    // 未选择场景且没有绑定过
    if (this.data.isBindSid <= 0) {
      return this.back();
    }
    // 提示等待中
    this.showLoading('解绑中…');
    let _sid = this.data.isBindSid;
    let _rid = this.data.isBindRid || 0;
    let {a} = app.globalData.devInfo || {};
    // let _roomId = this.data.isBindRid;
    // _roomId = (_roomId < 0) ? 0 : _roomId;
    console.log("unbindPanel this.data.keyIndex, a = ", this.data.keyIndex, a);
    // 恢复至默认情景
    app.globalData.blsdk.resetPannelBindCommandWithShortAddr(a, this.data.keyIndex, (result) => {
      console.log("next result = ", result);
      this.hideLoading();
      let _str = "";
      if (result < 0 || result === false || result == undefined || result.errCode) {
        _str =  '解绑失败';
      } else {
        _str =  '解绑成功';
        setTimeout(() => {
          this.data.sceneIndex = -1;
          let _sceneArray = app.globalData.devs[_rid]?.scenes || [];
          // 根据sid和rid取得sceneInfo
          let _index = (_sceneArray).findIndex(item=>item.sid==_sid);
          if (_index < 0) {
            return this.back();
          }
          // filterScenes
          // 绑定
          this.manageScene(_sceneArray[_index], app.globalData.devInfo, false);
          return this.back();
        }, 1500);
      }
      ability.showToast({
        title: _str
      });
    }, ((_rid << 16) + (_sid || 1)));
  },
  bindPanel () {
    this.managePanel(this.data.sceneIndex, "绑定中…");
  },

  managePanel (sceneIndex, str) {
    let {a} = app.globalData.devInfo || {};
    let _roomId = this.data.roomInfo.id;
    let _selectIndex = sceneIndex; // this.data.sceneIndex;
    let _sceneInfo = this.data.sceneList[_selectIndex];
    console.log("next _sceneInfo = ", _selectIndex, this.data.sceneList, this.data.keyIndex, this.data.isBindSid);
    // 参数
    let _sid = 0;
    // 回调方法
    // let _callBack = ;
    // 如果不选择场景
    if (_selectIndex < 0) {
      ability.showToast({
        title: "请选择要绑定的场景"
      });
      return;
    } else {
      // 提示等待中
      this.showLoading(str || '');
      _sid = _sceneInfo.sid;

      // 更新场景
      app.globalData.blsdk.roomSceneAddWithPannelAddr(a, this.data.keyIndex, (_roomId << 16) + _sid, (result) => {
        console.log("next result = ", result);
        this.hideLoading();
        let _str = "";
        if (result < 0 || result === false || result == undefined || result.errCode) {
          _str =  '失败';
        } else {
          _str =  '成功';
          setTimeout(() => {
            this.dealAfterSave(_sceneInfo, app.globalData.devInfo);
          }, 1500);
        }
        ability.showToast({
          title: _str
        });
      });
    }
  },
  getSceneInfoBySid () {
    let _roomId = this.data.isBindRid;
    let {scenes} = app.globalData.devs[(_roomId < 0) ? 0 : (_roomId || 0)] || [];
    if (!scenes) {
      return;
    }
    let _sid = this.data.isBindSid;
    for (let item of scenes) {
      if (item.sid == _sid) {
        return item;
      }
    }
    return undefined;
  },
  dealAfterSave (sceneInfo, devInfo) {
    console.log("dealAfterSave sceneInfo = ", sceneInfo);
    if (!sceneInfo) {
      let _sceneInfo = this.getSceneInfoBySid();
      console.log("dealAfterSave sceneInfo = ", _sceneInfo, this.data.isBindRid, this.data.isBindSid);
      if (!!_sceneInfo) {
        // 解绑 根据sid和pid取得
        this.manageScene(_sceneInfo, devInfo, false);
      }
      return this.back();
    }
    let _selectIndex = this.data.sceneIndex;
    if (_selectIndex >= 0) {
      console.log("dealAfterSave this.data.isBindSid, _selectIndex = ", this.data.isBindSid, _selectIndex, this.data.isBindRid);
      // 如果之前有绑定过，则需要删除之前的数据
      if (this.data.isBindSid > 0) {
        let _devs = app.globalData.devs;
        // 取得所有场景
        let scenes = bizUtil.getAllDevsFilter(_devs, undefined, 'scenes');
        // let {scenes} = bizUtil.getAllDevsBySort(_devs, _devs, 'scenes');
        let _allScenes = scenes || [];
        // 取得老场景
        let _filterScene = _allScenes.filter((a) => (a.sid == this.data.isBindSid && (this.data.isBindRid < 0 || (this.data.isBindRid >= 0 && this.data.isBindRid == a.rid))))
        console.log("dealAfterSave _filterScene = ", _filterScene);
        if (_filterScene.length > 0) {
          // 删除老场景的该设备的该按键
          this.manageScene(_filterScene[0], devInfo);
        }
      }
      // 绑定
      this.manageScene(sceneInfo, devInfo, true);
    } else {
      // 解绑
      this.manageScene(sceneInfo, devInfo, false);
    }
    // 返回
    this.back();
  },
  manageScene (sceneInfo, devInfo, isBind) {
    console.log("manageScene sceneInfo, devInfo, isBind = ", sceneInfo, devInfo, isBind);
    // 保存场景
    let _keyIndex = Number(this.data.keyIndex);
    console.log("manageScene _keyIndex = ", _keyIndex);
    let {d, id} = devInfo || {};
    sceneInfo["panels"] = sceneInfo["panels"] || [];
    // 是否解绑
    if (!isBind) {
      bizUtil.delPanelInScene(sceneInfo["panels"], d, _keyIndex);
      console.log("manageScene unbind allDevs = ", app.globalData.devs, sceneInfo["panels"]);
      return;
    }
    // 添加数据
    sceneInfo["panels"].push(bizUtil.addScenePanelInfo(d, _keyIndex));
  },
  showLoading (str) {
    console.log("showLoading this.notLow = ", this.notLow);
    if (this.notLow) {
      ability.showLoading({
        title: str || "",
        mask: true
      });
      return 
    }
    this.manageDialog(true);
  },
  hideLoading () {
    if (this.notLow) {
      ability.hideLoading();
      return 
    }
    this.manageDialog(false);
  },
  manageDialog (value) {
    this.data.showSaveDialog = value || false;
    this.setData({
      showSaveDialog: this.data.showSaveDialog
    })
  },
  back () {
    ability.navigateBack();
  }
})