// device/temphumidity/control/index.js
import * as echarts from '../js/echarts/ec-canvas/echarts';
import {Utils} from '../../../utils/util'
import GROUPS from "../../../static/js/jsConfig/groups"
const bizUtil = require("../../../utils/bizUtil")
const ability = require("../../../utils/classDef/ability")
const historyCommon = require("../../jsConfig/history")
const app = getApp();

Page({

  /**
   * 页面的初始数据
   */
  data: {
    // 获取的最近的上报数据
    dataInfo: {
      curTemp: "-", // 当前温度 获取到的数据除以10
      unit: "℃", // 1是摄氏，2是华氏
      curhumidity: "-", // 当前温度
      percent: "%",
    },
    controlList: [], // 底部列表

    // 绘图
    ec: {},

    isAllEmpty: false, // 数据是否全部为空

    showDetail: false, // 是否展示详情
    detailList: [], // 详情列表

    // 懒加载绘图
    lazyEc: {
      lazyLoad: true
    }
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    // let _devInfo = app.globalData.devInfo;
    // 设置标题
    // this.setNavTitle(_devInfo?.n);
    // 发现数据
    this.dataComplete = false;
    // 详细列表初始化
    let _list = this.getDetailListByV();
    this.setData({
      detailList: _list
    })
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(title || "");
  },
  getDetailListByV () {
    let _list = this.getAllDetailList();
    return _list;
  },
  getAllDetailList () {
    return [{
      title: "设备详情",
      bindTap: "jumpDetail",
      key: "detail"
    }, {
      title: "功能设置",
      bindTap: "jumpFuncSet",
      key: "funcset"
    }]
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {
    console.log("onReady");
    // 获取数据
    this.isReady = true;
    // 尝试绘图
    this.initLazyChart();
  },
  // for test
  destoryChart () {
    if (!this.lazyChart) {
      return;
    }
    this.lazyChart.dispose();
  },
  // 懒加载绘图 // for test
  initLazyChart () {
    // 如果页面没有ready或者已经初始化懒加载实例
    if (!this.isReady || !this.dataComplete) {
      return;
    }
    this.lazyComponent = this.selectComponent('#lazy-echart-pie');
    this.lazyComponent.init((canvas, width, height, dpr) => {
      this.lazyChart = this.initChart(canvas, width, height, dpr);
      return this.lazyChart;
    });
  },

  // 初始化绘图
  initChart (canvas, width, height, dpr) {
    console.log("initChart this.isReady = ", this.isReady, this.timeList, this.temperatureList, this.humidityList);
    var chart = echarts.init(canvas, null, {
      width: width,
      height: height,
      devicePixelRatio: dpr
    });
    // for test
    // this.echartInfo = chart;
    canvas.setChart(chart);
    chart.setOption(this.getCharsOptions());
    return chart;
  },
  refreshEcharts() {
    // if (!this.echartInfo) {
    //   return;
    // }
    // this.echartInfo.setOption(this.getCharsOptions());
  },
  getCharsOptions () {
    // 是否数据为空
    let _isEmpty = (this.timeList?.length || 0) == 0;
    let _unitStr = this.data.dataInfo?.unit || "";

    var option = {
      // title: {},
      legend: {
        data: ['当前温度' + _unitStr, '当前湿度%'],
        top: 5,
        bottom: 10,
        z: 100
      },
      grid: {
        containLabel: true,
        top: 50, 
        bottom: 10,
        left: 0,
        right: 30
      },
      tooltip: {
        show: false,
        // trigger: 'axis',
        // formatter: function(params1, params2) {
        // },
      },
      xAxis: {
        type: 'category',
        boundaryGap: false,
        show: !_isEmpty,
        splitLine: {
          show: true, // Y轴分割线显示
          lineStyle: {
            color: '#BFBFBF',
            type: "dashed"
          }
        },
        data: this.timeList || [],
        axisLine: { // 坐标轴轴线相关设置
          lineStyle: {
            type: "dashed",
            width: 1,
            color: "#BFBFBF",
          }
        },
        axisTick: { // 刻度设置
          length: 0
        },
        axisLabel: {
          // interval: 0, // 刻度不间隔
          formatter: function (value, index) {
            return value.replace(" ", "\n");
            // let _lines = [];
            // var newParamsName = "";
            // var allLength = value.length;
            // var setLength = 7;  //一行显示几个字
            // var rowNumber = Math.ceil(allLength / setLength);
            // if (allLength > setLength) {
            //   for (var p = 0; p < rowNumber; p++) {
            //     var start = p * setLength;
            //     var end = start + setLength;
            //     _lines.push(value.substring(start, end));
            //   }
            // } else {
            //   _lines.push(value);
            // }
            // return _lines.join("\n");
          }
        }
      },
      yAxis: {
        show: false,
        axisLine: { onZero: false },
      },
      series: [{
        name: '当前温度' + _unitStr,
        type: 'line',
        // smooth: true,
        data: this.temperatureList || [],
        label: {
          show: true,
          position: 'top',
          // formatter: '{c}℃',
          formatter: (params) => {
            return ((params?.value || 0) - 100).toFixed(1) + _unitStr
          },
          // valueFormatter: (value) => (value - 100) + "℃", // 5.3.0+支持
          color: "#999999",
          fontSize: 12
        },
        itemStyle: {
          color: "#70B6D0"
        }
      }, {
        name: '当前湿度%',
        type: 'line',
        // smooth: true,
        data: this.humidityList || [],
        label: {
          show: true,
          position: 'bottom',
          formatter: '{c}%',
          color: "#999999",
          fontSize: 12
        },
        itemStyle: {
          color: "#6086E4"
        }
      }]
    };
    return option
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    let _devInfo = app.globalData.devInfo || {};
    // 设置标题
    this.setNavTitle(_devInfo.n);
    // 初始化单位
    this.initUnit();
    // 获取绘图数据
    this.getCurrentData();
    // 底部列表
    this.initControlList();
    // 隐藏对话框
    this.showDetailDialog(false);
  },

  // 初始化温度单位
  initUnit () {
    let _status = this.getDevStatus();
    console.log("initUnit _status = ", _status);
    this.data.dataInfo.unit = _status.unit;
    this.setData({
      dataInfo: this.data.dataInfo
    })
  },

  getDevStatus (isFirst) {
    // 第一次
    // if (isFirst) {
    // 取得所有设备保存的状态数据
    let _allDevStatus = app.globalData.allStatus || {};
    // 根据设备did+短地址获取设备状态
    let {d, a} = app.globalData.devInfo || {};
    // 设备状态
    let _devStatus = _allDevStatus[d + "_" + a] || this.data.dataInfo || {};
    // }
    // 取得当前设备的状态
    return _devStatus || {};
  },

  initControlList () {
    let _isEdit = app.globalData.roleNum != 2;
    let _list = this.getAllControlList(_isEdit);
    if (!_isEdit) {
      let _index = _list.findIndex(item => item.key == "set");
      console.log("initControlList _index = ", _index);
      if (_index >= 0) {
        _list.splice(_index, 1);
      }
    }
    this.setData({
      controlList: _list
    })
  },
  getAllControlList (isEdit) {
    return [{
      title: '设置',
      key: "set",
      id: -1,
      value: '',
      edit: true,
      img: "/device/img/bodysensor/set.png"
    }, {
      title: '历史记录',
      key: "history",
      id: -1,
      value: '',
      edit: true,
      img: "/device/img/history.png"
    }, {
      title: '智能场景',
      id: 0,
      value: '',
      edit: true, // isEdit || false,
      img: "/device/img/metapad/scene.png"
    }]
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // // 销毁绘图
    // this.destoryChart();
    // 关闭蓝牙监听器
    !!app.globalData.blsdk && (app.globalData.blsdk.destory());
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  // 取得温湿度数据
  getCurrentData () {
    // let _curArray = this.getCurrentYMD();
    let _curStr = this.getDayStartByYYYYMMDD(); // _curArray
    console.log("getCurrentData _curStr = ", _curStr);
    this.getDevHistoryByDate(_curStr);
  },
  // 取得当天时间
  getCurrentYMD () {
    let {year, month, day, hour, minute, second} = Utils.getDateObj();
    return [year, month, day, hour, minute, second];
  },
  // 取得某天的0点到23：59：59秒
  getDayStartByYYYYMMDD () {
    return Utils.formatTime(false, {
      sep1: "/", // ios需要2018/10/07 11:00:00这个格式才能转
      sep2: " "
    });
    // let _dateArray = dateArray || [];
    // // 转成标准格式
    // let _formatArray = _dateArray.map(item => (item || 0).toString().padStart(2, 0))
    // return _formatArray.join("-");
  },

  getTimeByZone (dataStr) {
    let _dateStr = dataStr || "";
    console.log("getDevHistoryByDate _dateStr = ", _dateStr);
    let _seperatorMinute = Utils.getSeperatorMinite();
    // 这里注释的是原先从当天0点查到23：59，后面改成查24小时之内
    // let _startTimeTmpStr = _dateStr + " 00:00:00";
    // let _startDate = new Date(new Date(_startTimeTmpStr).getTime() + _seperatorMinute * 60 * 1000);
    
    console.log("getDevHistoryByDate _seperatorMinute = ", _seperatorMinute);
    
    let _utcTime = new Date(_dateStr);
    // console.log("getDevHistoryByDate _utcTime = ", _utcTime);
    let _utcTimestamp = _utcTime.getTime();
    // console.log("getDevHistoryByDate _utcTimestamp = ", _utcTimestamp);
    let _startDate = new Date(_utcTimestamp - _seperatorMinute * 60 * 1000);
    // console.log("getDevHistoryByDate _startDate = ", _startDate);
    
    let _startTimeStr = Utils.formatTime(_startDate);
    console.log("getDevHistoryByDate _startTimeStr = ", _startTimeStr);
    let _endDate = new Date(_startDate.getTime() - (24 * 3600 - 1) * 1000);
    // let _endDate = new Date(_startDate.getTime() + (24 * 3600 - 1) * 1000);
    let _endTimeStr = Utils.formatTime(_endDate);
    console.log("getDevHistoryByDate _endTimeStr = ", _endTimeStr);
    return {
      startStr: _endTimeStr.replace(" ", "_"),
      endStr: _startTimeStr.replace(" ", "_")
    }
  },

  getDevHistoryByDate (dataStr) {
    // 当前家庭信息
    let {sid, skey, groupIndex} = app.globalData.familyInfo;
    let _devInfo = app.globalData.devInfo;
    let {startStr, endStr} = this.getTimeByZone(dataStr); // dataStr || "";

    // 上报的集群域名
    let _groupStr = GROUPS[Number(groupIndex) || 0]?.value || GROUPS[0].value;

    console.log("getDevHistory sid, skey, app.globalData.familyInfo = ", sid, skey, app.globalData.familyInfo);
    app.globalData.blsdk.getDevHistory({
      remoteHost: _groupStr,
      did: sid,
      token: skey,
      devInfo: _devInfo,
      timeObj: {
        start: startStr, // _dateStr + "_00:00:00",
        end: endStr, // _dateStr + "_23:59:59", 
        step: 1000, 
        offset: 0
      }
    }).then(res => {
      console.log("getDevHistoryByDate res = ", res);
      let _json = this.dealResponse(res);
      console.log("getDevHistoryByDate _json = ", _json);
      // 更新温湿度数据 初始化表格
      this.updateChartsAndDataInfo(_json);
    }).catch(e => {
      // 设置为空数据
      console.error("getDevHistoryByDate e = ", e);
      ability.showToast({
        title: "查询数据失败"
      });
      // 初始化表格
      this.updateChartsAndDataInfo({
        humidityList: [], temperatureList: [], timeList: []
      });
    });
  },
  updateChartsAndDataInfo (json) {
    let {humidityList, temperatureList, timeList} = json || {};
    this.humidityList = humidityList;
    this.temperatureList = temperatureList;
    this.timeList = timeList;

    // 发现数据
    this.dataComplete = true;

    // 是否全部为空
    let _isAllEmpty = (humidityList.length + temperatureList.length + timeList.length) == 0;
    if (!_isAllEmpty) {
      console.log("updateChartsAndDataInfo temperatureList[0] = ", temperatureList[0]);
      this.data.dataInfo.curTemp = temperatureList[0] == undefined ? "-" : (temperatureList[0] - 100).toFixed(1); // 温度
      this.data.dataInfo.curhumidity = humidityList[0] == undefined ? '-' : humidityList[0]; // 湿度
    } else {
      this.data.dataInfo.curTemp = "-"; // 温度
      this.data.dataInfo.curhumidity = "-"; // 湿度
    }
    console.log("updateChartsAndDataInfo _isAllEmpty = ", _isAllEmpty);

    // 如果为空不绘图
    if (!_isAllEmpty) {
      // 尝试绘图
      this.initLazyChart();
    }

    // 设置图表
    this.setData({
      // ec: {
      //   onInit: _isAllEmpty ? null : this.initChart
      // },
      isAllEmpty: _isAllEmpty,
      dataInfo: this.data.dataInfo
    });
    // // 更新数据
    // this.refreshEcharts();
  },

  // 条目点击
  itemClick (e) {
    console.log("itemClick e = ", e);
    let {index} = e.detail;
    let {sid, rid, id, title, key} = this.data.controlList[index] || {};

    // 是否进入历史记录页面
    if (key == "history") {
      return ability.navigateTo("/device/sensor/history/index?devTypeStr=" + bizUtil.getDevTypeStr(app.globalData.devInfo?.t));
    } else if (key == "set") {
      // 人体传感器设置页
      return ability.navigateTo("../set/index?title=" + encodeURIComponent(title));
    }

    // 智能场景 判断是否绑定
    // let _editIndex = (sid >= 0) ? sid : -1;
    ability.navigateTo("../scene/index?title=" + encodeURIComponent(title)); // ?editIndex=" + _editIndex)
  },
  // 解析数据
  dealResponse (res) {
    // 温度、湿度、时间
    let humidityList = [];
    let temperatureList = [];
    let timeList = [];

    let {table} = res || {};
    let {cnt, did, total, values} = (table || [])[0] || {};
    if (!values || values.length == 0) {
      return {
        humidityList, temperatureList, timeList
      };
    } 
    for (let item of values) {
      // 数据处理
      let {occurtime, lowbattery, temperature, humidity, tempunit} = item;
      if (temperature == undefined && humidity == undefined) {
        continue;
      }

      // 原先没有增加时区的适配
      let _time = bizUtil.getShowTimeStr(occurtime);
      // console.log("dealResponse _time = ", _time);
      timeList.push(_time);

      // // 时间 这里偷懒 没有做时区转换
      // let _time = (occurtime?.split("_") || [])[1]?.slice(0, 5);
      // timeList.push(_time);

      // 温度 这里特殊处理把温度+100，为了保证温度在湿度上面，偷懒用 这里也要判断温度单位
      if (temperature == undefined) {
        temperatureList.push(undefined);
      } else {
        let {temp, unit} = historyCommon.getTempByHistoryJson(item);
        temperatureList.push((temp + 100));
      }
      
      // 湿度
      if (humidity == undefined) {
        humidityList.push(undefined);
      } else {
        humidityList.push(humidity || 0);
      }
      // 判断个数
      if (humidityList.length >= 5) {
        break;
      }
    }
    return {
      humidityList, temperatureList, timeList
    };
  },
  // 跳转设备详情页面
  jumpDetail () {
    // 跳转页面
    ability.navigateTo("../../light/devProperty/index?remote=0");
  },

  
  jumpDetailClick () {
    // 判断是否大于2个选项
    if (this.data.detailList?.length > 1) {
      // 显示
      this.showDetailDialog(!this.data.showDetail);
      return;
    }
    this.jumpDetail();
  },
  // 详情按钮
  showDetailDialog(value) {
    this.setData({
      showDetail: value || false
    })
  },
  // 跳转设备详情页面
  jumpDetail () {
    // 跳转页面
    ability.navigateTo("../../light/devProperty/index?remote=0");
  },
  jumpFuncSet () {
    // 是否支持遥控器
    let _remote = 0; // (this.data.abilitys || {}).remote || 0;
    let _itemArray = this.data.detailList.filter(item => item.key == "funcset");
    let _title = _itemArray[0]?.title || "";
    // 跳转详情配置页面
    ability.navigateTo("/device/light/funcSet/index?remote=" + _remote + "&title=" + encodeURIComponent(_title))
  }
})