// device/temphumidity/scene/index.js
const ability = require("../../../utils/classDef/ability");
const bizUtil = require("../../../utils/bizUtil")
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    tempList: [], // 温度按钮
    humidityList: [], // 湿度按钮
    actions: [], // 底部按钮
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title} = options || {};
    // 设置标题
    this.setNavTitle(decodeURIComponent(title || ""));
    // 是否只有查看权限
    let _canEdit = this.canEdit();
    console.log("onLoad _canEdit = ", _canEdit);
    // 取得温度列表
    this.data.tempList = this.getTempList(_canEdit);
    // 取得湿度列表
    this.data.humidityList = this.getHumidityList(_canEdit);
    // 根据参数取得底部按钮个数 根据权限判断是否可以编辑
    let _actions = _canEdit ? (this.getBottomListByIndex()) : [];
    this.setData({
      actions: _actions,
    })
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(title || "");
  },
  canEdit () {
    return app.globalData.roleNum != 2;
  },
  getTempList (edit) {
    return [{
      title: '高温',
      key: "highTemp",
      id: 0,
      value: '默认设置',
      edit: edit || false,
      img: ""
    }, {
      title: '低温',
      key: "lowTemp",
      id: 1,
      value: '默认设置',
      edit: edit || false,
      img: ""
    }]
  },
  getHumidityList (edit) {
    return [{
      title: '高湿度',
      key: "highHumidity",
      id: 2,
      value: '默认设置',
      edit: edit || false,
      img: ""
    }, {
      title: '低湿度',
      key: "lowHumidity",
      id: 3,
      value: '默认设置',
      edit: edit || false,
      img: ""
    }]
  },
  getBottomListByIndex (index) {
    // 取得所有按钮
    let _list = this.getAllActions();
    // if (index == undefined || index < 0) {
    //   // 编辑
    //   _list = _list.slice(1);
    //   _list[0].classStr = _list[0].classStr.replace("marginL49", "");
    // }
    return _list;
  },
  getAllActions () {
    return [/*{
      name: "解绑",
      classStr: 'delBtn',
      handles: "unbindPanel"
    }, */{
      name: "完成",
      classStr: 'completeBtn bgF78D3F white',
      handles: "back"
    }]
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 是否绑定场景
    this.initDataByBindInfo();
  },
  initDataByBindInfo () {
    // 所有设备
    let _allDevs = app.globalData.devs;
    // 取得所有的场景
    let _allSceneDevJson = this.getAllSceneDevJson(_allDevs);
    // 根据场景状态初始化列表值
    let _tempDataList = this.initKeyStatusByScene(this.data.tempList, _allSceneDevJson, _allDevs);
    let _humidityDataList = this.initKeyStatusByScene(this.data.humidityList, _allSceneDevJson, _allDevs);
    this.setData({
      tempList: _tempDataList, // 温度按钮
      humidityList: _humidityDataList, // 湿度按钮
    })
  },
  // 取得所有面板等绑定的场景数据
  getAllSceneDevJson (allDevs) {
    // let {panelScenes, sids} = 
    return bizUtil.getAllPanels(allDevs || {});
  },
  // 根据列表设置绑定数据
  initKeyStatusByScene (array, allSceneDevJson, allDevs) {
    if (!array) {
      return [];
    }
    console.log("initKeyStatusByScene array = ", JSON.stringify(array));
    let {d} = app.globalData.devInfo || {};
    let _allDevs = allDevs || {};
    let {panelScenes, sids} = allSceneDevJson || {};

    console.log("initKeyStatusByScene panelScenes, sids = ", panelScenes, sids, _allDevs);
    // 设备的场景绑定数据
    let _allKeys = panelScenes[d];

    // 数组真实有效的长度 防止有额外的标题等无效长度
    let _extraLen = array.filter(item => item.id < 0);
    let _length = _extraLen.length;

    console.log("initKeyStatusByScene _keys = ", _allKeys);
    // 如果不存在则返回
    if (!_allKeys) {
      return array;
    }
    // 遍历列表
    for (let item of array) {
      let {id} = item || {};
      let _sid = _allKeys[id]; // 场景id
      let _sceneInfo = sids[_sid]; // 场景信息
      if (_sceneInfo == undefined) {
        item.value = "未设置";
        delete item.sid;
        delete item.rid;
        continue;
      }
      // 房间id
      let _roomId = Number(_sceneInfo.rid) || 0;
      // 是否多楼层
      let _name = this.getShowNameByFloorAndRoomId(_allDevs, _roomId);
      item.value = _name + (_sceneInfo.n || '');
      item.sid = _sceneInfo.sid;
      item.rid = _roomId;
    }
    return array;
  },
  getShowNameByFloorAndRoomId (allDevs, roomId) {
    if (!allDevs) {
      return "";
    }
    // 是否多楼层
    let _name = allDevs[roomId]?.n || "";
    if (roomId % 32 != 0) {
      // 不管有没有多楼层 根据房间id取得楼层信息
      let _floorName = allDevs[roomId + (32 - roomId % 32)]?.n || "";
      // 房间名称
      _name = _floorName ? (_floorName + "|" + _name) : _name;
    }
    return _name
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 关闭蓝牙监听器
    !!app.globalData.blsdk && (app.globalData.blsdk.destory());
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  back () {
    ability.navigateBack();
  },
  tempItemClick (e) {
    this.itemClick(e, this.data.tempList)
  },
  HumidityItemClick (e) {
    this.itemClick(e, this.data.humidityList)
  },
  itemClick (e, array) {
    console.log("itemClick e = ", e);
    if (!array) {
      return;
    }
    // 设备信息
    let _devInfo = app.globalData.devInfo;
    let {index} = e.detail;
    let _item = array[index];
    console.log("itemClick _item = ", _item);

    // 如果没有编辑权限
    if (!_item?.edit) {
      return;
    }

    let _devTypeStr = bizUtil.getDevTypeStr(_devInfo?.t);

    // 是否绑定过场景信息
    let _extendStr = "";
    if (_item.sid >= 0 && _item.rid >= 0) {
      _extendStr = "&rid=" + _item.rid + "&sid=" + _item.sid;
    }
    console.log("itemClick _item, _extendStr = ", _item, _extendStr);

    // 跳转配置 绑定场景 key, sid, rid, title, devTypeStr
    ability.navigateTo("/device/sensor/configDetail/index?key=" + _item.id + "&devTypeStr=" + _devTypeStr + "&title=" + encodeURIComponent(_item.title) + _extendStr);
  }
})