// device/temphumidity/set/index.js
const ability = require("../../../utils/classDef/ability");
const bizUtil = require("../../../utils/bizUtil")
const app = getApp();
const historyCommon = require("../../jsConfig/history")

Page({

  /**
   * 页面的初始数据
   */
  data: {
    tempList: [], // 温度按钮
    humidityList: [], // 湿度按钮
    actions: [], // 底部按钮

    showDialogObj: {
      show: false, // 默认不显示对话框
      title: "",
      buttons: [{
          type: 'default',
          className: 'font32 color333',
          text: '取消',
          value: 0
      }, {
          type: 'primary',
          className: 'font32 color333',
          text: '确定',
          value: 1
      }],
      // 滑块配置
      sliders: {
        leftImg: "/device/img/light/mode/del.png",
        rightImg: "/device/img/light/mode/add.png",
        leftClass: "sDown padding30",
        rightClass: "sUp padding30",
        bgImg: "",
        bgClass: "tempImg",
        width: 590, // rpx
        percent: 50, // 百分比
        minPercent: 0,
        bgHeight: "height50",
        // gradient: true, // 默认不支持滑动时调节
        upEventWhileMove: true,
        // labelTxt: "", // 没有tip
        // unit: "",
        minVal: 0, // 单位摄氏度
        maxVal: 100, // 单位摄氏度
      },
    }, 

    showSaveDialog: false, // 低功耗唤醒需要提示框
    sendingConfig: {
      devTypeStr: 'temphumidity'
    },

    // // 温度单位
    // tempUnit: 
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title} = options || {};
    // 设置标题
    this.setNavTitle(decodeURIComponent(title || ""));
    // 是否只有查看权限
    let _canEdit = this.canEdit();
    // 取得设备状态
    let _devStatus = this.getDevDataJson(app.globalData.devInfo);
    console.log("onLoad _devStatus = ", _devStatus);
    let _unitStr = bizUtil.getUnitByValue(_devStatus?.unit);
    // 温度单位
    this.unitStr = _unitStr;
    console.log("onLoad _unitStr = ", _unitStr);
    // 取得温度列表
    this.data.tempList = this.getTempList(_canEdit, _unitStr);
    // 取得湿度列表
    this.data.humidityList = this.getHumidityList(_canEdit);
    // // 根据温度数值取得滑块的最大最小值
    // this.data.sliders = this.getMaxMinByTempUnit(_unitStr);
    // 根据参数取得底部按钮个数
    let _actions = this.getBottomListByIndex();
    this.setData({
      actions: _actions,
    })
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(title || "");
  },
  getMaxMinByTempUnit (unitStr, {minVal, maxVal}) {
    // let {minVal, maxVal} = this.data.showDialogObj?.sliders || {};
    console.log("getMaxMinByTempUnit minVal, maxVal = ", minVal, maxVal);
    if (unitStr == 2 || unitStr == "°F") {
      minVal = Math.round(historyCommon.turnSheshiToHuaShi(minVal || 0));
      maxVal = Math.round(historyCommon.turnSheshiToHuaShi(maxVal || 0));
    }
    return {
      minVal: minVal,
      maxVal: maxVal
    }; // this.data.sliders;
  },
  canEdit () {
    return app.globalData.roleNum != 2;
  },
  getTempList (edit, unitStr) {
    let _unitStr = unitStr || "";
    console.log("getTempList unitStr = ", unitStr);
    return [{
      title: '高温(≥)',
      key: "ht", // highTemp
      id: 0,
      value: '未设置',
      valeInt: 0,
      edit: edit || false,
      unit: _unitStr
    }, {
      title: '低温(≤)',
      key: "lt", // lowTemp
      id: 1,
      value: '未设置',
      valeInt: 0,
      edit: edit || false,
      unit: _unitStr
    }]
  },
  getHumidityList (edit) {
    return [{
      title: '高湿度(≥)',
      key: "hh", // highHumidity
      id: 2,
      value: '未设置',
      valeInt: 0,
      edit: edit || false,
      unit: "%"
    }, {
      title: '低湿度(≤)',
      key: "lh", // lowHumidity
      id: 3,
      value: '未设置',
      valeInt: 0,
      edit: edit || false,
      unit: "%"
    }]
  },
  getBottomListByIndex (index) {
    // 取得所有按钮
    let _list = this.getAllActions();
    return _list;
  },
  getAllActions () {
    return [/*{
      name: "解绑",
      classStr: 'delBtn',
      handles: "unbindPanel"
    }, */{
      name: "完成",
      classStr: 'completeBtn bgF78D3F white',
      handles: "back"
    }]
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 所有设备
    let _devInfo = app.globalData.devInfo;
    // 设备信息
    this.devStatus = this.getDevDataJson(_devInfo); // 保存临时设备状态
    // 默认温度单位
    this.devStatus.unit = bizUtil.getUnitByValue(this.devStatus?.unit);
    //  上次设置的数据
    this.initDataByDevStaus(this.devStatus);
  },
  initDataByDevStaus (devStatus) {
    // 根据场景状态初始化列表值
    let _tempDataList = this.initKeyStatusByScene(this.data.tempList, devStatus);
    let _humidityDataList = this.initKeyStatusByScene(this.data.humidityList, devStatus);
    this.setData({
      tempList: _tempDataList, // 温度按钮
      humidityList: _humidityDataList, // 湿度按钮
    })
  },
  // 根据列表设置绑定数据
  initKeyStatusByScene (array, allSceneDevJson) {
    if (!array) {
      return [];
    }
    // console.log("initKeyStatusByScene array = ", JSON.stringify(array));
    // let {d} = app.globalData.devInfo || {};
    let _allSceneDevJson = allSceneDevJson || {};

    console.log("initKeyStatusByScene _allSceneDevJson = ", _allSceneDevJson);

    // 数组真实有效的长度 防止有额外的标题等无效长度
    // let _extraLen = array.filter(item => item.id < 0);
    // let _length = _extraLen.length;


    // // 是否温度 如果是温度设置，需要涉及到温度单位的转换
    // let _needTempChange = false; // 默认不需要转
    // let _tempFilterArray = array.filter(item => item.key == "ht" || item.key == "lt");
    // let _isTempList = _tempFilterArray.length > 0;
    // if (!!_isTempList) {
    //   let _lastUnit = _tempFilterArray[0].unit;
    //   // 之前的温度单位
    //   let _currentUnit = bizUtil.getUnitByValue(this.devStatus?.unit);
    //   if (_lastUnit == _currentUnit || ()) {
    //     // 前后单位一致
    //     _needTempChange = false;
    //   } else {
    //     _needTempChange = true;
    //   }
    // }

    // 遍历列表
    for (let item of array) {
      let {id, key} = item || {};
      let _data = _allSceneDevJson[key]; // 设置的按键数据

      // // 判断是否温度需要转换
      // if (_needTempChange) {
      //   let {temp, unit} = historyCommon.getTempByHistoryJson({
      //     temperature: Number(_data) || 0,
      //     tempunit:  // 2是华氏
      //   }, item?.unit || "");

      //   item.value = _data == undefined ? "未设置" : (temp + unit);
      // } else {

        item.value = _data == undefined ? "未设置" : ((_data || 0) + (item?.unit || ""));
      // }
    }
    return array;
  },
  getShowNameByFloorAndRoomId (allDevs, roomId) {
    if (!allDevs) {
      return "";
    }
    // 是否多楼层
    let _name = allDevs[roomId]?.n || "";
    if (roomId % 32 != 0) {
      // 不管有没有多楼层 根据房间id取得楼层信息
      let _floorName = allDevs[roomId + (32 - roomId % 32)]?.n || "";
      // 房间名称
      _name = _floorName ? (_floorName + "|" + _name) : _name;
    }
    return _name
  },
  // 取得所有面板等绑定的场景数据
  getDevDataJson (devInfo) {
    if (!devInfo) {
      return {};
    }
    let _key = this.getKeyByInfo(devInfo);
    let _devStatus = app.globalData.allStatus[_key] || {};
    // 业务处理
    return _devStatus;
  },
  // dealBizJson (devStatus) {
  //   if (!devStatus || Object.keys(devStatus).length == 0) {
  //     return {};
  //   }
  //   // 0和1 2和3   高 - 低校验
  //   let _byte0 = devStatus[0];
  //   let _byte1 = devStatus[1];
  //   let _byte2 = devStatus[2];
  //   let _byte3 = devStatus[3];
  //   // if ()
  // },
  getKeyByInfo (devInfo) {
    if (!devInfo) {
      return "";
    }
    return devInfo.d + "_" + devInfo.a;
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {},

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    // 关闭蓝牙监听器
    !!app.globalData.blsdk && (app.globalData.blsdk.destory());
    // 去除定时器
    bizUtil.clearTimer(this.timeoutTimer);
    // 保存设备状态
    this.saveStatus();
  },
  saveStatus () {
    let _info = app.globalData.devInfo;
    let _key = this.getKeyByInfo(_info);
    app.globalData.allStatus[_key] = this.devStatus || {};
    // 保存设备信息
    app.setBizStorage("devStatus", app.globalData.allStatus);
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  back () {
    ability.navigateBack();
  },
  tempItemClick (e) {
    this.itemClick(e, this.data.tempList)
  },
  HumidityItemClick (e) {
    this.itemClick(e, this.data.humidityList)
  },
  itemClick (e, array) {
    console.log("itemClick e = ", e);
    if (!array) {
      return;
    }
    // 设备信息
    // let _devInfo = app.globalData.devInfo;
    let {index} = e.detail;
    let _item = array[index];
    // 当前点击的id
    let {id, key} = _item;
    this.selectItem = _item;
    console.log("itemClick key = ", key);
    // 默认值
    let _tempValue = this.devStatus[key] || 0;
    // 设置温湿度标题
    this.data.showDialogObj.show = true;

    if (key == "hh" || key == "lh") {
      let _percent = bizUtil.getPercentByVal(_tempValue, this.data.showDialogObj.sliders);
      this.data.showDialogObj.title = _item.title + " " + _tempValue + (_item.unit || "");
      this.data.showDialogObj.sliders.percent = _percent; // 初始化
      // 湿度 0 - 100
      this.data.showDialogObj.sliders.maxVal = 100;
      // 湿度为默认的百分比调节
      this.data.showDialogObj.sliders.changeValue = false;
    } else {
      // 温度 0 - 90
      let _minSheShiVal = 0;
      let _maxSheShiVal = 90;
      // this.data.showDialogObj.sliders.maxVal = _maxSheShiVal;
      // 温度为值调节
      this.data.showDialogObj.sliders.changeValue = true;
      // 根据温度数值取得滑块的最大最小值
      let {maxVal, minVal} = this.getMaxMinByTempUnit(this.unitStr, {minVal: _minSheShiVal, maxVal: _maxSheShiVal});
      this.data.showDialogObj.sliders.maxVal = maxVal;
      this.data.showDialogObj.sliders.minVal = minVal;
      // 当前滑块的值
      let _sliderVal = _tempValue;
      if (_tempValue < minVal) {
        // 大于最大值 通常用于摄氏转华氏时
        _sliderVal = minVal;
      } else if (_tempValue > maxVal) {
        // 大于最大值 通常用于华氏转摄氏时
        _sliderVal = maxVal;
      }
      this.data.showDialogObj.sliders.value = _sliderVal;
      this.data.showDialogObj.title = _item.title + " " + _sliderVal + (_item.unit || "");
      let _percent = bizUtil.getPercentByVal(_sliderVal, this.data.showDialogObj.sliders);
      this.data.showDialogObj.sliders.percent = _percent; // 初始化
    }
    
    console.log("itemClick showDialogObj = ", this.data.showDialogObj);

    // 弹出提示框
    this.setData({
      showDialogObj: this.data.showDialogObj
    })
  },
  showDialog () {
    this.manageDialog(true);
  },
  hideDialog () {
    this.manageDialog();
  },
  manageDialog (value) {
    this.setData({
      [`showDialogObj.show`]: value || false
    })
  },
  // getDataByTitle (title) {
  //   if (!title) {
  //     return {};
  //   }
  //   let _array = title.split(" ");
  //   let _byte1 = _array[1] || "";
  //   let _value = parseInt(_byte1) || 0;
  //   return {
  //     title: _array[0] || "",
  //     value: _value,
  //     unit: _byte1.replace(_value, "")
  //   }
  // },
  isDataSetError (curValue, selectItem, devStatus) {
    if (!devStatus) {
      return;
    }
    let {key} = selectItem || {};
    if (key == "ht") { // id = 0
      let _lowTemp = devStatus['lt'];
      if (_lowTemp == undefined) {
        return;
      }
      if (curValue < _lowTemp) {
        ability.showToast({
          title: "高温值应该大于低温值"
        });
        return true;
      }
    } else if (key == "lt") { // id = 1
      let _highTemp = devStatus['ht'];
      if (_highTemp == undefined) {
        return;
      }
      if (curValue > _highTemp) {
        ability.showToast({
          title: "低温值应该小于高温值"
        });
        return true;
      }
    } else if (key == "hh") { // id = 2
      let _tmp = devStatus["lh"];
      if (_tmp == undefined) {
        return;
      }
      if (curValue < _tmp) {
        ability.showToast({
          title: "高湿度值应该大于低湿度值"
        });
        return true;
      }
    } else if (key == "lh") { // id = 3
      let _tmp = devStatus["hh"];
      if (_tmp == undefined) {
        return;
      }
      if (curValue > _tmp) {
        ability.showToast({
          title: "低湿度值应该大于高湿度值"
        });
        return true;
      }
    }
  },
  sliderChange (e) {
    console.log("sliderChange e = ", e);
    let {isMoving, originValue, percent, value} = e.detail;

    // // 校验高低温和高低湿度是否合理
    // let _result = this.isDataSetError(value, this.selectItem, this.devStatus);
    // if (!!_result) {
    //   return;
    // }

    // 更新标题
    let _title = this.data.showDialogObj.title.split(" ")[0] + " " + value + (this.selectItem?.unit || "");
    this.data.showDialogObj.title = _title;
    this.data.showDialogObj.sliders.value = value;
    this.data.showDialogObj.sliders.percent = percent;
    // 弹出提示框
    this.setData({
      showDialogObj: this.data.showDialogObj
      // [`showDialogObj.title`]: _title,
      // [`showDialogObj.sliders.value`]: value
    })
  },
  bottomClick (e) {
    let {index} = e.currentTarget.dataset;
    // let _item = this.data.showDialogObj.buttons[index];
    console.log("bottomClick index = ", index);
    // 隐藏提示框
    this.hideDialog();

    if (index == 0) {
      // 取消按钮
    } else {
      // 如果没有id点击，则不需要
      if (this.selectItem == undefined) {
        return;
      }
      let _key = this.selectItem?.key;
      let _value = parseInt(this.data.showDialogObj.title.split(" ")[1]);

      // 校验高低温和高低湿度是否合理
      let _result = this.isDataSetError(_value, this.selectItem, this.devStatus);
      if (!!_result) {
        // if (_key == "ht" || _key == "lt") {
        //   // 高低温
        // } else if (_key == "hh"|| _key == "lh") {
        //   // 高低湿
        // }
        return;
      }

      // 保存
      this.devAction(_key, _value);
    }
  },
  // 单个设置
  devAction (key, value) {
    // 展开对话框
    this.manageShowSaveDialog(true);

    // 发送的命令
    let _sendObj = {
      [key]: value
    };
    if (key == 'ht' || key == 'lt') {
      let _unit = this.data.tempList[0]?.unit;
      _sendObj.unit = _unit;
    }
    // 发送命令
    this.actDev(_sendObj, (code) => {
      if (code != 0) {
        // 失败 关闭对话框
        return this.manageShowSaveDialog(false);
      }
      // 确定按钮
      this.devStatus[key] = value;
      // // 判断是否温度单位
      // if (key == "ht" || key == "lt") {
      //   // 保存温度单位
      //   this.devStatus.unit = ;
      // }
      // 刷新数据
      this.initDataByDevStaus(this.devStatus);
      // 定时器五秒后自动关闭
      this.timeoutTimer = setTimeout(() => {
        // 隐藏对话框
        this.manageShowSaveDialog(false);
        // 提示
        ability.showToast({
          title: "保存成功"
        });
      }, 4500);
    });
  },
  actDev (cmdJson, callback) {

    // // for test
    // !!callback && (callback(0));
    // return;

    let _devInfo = app.globalData.devInfo;
    let _result = app.globalData.blsdk.controlWithDevice(_devInfo, cmdJson);
    if (_result?.then) {
      _result.then(res => {
        !!callback && (callback(0));
      })
    } else {
      !!callback && (callback(-1));
    }
  },
  manageShowSaveDialog (value) {
    this.setData({
      showSaveDialog: value || false
    })
  },
})