// device/thermostat/setConfig/index.js
import ability from '../../../utils/classDef/ability'
import bizUtil from '../../../utils/bizUtil'

const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    bottomBtns: [], // 底部按钮
    list: [], // 中间列表条目
    selectedIds: {}, // 默认选中的条目
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title, key, valueStr} = options || {};
    let _devInfo = app.globalData.devInfo;
    // 设置标题
    this.setNavTitle(title);
    // 根据key取得列表
    this.key = key;
    this.data.list = this.getListByKey(key);
    // 初始化默认数据
    this.data.selectedIds = this.getSelectedIds(valueStr);
    console.log("selectedIds, valueStr = ", this.data.selectedIds, valueStr, typeof valueStr, this.key);
    // 底部按钮
    this.data.bottomBtns = this.getBottomBtns();
    // 同步到页面渲染
    this.setData({
      list: this.data.list,
      selectedIds: this.data.selectedIds,
      bottomBtns: this.data.bottomBtns
    })
  },

  // 取得之前缓存的数据
  getSelectedIds (valStr) {
    let _result = {};
    if (!valStr) {
      return _result;
    }
    let _array = decodeURIComponent(valStr).split(",");
    for (let item of _array) {
      _result[item] = true;
    }
    return _result;
  },

  // 设置标题
  setNavTitle (str) {
    ability.setNavigationBarTitle(decodeURIComponent(str));
  },

  // 取得中间的渲染列表
  getListByKey (key) {
    let _list = [];
    if (key == 'hot') {
      _list = [{
        n: '空调',
        id: 'air',
        val: 0
      }, {
        n: '地暖',
        id: 'hot',
        val: 1
      }];
    } else {
      _list = [{
        n: '空调',
        id: 'air',
        val: 0
      }, {
        n: '新风',
        id: 'wind',
        val: 0
      }];
    }
    return _list;
  },

  // 底部按钮
  getBottomBtns () {
    return [{
      text: "恢复默认",
      className: "delBtn",
      tap: "resetInfo"
    }, {
      text: "保存",
      className: "save",
      tap: "saveInfo"
    }]
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  selectClick (e) {
    let {index} = e.currentTarget.dataset;
    let {n, id} = this.data.list[index];
    console.log("selectClick id = ", id);
    if (this.data.selectedIds[id]) {
      delete this.data.selectedIds[id];
    } else {
      this.data.selectedIds[id] = true;
    }
    this.setData({
      selectedIds: this.data.selectedIds
    })
  },

  resetInfo () {
    if (this.key == "hot") {
      // 默认数据 先放这里，后面整理
      this.data.selectedIds = {
        air: true,
        hot: true
      }
    } else if (this.key == "wind") {
      // 默认数据 先放这里，后面整理
      this.data.selectedIds = {
        air: true,
        wind: true
      }
    }
    // 返回
    this.saveInfo(true);
  },
  saveInfo (isReset) {
    let _selectedIdObj = this.data.selectedIds || {};
    let _keys = Object.keys(_selectedIdObj);
    if (_keys.length == 0) {
      return ability.showToast({
        title: "请至少选择一项"
      });
    }
    console.log("saveInfo _keys, selectedIds = ", _keys, _selectedIdObj);

    ability.showLoading({});

    let _result = this.setThermostatConfig(_selectedIdObj, isReset);
    if (!_result?.then) {
      ability.hideLoading();
      ability.showToast({
        title: '保存失败'
      })
      return;
    }
    _result.then(() => {
      ability.hideLoading();
      ability.showToast({
        title: '保存成功'
      })
      this.saveDevStatus({
        [this.key]: _keys.join(",")
      });
      // 返回
      setTimeout(this.back, 800);
    })
  },
  getCmdBySelect(selectedIds, isReset) {
    let {air, hot, wind} = selectedIds || {};
    let _key = 0;
    let _value = 0;
    if (this.key == "hot") {
      // 制热设置
      /* 
        0 空调
        1 地暖
        2 空调&地暖
        3 恢复默认
      */
      _key = 1;
      if (air && !hot) {
        _value = 0;
      } else if (hot && !air) {
        _value = 1;
      } else if (hot && air) {
        _value = 2;
      } else if (!hot && !air) {
        _value = 3;
      }
    } else if (this.key == "wind") {
      // 通风设置
      /*
        0 空调
        1 新风
        2 空调&新风
        3 恢复默认
      */
      _key = 4;
      if (air && !wind) {
        _value = 0;
      } else if (wind && !air) {
        _value = 1;
      } else if (wind && air) {
        _value = 2;
      } else if (!wind && !air) {
        _value = 3;
      }
    }
    // 如果是恢复默认，则是3
    !!isReset && (_value = 3);
    return [1, 1, _key, _value];
  },
  // 发送命令
  setThermostatConfig (selectedIdObj, isReset) {
    // 设备信息
    let _devInfo = app.globalData.devInfo || {};
    // 取得发送的命令
    let _cmdArray = this.getCmdBySelect(selectedIdObj, isReset);
    console.log("setThermostatConfig _cmdArray = ", _cmdArray);
    // 发送
    return app.globalData.blsdk.controlWithDevice(_devInfo, _cmdArray);
  },
  // 保存状态
  saveDevStatus (extendObj) {
    // 保存的设备状态
    let _allDevStatus = app.globalData.allStatus || {};
    let {d, a} = app.globalData.devInfo || {};
    let _keyStr = d + "_" + a;
    // 设备状态
    let _status = _allDevStatus[_keyStr] || {};
    console.log("saveDevStatus _status, extendObj = ", _status, extendObj);
    // 扩展空调地暖新风等参数
    _allDevStatus[_keyStr] = Object.assign({}, _status, extendObj);
    app.globalData.allStatus = _allDevStatus;
    // 保存到缓存中
    app.setBizStorage("devStatus", _allDevStatus);
  },
  back () {
    ability.navigateBack();
  }
})