// family/list/index.js
import ability from "../../utils/classDef/ability";
import bizUtil from "../../utils/bizUtil"

const app = getApp()
var familyBehavior = require("../../static/js/behavior/family")
Page({
  behaviors: [familyBehavior],
  /**
   * 页面的初始数据
   */
  data: {
    familys: [], // 家庭列表
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {

  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 初始化家庭列表信息
    this.initFamilys();
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
    console.log("family list onHide");
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
    // this.saveFamilys();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  initFamilys () {
    this.data.familys = app.globalData.familys || [];
    this.setData({
      familys: this.data.familys
    })
  },

  // 保存家庭信息
  saveFamilys () {
    let _list = this.data.familys || [];
    app.globalData.familys = _list;
    // 保存缓存
    app.setBizStorage("familys", _list);
  },

  delItemClick (e) {
    console.log("delItemClick");
    this.itemClick(e, "del")
  },
  editItemClick (e) {
    this.itemClick(e, "edit")
  },
  itemClick (e, type) {
    let {index} = e.currentTarget.dataset;
    if (type == 'del') {

      // 判断是否最后一个，如果是，则提示用户清空家庭还是
      let _familys = this.data.familys || [];
      if (_familys.length == 1) {
        return wx.showModal({
          title: "确定删除家庭吗？",
          content: "当前家庭为最后一个家庭，点击删除会清除当前家庭所有数据",
          showCancel: true,
          confirmText: "确定",
          cancelText: "取消",
          cancelColor: '#000000'
        }).then(result => {
          result?.confirm && (this.deleteFamily(index));
        })
      }
      // 删除家庭
      // this.deleteFamily(index)
      let _familyInfo = _familys[index];
      let _n = (_familyInfo.n || '');
      // 如果家庭名称太长，则省略成16个长度
      _n = (_n.length > 16) ? _n.substr(0, 16) : _n;
      // 删除家庭
      wx.showModal({
        title: "",
        content: "是否删除家庭【" + _n + '】?',
        showCancel: true,
        confirmText: "是",
        cancelText: "否",
        cancelColor: '#000000'
      }).then(result => {
        result?.confirm && (this.deleteFamily(index));
      })
    } else if (type == 'edit') {
      this.jumpEdit(index);
    }
  },
  setFamilys (familyArray) {
    // 家庭数据
    this.data.familys = familyArray || [];
    this.setData({
      familys: this.data.familys
    })
  },
  deleteFamily (index) {
    // 家庭列表
    let _familys = this.data.familys || [];
    let _familyInfo = _familys[index] || {};

    // 删除对应的家庭id下的所有设备信息
    delete app.globalData.allFamilysMap[_familyInfo.id];

    if (_familys.length == 1) {
      // 清空该家庭下的所有数据
      let _newFamilyInfo = this.getResetFamilys();
      let _allFamilys = [_newFamilyInfo];
      // 家庭数据
      this.setFamilys(_allFamilys);
      // 清空该家庭下的所有数据
      return this.clearAll(_allFamilys);
    }

    // 判断删除房间是否当前房间，如果是的话，则默认选择第一个
    let _isCurrent = _familyInfo.id == app.globalData.familyInfo?.id;
    // 删除家庭
    _familys.splice(index, 1);
    // 家庭数据
    this.setFamilys(_familys);
    // this.setData({
    //   familys: _familys
    // });
    // 非当前家庭，则忽略
    if (!_isCurrent) {
      return;
    }
    // 如果原先的家庭只有只读权限
    if (app.globalData.roleNum != 2) {
      // 删除之前的网关
      app.deletePrivateData();
    }

    // 默认选择第一个家庭
    let _currentFamily = _familys[0];
    // 更新家庭数据
    app.globalData.familyInfo = _currentFamily;
    // 更新角色
    app.globalData.roleNum = _currentFamily.roleNum || 0;
    // 更新控制密钥
    app.globalData.blsdk.updateBleCode(_currentFamily.bleCode);
    app.globalData.blsdk.updateSeq(_currentFamily.seq);
    
    // 更新
    if (app.globalData.roleNum == 2) {
      app.globalData.blsdk.updateCtrlTimestamp(_currentFamily.devTimestamp || 0);
    } else {
      app.globalData.blsdk.updateCtrlTimestamp(0);
    }
    
    // 设备信息更新
    app.globalData.devs = app.globalData.allFamilysMap[_currentFamily.id];

    // 查找当前家庭有无网关设备
    let _newGateway = bizUtil.getGatewayInfo(app.globalData.devs);
    app.globalData.blsdk.updateGatewayInfo(_newGateway);

    // 保存数据
    app.saveWhenHide();
  },

  jumpEdit (index) {
    let _item = this.data.familys[index];
    ability.navigateTo('/family/name/index?index=' + index + "&name=" + encodeURIComponent(_item.n || ''))
  },
  jumpAdd () {
    ability.navigateTo('/family/name/index?index=-1')
  },
  

  familyClick (e) {
    let {index} = e.currentTarget.dataset;
    let {id} = this.data.familys[index];
    console.log("itemClick id = ", id);
    ability.navigateTo("../detail/index?index=" + index + "&familyid=" + (id || 0))
  }
})