// family/list/index.js
import ability from "../../utils/classDef/ability";
import bizUtil from "../../utils/bizUtil"
const app = getApp()
Page({

  /**
   * 页面的初始数据
   */
  data: {
    floors: [], // 家庭列表

    showActionsheet: false, // 默认不显示楼层
    allFloors: [], // 所有的楼层

    familyId: -1, // 房间id
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 如果有家庭id传过来，则说明是通过家庭管理进入的楼层管理
    let _familyId = options.familyId;
    if (app.globalData.familyInfo?.id == _familyId) {
      _familyId = undefined;
    }
    this.data.familyId = _familyId;
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 数据获取
    this.reloadData();
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
    console.log("floor list onHide");
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
    // this.saveFamilys();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  reloadData () {
    // 所有楼层信息获取
    this.initAllFloors();
    // 初始化家庭列表信息
    this.initFloors();
  },

  // 根据家庭id取得数据
  getAllDevsByFamilyId () {
    let _familyId = this.data.familyId;
    let _devs = undefined;
    if (_familyId >= 0) {
      _devs = app.globalData.allFamilysMap[_familyId];
    }
    return _devs || app.globalData.devs;
  },

  initAllFloors () {
    
    // 取得家庭数据
    let _selFamilyDevs = this.getAllDevsByFamilyId();

    // 取得已经存在的楼层
    let _exists = bizUtil.getFloorNames(_selFamilyDevs) || [];
    let _ids = _exists.map(item=>parseInt(item.id / 33) || 0);
    // 所有的楼层数据
    let _floor = this.getAllFloors(_ids);
    console.log("initAllFloors _ids = ", _ids, _exists, _floor);
    this.data.allFloors = _floor;
    this.setData({
      allFloors: this.data.allFloors
    })
  },

  // 取得所有楼层
  getAllFloors (ids) {
    let _result = [];
    let _ids = ids || [];
    for (let i = 0; i < 6; i++) {

      // 如果相同的id，则抛出
      if (_ids.indexOf(i) >= 0) {
        continue;
      }

      _result.push({
        text: (i + 1) + "楼",
        value: i
      })
    }
    return _result;
  },

  initFloors () {

    // 取得家庭数据
    let _selFamilyDevs = this.getAllDevsByFamilyId();
    // 楼层列表
    let _floors = bizUtil.getFloorNames(_selFamilyDevs);
    // 取得各个楼层对应的设备个数
    this.data.floors = _floors.map(floor => {
      floor.devCount = bizUtil.getDevsCountByFloor(_selFamilyDevs, bizUtil.getFloorByRoomId(floor.id));
      return floor;
    });
    console.log("initFloors this.data.floors = ", this.data.floors);
    this.setData({
      floors: this.data.floors
    })
  },

  // 保存家庭信息
  saveFamilys () {
    // let _list = this.data.floors || [];
    // app.globalData.floors = _list;
    // // 保存缓存
    // app.setBizStorage("floors", _list);
  },

  delItemClick (e) {
    this.itemClick(e, "del");
  },
  editItemClick (e) {
    this.itemClick(e, "edit")
  },
  itemClick (e, type) {
    let {index} = e.currentTarget.dataset;
    if (type == 'del') {

      // 判断是否最后一个，如果是，则提示用户清空家庭还是
      let _floors = this.data.floors || [];
      // 待删除的家庭信息
      let _floorInfo = _floors[index];

      // 判断楼层下的设备是否为空
      if (_floorInfo?.devCount > 0) {
        return ability.showToast({title: "请先移除楼层下的设备再删除"});
      }

      
      if (_floors.length == 1) {
        return wx.showModal({
          title: "确定删除楼层吗？",
          content: "当前楼层为最后一个楼层，点击确定会清除当前楼层所有数据",
          showCancel: true,
          confirmText: "确定",
          cancelText: "取消",
          cancelColor: '#000000'
        }).then(result => {
          result?.confirm && (this.deleteFloor(index));
        })
      }
      let _n = (_floorInfo.n || '');
      // 如果家庭名称太长，则省略成16个长度
      _n = (_n.length > 16) ? _n.substr(0, 16) : _n;
      // 删除家庭
      wx.showModal({
        title: "确定删除【" + _n + "】吗？",
        content: "点击确定会清除当前楼层所有数据",
        showCancel: true,
        confirmText: "确定",
        cancelText: "取消",
        cancelColor: '#000000'
      }).then(result => {
        result?.confirm && (this.deleteFloor(index));
      })
    } else if (type == 'edit') {
      this.jumpEdit(index);
    }
  },
  deleteFloor (index) {
    // 家庭列表
    let _floors = this.data.floors || [];
    let _floor = _floors[index];

    // // 最后一个楼层
    // if (_floors.length == 1) {
    //   // 清空该家庭下的所有数据
    //   return this.clearAll();
    // }

    // 取得家庭数据
    let _selFamilyDevs = this.getAllDevsByFamilyId();

    // 删除楼层
    bizUtil.deleteFloor(_selFamilyDevs, _floor.id);
    console.log("deleteFloor app.globalData.devs = ", _selFamilyDevs, this.data.familyId);
    // 刷新楼层
    this.reloadData();

    // 如果当前家庭和传入的家庭不一致，则不需要更新楼层信息
    if (this.data.familyId == undefined) {

      // 如果当前楼层被删除
      if (app.globalData.familyInfo?.floor == bizUtil.getFloorByRoomId(_floor.id)) {
        // 如果是当前楼层则更新楼层信息
        app.globalData.familyInfo.floor = bizUtil.getCurFloor(_selFamilyDevs);
      }
      // 更新私有数据
      app.uploadPrivateData();
    }
  },

  jumpEdit (index) {
    // 家庭id
    let _familyId = this.data.familyId;
    // 当前选择的楼层
    let _item = this.data.floors[index];
    ability.navigateTo('/floor/name/index?roomId=' + _item.id + "&name=" + encodeURIComponent(_item.n || '') + ((_familyId != undefined) ? ("&familyId=" + _familyId) : ""));
  },
  jumpAdd () {

    // 如果没有可创建的楼层，则提示
    if (this.data.allFloors.length == 0) {
      return ability.showToast({
        title: "楼层已满"
      })
    }

    // 创建楼层
    this.manageActionSheet(true);
    // ability.navigateTo('/floor/name/index?index=-1')
  },

  manageActionSheet (val) {
    this.data.showActionsheet = val || false;
    this.setData({
      showActionsheet: this.data.showActionsheet
    })
  },


  // checkName (name, roomId) {
  //   let _names = bizUtil.getFloorNames(app.globalData.devs);
  //   let _filter = _names.filter(item=>(item.n==name&&roomId!=item.id));
  //   return _filter.length;
  // },

  actionBtnClick (e) {

    // 取得家庭数据
    let _selFamilyDevs = this.getAllDevsByFamilyId();

    console.log("actionBtnClick e = ", e);
    let {value, index} = e.detail || {};
    // 默认的名称
    let text = this.data.allFloors[index]?.text;
    // 已经存在的楼层名称
    let _names = this.data.floors.map(item=>item.n);
    // 取得名称
    let _newName = bizUtil.getN(_names, text);
    // 加入楼层数据
    bizUtil.addFloor(_selFamilyDevs, value, _newName);
    
    // 刷新数据
    this.reloadData();
    // 隐藏actionsheet
    this.manageActionSheet(false);
    // 保存数据
    app.saveAllDevsMap();

    // 如果当前家庭和传入的家庭不一致，则不需要更新楼层信息
    if (this.data.familyId == undefined) {

      // 如果没有默认楼层对应的房间id,则自动设置楼层id
      let _defaultFloor = app.globalData.familyInfo?.floor;
      if (!_selFamilyDevs[(_defaultFloor + 1) * 32]) {
        // 如果是当前楼层则更新楼层信息
        app.globalData.familyInfo.floor = bizUtil.getCurFloor(_selFamilyDevs);
      }
      // 更新私有数据
      app.uploadPrivateData();
    }
  },

  // clearAll () {
  //   // 保存家庭相关数据
  //   app.saveWhenHide();
  //   // 提示
  //   ability.showToast({
  //     title: "清理完成",
  //     duration: 1000,
  //     icon: "success",
  //     mask: true,
  //     complete: () => {
  //       // 刷新数据
  //       ability.showToast({
  //         title: "设备已清空"
  //       });
  //     }
  //   })
  // },
  getDevList (array) {
    if (!array) {
      return {};
    }
    let _result = {};
    for (let item of array) {
      item.devs = [];
      item.scenes = bizUtil.getDefaultScenes(item.id); // 默认情景
      _result[item.id] = item;
    }
    return _result;
  },
})