// groups/manage/index.js
import bizUtil from '../../utils/bizUtil'
import ability from "../../utils/classDef/ability"
import groupBehavior from "../../static/js/behavior/group"

const app = getApp();
Page({

  behaviors: [groupBehavior],

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "房间详情"
      }
    },

    // 编辑模式
    isEdit: false,
    // 群组内设备
    groupDevList: [],
    /*configObj: {
      height: 135 / app.globalData.customPageSize.multip,
      seperator: 10,
      disUpEvent: true,
      unable: true, // 是否可以移动
    },*/
    // roomId: 0,

    bleUnavailable: false,

    roleNum: 0, // 用户权限

    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined, // 家庭下所有设备信息

    roomJson: {}, // 房间key
    isFloor: false, // 是否多楼层

    // 设备品类
    type: app.globalData.groupInfo?.t,

    // 页面类型
    isDelete: false
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    let {familyId} = options || {};
    // 初始化家庭和家庭下的设备数据
    this.initAllDataByFamily(familyId);

    console.log("onLoad this.data.familyInfo = ", this.data.familyInfo, app.globalData.groupInfo);
    // 取得当前家庭的数据
    let {roleNum, floor} = this.data.familyInfo || {};

    // 所有房间数据
    let {roomJson} = bizUtil.getRoomsByDevs(this.data.allDevs);
    this.data.roomJson = roomJson;
    this.data.isFloor = floor >= 0;

    // 权限
    this.data.roleNum = roleNum == undefined ? app.globalData.roleNum : roleNum;

    console.log("onLoad this.data.roleNum, this.data.roomJson = ", this.data.roleNum, this.data.roomJson);

    // 品类
    this.data.type = app.globalData.groupInfo?.t;

    // 传递的房间数据
    this.setData({
      roleNum: this.data.roleNum,
      roomJson: this.data.roomJson,
      type: this.data.type
    });
  },
  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },
  initTitle () {
    let _title = app.globalData.groupInfo?.n || "群组设备";
    ability.setNavigationBarTitle(_title);
  },
  setProperValue (key, val) {
    if (!key) {
      return;
    }
    this.data[key] = val || '';
    this.setData({
      [key]: val || ""
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 更新标题
    this.initTitle();
    // 更新设备信息
    this.initGroupDevs();
  },

  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    })
  },
  hideLoading () {
    ability.hideLoading();
  },
  // 取得当前群组中的主设备
  getGroupMainDByDevArray (devArray, gid) {
    let _mainJson = this.getMainDeviceJsonInArray(devArray);
    return _mainJson[gid] || {};
  },
  // 取得设备中当前群组的主设备
  getMainDeviceJsonInArray (devArray) {
    let _result = {}; // groupId: devInfo
    if (!devArray || devArray.length == 0) {
      return _result;
    }
    // 遍历
    for (let devInfo of devArray) {
      let {e, d} = devInfo || {};
      if (!e) {
        continue;
      }
      let {group} = typeof e == "string" ? JSON.parse(e) : e;
      if (group?.main == undefined) {
        continue;
      }
      _result[group.main] = devInfo;
      // _result.push(group.main);
    }
    return _result;
  },

  // 初始化房间信息
  initGroupDevs () {
    console.log("initGroupDevs this.data.allDevs = ", this.data.allDevs);
    // 取得房间下所有的d对应的详细信息
    let _didMaps = bizUtil.getDidMapInJson(this.data.allDevs);
    // 群组设备列表
    let {dids: _groupDevs, gid} = app.globalData.groupInfo || {};
    // let _groupDevs = app.globalData.groupInfo?.dids || [];
    // 群组设备列表详细
    let _groupDevList = _groupDevs.map(did => _didMaps[did]);
    console.log("initGroupDevs _groupDevList = ", _groupDevList);
    // 房间内的数据按照名称排序
    this.data.groupDevList = bizUtil.sortArrayByChar(_groupDevList, "n");
    console.log("this.data.groupDevList = ", this.data.groupDevList);
    // 取得所有的主设备
    let _mainDevInfo = this.getGroupMainDByDevArray(_groupDevList, gid);
    console.log("_mainDevInfo = ", _mainDevInfo);
    
    this.setData({
      groupDevList: this.data.groupDevList,
      mainDevInfo: _mainDevInfo
    })
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
    console.log("roomDetail onHide");
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
    // 如果是固定群组且dids为空，则多返回一次
    let {dids} = app.globalData.groupInfo || {};
    if (dids?.length > 0) {
      return;
    }
    ability.navigateBack();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {
  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {
  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {
  },

  // 长按删除群组中某个设备
  longItemClick (e) {
    // 如果是分享权限，则无法长按删除
    if (this.data.roleNum == 2) {
      return;
    }
    let {index} = e.currentTarget.dataset;
    let {n, d} = this.data.groupDevList[index] || {};
    // 提示
    wx.showModal({
      title: "",
      content: "是否确认删除" + (n || "") + "？",
      showCancel: true,
      confirmText: "是",
      cancelText: "否",
      cancelColor: '#000000'
    }).then(result => {
      if (!(result?.confirm)) {
        return;
      }
      // 删除方法
      return this.delDevInGroupByIndex(index);
    })
  },

  // 从群组中删除对应的设备id
  delDevInGroupByIndex (index) {
    let _list = this.data.groupDevList || [];
    let _dataLen = _list.length || 0;
    if (index >= _dataLen || index < 0) {
      console.error("delDevInGroupByIndex index, _dataLen = ", index, _dataLen);
      return;
    }
    // 设备信息
    let _devInfo = _list[index];
    // 剩下的数组
    // let _resultArray = _list.filter((item, idx)=> index != idx);
    // // 判断是否全部删除
    // let _isDelAll = _resultArray.length == 0;
    // 调用接口 _resultArray
    this.delGroupDevs(app.globalData.groupInfo, [_devInfo?.d], () => {
      // 重新加载
      this.showDelSuccessToast();
      // 保存数据
      app.saveWhenHide();
      // 返回上一页
    }); // , _isDelAll
  },
  // 删除成功的提示
  showDelSuccessToast () {
    // 提示删除结果
    ability.showToast({
      title: "删除成功",
      icon: "success",
      mask: true,
      complete: () => {
        // 返回首页，重新加载
        setTimeout(() => {
          // 重新加载页面
          this.initGroupDevs();
        }, 1200);
      }
    })  
  },

  // 跳转详情
  jumpGroupDetail () {
    ability.navigateTo("/device/light/devProperty/index?remote=0")
  },

  getGroupRoomId () {
    return app.globalData.groupInfo?.rid || 0;
  },

  // 编辑按钮点击
  manageDevs () {
    // 群组房间
    let _roomId = this.getGroupRoomId();
    // // 房间名称
    // let _rn = this.data.allDevs[_roomId]?.n || '';
    // console.log("manageDevs _rn = ", _rn);
    // 来源页面
    let _origin = "groupdel";

    // 跳转
    this.jumpGroupManage({
      from: _origin,
      title: "编辑群组设备",
      rid: _roomId,
      familyId: this.data.familyId
    })
    // ability.navigateTo("/pages/batch/index?title=" + encodeURIComponent(_rn + "编辑") + "&from=" + _origin + "&rid=" + _roomId + "&familyId=" + this.data.familyId);
  },
  addGroupDevice () {
    // 来源地址
    let _from = "groupadd";
    // 标题 品类
    let {n, t} = app.globalData.groupInfo || {};
    // 跳转
    this.jumpGroupManage({
      from: _from,
      title: n,
      type: t
    })
    // ability.navigateTo("/pages/batch/index?from=" + _from + "&title=" + decodeURIComponent(n) + "&type=" + t);
  },
  jsonToUrlParamsStr (json) {
    let _result = [];
    if (!json) {
      return "";
    }
    let _keyArray = Object.keys(json);
    let _valueArray = Object.values(json);
    for (let index in _keyArray) {
      let _key = _keyArray[index];
      let _value = _valueArray[index];
      if (_key == undefined || _value == undefined) {
        continue;
      }
      _result.push(_key + "=" + encodeURIComponent(_value));
    }
    return "?" + _result.join("&");
  },
  jumpGroupManage (params) {
    ability.navigateTo("/pages/batch/index" + this.jsonToUrlParamsStr(params));
  },

  openLightDev (e) {
    let _devInfo = this.getDevInfoByE(e);
    // 之前的亮度值
    let _status = app.globalData.allStatus[_devInfo?.d + "_" + _devInfo?.a] || {};
    this.actDev({
      pwr: 1,
      brightness: _status.brightness || 127
    }, _devInfo)
  },
  closeLightDev (e) {
    let _devInfo = this.getDevInfoByE(e);
    this.actDev({
      pwr: 0,
      brightness: 0
    }, _devInfo)
  },
  getDevInfoByE (e) {
    let {index} = e.currentTarget.dataset;
    return this.data.groupDevList[index];
  },
  actDev (cmd, deviceInfo) {
    // 控制命令
    app.globalData.blsdk.controlWithDevice(deviceInfo, cmd);
  },

  closeCurtainDev (e) {
    console.log("closeDev");
    let _devInfo = this.getDevInfoByE(e);
    this.actDev("down", _devInfo);
  },
  pauseCurtainDev (e) {
    console.log("pauseDev");
    let _devInfo = this.getDevInfoByE(e);
    this.actDev("pause", _devInfo);
  },
  openCurtainDev (e) {
    console.log("openCurtainDev");
    let _devInfo = this.getDevInfoByE(e);
    this.actDev("up", _devInfo);
  },
})