const ability = require("../../utils/classDef/ability");
const app = getApp();
const updateManager = wx.getUpdateManager();
// minePages/about/index.js
Page({

  /**
   * 页面的初始数据
   */
  data: {
    appName: "", // 小程序名称
    version: "", // 小程序版本号
    list: [], // 底部列表
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title} = options || {};
    // 设置名称
    this.setNavTitle(title);
    // 设置app信息
    this.initAppInfo();
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(decodeURIComponent(title || ""))
  },
  getAppInfo () {
    return app.globalData.companyInfo || {};
  },
  initAppInfo () {
    let {version, wxName} = this.getAppInfo();
    this.setData({
      appName: wxName || "",
      version: version || ""
    })
  },  

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 检查更新
    this.checkForUpdate();
  },
  
  // 是否有更新数据
  setListByUpdate (hasUpdate) {
    let _hasUpdate = hasUpdate || false;
    let _list = [{
      title: "版本更新",
      tip: _hasUpdate ? "有更新": "已是最新版本",
      right: _hasUpdate
    }]; // 底部列表
    this.setData({
      list: _list
    })
  },

  // 列表点击
  itemClick (e) {
    let {index} = e.currentTarget.dataset;
    let {right} = this.data.list[index] || {};
    if (!right) {
      return;
    }
    // 更新
    this.updateWechatProgram();
  },

  // 检查更新
  checkForUpdate () {
    updateManager.onCheckForUpdate((res) => {
      // 请求完新版本信息的回调
      console.log("checkForUpdate res = ", res);
      let {hasUpdate} = res || {};
      if (!hasUpdate) {
        // 列表显示文字更新
        this.setListByUpdate(false);
        return;
      }
      // 列表显示文字更新
      this.setListByUpdate(true);
      // 更新
      this.updateWechatProgram();
    })
  },
  // 更新小程序
  updateWechatProgram () {
    updateManager.onUpdateReady(function () {
      wx.showModal({
        title: '更新提示',
        content: '新版本已经准备好，是否重启应用？',
        success: function (res) {
          if (res.confirm) {
            // 新的版本已经下载好，调用 applyUpdate 应用新版本并重启
            updateManager.applyUpdate()
          }
        }
      })
    });
    updateManager.onUpdateFailed(function () {
      // 新版本下载失败
      ability.showToast({
        title: "更新失败",
        icon: "error"
      })
    })
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  }
})