// minePages/backup/index.js
import bizUtil from '../../utils/bizUtil'
import ability from '../../utils/classDef/ability'
const app = getApp();
const accountBehavior = require("../js/behavior/account")
// const CustomPage = require("../../static/js/baseClass/page")

Page({
  behaviors: [accountBehavior],

  /**
   * 页面的初始数据
   */
  data: {
    phone: "",
    code: '',

    // 传过来的两个参数
    index: 0, 
    tapKey: "",

    // 验证码倒计时定时器
    timer: null,
    count: 0,
    // 登陆类型
    loginType: 1, // 1 默认手机 0 邮箱登陆 2：密码登陆
    needShow: false, // 是否需要显示密码
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    console.log("onLoad options = ", options);
    let {tapKey, index} = options || {};
    this.data.tapKey = tapKey || "";
    this.data.index = index || "";
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow () {
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    // 清理定时器
    bizUtil.clearTimer(this.data.timer, "interval");
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {
  },

  phoneChange (e) {
    this.setProperty("phone", e.detail.value);
  },
  codeChange (e) {
    this.setProperty("code", e.detail.value);
  },
  passwordChange (e) {
    let _val = e.detail.value;
    if (_val > 18) {
      _val = _val.substring(0, 18);
    }
    this.setProperty("code", _val);
  },
  setProperty (key, val) {
    if (!key || val == undefined) {
      return;
    }
    this.data[key] = val;
    this.setData({
      [key]: val
    })
  },

  showToast (str, icon) {
    return ability.showToast({
      title: str || '',
      icon: icon || ''
    })
  },
  // 获取验证码
  sendCode () {
    // 判断是否在发送中
    if (this.data.count > 0) {
      return;
    }
    // 手机号码
    let _phone = this.data.phone;
    if (_phone.length == 0) {
      return this.showToast("请输入手机号码")
    }
    let _type = this.checkLoginType(_phone);
    if (!_type) {
      return this.showToast("格式不正确")
    }
    app.globalData.blsdk.sendAccountCode({
      type: _type,
      apiKey: "fastlogincode",
      [_type]: _phone
    }).then(res => {
      let {error, msg} = res || {};
      if (error == 0) {
        this.showToast("发送验证码成功", "success");
        // 开始定时器计数
        this.startTimer();
      } else {
        console.log("sendCode res = ", res);
        this.showToast(this.dealErrorMsg(error) || msg || "发送验证码失败", "none")
      }
    }).catch(e => {
      console.log("sendCode e = ", e);
      this.showToast("发送验证码失败", "none")
    })
  },
  // 开始定时器
  startTimer () {
    // if (this.data.timer) {
    //   return;
    // }
    // 设置数据
    this.setProperValueByKey("count", 60);
    // 定时器
    this.data.timer = setInterval(() => {
      let _count = this.data.count - 1;
      if (!this.data.timer) {
        return;
      }
      if (_count <= 0) {
        _count = 0;
        // 清除定时器
        bizUtil.clearTimer(this.data.timer);
      }
      // 更新数据
      this.setProperValueByKey("count", _count)
    }, 1000);
  },
  fastLogin () {
    let _loginType = this.data.loginType;
    if (_loginType == 2) {
      // 账号密码登陆
      this.passwordLogin();
    } else {
      // 验证码登陆
      this.fastCodeLogin();
    }
  },
  // 账号密码登陆
  passwordLogin () {
    let _phoneEmail = this.data.phone;
    let _typeStr = this.checkLoginType(_phoneEmail);
    if (!_typeStr) {
      return this.showToast("格式不正确", "none");
    }
    // 密码长度
    let _code = this.data.code;
    let _codeLength = _code?.length || 0;
    if (_codeLength < 6 || _codeLength > 18) {
      return this.showToast("密码长度为6-18", "none");
    } else if (!this.checkPassword(_code)) {
      return this.showToast("密码包含数字和字母", "none");
    }

    // 判断是邮箱还是手机登陆
    app.globalData.blsdk.passwordLogin({
      password: _code, 
      type: _typeStr,
      [_typeStr]: _phoneEmail,
      apiKey: "login"
    }).then(res => {
      console.log("passwordLogin res = ", res);
      let {error, status, msg} = res || {};
      if (error == 0 && status == 0) {
        this.showToast("登陆成功", "success");
        // 保存用户信息
        this.saveUserInfo(res);
        // 处理备份还是还原
        this.dealAfterLogin();
      } else {
        this.showToast(this.dealErrorMsg(error) || msg || "登陆失败", "none")
      }
    }).catch(e => {
      console.log("passwordLogin e = ", e);
      // 显示loading
      ability.hideLoading();
      this.showToast(e?.data?.msg || "登陆失败", "none")
    })
  },
  // 快捷登陆
  fastCodeLogin () {
    let _phone = this.data.phone;
    let _code = this.data.code;
    if (_phone.length == 0) {
      return this.showToast("请输入手机号码")
    } else if (_code.length == 0) {
      return this.showToast("请输入验证码")
    }
    // 判断登陆类型
    let _type = this.checkLoginType(_phone);
    if (!_type) {
      return this.showToast("邮箱/手机格式不正确");
    }
    // 显示loading
    ability.showLoading({});
    // 发送数据
    app.globalData.blsdk.fastLogin(_phone, _code, {
      type: _type,
      apiKey: "fastLogin"
    }).then(res => {
      // 显示loading
      ability.hideLoading();
      let {error, msg} = res || {};
      if (error == 0) {
        this.showToast("登陆成功", "success");
        // 保存用户信息
        this.saveUserInfo(res);
        // 处理备份还是还原
        this.dealAfterLogin();
      } else {
        this.showToast(this.dealErrorMsg(error) || msg || "登陆失败", "none")
      }
    }).catch(e => {
      console.log("fastLogin e = ", e);
      // 显示loading
      ability.hideLoading();
      this.showToast("登陆失败", "error")
    })
  },
  dealAfterLogin () {
    let _tapKey = this.data.tapKey;
    // 登陆之后的处理
    if (!_tapKey || !this[_tapKey]) {
      // 跳转到登陆成功的提示
      ability.redirectTo("../toastPage/index?imgKey=success&type=1")
      return;
      // return this.back();
    }
    // 调用对应的方法
    this[_tapKey](undefined, true);
  },
  back () {
    ability.navigateBack();
  },
  // 设置并更新属性
  setProperValueByKey (key, val, noUpdate) {
    if (this.data == undefined) {
      return;
    }
    this.data[key] = val;

    // 不需要更新
    if (noUpdate) {
      return;
    }

    this.setData({
      [key]: val
    })
  },
  // 注册
  register () {
    ability.navigateTo("./register/index?type=regist&title=" + encodeURIComponent("账号注册"));
  },
  // 切换邮箱登陆、手机号登陆
  changeLoginType () {
    let _loginType = this.data.loginType;
    if (_loginType < 2) {
      _loginType = 2;
    } else {
      _loginType = 1;
    }
    this.setData({
      loginType: _loginType,
      code: ""
    })
  },
  // 忘记密码
  forget () {
    let _phoneEmail = this.data.phone || "";
    ability.navigateTo("./register/index?type=forget&title=" + encodeURIComponent("忘记密码") + "&phoneEmail=" + encodeURIComponent(_phoneEmail));
  },
  // 是否显示可见的密码
  setNeedShow () {
    let _needShow = !this.data.needShow;
    this.setData({
      needShow: _needShow
    })
  },

  
  loginByWX (e) {
    console.log("loginByWX e.detail = ", e.detail)
    let {iv, encryptedData} = e?.detail || {};

    ability.login().then((res) => {
      let {code} = res || {};
      console.log("loginByWX code = ", code)

      let _appid = "wxb6ecbcea30e9283b";
      app.globalData.blsdk.loginByWX({
        code: code,
        topsign: JSON.stringify({
          "encryptedData": encryptedData,
          "iv": iv
        }),
        thirdtype: "wechatmini_" + _appid,
      }).then(res => {
        console.log("loginByWX res = ", res);
      }).catch(e => {
        console.error("loginByWX e = ", e);
      })

    }).catch(e => {
      console.error("loginByWX e = ", e);
      ability.showToast({
        title: "微信登陆失败"
      })
    })
  }
})