// import drawQrcode from 'weapp-qrcode'
import QR from '../js/qrcode/weapp-qrcode'

import {Utils} from "../../utils/util"
import ability from '../../utils/classDef/ability'
import bizUtil from '../../utils/bizUtil'
import sdkUtil from '../../utils/classDef/util'
import shareBehavior from "../js/behavior/share"

const ROLES = [{
  text: '管理权限',
  value: 1,
}, {
  text: '控制权限',
  value: 2
}];

const app = getApp();

Page({
  behaviors: [shareBehavior],
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "邀请成员"
      }
    },
    nickName: app.globalData.familyInfo.n,
    bleUnavailable: false,

    qrcodeURL: [],
    tip: '',

    items: [],

    // 显示分享权限
    showRoleNum: false,
    roleNum: ROLES[1].value, // 默认分享的权限为2
    roles: ROLES,

    // 显示设备可用时长
    // showAvalible: false,
    // time: '12:01',
    avalibles: [],

    devData: [], // 缓存数据，防止重复生成

    currentSteps: 0, // 同步智慧面板的当前步骤，默认未开始
    indicatorDots: true, // 可以滑动

    isOuttime: false, // 是否过期
    timestamp: 0,
    timer: null,

    useTime: 15 // 单位分钟 有效时长
  },
  onLoad (option) {
    let {title} = option || {};
    // 设置标题
    this.setNavTitle(decodeURIComponent(title || ""));

    // 昵称
    this.data.nickName = app.globalData.familyInfo.n;
    this.setData({
      nickName: this.data.nickName
    })

    // 获取数据 测试分享86屏数据
    let _is86Share = false; // 默认是false
    let _devData = this.getDevData(_is86Share, app.globalData.devs, {
      needDefaultScene: true
    });
    console.log("onLoad _devData = ", _devData);
    this.data.devData = _devData;
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(title);
  },
  onReady () {
    // 开始广播发现86屏
    this.probeDevice();
  },
  onUnload () {
    this.isClose = true;
    // 清理定时器
    bizUtil.clearTimer(this.data.timer);
    // 清空数据
    if (!app.globalData.socket) {
      return;
    }
    ability.closeSocket(app.globalData.socket);
    app.globalData.socket = null;
  },

  onShow () {
    this.isClose = false;

    // 更新二维码有效时长
    this.data.useTime = app.globalData.tmpData?.avalible || this.data.useTime;

    // 更新当前时间戳
    this.updateTime();
    // 生成二维码
    this.createQrcode();
  },
  createQrcode () {
    // 这里需要和实际权限2为分享者对应上
    let _roleNum = this.data.roleNum;
    let _shareTime = 0;
    if (_roleNum == 2) {
      let _delay = app.globalData.tmpData.qrTime || 0;
      console.log("createQrcode _shareTime = ", _shareTime);
      if (_delay > 0) {
        _shareTime = _delay + parseInt(Date.now() / 1000);
      }
    }
    console.log("createQrcode _shareTime, _roleNum = ", _shareTime, _roleNum, app.globalData?.tmpData?.qrTime);
    // 获取小程序数据的字符串
    let _base64Data = this.getWxData(undefined, {
      devs: app.globalData.devs,
      family: app.globalData.familyInfo,
      needDefaultScene: true,
      roleNum: _roleNum,
      shareTime: _shareTime // 设备可用时长
    });
    // 生成云端二维码
    return this.createCloudQrcode(_base64Data, _roleNum, app.globalData.tmpData?.avalible);

    // 生成本地二维码
    // this.createQrcodeWithoutLogo(_base64Data);
  },
  setTimerCount () {
    // 设置未过期
    this.setProperValue("isOuttime", false);
    // 创建一个定时器
    let _sec = (this.data.useTime || 15) * 60 ; // 单位是分钟 转成定时器的秒
    if (_sec <= 0) {
      // 永久用
      return;
    }

    // 保存创建二维码的时间戳
    this.data.timestamp = this.getTimeStamp();

    // 初始化定时器
    bizUtil.clearTimer(this.data.timer);
    this.data.timer = setTimeout(() => {
      // _sec毫秒过后，二维码会过期
      let _now = this.getTimeStamp();
      // 是否过期
      let _isOuttime = (_now - this.data.timestamp) >= _sec;
      this.setProperValue("isOuttime", _isOuttime);
    }, _sec * 1000);
  },
  setProperValue (key, val) {
    if (this.data == undefined) {
      return;
    }
    this.data[key] = val;
    this.setData({
      [key]: val
    })
  },
  getTimeStamp () {
    return parseInt(Date.now() / 1000);
  },
  createCloudQrcode (base64Str, roleNum, avalibleTime) {
    // 选择的集群
    let _groupIndex = app.globalData.familyInfo.groupIndex || 0;
    let _allGroups = app.globalData.blsdk.getDefaultGroup();
    let _cloud = _allGroups[_groupIndex] || _allGroups[0]; // 默认第0个国内集群

    // 此处增加永久二维码存储，如果是管理员权限或者控制者但选择了有效时长为永久的时候存到永久数据中
    console.log("createCloudQrcode roleNum, avalibleTime = ", roleNum, avalibleTime);
    if (roleNum == 1 || avalibleTime <= 0) {
      app.uploadPrivateData((sid, skey, res) => {
        console.log("createCloudQrcode sid, skey = ", sid, skey, res);
        // 生成二维码
        this.uploadResult(res, _cloud, base64Str);
      }, false, false, false, base64Str);
      return;
    }

    // 先上传云端数据 base64 临时数据
    app.globalData.blsdk.createOnlineCode({
      cloud: _cloud,
      validTime: Number(app.globalData.tmpData?.qrTime) || (15 * 60), // 有效时长，最大值2592000s 30天 单位s
      data: base64Str
    }).then(res => {
      console.log("res = ", res);
      this.uploadResult(res, _cloud, base64Str);
    }).catch(e => {
      // 设置15分钟的有效期倒计时 云端二维码和本地二维码有个时间差
      this.uploadResult({}, _cloud, base64Str);
    });
  },
  uploadResult (res, groupStr, base64Str) {
    console.log("uploadResult res = ", res);
    // 设置15分钟的有效期倒计时 云端二维码和本地二维码有个时间差
    this.setTimerCount();
    if (res?.status == 0) {
      // 生成在线二维码
      this.createOnlineQrcode(res?.data, groupStr);
    } else {
      // 生成本地二维码
      this.createQrcodeWithoutLogo(base64Str);
    }
  },
  // 生成二维码
  createOnlineQrcode (dataArray, groupHost) {
    if (!dataArray || dataArray.length == 0) {
      return;
    }
    let _resInfo = dataArray[0] || {};
    // 域名短地址，用于小程序通过微信扫一扫进入
    let _host = app.globalData.companyInfo.host;
    // 生成二维码，此处默认只生成一个二维码
    this.drawOnlineImg(_host, "?cloud=app-service-" + groupHost + "&did=" + _resInfo.did + "&token=" + _resInfo.token);
  },
  // 二维码
  drawOnlineImg (host, paramsStr) {
    if (!paramsStr) {
      return;
    }
    let _lastStr = host + paramsStr;
    console.log("_lastStr = ", _lastStr);
    let _result = [QR.drawImg(_lastStr, {
      typeNumber: 16,          // 密度
      errorCorrectLevel: 'H', // 纠错等级
      size: 243, // 白色边框
      // typeNumber: 20,          // 密度
      // errorCorrectLevel: 'H', // 纠错等级
      // size: (20 * 4 + 17) * 6,              // 白色边框
    })];
    let _updateObj = {
      qrcodeURL: _result,
      indicatorDots: false // 云端二维码只有一个，所以不需要滑动
    };
    this.setData(_updateObj)
  },

  updateTime () {
    let {qrTime: _time, avalible} = app.globalData.tmpData || {};
    let _roleNum = this.data.roleNum;
    if (_time == undefined || _time == 0) {
      // 永久
      this.updateItems(_roleNum, "永久", this.getStrByTimestamp(avalible));
    } else {
      // 其他时间
      this.updateItems(_roleNum, this.getStrByTimestamp(_time), this.getStrByTimestamp(avalible));
    }
  },
  getStrByTimestamp (timestamp) {
    if (timestamp == undefined || timestamp == 0) {
      return "永久";
    }
    if (timestamp < 60) {
      return "0"
    }
    let _day = parseInt(timestamp / (24 * 3600));
    let _h = parseInt((timestamp - _day * 24 * 3600) / 3600);
    let _m = parseInt((timestamp - _day * 24 * 3600 - _h * 3600) / 60);
    let _str = "";
    (_day) && (_str += _day + '天');
    (_h) && (_str += _h + '小时');
    (_m) && (_str += _m + '分钟');
    // return `${_day}天${_h}小时${_m}分钟`;
    return _str;
  },
  updateItems (roleNum, timer, qrTimer) {
    // 权限数据
    let _roleArray = ROLES.filter(item=>item.value==roleNum); // ROLES[roleNum] || {};
    let _roleItem = _roleArray[0] || ROLES[1];
    var _allList = [{
      text: '分享权限',
      img: '../img/share/share.png',
      imgClass: 'shareImg flexNone',
      value: _roleItem.text || '',
      handle: "showShare"
    }, {
      text: '设备可用时长:',
      img: '../img/share/avalible.png',
      imgClass: 'avalibleImg flexNone',
      value: '' + timer,
      handle: "showAvalible"
    }, {
      text: '二维码有效时长:',
      img: '../img/share/qrcode.png',
      imgClass: 'qrcodeAvalibleImg flexNone',
      value: qrTimer || '',
      handle: "showQrAvalible"
    }];
    console.log("updateItems roleNum = ", _roleItem);
    // 如果是管理权限，则不显示可用时长
    if (_roleItem.value == 1) {
      this.data.items = _allList.slice(0, 1);
    } else {
      this.data.items = _allList;
    }
    this.setData({
      items: this.data.items
    })
  },
  
  // 把rooms数组转成json格式
  roomArrayToJson () {
    return bizUtil.arrayToJsonByKey(bizUtil.getOriginRooms(), true);
  },
  
  getFixStrByIndex (index, total, random) {
    return "&total=" + total + "&random=" + random + "&index=" + index;
  },
  // 二维码
  async drawImg (host, text) {
    if (!text) {
      return;
    }
    let params = text;
    // 每1000个分成1组
    let _per = 1000;
    let _num = params.length;
    let _result = [];
    let _count = Math.ceil(_num / _per);

    // 美化二维码，把总长度平均分成_count份
    _per = Math.ceil(_num / _count);

    console.log("_count = ", _count);
    let _isPages = _count >= 1;
    let _random = parseInt(Math.random() * 10000);
    for (let i = 0; i < _count; i++) {
      let _str = params.slice(i * _per, _per + i * _per);
      let _lastStr = _isPages ? (host + _str + this.getFixStrByIndex(i, _count, _random)) : (host + _str);
      console.log("_lastStr = ", _lastStr);
      _result.push(QR.drawImg(_lastStr, {
        typeNumber: 26,          // 密度
        errorCorrectLevel: 'M', // 纠错等级
        size: 242 // (20 * 4 + 15) * 6, // 白色边框
      }))
    }

    // var imgData = QR.drawImg(params, {
    //     typeNumber: 11,          // 密度
    //     errorCorrectLevel: 'H', // 纠错等级
    //     // size: 700,              // 白色边框
    // })
    let _updateObj = {
      qrcodeURL: _result
    };
    if (_count > 1) {
      _updateObj.tip = `(共${_count}页) 左滑`
      _updateObj.indicatorDots = true;
    } else {
      _updateObj.indicatorDots = false;
    }
    this.setData(_updateObj)
  },
  getDevCountByArray (array) {
    let _result = 0;
    if (!array || array.length == 0) {
      return _result;
    }
    for (let index in array) {
      _result += (array[index]?.devs?.length || 0);
    }
    return _result;
  },
  createQrcodeWithoutLogo (base64Str) {

    // 本地生成二维码个数不能超过255个
    let _count = this.getDevCountByArray(this.data.devData);
    if (_count > 255) {
      return ability.showToast({
        title: "本地二维码最多支持255个设备"
      })
    }

    let _host = app.globalData.companyInfo.host;
    console.log("createQrcodeWithoutLogo _host = ", _host);
    // 等待条
    ability.showLoading({});
    this.drawImg(_host + "?data=", base64Str).then(()=>ability.hideLoading()).catch(e=>ability.hideLoading());
  },

  showShare () {
    this.setDataByKey("showRoleNum", true);
  },
  showAvalible () {
    // 是否有可用时长的参数
    let _timestamp = app.globalData.tmpData?.qrTime || 0;
    console.log("showAvalible _timestamp = ", _timestamp);
    // 跳转到新页面
    ability.navigateTo('./lastTime/index?key=qrTime&timestamp=' + _timestamp)
  },
  // 二维码有效时长
  showQrAvalible () {
    // 是否有可用时长的参数
    let _timestamp = app.globalData.tmpData?.avalible || 0;
    console.log("showQrAvalible _timestamp = ", _timestamp);
    // 跳转到新页面
    ability.navigateTo('./lastTime/index?key=avalible&timestamp=' + _timestamp);
  },
  setDataByKey (key, value) {
    this.data[key] = value;
    this.setData({
      [key]: value
    })
  },
  shareBtnClick (e) {
    // 设置隐藏
    this.setDataByKey("showRoleNum", false);
    // 取得选中的数据
    let {value, index} = e.detail || {};
    // 取得选中的权限数据
    let {value: roleNum, text} = this.data.roles[index];
    // 角色信息
    this.data.roleNum = roleNum;
    // 更新数据
    this.updateTime();
    // 重新生成二维码
    this.createQrcode();
  },
  avalibleChange (e) {
    // this.setDataByKey("showAvalible", false);
    console.log("avalibleChange = ", e);
    let {value} = e.detail;
    this.setData({
      time: value
    })
  },

  // 86屏功能
  createUDPSocket () {
    return ability.createUDPSocket();
  },
  probeDevice () {
    // 创建udp socket监听
    let _socket = app.globalData.socket || this.createUDPSocket();
    app.globalData.socket = _socket;
    // 绑定端口
    ability.bindPort(_socket, 51813);
    // 设置可选头数据
    let _optionHeadData = "";
    // 取得协议头
    let _headerArrays = this.getProbeHeader(0, (_optionHeadData.length / 2));
    console.log("_headerArrays = ", Utils.ab2hex(_headerArrays));
    // 取得checksum
    let _checkSumHex = this.getCheckSumHex(_headerArrays);
    _headerArrays[8] = Number("0x" + _checkSumHex.slice(2));
    _headerArrays[9] = Number("0x" + _checkSumHex.slice(0, 2));

    // 发送的数据
    let _buffer = new Uint8Array(_headerArrays).buffer;

    // 发送
    var sendSocket = () => {
      // 如果有接收到数据则不发送
      if (this.data.currentSteps > 0) {
        return;
      }
      ability.sendSocket(_socket, {
        address: "255.255.255.255", // "192.168.1.1", // 
        port: 51812,
        message: _buffer,
        setBroadcast: true // 开启广播
      });
    }
    // 发送数据
    sendSocket();
    // 1s后再次发送数据
    setTimeout(sendSocket, 2000);

    ability.onMessageSocket(_socket, this.data.onMessage = (res) => {
      console.log("onMessageSocket res = ", res);
      let _array = Array.prototype.slice.call(new Uint8Array(res.message), 0);

      console.log("onMessageSocket _array = ", _array);

      // 取得前12个字节的数据
      let _magic = _array.slice(0, 4);
      let _msgtype = _array.slice(4, 6);
      // let _seq = _array.slice(6, 8);
      // let _checksum = _array.slice(8, 10);
      // let _optlen = _array.slice(10, 11);
      // let _version = _array.slice(11, 12);

      // 如果magic不一致，则退出
      let _magicStr = Utils.ab2hex(_magic);
      if (_magicStr != "a5a55a5a") {
        console.error("_magicStr = ", _magicStr);
        return;
      } else if (_msgtype[0] == 0) {
        // 如果等于0，则返回
        return;
      }

      // 判断数据是否要解析
      let _msgTypeStr = _msgtype[0];
      console.log("onMessageSocket _msgTypeStr = ", _msgTypeStr);
      // 如果是probe的返回
      if (_msgTypeStr == 1) {
        // 如果当前步骤大于0，则说明用户做过操作，不弹出对话框
        if (this.data.currentSteps > 0) {
          return;
        }
      }

      // 取得完整数据
      let _payload = [];
      // json数据
      let _jsonInfo = {};
      // 判断数据长度 授权时返回长度只有12
      if (_array.length > 12) {
        _payload = _array.slice(12);
        console.log("onMessageSocket _payload, _msgtype = ", _payload, Utils.ab2hex(_payload), _msgtype);
        // 解析数据
        let _str = "";
        for (let val of _payload) {
          let _t = String.fromCodePoint(val);
          _str += _t;
        }
        console.log("_str = ", _str);
        try {
          console.log("escape(_str) = ", escape(_str));
          // 防止中文乱码
          _str = decodeURIComponent(escape(_str));
          console.log("decodeURIComponent = ", _str);
          // '{"ip":"192.168.128.167","mac":"2418C69B1C1C","name":"蒙光面板"}'
          _jsonInfo = JSON.parse(_str.trim());
          console.log("parse _jsonInfo = ", _jsonInfo);
        } catch (e) {
          console.log("e = ", e);
          return;
        }
      };

      // 如果是probe的返回
      if (_msgTypeStr == 1) {

        // // 如果当前步骤大于0，则说明用户做过操作，不弹出对话框
        // if (this.data.currentSteps > 0) {
        //   return;
        // }

        // 当前步骤为2
        this.data.currentSteps = 1;

        // 保存86屏信息
        app.globalData.screenInfo = _jsonInfo || {};
        
        console.log("_jsonInfo = ", _jsonInfo)
        // 保存权限
        app.globalData.screenInfo.p = Number(this.data.roleNum == 1);

        wx.showModal({
          title: "连接到智慧面板",
          content: "识别到智慧面板(" + (_jsonInfo.name || "") + ")，是否数据同步",
          showCancel: true,
          confirmText: "是",
          cancelText: "否",
          cancelColor: '#000000',
          success: (res) => {
            console.log("showModal res = ", res);
            if (res.confirm) {
              // 当前步骤为2
              this.data.currentSteps = 2;
              // 授权请求
              this.oauth();
            } else if (res.cancel) {}
          },
          fail () {}
        })
      } else if (_msgTypeStr == 3) {

        if (this.data.currentSteps >= 3) {
          return;
        }

        // 当前步骤为2
        this.data.currentSteps = 3;
        // 去掉等待条
        ability.hideLoading();
        // 取消监听时间
        ability.offMessage(app.globalData.socket, this.data.onMessage);
        ability.offError(app.globalData.socket, this.data.onError);
        // 跳转下一页
        this.jumpNext();
      } 
    });
    // ability.onListeningSocket(_socket, (res) => {
    //   console.log("onListeningSocket res = ", res);
    // });
    ability.onErrorSocket(_socket, this.data.onError = (res) => {
      console.log("onError res, app.globalData.socket = ", res, app.globalData.socket);
      if (!app.globalData.socket) {
        return;
      }
      if (this.isClose) {
        return;
      }

      if (res?.errCode == -1 && res?.errNum == 500) {
        ability.showToast({
          title: "局域网同步失败，关闭微信后重试" // res?.errMsg || JSON.stringify(res || "")
        });
      }
    });
  },
  oauth () {
    // 设置可选头数据
    let _optionHeadData = "";
    // 取得协议头
    let _headerArrays = this.getProbeHeader(2, (_optionHeadData.length / 2));
    // 保存家庭密钥
    let _bleCode = Utils.ab2hex(app.globalData.blsdk.getBleCode());

    // 保存授权的家庭密钥
    app.globalData.screenInfo = app.globalData.screenInfo || {};
    app.globalData.screenInfo.bleCode = _bleCode;
    // 取得payload数据 "045b7312"
    let _origin = JSON.stringify({
      familyKey: _bleCode // ("045b7312") // _bleCode // "de443507"
    });
    let _hexStr = Utils.ab2hex(this.getCharFromStr(_origin));
    console.log("oauth _hexStr = ", _hexStr, _bleCode);

    let _payloadAes = sdkUtil.aesUtil.aesHexEncrypt(_hexStr, {
      padding: "Pkcs7"
    });

    let _payload = _payloadAes; // this.getPayload();
    // 加上payload数据
    _headerArrays = _headerArrays.concat(Utils.hexStrToArray(_payload))

    // 取得checksum
    let _checkSumHex = this.getCheckSumHex(_headerArrays);
    _headerArrays[8] = Number("0x" + _checkSumHex.slice(2));
    _headerArrays[9] = Number("0x" + _checkSumHex.slice(0, 2));

    // 等待条
    ability.showLoading({
      title: "授权中…"
    });

    // 发送数据
    let _address = app.globalData.screenInfo?.ip ||"255.255.255.255";
    let _uint8Array = new Uint8Array(_headerArrays);
    console.log("_address = ", _address);

    // 发送数据
    var sendSocket = () => {
      ability.sendSocket(app.globalData.socket, {
        address: _address,
        port: 51812,
        message: _uint8Array.buffer
      });
    };
    // 发送数据
    sendSocket();
    // 1s后再次发送数据
    setTimeout(sendSocket, 1000);

    // 等待条
    setTimeout(() => {
      // 如果当前状态还是处于授权状态，则提示失败
      if (this.data.currentSteps >= 3) {
        return;
      }
      // 去掉等待条
      ability.hideLoading();
      // 提示信息
      ability.showToast({
        title: "授权失败",
        icon: "error"
      })
    }, 8000);
  },
  getCharFromStr (str) {
    let _str = str || '';
    let _result = [];
    for (let i = 0, len = _str.length; i < len; i++) {
      _result.push(_str.codePointAt(i));
    }
    return _result;
  },
  jumpNext () {
    console.log("app.globalData.screenInfo = ", app.globalData.screenInfo)
    // 同步
    ability.navigateTo("/minePages/deviceShare/shareSocket/chooseRoom/index");
  },
  getProbeHeader (msgtype, optlen) {
    // 固定头 hex 4个字节
    let _result = [0xA5, 0xA5, 0x5A, 0x5A];
    // msgtype 2个字节
    let _msgTypeStr = (msgtype || '0').toString(16).padStart(4, 0);
    _result.push(Number(_msgTypeStr.slice(2, 4)) || 0);
    _result.push(Number(_msgTypeStr.slice(0, 2)) || 0);
    // Sequenece  2个字节
    let _seq = "c7b6"; // app.globalData.blsdk.getNextSeq();
    // _seq = _seq.toString(16).padStart(4, 0);
    _result.push(Number("0x" + _seq.slice(2, 4)) || 0);
    _result.push(Number("0x" + _seq.slice(0, 2)) || 0);
    console.log("_seq = ", _seq);
    // checksum  2个字节
    let _checkSum = [0, 0];
    _result.push(_checkSum[1]);
    _result.push(_checkSum[0]);
    // 可选头 optlen 1个字节
    let _optlen = optlen || 0;
    _result.push(_optlen);
    // 协议版本 1个字节 version
    let _version = 0;
    _result.push(_version);

    return _result; /* {
      magic: "5A5AA5A5", // 
      msgtype: _msgTypeStr, // hex 2个字节
      seq: _seq, // hex 2个字节
      checksum: "0000",  // hex 2个字节
      optlen: (optlen || 0).toString(16).padStart(2, 0), // 可选头部的长度，当前为0 1字节
      version: "00" // 1字节
    }*/
  },
  getCheckSumHex (data) {
    let _result = 0xbeaf;
    // 判断类型
    if (!data) {
      return _result;
    } else if (typeof data == "string") {
      // 16进制的算法
      _result += bizUtil.getSumByHexStr(data);
    } else if (data instanceof Object) {
      let _vals = Object.values(data);
      for (let item of _vals) {
        if (typeof item == "string") {
          // 16进制的算法
          _result += bizUtil.getSumByHexStr(item);
        } else if (typeof item == "number") {
          _result += item;
        }
      }
    } else if (data instanceof Array) {
      for (let item of data) {
        if (typeof item == "string") {
          // 16进制的算法
          _result += bizUtil.getSumByHexStr(item);
        } else if (typeof item == "number") {
          _result += item;
        }
      }
    }
    // 返回两个字节的数据
    let _resultStr = _result.toString(16).padStart(4, 0);
    return _resultStr.slice(-4); // 从后面开始往前数4个长度
  },


  
})
