// family/name/index.js
import ability from "../../utils/classDef/ability";
import bizUtil from "../../utils/bizUtil"
import allRooms from "../js/allroom"
const app = getApp()
Page({

  /**
   * 页面的初始数据
   */
  data: {
    index: -1, // 家庭修改的下表，如果新增，则为-1
    name: '',

    // 选中的房间
    selKey: "custom",
    // 枚举数据
    examples: [{
      n: '客厅',
      key: "kt"
    }, {
      n: '主卧',
      key: "zw"
    }, {
      n: '次卧',
      key: "cw"
    }, {
      n: '餐厅',
      key: "ct"
    }, {
      n: '书房',
      key: "sf"
    }, {
      n: '客房',
      key: "kf"
    }, {
      n: '阳台',
      key: "yt"
    }, {
      n: '厨房',
      key: "cf"
    }, {
      n: '卫生间',
      key: "wsj"
    }, {
      n: '玄关',
      key: "xg"
    }, {
      n: '庭院',
      key: "ty"
    }, {
      n: '自定义',
      key: "custom"
    }],

    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined // 家庭下所有设备信息
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 根据下标初始化标题
    this.initTitleByType(options);
    // 初始化房间信息
    this.initRoomName(this.data.selKey, this.data.name, true);
  },

  initRoomName (selKey, defaultName, showToast) {
    let _info = this.getRealName(selKey, defaultName, showToast);
    if (!_info) {
      return;
    }
    this.data.name = _info.n;
    this.setData({
      name: this.data.name
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    // 保存数据
    app.saveWhenHide();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },
  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },
  // 根据type初始化页面标题
  initTitleByType ({roomId, name, familyId}) {
    // 初始化所有相关数据
    this.initAllDataByFamily(familyId);
    let _str = "";
    let _roomId = Number(roomId) || 0;
    if (roomId >= 0) {
      _str = '房间名称修改';
      // 更新名称
      this.initEditName(decodeURIComponent(name));
      // // 设置默认的房间id
      // let _filter = this.data.examples.filter(item=>item.id == roomId);
      // if (_filter.length > 0) {
      // }
    } else {
      _str = '添加房间';
    }
    // 原始的房间id
    this.data.index = roomId;
    // 设置标题
    ability.setNavigationBarTitle(_str);
    // 设置默认数据
    let _key = this.getKeyByRoomId(roomId);
    console.log("initTitleByType _key = ", _key, roomId);
    this.setSelKey(_key);
  },
  // 根据房间id取得key
  getKeyByRoomId (roomId) {

    // 如果没有房间id，则认为是默认
    if (roomId == undefined) {
      return "custom";
    }

    let _rid = (roomId || 0) % 32;
    let _keyArray = Object.keys(allRooms);
    let _valArray = Object.values(allRooms);
    for (let i in _valArray) {
      let _val = _valArray[i];
      if (_val.indexOf(_rid) >= 0) {
        continue;
      }
      return _keyArray[i];
    }
    return undefined;
  },
  // 初始化名称
  initEditName (name) {
    this.data.name = name || '';
    this.setData({
      name: this.data.name
    })
  },
  
  clearName () {
    this.data.name = "";
    this.setData({
      name: ''
    })
  },

  nameInput (e) {
    let _name = e.detail.value;
    if (_name.length > 16) {
      this.showToast("房间名称长度不能大于16").then((isConfirm) => {
        this.setData({
          devName: _name.substr(0, 16)
        })
      });
      return;
    }
    this.data.name = _name;
    console.log("nameInput _name = ", _name);
  },
  showToast (str) {
    return wx.showModal({
      showCancel: false,
      content: str
    });
  },

  roomClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.examples[index];
    this.setSelKey(_item.key);
    // 初始化房间信息
    this.initRoomName(this.data.selKey);
  },
  setSelKey (val) {
    console.log("setSelKey val = ", val);
    this.data.selKey = val || "custom";
    this.setData({
      selKey: this.data.selKey
    })
  },

  saveName () {
    let _key = this.data.selKey;
    let _newName = this.data.name || "";
    // 判断是否为空
    if (_key == "custom") {
      if (!_newName) {
        return ability.showToast({
          title: "名称不能为空"
        });
      } else {
        // 取得所有的房间名称
        let _allRoomJson = this.getNamesByFloor() || {};
        // 如果重复
        if (_allRoomJson[_newName]) {
          return ability.showToast({
            title: "名称已存在,请重新输入"
          });
        }
      }
    }
    // 编辑还是新增家庭
    let _index = this.data.index;
    if (_index >= 0) {
      // 编辑
      this.data.allDevs[_index].n = _newName;
    } else {
      // 楼层信息
      let _floor = this.data.familyInfo?.floor;
      if (_floor == undefined || _floor < 0) {
        _floor = 0;
      }
      // 判断
      let _info = this.getRealName(_key, _newName, undefined, _floor);
      console.log("saveName _info = ", _info);
      if (!_info) {
        return;
      }
      // 房间id
      let _roomId = _info.id;

      // 场景
      let _scenes = [];
      // 判断是否有默认场景
      let _allScenes = bizUtil.getScensConfig();
      let _notRealRid = (_roomId % 32);
      if (_allScenes[_notRealRid]) {
        // 真实场景id
        _scenes = _allScenes[_notRealRid] || [];
        // 设置rid
        for (let item of _scenes) {
          item.rid = _roomId;
        }
      }

      // 取得当前楼层下的房间 然后设置sort字段
      let _sort = this.getRoomsByFloor(_floor);

      // 加入
      this.data.allDevs[_roomId] = {
        id: _roomId,
        n: _info.n,
        sort: _sort || 0, // 排序在后面
        devs: [],
        groups: [],
        scenes: _scenes
      };
    }
    // 返回上一页
    ability.navigateBack();
  },
  getRoomsByFloor (floor) {
    // json数据
    let _json = bizUtil.getDevsArrayByFloor(this.data.allDevs, floor);
    let _roomIdArray = Object.keys(_json);
    return _roomIdArray.length;
  },
  // 当前楼层的所有房间名称或者没有楼层时取得所有房间名称
  getNamesByFloor () {
    // 判断当前是否编辑房间
    // let _oldRid = 

    let _floor = this.data.familyInfo?.floor;
    if (_floor == undefined || _floor < 0) {
      return this.getAllNames(this.data.allDevs)
    } else {
      // 取得所有房间集合
      let _json = bizUtil.getDevsArrayByFloor(this.data.allDevs, _floor);
      return this.getAllNames(_json);
    }
  },
  getAllNames (obj, exceptRid) {
    let _result = {};
    if (!obj) {
      return _result;
    }
    for (let index in obj) {
      let _n = obj[index]?.n || "";
      _result[_n] = true;
    }
    return _result;
  },
  // 取得真实名称
  getRealName (_key, dataName, notToast, floor) {
    // 选中的数据
    let _info = this.data.examples.filter(item=>item.key == _key);
    if (_info.length == 0) {
      console.error("not exist selKey")
      return;
    }
    // 根据key和名称取得房间信息
    let _item = this.getRoomInfo(_info[0], dataName, floor);
    if (_item?.id == undefined) {
      console.error("not exist roomInfo");
      !notToast && (ability.showToast({
        title: "房间已满，请选择其他名称"
      }));
      return;
    }
    return _item;
  },
  // 根据数据取得选中的房间
  getRoomInfo (roomInfo, newName, floor) {
    let {key, n} = roomInfo || {};
    console.log("getRoomInfo roomInfo = ", roomInfo);
    // 取得选中的数组
    let _array = allRooms[key];
    // console.log("getRoomInfo _array = ", _array, allRooms, app.globalData.devs);
    if (!_array || _array.length == 0) {
      return {};
    }
    // 房间id
    let _roomInfo = {};
    // 已经存在的房间名称
    let _existNames = [];
    
    // 楼层信息
    let _floor = this.data.familyInfo?.floor;
    if (_floor == undefined || _floor < 0) {
      _floor = 0;
    }
    // 房间基数
    let _baseRoomId = (_floor >= 0 ? (_floor * 32) : 0);
    console.log("getRoomInfo _array, _baseRoomId = ", _array, _floor);
    for (let i = 0, len = _array.length; i < len; i++) {
      let roomId = _array[i] + _baseRoomId;
      // 如果房间id已经存在，则切换下一个
      let _info = this.data.allDevs[roomId];
      // console.log("getRoomInfo roomId = ", roomId, _info);
      if (!!_info) {
        // 保存房间名称
        _existNames.push(_info.n);
        continue;
      }
      _roomInfo.id = roomId;
      _roomInfo.n = bizUtil.getN(_existNames, newName || n);
      break;
    }
    return _roomInfo;
  },
})