// miniprogram/minePages/roomDetail/index.js
import bizUtil from '../../utils/bizUtil'
import ability from "../../utils/classDef/ability.js"
const app = getApp();
// 引用文件
var roomBehavior = require('../js/behavior/room.js')
Page({
  behaviors: [roomBehavior],
  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "房间详情"
      }
    },
    // 设备名称
    devName: "",
    // 游离设备
    offList: [],
    // 房间内设备
    roomList: [],
    configObj: {
      height: 115 / app.globalData.customPageSize.multip,
      seperator: 10,
      disUpEvent: true,
      unable: true, // 是否可以移动
    },
    offConfigObj: {
      height: 115 / app.globalData.customPageSize.multip,
      seperator: 10,
      unable: true, // 是否可以移动
      disUpEvent: true,
      showRoom: true
    },
    roomId: 0,

    devInfo: undefined, // 当前点击的设备信息
    clickType: "", // 当前点击的是删除还是添加

    maxHeight: 0, // 最大高度
    roomMaxHeight: 0, // 房间设备最大高度
    offMaxHeight: 0, // 游离设备最大高度
    bleUnavailable: false,

    roleNum: app.globalData.roleNum, // 用户权限

    // 发送时唤醒文案配置
    showSaveDialog: false,
    sendingConfig: {
      devTypeStr: ''
    },

    // 非唤醒的提示
    showLoadingDialog: false,
    loadingConfig: {
      forward: false
    },
    // 定时器
    timer: null,
    // 保存点击的房间id
    targetRoomId: 0,
    // 上一个修改房间方法是否过期
    lastAvalible: false,
    // 修改房间的方法
    modifyFunc: undefined,

    // 增加楼层显示
    floors: {},

    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined // 家庭下所有设备信息
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    
    // 初始化家庭和家庭下的设备数据
    this.initAllDataByFamily(options.familyId);
    // 楼层名称
    // this.data.roomJson = bizUtil.getRoomsByDevs(app.globalData.devs)?.roomJson || {};
    this.data.roomJson = this.initRoomJson(this.data.allDevs, this.data.familyInfo?.floor);
    // console.log("onLoad this.data.roomJson = ", this.data.roomJson);

    // 权限
    let _roleNum = this.data.familyInfo?.roleNum;
    this.data.roleNum = _roleNum == undefined ? app.globalData.roleNum : _roleNum;
    // this.data.roleNum = app.globalData.roleNum;

    console.log("onLoad options = ", options);
    // 传递的房间数据
    let _roomId = options.roomId || 0;
    this.data.roomId = parseInt(_roomId);
    this.setData({
      roomId: this.data.roomId,
      roleNum: this.data.roleNum,
      roomJson: this.data.roomJson
    });
  },
  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },
  initRoomJson (roomJson, floor) {
    let _result = {};
    if (!roomJson) {
      return _result;
    }
    let _isFloor = floor;
    if (_isFloor >= 0) {
      for (let roomId in roomJson) {
        if (roomId % 32 == 0) {
          _result[roomId] = roomJson[roomId].n || "";
        } else {
          let _t = Number(roomId) + 32 - (roomId % 32);
          // console.log("_t, roomId = ", _t, roomId);
          _result[roomId] = roomJson[_t].n + ' | ' + roomJson[roomId].n
        }
      }
    } else {
      for (let roomId in roomJson) {
        _result[roomId] = roomJson[roomId].n || "";
      }
    }
    return _result;
  },
  initName () {
    // 设备名称初始化
    this.data.devName = this.data.allDevs[this.data.roomId || 0].n || '';
    console.log("initName this.data.devName = ", this.data.devName);
    this.setData({
      devName: this.data.devName
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 初始化设备名称
    this.initName();
    // 更新设备信息
    this.initDevs();
    // 初始化
    this.initScrollHeight();
  },

  initScrollHeight () {
    // 比例
    let {multip, contentHeight} = app.globalData.customPageSize;
    // 保存按钮
    let _saveBtnHeight = 0; // 80 + 20 * 2;
    // 房间列表高度
    let _roomLen = this.data.roomList.length;
    let _offLen = this.data.offList.length;
    let {height: roomUnitH, seperator: roomUnitS} = this.data.configObj;
    let {height: offUnitH, seperator: offUnitS} = this.data.offConfigObj;
    let _roomHeight = _roomLen ? (_roomLen * (roomUnitH + roomUnitS) + roomUnitS): roomUnitS; // px
    let _offHeight = _offLen ? (_offLen * (offUnitH + offUnitS) + offUnitS): offUnitS; // px
    // 页面剩余高度
    let _height = this.data.maxHeight || (this.data.maxHeight = contentHeight - (115 + 31 + (31 + 21 + 44) * ((_roomLen?1:0) + (_offLen?1:0)) + _saveBtnHeight) / multip);
    console.log("_roomHeight, _offHeight, _height = ", _roomHeight, _offHeight, _height);
    // 判断高度
    if (_roomHeight >= (_height / 2)) {
      // 房间数据大于一半
      if (_offHeight >= (_height / 2)) {
        this.data.roomMaxHeight = this.data.offMaxHeight = false; // _height / 2;
      } else {
        this.data.offMaxHeight = true; // _offHeight;
        this.data.roomMaxHeight = false; // _height - _offHeight;
      }
    } else {
      // 房间数据小于一半
      this.data.roomMaxHeight = true; // _roomHeight;
      this.data.offMaxHeight = false; // _height - _roomHeight;
    }
    // console.log("this.data.roomMaxHeight, this.data.offMaxHeight = ", this.data.roomMaxHeight, this.data.offMaxHeight);
    this.setData({
      // maxHeight: _height / 2,
      roomMaxHeight: this.data.roomMaxHeight, // 房间设备最大高度
      offMaxHeight: this.data.offMaxHeight // 游离设备最大高度
    })
  },

  showLoading (str, _devInfoStr, pid) {

    if (_devInfoStr == "watersensor" ||  _devInfoStr == "bodysensor" || _devInfoStr == "doorsensor" || (_devInfoStr == 'panel' && !bizUtil.isPwrAvalibleDev(pid))) {
      this.data.showSaveDialog = true;
      this.data.sendingConfig.devTypeStr = _devInfoStr;
      this.setData({
        showSaveDialog: this.data.showSaveDialog,
        sendingConfig: this.data.sendingConfig
      })
    } else if (_devInfoStr == "light") {
      this.data.showLoadingDialog = true;
      this.setData({
        showLoadingDialog: this.data.showLoadingDialog
      })
    } else {
      ability.showLoading({
        title: str || "",
        mask: true
      })
    }
  },
  hideLoading () {
    // 如果显示了dialog，则隐藏
    if (this.data.showSaveDialog) {
      this.data.showSaveDialog = false;
      this.setData({
        showSaveDialog: this.data.showSaveDialog,
      })
    } else {
      ability.hideLoading();
    }

    // 灯泡的提示
    if (this.data.showLoadingDialog) {
      this.data.showLoadingDialog = false;
      this.data.loadingConfig.forward = false;
      this.setData({
        showLoadingDialog: this.data.showLoadingDialog,
        loadingConfig: this.data.loadingConfig
      })
    }
  },

  setAllValByKey (arrays, key, value, exceptStr) {
    let _arrays = (arrays || []).concat([]);

    for (let item of _arrays) {

      // 如果有例外
      if (!!exceptStr) {
        let _devTypeStr = bizUtil.getDevTypeStr(item.t);
        if (exceptStr.indexOf(_devTypeStr) >= 0) {
          if (_devTypeStr == "aircondition" && item?.sub > 0) {
            item.notMove = true; // 网关设备
            continue;
          } else if (_devTypeStr == "gateway") {
            item.notMove = true; // 网关设备
            continue;
          }
        }
      }

      item[key] = value;
    }

    // 按照图片显示
    if (!!exceptStr) {
      console.log("_arrays = ", _arrays, key);
      _arrays = _arrays.sort((a, b) => {
        let _bLen = (b[key])?(b[key].length):0;
        let _aLen = (a[key])?(a[key].length):0;
        return (_bLen) - (_aLen);
      });
      console.log("after _arrays = ", _arrays);
    }

    return _arrays;
  },

  // 房间取得房间内的所有列表
  getOtherDevs (obj, roomId) {
    if (!obj || roomId == undefined) {
      return [];
    }
    let _result = [];
    for (let rId in obj) {
      let _item = obj[rId];
      if (!_item || roomId == rId) {
        continue;
      }
      let _devsArray = _item.devs || [];
      // 更新房间名称
      this.setAllValByKey(_devsArray, "rn", _item.n);
      _result = _result.concat(_devsArray);
    }
    return _result;
  },

  // 初始化房间信息
  initDevs () {
    console.log("initDevs this.data.roomId = ", this.data.roomId);
    if (this.data.roomId != 0) {
      let _devs = this.data.allDevs || {};
      let _roomId = this.data.roomId || 0;
      // 房间内的设备
      let _roomDevs = _devs[_roomId].devs || [];

      // 更新房间名称
      this.setAllValByKey(_roomDevs, "rn", _devs[_roomId]?.n || "");
      
      console.log("initDevs app.globalData.devs = ", _devs)

      // 不在房间的设备
      let _offDevs = this.getOtherDevs(this.data.allDevs, _roomId);

      // 如果是分享权限则没有图片
      if (this.data.roleNum != 2) {
        this.data.roomList = this.setAllValByKey(_roomDevs, "img", './img/del.png');
        
        this.data.offList = this.setAllValByKey(_offDevs, "img", './img/add.png', ['gateway', 'aircondition']);
      } else {
        this.data.roomList = _roomDevs;
        this.data.offList = _offDevs;
      }
      
      // 禁止滑动
      // this.data.configObj.unable = (this.data.roomList.length <= 1);
    } else {
      let _devs = this.data.allDevs || {};
      let {rooms} = bizUtil.getRoomsByDevs(_devs);
      // 取得所有房间的设备
      let {devs} = bizUtil.getAllDevsBySort(rooms, _devs); // _roomDevs // this.setAllValByKey(_roomDevs);
      let _allDevs = devs || [];

      // 房间内的数据按照名称排序
      _allDevs = bizUtil.sortArrayByChar(_allDevs, "n");

      // 去除img
      _allDevs.map(item => item.img = "");
      this.data.roomList = _allDevs;
      this.data.offList = [];
    }
    // 房间内的数据按照名称排序
    this.data.roomList = bizUtil.sortArrayByChar(this.data.roomList, "n");
    this.data.offList = this.sortDevsByNameExceptGateway(this.data.offList); // bizUtil.sortArrayByChar(this.data.offList, "n");
    console.log("this.data.roomList, this.data.offList = ", this.data.roomList, this.data.offList);
    
    this.setData({
      roomList: this.data.roomList,
      offList: this.data.offList,
      configObj: this.data.configObj
    })
  },

  // 把设备按照名称排序，且网关排在最后
  sortDevsByNameExceptGateway (array) {
    if (!array || array.length == 0) {
      return [];
    }
    let _gateways = [], others = [];
    array.forEach(item => {
      if (typeof item == "object") {
        if (item.notMove) {
          _gateways.push(item);
        } else {
          others.push(item);
        }
        // let _devTypeStr = bizUtil.getDevTypeStr(item?.t);
        // if (_devTypeStr == "gateway") {
        //   _gateways.push(item);
        // } else if (_devTypeStr == "") {

        // } else {
        //   others.push(item);
        // }
      }
    })
    // 按照名称排序
    _gateways = bizUtil.sortArrayByChar(_gateways, "n");
    others = bizUtil.sortArrayByChar(others, "n");
    return others.concat(_gateways);
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
    console.log("roomDetail onHide");
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    console.log("onUnload");
    // 停止接收
    !!app.globalData.blsdk && (app.globalData.blsdk.destory());
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {
  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {
  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {
  },

  editName () {
    // // 分享者无权限
    // if (this.data.roleNum == 2) {
    //   return;
    // }
    let _roomId = this.data.roomId || 0;
    // 如果是全屋，则不允许修改名称
    if (_roomId == 0) {
      return;
    }

    let _devName = this.data.devName;
    ability.navigateTo('/device/curtain/deviceName/index?name='+encodeURIComponent(_devName) + "&from=roomDetail&roomId=" + _roomId + "&familyId=" + this.data.familyId)
  },
  // 保存房间
  saveDev (e) {
    this.saveInfo();
  },
  saveInfo (onlySave) {
    console.log("saveInfo");
    let _roomId = this.data.roomId || 0;

    // 如果房间id==0，则不更新
    if (_roomId != 0 && this.data.allDevs[_roomId]) {
      this.data.allDevs[_roomId].devs = this.data.roomList;
      // 这里由于更新了设备信息，所以需要更新每个房间的设备树(这里其实可以把记住每个更改房间，再更新更改的房间或者在移动的时候就把房间数据更新掉)
      let _obj = {};
      let _devs = this.data.offList;
      // console.log("saveInfo this.data.roomList, this.data.offList = ", this.data.roomList, this.data.offList);
      
      // 先取得所有不在房间内数据的集合 房间id: []列表
      for (let index in _devs) {
        let _item = _devs[index];
        if (!_item) {
          continue;
        }
        let _rId = _item.id || 0;
        if (_rId == _roomId) {
          continue;
        }
        _obj[_rId] = _obj[_rId] || [];
        _item.sort = _obj[_rId].length; // 排序用
        _obj[_rId].push(_item);
      }
      console.log("saveInfo _obj = ", _obj);
      // 再根据上一步获取到的数据集合更新数据
      let _allDevs = this.data.allDevs;
      for (let roomId in _allDevs) {
        let _objs = _allDevs[roomId];
        if (!_objs || roomId == _roomId) {
          continue;
        }
        _objs.devs = _obj[roomId] || [];
      }
      // 保存设备信息
      app.saveAllDevsMap();
      // app.setBizStorage("devs", app.globalData.devs);
    }

    // 返回上一页
    !onlySave && (ability.navigateBack());
  },

  // 房间列表中的删除图片点击
  imgDelClick (e) {
    console.log("imgDelClick");
    this.imgItemClick(e, true);
  },
  // 游离设备列表中的添加按钮点击
  imgAddClick (e) {
    console.log("imgAddClick");
    this.imgItemClick(e, false);
  },
  imgItemClick (e, isDel) {
    let _index = e.detail.index;
    console.log("imgItemClick _index = ", _index);

    let _originData = [];
    if (isDel) {
      _originData = this.data.roomList;
    } else {
      _originData = this.data.offList;
    }
    let _item = _originData[_index];
    if (!_item) {
      console.error("imgItemClick item is not exist");
      return;
    }
    // 发送数据
    let _roomId = isDel ? 0 : (this.data.roomId || 0);

    // 判断新房间下是否有重复设备名称
    let _existArray = bizUtil.getExistNameCountByDevInJson(app.globalData.devs, {
      id: _roomId,
      // n: _item.n || ""
    }, _item.n || "");
    if (_existArray.length > 0) {
      ability.showToast({
        title: "新房间存在重复名称，修改后移动房间"
      })
      return;
    }

    
    // 保存设备信息
    this.data.devInfo = _item;
    this.data.devIndex = _index;
    // 初始化
    this.data.lastAvalible = false;
    // 发送
    this.roomIdSet(_item, _roomId);

    // 不是修改当前家庭
    if (this.data.familyId != app.globalData.familyInfo?.id) {
      return;
    }
    // 上报一次
    app.uploadPrivateData();
  },

  // 判断是否已经有这个设备
  devAlreayExist (devInfo, array) {
    if (!devInfo || !array) {
      return true;
    }
    for (let idx in array) {
      let _item = array[idx];
      // 设备did相同
      if (devInfo.d == _item.d) {
        return true;
      }
    }
    return false;
  },

  initTimer () {
    // 先清除定时器
    this.destoryTimer();
    // 初始化定时器
    this.data.timer = setTimeout(() => {
      // 如果定时器还存在，则退出
      if (!this.data.timer) {
        return;
      }
      // 进一步提示
      this.data.loadingConfig.forward = true;
      this.setData({
        loadingConfig: this.data.loadingConfig
      })
    }, 5000);
  },
  destoryTimer (timer) {
    let _timer = timer || this.data.timer;
    if (!_timer) {
      return
    }
    clearTimeout(_timer);
    _timer = null;
  },

  // 对话框
  dialogClick (e) {
    let {data} = e.detail || {};
    // 是否显示对话框
    let _showLoadingDialog = false;
    if (data == 'retry') {
      // 重试按钮点击 不做
      // // 上一个修改房间方法是否过期
      // this.data.lastAvalible = true;
      // // 修改房间的方法
      // modifyFunc: undefined
    } else if (data == 'cancel') {
      // // 未响应
      // _showLoadingDialog = true;
      // 停掉之前的数据
      app.globalData.blsdk.destory();
      // 去掉接受的参数
      this.data.lastAvalible = true;
    } else if (data == 'confirm') {
      // 设置成功之后 如果有房间则是新增，如果0表示设置为全屋
      this.dealAfterSet(!this.data.targetRoomId)
      // this.dealAfterSet(this.data.targetRoomId);
      // 停掉之前的数据
      app.globalData.blsdk.destory();
      // 去掉接受的参数
      this.data.lastAvalible = true;
    }
    // 数据提示
    this.data.loadingConfig.forward = false;
    this.data.showLoadingDialog = _showLoadingDialog;
    this.setData({
      showLoadingDialog: this.data.showLoadingDialog,
      loadingConfig: this.data.loadingConfig
    })
  },

  // 设置房间id
  roomIdSet (devInfo, roomId) {
    // 设备类型
    let _t = devInfo?.t;
    let _devInfoStr = bizUtil.getDevTypeStr(_t);
    // 显示等待条
    this.showLoading('修改中…', _devInfoStr, _t);

    // 判断是否灯泡，如果是灯泡，则初始化定时器
    let _isLight = _devInfoStr == 'light';
    if (_isLight) {
      // 保存点击的房间id
      this.data.targetRoomId = roomId;
      // 初始化定时器
      this.initTimer();
    }

    app.globalData.blsdk.modefyDevice(devInfo, roomId, this.data.modifyFunc = (newRoomId) => {
      console.log("roomIdSet newRoomId = ", newRoomId);
      // 是否失效
      if (this.data.lastAvalible) {
        return;
      }
      // 隐藏等待条
      this.hideLoading();
      // 成功还是失败 这里可能是0
      if (newRoomId == undefined || (typeof newRoomId == "object" && !!newRoomId.errCode)) {
        this.setData({
          bleUnavailable: true
        })
        return;
      } else if (newRoomId === false || newRoomId < 0) {
        ability.showToast({
          title: "请检查设备状态" // 提示信息
        })
        return;
      }
      // 设置成功之后 如果有房间则是新增，如果0表示设置为全屋
      this.dealAfterSet(!newRoomId)
    });
  },
  dealAfterSet (isDel) {
    // 数据处理
    let _originData = [];
    let _destData = [];
    let _imgName = '';
    if (isDel) {
      _originData = this.data.roomList;
      _destData = this.data.offList;
      _imgName = 'add';
    } else {
      _originData = this.data.offList;
      _destData = this.data.roomList;
      _imgName = 'del';
    }
    let _item = this.data.devInfo;
    // 判断设备是否已经存在
    let _isExist = this.devAlreayExist(_item, _destData);
    if (_isExist) {
      console.error("device is exist");
      return;
    }
    _item.sort = _destData.length;
    _item.id = isDel?0:(this.data.roomId || 0);
    _item.img = `./img/${_imgName}.png`;
    _originData.splice(this.data.devIndex, 1);
    _destData.push(_item);

    // 删除对应的场景 增加判断，如果是触发设备，则不清除
    let _devTypeStr = bizUtil.getDevTypeStr(_item.t);
    let _isActionDev = bizUtil.isActionDev(_devTypeStr);
    let _idx = _item.id;
    if (_isActionDev && _devTypeStr != "thermostat") {
      bizUtil.moveDev(this.data.allDevs, _item, !_idx ? [0] : (_idx % 32 == 0 ? [0, _idx] : [0, _idx, _idx + 32 - (_idx % 32)]));
    }

    // 添加默认群组
    bizUtil.addDefaultGroupByDevs(this.data.allDevs);

    // 重新配置高度
    this.initScrollHeight();
    // 房间设备列表
    let _roomList = isDel ? _originData : _destData;
    let _offList = isDel ? _destData : _originData;
    // // 如果只有一个设备，则不允许滑动
    // this.data.configObj.unable = (_roomList.length <= 1);

    // 增加字母排序
    _roomList = bizUtil.sortArrayByChar(_roomList, 'n');
    _offList = bizUtil.sortArrayByChar(_offList, 'n');

    // 把网关排在最后
    let _key = "img";
    _destData = _destData.sort((a, b) => {
      let _bLen = (b[_key])?(b[_key].length):0;
      let _aLen = (a[_key])?(a[_key].length):0;
      return (_bLen) - (_aLen);
    });

    console.log("dealAfterSet _roomList, _offList = ", _roomList, _offList);

    this.setData({
      roomList: _roomList,
      offList: _offList,
      configObj: this.data.configObj
    });
    // 保存设备 写在这里是因为在这个页面点击物理按键的【重新进入小程序】，不会触发保存方法，所以需要都保存一下,且ios不走app.js的hide方法
    this.saveInfo(true);
    
    // 上报数据
    if (this.data.familyId != app.globalData.familyInfo?.id) {
      return;
    }
    app.uploadPrivateData();
  },

  dragEnds (e) {
    // 点击的下标
    let _data = e.detail.data;
    console.log("dragEnds _data = ", _data);
    // 设置sort属性
    this.initDevsByDragEnd(_data);
  },
  initDevsByDragEnd (array) {
    if (!array) {
      return;
    }
    // let _array = [];
    for (let i = 0, len = array.length; i < len; i++) {
      let _item = array[i];
      if (!_item) {
        return;
      }
      _item.sort = i;
    }
    // 增加字母排序
    this.data.roomList = bizUtil.sortArrayByChar(array, 'n');
  },

  // 编辑按钮点击
  manageDevs () {
    // 房间名称
    let _rn = this.data.allDevs[this.data.roomId || 0]?.n || '';
    console.log("manageDevs _rn = ", _rn);
    // 来源页面
    let _origin = "roomDetail";
    ability.navigateTo("/pages/batch/index?title=" + encodeURIComponent(_rn + "编辑") + "&from=" + _origin + "&rid=" + this.data.roomId + "&familyId=" + this.data.familyId);
  },

  // 删除房间
  delRoom () {
    if (this.data.roleNum == 2) {
      return;
    }

    // 如果房间下有设备，则提示先移除房间
    if (this.data.roomList?.length > 0) {
      return ability.showToast({title: "请先移除房间下的设备再删除"});
    }

    let _roomId = this.data.roomId;
    let _roomInfo = {
      id: _roomId,
      n: this.data.roomJson[_roomId] || ""
    };
    console.log("delRoom _roomInfo = ", _roomInfo);
    // 删除房间
    this.deleteRoom(this.data.allDevs, _roomInfo, () => {
      // 返回上一页
      this.back();
    })
  }
})