// miniprogram/manage/roomManage/index.js
import bizUtil from '../../utils/bizUtil'
import ability from "../../utils/classDef/ability.js"
// 引用文件
var roomBehavior = require('../js/behavior/room.js')
// 获取应用实例
var app = getApp();
Page({
  behaviors: [roomBehavior],
  data: {
    //渲染的列表数据
    dataList: [],
    //是否显示loadmore
    loadMoreShow: 0,
    //懒加载状态，用于异步不遮罩查询列表时，多次滑到底部
    isBottomLoading: false,

    //是否显示遮罩层
    isSearchViewShow: false,
    // 顶部条
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "房间管理",
        type: ''
      },
      right: {
        text: ''
      }
    },
    bleUnavailable: false,
    // 拖拽的配置
    config: {
      unable: false,
      height: 115
    },

    // 全屋的配置
    allConfig: {
      unable: false,
      height: 115
    },
    allList: [],

    // 是否楼层模式 默认不是
    isFloor: false,

    // 家庭数据
    showFloorPop: false,
    top: 0,

    // 楼层信息
    floorInfo: {
      // id: -1,
      n: '', // 楼层名称
      count: 0 // 设备数量
    },
    
    lightSupport: false, // 灯泡是否支持多楼层
    isLongTap: false, // 防止长按后又进入详情页面
    roleNum: 0,

    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined // 家庭下所有设备信息
  },
  onLoad (options) {
    console.log("roomManage onLoad");
    // 初始化家庭和家庭下的设备数据
    this.initAllDataByFamily(options.familyId);
    // 初始化高度
    let _height = 115 / (app.globalData.customPageSize.multip || 2);
    this.data.config.height = _height;
    this.data.allConfig.height = _height;

    // 权限
    let _roleNum = this.data.familyInfo?.roleNum;
    this.data.roleNum = _roleNum == undefined ? app.globalData.roleNum : _roleNum;
    this.setData({
      config: this.data.config,
      allConfig: this.data.allConfig,
      roleNum: this.data.roleNum
    })
    // 初始化是否支持楼层
    this.initSupport();
  },
  onUnload() {
    this.saveInfo();
  },

  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },

  initSupport () {
    this.data.lightSupport = this.getSupport();
    this.setData({
      lightSupport: this.data.lightSupport
    })
  },
  getSupport () {
    // console.log("getSupport app.globalData.devs = ", this.data.allDevs);
    // 取得家庭中所有的灯泡设备
    let _devArray = bizUtil.getAllDevsFilter(this.data.allDevs) || [];
    let {light} = bizUtil.getAllPidByArray(_devArray);
    console.log("getSupport light = ", light);
    // 如果不存在灯泡
    if (!light || light.length == 0) {
      return false;
    } else {
      for (let item of light) {
        let {v} = item;

        let _supportFloor = bizUtil.getSupportFloorByV(v);
        if (!_supportFloor) {
          return false;
        }
      }
    }
    return true;
  },

  initFloorInfo () {
    let {floor} = this.data.familyInfo || {}; // app.globalData.familyInfo;
    if (floor == undefined || floor < 0) {
      return;
    }
    // console.log("initFloorInfo floor, app.globalData.devs = ", floor, app.globalData.devs);
    // 楼层信息
    this.data.floorInfo.n = bizUtil.getFloorName(this.data.allDevs, floor);
    this.data.floorInfo.count = bizUtil.getDevsCountByFloor(this.data.allDevs, floor);
    // 是否多楼层
    this.data.isFloor = true;
    this.setData({
      floorInfo: this.data.floorInfo,
      isFloor: this.data.isFloor
    })
  },
  saveInfo () {
    console.error("roomManage");
    // 保存设备信息
    app.saveAllDevsMap();
    // app.setBizStorage("devs", app.globalData.devs);
  },
  onShow: function () {
    // 初始化房间数据
    this.initFloorInfo();
    // 初始化房间列表数据
    this.initRoomsByDevs();
  },
  getAllDevCount (json) {
    let _count = 0;
    if (!json) {
      return _count;
    }
    for (let roomId in json) {
      let {devs, panels, curtains} = json[roomId] || {};
      _count += (devs || []).length + (panels || []).length + (curtains || []).length
    }
    return _count;
  },

  initRoomsByDevs () {
    let _devs = bizUtil.getDevsByFloor(this.data.allDevs || {}, this.data.familyInfo?.floor);
    console.log("initRoomsByDevs _devs = ", _devs);
    let _list = bizUtil.getRoomsByDevs(_devs).rooms || [];

    // 全屋和其他房间的列表
    let _allList = [];
    let _otherList = [];

    // 遍历所有数据
    for (let key in _list) {
      let _item = _list[key];
      let _roomId = _item.id;
      // 如果是全屋的设备，则获取所有设备总数
      let _array = []; // bizUtil.getAllDevsFilter(_devs, (_roomId == 0) ? undefined : _roomId);
      if (_roomId == 0) {
        _array = bizUtil.getAllDevsFilter(this.data.allDevs);
      } else {
        _array = bizUtil.getAllDevsFilter(_devs, _roomId)
      }
      _item.count = _array.length + ' 设备';

      // 如果是全屋
      if (_roomId == 0) {
        _allList.push(_item);
      } else if (_roomId % 32 != 0) {
        _otherList.push(_item);
      }
    }
    this.data.dataList = bizUtil.sortArrayByKey(_otherList, "sort");
    this.data.allList = _allList;
    console.log("initRoomsByDevs this.data.dataList = ", this.data.dataList);
    // 设置的数据
    let _setObj = {
      dataList: this.data.dataList,
      allList: this.data.allList
    };
    // 如果条数小于2，则不显示排序
    if (this.data.dataList.length <= 2) {
      this.data.config.unable = true;
      _setObj.config = this.data.config;
    }
    this.setData(_setObj)
  },

  //下拉刷新
  refesh() {
    console.log("下拉刷新");
    // 停止下拉刷新
    wx.stopPullDownRefresh();
  },

  scrolltoupper () {
    console.log("scrolltoupper");
  },

  //懒加载
  loadMore() {
  },
  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    })
  },
  hideLoading () {
    ability.hideLoading();
  },

  isDevExist (devInfo) {
    let _devs = this.data.allDevs || [];
    let _filters = _devs.filter((item) => {
      return item.d == devInfo.d;
    })
    return !!_filters.length;
  },

  //查询数据
  onPullDownRefresh() {
  },
  goBack () {
    // 返回
    ability.navigateBack(2);
  },

  moveChange (e) {
    console.log("moveChange e = ", e);
  },
  touchEnd (e) {
    console.log("touchEnd e = ", e);
  },

  // 拖拽结束事件
  dragEnds (e) {
    // 点击的下标
    let _data = e.detail.data;
    console.log("dragEnds _data = ", _data);
    // 房间排序
    this.initRooms(_data);
  },

  initRooms (array) {
    if (!array) {
      return;
    }
    let _result = [];
    for (let i = 0, len = array.length; i < len; i++) {
      let _roomInfo = array[i];
      if (!_roomInfo) {
        continue;
      }
      let _id = _roomInfo.id;
      let _roomDetail = this.data.allDevs[_id] || {};
      _roomDetail.sort = _roomInfo.sort = i;
      _result.push({
        n: _roomDetail.n || _roomInfo.n,
        id: _id,
        sort: i
      })
    }
    // 保存本地房间列表数据
    this.data.dataList = _result;
  },
  allItemClick (e) {
    this.itemClick(e, "allList");
  },
  itemClick (e, key) {
    // 长按后不进入详情页面
    if (!key && this.data.isLongTap) {
      return;
    }
    console.log("itemClick e, data = ", e);
    // 点击的下标
    let _index = e.detail.index;
    // 设备信息
    let _lists = this.data[key || 'dataList'];
    // 保存房间信息
    let _roomInfo = _lists[_index];
    console.log("itemClick _roomInfo = ", _roomInfo);
    // app.globalData.roomInfo = _roomInfo
    // 保存房间信息
    ability.navigateTo("../../minePages/roomDetail/index?roomId=" + (_roomInfo.id || 0) + "&familyId=" + this.data.familyId);
  },
  // 长按事件
  longItemClick (e) {
    console.log("longItemClick e, data = ", e);
    // 如果只有控制权限则退出
    if (this.data.roleNum == 2) {
      return;
    }

    // 点击的下标
    let _index = e.detail.index;
    // 设备信息
    let _lists = this.data['dataList'];
    // 保存房间信息
    let _roomInfo = _lists[_index];
    console.log("longItemClick _roomInfo = ", _roomInfo);

    // 判断房间设备是否为空，如果为空，则提示无法删除
    let _countStr = (_roomInfo.count || "").split(" ")[0];
    if (_countStr != 0) {
      return ability.showToast({title: "请先移除房间下的设备再删除"});
    }

    // 长按
    this.data.isLongTap = true;
    // 删除房间
    this.deleteRoom(this.data.allDevs, _roomInfo, () => {
      // 取消长按
      this.data.isLongTap = false;
      // 页面重新加载
      this.initRoomsByDevs();
    });
  },

  // 切换到楼层模式
  changeFloor () {
    ability.showModal({
      title: "确定切换多楼层模式吗？",
      content: "切换多楼层模式后，该空间默认设置为1楼",
      showCancel: true
    }).then(res => {
      if (!res) {
        return;
      }
      // 切换多楼层
      this.setIsFloor(true);
      
      console.log("updateFamilyInfo this.data.floorInfo = ", this.data.floorInfo, this.data.allDevs);
      // 更新家庭信息
      this.updateFamilyInfo();
      // 创建默认的群组
      bizUtil.addDefaultGroupByDevs(this.data.allDevs);
    })
  },
  setIsFloor (val) {
    this.data.isFloor = val || false;
    this.setData({
      isFloor: this.data.isFloor
    })
  },

  // 取得所有楼层列表
  getAllFloorName () {
    return bizUtil.getFloorNames(this.data.allDevs);
  },

  updateFamilyInfo (floor) {
    let _allDevs = this.data.allDevs;
    // 默认楼层 floor == undefined 为切换成多楼层 否则为楼层选择
    let _floor = Number(floor) || 0;
    if (floor == undefined) {
      // 楼层信息
      this.data.floorInfo.n = bizUtil.getDefaultName(_floor);
      // 新增楼层
      bizUtil.addFloor(_allDevs, _floor, this.data.floorInfo.n, true);
    } else {
      // 楼层信息
      this.data.floorInfo.n = bizUtil.getFloorName(_allDevs, _floor);
    }
    this.data.familyInfo.floor = _floor;
    this.data.floorInfo.count = bizUtil.getDevsCountByFloor(_allDevs, _floor);

    // 保存
    app.saveWhenHide();
    // 显示数据
    this.setData({
      floorInfo: this.data.floorInfo
    })
  },

  getDefaultFloorInJson (json) {
    if (!json) {
      return {};
    }
    let _defaultFloor = 0;
    for (let roomId in json) {
      if (roomId == 0) {
        continue;
      }
      _defaultFloor = bizUtil.getFloorByRoomId(roomId);
    }
    return parseInt(_defaultFloor);
  },





  // initFloorTop () {
  //   // 取得家庭弹出层的高度
  //   let {statusHeight, navHeight, multip} = app.globalData.customPageSize;
  //   this.data.top = (statusHeight * multip + navHeight) || 0;
  //   this.setData({
  //     top: this.data.top
  //   })
  // },
  closePop () {
    this.floorManage(false);
  },
  showFloorClick () {
    this.floorManage(true);
  },

  floorManage (val) {
    this.data.showFloorPop = val || false;
    this.setData({
      showFloorPop: this.data.showFloorPop
    })
  },

  floorJump () {
    ability.navigateTo('/floor/list/index');
  },

  floorChange (e) {
    console.log("floorChange e = ", e);
    let {index, id} = e.detail;

    // 取得楼层名称
    let _floor = bizUtil.getFloorByRoomId(id);
    this.updateFamilyInfo(_floor);
    // 取得楼层下的房间列表
    this.initRoomsByDevs();

    // 关闭弹出
    this.closePop();
  },


  addRoom () {
    // ?floor=
    ability.navigateTo('/minePages/roomAdd/index');
  }
})
