import ability from "../../utils/classDef/ability";
import bizUtil from "../../utils/bizUtil"
import GROUPS from "../../static/js/jsConfig/groups"
// import WebSocket from "../../static/js/behavior/websocket"
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    list: []
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    let {title} = options;
    // 设置标题
    this.setNavTitle(title);

    // 初始化网关远程调试数据
    this.initList();
  },
  setNavTitle (title) {
    ability.setNavigationBarTitle(decodeURIComponent(title || ""));
  },
  initList () {
    // 远程调试数据
    let {isRemote} = app.globalData.familyInfo || {};
    // 默认可以点击远程调试
    let _diasbled = false;
    // 先获取所有数据，如果有网关才能开启远程调试
    this.gateways = this.getAllGateways();
    let _gatewayCount = this.gateways?.length || 0;
    if (_gatewayCount == 0) {
      _diasbled = true;
    }

    // 判断网关是否都不在线
    let _onlineGateways = this.getOnlineGateway(this.gateways);
    if (_onlineGateways.length == 0) {
      _diasbled = true;
    } else {
      this.gateway = _onlineGateways[0];
    }

    console.log("initList _diasbled = ", _diasbled);
    // 列表
    let _allList = this.getAllLisByRemote({isRemote, diasbled: _diasbled});
    this.setData({
      list: _allList
    })
  },
  getAllLisByRemote ({isRemote, diasbled}) {
    return [[{
      title: "房间管理",
      bindtap: "toRoomManage",
      tip: "",
    }, {
      title: "家庭设备总数量",
      tip: "",
    }], [{
      title: "网关远程调试",
      value: isRemote || 0, // 默认关闭
      key: "remote",
      disabled: diasbled || false,
      type: "switch",
      tip: "",
    }], [{
      title: "关于",
      bindtap: "toAbout",
      tip: "",
    }]];
  },
  getAllGateways () {
    let _allDevArray = bizUtil.getAllDevsFilter(app.globalData.devs);
    let {gateway} = bizUtil.getAllPidByArray(_allDevArray);
    // 非ihg
    let _gateways = gateway.filter(item => !bizUtil.isIhg(item.t));
    return _gateways || [];
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    // 取得房间个数和设备个数
    this.initRoomAndDevCount();
  },
  // 获取房间个数
  getRoomCount () {
    return Object.keys(app.globalData.devs || {}).length;
  },
  getDevCount () {
    // 家庭总设备数量
    let _tmpArray = bizUtil.getAllDevsFilter(app.globalData.devs, undefined, 'devs') || [];
    return _tmpArray?.length || 0;
  },
  // 初始化房间个数和设备个数
  initRoomAndDevCount () {
    // 房间个数
    let _roomCount = this.getRoomCount();
    // 设备个数
    let _devCount = this.getDevCount();
    // 设置数据
    let _firstList = this.data.list[0] || [];
    let _firstLen = _firstList.length;
    if (_firstLen == 0) {
      return;
    }
    (_firstLen > 0) && (_firstList[0].tip = _roomCount + "个房间");
    (_firstLen > 1) && (_firstList[1].tip = _devCount);
    // 设置
    this.setData({
      list: this.data.list
    })
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  // 跳转房间管理页面
  toRoomManage (e) {
    ability.navigateTo("/minePages/roomManage/index")
  },
  // 跳转关于页面
  toAbout (e) {
    let {index, idx} = e.currentTarget.dataset;
    let {title} = (this.data.list[index] || [])[idx] || {};
    ability.navigateTo("/minePages/about/index?title=" + encodeURIComponent(title || ""))
  },

  // 取得任意在线的网关
  getOnlineGateway (gateways) {
    let _gateways = gateways || [];
    // 判断有没有网关，或者
    let _onlineGateway = _gateways.filter(item => item.online);
    return _onlineGateway;
    // if (_onlineGateway.length > 0) {
    //   return _onlineGateway[0];
    // }
    // return _gateways[0];
  },

  // 远程调试
  switchChange (e) {
    let {value} = e?.detail || {};
    let {disabled} = e?.currentTarget?.dataset || {};
    if (disabled) {
      ability.showToast({
        title: "远程调试必须有在线网关"
      });
      return false;
    }
    // 这是点击的事件
    if (value == undefined) {
      return;
    }
    console.log("switchChange value, e = ", value, e);
    // 如果是关闭远程调试，则取消
    if (!value) {
      app.globalData.blsdk.clseRemoteDebug();
      // 远程调试关闭
      app.globalData.familyInfo.isRemote = false;
    } else {

      // 判断网关是否都不在线
      let _onlineGateway = this.gateway; // this.getOnlineGateway(this.gateways);
      if (!_onlineGateway) {
        ability.showToast({
          title: "远程调试必须有在线网关"
        });
        return false;
      }

      // loadding
      ability.showLoading({});
      // 获取websocket的url
      this.getRemoteSocket((url) => {
        if (!url) {
          ability.hideLoading();
          return;
        }
        let {uuid, sid, skey} = app.globalData.familyInfo || {};
        console.log("getRemoteSocket url, uuid, sid, skey = ", url, uuid, sid, skey);
        // 创建websocket
        app.globalData.websocket = app.globalData.blsdk.startRemoteDebug({
          url: url,
          gatewayInfo: _onlineGateway, // 网关信息
          uuid: uuid,
          sid: sid,
          skey: skey,
          // host: ,
          onOpenSuccess (res) {
            ability.showToast({
              title: "打开远程调试成功"
            })
            // 远程调试使能
            app.globalData.familyInfo.isRemote = true;
          },
          onOpen: () => {
            console.log("onOpen callback");
            ability.hideLoading();
          }
        });
      });
    }
  },
  // 获取socket
  getRemoteSocket (callback) {
    let {sid, skey, uuid, groupIndex} = app.globalData.familyInfo || {};
    // 上报的集群域名
    let _groupStr = GROUPS[Number(groupIndex) || 0]?.value || GROUPS[0].value;
    app.globalData.blsdk.getRemoteSocket({
      familyid: sid,
      token: skey,
      userid: uuid, // 不校验，有值即可
      remoteHost: _groupStr // "chn-ee08f451.ibroadlink.com", // 目前只能测试集群
    }).then(res => {
      console.log("getRemoteSocket res = ", res);
      let {status, data} = res || {};
      if (status != 0 || !data || !data.url) {
        !!callback && (callback());
      } else {
        !!callback && (callback(data.url[0]));
      }
    }).catch(e => {
      console.log("getRemoteSocket e = ", e);
      !!callback && (callback());
    })
  },
  // 解析数据
  hexStrToJson (hexStr) {
    if (!hexStr) {
      return;
    }
    let _str = "";
    for (let i = 0, len = hexStr.length; i < len; ) {
      let _tmp = hexStr.substr(i, 2) || "";
      _str += (String.fromCharCode('0x' + _tmp) || "");
      i += 2;
    }
    _str = _str.trim();
    console.log("hexStrToJson _str2 = ", _str);
    return JSON.parse(_str); 
  }
})