// device/gateway/thirdparty/config/index.js
import ability from "../../utils/classDef/ability"
import bizUtil from "../../utils/bizUtil"

const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    // 配置文件
    configList: [{
      key: "scene",
      hidden: false, // 是否显示
      total: 0, // 总路数
      title: "智能场景",
      isAllSelected: true, // 默认全部选中
      tip: "",
      isOpen: true,
      list: [],
      unselectedList: {} // 是否选中，这里单独拿出来是为了防止没有点击保存的时候数据会更新掉  默认选中
    }, {
      key: "group",
      title: "群组",
      hidden: false, // 是否显示
      isAllSelected: true, // 默认全部选中
      total: 0, // 总路数
      tip: "",
      isOpen: true,
      list: [],
      unselectedList: {} // 是否选中，这里单独拿出来是为了防止没有点击保存的时候数据会更新掉  默认选中
    }, {
      key: "device",
      title: "设备",
      isAllSelected: true, // 默认全部选中
      hidden: false, // 是否显示
      total: 0, // 总路数
      tip: "",
      isOpen: true,
      list: [],
      unselectedList: {} // 是否选中，这里单独拿出来是为了防止没有点击保存的时候数据会更新掉  默认选中
    }],

    roomChanged: false, // 是否显示房间选择页面
    rooms: [], // 选择房间列表
    roomInfo: {}, // 当前选中的房间

    allCollection: {
      scene: {},
      group: {},
      device: {}
    }, // 是否选中的表示 单独拿出来是为了切换房间时数据会变
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    // 初始化设备名称
    this.initDevName(decodeURIComponent(options?.title || ""));
    // 是否多楼层
    let _floorIndex = app.globalData.familyInfo?.floor;
    // 根据多楼层取得房间列表
    let _roomList = this.getRoomsByFloorAndDevs(_floorIndex, app.globalData.devs);

    // 初始化房间信息
    let _roomId = (app.globalData.roomInfo || {}).id;
    let _roomInfo = this.getRoomInfoByRoomId(_roomId);
    console.log("onLoad _roomInfo = ", _roomInfo);

    // 初始化所有设备信息
    this.data.allCollection = this.getListByKeyAndList();

    this.setData({
      rooms: _roomList,
      roomInfo: _roomInfo,
      allCollection: this.data.allCollection
    })
  },

  // 根据房间id取得房间信息
  getRoomInfoByRoomId (roomId) {
    // 初始化房间信息
    let _result = {};
    let _roomId = roomId || 0;
    console.log("getRoomInfoByRoomId _roomId = ", _roomId);
    let _allDevs = app.globalData.devs;
    if (_roomId != undefined) {
      let _roomInfo = _allDevs[_roomId] || _allDevs[0] || {};
      console.log("getRoomInfoByRoomId _roomInfo = ", _roomInfo);
      _result.id = _roomInfo.id;
      _result.n = _roomInfo.n;
    } else {
      _result = {
        id: 0, // 默认全屋
        n: '全屋'
      };
    }
    return _result;
  },

  // 初始化房间
  getRoomsByFloorAndDevs (floor, allDevs) {
    // // 所有设备
    // let _allDevs = allDevs || {};
    // // 当前楼层
    // let _floor = floor || 0;
    // // 根据多楼层取得数据
    // let _devs = bizUtil.getDevsByFloor(_allDevs, _floor);
    // let {rooms} = bizUtil.getRoomsByDevs(_devs);
    // return rooms;
    return this.getAllRoomByAllDevs(allDevs);
  },
  getAllRoomByAllDevs (allDevs) {
    let _result = [];
    if (!allDevs) {
      return _result;
    }
    // 为了把楼层放在房间之前，增加了临时变量
    let _addedRoomIds = [];
    // 排序 从小到大
    let _roomIdArray = Object.keys(allDevs).map(item => Number(item)).sort((a, b) => a - b);
    console.log("getAllRoomByAllDevs _roomIdArray = ", _roomIdArray);
    for (let index in _roomIdArray) {
      let _roomId = _roomIdArray[index];
      let _roomIdInt = parseInt(_roomId);
      let _roomName = allDevs[_roomId]?.n || '';
      if (_roomId % 32 != 0) {

        let _floorId = _roomId + 32 - _roomId % 32;
        let _floorName = (allDevs[_floorId]?.n || "");
        if (_floorName) {

          // 加入楼层
          if (_addedRoomIds.indexOf(_floorId) < 0) {
            _result.push({
              id: _floorId,
              n: _floorName,
              sort: _result.length
            });
            _addedRoomIds.push(_floorId);
          }

          _roomName = "  " + _floorName + " | " + _roomName;
        }
      } else if (_roomId > 0) {
        continue;
      }
      // 防止重复
      _addedRoomIds.push(_roomIdInt);
      _result.push({
        id: _roomIdInt,
        n: _roomName,
        sort: _result.length
      })
    }
    return _result;
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
    // 初始化列表数据
    this.initListByRoomInfo(true);
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  // 获取完整的数据
  getListByKeyAndList () {
    let _result = {
      scene: {},
      group: {},
      device: {}
    };
    // if (!key || !list) {
    //   return _result;
    // }
    let _allDevs = app.globalData.devs || {};
    let {devs, scenes, groups} = bizUtil.getAllDevsBySort(_allDevs, _allDevs);
    console.log("getListByKeyAndList scenes = ", scenes);
    // if (key == "scene") {
      // 这里要取得所有的场景信息
      for (let scene of scenes) {
        let _sidSDK = bizUtil.getSceneId(scene?.sid, scene?.rid);
        _result.scene[_sidSDK] = scene.ig || false;
      }
    // } else if (key == "group") {
      for (let group of groups) {
        let _mapKey = (group.gid || 0) + "-" + (group.t || "");
        _result.group[_mapKey] = group.ig || false;
      }
    // } else if (key == "device") {
      for (let dev of devs) {
        _result.device[dev?.d] = dev.ig || false;
      }
    // }
    console.log("getListByKeyAndList _result = ", _result);
    return _result
  },

  // 根据ability初始化面板列表数据
  initListByRoomInfo (isFirst) {
    // 所有设备信息
    let _allDevs = app.globalData.devs || {};
    // 房间id
    let _roomId = this.data.roomInfo?.id || 0;
    let _roomDetailInfo = _allDevs[_roomId] || {};
    // 取得房间下的场景数据
    let _sceneInfo = _roomDetailInfo.scenes || [];
    // 取得设备信息和群组信息
    let _devs = bizUtil.getDevsOrGroupsByRoomId(_roomId, _allDevs, "devs");
    let _groups = bizUtil.getDevsOrGroupsByRoomId(_roomId, _allDevs, "groups");

    // 遍历数据
    for (let item of this.data.configList) {
      let {key} = item;
      if (key == "scene") {
        // 这里场景信息需要增加一个sidSDK,为了判断是否选中
        // item.list = _sceneInfo;
        item.list = (_sceneInfo.concat([])).map(item => {
          item.mapkey = bizUtil.getSceneId(item?.sid, item?.rid);
          return item;
        });
      } else if (key == "group") {
        let _list = bizUtil.sortArrayByKey(_groups, "gid");
        item.list = (_list.concat([])).map(item => {
          item.mapkey = (item.gid || 0) + "-" + (item.t || "");
          return item;
        });
        // item.list = bizUtil.sortArrayByKey(_groups, "gid");
      } else if (key == "device") {
        // let _list = bizUtil.sortArrayByKey(_devs, "gid");
        item.list = (_devs.concat([])).map(item => {
          item.mapkey = item.d || "";
          return item;
        });
        // item.list = _devs;
      }

      // // 这里首页进入默认房间
      // if (isFirst) {
      //   // 这里改成全局抽象的变量 防止切换房间时数据变化
      //   this.data.allCollection[key] = this.data.allCollection[key] || {};
      //   this.data.allCollection[key] = 
      // }
      
      // item.unselectedList = item.list.map((item) => {
      //   return item.ig || false;
      // })

      // 判断个数
      let _length = item?.list?.length || 0;
      item.hidden = _length == 0;
      // 是否全部选中
      item.isAllSelected = this.isItemAllSelect(key, item.unselectedList);
      // 更新tip
      item.tip = _length + item.title;
    }

    // 设置数据
    this.setData({
      configList: this.data.configList,
    });
  },

  // 判断条目是否全选
  isItemAllSelect (key, list) {
    if (!key) {
      return false;
    }
    let _item = this.data.allCollection[key];
    let _list = this.data.configList.filter(item => item.key == key);
    console.log("isItemAllSelect _list = ", _list);
    if (_list.length == 0) {
      return;
    }
    let _itemObj = _list[0] || {};
    let _itemList = _itemObj.list || [];
    console.log("isItemAllSelect _itemList = ", _itemList);
    // let _key = _itemObj
    let _allSelect = true;
    for (let item of _itemList) {
      let _mapValue = _item[item.mapkey];
      console.log("isItemAllSelect _mapValue, item.mapkey = ", _mapValue, item.mapkey);
      if (_mapValue) {
        _allSelect = false;
        break; 
      }
    }
    console.log("isItemAllSelect _allSelect, _itemList = ", _allSelect, _itemList);
    // return !(list.some(item => item.ig));
    // return !(_itemList.some(item => item));
    return _allSelect;
  },

  // 初始化设备名称
  initDevName (name) {
    ability.setNavigationBarTitle(name);
  },
  setProperValue (key, val) {
    if (!key) {
      return;
    }
    this.data[key] = val || '';
    this.setData({
      [key]: val || ""
    })
  },
  // 点击展开
  collapeClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.configList[index];
    // 设置是否展开
    _item.isOpen = !_item.isOpen;

    let _str = `configList[${index}].isOpen`;
    // 设置属性
    this.setData({
      [_str]: _item.isOpen
    })
  },
  // 条目点击
  itemClick (e) {
    let {index, idx} = e.currentTarget.dataset;
    let _obj = this.data.configList[index] || {};
    let {list, key, type, unselectedList} = _obj;
    let _childInfo = list[idx] || {};
    // let _itemInfo = (unselectedList || [])[idx] || {};
    // 跳转到对应的页面
    // console.log("itemClick _itemInfo = ", _itemInfo);
    // 判断跳转的页面
    // unselectedList[idx] = !unselectedList[idx];
    // _itemInfo.ig = !_itemInfo.ig;
    
    let _info = this.data.allCollection[key] || {};
    console.log("itemClick _info, key, _childInfo = ", _info, key, _childInfo);
    _info[_childInfo.mapkey] = !_info[_childInfo.mapkey];

    
    console.log("itemClick _info = ", _info);
    // 是否全部选中
    _obj.isAllSelected = this.isItemAllSelect(key, unselectedList);

    // console.log("itemClick this.data.configList = ", this.data.configList, this.data.allCollection);
    // 设置数据
    this.setData({
      [`configList[${index}]`]: _obj,
      allCollection: this.data.allCollection
      // [`allCollection[${key}]`]: _info
    })
  },
  // 跳转详情页面
  jumpGroupDetail () {
    ability.navigateTo("/device/light/devProperty/index?remote=0");
  },
  
  showPop () {
    this.hidePop(true);
  },
  closePop () {
    this.hidePop();
  },
  hidePop (isOpen) {
    this.setProperValue("roomChanged", isOpen || false);
  },

  chooseRoom (e) {
    let {rid, index} = e.currentTarget.dataset;
    let _roomInfo = app.globalData.devs[rid || 0];
    this.data.roomInfo = this.data.rooms[index];
    // this.data.roomInfo.n = _roomInfo.n;
    // this.data.roomInfo.id = _roomInfo.id;
    this.setProperValue("roomInfo", this.data.roomInfo);
    // 隐藏
    this.hidePop();
    // 显示对应的房间信息
    this.initListByRoomInfo();
  },
  // 全选按钮点击
  itemAllSelect (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.configList[index];
    _item.isAllSelected = !_item.isAllSelected;
    console.log("itemAllSelect isAllSelected = ", _item.isAllSelected);

    let _list = _item.list || [];
    let _info = this.data.allCollection[_item.key] || {};
    for (let item of _list) {
      _info[item.mapkey] = !_item.isAllSelected;
    }
    // // 设置数据
    // if (_item.unselectedList?.length > 0) {
    //   _item.unselectedList.fill(!_item.isAllSelected);
    //   // _item.unselectedList = _item.unselectedList.map(item => !_item.isAllSelected);
    // }
    // bizUtil.setArrayItem(_item.unselectedList, ["ig"], [!_item.isAllSelected])
    // 设置数据
    this.setData({
      [`configList[${index}]`]: _item,
      allCollection: this.data.allCollection
    })
  },
  // 更新数据
  updateSelectDataByList (configList) {
    if (!configList) {
      return;
    }
    let _allDevs = app.globalData.devs;
    let {scene, group, device} = this.data.allCollection;
    for (let rid in _allDevs) {
      let {devs, groups, scenes} = _allDevs[rid] || {};
      if (!!devs && devs.length > 0) {
        for (let item of devs) {
          item.ig = device[item?.d] || false;
        }
      }
      if (!!groups && groups.length > 0) {
        for (let item of groups) {
          let _key = (item.gid || 0) + "-" + (item.t || "");
          item.ig = group[_key] || false;
        }
      }
      if (!!scenes && scenes.length > 0) {
        for (let item of scenes) {
          let _key = bizUtil.getSceneId(item?.sid, item?.rid);
          item.ig = scene[_key] || false;
        }
      }
    }

    // for (let item of configList) {
    //   let {list, unselectedList} = item || {};
    //   if (!list || !unselectedList || list.length == 0) {
    //     continue;
    //   }
    //   for (let index in list) {
    //     let _info = list[index];
    //     let _value = unselectedList[index];
    //     if (!_info) {
    //       continue;
    //     }
    //     _info.ig = !!_value || false;
    //   }
    // }
  },
  // 保存按钮点击
  saveBtnClick () {
    // 更新点击的数据
    this.updateSelectDataByList(this.data.configList);
    // 保存用户私有数据
    app.saveAllDevsMap();
    // 上报一次私有数据
    app.uploadPrivateData((res) => {
      if (!res) {
        ability.showToast({
          title: "请检查网络",
          icon: "error"
        })
        return;
      }
      ability.showToast({
        title: "保存成功",
        icon: "success"
      });
      setTimeout(this.back, 1000);
    });
  },
  back () {
    ability.navigateBack();
  }
})