// minePages/success/index.js
import bizUtil from "../../utils/bizUtil"
import ability from "../../utils/classDef/ability"
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    imgKey: "", // 图标
    tip: '', // 提示
    contents: [], // 内容
    buttons: [],
    bottomBts: [], // 底部按钮

    intervalTimer: null,
    count: 0, // 倒计时秒
  },

  /**
   * 生命周期函数--监听页面加载
   * @options
   *  type: 
   *    1, 有两个中间按钮（更新备份和立即还原）,tip(登陆成功),无底部按钮和附加内容 
   *    2, 有底部按钮(返回首页)，tip（备份成功）和附加内容，无中间按钮
   *    3，有底部按钮（重试），tip（备份失败），无附件内容和中间按钮
   *    4, 有1个中间按钮(更新备份),tip（暂无备份）,无底部按钮和附加内容
   *    5，有底部按钮（重试），tip（还原失败），无附件内容和中间按钮
   *  imgKey:
   *    0, 无
   *    1, 成功 success
   *    2，失败 fail
   *    3，空 empty
   *  count: 
   *    0 无倒计时
   *    > 1 倒计时计数
   *  extend： 扩展数据
   *    type = 2，用于传递更新时间和备份名称
   */
  onLoad(options) {
    let {imgKey, type, count, extend} = options || {};
    // 页面图标
    let _imgKey = this.getImgKeyByKey(imgKey);
    // 扩展数据
    let _extendData = this.getExtendDataByOption(extend);
    // 页面配置
    let {tip, buttons, bottomBts, contents} = this.getConfigByType(type, _extendData);
    // 保存定时器数据
    this.data.count = count || 0
    // 初始化定时器
    this.initIntervalTimerByCount(count);

    this.setData({
      imgKey: _imgKey,
      count: this.data.count,
      tip: tip || "", // 提示
      contents: contents || [], // 内容
      buttons: buttons || [],
      bottomBts: bottomBts || [], // 底部按钮
    })
  },
  getExtendDataByOption (str) {
    if (!str || str.length == 0) {
      return {};
    }
    let _json = JSON.parse(decodeURIComponent(str));
    console.log("getExtendDataByOption _json = ", _json);
    return _json;
  },
  getImgKeyByKey (imgKey) {
    return imgKey || "";
  },
  getConfigByType (type, extendData) {
    let _result = {};
  /*    1, 有两个中间按钮（更新备份和立即还原）,tip(登陆成功),无底部按钮和附加内容 
   *    2, 有底部按钮(返回首页)，tip（备份成功）和附加内容，无中间按钮
   *    3，有底部按钮（重试），tip（备份失败），无附件内容和中间按钮
   *    4, 有1个中间按钮(更新备份),tip（暂无备份）,无底部按钮和附加内容
   *    5，有底部按钮（重试），tip（还原失败），无附件内容和中间按钮
   */
    if (type == 1) {
      _result.tip = "登陆成功";
      _result.buttons = [{
        title: "更新备份",
        bindFunc: "updateBackup",
        classStr: "borderF78D3F bgWhite colorF78D3F"
      }, {
        title: "立即还原",
        bindFunc: "reBack",
        classStr: "bgF78D3F white"
      }];
    } else if (type == 2) {
      _result.tip = extendData?.content || "备份成功";
      _result.contents = extendData?.contents || [/*{
        text: "备份名称：" + (extendData?.n || ""),
      }, */{
        text: "备份时间：" + (extendData?.tip || ""), // 2022/02/22 11:50
      }];
      _result.bottomBts = [{
        title: "返回首页",
        bindFunc: "reLaunch"
      }];
    } else if (type == 3 || type == 5) {
      _result.tip = type == 3 ? "备份失败" : "还原失败";
      _result.bottomBts = [{
        title: "重试",
        bindFunc: "reTry"
      }];
    } else if (type == 4) {
      _result.tip = "暂无备份";
      _result.buttons = [{
        title: "更新备份",
        bindFunc: "updateBackup",
        classStr: "borderF78D3F bgWhite colorF78D3F"
      }];
    }
    return _result;
  },
  initIntervalTimerByCount (count) {
    if (!count || count < 0) {
      return;
    }
    bizUtil.clearTimer(this.data.intervalTimer, "interval");
    this.data.intervalTimer = setInterval(() => {
      this.data.count -= 1;
      if (this.data.count <= 0) {
        // 如果时间到了，则根据底部按钮的第一个执行
        let _funcKey = this.data.bottomBts[0]?.bindFunc;
        if (_funcKey && this[_funcKey]) {
          this[_funcKey]();
        } else {
          // 如果没有则返回上一页
          ability.navigateBack();
        }
        return;
      }
      // 如果时间没到，则更新底部第一个按钮的内容
      this.updateBottomBtnsByCount(this.data.count);
    }, 1000);
  },
  updateBottomBtnsByCount (count) {
    // 如果时间到了，则根据底部按钮的第一个执行
    this.setData({
      count: count || 0
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    bizUtil.clearTimer(this.data.intervalTimer, "interval");
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  // 更新备份
  updateBackup () {
    ability.redirectTo("/minePages/backup/list/index?type=backup")
  },
  // 立即还原
  reBack () {
    ability.redirectTo("/minePages/backup/list/index?type=download")
  },
  // 返回首页
  reLaunch () {
    // 获取页面层数
    let _pageLength = ability.getPages()?.length || 1;
    ability.navigateBack(_pageLength);
  },
  reTry () {
    // 返回上一页
    ability.navigateBack();
  }
})