// pages/batch/index.js
/*
 * 页面入口： 1，首页 批量控制窗帘和灯泡；（type=xxx） 2，房间管理删除设备(familyId=xxx&from=roomDetail&rid=xxx); 3，创建群组选择(rid=xxx&from=add)
*/
import bizUtil from "../../utils/bizUtil"
import ability from "../../utils/classDef/ability";
const app = getApp();
const devBehavior = require("../../static/js/behavior/device")
const groupBehavior = require("../../static/js/behavior/group")
Page({
  behaviors: [devBehavior, groupBehavior],
  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "批量控制"
      }
    },
    devList: [],
    selectData: {}, // key: 设备id, value:addr
    // batchCount: 0,
    isAllSelected: false, // 默认不选中 

    roomChanged: false, // 是否弹出选择房间
    roomList: [],
    roomInfo: {
      id: 0, // 默认全屋
      n: '全屋'
    },
    // 是否显示下拉图标
    showCollection: false,
    // 来源地址
    from: "",
    // 标题
    title: "",
    // 显示的数据品类
    type: "",

    isFloor: false, // 是否多楼层
    roomJson: {},

    // 用于家庭管理传入其他家庭用
    familyId: undefined,
    familyInfo: undefined, // 房间信息
    allDevs: undefined, // 家庭下所有设备信息

    bottomBtnName: "", // 底部按钮名称
    bottomBtnClass: "", // 底部按钮样式
    acts: [{
      text: "照明",
      value: "light",
      key: 1
    }, {
      text: "通用电机",
      value: "curtain",
      key: 2
    }],
    // defaultType: 0 // 默认选中的标题
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {

    console.log("batch options = ", options);

    // 初始化家庭和家庭下的设备数据
    this.initAllDataByFamily(options.familyId);

    // 来源
    let _from = options.from;
    this.setProperValue("from", _from);
    // 根据来源是否显示下拉
    this.setProperValue("showCollection", ['roomDetail', 'groupdel'].indexOf(_from) < 0);
    // 显示的设备品类
    this.data.type = options.type || "light";

    // 标题
    this.data.title = decodeURIComponent(options?.title || "");
    // 初始化标题
    this.initTitle(this.data.title, _from);
    // 初始化底部按钮名称
    this.initBottomBtnName(_from);

    // 初始化房间数据
    this.initRooms();

    // 房间id
    let _roomId = options.rid;
    // 如果传入了房间信息，则使用房间信息
    let _roomInfo = {};
    if (_roomId != undefined) {
      let _roomDetail = this.data.allDevs[_roomId || 0];
      _roomInfo.id = _roomDetail.id;
      _roomInfo.n = _roomDetail.n;
    } else {
      _roomInfo = app.globalData.roomInfo;
    }
    // 初始化默认选择的房间
    if (_roomInfo?.id != undefined) {
      this.data.roomInfo = _roomInfo;
      this.setData({
        roomInfo: this.data.roomInfo
      });
    }
    // 初始化数据
    this.initDevList(this.data.roomInfo?.id || 0);
  },

  // 根据家庭id取得数据
  initAllDataByFamily (familyId) {
    this.data.familyId = familyId;
    let _devs = undefined;
    let _familyInfo = undefined;
    if (familyId >= 0) {
      let _familyInfos = app.globalData.familys.filter(item=>item.id==familyId);
      _familyInfo = _familyInfos[0];
      _devs = app.globalData.allFamilysMap[familyId];
    }
    this.data.familyInfo = _familyInfo || app.globalData.familyInfo; // 房间信息
    this.data.allDevs = _devs || app.globalData.devs;
  },

  // 初始化标题
  initTitle (title, fromStr) {
    let _title = title == undefined ? "批量控制" : title;

    // ability.setNavigationBarTitle(_title || "");
    this.data.navObj.center.text = _title || "";

    // 如果是来自群组页面，则设置选中类型
    if (fromStr == "groupcreate") {
      let {text, value} = this.data.acts.filter(item=>item.value==this.data.type)[0]; // [this.data.defaultType || 0] || {};
      this.data.navObj.center.text = text;
      this.data.navObj.center.centerType = "selection";
      // this.data.type = value;
      this.data.navObj.center.hander = () => {
        console.log("hander");
        this.showDialog();
      };
    }
    this.setProperValue("navObj", this.data.navObj);
  },

  // 初始化底部按钮名称
  initBottomBtnName (from) {
    let _name = "";
    let _class = "";
    if (from == 'groupdel') {
      _name = '删除设备';
      _class = "colorFF0202 borderFF0202";
    } else if (from == 'groupadd') {
      _name = '保存';
    } else if (from == 'groupcreate') {
      _name = '创建群组';
    } else if (from == 'roomDetail') {
      _name = '删除';
      _class = "colorFF0202 borderFF0202";
    } else {
      _name = '批量控制';
    }
    this.setProperValue("bottomBtnName", _name);
    this.setProperValue("bottomBtnClass", _class);
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    // 清除临时group
    // app.globalData.tgroupId = undefined;
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },
  // 初始化房间
  initRooms () {
    
    // 所有设备
    let _allDevs = this.data.allDevs || {};
    // 当前楼层
    let _floor = this.data.familyInfo?.floor || 0;
    // 根据多楼层取得数据
    let _devs = bizUtil.getDevsByFloor(_allDevs, _floor);

    let {rooms} = bizUtil.getRoomsByDevs(_devs);

    // 所有房间数据
    let {roomJson} = bizUtil.getRoomsByDevs(_allDevs);
    this.data.isFloor = _floor >= 0;

    this.data.roomList = rooms;
    this.data.roomJson = roomJson;
    this.setData({
      roomJson: this.data.roomJson,
      roomList: rooms,
      isFloor: this.data.isFloor
    })
  },

  // 点击scrollview事件
  initDevList (id) {
    // 所有的设备数据
    let _allDevs = this.data.allDevs;
    // 选中的房间id
    let _roomId = parseInt(id) || 0;
    // 页面来源地址
    let _from = this.data.from;
    console.log("initDevList id, _from = ", id, _from);
    
    // 设备列表
    let _devList = [];
    // 如果是从群组设备删除进入，则
    if (_from == "groupdel") {
      _devList = this.getDevInfoByDids(app.globalData.groupInfo?.dids || []);
      console.log("initDevList groupdel _devList = ", _devList);
    } else {
      // 判断是否需要取得所有的设备列表
      if (_roomId == 0) {
        // 取得所有的设备数据
        _devList = bizUtil.getAllDevsFilter(_allDevs) || [];
      } else if (_roomId % 32 == 0) {
        // 取得楼层数据
        let _devJson = bizUtil.getDevsArrayByFloor(_allDevs, parseInt(_roomId / 32 - 1)) || {};
        console.log("getFuncList _devJson = ", _devJson);
        _devList = bizUtil.getAllDevsFilter(_devJson);
      } else {
        // 取得所有的设备数据
        _devList = bizUtil.getAllDevsFilter(_allDevs, _roomId) || [];
      }
      // 判断是否从房间管理详情页面进入
      if (_from == "roomDetail") {
        console.log("initDevList roomDetail before _devList = ", _devList);
        // 是否控制权限，控制权限不能显示双控设备
        let _isOnlyCtrl = app.globalData.roleNum == 2;
        _devList = _devList.filter(item => {
          // 去除空调内机
          let _isSubAircondition = !bizUtil.isAirconditionHost(item);
          if (_isSubAircondition) {
            return false;
          }
          // 控制权限
          if (_isOnlyCtrl) {
            let _isDouble = bizUtil.getDevTypeStr(item.t) == 'virtualPanel';
            if (_isDouble) {
              return false;
            }
          }
          return true;
        }); // bizUtil.delSubAirconditionInArray(_devList);
        console.log("initDevList roomDetail after _devList = ", _devList);
      } else {
        // 设备类型
        let _type = this.data.type;
        // 取得照明数据
        _devList = (bizUtil.getAllPidByArray(_devList) || {})[_type] || [];
        console.log("initDevList _devList = ", _devList);
        // 如果是固定群组，则窗帘版本需要大于等于21才能加入
        if (_from && _from.indexOf("group") == 0) {
          if (_type == "curtain") {
            // 来源页面是否以group开头，是的话则判断窗帘
            _devList = _devList.filter(item=>bizUtil.getVersionByV(item.v) >= 21);
          } else if (_type == "light") {
            // 来源页面是否以group开头，是的话则判断窗帘
            _devList = _devList.filter(item=>bizUtil.getVersionByV(item.v) >= 54);
          }
        }
        // 判断群组添加
        if (_from == "groupadd") {
          let _dids = app.globalData.groupInfo?.dids || [];
          console.log("initDevList _dids = ", _dids);
          // 如果选中的是群组设备添加，则需要去除did列表
          _devList = _devList.filter(item=>_dids.indexOf(item.d)<0);
          console.log("initDevList _devList = ", _devList);
        } 
      }
    }
 
    // 数据排序
    let _devSortList = [];
    // 增加特殊处理：添加过群组的设备显示在后面，未添加过的显示在前面
    if (_from == "groupadd" || _from == "groupcreate") {
      // 所有设备d对应的gid数组数据
      let _dGroupMap = bizUtil.getGroupDMapInJson(_allDevs);
      console.log("initDevList _allDevs, _dGroupMap = ", _allDevs, _dGroupMap);
      // 取得处理后的数据
      _devSortList = this.dealArrayByGrouped(_dGroupMap, _devList);
    } else {
      _devSortList = bizUtil.sortArrayByChar(_devList, 'n') || [];
    }
    console.log("initDevList _devSortList = ", _devList, _devSortList);
    
    // 设置设备的图标
    _devSortList = _devSortList.map(item => {
      // 图片类型
      item.tStr = item.tStr || bizUtil.getDevImgByT(item);
      return item;
    })

    // 设置房间选择
    this.setProperValue("devList", _devSortList);

    // 判断是否全选
    let _selectData = this.data.selectData;
    console.log("_selectData = ", _selectData, _devSortList);
    this.setAllSelected(Object.keys(_selectData).length > 0 && _devSortList.filter(item=>_selectData[item.d]==undefined).length == 0);
  },

  // 增加特殊处理：添加过群组的设备显示在后面，未添加过的显示在前面
  dealArrayByGrouped (dGroupMap, devList) {
    // 如果没有设备，则返回空
    if (!devList || devList.length == 0) {
      return [];
    }
    // 如果没有群组则返回原先数据  || (Object.keys(dGroupMap)?.length == 0)
    if (!dGroupMap) {
      return bizUtil.sortArrayByChar(devList || [], 'n');
    }
    let _inGroup = []; // 群组中
    let _outGroup = []; // 未在群组中
    console.log("dealArrayByGrouped dGroupMap = ", dGroupMap);
    // 遍历数据
    for (let item of devList) {
      let {d} = item;
      let _groupLen = dGroupMap[d]?.length || 0;
      item.groupLen = _groupLen; // 保存群组个数
      if (_groupLen > 0) {
        // 存在
        _inGroup.push(item);
      } else {
        _outGroup.push(item);
      }
    }
    // 分别排序群组中数据和未在群组中数据
    return bizUtil.sortArrayByChar(_outGroup || [], 'n').concat(bizUtil.sortArrayByChar(_inGroup || [], 'n'));
  },

  // 根据did取得所有设备详细
  getDevInfoByDids (dids) {
    // 取得房间下所有的d对应的详细信息
    let _didMaps = bizUtil.getDidMapInJson(this.data.allDevs);
    // 群组设备列表
    let _groupDevs = dids || [];
    // 群组设备列表详细
    let _groupDevList = _groupDevs.map(did => _didMaps[did]);
    console.log("initGroupDevs _groupDevList = ", _groupDevList);
    return _groupDevList;
  },

  getBatchLen (obj, count) {
    let _length = Object.keys(obj || {}).length;
    // if ((_length + count) > 5) {
    //   ability.showToast({
    //     title: "最多选择五个设备"
    //   });
    //   return false;
    // }
    return _length + count;
  },
  deviceDetail (e) {
    let _itemIdx = e.currentTarget.dataset?.index || 0;
    // 保存当前数据
    let _deviceInfo = this.data.devList[_itemIdx];
    // 如果房间id为空则返回
    if (!_deviceInfo || _deviceInfo.id == undefined) {
      console.error("_deviceInfo or id isnot exist");
      return;
    }
    // 判断是否批量控制时点击
    this.dealwithBatch(_deviceInfo);
    // 判断是否全部选中 // 设置为全选或者不选
    let _selectCount = Object.keys(this.data.selectData).length;
    let _allLenth = this.data.devList.length;
    console.log("_selectCount = ", _selectCount, _allLenth);
    if (_allLenth == _selectCount) {
      this.setAllSelected(true)
    } else {
      this.setAllSelected(false)
    }
  },
  // 批量选择
  dealwithBatch (deviceInfo) {
    if (!deviceInfo) {
      return;
    }
    let _did = deviceInfo.d;
    let _value = this.data.selectData[_did];
    // 至多选择五个设备
    let _length = this.getBatchLen(this.data.selectData, _value != undefined ? -1 : 1);
    if (_length === false) {
      return;
    }
    // 处理
    if (_value !== undefined) {
      delete this.data.selectData[_did];
    } else {
      this.data.selectData[_did] = deviceInfo.a;
    }
    this.setData({
      selectData: this.data.selectData
    })
  },

  setAllSelected (isSelected) {
    console.log("setAllSelected isSelected = ", isSelected);
    // 设置是否选中
    this.data.isAllSelected = isSelected;
    this.setData({
      isAllSelected: this.data.isAllSelected
    })
  },

  allSelect () {
    let _isSelected = this.data.isAllSelected;
    // 设置是否选中
    this.setAllSelected(!_isSelected);
    // 设置选中的icon
    if (_isSelected) {
      this.data.selectData = {};
    } else {
      let _collects = {};
      for (let item of this.data.devList) {
        _collects[item.d] = item.a;
      }
      this.data.selectData = _collects;
    }
    this.setData({
      selectData: this.data.selectData
    })
  },
  // 删除前的对话框提示
  showDelModal (devs) {
    console.log("showDelModal devs = ", devs);
    wx.showModal({
      title: "",
      content: "是否确认删除设备？",
      showCancel: true,
      confirmText: "是",
      cancelText: "否",
      cancelColor: '#000000'
    }).then(result => {
      result?.confirm && (this.delDevs(devs));
    })
  },
  // 批量删除双控设备
  delVirtualPanelByArray (array, complete) {
    if (!array || array.length == 0) {
      return;
    }
    // 双控短地址集合
    let _addrArray = array.map(item => item.a);
    // 家庭密钥
    let _bleCode = this.data.familyInfo?.bleCode;
    console.log("delVirtualPanelByArray _addrArray = ", _addrArray);
    this.delVirtualPanelCmd(_addrArray, 0, complete, _bleCode);
  },
  delVirtualPanelCmd (virtualAddrArray, index, complete, blecode) {
    let _addr = (virtualAddrArray || [])[index];
    if (_addr == undefined) {
      !!complete && (complete(0));
      return;
    }
    app.globalData.blsdk.unbindDeviceWithAddr(0, _addr, (rescode) => {
      if (rescode != 0) {
        !!complete && (complete(rescode));
        return;
      }
      this.delVirtualPanelCmd(virtualAddrArray, index + 1, complete, blecode);
    }, {
      bleCode: blecode
    })
  },
  // 批量删除设备
  delDevsDetail (devs, virtualExist) {
    // 删除设备
    this.delDeviceByInfo(devs, () => {
      // 等待条
      !!virtualExist && (ability.hideLoading());
      // 返回首页，重新加载
      setTimeout(() => {
        // 返回上一页
        ability.navigateBack();
      }, 1000);
    }, this.data.allDevs, this.data.familyInfo);
    return;
  },
  // 批量删除设备
  delDevs (devs) {
    if (!devs || devs.length == 0) {
      return;
    }
    console.log("delDevs devs = ", devs);
    // 过滤出多少个双控设备
    let {virtualPanel} = bizUtil.getAllPidByArray(devs);
    let _virtualExist = virtualPanel?.length > 0;
    if (_virtualExist) {
      // 如果有双控设备，则判断是否蓝牙可用
      let _isBluetoochAvalible = app.globalData.blsdk.isBleAvailable();
      if (!_isBluetoochAvalible) {
        return ability.showToast({
          title: "包含双控设备，请打开蓝牙"
        })
      }
      // 等待条
      ability.showLoading({});
      // 批量删除双控设备
      this.delVirtualPanelByArray(virtualPanel, (rescode) => {
        if (rescode != 0) {
          ability.hideLoading();
          return ability.showToast({
            title: "删除失败"
          });
        }
        this.delDevsDetail(devs, true);
      });
      return;
    }
    this.delDevsDetail(devs);
    // // 是否删除了网关
    // let _gatewayDel = false;
    // // 需要计算是否有执行设备和触发设备的
    // let _ds = [];
    // let _devTypeObjs = {};
    // // 所有设备集合
    // let _allDevs = bizUtil.getAllDevsFilter(this.data.allDevs);
    // // 遍历数据
    // for (let devInfo of devs) {

    //   let {t, d, sub} = devInfo;

    //   // 删除群组下的设备信息
    //   let _devTypeStr = bizUtil.getDevTypeStr(t);

    //   // 当前只支持灯泡和窗帘的群组
    //   if (_devTypeStr == "light" || _devTypeStr == "curtain") {
    //     bizUtil.delGroupDByDev(app.globalData.devs, devInfo);
    //   }

    //   // 加入设备信息列表
    //   _ds.push(d);

    //   // 设备类型
    //   if (bizUtil.isActionDev(t)) {
    //     _devTypeObjs["devs"] = true;
    //   } else if (bizUtil.isPanelDev(t)) {
    //     _devTypeObjs["panels"] = true;
    //   }
    //   // 删除设备列表中的设备数据
    //   let _delGataway = this.delDevAndStatus(devInfo);
    //   if (!!_delGataway) {
    //     _gatewayDel = _delGataway;
    //   }

    //   // 如果是空调网关,则需要删除下面的子设备
    //   if (_devTypeStr == "aircondition" && sub > 0) {
    //     // 取得空调子设备
    //     let _subs = bizUtil.getAirconditionsByDev(_allDevs, devInfo);

    //     _ds = _ds.concat(_subs.map(item => item.d));
    //     // // 删除场景下面触发设备
    //     // for (let i = 1; i <= sub; i++) {
    //     //   _ds.push(d + "_" + i);
    //     // }
    //     // 删除网关下的子设备
    //     // let _subs = _allDevs.filter(item => _ds.indexOf(item.d) >= 0);
    //     console.log("_subs = ", _subs);
    //     for (let _subInfo of _subs) {
    //       this.delDevAndStatus(_subInfo);
    //     }
    //   }
    // }
    // // 删除场景下的设备
    // bizUtil.moveDev(this.data.allDevs, {
    //   t: Object.keys(_devTypeObjs),
    //   d: _ds
    // }, []);
    // // 判断是否有网关数据
    // _gatewayDel && (this.data.familyInfo?.id == app.globalData.familyInfo?.id) && (this.updateNewGateway());
    
    // // 上报 保存设备状态 提示成功等
    // this.delDevLast();
  },
  // 返回是否删除了当前网关
  delDevAndStatus (devInfo) {
    if (!devInfo) {
      return false;
    }
    let _devInfo = devInfo;
    let _id = _devInfo.id;
    let _d = _devInfo.d;
    let _t = _devInfo.t;
    let _devs = (this.data.allDevs[_id] || {}).devs || [];
    console.log("delDev _devs, _id = ", _devs, _id);
    // 删除设备信息
    let _idx = _devs.findIndex(item=>item.d == _d);
    console.log("delDev _idx = ", _idx);
    (_idx >= 0) && _devs.splice(_idx, 1);

    // 判断是否有设备状态
    let _allStatus = app.globalData.allStatus || {};
    if (_allStatus[_devInfo.d + "_" + _devInfo.a]) {
      delete _allStatus[_devInfo.d + "_" + _devInfo.a];
    }
    
    // 判断删除的是否网关，如果不是，则忽略
    let _devTypeStr = bizUtil.getDevTypeStr(_t);
    if (_devTypeStr == "gateway") {
      // 先判断删除的是否当前的网关，如果是，则清除
      // 取得当前的网关
      let _gatewayInfo = app.globalData.blsdk.getCurrentGateway() || {};
      console.log("delDev _gatewayInfo, _d = ", _gatewayInfo, _d);
      if (_gatewayInfo.d == _d) {
        if (this.data.familyInfo?.roleNum != 2) {
          // 删除原先的网关数据
          app.deletePrivateData();
          // return true;
        }
        // 如果控制权限删除网关，则只需要更新sdk状态
        app.globalData.blsdk.updateGatewayInfo();

        return this.data.familyInfo?.roleNum != 2;
      }
    }
    return false;
  },
  delDevLast () {
    // 保存设备状态数据
    app.setBizStorage("devStatus", app.globalData.allStatus);
    // 保存设备信息
    app.saveAllDevsMap();
    // 上报数据
    (this.data.familyInfo?.id == app.globalData.familyInfo?.id) && (app.uploadPrivateData());
    
    // 删除提示
    this.delSuccessToast();
  },
  // 删除成功的提示
  delSuccessToast () {
    // 提示删除结果
    ability.showToast({
      title: "删除成功",
      icon: "success",
      mask: true,
      complete: () => {
        // 返回首页，重新加载
        setTimeout(() => {
          // 返回上一页
          ability.navigateBack();
        }, 1200);
      }
    }) 
  },
  // 找到新的网关信息
  updateNewGateway () {
    // 找是否有下一个网关
    let _newGateway = bizUtil.getGatewayInfo(this.data.allDevs);
    console.log("_newGateway = ", _newGateway);
    // 更新设备版本
    app.globalData.blsdk.updateGatewayInfo(_newGateway);
  },
  // 批量控制
  bottomBtnClick () {
    console.log("bottomBtnClick this.data.selectData = ", this.data.selectData);
    // 取得批量控制的addr
    let {addrs, vs, devs} = this.getBatchAddr(this.data.selectData, this.data.devList);
    let _len = devs.length;
    if (_len == 0) {
      return ability.showToast({
        title: "请选择设备"
      });
    }

    // 如果是从详情页面跳转，则处理
    let _from = this.data.from;
    if (_from == "roomDetail") {
      return this.showDelModal(devs);
    } else if (_from == "groupcreate") {
      return this.createFixedGroup(devs);
    } else if (_from == "groupdel") {
      return this.delGroupDidByDevs(devs);
    } else if (_from == "groupadd") {
      let _groupInfo = app.globalData.groupInfo || {};
      // let {gid, dids} = _groupInfo;
      return this.fixedGroupSendByDevs(_groupInfo, devs, _groupInfo.dids || [], (dids) => {
        console.log("fixedGroupSendByDevs dids = ", dids);
        // 赋值
        app.globalData.groupInfo.dids = dids || [];
        // 保存数据
        app.saveWhenHide();
        // 返回上一页
        !!this.back && (this.back());
      }, {
        sendTime: 3000
      });
    }

    // 保存房间信息
    let {id, n} = this.data.roomInfo;
    app.globalData.roomInfo = app.globalData.roomInfo || {};
    app.globalData.roomInfo.id = id || 0;
    app.globalData.roomInfo.n = n || '';

    if (_len == this.data.devList.length) {
      console.log("全屋照明 app.globalData.roomInfo = ", app.globalData.roomInfo);
      // 全屋照明
      app.globalData.groupIds = [];
      app.globalData.devInfo = undefined;
      app.globalData.tgroupId = undefined;
    } else {
      // 判断是否都支持临时group 默认支持
      let _supportTemp = true;

      for (let v of vs) {
        if (!v) {
          _supportTemp = false;
          break;
        }
        let {tempGroup} = bizUtil.getJsonByAblitys(v) || {};
        // 如果不支持，则退出
        if (!tempGroup) {
          _supportTemp = false;
          break;
        }
      }
      // _supportTemp = false;
      console.log("bottomBtnClick _supportTemp = ", _supportTemp);
      // 如果支持临时group
      if (_supportTemp) {
        // 等待条
        ability.showLoading({});
        // 发送临时group
        let _tGroupId = app.globalData.blsdk.createTGroupId();
        app.globalData.blsdk.tempGroupCtrl(_tGroupId, devs, {}, (res) => {
          if (res != 0) {
            if (typeof res != "object") {
              ability.showToast({
                title: "临时组下发失败"
              })
            }
            return;
          }
          // 临时group
          app.globalData.groupIds = []; // 之前用于短地址集合
          app.globalData.tgroupId = _tGroupId;
          app.globalData.devInfo = undefined;
          // 保存批量控制的数组
          app.globalData.tmpData = app.globalData.tmpData || {};
          app.globalData.tmpData.tdevs = devs || [];
          
          // 跳转页面
          // setTimeout(() => {
            ability.hideLoading();
            // 跳转页面
            this.jumpControlPage();
          // }, 330);
        });
        return;
      } else {
        console.log("bottomBtnClick addrs = ", addrs);
        // 批量照明
        app.globalData.groupIds = addrs;
        app.globalData.tgroupId = undefined;
        app.globalData.devInfo = undefined;
        // 保存批量控制的数组
        app.globalData.tmpData = app.globalData.tmpData || {};
        app.globalData.tmpData.tdevs = devs || [];
      }
    }

    // 跳转页面
    this.jumpControlPage();
  },
  // 删除群组中的设备
  delGroupDidByDevs (delDevs) {
    ability.showModal({
      title: "",
      content: "确定从群组中移除选择设备？",
      showCancel: true,
      confirmText: "确定",
      confirmColor: "#F78D3F",
      cancelText: "取消",
      cancelColor: '#000000'
    }).then(result => {
      if (!result) {
        return;
      }
      // 群组信息
      let _groupInfo = app.globalData.groupInfo || {};
      // 当前设备集合
      let _devList = this.data.devList;
      // 删除的d集合
      let _delDs = delDevs.map(item=>item.d);
      // 新存在的设备
      let _newDevInfos = _devList.filter(item=>_delDs.indexOf(item.d)<0);
      // 删除设备
      this.delGroupDevs(_groupInfo, _delDs, () => {
        // 重新加载
        this.showDelSuccessToast();
        // 保存数据
        app.saveWhenHide();
        // 刷新数据
        this.setData({
          devList: _newDevInfos
        })
      }); // , _isDelAll
    })
  },
  // 删除成功的提示
  showDelSuccessToast () {
    // 提示删除结果
    ability.showToast({
      title: "删除成功",
      icon: "success",
      mask: true,
      // complete: () => {}
    })  
  },
  // // 添加群组中的设备
  // fixedGroupSendByDevs (groupInfo, devs, extendDids, callback) {
  //   if (!devs || devs.length == 0) {
  //     return this.back();
  //   }
  //   let _newDids = devs.map(item=>item.d);
  //   // 加入到群组中显示
  //   let _allDids = (extendDids || []).concat(_newDids);
  //   // 去重
  //   let _allDidsByFilter = Array.from(new Set(_allDids));
  //   // 取得所有的设备信息
  //   let _allDevs = this.getDevInfoByDids(_allDidsByFilter, );
  //   // 下发数据
  //   this.setGroupDevsByNewDevs(groupInfo, _allDevs, (result) => {
  //     console.log("fixedGroupSendByDevs result = ", result);
  //     !!callback && (callback(_allDidsByFilter));
  //   });
  // },
  // // 发送群组数据
  // setGroupDevsByNewDevs (groupInfo, devs, callback, configs) {
  //   // 显示等待条
  //   ability.showLoading({});
  //   // 下发数据
  //   app.globalData.blsdk.fixedGroupCtrl(groupInfo?.gid, devs || [], {
  //     noRemote: true,
  //     isDelete: configs?.isDelete || false // 是否删除设备
  //   }, (res) => {
  //     console.log("setGroupDevsByNewDevs res = ", res);
  //     // 隐藏等待条
  //     ability.hideLoading();
  //     // 判断是否发送成功
  //     if (res != 0) {
  //       return ability.showToast({
  //         title: "下发数据失败"
  //       });
  //     } else {
  //       // 跳转页面
  //       !!callback && (callback());
  //     }
  //   });
  // },
  // // 创建群组
  // createFixedGroup (devs) {
  //   console.log("createFixedGroup devs = ", devs);
  //   // 取得家庭下所有的固定群组列表
  //   let _groups = bizUtil.getAllDevsFilter(app.globalData.devs, undefined, "groups");
  //   console.log("createFixedGroup _groups = ", _groups);
  //   let _id = bizUtil.getIdByArray(_groups, "gid", 193, 253);
  //   if (_id < 0) {
  //     return ability.showToast({
  //       title: "固定群组已满，请删除后再试"
  //     });
  //   }
  //   // 群组信息
  //   let _groupInfo = {
  //     gid: _id,
  //     rid: _roomId,
  //     t: this.data.type || "light", // _type, // 默认照明
  //     n: this.data.title,
  //     dids: dids
  //   };
  //   // 根据设备数组发送固定组数据
  //   this.fixedGroupSendByDevs(_groupInfo, devs, [], (dids) => {
  //     // 房间id
  //     let _roomId = this.data.roomInfo?.id || 0; // app.globalData.roomInfo?.id || 0;
  //     app.globalData.devs[_roomId].groups = app.globalData.devs[_roomId]?.groups || [];
  //     // 加入显示
  //     app.globalData.devs[_roomId].groups.push(_groupInfo);
  //     // 保存
  //     app.saveAllDevsMap();
  //     // 提示创建成功
  //     ability.showToast({
  //       title: "创建群组成功",
  //       icon: "success"
  //     });
  //     setTimeout(() => {
  //       ability.navigateBack();
  //     }, 1200);
  //     /* 
  //     LIGHT = 1, //照明设备类型
  //     CURTAIN, //窗帘设备类型
  //     SWITCH, //开关设备类型
  //     PANNEL, //⾯板设备类型
  //     SENSOR, //传感器设备类型
  //     GATEWAY */
  //   });
  // },
  jumpControlPage () {
    // 固定群组没有
    app.globalData.groupInfo = undefined;
    // 判断设备类型跳转到不同页面
    if (this.data.type == "curtain") {
      return ability.navigateTo('../../device/curtain/control/index')
    }
    // 如果是从首页进入，则需要使用redirectTo,因为被提了一个bug(批量控制页面，返回应该直接返回到首页)
    if ("deviceList" == this.data.from) {
      return ability.redirectTo('../../device/light/control/index');
    }
    ability.navigateTo('../../device/light/control/index')
  },
  getBatchAddr (selectMap, devList) {
    // 这里有个问题： 如果切换房间之后，需要重新取得选中的设备，防止设备不在本房间
    let _devList = devList || [];
    let _result = {
      addrs: [],
      vs: [],
      devs: []
    };
    // 遍历可见的列表数据
    for (let item of _devList) {
      // 如果此设备的id被选中，则加入计数
      let _itemA = selectMap[item.d];
      if (_itemA >= 0) {
        _result.addrs.push(_itemA);
        _result.vs.push(item.v);
        _result.devs.push(item);
      }
    }
    return _result;
  },

  chooseRoom (e) {
    let {rid} = e.currentTarget.dataset;
    let _roomInfo = this.data.allDevs[rid || 0];
    this.data.roomInfo.n = _roomInfo.n;
    this.data.roomInfo.id = _roomInfo.id;
    this.setData({
      roomInfo: this.data.roomInfo
    });
    // 隐藏
    this.hidePop();
    // 更新列表
    this.initDevList(this.data.roomInfo.id);
  },
  showPop () {
    // 如果不显示下拉条，则屏蔽显示弹出框
    if (!this.data.showCollection) {
      return;
    }
    this.setProperValue("roomChanged", true);
  },
  hidePop () {
    this.setProperValue("roomChanged", false);
  },
  // 显示dialog，用于显示切换的品类
  showDialog() {
    this.setProperValue("showTypesheet", true);
  },
  setProperValue (key, val) {
    if (this.data == undefined) {
      return;
    }
    this.data[key] = val;
    this.setData({
      [key]: val
    })
  },
  // 切换显示的品类
  typeClick (e) {
    let {index} = e.detail || {};
    let {value, text} = this.data.acts[index];
    console.log("typeClick value, text, this.data.type = ", value, text, this.data.type);
    // 判断之前的类型与现在的是否一致，如果不一致，则清空数据
    if (value != this.data.type) {
      this.data.selectData = {};
    }
    // 更新属性
    this.setProperValue("type", value);
    // 重新加载列表数据
    this.initDevList(this.data.roomInfo.id);
    // 更新标题
    this.initTitle(text);
    // 隐藏actionsheet
    this.setProperValue("showTypesheet", false);
  }
})