// miniprogram/pages/deviceList/index.js
import ability from "../../utils/classDef/ability";
import bizUtil from "../../utils/bizUtil"
import GROUPS from '../../static/js/jsConfig/groups';

// 获取应用实例
var app = getApp()
var tabBehavior = require('../../static/js/behavior/tab')
const CustomPage = require("../../static/js/baseClass/page")

new CustomPage({
  onShareAppMessage: function () {
    return {
      title: app.globalData.companyInfo?.wxName,
      path: '/pages/deviceList/index'
    }
  },
  // 页面卸载
  onUnload () {
    // 销毁 关闭蓝牙监听器
    !!app.globalData.blsdk && (app.globalData.blsdk.destorySDK());
  },
  behaviors: [tabBehavior],
  data: {
    // 顶部条
    topConfig: {
      navObj: {
        center: {
          text: app.globalData.companyInfo.wxName
        }
      },
      // 内容头部的配置
      topObj: {
        left: {
          hander: undefined,
          text: '',
          titleClass: 'color333 font48 semibold',
          tipClass: 'color666 font26',
          tip: '欢迎来到你的全屋智能',
          bottomIcon: true
        }
      }
    },
    isEmpty: false, // 全部设备是否未空
    // 是否主权限
    roleNum: 0,
    // 房间列表
    rooms: {
      data: [],
      selRoomId: 0, // 选中的房间 默认全屋
      selRoomIndex: 0,
      handle: undefined
    },
    roomJson: {}, // json房间数据
    bleUnavailable: false, // 蓝牙不可用标识
    
    supportList: [], // 支持的设备列表

    showAddDialog: false, // 默认隐藏
    promptConfig: {
      showPrompt: false, // 默认不编辑设备名称
      devName: "", // 设备名称
    },

    isFloor: false, // 是否支持多楼层

    isRoomEmpty: false, // 房间是否为空

    showRemote: false, // 是否需要远程调试
    remoteConfig: {
      title: "网关远程调试已开启"
    }
  },
  detached() {
    this.releasedPage();
  },
  ready () {
  },
  onLoad () {
    // 初始化scrollview点击事件
    this.initScrollView();

    // 处理分享的二维码
    let _shareStr = app.globalData.shareStr;
    // _shareStr=true，说明还需要再配置
    if (!_shareStr || _shareStr === true) {
      return;
    }
    // 初始化房间json
    this.initRoomJsonByFloorAndAllDevs();
    // 处理扫码设备
    this.scanCodeDetail(_shareStr);
    // 重新恢复为空的二维码
    app.globalData.shareStr = "";
  },
  onShow () {
    // 初始化
    this.initShowMethod();
    // 发送校时命令
    this.sendUpdateTime();

    // 判断是否开启远程调试
    this.dealRemote();
  },
  // 校时数据下发
  sendUpdateTime () {
    // // 如果是控制权限，不下发校时数据
    // let _roleNum = app.globalData.familyInfo?.roleNum;
    // if (_roleNum == 2) {
    //   return;
    // }
    // 取得蓝牙可用状态
    let _blueStatus = app.globalData.blsdk.isBleAvailable();
    console.log("sendUpdateTime _blueStatus = ", _blueStatus);
    // 如果没有校时方法
    if (!(app.globalData.blsdk?.updateDevTime) || !_blueStatus) {
      return;
    }
    // 下发校时数据
    app.globalData.blsdk.updateDevTime({
      showDialog: false // 不需要超时的提示
    });
  },
  initShowMethod () {
    // 初始化房间json
    this.initRoomJsonByFloorAndAllDevs();
    // 判断用户状态 取得上次保存的数据
    this.initDataByRole(app.globalData.roleNum);
  },
  
  // 禁止下拉
  customEvent () {
    console.log("customEvent");
    ability.stopPullDownRefresh();
  },
  sliderChange (e) {
    let {index, id} = e.detail || {};
    console.log("sliderChange index, id = ", index, id);
    this.scrollClick(id)
  },
  // 初始化scrollview点击事件
  initScrollView () {
    this.data.rooms.handle = (index, id) => {
      console.log("initScrollView handle index = ", index, id);
      this.scrollClick(id)
    }
  },
  
  // 更新房间信息
  updateRoomInfo (roomData, roomId) {
    // 保存选中的房间 这里有个bug 不能按照序号来
    let _roomData = roomData || [];
    let _findIndex = _roomData.findIndex(item => item.id == roomId);
    app.globalData.roomInfo = (_findIndex >= 0) ? _roomData[_findIndex] : _roomData[0];

    // 临时保存下标
    this.data.rooms.selRoomIndex = _findIndex;
  },
  // 面板的特殊处理：根据设备t取得面板的按键个数
  dealPanelByArray (array) {
    if (!array || array.length == 0) {
      return;
    }
    // 面板数据需要特殊处理，
    for (let item of array) {
      item.tStr = bizUtil.getDevImgByT(item);
    }
  },
  dealUartByArray (array) {
    // 取得透传数据中的超级面板数据和透传数据
    let _result = {
      uarts: [],
      metaPads: []
    };
    if (!array) {
      return _result;
    }
    for (let i = 0, len = array.length; i < len; i++) {
      let _devInfo = array[i] || {};
      let _abilityArray = bizUtil.getVersionArrayByV(_devInfo?.v);
      if (_abilityArray.join("") == 0) {
        // 透传设备
        _result.uarts.push(_devInfo);
      } else {
        // 超级面板
        _result.metaPads.push(_devInfo);
      }
    }
    return _result; 
  },
  // 取得需要显示的名称
  getRoomName (devInfo) {

    return bizUtil.getRoomName(devInfo, this.data.roomJson, this.data.isFloor);

    // if (!devInfo) {
    //   return "";
    // }
    // let isFloor = this.data.isFloor;
    // let roomJson = this.data.roomJson;
    // let _id = devInfo?.rid || devInfo?.id || 0;
    // return ((_id % 32 == 0) || !isFloor) ? (roomJson[_id]?.n) : (roomJson[_id + 32 - _id % 32]?.n + ' | ' + roomJson[_id]?.n)
  },

  // 点击scrollview事件
  scrollClick (id) {
    // 选中的房间id
    let _roomId = parseInt(id) || 0;
    this.data.rooms.selRoomId = _roomId;
    // console.log("scrollClick _roomId, roomInfo = ", _roomId, app.globalData.roomInfo);
    // 更新globalData中的房间信息
    this.updateRoomInfo(this.data.rooms?.data, _roomId);
  
    // 数据集合
    let {devs: _array, groups: _groupList} = bizUtil.getDevsAndGroupsByRoomId(_roomId, app.globalData.devs);
    // 特殊处理 一共显示1000个设备
    _array = _array.slice(0, 1000);
    // 先把_array排序好 后面的就不用排序了
    _array = bizUtil.sortArrayByChar(_array, "n");
    let _groups = bizUtil.sortArrayByKey(bizUtil.getFilterGroups(_groupList, _roomId), "gid");
    // console.log("scrollClick _groups = ", _groups);

    // 增加tStr字段，用于图标  roomTip用于显示房间数据
    _array.forEach(item => {
      item.tStr = bizUtil.getDevImgByT(item);
      item.roomTip = this.getRoomName(item);
    })
    _groups.forEach(item => {
      item.tStr = bizUtil.getDevImgByT(item);
      item.roomTip = this.getRoomName(item);
      item.id = item.rid || item.id || 0; // 特殊处理 给群组增加一个id字段
    });
    
    // 设备列表是否为空
    let _isEmpty = _array.length + _groups.length;

    console.log("scrollClick _array, temphumidity = ", _array, temphumidity);
    // 取得设备的品类
    let {light, panel, curtain, fanLight, gateway, doorsensor, bodysensor, watersensor, uart, aircondition, radarsensor, thermostat, fan, virtualPanel, coathanger, powerPlug, tvLamp, temphumidity, powerstrip, sossensor, panelsensor, commonMotor} = bizUtil.getAllPidByArray(_array);
    // 取得品类对应的群组
    let {light: lightGroup, curtain: curtainGroup} = bizUtil.getAllPidByArray(_groups);
    console.log("scrollClick _groups = ", _groups);
    
    // 透传设备的特殊处理: 超级面板和透传共用一个t
    let {uarts, metaPads} = this.dealUartByArray(uart);

    // 群组个数不计算在设备总数中，所以需要额外处理群组的个数
    let _lightCount = light?.length || 0;
    let _curtainCount = curtain?.length || 0;
    let _airconditionCount = (aircondition?.length || 0) + (thermostat?.length || 0); // (aircondition.filter(item => !item?.sub))?.length || 0; // 空调个数
    // console.log("scrollClick aircondition = ", aircondition, thermostat, thermostat?.length || 0);

    console.log("scrollClick lightGroup, light = ", lightGroup, light);
    let _lights = lightGroup.concat(light); // 照明设备和群组
    let _curtains = curtainGroup.concat(curtain); // 窗帘和群组

    // 这里需要区分开关面板还是情景面板，超级面板归属于情景面板
    let _allPanels = panel.concat(metaPads); // 所有面板
    let {pwrs, panels} = this.getPwrAndPanelByArray(_allPanels);
    let _panels = panels.concat(powerPlug || []).concat(panelsensor || []);
    let _pwrPanels = bizUtil.sortArrayByKey(virtualPanel || [], "a").concat(pwrs);

    // 生活电器 晾衣架和风扇 电视氛围灯
    let _fans = (fan || []).concat(coathanger || []).concat(tvLamp || []); // 传感器

    let _allSensors = (doorsensor || []).concat(bodysensor || []).concat(watersensor || []).concat(radarsensor || []).concat(temphumidity || []).concat(sossensor || []);
    
    // 插排
    let _powerstrips = powerstrip || [];
    // 通用电机
    let _commonMotor = commonMotor || [];
    // let _sensors = // bizUtil.sortArrayByChar(_allSensors, "n"); // 传感器
    // this.data.uarts = uarts; // 透传模块

    // 空调处理 1，先根据空调网关的名称排序；2，再根据
    let _airconditions = bizUtil.dealAirconditionSort(aircondition); // 中央空调网关
    let _thermostat = _airconditions.concat(thermostat || []);

    // 设备列表配置项
    let _supportList = ([{
      title: "照明 " + _lightCount,
      key: "light",
      rightBtns: [{
        imgStr: "/static/img/devList/icon_batch.png",
        text: "批量控制",
        classStr: "lightActionImg marginR9"
      }],
      list: _lights
    }, {
      title: "窗帘 " + _curtainCount, // 不计算群组
      key: "curtain",
      rightBtns: [{
        imgStr: "/static/img/devList/icon_batch.png",
        text: "批量控制",
        classStr: "lightActionImg marginR9"
      }],
      list: _curtains
    }, {
      title: "情景面板 " + (_panels?.length || 0), // 不计算群组
      key: "scenePanel", // 面板需要区分开关还是情景面板
      realKey: "panel",
      // rightBtn: [],
      list: _panels
    }, {
      title: "开关面板 " + (_pwrPanels?.length || 0),
      key: "pwrPanel", // 面板需要区分开关还是情景面板
      realKey: "panel",
      rightBtns: [{
        imgStr: "/static/img/devList/adddouble.png",
        text: "按键同步",
        classStr: "size24 marginR12"
      }],
      list: _pwrPanels
    }, {
      title: "风扇灯 " + (fanLight?.length || 0),
      key: "fanLights",
      // rightBtn: [],
      list: fanLight
    }, {
      title: "传感器 " + (_allSensors?.length || 0),
      key: "sensor", // 这里是所有传感器的集合 包括人体、门磁，水浸，雷达等
      // rightBtn: [],
      list: _allSensors
    }, {
      title: "温控器 " + (_airconditionCount || 0),
      key: "thermostat", // 这里是所有温控器的集合 包括空调和温控器
      // rightBtn: [],
      list: _thermostat
    }, {
      title: "生活家电 " + (_fans?.length || 0),
      key: "fan", // 这里是所有生活家电的集合
      // rightBtn: [],
      list: _fans
    }, {
      title: "插座 " + (_powerstrips?.length || 0),
      key: "powerstrip", // 这里是所有插排的集合
      // rightBtn: [],
      list: _powerstrips
    }, {
      title: "电机 " + (_commonMotor?.length || 0),
      key: "commonMotor", // 这里是通用电机的集合
      // rightBtn: [],
      list: _commonMotor
    }, {
      title: "网关 " + (gateway?.length || 0),
      key: "gateway",
      list: gateway
    }, {
      title: "透传设备 " + (uarts?.length || 0),
      key: "uart", // 这里是所有传感器的集合
      list: uarts
    }]);

    console.log("roleNum = ", app.globalData.roleNum)
    // 如果控制权限，则不能添加按键同步
    if (app.globalData.roleNum == 2) {
      let _filter = _supportList.filter(item => item.key == 'pwrPanel');
      if (_filter.length > 0) {
        _filter[0].rightBtns = [];
      }
    }

    // 遍历所有数据，如果list为空，则不显示按钮
    for (let tmpItem of _supportList) {
      let {list, key, rightBtns} = tmpItem || {};
      if (!list?.length) {
        tmpItem.rightBtns = [];
      } else if (key == "pwrPanel" && rightBtns?.length > 0) {
        // 开关面板需要判断ability是否支持双控
        let _existVirtual = list.filter(item => {
          let {t, v} = item;
          return bizUtil.getJsonByAblitys(v, bizUtil.getDevTypeStr(t), {t})?.virtualPanel;
        });
        if (_existVirtual.length == 0) {
          tmpItem.rightBtns = [];
        }
      }
    }

    // 网关定时查询在线状态
    this.initGetStatusTimer(_supportList);

    console.log("_supportList = ", _supportList);
    this.data.supportList = _supportList;
    // 设置房间选择
    this.setData({
      rooms: this.data.rooms,
      isRoomEmpty: _isEmpty == 0,
      supportList: _supportList,
    });
  },
  // 根据数组取得开关面板和情景面板
  getPwrAndPanelByArray (array) {
    let _result = {
      pwrs: [],
      panels: []
    }
    if (!array || array.length == 0) {
      return _result;
    }
    for (let item of array) {
      if (bizUtil.isPwrAvalibleDev(item?.t)) {
        _result.pwrs.push(item);
      } else {
        _result.panels.push(item);
      }
    }
    return _result;
  },
  // 设备数据处理 减少不必要的数据
  dealArray (array, keys) {
    if (!array || !keys) {
      return array || [];
    }
    return array.map(item => {
      let _item = {};
      for (let key of keys) {
        _item[key] = item[key];
      }
      return _item; 
    })
  },
  // 初始化数据
  initDataByRole (roleNum) {
    // 取得是否主设备还是分享设备
    let _roleNum = (roleNum == undefined) ? app.globalData.roleNum : roleNum;
    app.globalData.roleNum = _roleNum;
    this.data.roleNum = _roleNum;
    // 新视觉去除首页的图标
    let {tip, imgStr} = bizUtil.getDefaultTip();
    this.data.topConfig.topObj.left.img = `../../static/img/${imgStr}.png`;
    this.data.topConfig.topObj.left.text = app.globalData.familyInfo?.n || ''; // tip;
    this.data.topConfig.topObj.left.textHander = this.familyClick; // 家庭按钮点击

    // 是否支持楼层
    let _supportFloor = this.initSupportFloor();
    let _floorName = "";
    if (_supportFloor) {
      _floorName = this.getCurFloor();
    }
    this.data.topConfig.topObj.left.supportFloor = _supportFloor;
    this.data.topConfig.topObj.left.floorName = _floorName;
    // 是否支持楼层
    this.data.isFloor = _supportFloor;

    // 初始化
    this.scrollClick(this.data.rooms.selRoomId);
    // 是否全部为空
    let _devLength = this.getDevsCount(app.globalData.devs);
    this.data.isEmpty = _devLength == 0;
    // 分享设备 未登录的时候和没有设置角色的时候不显示右侧
    let _img = "";
    let _hander = undefined;
    if (!this.data.isEmpty) {
      if (_roleNum == 2) {
        _img = "../../static/img/devList/scan.png";
        _hander = (() => {
          // 保存数据
          app.saveWhenHide();
          // 扫描二维码
          this.scanCode();
        });
      } else {
        _img = "../../static/img/devList/add.png";
        _hander = (() => {this.showAddDialog();});
      }
    }
    // 这里需要判断right可能为undefined
    this.data.topConfig.topObj.right = this.data.topConfig.topObj.right || {};
    this.data.topConfig.topObj.right.img = _img;
    this.data.topConfig.topObj.right.hander = _hander;
    // 设置用户角色
    this.setData({
      isFloor: this.data.isFloor,
      topConfig: this.data.topConfig,
      roleNum: _roleNum || 0,
      isEmpty: this.data.isEmpty
    })
  },
  getDevsCount (devs) {
    let count = 0;
    let _devs = devs || {};
    for (let key in _devs) {
      let {devs, groups} = _devs[key];

      count += (devs?.length || 0) + (groups?.length || 0);
    }
    return count;
  },
  releasedPage () {
    console.log("main onUnload");
  },
  getBatchLen (obj, count) {
    let _length = Object.keys(obj || {}).length;
    if ((_length + count) > 5) {
      ability.showToast({
        title: "最多选择五个设备"
      });
      return false;
    }
    return _length + count;
  },
  deviceDetail (devInfo, devTypeStr) {
    let _deviceInfo = devInfo || {};
    console.log("deviceDetail _deviceInfo = ", _deviceInfo);
    // 是否全屋控制或者房间控制
    if (bizUtil.isAllRoomCtrl(_deviceInfo)) {
      return this.jumpAllControl(_deviceInfo);
    }
    !!(app.globalData.tmpData?.roomInfo) && (delete app.globalData.tmpData.roomInfo);
    // 如果房间id为空则返回
    if (!_deviceInfo || _deviceInfo.a == undefined) {
      // 群组点击 固定group
      app.globalData.groupIds = [];
      app.globalData.tgroupId = Number(_deviceInfo.gid);
      app.globalData.devInfo = undefined;
      app.globalData.groupInfo = _deviceInfo;
      // 进入控制页面
      this.jumpControl({
        devType: _deviceInfo.t
      });
      return;
    }

    // 空调网关设备特殊处理
    if (_deviceInfo?.sub != undefined) {
      app.globalData.groupInfo = _deviceInfo;
      app.globalData.devInfo = _deviceInfo;
    } else {
      app.globalData.groupInfo = undefined;
      app.globalData.devInfo = _deviceInfo;
    }
    // 临时缓存到全局数据
    app.globalData.devStatus = {};
    app.globalData.groupIds = [];
    app.globalData.tgroupId = undefined;

    // 进入控制页面
    this.jumpControl({
      devType: _deviceInfo.t,
      devTypeStr: devTypeStr
    });
  },
  // 全屋控制
  jumpAllControl (devInfo) {
    if (!devInfo) {
      return;
    }
    // 房间列表
    app.globalData.roomInfo = this.data.roomJson[this.data.rooms.selRoomId] || {};
    console.log("jumpAllControl app.globalData.roomInfo, devInfo = ", app.globalData.roomInfo, devInfo);
    // 保存设备信息
    app.globalData.groupIds = [];
    app.globalData.devInfo = undefined;
    app.globalData.groupInfo = undefined;
    app.globalData.tgroupId = undefined;
    // 临时增加保存数据
    app.globalData.tmpData = app.globalData?.tmpData || {};
    app.globalData.tmpData.roomInfo = devInfo;
    // 跳转页面
    this.jumpControl({
      devType: devInfo.t || 43050,
      title: devInfo?.n
    });
  },

  /*
    * 跳转控制页面
    * @params devType 设备类型 int
    * @params devTypeStr 设备品类key 用于超级模块和透传模块共用一个固件
    */
  jumpControl ({devType, devTypeStr, title}) {
    // 如果是透传工具，则进入透传页面
    let {namespace} = app.globalData.companyInfo;
    if (namespace.indexOf('aux') >= 0) {
      return ability.navigateTo('/device/aircondition/control/index');
    } else if (namespace.indexOf('uart') >= 0) {
      return ability.navigateTo('../../device/uart/control/index');
    }

    // for test
    // app.globalData.devInfo.v = "0.0.0.0.0";
    // return ability.navigateTo('../../device/powerstrip/control/index');
    
    // 是否需要传递标题
    let _extendTitleStr = (title == undefined) ? "" : ("title=" + encodeURIComponent(title));
    
    // 判断是否空群组
    let _info = app.globalData.devInfo || app.globalData.groupInfo;
    let _isEmptyGroup = this.isEmptyGroup(_info);
    console.log("jumpControl _info, _isEmptyGroup, _extendTitleStr = ", _info, _isEmptyGroup, _extendTitleStr)
    // 取得设备品类key
    let _devTypeStr = bizUtil.getDevTypeStr(devType);
    switch (_devTypeStr) {
      case "curtain":
        // 窗帘
        // 判断是否空群组
        if (_isEmptyGroup) {
          return ability.navigateTo("/device/light/devProperty/index");
          // return ability.navigateTo("/groups/manage/index?remote=0" + "&" + _extendTitleStr);
        }
        return ability.navigateTo('/device/curtain/control/index' + "?" + _extendTitleStr);
      case 'panel':
        // for test
        console.error("getLocalDevStatus devInfo = ", app.globalData.devInfo);
        // app.globalData.blsdk.getLocalDevStatus([app.globalData.devInfo], (res) => {
        //   console.error("getLocalDevStatus res = ", res);
        // });
        // return;
        // 面板
        return ability.navigateTo('/device/panel/control/index');
      case "light":
        // 照明
        // 判断是否空群组
        if (_isEmptyGroup) {
          return ability.navigateTo("/device/light/devProperty/index");
          // return ability.navigateTo("/groups/manage/index?remote=0" + "&" + _extendTitleStr);
        }
        // 判断是否风扇灯能力
        let {isFanLight} = bizUtil.getJsonByAblitys(_info?.v, _devTypeStr);
        if (isFanLight) {
          // 风扇灯
          return ability.navigateTo('/device/fan/control/index');
        }
        // 判断是否kcwrgb双重控制
        if (bizUtil.isCwrgb(devType)) {
          // 跳转kcwrgb双重控制
          return ability.navigateTo("/device/light/doublecontrol/index" + "?" + _extendTitleStr);
        }
        return ability.navigateTo('/device/light/control/index' + "?" + _extendTitleStr);
      case "fanLight":
      case "fan":
        // 风扇灯
        return ability.navigateTo('/device/fan/control/index');
      case "doorsensor": // 门磁
      case "bodysensor": // 人体传感器
      case "watersensor": // 水浸传感器
      case "radarsensor": // 水浸传感器
      case "sossensor": // 水浸传感器
        return ability.navigateTo('/device/sensor/index/index?devTypeStr=' + _devTypeStr);
      case "uart":
        if (devTypeStr == "panel") {
          console.log("metapad = ", app.globalData.devInfo);
          // 超级面板
          return ability.navigateTo('/device/metapad/index/index');
        }
        // 透传模块 
        return ability.navigateTo('/device/uart/control/index');
      case "gateway":
        // 如果是控制权限
        if (app.globalData.roleNum == 2) {
          return ability.navigateTo("/device/light/devProperty/index?remote=0");
        }
        // 网关 不能解除遥控器
        return ability.navigateTo('/device/gateway/index/index?detail=1');
        // return ability.navigateTo('/device/light/devProperty/index?remote=0');
      case "aircondition":
      case "thermostat":
        // 空调
        // // 空调网关还是空调内机
        // if (_info?.sub != undefined) {
        //   return ability.navigateTo('/device/aircondition/manage/index');
        // }
        return ability.navigateTo('/device/aircondition/control/index');
      case "virtualPanel":
        return ability.navigateTo("/device/virtualPwrPanel/list/index?edit=" + (_info?.a || 0));
      case "powerPlug":
        // 插卡取电设备
        return ability.navigateTo("/device/panel/control/index");
      case "tvLamp":
        // 电视氛围灯 界面有全屏背景所以不与面板一致
        return ability.navigateTo("/device/tvLamp/control/index");
      case "coathanger":
        // 晾衣架
        return ability.navigateTo("/device/coathanger/control/index");
      case "temphumidity":
        // 秒秒测
        return ability.navigateTo("/device/temphumidity/control/index");
      case "powerstrip":
        // 秒秒测
        return ability.navigateTo("/device/powerstrip/control/index");
      case "panelsensor":
        // 四键传感器情景面板（干接点蓝牙控制盒）
        return ability.navigateTo("/device/sensor/panelSensor/index?devTypeStr=panelsensor");
      default:
        ability.showToast({
          title: "暂不支持该设备"
        });
    }
  },
  // 是否空群组
  isEmptyGroup (devInfo) {
    let {gid, dids, bledevtype} = devInfo || {};
    if (!gid || gid < 0 || bledevtype == "GROUP") {
      return false;
    }
    return (dids || []).length == 0;
  },
  refesh (e) {
    console.log("refresh e = ", e)
  },
  loadMore (e) {
    console.log("loadMore e = ", e)
  },
  refreshStop (e) {
    console.log("refreshStop e = ", e)
  },
  refreshAbort (e) {
    console.log("refreshAbort e = ", e)
    // 重新查询数据
    // to do list
  },
  dragEnd (e) {
    console.log("dragEnd e = ", e)
  },
  refreshed (e) {
    console.log("refreshed e = ", e)
  },
  

  /***************** 设备控制相关 ***************/
  showLoading (str) {
    ability.showLoading({
      title: str || "",
      mask: true
    })
  },
  hideLoading () {
    ability.hideLoading();
  },

  setDataByKey (key, value) {
    this.setData({
      [key]: value
    });
    setTimeout(() => {
      this.setData({
        [key]: !value
      });
    }, 100);
  },
  actDev (cmd, deviceInfo, config) {
    // 控制命令
    console.log("setControl deviceInfo, cmd = ", deviceInfo, cmd)
    // 房间信息
    let _roomInfo = {};
    // 是否单个设备控制
    if (!deviceInfo) {
      // 取得房间信息
      let {selRoomId} = this.data.rooms;
      _roomInfo = this.data.roomJson[selRoomId] || {};
      // 基于房间的控制
      app.globalData.blsdk.groupControlWithType(43050, _roomInfo.id, cmd);
    } else {
      // 固定群组控制
      let {gid, t} = deviceInfo;
      if (gid != undefined) {
        let _t = bizUtil.getCtrlPidByT(t);
        app.globalData.blsdk.groupControlWithType(_t, gid, cmd)
      } else {
        app.globalData.blsdk.controlWithDevice(deviceInfo, cmd, config);
      }
    }
  },
  roomManage () {
    ability.navigateTo("/minePages/roomManage/index")
  },

  unable () {
    console.log("unable");
    this.data.bleUnavailable = false;
    this.setData({
      bleUnavailable: false
    })
  },
  // jumpScene () {
  //   ability.navigateTo("/scene/list/index")
  // },

  jumpBatch (type) {
    ability.navigateTo("/pages/batch/index?from=deviceList&type=" + (type || "") + "&title=" + encodeURIComponent("批量控制"))
  },

  itemLongClick (json) {
    if (!json) {
      return;
    }
    let {item, child, index, idx} = json;
    // 设备信息
    let _item = child || {};
    let _n = _item.n || '';

    // // 如果是默认群组，则不弹出
    // if (bizUtil.isAllRoomCtrl(_item)) {
    //   return;
    // }

    // 显示对话框
    this.showPrompt(_n, json);
  },
  showPrompt (name, json) {
    // 设置设备名称
    this.data.promptConfig.devName = name || '';
    this.data.promptConfig.json = json || {};
    // 显示对话框
    this.managePrompt(true);
  },
  hidePrompt () {
    this.managePrompt(false);
  },
  editName (e) {
    let {name} = e.detail;
    console.log("editName name = ", name);

    // 如果是取消按钮则隐藏
    if (name == undefined) {
      return this.hidePrompt();
    } else if (name.length == 0) {
      return ability.showToast({
        title: "名称不能为空"
      });
    }

    // 数据
    let {json} = this.data.promptConfig || {};
    let {item, child, index, idx} = json || {};
    if (!item || !child || index == undefined) {
      return;
    }
    console.log("editName child, index, idx = ", child, index, idx);
    // 判断是群组还是设备
    let _saveArray = bizUtil.getExistNameCountByDevInJson(app.globalData.devs, child, name);
    console.log("editName _saveArray = ", _saveArray);
    if (_saveArray.length > 0) {
      return ability.showToast({
        title: "名称不能重复"
      }); 
    }
    // 名称赋值
    child.n = name;
    // 更新状态
    this.hidePrompt();
    // 更新
    this.setData({
      [`supportList[${index}]`]: item
    });

    // 上报数据
    app.uploadPrivateData();
  },
  managePrompt (value) {
    this.data.promptConfig.showPrompt = value || false;
    this.setData({
      promptConfig: this.data.promptConfig
    })
  },

  // 添加开关面板的双控
  addPwrPanel () {
    let _a = this.getAddrInByte();
    console.log("addPwrPanel _a = ", _a);
    // 先判断是否超出255 超出则提示加不了
    if (_a <= 0 || _a >= 255) {
      return ability.showToast({
        title: "设备已满"
      })
    }
    // 所有设备json
    let _allDevJson = app.globalData.devs;
    // 取得所有设备信息
    let {devs} = bizUtil.getAllDevsBySort(_allDevJson, _allDevJson);
    // 取得同步面板设备
    let {virtualPanel} = bizUtil.getAllPidByArray(devs);
    // 取得个数，赋值名称
    let _length = (virtualPanel || []).length;
    // 未超出则添加
    app.globalData.devInfo = {
      d: this.getVirtualDid(),
      a: _a,
      t: 44063,
      id: 0, // 默认房间为全屋
      n: "按键同步" + (_length + 1),
      isNew: true // 用于新老设备标识
    };
    // // 同时加入到房间列表中
    // app.globalData.devs[0].devs.push(app.globalData.devInfo);

    // 跳转到添加同步页面
    ability.navigateTo("/device/virtualPwrPanel/list/index")
  },
  // 取得双控设备虚拟did
  getVirtualDid () {
    // 使用时间戳取得
    let _timestamp = parseInt(Date.now() / 1000);
    return (_timestamp + "").padStart(12, '0');
  },
  // 先取得256之内的所有短地址集合
  getAddrInByte () {
    let _allDevArray = bizUtil.getAllDevsFilter(app.globalData.devs);
    let _addrs = bizUtil.getIdByArray(_allDevArray, "a", 1, 255);
    if (_addrs <= 0) {
      return -1;
    }
    return _addrs;
  },


  /* 优化之后 */
  // 右侧按钮点击
  rightBtnClick (e) {
    let {item, child} = this.getJsonByE(e);
    console.log("rightBtnClick item, child = ", item, child);
    let {key} = item;
    if (key == "pwrPanel") {
      // 添加按键同步
      this.addPwrPanel();
    } else if (["curtain", "light"].indexOf(key) >= 0) {
      // 批量控制页面
      this.jumpBatch(key);
    }
  },
  getJsonByE (e) {
    let {index, idx} = e.currentTarget.dataset;
    let item = this.data.supportList[index] || {};
    let child = (item.list || [])[idx] || {};
    return {
      item,
      child,
      index,
      idx
    }
  },
  devItemClick (e) {
    let {item, child} = this.getJsonByE(e);
    console.log("devItemClick item, child = ", item, child);
    this.deviceDetail(child, item.realKey || item.key);
  },
  openDev (e) {
    this.controlManage(e, 1);
  },
  closeDev (e) {
    this.controlManage(e, 0);
  },
  pauseDev (e) {
    this.controlManage(e, 2);
  },
  // value : 1开 0关 2暂停
  controlManage (e, value) {
    let {item, child} = this.getJsonByE(e);
    console.log("openDev item, child, value = ", item, child, value);
    // 判断设备类型
    let {tStr, t} = child;
    if (tStr == "virtualPanel") {
      // 双控
      this.pwrPanelCtrl(child, value);
    } else if (tStr == "sub") {
      // 空调内机
      this.airconditionCtrl(child, value);
    } else if (tStr.indexOf("light") >= 0) {
      // 照明
      this.lightCtrl(child, value);
    } else if (tStr == "curtain") {
      // 窗帘
      this.curtainCtrl(child, value);
    } else if (tStr == "thermostat") {
      // 温控器
      this.thermostatCtrl(child, value);
    } else if (tStr.indexOf("group") >= 0) {
      // 群控
      if (t == "light") {
        // 照明
        this.lightCtrl(child, value);
      } else if (t == "curtain") {
        // 窗帘
        this.curtainCtrl(child, value);
      }
    }
  },
  thermostatCtrl (devInfo, value) {
    this.actDev({
      pwr: value
    }, devInfo);
  },
  curtainCtrl (devInfo, value) {
    let _key = "down"; // 默认关
    if (value == 1) {
      _key = "up"; // 开
    } else if (value == 2) {
      _key = "pause"; // 暂停
    }
    this.actDev(_key, devInfo);
  },
  airconditionCtrl (devInfo, value) {
    this.actDev({
      pwr: value
    }, devInfo);
  },
  // 双控
  pwrPanelCtrl (devInfo, value) {
    this.actDev([1 << 0, value], devInfo);
  },
  lightCtrl (devInfo, value) {
    // 判断开关
    if (value == 0) {
      // 关
      this.actDev({
        pwr: 0, brightness: 0
      }, devInfo);
      return;
    }
    let {a, d, gid} = devInfo;
    // 判断是否有缓存数据
    let _key = "";
    if (bizUtil.isAllRoomCtrl(devInfo)) {
      // 判断是否全屋的控制
      _key = "all";
    } else if (gid) {
      // 判断是否是灯的群组
      _key = gid;
    } else {
      _key = d + "_" + a;
    }
    let _status = app.globalData.allStatus[_key] || {};
    console.log("openLightDev gid, _status = ", gid, _status);
    this.actDev({
      pwr: 1,
      brightness: _status.brightness || 127
    }, devInfo);
  },
  
  // 长按
  devLongTap (e) {
    let _json = this.getJsonByE(e);
    console.log("devLongTap _json = ", _json);
    this.itemLongClick(_json);
  },

  // 取得当前网关在线状态
  getGatewayListInList (arrayConfig) {
    let _config = arrayConfig || this.data.supportList;
    let _gatewayIndex = _config.findIndex(item => item.key == "gateway");
    console.error("getGatewayListInList _gatewayIndex = ", _gatewayIndex);
    let _gateways = _config[_gatewayIndex];
    if (!_gateways) {
      // 没有网关配置项不查
      console.error("getGatewayListInList gateway not exist");
      return {};
    }
    let _list = _gateways?.list || [];
    if (_list?.length == 0) {
      // 数据为空不查
      console.error("getGatewayListInList gateway empty");
      return {};
    }
    return {
      list: _list,
      index: _gatewayIndex
    };
  },
  onHide () {
    console.error("clearGatewayTimer");
    // 清除网关定时器
    this.clearGatewayTimer();
  },
  clearGatewayTimer () {
    bizUtil.clearTimer(this.statusTimer, 'interval');
  },
  // 初始化定时器
  initGetStatusTimer (arrayConfig) {
    // 清除网关定时器
    this.clearGatewayTimer();
    // 取得网关数据
    let {list, index} = this.getGatewayListInList(arrayConfig);
    if (!list?.length) {
      console.error("initGetStatusTimer gateway not exist");
      return;
    }
    // 保证
    this.gatewayTimerList = list;
    this.gatewayTimerIndex = index;
    // 启动定时器
    this.statusTimer = setInterval(() => {
      this.getGatewayStatus(this.gatewayTimerList);
    }, 60000);
    this.getGatewayStatus(this.gatewayTimerList);
  },
  // 定时获取网关状态
  getGatewayStatus (gatewayArray) {
    let _groupStr = GROUPS[Number(app.globalData.familyInfo?.groupIndex) || 0]?.value || GROUPS[0].value;

    // console.log("getGatewayStatus _groupStr = ", _groupStr);
    // for test
    // _groupStr = "chn-ee08f451.ibroadlink.com";

    for (let gatewayInfo of gatewayArray) {
      this.getDevVersion({
        d: gatewayInfo?.wd || gatewayInfo?.d,
        t: gatewayInfo?.t,
        k: gatewayInfo?.k
      }, _groupStr, (res) => {
        let {v, sv, d, isReset} = res || {};
        if (!d) {
          return;
        }
        let _index = this.gatewayTimerList.findIndex(item => item.wd == d || item.d == d);
        let _gatewayInfo = this.gatewayTimerList[_index];
        if (_index < 0 || !_gatewayInfo) {
          return;
        }
        console.error("getGatewayStatus");
        let _isOnline = (v > 0);
        _gatewayInfo.online = _isOnline;
        _gatewayInfo.statusTip = _isOnline ? " | 在线" : (isReset ? " | 已复位" : " | 离线");
  
        // 判断选中的网关是否在线，如果在线不更新，如果不在线，则更新为在线的网关
        let _curGatewayInfo = app.globalData.blsdk.getCurrentGateway();
        if ((!_curGatewayInfo || !_curGatewayInfo.online) && _isOnline) {
          // 更新在线的网关
          app.globalData.blsdk.updateGatewayInfo(_gatewayInfo);
        }
  
        this.setData({
          [`supportList[${this.gatewayTimerIndex}].list`]: this.gatewayTimerList
        })
      })
    }
  },
  // 如果是网关设备，取得网关的授权码
  getDevVersion (devInfo, groupStr, callback) {
    if (!devInfo || !groupStr) {
      !!callback && (callback(false));
      return;
    }
    console.log("getDevVersion devInfo = ", devInfo);
    // 网关链接的集群
    let _groupStr = groupStr || "";
    // 设备信息 ihg需要用wd去获取版本号
    let _devInfo = devInfo;
    if (_devInfo?.wd) {
      _devInfo = Object.assign({}, devInfo, {d: _devInfo.wd});
    };
    // 取得网关的code
    let _result = app.globalData.blsdk.getDeviceVersion(_devInfo, _groupStr);
    if (!_result || !_result.then) {
      console.error("getDevVersion _result = ", _result);
      !!callback && (callback(false));
      return;
    }
    _result.then((result) => {
      // // 回调函数
      // if (!callback) {
      //   return;
      // }
      // 设备d
      let _endpointId = result?.event?.endpoint?.endpointId || "";
      // 设备去除多余的0 默认12位
      _endpointId = _endpointId.slice(-12);
      // 数据
      let _payload = result?.event?.payload;
      // 设备当前固件版本
      let _v = _payload?.version;
      // 设备服务器版本
      let _sv = _payload?.serverversion?.version;
      console.log("getDevVersion _v, _endpointId = ", _v, _endpointId);
      // 回调函数
      !!callback && (callback({
        v: _v,
        sv: _sv,
        d: _endpointId,
        isReset: _payload?.status == -7
      }));
    }).catch((e) => {
      console.error("getDevVersion catch e = ", e);
      !!callback && (callback(false));
    });
  },
  // 关闭远程调试
  closeRemote () {
    console.log("closeRemote");
    // 关闭远程
    app.globalData.blsdk.clseRemoteDebug();
    // 远程调试关闭
    app.globalData.familyInfo.isRemote = false;
    // this.data.showRemote = false;
    this.setData({
      showRemote: false
    })
  },

  // 判断是否开启远程调试
  dealRemote () {
    let {isRemote} = app.globalData.familyInfo || {};
    console.log("dealRemote isRemote = ", isRemote);
    this.setData({
      showRemote: isRemote
    })
    // 是否远程调试 或者已经开启了
    if (!isRemote) {
      if (app.globalData.websocket) {
        // 关闭
        app.globalData.websocket.close();
        app.globalData.websocket = null;
      }
      return;
    } else if (this.subscribe) {
      return;
    }
    // 增加远程调试订阅
    this.startSubscribe();
  },

  // 开始订阅远程调试
  startSubscribe () {
    // 取得家庭下网关列表
    let _alls = this.data.supportList.filter(item => item.key == "gateway") || [];
    let _allIhgGateways = (_alls[0] || {}).list || [];
    if (_allIhgGateways.length == 0) {
      return;
    }
    let {uuid, sid, skey} = app.globalData.familyInfo || {};
    let _result = [];
    for (let item of _allIhgGateways) {
      let {t, d} = item || {};
      if (bizUtil.isIhg(t)) {
        continue;
      }
      _result.push({
        "endpointId": (d || "").padStart(32, 0),    // 订阅设备endpointId
        "pid": this.getDevType(t),    // 订阅设备 pid
        "gatewayId": sid,    // 当前家庭ID
        "token": skey,  // 当前家庭token
      })
    }
    console.log("startSubscribe _result = ", _result);
    // // 默认发送初始化命令
    // let _data = app.globalData.websocket.getSubscribeData(_result);
    // console.log("startSubscribe _data = ", _data);
    // app.globalData.websocket.send({data: _data});

    app.globalData.websocket.initSubScribe(_result, (status) => {
      this.subscribe = true;
      // 订阅成功
      ability.showToast({
        title: status == 0 ? "订阅成功" : "订阅失败"
      })
    });
  },

  
  // 取得小段的pid
  getReversePidStr (t) {
    let _pidStr = (Number(t) || 0).toString(16);
    // 判断长度，如果小于偶数，则前面补0，然后小端模式
    let _pidLen = _pidStr.length;
    _pidStr = (_pidLen % 2) ? ('0' + _pidStr) : _pidStr;
    // 每隔两位取数组再小端
    // let _reversePid = this.getArrayByHexStr(_pidStr).reverse().join("");
    return this.getArrayByHexStr(_pidStr).reverse();
  },
  // 取得32位完整的pid
  getDevType (t) {
    let _reversePid = this.getReversePidStr(t).join("");
    // 后补4个0，前面补足32位的长度
    let _reversePidLen = _reversePid.length;
    _reversePid = _reversePid.padEnd(_reversePidLen + 4, 0);
    _reversePid = _reversePid.padStart(32, 0);
    return _reversePid;
  },
  // hex的每隔两位插入：
  getArrayByHexStr (str) {
    if (!str) {
      return '';
    }
    let _result = [];
    for (let i = 0, len = str.length; i < len;) {
      _result.push(str.substr(i, 2));
      i += 2;
    }
    return _result;
  },

  swipeChange (e) {
    let {current, source} = e.detail;
    console.log("swipeChage current, source = ", current, source);
    if (source != "touch") {
      return;
    }
    let _selectedId = this.data.rooms.data[current].id;
    console.log("swipeChage _selectedId = ", _selectedId);
    this.data.rooms.selRoomId = _selectedId;
    this.data.rooms.selRoomIndex = current;

    // 初始化
    this.scrollClick(_selectedId);

    this.setData({
      rooms: this.data.rooms
    })
  }
})