import bizUtil from '../../utils/bizUtil'
const ability = require("../../utils/classDef/ability")
const app = getApp();
// scene/panel/inde.js
Page({

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "面板选择"
      }
    },
    panelList: [],
    panelIndex: -1,
    roomChanged: false, // 是否弹出选择房间
    rooms: [],
    roomInfo: {
      id: 0, // 默认全屋
      n: '全屋'
    },

    roomJson: {}, // 所有房间json
    isFloor: false, // 是否多楼层

    devTypeStr: "", // 是否需要某一品类的 暂时没用到
    from: "" // 是否从某页面进入  除了场景，还有双控设备
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    console.log("onLoad options = ", options);
    // 页面来源
    let {from} = options || {};
    this.data.from = from;
    // 初始化所有房间的roomJSON
    let _roomJson = this.getRoomJson();
    // 初始化多楼层信息
    let _isFloor = app.globalData.familyInfo?.floor >= 0;
    // 初始化房间信息
    let _roomId = (app.globalData.roomInfo || {}).id;
    if (_roomId != this.data.roomInfo.id && _roomId != undefined) {
      let _roomInfo = app.globalData.devs[_roomId] || app.globalData.devs[0];
      this.data.roomInfo.id = _roomInfo.id;
      this.data.roomInfo.n = _roomInfo.n;
    }
    // 取得当前楼层下的房间列表
    let _rooms = this.getRoomsByFloor();
    
    // 如果是双控，则取得每个did对应的双控个数
    if (this.isFromPwrAvalable()) {
      // 所有设备d对应的双控个数，最大4个
      this.allDMap = this.getAllDMap();
    }
    
    let _panelList = this.getListByRoom();

    this.setData({
      isFloor: _isFloor,
      roomJson: _roomJson,
      roomInfo: this.data.roomInfo,
      rooms: _rooms,
      from: from,
      panelList: _panelList
    });
  },

  // 取得所有设备集合
  getAllDMap () {
    let _result = {};
    // 所有设备列表
    let _allDevArray = bizUtil.getAllDevsFilter(app.globalData.devs);
    // 筛选出双控设备
    let {virtualPanel} = bizUtil.getAllPidByArray(_allDevArray);
    if (!virtualPanel || virtualPanel.length == 0) {
      return _result;
    }
    for (let item of virtualPanel) {
      let {bindDevices} = item?.e || {};
      if (!bindDevices || bindDevices.length == 0) {
        continue;
      }
      for (let child of bindDevices) {
        let {d, key} = child || {};
        _result[d] = _result[d] || 0;
        _result[d] += 1; // 个数+1
      }
    }
    return _result;
  },

  getRoomJson () {
    // 取得多楼层的json数据
    let {roomJson} = bizUtil.getRoomsByDevs(app.globalData.devs);
    return roomJson;
  },

  // 初始化房间
  getRoomsByFloor () {
    // 所有设备
    let _allDevs = app.globalData.devs || {};
    // 当前楼层
    let _floor = app.globalData.familyInfo?.floor || 0;
    // 根据多楼层取得数据
    let _devs = bizUtil.getDevsByFloor(_allDevs, _floor);
    let {rooms} = bizUtil.getRoomsByDevs(_devs);
    return rooms;
  },

  // 是否从双控进入
  isFromPwrAvalable () {
    return this.data.from == "pwrAvalible"
  },

  getListByRoom () {
    let _roomInfo = this.data.roomInfo;
    let _roomId = _roomInfo.id || 0;
    console.log("getListByRoom _roomInfo = ", _roomInfo);

    let _panelList = [];
    // 如果页面来源是双控
    if (this.isFromPwrAvalable()) {
      // 取得开关设备列表
      let {devs: _allDevs} = bizUtil.getDevsAndGroupsByRoomId(_roomId, app.globalData.devs);
      let {panel} = bizUtil.getAllPidByArray(_allDevs) || {};
      console.log("getListByRoom panel = ", panel);
      // 只需要开关面板
      let _pwrPanelList = (panel || []).filter(item => bizUtil.isPwrAvalibleDev(item.t));
      console.log("getListByRoom _pwrPanelList = ", _pwrPanelList);
      // 去重 如果已经绑定过的则不再绑定 且根据ability过滤不需要显示的设备
      let _filterList = this.checkByAbility(this.dealFilterByArray(_pwrPanelList));
      console.log("getListByRoom _filterList = ", _filterList);
      // 添加tip 群控个数
      _panelList = this.setTipByDMap(_filterList, this.allDMap);
      console.log("getListByRoom this.dMap = ", this.allDMap);
    } else {
      // 取得触发设备列表
      _panelList = bizUtil.getTriggerArrayByRidAndDevs(app.globalData.devs, _roomId);
    }
    
    console.log("getListByRoom _panelList = ", _panelList);
    // 先把_array排序好 后面的就不用排序了
    return bizUtil.sortArrayByChar(_panelList, "n");
  },
  // 根据设备的e: bindDevices中数据去重
  dealFilterByArray (array) {
    if (!array || array.length == 0) {
      return [];
    }
    let {bindDevices} = app.globalData.devInfo?.e || {};
    console.log("dealFilterByArray array, bindDevices = ", array, bindDevices);
    // 根据d去重
    let _dArray = (bindDevices || []).map(item => item.d);
    console.log("dealFilterByArray _dArray = ", _dArray);
    // 去重
    let _result = array.filter(item => _dArray.indexOf(item.d) < 0);
    console.log("dealFilterByArray _result = ", _result);
    return _result;
  },
  // 每个设备是否支持双控，如果支持，默认最大4个
  checkByAbility (array) {
    if (!array || array.length == 0) {
      return [];
    }
    return array.filter(item => {
      let {t, v, d} = item;
      let {virtualPanel} = bizUtil.getJsonByAblitys(v, bizUtil.getDevTypeStr(t), {t: t});
      console.log("checkByAbility d, virtualPanel = ", d, virtualPanel);
      if (!virtualPanel) {
        // 不支持能力
        return false;
      }
      if (!this.allDMap) {
        return true;
      }
      // 支持但是4个限制
      let _count = this.allDMap[d];
      if (_count >= 4) {
        return false;
      }
      return true;
    })
  },
  setTipByDMap (array, dMap) {
    if (!array) {
      return [];
    }
    if (!dMap) {
      bizUtil.setArrayItem(array, ['tip'], ['未添加']);
      return array;
    }
    for (let item of array) {
      let {d} = item || {};
      item.tip = ""; // (dMap[d]) ? (dMap[d] + '个按键同步') : '未添加';
    }
    return array;
  },


  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  selectPanel (e) {
    let {index} = e.currentTarget.dataset;
    let {selected, rt} = !this.data.panelList[index] || {};

    // 如果rt，则不选择
    if (rt) {
      return ability.showToast({
        title: "个数已满"
      })
    }

    this.data.panelList[index].selected = selected;

    this.data.panelIndex = index;

    let _text = 'panelList['+ index +'].selected';
    this.setData({
      [_text]: selected,
      panelIndex: index
    })
  },

  chooseRoom (e) {
    let {rid} = e.currentTarget.dataset;
    let _roomInfo = app.globalData.devs[rid || 0];
    this.data.roomInfo.n = _roomInfo.n;
    this.data.roomInfo.id = _roomInfo.id;
    
    // 隐藏
    this.hidePop();
    // 更新设备列表数据
    let _panelList = this.getListByRoom();
    this.setData({
      roomInfo: this.data.roomInfo,
      panelList: _panelList
    });
  },
  showPop () {
    this.hidePop(true);
  },
  hidePop (isOpen) {
    this.data.roomChanged = isOpen || false;
    this.setData({
      roomChanged: this.data.roomChanged
    })
  },
  hidePops () {
    this.hidePop();
  },

  next () {
    let _panelIndex = this.data.panelIndex;
    console.log("next this.data.panelIndex = ", _panelIndex);
    if (_panelIndex < 0) {
      return ability.showToast({
        title: "请选择面板"
      });
    }
    // 保存面板信息
    app.globalData.panelInfo = this.data.panelList[_panelIndex] || {};

    // 设备品类
    let _devTypeStr = bizUtil.getDevTypeStr(app.globalData.panelInfo?.t);

    // 是否需要携带额外参数
    let _extendStr = this.getExtendParamsByFrom(this.data.from, _devTypeStr);

    ability.redirectTo("../panelDetail/index?" + _extendStr)
  },
  getExtendParamsByFrom (fromStr, devTypeStr) {
    let _result = "";
    if (devTypeStr == "sossensor") {
      _result = "lowPower=0";
    }
    // 是否需要携带参数到下一步
    if (!fromStr) {
      return _result;
    }
    let _extend = "from=" + fromStr;
    return _result ? (_result + "&" + _extend) : _extend;
  }
})