// scene/status/index.js
const ability = require("../../utils/classDef/ability")
const app = getApp();
const bizUtil = require("../../utils/bizUtil");
Page({

  /**
   * 页面的初始数据
   */
  data: {
    navObj: {
      left: {
        type: "img"
      },
      center: {
        text: "设备状态"
      }
    },
    pwrList: [{
      name: '开',
      value: 1
    }, {
      name: '关',
      value: 0
    }],
    statusList: [],
    rgbSliders: {
      bgImg: "../../static/img/common/rgb.png",
      bgClass: "cwImg",
      width: 616, // rpx
      percent: 100, // 百分比
      // labelTxt: "亮度",
      minVal: 0,
      maxVal: 360,
      minPercent: 1, // 最小百分比
      gradient: false // 默认不支持滑动时调节
    },
    // 亮度配置
    brightSliders: {
      leftImg: "../../static/img/common/bright_down.png",
      rightImg: "../../static/img/common/bright_up.png",
      leftClass: "bDown",
      rightClass: "bUp",
      width: 480, // rpx
      percent: 100, // 百分比
      labelTxt: "亮度",
      minVal: 2,
      maxVal: 127,
      minPercent: 1, // 最小百分比
      gradient: false // 默认不支持滑动时调节
    },
    // 色温配置
    cwSliders: {
      leftImg: "",
      rightImg: "",
      bgImg: "../../static/img/common/temp.png",
      leftClass: "cDown",
      rightClass: "cUp",
      bgClass: "cwImg",
      width: 616, // rpx
      percent: 50, // 百分比
      minPercent: 0,
      minVal: 0,
      maxVal: 255,
      gradient: false // 默认不支持滑动时调节
    },
    sceneInfo: {
    },
    status: {
      pwr: 0,
      brightness: 100
    },
    statusKey: 'white',
    devName: '', // 设备名称

    showCustomToast: false, // 是否显示自定义的提示框
    customToastConfig: {}, // 自定义提示框配置信息

    // 按钮 根据编辑状态动态显示
    actions: [],

    types: [],  // 顶部选项条
    currentType: 0,

    devType: 'light', // 默认灯泡设备

    // 非唤醒的提示
    showLoadingDialog: false,
    loadingConfig: {
      forward: false,
      title: '', // 默认是灯泡的提示信息
      content: "" // 默认正在设置中
    },
    // 定时器
    timer: null,
    // 上一个修改房间方法是否过期
    lastAvalible: false,
    // 修改房间的方法
    // modifyFunc: undefined,

    // isGroup: false // 是否群组编辑
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    let {index} = options || {};
    // 是否编辑状态
    this.editIndex = (index == undefined) ? -1 : Number(index);
    console.log("onLoad this.editIndex = ", this.editIndex);
    // 是否群组编辑
    this.isGroup = this.initIsGroup();
    // 初始化状态 如果群组，则返回数组
    this.data.types = this.isGroup ? [] : (this.initTabByAbility());
    // 初始化设备品类
    this.data.devType = this.initDevType();
    // 初始化开关列表
    let _pwrList = this.initPwrList();
    // 根据品类初始化滑块
    let _brightnessSlider = this.getSliderByDevType(this.data.devType);
    // 根据编辑模式初始化按钮
    // this.initActionByMode();
    // 初始化数据 设备状态
    this.initDataByDevTypeAndEditIndex();
    // 初始化设备状态
    this.initStatusListByStatus();
    // 初始化默认状态选项
    this.initStatusIndex();

    this.setData({
      pwrList: _pwrList,
      devType: this.data.devType,
      // isGroup: this.isGroup, // 是否群组场景
      types: this.data.types,
      brightSliders: _brightnessSlider
    });
  },
  // 是否群组编辑
  initIsGroup () {
    return app.globalData.devInfo?.gid >= 0;
  },
  initTabByAbility () {
    let _devInfo = app.globalData.devInfo;
    console.log("initTabByAbility _devInfo = ", _devInfo);
    let _v = _devInfo.v || ""; // 取得版本信息

    let _support = false; // 默认不支持
    // 如果版本信息不为空
    if (_v.length > 0) {
      // 版本号信息
      let _json = bizUtil.getJsonByAblitys(_v, bizUtil.getDevTypeStr(_devInfo.t), {t: _devInfo.t});
      _support = _json?.sceneException;
      // let _byte = ((_v.split(".")[2] || 0) >> 8) & 0x02; // 第2个ability里面的BIT1
      // _support = (_byte == 2);
    }

    let _types = this.getAllTypes(_support);
    if (_v == undefined && _devInfo.gid > 0) {
      _types = this.data.types.slice(0, 1); // 群组暂时只支持自定义状态
      // _support = true; // 如果是场景则默认都支持当前状态
    } else if (bizUtil.getDevTypeStr(_devInfo?.t) == 'fan') {
      _types = this.data.types.slice(0, 1); // 风扇暂时只支持自定义状态
    }
    return _types;
  },
  getAllTypes (support) {
    let _array = [{
      name: "设备状态设置"
    }, {
      name: "保存当前状态"
    }];
    if (support) {
      _array.push({
        name: "不执行场景"
      });
    }
    return _array;
  },
  // 初始化设备品类
  initDevType () {
    // 取得设备品类
    let _devTypeKey = bizUtil.getDevTypeStr(app.globalData.devInfo.t);
    console.log("initDevType _devTypeKey = ", _devTypeKey, app.globalData.devInfo.t);
    return _devTypeKey;
  },
  initPwrList () {
    // 取得设备品类
    let _devTypeKey = this.data.devType;
    let _openVal = 1, _closeVal = 0;
    if (_devTypeKey == "curtain") {
      _openVal = 1;
      _closeVal = 2;
    }
    return [{
      name: '开',
      value: _openVal
    }, {
      name: '关',
      value: _closeVal
    }];
  },
  getSliderByDevType (devType) {
    let _devType = devType || ""; // this.data.devType;
    // if (["curtain", 'fan'].indexOf(_devType) < 0) {
    //   return;
    // }
    if ("curtain" == _devType) {
      this.data.brightSliders.leftImg = undefined;
      this.data.brightSliders.rightImg = undefined;
      this.data.brightSliders.leftText = "全关";
      this.data.brightSliders.rightText = "全开";
      this.data.brightSliders.minVal = 0;
      this.data.brightSliders.maxVal = 100;
      this.data.brightSliders.minPercent = 0;
      this.data.brightSliders.labelTxt = "开合程度";
      this.data.brightSliders.percent = 100; // 百分比
      this.data.brightSliders.devType = "curtain"; // 百分比
    } else if ("fan" == _devType) {
      this.data.brightSliders.labelTxt = "风速";
    }

    return this.data.brightSliders;
  },
  getAllActions () {
    return [{
      n: '删除',
      classStr: 'delBtn',
      handle: 'delDev'
    }, {
      n: '保存',
      classStr: 'saveBtn marginL49',
      handle: 'saveInfo'
    }]
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {
    // this.initCurrentType();
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    // 清除定时器
    this.destoryTimer();
    // 清除数据
    this.resetGroupSceneTimer();
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  },

  // 取得全部设备的dmap
  getAllDevDMap(allDevs) {
    // 取得房间下的所有面板信息
    let _allDevInfo = bizUtil.getAllDevsFilter(allDevs || {});
    let _allDevJson = bizUtil.arrayToJsonByKey(_allDevInfo, true, 'd');
    console.log("getAllDevDMap _allDevJson = ", _allDevJson);
    return _allDevJson;
  },
  // 取得群组下的设备能力集合
  getMaxFuncByDids (dids) {
    let _maxStatus = {};
    if (!dids || dids.length == 0) {
      return _maxStatus;
    }
    let _allDMap = this.getAllDevDMap(app.globalData.devs);
    for (let d of dids) {
      let _devInfo = _allDMap[d];
      if (!_devInfo) {
        continue;
      }
      // 取得能力集
      let _devStatus = bizUtil.getFuncByPid({pwr: 1}, _devInfo.t);
      // 添加最大能力集
      Object.assign(_maxStatus, _devStatus);
    }
    return _maxStatus;
  },
  getMaxFuncByRoomId (roomId) {
    let _maxStatus = {};
    let _roomIdInt = Number(roomId);
    if (!_roomIdInt && _roomIdInt != 0) {
      return _maxStatus;
    }
    let _devArray = bizUtil.getDevsOrGroupsByRoomId(_roomIdInt, app.globalData.devs);
    let _pidDevMap = bizUtil.getAllPidByArray(_devArray);
    let _devTypedArray = _pidDevMap[this.data.devType] || {};
    for (let dev of _devTypedArray) {
      if (!dev) {
        continue;
      }
      // 取得能力集
      let _devStatus = bizUtil.getFuncByPid({pwr: 1}, dev.t);
      // 添加最大能力集
      Object.assign(_maxStatus, _devStatus);
    }
    return _maxStatus;
  },

  initDataByDevTypeAndEditIndex () {
    // 自定义时需要
    let _devInfo = app.globalData.devInfo || {};
    let _devStatus = _devInfo.cmd;
    let _devTypeStr = this.data.devType;

    // 取得当前选择的type
    let _currentType = this.getCurrentType(_devStatus);
    console.log("initDataByDevType _currentType = ", _currentType);
    this.data.currentType = _currentType;

    // 设备能力集合
    let _status = {};
    // 判断群组场景还是单个设备场景
    if (_devInfo.t == "light" && _devInfo.gid >= 0) {
      if (_devInfo.gid < 193) {
        // 默认房间
        _status = this.getMaxFuncByRoomId(_devInfo.gid);
      } else {
        // 群组
        _status = this.getMaxFuncByDids(_devInfo?.dids);
      }
    } else {
      _status = bizUtil.getFuncByPid({
        pwr: 1,
      }, _devInfo.t); // 默认数据
    }
    console.log("initDataByDevType _status = ", JSON.stringify(_status), _devStatus, _devTypeStr);

    // 如果status是二进制控制命令，则转成对应的json数据 这里其实只有灯泡和窗帘才进入
    if (!!_devStatus && _devStatus.pwr == undefined) {
      _devStatus = bizUtil.getJsonByCmdAndType(_devStatus, _devTypeStr);
      console.log("initDataByDevType _devStatus = ", JSON.stringify(_devStatus));
    }
    // 如果存在数据 这里其实没啥用了，因为去除了批量控制
    if (!!_devStatus && Object.keys(_devStatus)?.length > 0) {
      if (_devTypeStr == "light") {
        (_status.pwr != undefined) && (_status.pwr = _devStatus.pwr);

        (_status.brightness != undefined) && (_status.brightness = _devStatus.brightness);
        
        // rgbcw赋值 防止不支持功能加进来
        (_status.r != undefined) && (_status.r = _devStatus.r, _status.g = _devStatus.g, _status.b = _devStatus.b);
        (_status.c != undefined) && (_status.c = _devStatus.c, _status.w = _devStatus.w);
      } else if (_devTypeStr == "curtain") {
        if (_status.percent != undefined) {
          _status.percent = _devStatus.percent;
          // if (_status.percent == 0) {
          //   _status.pwr = 0;
          // }
        }
      }
    }
    console.log("initDataByDevType _status = ", JSON.stringify(_status), typeof _status);
    this.data.status = _status;
    // 默认亮度
    if (_devTypeStr == "curtain" || _devTypeStr == "fan") {
      let _percentInt = Number(_status.percent);
      this.data.brightSliders.percent = _percentInt == undefined ? 100 : _percentInt;
    } else {
      let _percent = bizUtil.getPercentByVal(_status.brightness, this.data.brightSliders) || 100; // parseInt(_status.brightness / 1.27) || 100;
      // 记住上一次的编辑
      let _index = this.editIndex;
      if (_index >= 0) {
        let _devInfo = (app.globalData.sceneInfo.devs || [])[_index];
        if (!!_devInfo && _devInfo.bp >= 0) {
          _percent = Number(_devInfo.bp);
        }
      }

      // 默认灯泡
      this.data.brightSliders.percent = _percent; // bizUtil.getPercentByVal(_status.brightness, this.data.brightSliders); // _percent;
      console.log("initDataByDevType this.data.brightSliders = ", this.data.brightSliders);
    }

    // 底部按钮特殊处理
    this.dealActions();
    
    // 设备名称
    this.data.devName = _devInfo.n || '';
    this.setData({
      brightSliders: this.data.brightSliders,
      status: this.data.status,
      devName: this.data.devName,
      currentType: this.data.currentType,
      // actions: this.data.actions
    });
  },

  getCurrentType (cmd) {
    console.log("getCurrentType cmd, this.editIndex = ", cmd, this.editIndex, this.data.types);
    // 新增时默认返回自定义
    if (this.editIndex < 0) {
      return 0;
    }

    let _num = 0;
    if (cmd === '' || cmd === undefined || cmd === "default") {
      _num = 2;
    } else if (cmd && (cmd.length == 0 || "current" == cmd)) {
      _num = 1;
    } else {
      _num = 0;
    }
    let _len = this.data.types?.length || 0;
    if (_num >= _len) {
      // 如果长度大于等于默认的下标
      _num = 0;
    }
    return _num;
  },

  initStatusIndex () {
    
    // 如果是窗帘，则不走
    if (this.data.devType != "light") {
      return;
    }

    console.log("initStatusIndex this.data.status = ", this.data.status)

    let {r, g, b, c, w} = this.data.status;
    console.log("initStatusIndex = ", r, g, b, c, w);
    if ((r == g && g == b && b == 255) || (c == w && w == 127)) {
      this.data.statusKey = "white";
    } else if ((r + g + b) > 0 && c == 0 && w == 0) {
      this.data.statusKey = "rgb";
      let _percent = bizUtil.getPercentByRgb(r, g, b);
      this.data.rgbSliders.percent = _percent;
      this.setData({
        rgbSliders: this.data.rgbSliders
      })
    } else if (r == g && g == b && g == 0 && (c + w) > 0) {
      this.data.statusKey = "cw";
      this.data.cwSliders.percent = w / (c + w) * 100;
      this.setData({
        cwSliders: this.data.cwSliders
      })
    }
    this.setData({
      statusKey: this.data.statusKey
    })
  },

  initStatusListByStatus () {
    // 如果是窗帘，则不走
    if (this.data.devType != "light") {
      return;
    }
    let _array = this.getLightStatusList();
    this.setData({
      statusList: _array
    })
  },
  getLightStatusList (status) {
    
    // 如果是窗帘，则不走
    if (this.data.devType != "light") {
      return [];
    }

    let _status = status || this.data.status;
    console.log("getStatusList _status = ", _status);
    let _allList = this.getAllStatusList();
    console.log("getStatusList _allList = ", _allList);
    let _isNotCW = _status.c == undefined;
    let _isNotRGB = _status.r == undefined && _status.g == undefined && _status.b == undefined;
    console.log("getStatusList _isNotCW, _isNotRGB = ", _isNotCW, _isNotRGB);
    // w灯只有亮度和开关
    if (_isNotCW && _isNotRGB) {
      return [];
    } else if (_isNotCW) {
      let _array = _allList.slice(0, 2);
      _array[0].value = 'rgb';
      return _array;
    } else if (_isNotRGB) {
      let _array = [_allList[0], _allList[2]];
      _array[0].value = 'cw';
      return _array;
    }
    return _allList;
  },
  getAllStatusList () {
    return [{
      name: '白光',
      value: '',
      key: 'white'
    }, {
      name: 'RGB',
      type: "rgb",
      key: 'rgb'
    },{
      name: '色温',
      type: "cw",
      key: 'cw'
    }];
  },

  // 不动作
  customNoAction () {
    let _devInfo = app.globalData.devInfo || {}; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    // 提示等待中
    // this.setShowCustomToast(true, 'loading', '设置中…', 20000);

    // 非删除
    this.data.isDelete = false;
    // 显示等待条
    this.showLoading("正在设置中…");

    let _sid = _sceneInfo.sid, _rid = _sceneInfo.rid || 0;
    app.globalData.blsdk.customSceneAction(_devInfo.a, (_rid << 16) + _sid, (result) => {
      console.error("save result = ", result);
      // 是否失效
      if (this.data.lastAvalible) {
        console.log("outdate");
        return;
      }
      
      // 初始化定时器
      this.destoryTimer();
      // 隐藏等待条
      this.hideLoading();

      if (result < 0 || result === false || result == undefined || result.errCode) {
        // this.setShowCustomToast(true, 'error', '设置失败', true);
        this.setShowCustomToast(true, 'error', '设置失败');
      } else {
        // this.setShowCustomToast(true, 'success', '设置成功', true);
        this.setShowCustomToast(true, 'success', '设置成功');

        setTimeout(this.dealAfterSend, 1000);
      }
    })
  },
  // 当前状态
  currentStatus () {
    // 保存当前状态
    let _devInfo = app.globalData.devInfo || {}; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    // // 提示等待中
    // this.setShowCustomToast(true, 'loading', '设置中…', 20000);

    // 显示等待条
    this.showLoading("正在设置中…");
 
    let _sid = _sceneInfo.sid, _rid = _sceneInfo.rid || 0;
    console.log("currentStatus _sid, _rid = ", _sid, _rid);
    app.globalData.blsdk.roomSceneAddWithDeviceAddr(_devInfo.a, (_rid << 16) + _sid, (result) => {
      console.error("save result = ", result);

      // 是否失效
      if (this.data.lastAvalible) {
        console.log("outdate");
        return;
      }
      
      // 初始化定时器
      this.destoryTimer();
      // 隐藏等待条
      this.hideLoading();

      if (result < 0 || result === false || result == undefined || result.errCode) {
        // this.setShowCustomToast(true, 'error', '设置失败', true);
        this.setShowCustomToast(true, 'error', '设置失败');
      } else {
        // this.setShowCustomToast(true, 'success', '设置成功', true);
        this.setShowCustomToast(true, 'success', '设置成功');
        
        setTimeout(this.dealAfterSend, 1000);
      }
    })
  },

  // 取得自定义设备状态的json数据
  getStatusJson () {
    // 设备状态
    let _status = this.data.status || {};
    if (!_status.pwr) {
      _status = {
        pwr: 0
      };
    } else {
      let _statusKey = this.data.statusKey;
      // 选中白光
      if (_statusKey == 'white' && this.data.statusList.length > 0) {
        _status.statueKey = "white";
        // 白光
        if (this.data.statusList[0].value == 'rgb') {
          _status.r = _status.g = _status.b = 255;
          _status.c = _status.w = 0;
        } else {
          _status.c = _status.w = 127;
          _status.r = _status.g = _status.b = 0;
        }
      } else if (!_statusKey) {
        // 未选择的情况下
        _status.r = _status.g = _status.b = _status.c = _status.w = 0;
        delete _status.statueKey;
      }
    }
    // 设备类型加入，用于校验是哪个设备，走不同得协议
    _status.devTypeStr = this.data.devType;
    // // 如果是窗帘的场景
    // if (_status.devTypeStr == "curtain") {
    //   _status.direction = 8;
    // }
    return _status;
  },

  // 保存群组的场景
  saveSceneGroupInfo () {
    // 取得自定义命令集合
    let _status = this.getStatusJson();
    // 场景信息
    let _sceneInfo = app.globalData.sceneInfo || {};
    // 群组信息
    let _groupInfo = app.globalData.devInfo || {};

    // 根据群组信息取得支持和不支持的设备列表
    let {supports, unsupports} = this.getGroupSceneConfig(_groupInfo, app.globalData.devs);
    console.log("saveSceneGroupInfo supports, unsupports = ", supports, unsupports);
    let _supportsLen = supports.length, _unsupportsLen = unsupports.length;
    if (_supportsLen == 0 && _unsupportsLen == 0) {
      return ability.showToast({
        title: "当前为空群组"
      });
    } else if (_unsupportsLen == 0) {
      this.showLoading("发送中…", true);
      // 所有设备都支持场景群组
      this.sendAllSupportSceneGroup({
        sceneInfo: _sceneInfo,
        groupInfo: _groupInfo,
        status: _status
      }, (res) => {
        ability.hideLoading();
        // 返回数据
        if (res != 0) {
          return;
        }
        // 后续处理
        this.dealAfterSendGroup(supports);
        // 返回页面
        setTimeout(this.back, 500);
      });
    } else {
      this.showLoading("发送中…", true);
      // 只发送自定义的设备
      let _sendArray = unsupports.concat([]);
      // 发送自定义和群组的设备
      if (_supportsLen > 0) {
        // 支持群组场景编辑
        _sendArray.push(_groupInfo);
      }
      console.log("saveSceneGroupInfo _sendArray = ", _sendArray);
      // 发送数据
      this.dealBeforeSend({
        array: _sendArray,
        sceneIdSDK: bizUtil.getSceneId(_sceneInfo?.sid, _sceneInfo?.rid),
        status: _status,
        sceneInfo: _sceneInfo,
        groupInfo: _groupInfo
      }, (res) => {
        ability.hideLoading();
        // 分别把数据加入到场景中
        if (res < 0) {
          ability.showToast({
            title: "下发失败"
          });
          return;
        }
        // 加入数据
        this.dealAfterSendGroup(supports.concat(unsupports));
        // 返回页面
        setTimeout(this.back, 500);
      })
    }
  },
  // 清除定时器和发送数据
  resetGroupSceneTimer () {
    this.sendCount = 0; // 发送起始
    bizUtil.clearTimer(this.intervalTimer, "interval");
  },
  // 不支持群组场景的一条条发送 或者不支持设备与支持群组场景的一条条发送
  dealBeforeSend (config, callback) {

    // 配置的数据
    let {array, sceneIdSDK, status, sceneInfo, groupInfo} = config || {};
    if (!array || array.length == 0 || !status) {
      !!callback && (callback(0));
      return;
    }
    // 发送之前初始化
    this.resetGroupSceneTimer();
    // 开启定时器
    this.intervalTimer = setInterval(() => {
      // 判断当前蓝牙状态，如果没有开启，则退出
      if (!app.globalData.blsdk.isBleAvailable()) {
        // 发送之前初始化
        this.resetGroupSceneTimer();
        !!callback && (callback(-1));
        return;
      }
      // 如果发送完成
      if (this.sendCount >= array.length) {
        // 发送之前初始化
        this.resetGroupSceneTimer();
        // 没有数据表示发送完成
        !!callback && (callback(0));
        return;
      }
      let _info = array[this.sendCount];
      if (!_info) {
        // 没有数据表示发送完成
        this.sendCount++;
        return;
      }
      // 判断发送单个设备还是群组
      if (_info?.gid > 0) {
        console.log("dealBeforeSend group _info = ", _info);
        this.sendAllSupportSceneGroup(config);
      } else {
        // 取得设备状态 根据rgbcw的能力来动态调整
        let _realStatus = bizUtil.getCmdJsonByT(status, _info?.t);
        console.log("dealBeforeSend device _info, status, _realStatus, sceneInfo = ", _info, status, _realStatus, sceneInfo);
        app.globalData.blsdk.sceneManageCtrl({
          // 这里可能是短地址可能是设备信息
          deviceInfo: _info,
          a: _info?.a,
          roomId: sceneInfo?.rid || 0,
          command: {
            actType: 0,
            sid: sceneInfo?.sid || 0,
            cmd: _realStatus
          },
          noRemote: true
        }, {
          noStop: true
        });
      }
      // 发送+1
      this.sendCount++;
    }, 1500);
  },
  // 都支持群组场景的场合
  sendAllSupportSceneGroup ({sceneInfo, groupInfo, status}, callback) {
    // 发送的参数
    let _paramObj = {
      sceneInfo: sceneInfo,
      groupId: groupInfo?.gid,
      cmd: status,
      actType: 0, // 添加/更新情景 1 - 删除情景 2-修改面板默认情景 3-情景例外 4-超级面板
      pid: bizUtil.getSendPidByType(groupInfo?.t)
    };
    // 群组的场景不需要等待返回，需要多发几次
    app.globalData.blsdk.sendGroupScene(_paramObj, (res) => {
      console.log("sendAllSupportSceneGroup res = ", res);
      !!callback && (callback(res));
      // // 返回数据
      // if (res != 0) {
      //   return;
      // }
      // // 后续处理
      // this.dealAfterSendGroup(_paramObj);
    });
  },
  // 取得场景中不支持的数据
  getGroupSceneConfig (groupInfo, allDevs) {
    let _result = {
      supports: [],
      unsupports: []
    };
    let {dids, gid} = groupInfo || {};

    // 判断如果是默认群组,则取得默认群组的房间
    if (gid < 193) {
      // 默认房间的群组
      let _devArray = bizUtil.getDevsOrGroupsByRoomId(gid, app.globalData.devs);
      // 取得群组中的设备品类,符合的才加入 当前只有照明和窗帘有默认群组
      let _devType = this.data.devType;
      let _configJson = bizUtil.getAllPidByArray(_devArray);
      let _devList = _configJson[_devType] || []; // 设备列表
      for (let devInfo of _devList) {
        // 是否支持组的场景编辑
        let {groupScene} = bizUtil.getJsonByAblitys(devInfo?.v, bizUtil.getDevTypeStr(devInfo?.t), {t: devInfo?.t});
        if (!!groupScene) {
          _result.supports.push(devInfo);
        } else {
          _result.unsupports.push(devInfo);
        }
      }
    } else {
      // 固定群组
      if (!dids || dids.length == 0) {
        return _result;
      }
      // 取得所有的设备信息
      let _dMap = bizUtil.getDidMapInJson(allDevs);
      for (let d of dids) {
        let _obj = _dMap[d];
        if (!_obj) {
          continue;
        }
        // 是否支持组的场景编辑
        let {groupScene} = bizUtil.getJsonByAblitys(_obj?.v, bizUtil.getDevTypeStr(_obj?.t), {t: _obj?.t});
        if (!!groupScene) {
          _result.supports.push(_obj);
        } else {
          _result.unsupports.push(_obj);
        }
      }
    }
    return _result;
  },
  dealAfterSendGroup (devArray) {
    // 取得所有的
    if (!devArray || devArray.length == 0) {
      return;
    }
    // 添加设备到场景中
    for (let item of devArray) {
      if (!item) {
        continue;
      }
      this.addDevIntoDevs(item);
    }
  },

  // 自定义设备状态
  saveInfo () {

    // 如果是场景，则发送场景数据
    if (this.isGroup) {
      return this.saveSceneGroupInfo();
    }

    // 根据当前选择的类型进行不同处理
    let _currentIndex = this.data.currentType;
    if (_currentIndex == 2) {
      // 不执行场景
      return this.back();
      // return this.customNoAction();
    } else if (_currentIndex == 1) {
      // 保存当前状态
      return this.back();
      // return this.currentStatus();
    }

    // 更新状态
    this.data.lastAvalible = false;

    // 取得自定义命令集合
    let _status = this.getStatusJson(); 
    console.log("saveInfo status = ", _status);

    // 非删除
    this.data.isDelete = false;
    // 显示等待条
    this.showLoading("正在设置中…");

    let _devInfo = app.globalData.devInfo || {}; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    let _sid = _sceneInfo.sid, _rid = _sceneInfo.rid || 0;
    app.globalData.blsdk.roomSceneAddWithCommand(_devInfo, (_rid << 16) + _sid, _status, (result) => {
      console.error("save result = ", result);
      // 是否失效
      if (this.data.lastAvalible) {
        console.log("outdate");
        return;
      }
      
      // 初始化定时器
      this.destoryTimer();
      // 隐藏等待条
      this.hideLoading();

      if (result < 0 || result === false || result == undefined || result.errCode) {
        // this.setShowCustomToast(true, 'error', '设置失败', true);
        this.setShowCustomToast(true, 'error', '设置失败');
      } else {
        // this.setShowCustomToast(true, 'success', '设置成功', true);
        this.setShowCustomToast(true, 'success', '设置成功');
        // 当前状态status = {}
        setTimeout(this.dealAfterSend, 1000);
      }
    })
  },
  dealAfterSend () {

    // 是否删除
    if (this.data.isDelete) {
      // 取得所有设备
      let _devs = app.globalData.sceneInfo?.devs || [];
      // 取得下表
      let _index = this.editIndex;
      // 删除设备
      _devs.splice(_index, 1);
      // 返回按钮
      setTimeout(this.back, 1000);
      // 重置
      this.data.isDelete = false;
      return;
    }

    // 设备添加
    this.addDevIntoDevs(app.globalData.devInfo);
    
    // console.log("_sceneInfo.devs = ", _sceneInfo.devs);
    // 返回上一页
    this.back();
  },

  // 添加执行设备到场景中
  addDevIntoDevs (devInfo) {
    if (!devInfo) {
      return;
    }
    let _devInfo = devInfo; // 选择的设备
    let _sceneInfo = app.globalData.sceneInfo || {}; // 场景信息
    let _status = this.getStatusByIndex(this.data.currentType); // 根据选择的
    console.log("dealAfterSend _status = ", _status, this.data.devType);
    // 把json的数据转string
    let _statusStr = '';
    let _tip = bizUtil.getTipByStatus(_status, _devInfo.t);
    if (_status == undefined) {
      _statusStr = '';
    } else {
      _statusStr = app.globalData.blsdk.getCmdBySceneInfo(_status, this.data.devType);
    }
    console.log("dealAfterSend = ", _statusStr, _tip, _sceneInfo, _devInfo, this.editIndex);
    // 防止不存在数据
    _sceneInfo.devs = _sceneInfo.devs || [];

    // 判断数组中是否有包含设备d的
    let _oldIndex = this.indexInArrayByKey(_sceneInfo.devs, "d", _devInfo?.d);
    console.log("dealAfterSend _oldIndex = ", _oldIndex);

    // 编辑的数据
    let _index = this.editIndex;
    if (_oldIndex < 0 && (_index < 0 || (_index >= 0 && _devInfo?.cmd == "default"))) {
      // 不存在
      _sceneInfo.devs.push({
        v: _devInfo.v,
        d: _devInfo.d,
        t: _devInfo.t,
        cmd: _statusStr,
        tip: _tip,
        bp: _status?.brightnessP || undefined
      });
    } else if (_index >= 0 || _oldIndex >= 0) {
      let _curIndex = _index >= 0 ? _index : _oldIndex;
      (_sceneInfo.devs[_curIndex] || {}).tip = _tip;
      (_sceneInfo.devs[_curIndex] || {}).cmd = _statusStr;
      (_sceneInfo.devs[_curIndex] || {}).bp = _status?.brightnessP || undefined;
      !!_devInfo.v && ((_sceneInfo.devs[_curIndex] || {}).v = _devInfo.v);
      !!_devInfo.t && ((_sceneInfo.devs[_curIndex] || {}).t = _devInfo.t);
    }
  },
  indexInArrayByKey (array, key, value) {
    if (!array || array.length == 0) {
      return -1;
    } else if (!key) {
      return -1;
    }
    return array.findIndex(item => item[key] == value);
  },

  back () {
    // 返回上一页
    ability.navigateBack();
  },
  
  // 滑块百分比
  sliderChange (event) {
    let _percent = event.detail.percent;
    let _value = event.detail.value;

    let _devTypeStr = this.data.devType;
    // 判断是否窗帘
    if (_devTypeStr == "curtain") {
      this.curtainControl(this.data.status.pwr || this.data.pwrList[0].value, _percent);
      return;
    } else if (_devTypeStr == 'fan') {
      this.fanControl(this.data.status.pwr || this.data.pwrList[0].value, _percent);
      return;
    }
    console.log("sliderChange _value, _percent = ", _value, _percent);
    // let _originValue = event.detail.originValue;
    // 增加处理 保存原始的值（精确到小数点 为了防止下次进入亮度不一致）
    this.data.status.brightness = _value;
    // 判断百分比是不是比最大的小
    let _minPercent = this.data.brightSliders?.minPercent || 0;
    if (_percent < _minPercent) {
      _percent = _minPercent;
    }
    this.data.status.brightnessP = _percent;
    // 执行
    let _status = {
      brightness: _value
    };
    this.actDev(_status, {
      noRemote: true
    });
  },

  rgbSliderChange (event) {
    let _value = event.detail.value;
    console.log("rgbSliderChange _value = ", _value);
    let _rgbs = bizUtil.getRgbByAngle(_value) || [];
    // 增加处理 保存原始的值（精确到小数点 为了防止下次进入亮度不一致）
    this.data.status.r = _rgbs[0];
    this.data.status.g = _rgbs[1];
    this.data.status.b = _rgbs[2];
    this.data.status.c = 0;
    this.data.status.w = 0;

    // 更新标题
    this.updateTitleByType(`rgb(${this.data.status.r}, ${this.data.status.g}, ${this.data.status.b})`, "rgb");

    this.actDev({
      pwr: 1,
      // brightness: this.data.status.brightness || 127,
      r: this.data.status.r,
      g: this.data.status.g,
      b: this.data.status.b
    }, {
      noRemote: true
    });
  },
  cwSliderChange (event) {
    let {percent, value: _value, isMoving} = event.detail;
    // let _value = event.detail.value;
    // let _isMoving = event.detail.isMoving;
    // let _val = event.detail.value;
    // 区间为 1 - 255 默认为0 - 100
    // _val = 1 + parseInt(_val * 2.54);
    console.log("sliderChange _value = ", _value);
    this.data.status.r = 0;
    this.data.status.g = 0;
    this.data.status.b = 0;
    this.data.status.c = 255 - _value;
    this.data.status.w = _value;

    // 更新标题
    this.updateTitleByType(percent, "cw");

    // 执行
    this.actDev({
      pwr: 1,
      brightness: this.data.status.brightness || 127,
      c: this.data.status.c,
      w: this.data.status.w
    }, {
      noRemote: true
    });
  },

  pwrClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.pwrList[index];
    // 发送数据
    this.pwrClickDetail(_item.value || 0);
  },
  pwrClickDetail (value) {
    let _value = value;

    // 如果是窗帘，则走窗帘的控制逻辑
    if (this.data.devType == 'curtain') {
      // let _isOpen = _value == this.data.pwrList[0].value;
      this.curtainControl(_value);
      return;
    }

    this.data.status.pwr = _value;
    console.log("this.data.status.pwr = ", _value);
    this.setData({
      status: this.data.status
    })
    let brightness = _value ? (this.data.status.brightness || 127) : 0;
    // 设备控制
    let _params = {
      brightness: brightness
    };
    if (brightness > 0) {
      _params.pwr = 1;
    }
    this.actDev(_params);
  },
  curtainControl (pwr, percent) {
    let _percent = Math.round(percent == undefined ? (this.data.status.percent || 0) : (Number(percent) || 0));
    this.data.status.percent = _percent;
    console.log("curtainControl this.data.status = ", this.data.status);
    // 下发数据
    this.actDev({
      // mode: _pwr,
      percent: _percent
    });
    // 设置数据
    this.setData({
      status: this.data.status
    })
  },
  fanControl (pwrr, percent) {
    // 风扇的风速
    let _percent = Math.round(percent == undefined ? (this.data.status.speed || 0) : (Number(percent) || 0));
    this.data.status.speed = _percent;
    if (_percent > 0) {
      this.data.status.pwr = 1;
    } else {
      this.data.status.pwr = 0;
    }
    console.log("fanControl this.data.status = ", this.data.status);
    // 下发数据
    this.actDev({
      speed: _percent
    });
    // 设置数据
    this.setData({
      status: this.data.status
    })
  },

  // 取得当前页面控制设备列表数组
  getAllCtrlDevsArray (devInfo) {
    // 是否群组设备
    let {dids, gid} = devInfo || {};
    if (gid >= 193) {
      // 固定群组
      // 取得房间下所有的d对应的详细信息
      let _didMaps = bizUtil.getDidMapInJson(app.globalData.devs);
      // 群组设备列表
      let _groupDevs = dids || [];
      // 群组设备列表详细
      return _groupDevs.map(did => _didMaps[did]);
    } else if (gid < 193) {
      // 房间控制 取得全部照明
      return this.getDevArrayByRoomIdInJson(app.globalData.devs, Number(gid))
    } else if (devInfo) {
      // 单个设备控制
      return [devInfo]
    }
  },

  // 根据房间id取得json中设备集合
  getDevArrayByRoomIdInJson (json, roomId) {
    let _devs = [];
    if (!json) {
      return _devs;
    }
    if (roomId == 0) {
      // 全屋设备
      _devs = bizUtil.getAllDevsFilter(json) || [];
    } else if (roomId % 32 == 0) {
      // 取得楼层数据
      let _devJson = bizUtil.getDevsArrayByFloor(json, bizUtil.getFloorByRoomId(roomId)) || {};
      _devs = bizUtil.getAllDevsFilter(_devJson);
    } else {
      // 全屋设备
      _devs = bizUtil.getAllDevsFilter(json, roomId) || [];
    }
    // 取得照明的设备
    let {light} = bizUtil.getAllPidByArray(_devs);
    console.log("getFuncList _devs = ", _devs);
    return light || [];
  },

  // 根据品类取得功能集合
  getFuncList (devs, config) {
    // 被控设备集合
    let _devs = devs || [];
    
    // 最终集合
    let versions = [], minStatus= {}, maxStatus = {};
    // let _minCount = 20, _maxCount = -1;
    // 遍历数据取得集合
    for (let index in _devs) {
      let {t, v} = _devs[index] || {};
      // 如果版本号v不存在，则continue
      if (v == undefined) {
        continue;
      }
      // // 如果版本号一致，则continue
      // if (versions.indexOf(v) >= 0) {
      //   continue;
      // }
      // 添加版本分支集合
      versions.push(v);
      // 取得能力集
      let _devStatus = bizUtil.getFuncByPid({}, t);
      // 添加最大能力集
      Object.assign(maxStatus, _devStatus);
    }
    console.log("getFuncList maxStatus = ", maxStatus);
    return {
      versions: versions, // 版本能力集合
      maxStatus: maxStatus
    };
  },
  // 是否只包含rgb设备
  checkOnlyRgb (status) {
    if (!status) {
      return false;
    }
    return status.c == undefined && status.w == undefined && status.r != undefined && status.g != undefined && status.b != undefined;
  },
  checkOnlyCW (status) {
    if (!status) {
      return false;
    }
    return status.c != undefined && status.w != undefined && status.r == undefined && status.g == undefined && status.b == undefined;
  },

  itemClick (e) {
    let {index} = e.currentTarget.dataset;
    let _item = this.data.statusList[index];
    this.data.statusKey = _item.key;
    console.log("itemClick this.data.status = ", this.data.status, _item.key);
    // 如果点击的是白光
    if (_item.key == 'white') {
      let {c, w, brightness} = this.data.status;
      this.data.status.statueKey = 'white';

      let _params = {
        pwr: 1,
        brightness: brightness || 127,
      };
      // 判断设备 this.isGroup
      if (app.globalData.devInfo) {
        // 取得设备信息
        let _actDevs = this.getAllCtrlDevsArray(app.globalData.devInfo);
        // 默认房间
        let {maxStatus} = this.getFuncList(_actDevs) || {};
        console.log("itemClick maxStatus = ", maxStatus);
        // 群组设备或者默认群组
        if (!maxStatus.c && !maxStatus.w) {
          _params.r = _params.g = _params.b = 255;
          _params.c = _params.w = 0;
        } else {
          _params.r = _params.g = _params.b = 0;
          _params.c = _params.w = 255;
        }
        // 发送白光
        this.actDev(_params);
      }
      // } else if (app.globalData.devInfo) {
      //   // 单个设备
      //   let _dev
      //   let _status = bizUtil.getCmdJsonByT({
      //     pwr: 1,
      //     brightness: this.data.status?.brightness || 127,
      //     statueKey: "white"
      //   }, app.globalData.devInfo?.t);
      //   // 发送白光
      //   this.actDev(_status);
      // }

      // 更新标题
      this.updateTitleByType("", key);

    } else if (_item.key == "rgb") {
      let _percent = this.data.rgbSliders?.percent || 0;
      this.rgbSliderChange({
        detail: {
          value: bizUtil.getRealValByPercent(_percent, this.data.rgbSliders),
          percent: _percent
        }
      });
    } else if (_item.key == "cw") {
      let _percent = this.data.rgbSliders?.percent || 0;
      this.cwSliderChange({
        detail: {
          value: bizUtil.getRealValByPercent(_percent, this.data.cwSliders),
          percent: _percent
        }
      });
    }

    this.setData({
      statusKey: this.data.statusKey
    })
  },
  actDev (key, config) {
    let _isLight = this.data.devType == 'light';

    if (!_isLight) {
      return;
    }

    // 保存设备信息
    if (!!key && _isLight && (typeof key != "string")) {
      // 如果只有亮度调节成0，则不保存亮度的值
      let _keyArray = Object.keys(key) || [];
      if (_keyArray.length == 1 && key['brightness'] == 0) {
        // 不保存
      } else {
        // 不能assign,会有cw等保存进去
        for (let _k in key) {
          if (this.data.status[_k] != undefined) {
            this.data.status[_k] = key[_k];
          }
        }
      }
    }
    console.log("actDev app.globalData.devInfo, key = ", app.globalData.devInfo, key);

    // 全部不需要远程
    let _config = config || {};
    (_config.noRemote == undefined) && (_config.noRemote = true);

    if (this.isGroup) {
      app.globalData.blsdk.groupControlWithType(false, app.globalData.devInfo?.gid, key, _config);
    } else {
      app.globalData.blsdk.controlWithDevice(app.globalData.devInfo, key, _config);
    }
  },

  // 设置是否显示自定义对话框
  setShowCustomToast (value, type, str, duration) {
    // console.log("setShowCustomToast ability = ", ability);
    if (!value) {
      ability.hideToast();
      // ability.hideLoading()
    } else {
      let _config = {
        title: str || '',
        icon: type || '',
        duration: duration || 1500
      };
      ability.showToast(_config);
    }
  },

  // 删除场景下已经绑定的设备
  roomSceneDeleteWithDeviceAddr (devInfo, sceneInfo, callback) {
    let _devInfo = devInfo || {}; // app.globalData.devInfo
    let {rid, sid} = sceneInfo || {}; // app.globalData.sceneInfo
    app.globalData.blsdk.roomSceneDeleteWithDeviceAddr(_devInfo.a, (rid << 16) + sid, callback)
  },

  // 删除场景下绑定的执行设备
  delDev () {
    // 很奇怪的问题  使用ability调用showModal不提示了
    wx.showModal({
      title: "",
      content: "是否删除该场景下的设备"
    }).then((res) => {
      if (!res || !res.confirm) { return; }
      
      // 显示loading
      this.showLoading();
      // 删除
      this.data.isDelete = true;

      // 设备信息
      let _devInfo = app.globalData.devInfo;

      // 删除设备
      this.roomSceneDeleteWithDeviceAddr(_devInfo, app.globalData.sceneInfo, (result) => {
        // 是否失效
        if (this.data.lastAvalible) {
          return;
        }
        
        // 初始化定时器
        this.destoryTimer();
        // 隐藏等待条
        this.hideLoading();

        console.error("delDev result = ", result);
        let _isSuccess = result >= 0;
        // 如果失败，则不处理后续
        if (!_isSuccess) {
          this.setShowCustomToast(true, 'error', '删除失败');
          return;
        }
        this.dealAfterSend();
      });
    })
  },

  // 设备执行类型设置
  typeItemClick (e) {
    // 设置选中的值
    let {index} = e.currentTarget.dataset;
    this.data.currentType = index || 0;

    // 底部按钮特殊处理
    this.dealActions();
    console.log("typeItemClick this.data.actions = ", this.data.actions);

    this.setData({
      currentType: this.data.currentType,
      // actions: this.data.actions
    });
  },

  // 特殊处理底部按钮
  dealActions () {
    // 所有按钮
    let _actions = this.getAllActions() || [];
    // 设置按钮文字
    // let _actionLen = _actions.length;
    let _currentType = this.data.currentType;
    if (this.editIndex < 0) {
      // 新增时
      if (_currentType >= 1) {
        _actions = [];
      } else {
        _actions = [_actions[1]];
      }
    } else {
      // 编辑时
      if (_currentType >= 1) {
        _actions[1].n = "返回";
        _actions[1].classStr = "finishBtn marginL49";
      }
      // 默认设备不能删除
      let _isDefault = app.globalData.devInfo?.cmd == "default";
      if (_isDefault) {
        _actions = _actions.filter(item => item.handle != "delDev");
      }
    }
    this.data.actions = _actions;
    this.setData({
      actions: this.data.actions
    })
  },

  // 根据不同的index取得不同的默认数据
  getStatusByIndex (index) {
    let _index = (index == undefined) ? this.data.currentType : index;
    if (_index == 1) {
      return {};
    } else if (_index == 2) {
      return undefined;
    } else {
      return this.data.status; 
    }
  },

  manageLoadingDialog (val) {
    this.setData({
      showLoadingDialog: val || false
    })
  },
  showLoading (str, isDefaultLoading) {

    // 如果是需要唤醒的设备
    if ((this.data.devType == 'light' || this.data.devType == 'curtain') && !isDefaultLoading) {
      this.manageLoadingDialog(true);
      // 初始化定时器
      this.initTimer(str);
    } else {
      ability.showLoading({
        title: str || "",
        mask: true
      });
    }
  },
  hideLoading () {
    if (this.data.showLoadingDialog) {
      this.manageLoadingDialog(false);

      // 取消进一步提示
      this.data.loadingConfig.forward = false;
      this.setData({
        loadingConfig: this.data.loadingConfig
      })
      return;
    }

    ability.hideLoading();
  },
  initTimer (msg) {
    // 先清除定时器
    this.destoryTimer();
    // 初始化定时器
    this.data.timer = setTimeout(() => {
      // 如果定时器还存在，则退出
      if (!this.data.timer) {
        return;
      }
      // 进一步提示
      this.data.loadingConfig.forward = true;
      this.data.loadingConfig.title = msg || "";
      if (this.data.devType == "curtain") {
        this.data.loadingConfig.content = "请确认设备是否有状态响应或抖动？";
        this.data.loadingConfig.cancelText = "未抖动";
        this.data.loadingConfig.confirmText = "抖动";
      } else {
        this.data.loadingConfig.content = "";
      }
      // this.data.loadingConfig.content = (this.data.devType == "curtain") ? "请确认设备是否有状态响应或抖动？" : "";
      this.setData({
        loadingConfig: this.data.loadingConfig
      })
    }, 5000);
  },
  destoryTimer (timer) {
    let _timer = timer || this.data.timer;
    if (!_timer) {
      return
    }
    clearTimeout(_timer);
    _timer = null;
  },
  // 对话框
  dialogClick (e) {
    let {data} = e.detail || {};
    // 是否显示对话框
    let _showLoadingDialog = false;
    if (data == 'retry') {
      // 重试按钮点击 不做
      // // 上一个修改房间方法是否过期
      // this.data.lastAvalible = true;
      // // 修改房间的方法
      // modifyFunc: undefined
    } else if (data == 'cancel') {
      // // 未响应
      // _showLoadingDialog = true;
      // 停掉之前的数据
      app.globalData.blsdk.destory();
      // 去掉接受的参数
      this.data.lastAvalible = true;
    } else if (data == 'confirm') {
      // 设置成功之后
      this.dealAfterSend();
      // 停掉之前的数据
      app.globalData.blsdk.destory();
      // 去掉接受的参数
      this.data.lastAvalible = true;
    }
    // 是否展示
    this.manageLoadingDialog(_showLoadingDialog);
    // 数据提示
    this.data.loadingConfig.forward = false;
    // this.data.showLoadingDialog = _showLoadingDialog;
    this.setData({
      // showLoadingDialog: this.data.showLoadingDialog,
      loadingConfig: this.data.loadingConfig
    })
  },

  updateTitleByType (percent, type) {
    let _list = this.data.statusList || [];
    let _valStr = "";
    if (type == "cw") {
      _valStr = bizUtil.getCWValueByPercent(percent || 0);
    } else if (type == "rgb") {
      _valStr = percent || "";
    } else if (type == "white") {
      _valStr = percent || "";
    }

    let _statusList = this.setTitleInArray(_list, type, _valStr);

    console.log("updateTitleByType _valStr, _statusList = ", _valStr, _statusList);
    this.setData({
      statusList: _statusList
    });
  },
  setTitleInArray (array, typeStr, value) {
    if (!array) {
      return [];
    }
    for (let item of array) {
      let {name, type} = item || {};
      let _nameFormat = name?.split(" ")[0] || "";
      if (type == typeStr) {
        item.name = _nameFormat + " " + (value || "");
      } else {
        item.name = _nameFormat;
      }
    }
    return array;
  },
})