import ability from "../../../utils/classDef/ability"
const app = getApp();
class Websocket {
  constructor ({url, onOpen, onMessage, onError}) {
    console.log("constructor url = ", url);
    // 创建
    let _socket = this.createSocket({
      url
    });
    // 增加监听
    this.onOpen(_socket, onOpen);
    this.onMessage(_socket, onMessage);
    this.onError(_socket, onError);
    // 保存socket
    this.socket = _socket;
    // 是否处于readyState=open状态 0-连接尚未建立 1-WebSocket的链接已经建立 2-连接正在关闭 3-连接已经关闭或不可用
    this.readyState = 0;
    // 计数
    this.timeCount = Date.now() / 1000;
    // 定时器
    this.interval = 30000; // 单位30s
  }
  createSocket (params) {
    return ability.connectSocket({
      url: params?.url || "",
    })
  }
  close (socket = this.socket) {
    if (!socket) {
      console.error("close socket = ", socket);
      return;
    }
    // 连接正在关闭
    this.readyState = 2;
    socket.close();
  }
  onClose (socket = this.socket) {
    if (!socket) {
      console.error("onClose socket = ", socket);
      return;
    }
    socket.onClose(res => {
      console.log("onClose res = ", res);
      // 连接已经建立
      this.readyState = 3;
    })
  }
  onError (socket = this.socket, callback) {
    if (!socket) {
      console.error("onError socket = ", socket);
      return;
    }
    socket.onError(res => {
      console.log("onError res = ", res);
      // 回调
      !!callback && (callback(res));
    })
  }
  onMessage (socket = this.socket, callback) {
    if (!socket) {
      console.error("onMessage socket = ", socket);
      return;
    }
    socket.onMessage(res => {
      console.log("onMessage res = ", res);
      // 回调
      if (!!res && res.data && (typeof res.data) == "string") {
        !!callback && (callback(JSON.parse(res.data)));
      } else {
        !!callback && (callback(res));
      }
    })
  }
  onOpen (socket = this.socket, callback) {
    if (!socket) {
      console.error("onOpen socket = ", socket);
      return;
    }
    console.log("onOpen");
    socket.onOpen(res => {
      console.log("onOpen res = ", res);
      // 连接已经建立
      this.readyState = 1;
      // 回调
      !!callback && (callback());
    })
  }
  send (params, socket = this.socket) {
    if (this.readyState == 1) {
      this.sendDetail(params, socket);
    } else {
      console.error("send error readyState = ", this.readyState);
    }
  }
  sendDetail (params, socket) {
    if (!socket) {
      console.error("send socket = ", socket);
      return;
    }
    console.log("sendDetail params = ", params);
    socket.send({
      data: params?.data || "",
      success: (res) => {
        console.log("send success res = ", res);
      },
      fail: (e) => {
        console.log("send fail e = ", e);
      },
      complete: (e) => {
        console.log("send complete e = ", e);
      }
    })
  }

  // 初始化
  getInitData ({uuid, sid, skey}) {
    return JSON.stringify({
      "msgtype": "init",
      "messageid": this.getMessageId(),
      "scope": {
        "scopetype": "familyidtoken",
        "userid": uuid || "",                // 手机UUID即可
        "familyid": sid || "",              // 当前家庭ID
        "token": skey || ""                  // 当前家庭token
      },
      "data": {
        "relayrule": "share",           // 连接类型: "share" or "single"
      }
    })
  }
  // 取得设备订阅数据
  getSubscribeData (gatewayList) {
    return JSON.stringify({
      "msgtype" : "sub",
      "topic" : "devpush",
      "messageid": this.getMessageId(),
      "data": {
        "devList": gatewayList || [] // 订阅设备列表, 只需要网关即可
        // {
        //   "endpointId": "00000000000000000000ec0baee55bb0",    // 订阅设备endpointId
        //   "pid": "00000000000000e0ac000000",    // 订阅设备 pid
        //   "gatewayId": "xxxxxxx",    // 当前家庭ID
        //   "token": "xxxxxxx",  // 当前家庭token
        // }
      }
    })
  }
  // 取得取消订阅数据
  getCancelSubData (gatewayList) {
    return JSON.stringify({
      "msgtype" : "unsub",
      "topic" : "devpush",
      "messageid": this.getMessageId(),
      "data": {
        "devList": gatewayList || []
      }
    })
  }
  // 取得心跳
  getHeartbeat () {
    return JSON.stringify({
      "msgtype" : "ping",
      "messageid" : this.getMessageId(),
    })
  }

  // 取得消息id
  getMessageId () {
    return app.globalData.blsdk.getMessageId(); // "wxb6ecbcea30e9283b-" + parseInt(Date.now() / 1000);
  }
  // 定时器
  startInterval () {
    
  }
}
// let _websocket = new Websocket();
module.exports = Websocket;