import {Utils} from './util'
import DEVCONFIG from './classDef/devs'
import ADAPTERAPP from "../static/js/jsConfig/adaptApp"
import AirConfig from "../static/js/jsConfig/aircondition"

module.exports = {
  initMultip (systemInfo) {
    // 屏幕的宽度 获取px与rpx之间的比列 
    var winWidth = ((systemInfo || {}).safeArea || {}).width || 375;
    return 750 / winWidth;
  },

  sortArrayByKey (array, key) {
    if (!array || array.length == 0) {
      return [];
    }
    if (key == undefined) {
      return array;
    }
    return array.sort((a, b) => {
      return (a[key]) - (b[key])
    })
  },
  // 特殊处理房间楼层排序
  sortArrayByKeyInFloorRoom (array, key) {
    if (!array || array.length == 0) {
      return [];
    }
    if (key == undefined) {
      return array;
    }
    let _func = (value) => {
      if (value == undefined || value == 0) {
        return -2;
      } else if (value % 32 == 0) {
        return -1;
      } else {
        return value % 32;
      }
    }
    return array.sort((a, b) => {
      let _aKey = _func(a[key]);
      let _bKey = _func(b[key]);
      return (_aKey) - (_bKey)
    })
  },

  // 根据key取得数组中的下标
  getIndexByKeyInArray(array, val, key) {
    if (!array || array.length == 0) {
      return -1;
    }
    for (let i = array.length - 1; i >= 0; i--) {
      let obj = array[i] || {};
      // 是否相等
      let _val = (key == undefined) ? (obj) : (obj[key]);
      if (_val == val) {
        return i;
      }
    }
    return -1;
  },

  // 根据房间id取得sort数组
  getAllDevsBySort (sortArray, jsonData, keyStr) {
    if (!sortArray) {
      return {
        devs: [],
        scenes: [],
        groups: []
      };
    }
    let _devs = []; // 设备
    let _scenes = []; // 场景
    let _groups = []; // 群组
    let _filter = {};

    for (let i in sortArray) {
      let _item = sortArray[i];
      let id = _item.id;
      let _jsonData = jsonData[id] || {};

      if (!!keyStr && keyStr== 'scenes') {
        let _array = _jsonData[keyStr] || [];
        for (let index in _array) {
          let item = _array[index];
          let {sid} = item;
          if (_filter[sid]) {
            continue;
          }
          _filter[sid] = true;
          _scenes.push(item);
        }
      } else {
        let _ts = _jsonData['devs'] || [];
      
        // 设备列表
        _devs = _devs.concat(_ts);
        // 场景列表
        _scenes = _scenes.concat(_jsonData['scenes'] || []);
        _groups = _groups.concat(_jsonData['groups'] || []);
      }
    }
    return {
      devs: _devs,
      scenes: _scenes,
      groups: _groups
    };
  },

  // 根据房间取得所有的设备
  getAllDevsFilter (jsonData, roomId, key) {
    let _result = [];
    let _json = jsonData || {};
    let _key = key || "devs";
    if (roomId != undefined) {
      // console.log("getAllDevsFilter jsonData[roomId] = ", jsonData[roomId].devs);
      return (jsonData[roomId] || {})[_key] || [];
    }
    // console.log("getAllDevsFilter jsonData, roomId = ", jsonData, roomId);
    for (let rId in _json) {
      let _item = _json[rId] || {};
      if (!_item[_key]) {
        continue;
      }
      _result = _result.concat(_item[_key] || []);
    }
    return _result;
  },

  // 设置每个数组的key对应的value
  setArrayItem (array, keys, vals) {
    let _array = array || [];
    // console.log("setArrayItem array, keys, vals = ", _array, keys, vals, _array instanceof(Array));
    if (!_array instanceof(Array)) {
      console.error("setArrayItem _array isnot array = ", _array);
      return [];
    }
    for (let idx in _array) {
      let obj = _array[idx];
      for (let index in keys) {
        let _key = keys[index];
        let _val = vals[index];
        // console.log("setArrayItem _key, _val = ", _key, _val);
        if (_key == 'n' && _val == 'defaults') {
          obj[_key] = obj[_key] || '';
        } else if (_val == 'index') {
          obj[_key] = idx;
        } else {
          obj[_key] = _val;
        }
      }
    }
    return _array;
  },

  // 根据所有设备信息取得每个房间的设备列表
  initDevByRooms (allDevs, rooms, isInit) {
    // 把所有的设备转成json格式
    let _devJson = {};
    let _devs = allDevs || {};
    if (_devs instanceof(Array)) {
      _devJson = this.arrayToJsonByKey(_devs);
    } else {
      _devJson = _devs;
    }
    console.log("initDevByRooms _devJson = ", _devJson);
    // 补上没有设备的房间数据
    // let _roomJson = this.arrayToJsonByKey(rooms, true);
    console.log("initDevByRooms rooms = ", rooms);
    let _result = {};
    // 设置数据
    for (let index in rooms) {
      let _item = rooms[index];

      let _existJson = _devJson[_item.id] || {};
      _item.devs = _existJson.devs || []; // 照明设备
      _item.scenes = _existJson.scenes || (isInit ? this.getDefaultScenes(_item.id) : []); // 场景
      // 保存房间名称
      _item.n = _existJson.n || _item.n;
      _result[_item.id] = _item;
    }
    console.log("initDevByRooms _result = ", _result);
    return _result; // Object.assign(_roomJson, _devJson);
  },
  getDidMapInJson (json) {
    let _result = {};
    if (!json) {
      return _result;
    }
    for (let rid in json) {
      let {devs} = json[rid] || {};
      if (!devs || devs.length == 0) {
        continue;
      }
      Object.assign(_result, this.arrayToJsonByKey(devs, true, 'd'));
    }
    return _result;
  },
  arrayToJsonByKey (devs, isJson, key) {
    let _devs = devs || [];
    let _result = {};
    let _key = (key != undefined ? key : 'id');
    for (let item of _devs) {

      // 增加特殊处理
      let _id = 0;
      if (key == "sid+rid") {
        _id = this.getSceneId(item?.sid, item?.rid);
      } else {
        _id = item[_key] || 0;
      }

      if (isJson) {
        _result[_id] = item;
      } else {
        let _array = _result[_id] || [];
        _array.push(item)
        _result[_id] = _array;
      }
    }
    return _result;
  },

  // 根据时间设置tip
  getDefaultTip () {
    // 设置图片
    let _imgStr = "";
    // 设置时间 早上 中午 晚上（18-24）
    let _hour = new Date().getHours();
    let _timeName = "";
    if (_hour >= 6 && _hour < 11) {
      _timeName = "早上";
      _imgStr = "moring";
    } else if (_hour >= 11 && _hour < 13) {
      _timeName = "中午";
      _imgStr = "noon";
    } else if (_hour >= 13 && _hour < 19) {
      _timeName = "下午";
      _imgStr = "noon";
    } else {
      _timeName = "晚上";
      _imgStr = "night";
    }
    // console.log("_timeName = ", _timeName);
    return {
      tip: `Hi ${_timeName}好`,
      imgStr: _imgStr
    };
  },
  // 根据用户信息设置显示的图标
  initByUserInfo (userInfo, topConfig) {
    if (!userInfo || !userInfo.avatarUrl) {
      let _configSet = userInfo || {};
      // 图标
      topConfig.topObj.left.img = _configSet.img || "../../static/img/icon.png";
      // 未登录
      topConfig.topObj.left.text = _configSet.nickname || (_configSet.nickname = "");
      // 向右的图标
      topConfig.topObj.left.rightIcon = true;
      // tip设置
      topConfig.topObj.left.tip = undefined;
    } else {
      // 图标
      topConfig.topObj.left.img = userInfo.avatarUrl;
      // 姓名
      topConfig.topObj.left.text = userInfo.nickname;
      // tip设置
      topConfig.topObj.left.tip = this.getDefaultTip().tip;
      // 向右的图标
      topConfig.topObj.left.rightIcon = false;
    }

    return topConfig;
  },

  // 取得设备的版本号 由于版本号数据结构会有变动，所以放在这里
  getVersionArrayByV (vStr) {
    // 版本号
    let _v = vStr || "";
    // 取得版本
    let _vArray = _v.split(".");
    // 根据版本号的长度来判断是否有多个字节
    let _len = _vArray.length;
    if (_len < 3) {
      return [0, 0];
    } else if (_len > 4) {
      // 有4个以上ability的数据
      return [_vArray[2], _vArray[3]];
    } else {
      return [0, _vArray[2]];
    }
  },
  // 取得完整的6个字节的能力集合
  getAllSixAbilityByV (v) {
    let _result = new Array(6).fill(0);
    // 版本号
    let _vArray = [0, 0];
    if (v instanceof Array) {
      _vArray = v;
      console.log("getAllSixAbilityByV _vArray = ", _vArray);
    } else {
      if (typeof v != "string") {
        console.log("getAllSixAbilityByV v = ", v);
        return _result;
      }
      _vArray = this.getVersionArrayByV(v);
    }
    console.log("getAllSixAbilityByV _vArray = ", _vArray);
    // 取得所有的版本号
    let _byte0 = Number(_vArray[0]) || 0; // 高位
    let _byte1 = Number(_vArray[1]) || 0; // 低位
    console.log("getAllSixAbilityByV _byte0, _byte1 = ", _byte0, _byte1);
    // 更新数据
    if (_byte0 > 0) {
      _result[4] = _byte0 & 0xff;
      _result[5] = (_byte0 >>> 8) & 0xff;
    }
    if (_byte1 > 0) {
      _result[0] = _byte1 & 0xff;
      _result[1] = (_byte1 >>> 8) & 0xff;
      _result[2] = (_byte1 >>> 16) & 0xff;
      _result[3] = (_byte1 >>> 24) & 0xff;
    }
    return _result;
  },
  // 取得能力的最大集合
  getMaxAbilityByVs (vs) {
    let _result = {};
    if (!vs || vs.length == 0 || !(vs instanceof Array)) {
      return _result;
    }
    // let _byte1 = 0, _byte0 = 0;
    let _excepts = ["minBrightness", "1024", "gradient"]; // 取得交集的集合
    // 遍历数据
    for (let vStr of vs) {
      // 如果为空则不获取
      if (!vStr) {
        continue;
      }
      // 需要取交集的数据亮度，滑动时下发渐变，1024灰度，其余都取并集
      // 取得ability json集合
      let _json = this.getJsonByAblitys(vStr);
      for (let key in _json) {
        let _val = _json[key] || 0;
        if (_excepts.indexOf(key) >= 0) {
          // 特殊处理的数据
          if (key == "minBrightness") {
            // 最小亮度值是个json
            let _bVal = _val?.value || 0;
            // 取得保存的数据
            let _oldVal = _result[key]?.value || 0;
            if (_oldVal < _bVal) {
              _result[key] = _val;
            }
          } else {
            _result[key] = _result[key] == undefined ? _val : (_result[key] & _val);
          }
          continue;
        }
        _result[key] = _result[key] == undefined ? _val : (_result[key] | _val);
      }
    }
    return _result;
  },
  // 根据能力取得所有的能力集合
  getJsonByAblitys (v, devTypeStr, extendJson) {
    if (v == undefined) {
      return {};
    }
    // 扩展字段 放不下
    let _extendJson = extendJson || {};
    // 总共6个字节的数组
    let _ablityArray = this.getAllSixAbilityByV(v);
    console.log("_ablityArray = ", _ablityArray);
    let _val = _ablityArray[0] || 0; // 第一个能力
    let _val1 = _ablityArray[1] || 0; // 第二个能力
    let _val2 = _ablityArray[2] || 0; // 第三个能力
    let _val3 = _ablityArray[3] || 0; // 第四个能力
    let _val4 = _ablityArray[4] || 0; // 第五个能力
    let _val5 = _ablityArray[5] || 0; // 第六个能力

    // 如果是ihg，目前只取第一个字节
    if (!!devTypeStr && devTypeStr == "ihg") {
      return {
        byte0: _val
      }
    }

    // 取得最低亮度的配置
    let _minBrightness = {
      0: {
        percent: 1,
        value: 2
      },
      1: {
        percent: 5,
        value: 7
      },
      2: {
        percent: 10,
        value: 13
      },
      3: {
        percent: 15,
        value: 20
      }
    };
    // 取得场景配置
    let _sceneCount = {
      0: 10,
      1: 16,
      3: 64
    };
    // 默认灯泡的能力集合
    let abilityJson = {
      remote: _val & 1, // 是否支持遥控器
      timer: (_val >> 1) & 1, // 定时
      delay: (_val >> 2) & 1, // 延时关闭
      scene: (_val >> 3) & 1, // 是否支持自定义场景 只有自有产品才支持
      gradient: (_val >> 4) & 1, // 颜色切换渐变
      sleep: (_val >> 5) & 1, // 起床/睡眠情景模式
      minBrightness: _minBrightness[(_val & 0xc0) >> 6], // 最低亮度

      "1024": (_val1 & 1), // 1024灰度
      sceneException: (_val1 >> 1) & 1, // 场景例外
      tempGroup: (_val1 >> 2) & 1, // 临时group
      cwGradient: (_val1 >> 5) & 1, // cw灯的渐变跳变能力
      sceneLen: _sceneCount[_val1 >> 6] || _sceneCount[0], // 设备支持的自定义场景个数

      // type 
      /* 1	主灯
      2	吸顶灯
      3	灯带
      4	筒射灯
      5	磁吸筒灯 */
      type: (_val2 >> 4) & 0x0f,

      // alexa使能
      alexaSet: (_val3 >> 0) & 0x1,
      // C和CW照明设备白光渐变跳变支持
      cAndcwWhite: (_val3 >> 1) & 0x1,
      // 炫彩灯带透传支持
      xuancai: (_val3 >> 2) & 0x1,
      // 基于组的场景编辑
      groupScene: (_val3 >> 3) & 0x1,
      // 缓起缓灭 支持渐变功能开打开关闭以及时间设置
      slowUpDown: (_val3 >> 4) & 0x1,
      // 上电勿扰
      noDisturb: (_val3 >> 6) & 0x1,
      // 凌动开关
      lingdong: (_val3 >> 7) & 0x1,

      // 配网锁定
      configLock: (_val4 >> 0) & 0x1,
      // 是否支持mic收音  0不支持 1支持
      mic: (_val4 >> 2) & 0x1,
      // 支持稳定查询
      // 主设备使能
      mainEnable: (_val4 >> 4) & 0x1,
    }

    // 如果非灯泡等则默认支持自定义场景 
    if (devTypeStr == "coathanger") {
      abilityJson.scene = 1; // 晾衣架支持自定义场景
      abilityJson.sceneException = 0; // 场景例外 晾衣架只支持自定义
    } else if (devTypeStr == "panel") {
      abilityJson.backlight = (_val2 & 0x01);

      console.log("getDetailListByV _extendJson, _val2 = ", _extendJson, _val2);

      // 是否继电器开关面板
      if (this.isPwrAvalibleDev(_extendJson.t)) {
        // 是否支持双控
        abilityJson.virtualPanel = (_val2 >> 1) & 0x01;
        // 是否支持超级面板使能
        abilityJson.metapadEnable = (_val2 >> 2) & 0x01;
        // 如果支持超级面板功能，则设置支持的品类和个数
      }

    } else if (devTypeStr == "uart") {
      // 超级面板是否支持背光灯
      abilityJson.backlight = _val2 & 0x01;
      // 超级面板是否低功耗
      abilityJson.lowPower = abilityJson.remote;
    } else if (devTypeStr == "light") {
      // 是否支持风扇灯能力
      abilityJson.isFanLight = (_val4 >> 1) & 0x1;
    }

    return abilityJson;
  },
  // 取得温控器的品类
  getThermostatAbility (v, pidStr) {
    let _result = {};
    if (v == undefined) {
      return _result;
    }
    // 取得设备类型
    let _devTypeStr = pidStr; // this.getDevTypeStr(pid);
    // 总共6个字节的数组
    let _ablityArray = this.getAllSixAbilityByV(v);
    console.log("_ablityArray = ", _ablityArray);
    let _val = _ablityArray[0] || 0; // 第一个能力

    // 空调第一个字节的能力
    let _airhotwind = _val & 0x07;
    _result.airhotwind = _airhotwind;

    // 如果是空调网关，则返回单空调数据
    if (_devTypeStr == "aircondition") {
      _result.air = true;
    } else if (_devTypeStr == "thermostat") {
      if (_airhotwind <= 1) {
        _result.air = true;
        // _result.hot = true;
        // _result.wind = true;
      } else if (_airhotwind == 2) {
        _result.hot = true;
      } else if (_airhotwind == 3) {
        _result.wind = true;
      } else if (_airhotwind == 4) {
        _result.hot = true;
        _result.air = true;
      } else if (_airhotwind == 5) {
        _result.wind = true;
        _result.air = true;
      } else if (_airhotwind == 6) {
        _result.wind = true;
        _result.hot = true;
      } else if (_airhotwind == 7) {
        _result.wind = true;
        _result.hot = true;
        _result.air = true;
      }
      // if (_val == 0) {
      //   _result.air = true;
      // } else {
      //   _result.air = (_val >> 0) & 0x01;
      //   _result.hot = (_val >> 1) & 0x01;
      //   _result.wind = (_val >> 2) & 0x01;
      // }
    }
    return _result;
  },

  // 命名空间
  getNameSpace (key) {
    let _format = "bles_";
    let _config = {
      bh: {
        namespace: _format + "bh",
        wxName: "奔豪智能照明",
        host: "https://bcs.pub/b90865/fNgCBCiK2L",
        version: '1.0.3',
        qrVersion: "5.0.0",
        appid: "wx051cb115b06f1fbe"
      },
      bl: {
        namespace: _format,
        wxName: "易控全屋",
        host: "https://bcs.pub/ais31b/bSCaU84xqp", // bSCaU84xqp
        version: '3.2.027',
        qrVersion: "6.1.0",
        appid: "wxb6ecbcea30e9283b"
      },
      qm: {
        namespace: _format + "qm",
        wxName: "启明智云",
        host: "https://bcs.pub/ais33b/hapUVTeBrP",
        version: '1.0.4',
        qrVersion: "5.0.0",
        appid: "wx4721e7fda632d311"
      },
      sun: {
        namespace: _format + "sun",
        wxName: "阳光智能窗帘",
        host: "https://bcs.pub/ais32b/eFq2iE2QHA",
        version: '1.0.0',
        qrVersion: "5.0.0",
        appid: "wxeb768e658862a6db"
      },
      wistar: {
        namespace: _format + "wistar",
        wxName: "阳光智能窗帘",
        host: "https://bcs.pub/ais47b/np3xs3qEw5",
        version: '1.0.51',
        qrVersion: "5.0.0",
        appid: "wxf29c2380c75ad4ef"
      },
      uart: { // 透传工具
        namespace: _format + "uart",
        wxName: "蓝牙ble透传",
        host: "https://bcs.pub/683c51/cPV63joyly",
        version: '3.1.9',
        qrVersion: "5.0.0",
        // wxdc04b3bce3d61bd5 个人测试版本
        appid: "wxdc04b3bce3d61bd5", // "wx6d48c1ab0c4f5e42"
      },
      test: { // 透传工具
        namespace: _format + "test",
        wxName: "测试用",
        host: "https://bcs.pub/ais31b/bSCaU84xqp",
        version: '3.0.352',
        qrVersion: "5.0.0",
        appid: "wxb6ecbcea30e9283b",
      },
      mg: { // 蒙光智能
        namespace: _format + "mg",
        wxName: "蒙光智能",
        host: "https://bcs.pub/ais48b/MTSMN1sSdW",
        version: '1.0.3',
        qrVersion: "4.0.0",
        appid: "wx6c49d6ac690ed427",
      },
      "aux": { // aux
        namespace: _format + "aux",
        wxName: "Aux",
        host: "https://bcs.pub/ais31b/bSCaU84xqp",
        version: '3.1.91',
        qrVersion: "5.0.0",
        appid: "wxdc04b3bce3d61bd5",
        magicArray: [0x41, 0x53]
      },
      yl: { // 奕朗照明
        namespace: _format + "yl",
        wxName: "奕朗照明",
        host: "https://bcs.pub/ais48b/SkvGeSsLOA",
        version: '1.0.1',
        qrVersion: "5.0.0",
        appid: "wxfa11ddf687e8e475",
      }, 
      cp: { // CP轻智能
        namespace: _format + "cp",
        wxName: "CP轻智能",
        host: "https://bcs.pub/ais48b/Mn7OhUWmcs",
        version: '1.0.0',
        qrVersion: "5.0.0",
        appid: "wx1c4f85d8d6804b92",
      }, 
      wistarwi: { // wistarwi
        namespace: _format + "wistarwi",
        wxName: "Wi智能",
        host: "https://bcs.pub/ais48b/5TYQqGwv11",
        version: '1.0.0',
        qrVersion: "5.0.0",
        appid: "wx6bfebf6eeb6a91c7",
      },
      magic: { // 博联魔法家
        namespace: _format + "magic",
        wxName: "博联魔法家",
        host: "https://bcs.pub/ais31b/oiTQsJQgMb",
        version: '2.4.0',
        qrVersion: "5.0.0",
        appid: "wxa7503e653edd4a63",
      }, 
      xs: { // 小顺
        namespace: _format + "xs",
        wxName: "格际智能",
        host: "https://bcs.pub/ais48b/tNwyqFjfnL",
        version: '3.1.24',
        qrVersion: "6.1.0",
        appid: "wxda583c9818dbd4db",
      },
      cf: { // 翠芳
        namespace: _format + "cf",
        wxName: "爱曲艺斯智能",
        host: "https://bcs.pub/ais48b/8xYBnn3ZZO",
        version: '3.1.1',
        qrVersion: "6.1.0",
        appid: "wxcb211deca656cb29",
      },
      xiandai: {
        namespace: _format + "xiandai",
        wxName: "現代智能",
        host: "https://bcs.pub/ais48b/S4h5uXzocA",
        version: '3.1.1',
        qrVersion: "6.1.0",
        appid: "wx7c0673701feeb38c",
      },
      leishi: {
        namespace: _format + "leishi",
        wxName: "小雷易家",
        host: "https://bcs.pub/ais48b/kOoWke6vrV",// https://bcs.pub/ais48b/bSCaU84xqp
        version: '1.0.01', // 3.1.0
        qrVersion: "6.1.0",
        appid: "wxbd8ad41efd3107e1",
        
        "lid":"3a4a15cce1df9a5ce402c2b4ecb515bb",
        "uid":"9d459dd2a6f6f0bbbf0113c378d5d4aa", // companyid
        "license":"OkoVzOHfmlzkAsK07LUVu3TLXdZ2ByUHOev2iAKndBjmat1O+tMMnbgkXR9Lgc8NHrS7YwAAAADoXExyLpe0e2jRza69U95I53HNHjgxNIttrohVQYuJALxlJHxwpE7+Qc8UBlSfYk74n2W4N5MzgfRJF9oJnDwLnUWd0qb28Lu/ARPDeNXUqgAAAAA=",
      }
    };
    return _config[key] || _config['bl'];
  },
  getDevTypeStr (pid) {
    let _pidNum = Number(pid);
    if (typeof pid == "string" && _pidNum != pid) {
      return pid;
    } else if (!_pidNum) {
      return "";
    }
    let _keys = Object.keys(DEVCONFIG) || [];
    for (let key of _keys) {
      let _item = DEVCONFIG[key]?.arrays || [];
      if (_item.length == 0) {
        continue;
      }
      if (_item.indexOf(_pidNum) < 0) {
        continue;
      }
      return key;
    }
    return "";
  },
  isCwrgb (pid) {
    return this.getDetailType(pid, "kcwrgb", "light")
  },
  isCw (pid) {
    return this.getDetailType(pid, "kcw", "light")
  },
  isRGB (pid) {
    return this.getDetailType(pid, "krgb", "light")
  },
  isRGBW (pid) {
    return this.getDetailType(pid, "krgbw", "light")
  },
  isRGBCW (pid) {
    return this.getDetailType(pid, "krgbcw", "light")
  },
  isSixLight (pid) {
    return this.getDetailType(pid, "sixLight", "light")
  },
  isXuancai (pid, v) {
    let _devTypeStr = this.getDevTypeStr(pid);
    let {xuancai} = this.getJsonByAblitys(v)
    return _devTypeStr == "light" && xuancai; // this.getDetailType(pid, "xuancai", "light")
  },
  // 非耦合设备
  isUncoupleMetapad (pid) {
    return this.getDetailType(pid, "uncouple", "uart")
  },
  isOneSocket (pid) {
    return this.getDetailType(pid, "one", "powerstrip")
  },
  neednotSetLimit (pid) {
    return this.getDetailType(pid, "dream", "curtain") || this.getDetailType(pid, "open", "curtain");
  },
  getDetailType (pid, key, parentKey) {
    let _parentKey = parentKey || "light";
    let _data = DEVCONFIG[_parentKey] || {};
    return _data[key]?.indexOf(pid) >= 0;
  },
  isOpenCurtain (pid) {
    return this.getDetailType(pid, "open", "curtain")
  },
  isRollCurtain (pid) {
    return this.getDetailType(pid, "role", "curtain")
  },
  isDreamCurtain (pid) {
    return this.getDetailType(pid, "dream", "curtain")
  },
  isPwrAvalibleDev (pid) {
    let {panel} = DEVCONFIG;
    return panel?.pwrAvalible?.indexOf(pid) >= 0
  },
  isIhg (pid) {
    let {gateway} = DEVCONFIG;
    return gateway?.ihg?.indexOf(pid) >= 0
  },
  isNormalPanel (pid) {
    let {panel} = DEVCONFIG;
    return panel?.normal?.indexOf(pid) >= 0
  },
  isPwrLightMetapad (pid) {
    return this.getDetailType(pid, "pwrLight", "uart")
  },
  isAbilityThresMetapad (pid) {
    return this.getDetailType(pid, "abilityThree", "uart")
  },
  isGatewayDev (pid) {
    let {gateway} = DEVCONFIG;
    return gateway.arrays?.indexOf(pid) >= 0
  },
  getDefaultScenes (roomId) {
    // 原始场景信息
    let SCENES = this.getScensConfig();
    if (roomId == undefined) {
      return SCENES;
    }
    return SCENES[roomId] || []
  },
  getOriginRooms () {
    // return ROOMS;
    return [
      {
        n: "全屋",
        id: 0,
        sort: 0,
      }, 
      {
        n: "客厅",
        id: 1,
        sort: 1,
      }, {
        n: "主卧",
        id: 2,
        sort: 2,
      }, {
        n: "次卧",
        id: 3,
        sort: 3,
      }, {
        n: "餐厅",
        id: 4,
        sort: 3,
      }, {
        n: "书房",
        id: 5,
        sort: 5,
      }/*, {
        n: "客房",
        id: 6,
        sort: 6,
      }, {
        n: "阳台",
        id: 7,
        sort: 7,
      }, {
        n: "厨房",
        id: 8,
        sort: 8,
      }, {
        n: "卫生间",
        id: 9,
        sort: 9,
      }, {
        n: "玄关",
        id: 10,
        sort: 10,
      }, {
        n: "庭院",
        id: 11,
        sort: 11,
      }*/
    ];
  },
  getOldOriginRooms () {
    return this.getOriginRooms().concat([{
      n: "客房",
      id: 6,
      sort: 6,
    }, {
      n: "阳台",
      id: 7,
      sort: 7,
    }, {
      n: "厨房",
      id: 8,
      sort: 8,
    }, {
      n: "卫生间",
      id: 9,
      sort: 9,
    }, {
      n: "玄关",
      id: 10,
      sort: 10,
    }, {
      n: "庭院",
      id: 11,
      sort: 11,
    }])
  },
  getFuncByPid (devStatus, pid, extendJson) {
    // console.log("getFuncByPid pid = ", pid)
    let _devStatus = devStatus || {};
    if (!pid) {
      return _devStatus;
    }
    // 根据pid去的设备的品类
    let _str = (typeof pid == "string") ? pid : this.getDevTypeStr(pid);
    // console.log("getFuncByPid _str = ", _str)
    if ('light' == _str) {
      console.log("light")
      // 亮度增加
      _devStatus.brightness = extendJson?.brightness || 127;
      // 默认的色温
      let _defaultC = extendJson?.c || 255;
      // 一次判断其他的品类
      let {kw, krgbcw, kcw, krgb, krgbw, kcwrgb, kwcodewarm} = DEVCONFIG[_str];
      if (kwcodewarm.indexOf(pid) >= 0) {
        delete _devStatus.r;
        delete _devStatus.g;
        delete _devStatus.b;
        delete _devStatus.c;
        delete _devStatus.w;
        // 增加调光调色
        _devStatus.supportCodewarm = true;
      } else if (kw.indexOf(pid) >= 0) {
        // w灯
        delete _devStatus.r;
        delete _devStatus.g;
        delete _devStatus.b;
        delete _devStatus.c;
        delete _devStatus.w;
      } else if (kcw.indexOf(pid) >= 0) {
        // cw
        _devStatus.c = _defaultC;
        _devStatus.w = 255 - _defaultC;

        delete _devStatus.r;
        delete _devStatus.g;
        delete _devStatus.b;
      } else if (krgb.indexOf(pid) >= 0) {
        // rgb
        _devStatus.r = _devStatus.g = _devStatus.b = 0;
      } else if (krgbw.indexOf(pid) >= 0) {
        // rgbw
        _devStatus.r = _devStatus.g = _devStatus.b = _devStatus.w = 255;
        delete _devStatus.c;
        delete _devStatus.w;
      } else {
        // rgbcw
        _devStatus.r = _devStatus.g = _devStatus.b = 0;
        _devStatus.c = _defaultC;
        _devStatus.w = 255 - _defaultC;
      }
    } else if ('curtain' == _str) {
      // 窗帘默认100
      _devStatus.percent = 100;
      // 窗帘默认开
      // _devStatus.mode = 1;
    } else if ('fan' == _str) {
      // 风扇默认100
      _devStatus.speed = 127;
    } else if ('coathanger' == _str) {
      _devStatus.updown = 3; // 0升1降2停止3无动作
      _devStatus.lamp = 0; // 0关1开2无动作
      _devStatus.fan = 0; // 0关1开2无动作
      _devStatus.dry = 0; // 0关1开2无动作
      _devStatus.disinfect = 0; // 0关1开2无动作
    }
    return _devStatus;
  },
  getRoomsByDevs (devJson) {
    if (!devJson) {
      return {
        rooms: [],
        roomJson: {}
      };
    }
    let _rooms = [];
    let _roomJson = {};
    for (let roomId in devJson) {

      let _item = devJson[roomId];
      if (!_item) {
        continue;
      }
      let _obj = {
        id: parseInt(roomId),
        n: _item.n || '',
        sort: _item.sort || 0
      };
      _rooms.push(_obj);
      _roomJson[roomId] = _obj;
    }
    // 排序
    _rooms = _rooms.sort(function (a, b) {
      return (((a.id || 0) % 32) - ((b.id || 0) % 32));
    });
    return {
      rooms: _rooms,
      roomJson: _roomJson
    };
  },
  showConnectedError () {
    wx.showModal({
      content:'蓝牙已连接其他设备，请重新打开蓝牙或者断开蓝牙设备',
      showCancel: false,
      // cancelColor: 'cancelColor',
    })
  },
  getSceneIcon (index) {
    let _index = 'icon_scene_custom';
    // let _numIndex = Number(index);
    if (index > 0 && index < 100) {
      _index = (index || 1) % 16;
    } else if (index > 100) {
      _index = index;
    }
    return `/static/img/common/scene/${_index}.png`;
  },
  getSceneBg (index) {
    const bgConfig = {
      1: "90deg, #86ECF0 5%, #65CFE8 95%",
      2: "269deg, #40A9FF 0%, #8DCCFF 100%",
      3: "270deg, #FF872F 4%, #FFA868 100%",
      4: "90deg, #FEB9B9 0%, #F27B7E 100%",
      5: "90deg, #FDC785 4%, #F1B22A 100%",
      6: "270deg, #3BDA8D 3%, #44F2BF 97%",
      7: "90deg, #FEB9B9 0%, #F27B7E 100%",
      8: "90deg, #ABE3FF 5%, #5DCDFF 97%",
      9: "270deg, #B579EC 0%, #D9B6F9 100%"
    };
    return bgConfig[(index || 1) % 10];
  },
  getPanelKeysByPid (pid, deviceKey) {
    let _deviceKey = deviceKey || "panel";
    let {eight, six, three, four, two, one, five} = DEVCONFIG[_deviceKey] || {};
    if (six.indexOf(pid) >= 0) {
      return 6;
    } else if (three.indexOf(pid) >= 0) {
      return 3;
    } else if (four.indexOf(pid) >= 0) {
      return 4;
    } else if (two.indexOf(pid) >= 0) {
      return 2;
    } else if (one.indexOf(pid) >= 0) {
      return 1;
    } else if (!!eight && eight.indexOf(pid) >= 0) {
      return 8;
    } else if (!!five && five.indexOf(pid) >= 0) {
      return 5;
    }
    return 0;
  },
  getDoorVGroups () {
    return [
      {"name": "开⻔"},
      {"name": "关⻔"}
    ]
  },
  getBodyVGroups () {
    return [
      {"name": "有⼈"},
      {"name": "⽆⼈"}
    ]
  },
  getWaterVGroups () {
    return [
      {"name": "水浸报警"},
      {"name": "水浸正常"}
    ]
  },
  getCurtainVGroups (defaultVal) {
    return {
      curtainLimit: {
        up: defaultVal || false,
        down: defaultVal || false
      }
    }
  },
  getCurtainLimitConfig (devInfo) {
    let _curtainLimit = devInfo?.e?.curtainLimit || {};
    return {
      up: _curtainLimit.up,
      down: _curtainLimit.down
    };
  },
  setCurtainLimit (devInfo, config) {
    if (!config || Object.keys(config).length == 0) {
      return;
    }
    let _curtainLimit = devInfo?.e?.curtainLimit || {};
    if (Object.keys(_curtainLimit).length == 0) {
      return;
    }
    return Object.assign(_curtainLimit, config || {});
  },
  // 取得继电器面板的各路名称
  getPanelVGroups (num, configJson) {
    if (!num) {
      return [];
    }
    let _result = [];

    let {extendId, isGroup, defaultNames, defaultFormat} = configJson || {};

    // 暂时只有一个id 留着后续扩展
    let _extendId = extendId != undefined ? extendId : "";
    
    for (let i = 0; i < num; i++) {
      let _obj = {};
      if (defaultFormat) {
        _obj.name = defaultFormat + (i + 1);
      } else {
        _obj.name = isGroup ? ((i + 1) + "路") : ("按键" + (i + 1));
      }
      // 是否有默认名称
      if (defaultNames) {
        _obj.name = defaultNames[i] || "";
      }
      !!_extendId && (_obj.id = i);
      _result.push(_obj)
    }
    return _result;
  },
  getIdByArray (array, key, minVal, maxVal) {
    let _array = array || [];
    let _tmps = [];
    for (let i = 0, _len = _array.length; i < _len; i++) {
      let _item = _array[i] || {};
      _tmps.push(Number(_item[key]));
    }
    let _minVal = minVal || 1;
    let _maxVal = maxVal || 256;
    for (let i = _minVal; i < _maxVal; i++) {
      if (_tmps.indexOf(i) >= 0) {
        continue;
      }
      return i;
    }
    // 如果定义了最大值 则返回-1失败
    if (maxVal) {
      return -1;
    }
    return 1;
  },
  getPanelStr (cmdArray, panelCount, keyName) {
    if (!cmdArray) {
      return '';
    }

    let _keyName = keyName || "按键"; 

    let _cmdArray = [];
    if (typeof cmdArray == "string") {
      _cmdArray = Utils.hexStrToArray(cmdArray);
    } else {
      _cmdArray = cmdArray;
    }
    let _byte0 = _cmdArray[0];
    let _byte1 = _cmdArray[1];

    // 打开和关闭的按键
    let _opens = [];
    let _closes = [];

    let _valStr = (_byte1 & 0x01) ? '打开' : '关闭';
    console.log("getPanelStr _valStr, panelCount = ", _valStr, panelCount)
    for (let i = 0; i < (panelCount || 0); i++) {
      let _tp = (_byte0 >> i) || 0;
      let _t = _tp & 0x01;
      if (!_t) {
        continue;
      }
      let _vp = (_byte1 >> i) || 0;
      let _vt = _vp & 0x01;
      _vt ? (_opens.push(i + 1)) : (_closes.push(i + 1));
    }
    // 如果按键全部选中
    if (_opens.length == panelCount) {
      return "所有" + _keyName + "打开";
    } else if (_closes.length == panelCount) {
      return "所有" + _keyName + "关闭";
    } else {
      // return _cmdTxtArray.join(',') + ' ' + _valStr;
      let _result = "";
      if (_opens.length) {
        _result += _keyName + _opens.join(" ") + "打开 ";
      }
      if (_closes.length) {
        _result += _keyName + _closes.join(" ") + "关闭";
      }
      return _result
    }
  },
  // 取得风扇得执行提示
  getFanTipByCmd (cmdStr) {
    let _resultStr = "";
    if (!cmdStr) {
      return _resultStr;
    }
    let _cmdArray = [];
    if (typeof cmdStr == "string") {
      console.log("getAirconditionTipByCmd string");
      _cmdArray = Utils.hexStrToArray(_cmdArray);
    } else if (cmdStr.pwr != undefined || cmdStr.speed != undefined) {
      _cmdArray = [cmdStr.speed || cmdStr.pwr || 0];
    }
    // 开关或者风速
    if (!_cmdArray[0]) {
      _resultStr = "风扇关闭"
    } else {
      _resultStr = "风扇开";
      if (cmdStr.speed <= 127) {
        _resultStr += ", 风速" + this.getPercentByVal(cmdStr.speed, {
          minVal: 1, 
          maxVal: 127,
          minPercent: 1, 
          maxPercent: 100
        }) + "%";
      }
    }
    return _resultStr;
  },
  // 取得晾衣架的执行提示
  getCoathangerTipByCmd (cmdStr) {
    console.log("getCoathangerTipByCmd cmdStr = ", cmdStr);
    let _cmdArray = cmdStr;
    if (typeof cmdStr == "string") {
      console.log("getCoathangerTipByCmd string");
      _cmdArray = Utils.hexStrToArray(cmdStr);
    } else if (cmdStr instanceof Object && cmdStr.pwr != undefined) {
      console.error("data error");
      return "";
    }
    console.log("getCoathangerTipByCmd _cmdArray = ", _cmdArray);
    let _byte0 = _cmdArray[0]; // 升降 0 – 上升 1 – 下降 2—停止 3—无动作
    let _byte1 = _cmdArray[1]; // 照明 0 – 关 1 – 开 2—无动作
    let _byte2 = _cmdArray[2]; // 风干 0 – 关 1 – 开 2—无动作
    let _byte3 = _cmdArray[3]; // 烘干 0 – 关 1 – 开 2—无动作
    let _byte4 = _cmdArray[4]; // 消毒 0 – 关 1 – 开 2—无动作
    var _getTip = (value, isUpDown) => {
      if (isUpDown) {
        switch (value) {
          case 0: 
            return '上升';
          case 1: 
            return '下降';
          case 2: 
            return '停止';
          // case 3: 
          //   return '无动作';
          default: 
            return '';
        };
      } else {
        switch (value) {
          case 0: 
            return '关';
          case 1: 
            return '开';
          // case 2: 
          //   return '';
          default: 
            return '';
        }
      }
    };
    let _resultArray = [];
    // 升降
    let _upDown = _getTip(_byte0 || 0, true);
    !!_upDown && (_resultArray.push(_upDown));
    // let _lamp = _getTip(_byte1 || 0);
    // !!_lamp && (_resultArray.push("照明" + _lamp));
    // let _fan = _getTip(_byte2 || 0);
    // !!_fan && (_resultArray.push("风干" + _fan));
    // let _dry = _getTip(_byte3 || 0);
    // !!_dry && (_resultArray.push("烘干" + _dry));
    // let _disinfect = _getTip(_byte4 || 0);
    // !!_disinfect && (_resultArray.push("消毒" + _disinfect));

    !!_byte1 && (_resultArray.push("照明"));
    !!_byte2 && (_resultArray.push("风干"));
    !!_byte3 && (_resultArray.push("烘干"));
    !!_byte4 && (_resultArray.push("消毒"));

    return _resultArray.join(",");
  },
  // 取得温控器的命令提示
  getThermostatTipByCmd (cmdStr) {
    return this.getAirconditionTipByCmd(cmdStr, "温控器");
  },
  // 取得空调的执行提示
  getAirconditionTipByCmd (cmdStr, title) {
    let _title = title || "空调";
    console.log("getAirconditionTipByCmd cmdStr, title = ", cmdStr, _title);
    let _cmdArray = cmdStr;
    if (typeof cmdStr == "string") {
      console.log("getAirconditionTipByCmd string");
      _cmdArray = Utils.hexStrToArray(cmdStr);
    } else if (cmdStr instanceof Object && cmdStr.pwr != undefined) {
      console.log("getAirconditionTipByCmd Object");
      // json转数组
      _cmdArray = [cmdStr.pwr || 0];
      !!cmdStr.temp && (_cmdArray.push(cmdStr.temp));
    }
    console.log("getAirconditionTipByCmd _cmdArray = ", _cmdArray);
    let _byte0 = _cmdArray[0];
    if (_byte0 == 0) {
      return _title + "关闭"
    } else {
      let _byte1 = _cmdArray[1]; // 模式 auto = 0x00 hot = 0x01 cold = 0x02 wind = 0x04 dewet = 0x08
      let _byte2 = _cmdArray[2]; // 温度 0x10 16摄氏度
      let _byte3 = _cmdArray[3]; // 风速 auto = 0x00 low = 0x01 medium = 0x02 high = 0x03
      // 模式名称
      let _modes = AirConfig.MODE.filter(item => item.value == _byte1);
      let _temp = _byte2 + "℃"
      let _speeds = AirConfig.SPEED.filter(item => item.value == _byte3);

      return _title + "打开 " + _temp + " " + (_modes[0]?.text || "") + " " + (_speeds[0]?.text || "");
    }
  },
  getTipByStatus (status, pid, devTypeStr, percentVal) {
    if (status === undefined || status === "" || status === "exception") {
      return "不执行"
    } else if ((status && status.length == 0) || status === "current") {
      return '当前状态';
    }

    // 默认以json解析
    let _json = status;
    // 取得pidStr 为了兼容
    if (pid != undefined || devTypeStr != undefined) {
      let _devTypeStr = devTypeStr || this.getDevTypeStr(pid);
      console.log("util getTipByStatus _devTypeStr = ", _devTypeStr, _json);
      if (_devTypeStr == "panel") {
        // 几键面板
        let _panelCount = this.getPanelKeysByPid(pid, _devTypeStr);
        let _tip = this.getPanelStr(_json, _panelCount);
        console.log("getTipByStatus _tip = ", _tip, _json, _panelCount);
        return _tip;
      } else if (_devTypeStr == "light") {
        // 照明设备
        if (_json instanceof(Array)) {
          // 取得json的配置
          _json = this.getJsonByCmdAndType(_json, _devTypeStr);
          console.log("util getTipByStatus _json = ", _json);
        } else if (typeof _json == "string" && _json.length > 0) {
          let _array = Utils.hexStrToArray(_json);
          console.log("getTipByStatus _array = ", _array);
          _json = this.getJsonByCmdAndType(_array, _devTypeStr);
          console.log("getTipByStatus _json = ", _json);
        }
      } else if (_devTypeStr == "curtain") {
        // 窗帘 1开2关
        let _typeStatus = typeof _json;
        if (_typeStatus == "string") {
          let _array = Utils.hexStrToArray(_json) || [];
          console.log("getTipByStatus _array, _json = ", _array, _json);
          let _percent = _array[1];
          if (_percent == 0) {
            return "全关"
          } else if (_percent == 100) {
            return "全开"
          } else {
            return "开合 " + _percent + "%";
          }
          // return (_array[0] == 1) ? '开' : (array[0] == 2 ? "关" : '暂停');
        } else if (_json instanceof Array) {
          return _json.length == 0 ? "当前状态" : ("开合 " + Math.round(_json[1] || 0) + "%");
        } else if (_json instanceof Object) {
          if (Object.keys(_json).length == 0) {
            return "当前状态";
          }
          console.log("getTipByStatus _json = ", _json);
          // 百分比
          return ("开合 " + Math.round(_json.percent || 0) + "%");
          // } else if (_typeStatus.mode == 2) {
          //   return "关";
          // } else if (_typeStatus.mode == 3) {
          //   return "暂停"
          // }
        } 
      } else if (_devTypeStr.indexOf("sensor") >= 0 || _devTypeStr == "powerPlug") {
        let _val = _json;
        if (_json instanceof Array) {
          _val = _json[0];
        }
        switch (_devTypeStr) {
          case "bodysensor":
          case "radarsensor":
            // 人体传感器
            return Number(_val) ? '无人' : '有人';
          case "doorsensor":
            // 门磁传感器
            return Number(_val) ? '门磁开' : '门磁关';
          case "watersensor":
            // 水浸传感器
            return Number(_val) ? '水浸正常' : '水浸报警';
          case "powerPlug":
            // 插卡取电
            return Number(_val) ? '有卡' : '无卡';
          case "sossensor":
            // sos传感器
            return Number(_val) ? '取消报警' : '报警';
          case "panelsensor":
            return Number(_val)
            // if (extension) {
            //   let {subs, vGroups} = extension;
            //   let _parentStr = subs[parseInt(index / 2)]?.name || "";
            //   let _childStr = vGroups[index]?.name || "";
            //   return _parentStr + " " + _childStr;
            // } else {
            //   return "" + index;
            // }
        }
      } else if (_devTypeStr == "aircondition") {
        return this.getAirconditionTipByCmd(_json);
      } else if (_devTypeStr == "fan") {
        return this.getFanTipByCmd(_json);
      } else if (_devTypeStr == "coathanger") {
        return this.getCoathangerTipByCmd(_json);
      } else if (_devTypeStr == "thermostat") {
        // 温控器
        return this.getThermostatTipByCmd(_json);
      } else if (_devTypeStr == "powerstrip") {
        // 智能排插
        let _panelCount = this.getPanelKeysByPid(pid, _devTypeStr);
        let _tip = this.getPanelStr(_json, _panelCount, "插孔");
        console.log("getTipByStatus _tip = ", _tip, _json, _panelCount);
        return _tip;
        // return this.getPowerstripTipByCmd(_json);
      }
    }

    let {pwr, brightness, r, g, b, c, w, brightnessP} = _json;
    // 如果pwr == undefined
    if (pwr == undefined) {
      return '当前状态';
    }
    if (!pwr) {
      return '关';
    } else {
      let _str = "开";
      if (percentVal != undefined) {
        _str += ` ${Number(percentVal)}%`;
      } else if (brightnessP != undefined) {
        _str += ` ${parseInt(brightnessP)}%`;
      } else if (brightness != undefined) {
        // 取得默认的百分比
        let _percent = this.getPercentByVal(brightness, {
          minVal: 2, 
          maxVal: 127,
          minPercent: 1, 
          maxPercent: 100
        })
        // 取得百分比
        _str += ` ${_percent}%`;
      }
      // 判断是否编辑过rgb和cw
      if (r + g + b + c + w == 0) {
        // 没有编辑过rgbcw
        return _str;
      } else if ((r == g && g == b && b == 255) || (c == w && w == 127)) {
        // 白光
        _str += ', 白光';
      } else {
        if (r >= 0 && g >= 0 && b >= 0 && (r + g + b) != 0) {
          // _str += (`, rgb(${r}, ${g}, ${b})`);
        } else if (c >= 0 && w >= 0 && (c + w) != 0) {
          let _percent = parseInt(w / 2.55);
          if (_percent < 0) {
            _percent = 0;
          } else if (_percent > 100) {
            _percent = 100;
          }
          let _value = Math.round(_percent / 100 * (6500 - 2700) + 2700);
          _str += (`, ${_value}K`);
          // _str += (`, 色温 ${_percent}%`);
        }
      }
      return _str;
    }
  },
  getCWValueByPercent (percent) {
    let _percent = Number(percent) || 0;
    return Math.round(_percent / 100 * (6500 - 2700) + 2700) + "K";
  },
  getPercentByRgb (rVal, gVal, bVal) {
    if (!rVal && !gVal && !bVal) {
      return 0;
    }
    let r = rVal || 0, g = gVal || 0, b = bVal || 0;
    let _val = 0;
    console.log("getPercentByRgb r, g, b = ", r, g, b);
    if (r == 0) {
      _val = (240 - parseInt((255 - b) / 255 * 120)) / 3.6;
    } else if (g == 0) {
      _val = (360 - parseInt((255 - r) / 255 * 120)) / 3.6;
    } else if (b == 0) {
      _val = (120 - parseInt((255 - g) / 255 * 120)) / 3.6;
      console.log("getPercentByRgb _val, g = ", _val, g);
    }
    return parseInt(_val);
  },
  getRgbByAngle (angle) {
    if (angle == undefined) {
      return false;
    }
    var _r = 0, _g = 0, _b = 0;
    if (angle >= 0 && angle < 120) {
      _r = parseInt((120 - angle) / 120 * 255);
      _g = 255 - _r;
      _b = 0;
    } else if (angle >= 120 && angle < 240) {
      _g = parseInt((240 - angle) / 120 * 255);
      _b = 255 - _g;
      _r = 0;
    } else if (angle >= 240 && angle <= 360) {
      _b = parseInt((360 - angle) / 120 * 255);
      _r = 255 - _b;
      _g = 0;
    }
    return [_r, _g, _b];
  },
  getPercentByVal (value, config) {
    let {minVal, maxVal, minPercent, maxPercent} = config || {};
    let _minVal = minVal || 0;
    let _maxVal = maxVal || 100;
    let _minPercent = minPercent || 0;
    if (value >= _maxVal) {
      return maxPercent || 100;
    } else if (value <= _minVal) {
      return minPercent || 0;
    }
    console.log("getPercentByVal percent = ", _minPercent, _maxVal, _minVal);
    return Math.round((value - _minVal) / (_maxVal - _minVal) * 100);
  },
  getRealValByPercent (percent, config) {
    let {minVal, maxVal, minPercent} = config || {};
    let _minVal = minVal || 0;
    let _maxVal = maxVal || 100;
    let _minPercent = minPercent || 0;
    if (_minVal >= _maxVal) {
      return _maxVal;
    } else if (percent <= _minPercent) {
      return _minVal;
    }
    console.log("getRealValByPercent percent = ", percent, _minPercent, _maxVal, _minVal);
    // return percent / 100 * (_maxVal - _minVal) + _minVal;
    return ((percent) / 100 * (_maxVal - _minVal) + _minVal);
  },
  // 取得所有数据中d对应的触发场景个数
  getAllSceneDevJson (json) {
    let _result = {};
    if (!json) {
      return _result;
    }
    for (let roomId in json) {
      let {scenes} = json[roomId] || {};
      if (!scenes || scenes.length == 0) {
        continue;
      }
      for (let sceneItem of scenes) {
        let {devs, sid, rid} = sceneItem || {};
        if (!devs || devs.length == 0) {
          continue;
        }
        // 真实的场景id
        let _sceneId = ((rid || 0) << 8) + (sid || 0);
        for (let dev of devs) {
          let {d} = dev || {};
          if (!d) {
            continue;
          }
          let _d = d; // .split("_")[0];
          _result[_d] = _result[_d] || [];
          _result[_d].push(_sceneId);
        }
      }
    }
    return _result;
  },
  getActionDAndGidMap (allJson, keyList) {
    let _result = {
      devs: {},
      groups: {},
    };
    if (!allJson) {
      return _result;
    }
    let _keyList = keyList || [];
    // 判断校验的数据
    let _checkDevs = _keyList.indexOf("devs") >= 0;
    let _checkGroups = _keyList.indexOf("groups") >= 0;
    for (let roomId in allJson) {
      let {devs, groups} = allJson[roomId];
      // 如果需要取
      if (_checkDevs) {
        let _devJson = this.arrayToJsonByKey(devs, true, "a");
        Object.assign(_result.devs, _devJson);
      }
      if (_checkGroups) {
        let _groupJson = this.arrayToJsonByKey(groups, true, "gid");
        Object.assign(_result.groups, _groupJson);
        // _result.groups = _groupJson;
      }
    }
    return _result;
  },
  // 取得所有d对应的设备信息
  getDidMapByConfig (allJson, keyList, config) {
    let _result = {
      devJson: {},
      sceneJson: {},
      // groupJson: {}
    };
    if (!allJson) {
      return _result;
    }
    let _keyList = keyList || [];
    // 判断校验的数据
    let _checkDevs = _keyList.indexOf("devs") >= 0;
    let _checkScenes = _keyList.indexOf("scenes") >= 0;
    for (let roomId in allJson) {
      let {devs, scenes, groups} = allJson[roomId];
      // 如果需要取
      if (_checkDevs) {
        let _devJson = this.arrayToJsonByKey(devs, true, "d");
        let _groupJson = this.arrayToJsonByKey(groups, true, "gid");
        Object.assign(_result.devJson, _devJson, _groupJson); // 数据赋值
      }
      if (_checkScenes) {
        let _sceneJson = this.arrayToJsonByKey(scenes, true, "sid+rid");
        Object.assign(_result.sceneJson, _sceneJson); // 数据赋值
      }
      // if (_keyList.indexOf("groups") >= 0) {
      //   let _groupJson = this.arrayToJsonByKey(groups, true, "gid");
      //   Object.assign(_result.groupJson, _devJson, _groupJson); 
      // }
    }
    return _result;
  },
  // , key
  getAllPanels (devsJson) {
    let _devScenes = {};
    let _panelScenes = {};
    let _sids = {};
    if (!devsJson) {
      return {};
    }
    // 面板的三个数据
    for (let roomId in devsJson) {
      let {scenes} = devsJson[roomId] || {};
      // 不存在场景
      if (!scenes || scenes.length == 0) {
        continue;
      }

      // 遍历场景
      for (let sceneInfo of scenes) {
        // panels 面板  devs 自定义设备
        let _sceneInfo = sceneInfo || {};
        let {sid, devs, rid, panels} = _sceneInfo;

        // 最终的场景id
        let _totalSid = this.getSceneId(sid, rid || 0);
        _sids[_totalSid] = sceneInfo;
        if (!!panels) {
          // 面板的场景信息
          for (let index in panels) {
            let _item = panels[index];
            if (!_item) {
              continue;
            }
            let _d = _item.d, _key = _item.key; 
            
            _panelScenes[_d] = _panelScenes[_d] || {};
            _panelScenes[_d][_key] = _totalSid;
          }
        }
        // 照明的场景信息
        if (!!devs) {
          for (let dev of devs) {
            let {d, cmd, tip} = dev;
            _devScenes[d] = _devScenes[d] || [];
            _devScenes[d].push({
              // rid: rid,
              sid: _totalSid,
              cmd: cmd,
              tip
            })
          }
        }
      }
    }
    return {
      panelScenes: _panelScenes,
      devScenes: _devScenes,
      sids: _sids
    };
  },
  getAllPidEmptyArray () {
    let _result = {};
    let _array = Object.keys(DEVCONFIG);
    for (let key in DEVCONFIG) {
      if (!DEVCONFIG[key].arrays) {
        continue;
      }
      _result[key] = [];
    };
    return _result;
  },
  // 根据设备类型取得不同品类的设备列表
  getAllPidByArray (array) {
    let _result = this.getAllPidEmptyArray();
    if (!array) {
      return _result;
    }
    for (let dev of array) {
      let {t} = dev || {};
      let _strType = this.getDevTypeStr(t);
      if (!_strType) {
        continue;
      }
      _result[_strType] = _result[_strType] || [];
      // 判断是照明设备还是窗帘还是面板
      _result[_strType].push(dev);
    }
    return _result;
  },
  // 根据房间id取得房间名称
  getRoomNameById (roomJson, id, seperator) {
    console.log("getRoomNameById roomJson, id, seperator = ", roomJson, id, seperator);
    let _name = "";
    if (!roomJson || id == undefined) {
      return _name;
    }
    let _key = "n";
    if (id % 32 == 0) {
      return this.getValueByKeyInJson(roomJson, [id, _key]);
    } else {
      // 楼层 + 房间
      let _floorId = this.getFloorByRoomId(id); // id + 32 - id % 32;
      let {n: floorName} = roomJson[this.getRoomIdByFloor(_floorId)] || {};
      console.log("getRoomNameById _floorName, _floorId = ", floorName, _floorId);
      if (floorName) {
        return floorName + (seperator || "") + this.getValueByKeyInJson(roomJson, [id, _key]);
      }
      return this.getValueByKeyInJson(roomJson, [id, _key]);
    }
  },
  getValueByKeyInJson (json, key) {
    let _type = typeof key;
    if (!json || key == undefined || key.length == 0) {
      return "";
    } else if (_type != "object") {
      return json[key] || "";
    } else {
      let _key = key[0];
      if (key.length == 1) {
        return json[_key] || "";
      }
      return this.getValueByKeyInJson(json[_key], key.slice(1))
    }
  },
  // 取得完整的场景ID
  getSceneId (sid, rid, type) {
    return ((rid || 0) << 16) + ((sid || 0) & 0xff) + ((type || 0) << 8);
  },
  // 解析sceneIdSDK
  getJsonBySidSDK (sidSDK) {
    let _result = {
      roomId: 0,
      type: 0,
      sid: 0
    };
    if (!!sidSDK) {
      _result.roomId = (sidSDK >> 16) & 0xff;
      _result.type = (sidSDK >> 8) & 0xff;
      _result.sid = (sidSDK >> 0) & 0xff;
    }
    return _result;
  },

  padStart (str, len, val) {
    let _str = str || '';
    let _type = typeof _str;
    if (_type == "number") {
      _str = _str.toString(16);
    } else if (_type != "string") {
      _str = _str.toString();
    }
    if (!len) {
      return _str;
    }
    return _str.padStart(Number(len) || 0, Number(val) || 0);
  },
  // 和之前不一样，这里必须要传type
  getJsonByCmdAndType (cmdArray, type) {
    console.log("getJsonByCmdAndType cmdArray, type = ", cmdArray, type);
    let _cmdArray = cmdArray || [];
    // 如果cmd空或者非数组，则返回
    if (_cmdArray.length == 0 || !(cmdArray instanceof(Array)) || !type) {
      return {};
    }
    // 如果type存在，则
    let _typeStr = typeof type;
    let _devTypeStr = ""; // 设备大品类
    // let _isW = // 是否w灯，如果是的话，循环场景中需要取电
    if (_typeStr == "string") {
      // 如果是string，则认为是light等区分出的
      _devTypeStr = type;
    } else if (_typeStr == "number") {
      _devTypeStr = this.getDevTypeStr(type);
    } else {
      // undefined
      return {};
    }
    console.log("getJsonByCmdAndType _devTypeStr = ", _devTypeStr);
    // 返回值
    let _result = {};
    // 判断设备品类
    if (_devTypeStr == 'light') {
      let _byte0 = cmdArray[0];
      _result.pwr = (_byte0 >> 7); // 开关
      let _brightness = (_byte0 & 0x7f); // 亮度
      if (cmdArray.length == 1) {
        if (_brightness > 0) {
          _result.pwr = 1;
          _result.brightness = _brightness; // 亮度
        } else {
          return _result;
        }
      } else {
        _result.brightness = _brightness; // 亮度
        // 这里有个问题，根据协议是brg格式，不是rgb
        let _byte1 = cmdArray[1];
        (_byte1 != undefined) && (_result.b = _byte1);
        let _byte2 = cmdArray[2];
        (_byte2 != undefined) && (_result.r = _byte2);
        let _byte3 = cmdArray[3];
        (_byte3 != undefined) && (_result.g = _byte3);
        let _byte4 = cmdArray[4];
        (_byte4 != undefined) && (_result.c = _byte4);
        let _byte5 = cmdArray[5];
        (_byte5 != undefined) && (_result.w = _byte5);
      }
    } else if (_devTypeStr == 'curtain') {
      // let _byte0 = cmdArray[0]; // 1 上 / 2 下 / 3 停
      // _result.mode = _byte0 & 0x03; // 开关
      // _result.pwr = _result.mode; // 开关
      // if (cmdArray.length == 1) {
      //   return _result;
      // }
      _result.percent = cmdArray[1] & 0xff;
    } else if (_devTypeStr == 'panel') {
      // 是否开关面板数据
      if (_typeStr == "number" && this.isPwrAvalibleDev(_typeStr)) {
        _result = this.pwrPanelCmdToJson(cmdArray);
      } else {
        _result = cmdArray;
      }
    } else if (_devTypeStr == 'powerstrip') {
      _result = cmdArray;
    } else if (_devTypeStr == "aircondition") {
      _result = this.airconditionCmdToJson(cmdArray);
      // _result = this.getAirconditionTipByCmd(_cmdArray);
    } else if (_devTypeStr == "coathanger") {
      // 晾衣架
      _result = this.coathangerCmdToJson(cmdArray);
      // _result = this.getAirconditionTipByCmd(_cmdArray);
    }
    return _result;
  },
  wxToAppSceneI (imgIndex) {
    let {sceneI} = ADAPTERAPP;
    return sceneI[imgIndex] || imgIndex || 'icon_scene_custom';
  },
  appToWXSceneI (imgStr) {
    let {sceneI} = ADAPTERAPP;
    let keys = Object.keys(sceneI) || [];
    let vals = Object.values(sceneI) || [];
    let _index = vals.indexOf(imgStr);
    if (_index >= 0) {
      return keys[_index];
    }
    return imgStr || '15';
  },
  getPrivateData (allRooms) {
    let devs = [];
    let scenes = [];
    let gateways = [];
    let groups = [];
    let roomList = [];
    if (!allRooms) {
      return {
        devs,
        scenes,
        gateways,
        groups,
        roomList
      };
    }
    // 新增需求: 照明群组需要把群组下所有设备都按照功能交集取出后设置blecatetory 所以这里取得d对应的设备信息
    let _dMaps = this.getDidMapInJson(allRooms);
    // 遍历所有数据
    for (let roomId in allRooms) {
      // 判断设备信息
      let _roomDetail = allRooms[roomId];
      if (!_roomDetail) {
        continue;
      }
      let _roomId = _roomDetail.id || 0;
      let _roomName = _roomDetail.n || '';
      let _devs = _roomDetail.devs || [];
      let _scenes = _roomDetail.scenes || [];
      let _groups = _roomDetail.groups || [];
      // 房间信息
      roomList.push({
        roomId: Number(_roomId) || 0,
        roomName: _roomName
      });
      for (let item of _devs) {
        let _devTypeStr = this.getDevTypeStr(item.t);
        // 这里判断是否空调主机
        let _blecatetory = this.getBleCatetoryByPid(_devTypeStr, item.t, {
          sub: item.sub
        });
        
        if (_devTypeStr == 'gateway') {
          // 网关设备则不加入
          gateways.push({
            // wd: item.wd, // ihg用，优先级wd > d
            t: item.t,
            d: item.wd || item.d,
            k: item.k,
            n: item.n || '',
            rn: _roomName,
            a: item.a,
            ig: Boolean(item.ig) || false,
            blecatetory: _blecatetory,
            id: (item.id) || _roomId || 0,
            online: item?.online || false,
            v: item.v
          })
          continue;
        }
        devs.push({
          id: (item.id) || _roomId || 0,
          rn: _roomName,
          blecatetory: _blecatetory,
          n: item.n || '',
          a: item.a,
          t: item.t,
          d: item.d,
          e: item.e, // 增加数据上报vGroups字段
          ig: Boolean(item.ig) || false,
          // 增加版本号判断
          v: item.v
        })
      }
      for (let scene of _scenes) {
        // 如果是默认场景，且没有改变过则不设置
        let {t, panels, devs, b, sid, n, ig} = scene || {};
        // // 默认场景如果没有设备则不传 注释没有设备的场景不传
        // if (t >= 3 && _devs.length == 0) {
        //   continue
        // }
        // if (t >= 3 && roomId != 0 && (devs || []).length == 0 && (panels || []).length == 0) {
        //   continue;
        // }
        let _rid = scene.id || _roomId || 0;
        scenes.push({
          rn: _roomName,
          rid: _rid,
          sid: sid,
          n: n,
          b: b || 0,
          ig: Boolean(ig) || false
        })
      }
      // 群组这里增加特殊处理 根据群组中照明设备的能力并集设置blecatetory
      for (let group of _groups) {
        let _devTypeStr = this.getDevTypeStr(group?.t);

        // 特殊处理照明设备
        let _blecatetory = "";
        if (_devTypeStr == "light") {
          // 群组中的照明设备交集
          _blecatetory = this.getLightBleCategoryByMapAndDs(_dMaps, group, allRooms);
        } else {
          _blecatetory = this.getBleCatetoryByPid(_devTypeStr);
        }
        let _item = {
          id: group?.rid || group?.id || _roomId || 0,
          rn: _roomName,
          blecatetory: _blecatetory,
          bledevtype: "GROUP",
          n: group.n || '',
          gid: group.gid,
          t: this.getSendPidByType(group.t),
          ig: Boolean(group.ig) || false
        };
        devs.push(_item); // 其他用
        groups.push(_item); // 86屏用
      }
    }
    // 这里必须把gateways按照设备在线状态排序，必须把在线的排在第一位
    gateways = gateways.sort((a, b) => (Number(b?.online) || 0) - (Number(a?.online) || 0));
    return {
      devs,
      scenes,
      gateways,
      groups,
      roomList
    }
  },
  // 
  // 取得照明设备音响中的交集品类
  getLightBleCategoryByMapAndDs (dMap, groupInfo, allDevs) {

    let _lightTypeArray = [];
    // 这里需要判断是否默认群组 如果默认群组，则使用全屋或者楼层或者房间下的所有数据
    if (this.isAllRoomCtrl(groupInfo)) {
      // 根据房间id取得全屋或者楼层或者全屋下的数据
      let {devs} = this.getDevsAndGroupsByRoomId(groupInfo.rid || groupInfo.id || 0, allDevs);
      // 取得照明数据
      let {light} = this.getAllPidByArray(devs);
      _lightTypeArray = light.map(item => this.getBleCatetoryByPid('light', item.t));
    } else {
      // 最大能力
      _lightTypeArray = this.getLightFuncConfigByMapAndDs(dMap, groupInfo);
    }
    // 根据最大能力取得映射值
    let _result = this.getLightBleCatetoryByType(_lightTypeArray);
    console.log("getLightBleCategoryByMapAndDs _result = ", _result)
    return _result;
  },
  // 取得照明设备音响中的交集能力
  getLightFuncConfigByMapAndDs (dMap, groupInfo) {
    let _pidArray = []; // 能力交集
    let dArray = groupInfo?.dids || [];
    if (!dMap || !dArray || dArray.length == 0) {
      return _pidArray;
    }
    for (let did of dArray) {
      let _devInfo = dMap[did];
      if (!_devInfo) {
        continue;
      }
      let _devTypeStr = this.getDevTypeStr(groupInfo?.t);
      if (_devTypeStr != "light") {
        continue;
      }
      // 取得设备的属性集合
      _pidArray.push(this.getBleCatetoryByPid(_devTypeStr, _devInfo?.t));
    }
    return _pidArray;
  },
  // 根据能力集合取得音响映射
  getLightBleCatetoryByType (lightTypeArray) {
    // 结果
    let _result = "LIGHT";
    if (!lightTypeArray || lightTypeArray.length == 0) {
      return _result;
    }
    // 去重
    let _lightTypeArray = [...new Set(lightTypeArray)];
    let _hasRGBCW = _lightTypeArray.indexOf("RGBCWLIGHT") >= 0;
    let _hasRGBW = _lightTypeArray.indexOf("RGBWLIGHT") >= 0;
    let _hasRGB = _lightTypeArray.indexOf("RGBLIGHT") >= 0;
    let _hasCW = _lightTypeArray.indexOf("CWLIGHT") >= 0;
    // 遍历数据取出最大能力集合
    if (_hasRGBCW) {
      _result = "RGBCWLIGHT";
    } else if (_hasRGBW) {
      if (_hasCW) {
        _result = "RGBCWLIGHT";
      } else {
        _result = "RGBWLIGHT";
      }
    } else if (_hasRGB) {
      if (_hasCW) {
        _result = "RGBCWLIGHT";
      } else {
        _result = "RGBLIGHT";
      }
    } else if (_hasCW) {
      _result = "CWLIGHT";
    }
    return _result;
  },
  // 音响中做映射
  getBleCatetoryByPid (devTypeStr, pid, config) {
    // console.log("getBleCatetoryByPid devTypeStr, pid, config = ", devTypeStr, pid, config);
    if (devTypeStr == 'curtain') {
      return 'CURTAIN'
    } else if (devTypeStr == 'panel') {
      if (this.isPwrAvalibleDev(pid)) {
        return "SWITCH";
      }
      return "PANNEL";
    } else if (devTypeStr == 'light') {
      // 一次判断其他的品类
      let {kw, krgbcw, kcw, krgb, krgbw, kcwrgb} = DEVCONFIG[devTypeStr];
      if (krgbcw.indexOf(pid) >= 0) {
        return "RGBCWLIGHT";
      } else if (krgb.indexOf(pid) >= 0) {
        return "RGBLIGHT";
      } else if (kcw.indexOf(pid) >= 0 || kcwrgb.indexOf(pid) >= 0) {
        return "CWLIGHT";
      } else if (krgbw.indexOf(pid) >= 0) {
        return "RGBWLIGHT";
      } else {
        return "LIGHT";
      }
    } else if (devTypeStr == "bodysensor" || devTypeStr == "radarsensor") {
      return "PIR_SENSOR"
    } else if (devTypeStr == "doorsensor") {
      return "DOOR_SENSOR"
    } else if (devTypeStr == "watersensor") {
      return "FLOOD_SENSOR"
    } else if (devTypeStr == "temphumidity") {
      return "TEMPERATURE_HUMIDITY_SENSOR"
    } else if (devTypeStr == "aircondition") {
      if (config?.sub > 0) {
        return ""; // 空调主机没有BleCatetory
      }
      return "BLEAC"
    } else if (devTypeStr == "gateway") {
      return "GATEWAY"
    } else if (devTypeStr == "powerPlug") {
      return "ELECCARD"
    }
    /* else if (devTypeStr == "GROUP") {
      return devTypeStr
    } */
    return '';
  },

  // 取得网关设备信息
  getGatewayInfo (devs) {
    // let _gateway = undefined;
    if (!devs) {
      return undefined;
    }
    for (let roomId in devs) {
      let _devs = devs[roomId]?.devs || [];
      if (_devs.length == 0) {
        continue;
      }
      for (let devInfo of _devs) {
        let _devTypeStr = this.getDevTypeStr(devInfo.t);
        if (_devTypeStr == "gateway") {
          return devInfo;
        }
      }
    }
    return undefined;
  },
  // 取得数组中的最大值
  getMaxValByKey (array, key) {
    if (!array) {
      return 0;
    }
    let _max = 0;
    for (let index in array) {
      let _item = array[index];
      if (!!key) {
        _max = Math.max(_max, _item[key]) || _max;
      } else {
        _max = Math.max(_max, _item) || _max;
      }
    }
    return Number(_max) || 0;
  },
  getDevImgByT (devInfo) {
    let _t = devInfo || 0;
    if (devInfo instanceof Object) {
      _t = devInfo.t || 0;
    }

    // 如果网关数据，则不显示
    let _devTypeKey = this.getDevTypeStr(_t);
    // 是否群组设备
    if (devInfo.gid >= 0) {
      if (_devTypeKey == "curtain") {
        return "group_curtain"
      } else if (_devTypeKey == "light") {
        return "group_light"
      }
      return "group";
    }
    // 如果是面板，还需要判断几键面板
    if (_devTypeKey == 'panel') {
      let _count = this.getPanelKeysByPid(_t, _devTypeKey);
      _devTypeKey = _devTypeKey + _count;
    } else if (_devTypeKey == "aircondition") {
        // 兼容传设备型号或者设备信息
        if (typeof devInfo != "object") {
            let _sub = devInfo?.sub || 0;
            _devTypeKey = _sub > 0 ? "aircondition" : "sub";
        } else {
            let _isHost = this.isAirconditionHost(devInfo);
            _devTypeKey = _isHost ? "aircondition" : "sub";
        }
    } else if (_devTypeKey == "fan") {
      _devTypeKey = "fansLight" 
    } else if (_devTypeKey == "light") {
      // // 是否炫彩灯
      // if (this.isXuancai(_t)) {
      //   return "xuancai"
      // } else 
      if (this.isSixLight(_t)) {
        return "sixlight"
      } else if (this.isCwrgb(_t) || this.isCw(_t)) {
        // 是否色温灯
        return "light_cw"
      } else if (this.isRGB(_t) || this.isRGBW(_t) || this.isRGBCW(_t)) {
        // 是否rgb灯
        return "light_rgb"
      }
      // 默认白灯
      return "light"
    } else if (_devTypeKey == "curtain") {
      // if (this.isOpenCurtain(_t)) {
      //   return "curtain_open"
      // }
      if (this.isRollCurtain(_t)) {
        return "curtain_roll";
      } else if (this.isDreamCurtain(_t)) {
        return "curtain_dream";
      }
      // 默认开合帘
      return "curtain"
    } else if (_devTypeKey.indexOf('sensor') >= 0) {
      if (_devTypeKey == "sossensor") {
        return "sos";
      } else if (_devTypeKey == "panelsensor" || _devTypeKey == "radarsensor" || _devTypeKey == "watersensor") {
        // 雷达传感器图标或者干节点传感器
        return _devTypeKey;
      }
      return _devTypeKey == "doorsensor" ? 'doorsensor' : "bodysensor";
    } else if (_devTypeKey.indexOf('powerstrip') >= 0) {
      let _count = this.getPanelKeysByPid(_t, this.getDevTypeStr(_t));
      return 'powerstrip_' + _count; 
      // let _isOne = this.isOneSocket(_t);
      // return _isOne ? 'powerstrip_1' : "powerstrip_3";
    } else if (_devTypeKey == "uart") {
      return "panel6";
    } else if (_devTypeKey == "commonMotor") {
      return "commonMotor"
    }
    console.log("getDevImgByT _devTypeKey = ", _devTypeKey);
    // 设备类型
    return _devTypeKey;
  },
  getSumByHexStr (str) {
    return Utils.getSumByHexStr(str);
  },
  getScensConfig () {
    return {
      // 0: 全屋批量控制
      // 1: 自定义
      // 2: 当前设备状态
      // 3: 默认场景-未编辑
      // 4: 默认场景-自定义
      // 5: 默认场景-当前状态
      0: [{
        n: '全开',
        rid: 0,
        sid: 1,
        t: 3,
        b: 1,
        or: 1,
        i: "1"
      }, {
        n: '全关',
        rid: 0,
        sid: 2,
        t: 3,
        b: 2,
        or: 2,
        i: "2"
      }],
      // 客厅
      1: [{
        n: '回家',
        rid: 1,
        t: 3,
        sid: 1,
        b: 1,
        or: 1,
        i: "1"
      }, {
        n: '全关',
        t: 3,
        sid: 2,
        rid: 1,
        b: 2,
        or: 2,
        i: "2"
      }, {
        n: '明亮',
        t: 3,
        sid: 3,
        rid: 1,
        b: 3,
        or: 3,
        i: "3"
      }, {
        n: '观影',
        t: 3,
        sid: 4,
        rid: 1,
        b: 4,
        or: 4,
        i: "6"
      }, {
        n: '温馨',
        t: 3,
        sid: 5,
        rid: 1,
        b: 5,
        or: 5,
        i: "101"
      }, {
        n: '阅读',
        t: 3,
        sid: 6,
        rid: 1,
        b: 6,
        or: 6,
        i: "4"
      }],
      // 主卧
      2: [{
        n: '明亮',
        t: 3,
        sid: 3,
        rid: 2,
        b: 3,
        or: 1,
        i: "3"
      }, {
        n: '全关',
        t: 3,
        sid: 2,
        rid: 2,
        b: 2,
        or: 2,
        i: "2"
      }, {
        n: '柔和',
        t: 3,
        sid: 5,
        rid: 2,
        b: 5,
        or: 3,
        i: "101"
      }, {
        n: '阅读',
        t: 3,
        sid: 6,
        rid: 2,
        b: 6,
        or: 4,
        i: "4"
      }, {
        n: '睡眠',
        t: 3,
        sid: 7,
        rid: 2,
        b: 7,
        or: 5,
        i: "5"
      }, {
        n: '晨起',
        t: 3,
        sid: 8,
        rid: 2,
        b: 8,
        or: 6,
        i: "10"
      }],
      // 次卧
      3: [{
        n: '明亮',
        t: 3,
        sid: 3,
        rid: 3,
        b: 3,
        or: 1,
        i: "3"
      }, {
        n: '全关',
        t: 3,
        sid: 2,
        rid: 3,
        b: 2,
        or: 2,
        i: "2"
      }, {
        n: '温馨',
        t: 3,
        sid: 5,
        rid: 3,
        b: 5,
        or: 3,
        i: "101"
      }, {
        n: '阅读',
        t: 3,
        sid: 6,
        rid: 3,
        b: 6,
        or: 4,
        i: "4"
      }, {
        n: '睡眠',
        t: 3,
        sid: 7,
        rid: 3,
        b: 7,
        or: 5,
        i: "5"
      }, {
        n: '晨起',
        t: 3,
        sid: 8,
        rid: 3,
        b: 8,
        or: 6,
        i: "10"
      }],
      // 餐厅
      4: [{
        n: '明亮',
        t: 3,
        sid: 3,
        rid: 4,
        b: 3,
        or: 1,
        i: "3"
      }, {
        n: '全关',
        t: 3,
        sid: 2,
        rid: 4,
        b: 2,
        or: 2,
        i: "2"
      }, {
        n: '用餐',
        t: 3,
        sid: 9,
        rid: 4,
        b: 9,
        or: 3,
        i: "102"
      }, {
        n: '浪漫',
        t: 3,
        sid: 10,
        rid: 4,
        b: 10,
        or: 4,
        i: "103"
      }],
      // 书房
      5: [{
        n: '明亮',
        t: 3,
        sid: 3,
        rid: 5,
        b: 3,
        or: 1,
        i: "3"
      }, {
        n: '全关',
        t: 3,
        sid: 2,
        rid: 5,
        b: 2,
        or: 2,
        i: "2"
      }, {
        n: '阅读',
        t: 3,
        sid: 6,
        rid: 5,
        b: 6,
        or: 3,
        i: "4"
      }, {
        n: '温馨',
        t: 3,
        sid: 5,
        rid: 5,
        b: 5,
        or: 4,
        i: "101"
      }]
    };
  },
  sortArrayByChar (array, key) {
    if (!array || array.length == 0) {
      return [];
    }
    /* 获取数组元素内需要比较的值 */
    var getValue = (option) => {
      if (!key) return option
      var data = option
      key.split('.').filter(function (item) {
        data = data[item]
      })
      return data + ''
    }
    return array.sort((a, b) => {
      return getValue(a).localeCompare(getValue(b), "zh");
    })
    // return key ? (array.sort((a,b)=>(a || {})[key] > (b || "")[key])) : (array.sort((a,b)=>(a || "") > (b || "")));
  },


  // 取得当前最新的楼层
  getCurFloor (json) {
    var _roomIds = Object.keys(json);
    let _floor = 0;
    for (let v of _roomIds) {
      if (v == 0) {
        continue;
      } else if (v % 32 == 0) {
        return parseInt(v / 32 - 1);
      }
      return parseInt(v / 32);
    }
    return _floor;
  },
  // 根据房间列表id取得楼层数量
  getAllFloors (json) {
    if (!json) {
      return ["1楼"];
    }
    let _rooms = Object.keys(json);
    let arr = _rooms.map(v=>parseInt(v / 33));
    return Array.from(new Set(arr));
  },
  getFloorNames (json) {
    if (!json) {
      return [];
    }
    // 返回的数据
    let _result = [];
    // 遍历数据
    for (let roomId in json) {
      if (roomId != 0 && roomId % 32 == 0) {
        let _info = {
          n: json[roomId]?.n || "",
          id: roomId
        };
        _result.push(_info);
      }
    }
    return _result;
  },
  getDevsCountByFloor (json, floor) {
    if (!json || floor < 0) {
      return 0;
    }
    let _count = 0;
    let _floor = floor || 0;
    // 根据楼层取得楼层下的数据集合
    let _floorDevsJson = this.getAllDevsByFloor(json, _floor, false);
    // let _min = _floor * 32 + 1;
    // let _max = (_floor + 1) * 32;
    for (let roomId in _floorDevsJson) {
      _count += (_floorDevsJson[roomId]?.devs?.length || 0);
    }
    return _count;
  },
  getDevsArrayByFloor (json, floor) {
    return this.getAllDevsByFloor(json, floor, false);
  },
  // 取得包含全屋的楼层下设备集合
  getDevsByFloor (json, floor) {
    return this.getAllDevsByFloor(json, floor, true);
  },
  filterRoomByFloor (roomInfo, floor, supportZero) {
    let {id} = roomInfo || {};
    if (floor < 0) {
      return true;
    }
    if (supportZero && id == 0) {
      return true
    }
    let _min = floor * 32 + 1;
    let _max = (floor + 1) * 32;
    return (id >= _min) && (id <= _max);
  },
  // 取得不包含全屋的楼层下设备集合
  getAllDevsByFloor (json, floor, supportZero) {
    if (!json) {
      return {};
    } else if (floor == undefined || floor < 0) {
      return json;
    }
    let _result = {};
    let _floor = floor || 0;
    let _min = _floor * 32 + 1;
    let _max = (_floor + 1) * 32;
    for (let roomId in json) {
      if ((roomId >= _min && roomId <= _max)) {
        _result[roomId] = json[roomId];
      } else if (supportZero && (roomId == 0)) {
        _result[roomId] = json[roomId];
      } else {
        continue;
      }
    }
    return _result;
  },
  // 取得默认的名称
  getDefaultName (floor) {
    let _floor = floor || 0;
    return ((parseInt(_floor) || 0) + 1) + "楼";
  },
  getFloorByRoomId (roomId) {
    if (!roomId) {
      console.error("getFloorByRoomId roomId = ", roomId);
      return 0;
    }
    return parseInt(roomId / 32.1);
  },
  getRoomIdByFloor (floor) {
    let _floor = floor || 0;
    return parseInt(_floor + 1) * 32;
  } ,
  getFloorName (json, floor) {
    if (!json) {
      return this.getDefaultName(0);
    }
    let _roomId = this.getRoomIdByFloor(floor);
    // 优先取得楼层名称 如果没有楼层名称，则使用全屋第一个楼层数据，如果还没有，则是全屋
    let _defaultName = json[_roomId]?.n;
    if (!_defaultName) {
      let _floors = this.getFloorNames(json);
      if (_floors?.length > 0) {
        _defaultName = _floors[0]?.n;
      } else {
        _defaultName = "全屋";
      }
    }
    return _defaultName;
    // return json[(floor + 1) * 32]?.n || this.getDefaultName(floor);
  },
  addFloor (json, floor, n, newFloor) {
    if (!json) {
      return;
    }
    // 楼层起始
    let _startRoomId = (floor || 0) * 32;

    // 如果只是改变模式
    if (!newFloor) {
      // 取得原始的所有房间 这里有个bug 需要修改房间id对应的数据
      let _oriRooms = this.getOriginRooms();
      // 去除全屋的id
      let _rooms = _oriRooms.filter(item=>item.id>0);
      // 默认场景
      let _defaultScenes = this.getDefaultScenes();
      // 遍历赋值
      for (let item of _rooms) {
        let _id = item.id;
        // 每个场景有自己的场景
        item.scenes = _defaultScenes[_id] || [];
        // 更新所有场景id 加上基数
        for (let item of item.scenes) {
          item.rid = Number(item.rid + _startRoomId);
        }

        // 更新房间id
        let _realRoomId = _startRoomId + parseInt(_id);
        item.id = _realRoomId;

        json[_realRoomId] = item;
      }
    }
    
    // 楼层
    let _rid = _startRoomId + 32;
    json[_rid] = {
      n: n || this.getDefaultName(floor),
      id: _rid,
      sort: 0,
      devs: [],
      scenes: [{
        n: '全开',
        rid: _rid,
        sid: 1,
        t: 3,
        b: 1,
        or: 1,
        i: "1"
      }, {
        n: '全关',
        rid: _rid,
        sid: 2,
        t: 3,
        b: 2,
        or: 2,
        i: "2"
      }]
    };
    return json;
  },
  deleteFloor (json, roomId) {
    // let _floor = parseInt(roomId / 32) - 1;
    if (!json || roomId == undefined) {
      console.error("deleteFloor json none")
      return;
    }
    if (roomId < 32) {
      console.error("deleteFloor < 32")
      return;
    }
    // 要删除的所有设备
    let _devs = [];
    // 要删除的所有场景id
    let _scenes = [];
    // 取得要删除的房间id
    let _roomIds = [];
    for (let i = roomId - 31; i <= roomId; i++) {
      let _roomInfo = json[i];
      if (!_roomInfo) {
        continue;
      }
      // 房间列表
      _roomIds.push(Number(i));
      // 设备列表
      _devs = _devs.concat(_roomInfo?.devs || []);
      // 场景列表
      let _sceneArray = (_roomInfo?.scenes || []).map(item => this.getSceneId(item.sid, item.rid));
      _scenes = _scenes.concat(_sceneArray);
      // 删除房间id
      delete json[i];
    }
    let _dids = (_devs || []).map(item=>item.d);
    // 删除设备
    this.deleteDevInJsonByDids(json, _dids);
    // 删除超级面板中绑定的场景数据
    this.delMetapadInfoBySceneId(json, _scenes);
    console.log("deleteFloor _roomIds = ", _roomIds);
  },
  // 删除房间数据
  deleteRoom (rid, json) {
    // 房间id不存在，所有数据不存在 或者房间不存在
    if (rid == undefined || !json || !json[rid]) {
      return;
    }
    console.log("deleteRoom rid = ", rid);
    // 房间下的所有设备
    let {devs, scenes} = json[rid] || {};
    // 所有要删除设备did集合
    let _dids = (devs || []).map(item=>item.d);
    // 场景列表
    let _sceneArray = (scenes || []).map(item => this.getSceneId(item.sid, item.rid));
    // 删除本房间
    delete json[rid];
    // 删除所有设备数据
    this.deleteDevInJsonByDids(json, _dids);
    // 删除超级面板中绑定的场景数据
    this.delMetapadInfoBySceneId(json, _sceneArray);
  },
  updateDevInfoByDid (json, newDevInfo) {
    if (!json || !newDevInfo) {
      return;
    }
    for (let roomId in json) {
      let {devs} = json[roomId] || {};
      if (!devs || devs.length == 0) {
        continue;
      }
      for (let devInfo of devs) {
        if (devInfo?.d == newDevInfo?.d) {
          if (newDevInfo.n) {
            devInfo.n = newDevInfo.n;
          }
          if (newDevInfo.id != undefined) {
            devInfo.id = newDevInfo.id || 0;
          }
          return;
        }
      }
    }
  },
  // 删除json中的设备数据
  deleteDevInJsonByDids (json, _dids) {
    if (!json || !_dids || _dids.length == 0) {
      return;
    }
    // 删除其他房间的所有本房间设备关联的场景
    for (let roomId in json) {
      // 遍历场景数据
      let {devs, scenes, groups} = json[roomId] || {};
      console.error("deleteRoom roomId, _scenes = ", roomId, scenes);

      // 取得超级面板设备
      let {uart} = this.getAllPidByArray(devs);
      if (uart) {
        for (let metapad of uart) {
          let {ds} = metapad || {};
          if (!ds || ds.length == 0) {
            continue;
          }
          metapad.ds = ds.filter(item => _dids.indexOf(item?.cmd) < 0);
        }
      }

      // 删除群组中的设备
      if (groups) {
        for (let groupInfo of groups) {
          let {dids} = groupInfo || {};
          if (!dids || dids.length == 0) {
            continue;
          }
          groupInfo.dids = dids.filter(d => _dids.indexOf(d) < 0);
        }
      }

      // 判断其他设备的场景
      if (scenes) {
        for (let sceneInfo of scenes) {
          if (!sceneInfo) {
            continue;
          }
          let panels = sceneInfo.panels;
          let devs = sceneInfo.devs;
          // 删除执行设备
          if (!!devs && devs.length > 0) {
            sceneInfo.devs = devs.filter(item => _dids.indexOf(item.d) < 0);
          }
          // 删除面板
          if (!!panels && panels.length > 0) {
            sceneInfo.panels = this.delPanelInScene(panels, _dids);
          }
        }
      }
    }
  },
  moveDev (json, devInfo, newRoomIds) {
    if (!json || !devInfo) {
      return;
    }
    console.log("moveDev newRoomIds, devInfo, json = ", newRoomIds, devInfo, json);
    let _newRoomIds = newRoomIds || [];
    // 设备信息
    let {d, t} = devInfo;
    // 场景属性
    let _keys = [];
    if (t instanceof Array) {
      _keys = t;
    } else {
      // 如果是绑定设备
      if (this.isPanelDev(t)) {
        // 删除面板
        _keys.push("panels");
      }
      if (this.isActionDev(t)) {
        // 删除执行设备
        _keys.push("devs");
      }
    }

    // 如果不存在属性
    if (!_keys || _keys.length == 0) {
      return;
    }
    
    console.log("moveDev devInfo, _keys, _newRoomIds = ", devInfo, _keys, _newRoomIds);
    // 删除
    for (let roomId in json) {
      // 如果是全屋或者楼层，则忽略
      if (_newRoomIds.indexOf(Number(roomId)) >= 0) {
        continue;
      }
      let {scenes} = json[roomId] || {};
      if (!scenes || scenes.length == 0) {
        continue;
      }
      for (let item of scenes) {

        // 执行设备
        if (_keys.indexOf("devs") >= 0) {
          this.delSceneDetail(item?.devs, d);
        }
        // 触发设备
        if (_keys.indexOf("panels") >= 0) {
          this.delSceneDetail(item?.panels, d);
        }
      }
    }
  },
  // 这里目前写着没问题，但是不适用于array中存在重复的ds值数据
  delSceneDetail (array, ds) {
    if (!array || array.length == 0) {
      return;
    }

    for (let i = 0, len = array.length; i < len; i++) {
      if (!array[i]) {
        continue;
      }
      if ((ds || '').indexOf(array[i]?.d) >= 0) {
        // console.log("moveDev d = ", d);
        array.splice(i, 1);
      }
    }
  },

  // 是否触发设备
  isActionDev (t) {
    let _t = t;
    if (typeof t == "number") {
      _t = this.getDevTypeStr(t);
    }
    return this.isPwrAvalibleDev(t) || _t == "light" || _t == "curtain" || _t == "fanLight" || _t == "aircondition" || _t == "thermostat" || _t == "powerstrip"
  },
  isPanelDev (t) {
    let _t = t;
    if (typeof t == "number") {
      _t = this.getDevTypeStr(t);
    }
    var _tArray = ["radarsensor", "powerPlug", "temphumidity", "tvLamp", "bodysensor", "watersensor", "doorsensor", "coathanger", "sossensor", "panelsensor"];
    return _tArray.indexOf(_t) >= 0 || (_t == "panel" && !this.isPwrAvalibleDev(t))
  },

  getN (array, name) {
    console.log("getN array = ", array, name);
    if (!array || array.length == 0) {
      return name;
    } else if (array.indexOf(name) < 0) {
      return name;
    }
    let _i = 1;
    while (array.indexOf(name + _i) >= 0) {
      _i ++;
    }
    return name + _i;
  },

  // 根据设备id删除场景中所有的按键
  delAllPanelByD (allJson, d) {
    if (!allJson) {
      return;
    }
    this.delAllPanelByDAndKey(allJson, d);
  },
  // 根据设备id删除场景中所有的按键
  delAllPanelByDAndKey (allJson, d, key) {
    if (!allJson) {
      return;
    }
    let _scenes = this.getAllDevsFilter(allJson, "scenes");
    this.delPanelInScene(_scenes, d, key);
  },

  // 删除场景中存在的面板数据
  delPanelInScene (scenes, td, tKey) {
    if (!scenes || scenes.length == 0) {
      return [];
    }
    let _exsits = this.getExistIndexBySceneInfo(scenes, td, tKey);
    console.log("delPanelInScene _exsits = ", _exsits);
    // 如果有存在的数据，则删除
    for (let i = 0, len = _exsits.length; i < len; i++) {
      let _v = Number(_exsits[i]);
      if (_v >= 0) {
        scenes.splice(_v, 1);
      }
    }
    return scenes;
  },
  // 解绑成功的事件
  delInArrayByKey (array, value, key) {
    if (!array || array.length == 0) {
      return [];
    }
    // 根据按键和类型删除对应的场景绑定数据
    let _key = key || "key";
    for (let i = array.length - 1; i >= 0; i--) {
      let _item = array[i];
      if (value == _item[_key]) {
        array.splice(i, 1);
      }
    }
    return array;
  },
  // 取得存在的下标
  getExistIndexBySceneInfo (scenes, td, tKey) {
    let _result = [];
    if (!scenes || scenes.length == 0 || td == undefined) {
      return _result;
    }
    // 是否数组
    let _isDArray = td instanceof Array;
    // let _isKeyArray = tKey instanceof Array;
    for (let i in scenes) {
      let {d, key} = scenes[i];
      // 批量删除d
      if (_isDArray && (td.indexOf(d) >= 0)) {
        // 两个都存在
        _result.push(i);
        continue;
      }
      // 按照key删除单个d 或者删除全部d
      if (d == td && (key == tKey || tKey == undefined)) {
        // 两个都存在
        _result.push(i);
      }
    }
    return _result.sort((a,b)=>b-a);
  },
  addScenePanelInfo (d, key) {
    return {
      d: d,
      key: key || 0
    }
  },
  clearTimer (timer, type) {
    if (!timer) {
      return
    }
    // 存在类型
    if (type == "interval") {
      clearInterval(timer);
    } else {
      clearTimeout(timer);
    }
    timer = null;
  },

  // 超级面板的映射
  getTypeByDevTypeStr (devType, json) {
    let _devTypeStr = this.getDevTypeStr(devType);
    // console.log("getTypeByDevTypeStr _devTypeStr = ", _devTypeStr);
    let _map = json || this.getMetaTypeMad();
    let _values = Object.values(_map);
    let _indexs = Object.keys(_map);
    let _index = _values.indexOf(_devTypeStr);
    return _index < 0 ? 0 : (Number(_indexs[_index]) || 0);
  },
  getDevTypeStrByType (type) {
    let _map = this.getMetaTypeMad();
    return _map[type] || "";
  },
  getMetaTypeMad () {
    return {

      0: "scene", // 场景
      1: "light",
      2: "curtain",
      3: "switch", // 继电器面板
      4: "aircondition",
      // 5: "sensor",
      // 6: "gateway"
    }
  },

  // 取得模式下发第一个数据的json值
  getModeParamsBeforeSet (modeJson, brightness) {
    let {mode, key} = modeJson || {};
    // 更新发送的命令
    let _jsonData = {
      pwr: 1,
      brightness: brightness
    };
    // 第一次发送mode字段的第一个模式，亮度满值
    let _modeItem = (mode || [])[0];
    if (typeof _modeItem == "object") {
      let {red, green, blue, cold, warm} = _modeItem || {};

      // 数据是否大于0
      if (red + green + blue > 0) {
        let _total = red + green + blue;
        let _per = 255 / _total;
        red = parseInt(red * _per);
        green = parseInt(green * _per);
        blue = 255 - red - green;
      }
      if (cold + warm > 0) {
        let _total = cold + warm;
        let _per = 255 / _total;
        cold = parseInt(cold * _per);
        warm = 255 - cold;
      }
      _jsonData.r = red;
      _jsonData.g = green;
      _jsonData.b = blue;
      _jsonData.c = warm; // 这里cold和warm需要调换位置
      _jsonData.w = cold;
      // (_json.red != undefined) && (_jsonData.r = _json.red);
      // (_json.green != undefined) && (_jsonData.g = _json.green);
      // (_json.blue != undefined) && (_jsonData.b = _json.blue);
      // (_json.cold != undefined) && (_jsonData.c = _json.cold);
      // (_json.warm != undefined) && (_jsonData.w = _json.warm);
    } else {
      if (this.data.isCW) {
        _jsonData.c = 0x80;
        _jsonData.w = 0x7f;
      } else {
        _jsonData.r = 0x55;
        _jsonData.g = 0x55;
        _jsonData.b = 0x55;
      }
    }
    return _jsonData;
  },
  // 根据版本号取得设备支持场景能力集合
  getSupportSceneInfoByV (v) {
    let _result = -2; // 不支持场景
    let _vStr = this.getVersionByV(v);
    if (_vStr >= 48) {
      _result = 0; // 支持默认场景和自定义场景
    } else if (_vStr >= 42) {
      _result = -1; // 不支持默认场景
    }
    return _result;
  },
  // 根据完整的v字段取得版本号
  getVersionByV (v) {
    if (!v) {
      return 0;
    }
    return v.split(".").reverse()[0];
  },
  getSupportFloorByV (v) {
    let _v = this.getVersionByV(v);
    return _v >= 53;
  },
  // 根据设备信息删除群组中的设备信息
  delGroupDByDev (json, dev) {
    let {d} = dev || {};
    if (!json || !dev || !d) {
      return;
    }
    for (let roomId in json) {
      let {groups} = json[roomId] || {};
      if (!groups || groups.length == 0) {
        continue;
      }
      for (let item of groups) {
        let {dids} = item || {};
        if (!dids || dids.length == 0) {
          continue;
        }
        item.dids = dids.filter(item => item != d);
      }
    }
  },
  // 取得数据中所有超级面板中的场景数据
  getAllSceneInMetapadByJsonAndSid (json, sceneId) {
    console.log("getAllSceneInMetapadByJsonAndSid sceneId = ", sceneId);
    let _result = {};
    if (!json) {
      return _result;
    }
    // 取得所有设备再取得超级面板设备
    let uart = this.getAllMetapadsInJson(json);
    console.log("getAllSceneInMetapadByJsonAndSid uart = ", uart);
    if (!uart || uart.length == 0) {
      return _result;
    }
    for (let metapad of uart) {
      let {ds, d, n, e, tStr, t} = metapad || {};
      if (!ds || ds.length == 0) {
        continue;
      }
      // 这里忽略type 因为取得时场景的，type = 0
      let _filter = ds.filter(item => (sceneId == this.getSceneId(item.sid, item.rid)) && !item.type);
      if (_filter.length == 0) {
        continue;
      }
      _result[d] = _result[d] || {};
      // 取得触发设备的icon
      let _tStr = tStr || this.getDevImgByT(metapad) || 'light';
      // 设备类型动态显示图片
      let _devStr = this.getDevTypeStr(t);
      for (let _ds of _filter) {
        let {key} = _ds || {};
        if (key == undefined) {
          continue;
        }
        _result[d][_ds?.key] = {
          d: d,
          n: n || '',
          img: '../../static/img/common/' + _tStr + '.png',
          tip: this.getTipByDevTyeStr(_devStr, key, e), // ('按键' + (_key + 1))
          id: key
        };
      }
    }
    return _result;
  },

  // 根据pid取得按键名称
  getTipByDevTyeStr (devTypeStr, index, extension) {
    let str = devTypeStr || "";
    if (!str) {
      return '';
    }
    // 是否开关面板
    let _isPwrAvalible = this.isPwrAvalibleDev(str);
    // 如果传入的是个num类型的
    if (typeof str == "number") {
      str = this.getDevTypeStr(str);
    }

    if ((str == 'panel' && !_isPwrAvalible) || str == 'uart') {
      return ('按键' + (index + 1));
    } else {
      // 先判断是否有extension
      if (str == 'bodysensor' || str == 'radarsensor') {
        if (extension) {
          return ((extension?.vGroups || [])[index] || {}).name || '';
        } else {
          return index ? "无人" : "有人"
        }
      } else if (str == 'doorsensor') {
        if (extension) {
          return ((extension?.vGroups || [])[index] || {}).name || '';
        } else {
          return index ? "门磁关" : "门磁开"
        }
      } else if (str == 'watersensor') {
        if (extension) {
          return ((extension?.vGroups || [])[index] || {}).name || '';
        } else {
          return index ? "水浸正常" : "水浸报警"
        }
      } else if (str == 'powerPlug') {
        return index ? "有卡" : "无卡"
      } else if (str == 'temphumidity') {
        if (index == 0) {
          return "高温"
        } else if (index == 1) {
          return "低温"
        } else if (index == 2) {
          return "高湿"
        } else if (index == 3) {
          return "低湿"
        }
        return ""
      } else if (str == 'sossensor') {
        return index ? "取消报警" : "报警"
      } else if (str == 'panelsensor') {
        if (extension) {
          let {subs, vGroups} = extension;
          let _parentStr = subs[parseInt(index / 2)]?.name || "";
          let _childStr = vGroups[index]?.name || "";
          return _parentStr + " " + _childStr;
        } else {
          return "" + index;
        }
        // return index ? "取消报警" : "报警"
      } else if (_isPwrAvalible && str == 'panel') {
        if (extension) {
          let {vGroups} = extension;
          return (vGroups || [])[index]?.name || "";
        } else {
          return ('按键' + (index + 1));
        }
      }
    }
  },
  // 超级面板的场景绑定放在场景信息中，所以需要去获取
  getBindPanelInfoByDInJson (d, json) {
    let {panelScenes, sids} = this.getAllPanels(json);
    let _allKeys = panelScenes[d];
    console.log("initKeyStatusByScene _keys = ", _allKeys);
    let _bindConfig = {};
    if (!!_allKeys) {
      let _keys = Object.keys(_allKeys) || [];
      for (let key of _keys) {
        let _sceneInfo = sids[_allKeys[key]];
        let _index = Number(key);
        // 加入显示
        _bindConfig[_index] = ({
          sid: _sceneInfo?.sid || 0,
          rid: _sceneInfo?.rid || 0,
          type: 0,
          key: _index,
          cmd: 0
        });
      }
    }
    return _bindConfig;
  },
  // 根据场景id删除超级面板中绑定的场景
  delMetapadInfoBySceneId (json, sceneIds) {
    if (!json || !sceneIds) {
      return;
    }
    // 取得所有设备再取得超级面板设备
    let uart = this.getAllMetapadsInJson(json);
    if (!uart || uart.length == 0) {
      return;
    }
    console.log("delMetapadInfoBySceneId sceneIds, uart = ", sceneIds, uart);
    for (let metapad of uart) {
      let {ds} = metapad || {};
      if (!ds || ds.length == 0) {
        console.error("delMetapadInfoBySceneId ds = ", ds);
        continue;
      }
      // 这里忽略type 因为取得时场景的，type = 0
      let _filter = ds.filter(item => sceneIds.indexOf(this.getSceneId(item.sid, item.rid)) < 0 || !!item.type);

      console.log("delMetapadInfoBySceneId _filter = ", _filter);
    
      metapad.ds = _filter;
    }
  },
  // 取得所有场景面板设备
  getAllMetapadsInJson (json) {
    // 取得所有设备再取得超级面板设备
    let _allDevs = this.getAllDevsFilter(json);
    let {uart} = this.getAllPidByArray(_allDevs);
    return uart;
  },
  // 取得所有娼妓面板设备
  getAllGroupsInJson (json) {
    // 取得所有设备再取得超级面板设备
    let _allGroups = this.getAllDevsFilter(json, undefined, "groups");
    ;
    return _allGroups;
  },
  getGroupsByGidInJson (json, gid) {
    let _groups = this.getAllGroupsInJson(json);
    return _groups.filter(item => item.gid == gid);
  },
  delGroupByGidInJson (json, gids) {
    console.log("delGroupByGidInJson json, gids = ", json, gids);
    if (!json || !gids) {
      return;
    }
    for (let roomId in json) {
      let {groups} = json[roomId] || {};
      if (!groups || groups.length == 0) {
        continue;
      }
      json[roomId].groups = groups.filter(item => gids.indexOf(item.gid) < 0)
    }
  },
  isAndroid (userAgent) {
    return (userAgent || '').toLowerCase().indexOf('android') >= 0;
  },
  getExistNameCountByDevInJson (json, devInfo, name) {
    let _key = this.getKeyByDev(devInfo);
    if (!_key || !json) {
      return [];
    }
    // 此处增加相同房间下的设备和群组名称不能重复，非同一房间不判断
    let _roomId = devInfo?.gid || devInfo?.id || 0;

    let {devs, groups} = json[_roomId] || {};
    let _devAndGroupArray = (devs || []).concat(groups || []);
    console.log("getExistNameCountByDevInJson name, _roomId, _key = ", name, _roomId, _key);
    return _devAndGroupArray.filter(item => name == item.n && ((_key == "groups") ? (item?.gid != devInfo?.gid) : (item?.d != devInfo?.d)));
    // let _array = this.getAllDevsFilter(json, _roomId, _key);
    // return _array.filter(item => name == item.n && ((_key == "groups") ? (item?.gid != devInfo?.gid) : (item?.d != devInfo?.d)));
  },
  getKeyByDev (devInfo) {
    let {gid} = devInfo || {};
    if (gid >= 0) {
      return "groups";
    } else {
      return "devs";
    }
  },
  // 取得群组数据设备d对应的json数据
  getGroupDMapInJson (json) {
    if (!json) {
      return {};
    }
    let _result = {};
    for (let roomId in json) {
      let {groups} = json[roomId] || {};
      if (!groups || groups.length == 0) {
        continue;
      }
      for (let groupInfo of groups) {
        let {dids, gid} = groupInfo || {};
        if (!dids || dids.length == 0) {
          continue;
        }
        for (let d of dids) {
          _result[d] = _result[d] || [];
          _result[d].push(gid);
        }
      }
    }
    return _result;
  },
  // 是否传感器
  isSensorByPid (t) {
    let _typeStr = this.getDevTypeStr(t);
    return _typeStr.indexOf("sensor");
  },

  // 处理空调网关设备
  dealAirconditionSort (array) {
    let _result = [];
    if (!array) {
      return _result;
    }
    return this.sortArrayByKey(array, "a");
    // return array.sort((a, b) => (a?.a || 0) - (b?.a || 0));
    // // 取得所有的网关设备和子设备集合
    // let {parents, subDMap} = this.getAirconditionMap(array);
    // return this.airconditionJoinArray(gateways, gatewayMap);
  },
  airconditionJoinArray (gateways, gatewayMap) {
    let _result = [];
    if (!gateways || !gatewayMap) {
      return gateways || _result;
    }
    // 网关的名称已经排序好，直接把子设备加入到后面即可
    for (let item of gateways) {
      // 空调网关加入
      _result.push(item);
      // 是否有子设备，有的话则排序后加入
      let _array = gatewayMap[item?.d] || [];
      if (_array.length == 0) {
        continue;
      }
      let _sortArray = this.sortArrayByKey(_array, "a");
      // let _sortArray = _array.sort((item1, item2) => item1?.a - item2?.a);
      _result = _result.concat(_sortArray);
    }
    return _result;
  },
  // 取得空调网关d对应的内机数组
  getAirconditionMap (array) {
    let _parents = [], _subDMap = {};
    if (!array) {
      return {
        parents: _parents,
        subDMap: _subDMap
      }
    }
    for (let i = 0, len = array.length; i < len; i++) {
      let item = array[i] || {};
      if (item.sub >= 0) {
        _parents.push(item);
      } else {
        let _d = item.d;
        if (!_d) {
          continue;
        }
        let _gatewayD = _d.split("_")[0];
        _subDMap[_gatewayD] = _subDMap[_gatewayD] || [];
        _subDMap[_gatewayD].push(item);
      }
    }
    return {
      parents: this.sortArrayByChar(_parents, "n"),
      subDMap: _subDMap
    }
  },
  // 取得空调网关子设备列表
  getAirconditionSub (devInfo) {
    let _result = [];
    if (!devInfo) {
      return _result;
    }
    let {sub, d} = devInfo;
    // 删除场景下面触发设备
    for (let i = 1; i <= sub; i++) {
      _result.push(d + "_" + i);
    }
    return _result;
  },
  // 根据空调网关取得子设备列表集合
  getAirconditionsByDev (devArray, devInfo) {
    let _result = [];
    if (!devArray || !devInfo) {
      return _result;
    }
    let {d} = devInfo;
    // 删除网关下的子设备
    let _subs = devArray.filter(item => (item.d.indexOf(d) >= 0) && (d != item.d));
    return _subs;
  },
  // 去除数组中的网关子设备
  delSubAirconditionInArray (array) {
    if (!array) {
      return [];
    }
    return array.filter(item => {
      return this.isAirconditionHost(item);
    })
  },
  // 判断是否空调主机
  isAirconditionHost (devInfo) {
    if (!devInfo) {
        return false;
    }
    let {sub, t, d} = devInfo || {};
    if (this.getDevTypeStr(t) == "aircondition" && !sub) {
        // 主机子设备有_
        if (!!d && d.indexOf("_") >= 0) {
            return false;
        }
        return true;
    }
    return true;
  },
  // // 取得设备的默认图标
  // getDefaultIcon (devInfo) {
  //   let {v, t, a, sub} = devInfo || {};
  //   let _iconKey = "";
  //   if (a == undefined) {
  //     _iconKey = "group"
  //   } else {
  //     let _devTypeStr = this.getDevTypeStr(t);
  //     if (_devTypeStr == "uart") {
  //       _iconKey = "panel6";
  //     } else if (_devTypeStr == "panel") {
  //       let _count = this.getPanelKeysByPid(t);
  //       _iconKey = _devTypeStr + (_count || 6); // 默认6
  //     } else if (_devTypeStr.indexOf("sensor") >= 0) {
  //       // 门磁 人体传感器
  //       _iconKey = _devTypeStr == "doorsensor" ? 'doorsensor' : "bodysensor"
  //     } else if (_devTypeStr == "aircondition") {
  //       _iconKey = sub > 0 ? "aircondition" : "sub";
  //     } else {
  //       _iconKey = _devTypeStr;
  //     }
  //   }
  //   return _iconKey;
  // },
  getDevInfoByDInJson (devJson, d) {
    let _result = {};
    if (!devJson || !d) {
      _result = {};
    } else {
      for (let rid in devJson) {
        let _devs = devJson[rid]?.devs || [];
        let _filter = _devs.filter(item => item.d == d);
        if (_filter.length == 0) {
          continue;
        }
        _result = _filter[0];
        break;
      }
    }
    return _result;
  },

  // 开关面板指令转json cmdArray=[5,4]前面表示按键 后面表示是否选中
  pwrPanelCmdToJson (cmdArray) {
    let _result = {};
    if (!cmdArray || cmdArray.length == 0) {
      return _result;
    }
    let _byte0 = cmdArray[0] || 0;
    let _byte1 = cmdArray[1] || 0;
    for (let i = 0; i < 8; i++) {
      let _bit0 = _byte0 >> i;
      if (!_bit0) {
        continue;
      }
      let _bit1 = _byte1 >> i;
      _result[_bit0] = _bit1;
    }
    return _result;
  },

  // 空调命令转json
  airconditionCmdToJson (cmdArray) {
    // 空调发送的字节数据转成控制命令
    let _json = {
      pwr: 0
    };
    if (!cmdArray || cmdArray.length == 0) {
      return _json;
    }
    let _array = (typeof cmdArray == "string") ? Utils.hexStrToArray(cmdArray) : cmdArray;

    let {cmdObj} = getApp().globalData.blsdk.getAirconditionConfig();
    console.log("airconditionCmdToJson cmdObj = ", cmdObj);
    let {keys, vals} = cmdObj || {};
    for (let i = 0, len = _array.length; i < len; i++) {
      let _key = keys[i];
      if (_key != undefined) {
        _json[_key] = _array[i];
      }
    }
    return _json;
  },
  // 晾衣架命令转json
  coathangerCmdToJson (cmdArray) {
    // 空调发送的字节数据转成控制命令
    let _json = {};
    if (!cmdArray || cmdArray.length == 0) {
      return _json;
    }
    let _array = (typeof cmdArray == "string") ? Utils.hexStrToArray(cmdArray) : cmdArray;
    
    _json.updown = _array[0] || 0;
    _json.lamp = _array[1] || 0; 
    _json.fan = _array[2] || 0; 
    _json.dry = _array[3] || 0; 
    _json.disinfect = _array[4] || 0;
    return _json;
  },

  /*** 网关定时 ***/
  // 定时数据 返回 秒_分_时_日_月_周_年 repeatStr空或者1，2，3，4结构  time取值hh:mm
  getTimerByRepeatAndTime (repeatStr, timeStr) {
    // 当前年月日时分秒
    let _currentJson = Utils.getDateObj();
    let {hour, minute} = _currentJson;
    // 设置时间处理
    let _setHour = "*", _setMinute = "*";
    if (!!timeStr) {
      let _setTime = timeStr.split(":");
      _setHour = Number(_setTime[0]) || 0;
      _setMinute = Number(_setTime[1]) || 0;
    }
    // 返回结果
    let _result = new Array(6).fill(0);
    // 目前定时按照时分来计算
    if (!repeatStr) {
      // 只执行一次
      let _dataJson = this.getOnceData(_setHour * 60 + _setMinute, hour * 60 + minute);
      console.log("repeat empty _dataJson = ", _dataJson);
      _result = this.getGatewayTimeFormat(_dataJson || _currentJson);
      console.log("repeat empty _result = ", _result);
      _result[5] = "*";
      _result[1] = _setMinute;
      _result[2] = _setHour;
      console.log("repeat empty _setMinute, _setHour = ", _setMinute, _setHour);
    } else {
      // 目前除了执行一次就是按照星期执行 此时判断是否每天，不是的话则按照星期组数据
      _result = this.getGatewayTimeFormat(_currentJson);
      let _repeatArray = repeatStr.split(",");
      if (_repeatArray.length == 7) {
        _result[6] = "*";
        _result[5] = "*";
        _result[3] = "*";
        _result[4] = "*";
        _result[1] = _setMinute;
        _result[2] = _setHour;
      } else {
        _result[3] = "*";
        _result[4] = "*";
        _result[5] = repeatStr;
        _result[6] = "*";
        _result[1] = _setMinute;
        _result[2] = _setHour;
      }
    }
    // 遍历补满2位 ihg添加定时不行
    let _resultTwoArray = [];
    for (let value of _result) {
      if (value >= 0) {
        let _strValue = value.toString();
        _resultTwoArray.push(this.padStart(_strValue, 2, 0));
      } else {
        _resultTwoArray.push(value);
      }
    }
    console.log("getTimerByRepeatAndTime _resultTwoArray = ", _resultTwoArray);
    return _resultTwoArray.join("_");
  },
  // 取得时间格式
  getGatewayTimeFormat (obj) {
    let {year, month, day, hour, minute, second} = obj || {};
    // 默认设置星期为0
    return [0, minute || 0, hour || 0, day || 0, month || 0, 0, year || 0];
  },
  // 执行一次的时候判断当前时间是否大于设置的时间，如果大于，则推后一天
  getOnceData (setTimeStr, currentTimeStr) {
    console.log("getOnceData setTimeStr, currentTimeStr = ", setTimeStr, currentTimeStr);
    if (setTimeStr <= currentTimeStr) {
      return Utils.getDateObj(undefined, {day: 1})
    }
    return false;
  },
  /*** end ***/

  // 取得默认场景支持的默认照明数据
  getDefaultSceneBySceneType (array, defaultRoomId) {
    if (!array || array.length == 0) {
      return [];
    }
    // 目前只支持灯泡的默认场景设置，所以只需要去除灯泡支持默认场景的设备即可
    // light = light.filter(item => this.getSupportSceneInfoByV(item.v) == -1);
    // 这里有个bug 楼层和房间下的默认场景切换全屋或楼层时不显示设备
    let _defaultRoomId = defaultRoomId || 0;
    let _defaultFloorId = this.getFloorByRoomId(_defaultRoomId);
    let _filterArray = array.filter(item => {

      // 是否支持默认场景
      let _notSupportDefault = this.getSupportSceneInfoByV(item.v) == -1;
      if (_notSupportDefault) {
        return true;
      }

      if (_defaultRoomId == 0) {
        // 全屋
        return false;
      } else {
        if (item.id == 0) {
          return true;
        }
        if (_defaultRoomId % 32 == 0) {
          // 默认是楼层 去除楼层下默认设备
          return this.getFloorByRoomId(item.id) != _defaultFloorId;
        } else {
          // 单个房间
          return item.id != _defaultRoomId;
        }
      }
    });
    return _filterArray;
  },
  // 根据房间id取得设备列表和群组列表
  getDevsAndGroupsByRoomId (roomId, allDevs) {
    // 取得透传数据中的超级面板数据和透传数据
    var _obj = {};
    let _allDevs = allDevs || {};
    // 暂时屏蔽全屋时显示所有场景
    if (roomId == 0 || roomId == undefined) {
      // 如果是全屋 取得所有设备
      _obj = this.getAllDevsBySort(_allDevs, _allDevs);
    } else if (roomId % 32 == 0) {
      // 楼层数据
      let _floor = this.getFloorByRoomId(roomId);
      let _data = this.getAllDevsByFloor(_allDevs, _floor, false);
      // let _data = this.data.rooms.data || [];
      // _data = _data.filter(item=>((item?.id || 0) > 0)); //  % 32
      console.log("scrollClick _data = ", _data);
      // 如果是全屋 取得所有设备
      _obj = this.getAllDevsBySort(_data, _allDevs);
    } else {
      _obj = _allDevs[roomId || 0] || {};
    }
    return {
      devs: _obj?.devs || [],
      groups: _obj?.groups || [] // _groups
    };
  },
  // 根据房间id取得群组
  getDevsOrGroupsByRoomId (roomId, json, key) {
    let _array = [];
    if (!json) {
      return _array;
    }
    let _roomId = roomId || 0;
    let _key = key || "devs";
    if (_roomId == 0) {
      // 全屋
      _array = this.getAllDevsFilter(json, undefined, _key) || [];
    } else if (roomId % 32 == 0) {
      // 取得楼层数据
      let _devJson = this.getDevsArrayByFloor(json, this.getFloorByRoomId(roomId)) || {};
      _array = this.getAllDevsFilter(_devJson, undefined, _key);
    } else {
      _array = this.getAllDevsFilter(json, _roomId, _key);
    }
    // console.log("getGroupsByRoomId _array = ", _array);
    return _array;
  },
  getGroupedByConfig (config) {
    if (!config) {
      return {};
    }
    console.log("getGroupedByConfig config = ", config);
    let _allConfigs = DEVCONFIG || {};
    let _json = new Object();
    for (let key in _allConfigs) {
      let _keyTitle = config[key] || "";
      console.log("getGroupedByConfig _keyTitle = ", _keyTitle);
      if (!_keyTitle) {
        continue;
      }
      _json[key] = {
        text: _keyTitle,
        key: key,
        selecteds: [],
        list: []
      };
    }
    console.log("getGroupedByConfig _json = ", _json);
    return _json;
  },
  // 执行设备分组
  getGroupedActionConfig () {
    var _config = {
      light: "照明",
      curtain: "窗帘",
      panel: "开关面板",
      aircondition: "空调",
      fanLight: "风扇",
      thermostat: "温控器",
      powerstrip: "插排/插座"
    };
    return this.getGroupedByConfig(_config);
    // return {
    //   light: {
    //     text: "照明",
    //     key: "light",
    //     selecteds: [],
    //     list: []
    //   },
    //   curtain: {
    //     text: "窗帘",
    //     key: "curtain",
    //     selecteds: [],
    //     list: []
    //   },
    //   panel: {
    //     text: "开关面板",
    //     key: "panel",
    //     selecteds: [],
    //     list: []
    //   },
    //   aircondition: {
    //     text: "空调",
    //     key: "aircondition",
    //     selecteds: [],
    //     list: []
    //   },
    //   fanLight: {
    //     text: "风扇",
    //     key: "fanLight",
    //     selecteds: [],
    //     list: []
    //   },
    //   thermostat: {
    //     text: "温控器",
    //     key: "thermostat",
    //     selecteds: [],
    //     list: []
    //   }
    // }
  },
  getOriginFamilyInfo (bleCode, seq) {
    return {
      id: 1,
      n: '我的家庭',
      bleCode: bleCode || [],
      seq: seq || 1,
      groupIndex: 0, // 默认第一个集群
      floor: -1, // 不支持多楼层
      roleNum: 0, // 管理员权限
      devTimestamp: 0, // 超时时间
      sid: "", // 数据上报相关
      skey: "", // 数据上报相关
      addr1: 1,
      addr2: 257
    };
  },
  
  dealErrorMsg (errorCode, allMsgs) {
    let _allMsgs = allMsgs || {};
    let _obj = _allMsgs["zh"];
    if (!_obj) {
      return "";
    }
    return _obj[errorCode] || "";
  },
  // getRoomArrayInDevs (devs) {
  //   let _result = {};
  //   if (!devs) {
  //     return _result;
  //   }
  //   for (let rid in devs) {
  //     _result[rid] = {
  //       id: rid,
  //       n: devs[rid]?.n || ""
  //     }
  //   }
  //   return _result;
  // },
  // 设备控制类型
  getSendPidByType (devTypeStr) {
    let _devTypeStr = devTypeStr;
    if (typeof _devTypeStr == "number") {
      _devTypeStr = this.getDevTypeStr(_devTypeStr);
    }
    // 除了照明和窗帘，其他待定，先写着
    if (_devTypeStr == "curtain") {
      return 43499;
    } else if (_devTypeStr == "fanLight") {
      return 43531;
    } else if (_devTypeStr == "thermostat") {
      return 43919;
    } else if (_devTypeStr == "aircondition") {
      return 43756;
    } else if (_devTypeStr == "powerstrip") {
      return 44181;
    }
    return 43050; // 默认照明的类型
  },
  // 取得命令json
  getCmdJsonByT(cmd, t) {
    let _result = Object.assign({}, cmd);
    console.log("getCmdJsonByT _result = ", JSON.stringify(_result));
    let _devTypeStr = this.getDevTypeStr(t);
    if (_devTypeStr != "light") {
      return cmd;
    }
    // 判断设备类型
    let _cmdJson = this.getFuncByPid(_result, t, {
      brightness: cmd?.brightness,
      c: cmd?.c
    });
    console.log("getCmdJsonByT _cmdJson = ", JSON.stringify(_cmdJson));
    // 如果是白光的场合，则
    let _isWhite = _result?.statueKey == "white";
    // 是否rgbw设备，此设备发送白光同cw发送
    let _isRgbw = this.isRGBW(t);
    // 每个设备按照各自的白光来设置
    if (_isWhite && (_isRgbw || (_cmdJson.c >= 0 && _cmdJson.r == undefined))) {
      _cmdJson.c = _cmdJson.w = 127;
      _cmdJson.r = _cmdJson.g = _cmdJson.b = 0;
    } else if (_cmdJson.r >= 0 && _cmdJson.c == undefined && _isWhite) {
      _cmdJson.r = _cmdJson.g = _cmdJson.b = 255;
      _cmdJson.c = _cmdJson.w = 0;
    } else {
      // 有属性的才加入属性
      for (let key in _cmdJson) {
        _cmdJson[key] = cmd[key] || 0;
      }
    }
    console.log("getCmdJsonByT _cmdJson = ", JSON.stringify(_cmdJson));
    return _cmdJson;
  },
  // 增加默认群组
  addDefaultGroupByDevs (allDevs) {
    // console.log("addDefaultGroupByDevs allDevs = ", allDevs);
    if (!allDevs) {
      return {};
    }
    let _supportGroups = ["light", "curtain"];
    for (let roomId in allDevs) {
      let _roomDetail = allDevs[roomId] || {};
      // 取得全屋设备数据
      let devs = this.getDevsOrGroupsByRoomId(roomId, allDevs, "devs");
      // let devs = _roomDetail.devs || [];
      let groups = _roomDetail.groups || [];
      // if (!devs || devs.length == 0 || !groups) {
      //   continue;
      // }
      let _devsConfig = this.getAllPidByArray(devs);
      // 是否存在数据
      let _configJson = this.getGroupCountByTypeInArray({
        groups: groups,
        supportGroups: _supportGroups
      });
      // console.log("addDefaultGroupByDevs roomId, _configJson, _devsConfig = ", roomId, _configJson, _devsConfig);
      let _roomIdInt = Number(roomId) || 0;

      // 遍历数据
      for (let key of _supportGroups) {
        let _devGroupStr = key + "Default";
        let _devCount = _devsConfig[key]?.length || 0;
        // console.error("addDefaultGroupByDevs _devCount = ", _devCount, _configJson[_devGroupStr]);
        if (_devCount == 0 && _configJson[_devGroupStr] >= 0) {
          // 删除默认群组设备
          (_roomDetail.groups || []).splice(_configJson[_devGroupStr], 1)
          // devs.splice(_configJson[_devGroupStr], 1);
        } else if (_devCount > 0 && _configJson[_devGroupStr] == undefined) {
          _roomDetail.groups = _roomDetail.groups || [];
          // 加入默认群组设备
          _roomDetail.groups.push(this.getItemByType({type: key, roomId: _roomIdInt, n: _roomDetail?.n}));
        }
      }
    }
  },
  getGroupCountByTypeInArray ({groups, supportGroups}) {
    let _result = {};
    if (!groups || groups.length == 0 || !supportGroups) {
      return _result;
    }
    for (let index in groups) {
      let _tmpItem = groups[index] || {};
      let {t, bledevtype, gid} = _tmpItem;
      let _devTypeStr = this.getDevTypeStr(t); // 设备信息
      if (supportGroups.indexOf(_devTypeStr) < 0) {
        continue;
      }
      // 是否默认群组
      if (this.isAllRoomCtrl(_tmpItem)) {
        _result[_devTypeStr + "Default"] = Number(index);
      } else {
        continue;
      }
    }
    return _result;
  },
  getItemByType ({type, roomId, n}) {
    let _result = {
      a: 0,
      bledevtype: "GROUP",
      id: roomId,
      rid: roomId,
      // gid: roomId,
      // t: 
      // n: roomName + ""
      // type: // 
    };
    let _n = ""; // 设备名称
    let _t = 0; // 设备类型
    if (type == "light") {
      _n = "照明";
      _t = 43050;
    } else if (type == "curtain") {
      _n = "窗帘";
      _t = 43499;
    }
    _result.n = (n || "") + _n; // + "设备";
    _result.t = type; // _t;
    // let _metapadType = this.getTypeByDevTypeStr(type);
    _result.gid = roomId;// 类型 房间id (_metapadType << 8) + 

    return _result;
  },
  isAllRoomCtrl (info) {
    return (info?.bledevtype == "GROUP") || (info?.gid < 193);
  },
  getFilterGroups (array, roomId) {
    if (!array) {
      return [];
    }
    let _roomId = roomId || 0;
    return array.filter(item => {
      let _isDefaultGroup = this.isAllRoomCtrl(item);
      if (_isDefaultGroup) {
        return ((item.gid || 0) == _roomId)
      }
      return true;
    });
  },
  // 根据品类取得控制的type
  getCtrlPidByT (t) {
    let _devTypeStr = this.getDevTypeStr(t);
    if (_devTypeStr == "curtain") {
      return 43499; // 窗帘
    } else {
      return 43050; // 照明设备
    }
  },
  // 取得触发设备列表
  getTriggerArrayByRidAndDevs (devs, roomId, config) {
    if (!devs) {
      return [];
    }
    // 取得设备信息
    let {devs: _allDevs} = this.getDevsAndGroupsByRoomId(roomId, devs);
    console.log("getTriggerArrayByRidAndDevs _allDevs = ", _allDevs);
    // 所有的配置信息
    let _devConfigJson = this.getAllPidByArray(_allDevs) || {};
    let {panel, doorsensor, bodysensor, uart, watersensor, radarsensor, temphumidity, sossensor, panelsensor, powerPlug} = _devConfigJson;
    // 去除开关面板 只需要情景面板
    let _array = (panel || []).filter(item => !this.isPwrAvalibleDev(item.t));
    console.log("getTriggerArrayByRidAndDevs _array = ", _array);
    
    // 通过配置过滤拿到需要的列表
    let {devTypeStr} = config || {};
    if (devTypeStr) {
      if (devTypeStr == "pwrAvalible") {
        // 取得开关面板
        return _array;
      } else {
        return _devConfigJson[devTypeStr] || [];
      }
    }
    // 设备列表
    return _array.concat(doorsensor || []).concat(bodysensor || []).concat(watersensor || []).concat(uart || []).concat(radarsensor || []).concat(temphumidity || []).concat(sossensor || []).concat(panelsensor || []).concat(powerPlug || []);
  },
  // 取得需要显示的名称
  getRoomName (devInfo, roomJson, isFloor) {
    if (!devInfo || !roomJson) {
      return "";
    }
    // let isFloor = this.data.isFloor;
    // let roomJson = this.data.roomJson;
    let _id = devInfo?.rid || devInfo?.id || 0;
    return ((_id % 32 == 0) || !isFloor) ? (roomJson[_id]?.n) : (roomJson[_id + 32 - _id % 32]?.n + ' | ' + roomJson[_id]?.n)
  },
  // 取得温度单位
  getUnitByValue (value) {
    // "°F" : "℃"
    return value || "℃"
  },
  // 取得完整的历史数据
  getTotalFormatByTimeStr (str) {
    if (!str) {
      return "";
    }
    let _data = Utils.getDateByTimezone(str);
    if (!_data) {
      return "";
    }
    return Utils.formatTime(_data);
  },
  // 历史数据转hhmm格式
  getHMByTimeStr (str) {
    let _str = this.getTotalFormatByTimeStr(str);
    if (!_str) {
      return "";
    }
    console.log("getHMByTimeStr _str = ", _str);
    // return (_str?.split(" ") || [])[1]?.slice(0, 5) || "";
    // str 2022-09-13 11:38:27
    return (_str?.split(" ") || [])[1]?.slice(0, 5) || "";
  },
  // 取得展示的hhmm时间，如果不是同一天，则显示日期 str: 2022-10-13_10:34:12
  getShowTimeStr (str) {
    console.log("getShowTimeStr str = ", str);
    if (!str) {
      return "";
    }
    // 取得当前时间
    let {year, month, day} = Utils.getDateObj(); // getZoneDataByZone(8);
    // 当前时区的时间
    let _cData = Utils.getDateByTimezone(str);
    let _cJson = Utils.getDateObj(_cData);
    let {year:cYear, month:cMonth, day:cDay} = _cJson;
    console.log("getShowTimeStr = ", year, month, day, cYear, cMonth, cDay);
    // 判断是否同一年
    let _sameYear = year == cYear;
    // 判断是否同一月
    let _saveMonth = month == cMonth;
    // 判断是否同一天
    let _saveDay = day == cDay;
    // 2022-09-13 11:38:27
    let _totalStr = Utils.formatTime(_cData);
    if (_sameYear) {
      if (_saveMonth) {
        if (_saveDay) {
          return _totalStr.substr(11, 5);
        } else {
          return _totalStr.substr(5, 11);
        }
      } else {
        // 2022-09-13 11:38:27
        return _totalStr.substr(5, 11);
      }
    } else {
      return _totalStr.substr(2, 14);
    }
  },
  // 排插格式为数组
  getPowerstripTipByCmd (cmdArray) {
    if (!cmdArray || cmdArray.length == 0) {
      return "";
    }
    // [255, 255] [0,0]
    return cmdArray[1] ? "全开" : "全关";
  },
  // 解析设备状态
  getDevStatusByTAndArray (t, array) {
    let _result = {};
    // 如果数组为空则状态为空
    if (!array?.length) {
      return _result;
    }
    // 品类
    let _devTypeStr = this.getDevTypeStr(t);
    // 判断设备品类
    if (_devTypeStr == "panel" && this.isPwrAvalibleDev(t)) {
      // 路数
      let _count = this.getPanelKeysByPid(t);
      console.log("getDevStatusByTAndArray _count = ", _count);
      // 解析开关面板状态
      for (let i = 0; i < _count; i++) {
        let _panelId = ((array[0] || 0) >> i) & 0x01;
        let _pwr = ((array[1] || 0) >> i) & 0x01;
        if (!_panelId) {
          continue;
        }
        _result[i] = _pwr;
      }
    }
    return _result;
  },

  // 初始化支持超级面板的继电器开关默认按键绑定
  initMetaPwrKeyBind (devInfo) {
    let _result = [];
    if (!devInfo) {
      return _result;
    }
    // 判断ability,如果支持超级面板，则增加缓存信息
    let _vJson = this.getJsonByAblitys(devInfo.v, "panel", {t: devInfo.t}) || {};
    console.error("initDefaultByPid _vJson, devInfo = ", _vJson, devInfo);
    // 继电器超级面板能力
    if (_vJson.metapadEnable) {
      // 判断按键路数
      let _count = this.getPanelKeysByPid(devInfo.t);
      console.error("initDefaultByPid _count = ", _count);
      // 超级面板的配置
      let _metaConfig = this.getMetaTypeMad();
      let _keys = Object.keys(_metaConfig);
      let _values = Object.values(_metaConfig);
      console.error("initDefaultByPid _keys, _values = ", _keys, _values);
      let _index = _values.findIndex(item => item == "switch");
      let _realType = _keys[_index];
      console.error("initDefaultByPid _index, _realType = ", _index, _realType);
      if (_realType == undefined) {
        return;
      }
      for (let i = 0; i < _count; i++) {
        _result.push({
          sid: devInfo.a,
          rid: devInfo.id || 0,
          type: Number(_realType) || 0,
          key: Number(i), // 超级面板的key
          cmd: devInfo.d, // 继电器设备d
          pwrPanelKey: i // 继电器的第几路 
        })
      }
    }
    return _result;
  }
};






