"use strict";
var HEX = "0123456789abcdef";
var formatNumbers = function (n) {
    var s = n.toString();
    return s[1] ? s : '0' + s;
};
var getZoneDataByZone = function (i) { 
  if (typeof i !== 'number') return;
  var d = new Date(); 
  //得到1970年一月一日到现在的秒数
  var len = d.getTime();
  // 本地时间与GMT时间的时间偏移差(注意：GMT这是UTC的民间名称。GMT=UTC）
  var offset = d.getTimezoneOffset() * 60000;
  //得到现在的格林尼治时间
  var utcTime = len + offset;
  return new Date(utcTime + 3600000 * i);
}
var getDateObj = function (date, config) {
    let _date = date || (new Date());

    // 是否往前往后推N天
    !!(config?.day) && (_date.setDate(_date.getDate() + config.day));

    // // 是否往前往后推N天
    // !!(config?.day) && (_date.setDate(_date.getDate() + config.day));

    var year = _date.getFullYear();
    var month = _date.getMonth() + 1;
    var day = _date.getDate();
    var hour = _date.getHours();
    var minute = _date.getMinutes();
    var second = _date.getSeconds();
    return {
        year, month, day, hour, minute, second
    }
}
var formatTime = function (date, config) {
    let {year, month, day, hour, minute, second} = getDateObj(date);
    let {sep1, sep2, sep3} = config || {};
    // var year = date.getFullYear();
    // var month = date.getMonth() + 1;
    // var day = date.getDate();
    // var hour = date.getHours();
    // var minute = date.getMinutes();
    // var second = date.getSeconds();
    return [year, month, day].map(formatNumbers).join(sep1 || '-') + (sep2 || ' ') + [hour, minute, second].map(formatNumbers).join(sep3 || ':');
};
var ab2hex = function (buffer) {
    var hexArr = Array.prototype.map.call(new Uint8Array(buffer), function (bit) {
        return ('00' + bit.toString(16)).slice(-2);
    });
    return hexArr.join('');
};
function hex2int(_hex) {
    var hex = (typeof _hex != "string") ? (_hex + "") : _hex;
    var len = hex.length, a = new Array(len), code;
    for (var i = 0; i < len; i++) {
        code = hex.charCodeAt(i);
        if (48 <= code && code < 58) {
            code -= 48;
        }
        else {
            code = (code & 0xdf) - 65 + 10;
        }
        a[i] = code;
    }
    return a.reduce(function (acc, c) {
        acc = 16 * acc + c;
        return acc;
    }, 0);
}
var str2ab = function (str) {
    var buf = new ArrayBuffer(str.length * 2);
    var bufView = new Uint16Array(buf);
    for (var i = 0, strLen = str.length; i < strLen; i++) {
        bufView[i] = str.charCodeAt(i);
    }
    return buf;
};
var inArray = function (arr, key, val) {
    for (var i = 0; i < arr.length; i++) {
        if (arr[i][key] === val) {
            return i;
        }
    }
    return -1;
};
var str2Bytes = function (str) {
    var pos = 0;
    var len = str.length;
    if (len % 2 != 0) {
        return null;
    }
    len /= 2;
    var hexA = '';
    for (var i = 0; i < len; i++) {
        var s = str.substr(pos, 2);
        var v = parseInt(s, 16);
        if (!v) {
            v = ((charToByte(s.charAt(0)) << 4) | charToByte(str.charAt(1)));
        }
        if (v >= 127)
            v = v - 255 - 1;
        hexA = (v + "");
        pos += 2;
    }
    return hexA;
};
var charToByte = function (c) {
    return "0123456789abcdef".indexOf(c);
};
var byteToString = function (bytes) {
    var str = '';
    for (var i = 0; i != bytes.length; ++i) {
        if ((bytes[i] & 0xf0) == 0) {
            str += "0";
            str += HEX.charAt(bytes[i] & 0x0f);
        }
        else {
            str += HEX.charAt((bytes[i] >> 4) & 0x0f);
            str += HEX.charAt(bytes[i] & 0x0f);
        }
        str += " ";
    }
    return str;
};
var hexStrToArray = function (hexStr) {
    var _result = [];
    var _length = (hexStr || "").length;
    if (_length > 0) {
        var i = 0, len = _length / 2;
        if (_length % 2) {
            hexStr = '0' + hexStr;
        }
        for (; i < len; i++) {
            _result[i] = parseInt('0x' + hexStr.substr(i * 2, 2));
        }
    }
    return _result;
};
var getIndexByKey = function (key, array) {
    for (var i = 0, len = array.length; i < len; i++) {
        var _item = array[i];
        // console.log("getIndexByKey _item.pagePath, key = ", _item.pagePath, key);
        if (_item.pagePath.indexOf(key) >= 0) {
            return i;
        }
    }
    return -1;
};
var utils = {
    getIndexByKey: getIndexByKey,
    getDateObj: getDateObj,
    getZoneDataByZone: getZoneDataByZone,
    formatNumbers: formatNumbers,
    formatTime: formatTime,
    inArray: inArray,
    ab2hex: ab2hex,
    str2ab: str2ab,
    str2Bytes: str2Bytes,
    byteToString: byteToString,
    hex2int: hex2int,
    hexStrToArray: hexStrToArray,
    
    // 取得16进制字符串的总和
    getSumByHexStr (str) {
        let _str = str || '';
        let _result = 0;
        let _isString = typeof str == "string";
        for (let i = 0, len = _str.length; i < len; ) {
          if (_isString) {
            _result += Number('0x' + _str.substr(i, 2));
            i += 2;
          } else {
            _result += _str[i];
            i++;
          }
        }
        return _result;
    },
    urlStrToJson (urlStr) {
      if (!urlStr) {
        return '';
      }
      // 提取出参数前的数据
      let _paramArray = urlStr.split("?") || [];
      if (_paramArray.length < 2) {
          return urlStr;
      }
      let _tmpArray = _paramArray[1];
      let _allParamStrs = _tmpArray.split("&");
      let _allJson = {};
      for (let i in _allParamStrs) {
          let _str = _allParamStrs[i] || '';
          let _array = _str.split('=');
          if (_array.length < 2) { continue; }
          _allJson[_array[0]] = _array[1];
      }
      return _allJson;
    },
    // 取得与GMT分钟差
    getCurrentTimezone () {
      // 取得完整的分钟差 / 60即为小时差 中国+8：00为-8 这里需要记住 是否有个半时区的
      let _utcMinite = new Date().getTimezoneOffset();
      return _utcMinite;
    },
    // 取得与东八区时间分钟差
    getSeperatorMinite () {
      let _minute = this.getCurrentTimezone();
      console.log("getSeperatorMinite _minute = ", _minute);
      return -480 - _minute; // 480 = 60 * 8 东八区
    },
    // 根据时间显示
    getDateByTimezone (str, zoneStr) {
      if (!str) {
        return "";
      }
      var _totalStr = str.replace("_", " ");
      // ios不识别yyyy-mm-dd格式，需要转成yyyy/mm/dd
      _totalStr = _totalStr.replaceAll("-", "/");
      // 判断原来的str有没有时区的数据，判断条件为字符串+
      if (_totalStr.indexOf("+") < 0) {
        // 存在时区
        _totalStr += (zoneStr || "+08:00"); // 默认东八区
      }
      console.log("getDateByTimezone _totalStr = ", _totalStr);
      return new Date(_totalStr);
    },
};
module.exports = {
    Utils: utils
};